/*
 * WCFG         An implementation of the wcfglib library.
 *		See wcfg_lib companion document.
 *
 *
 * Version:     0.1.0
 *
 * Authors:     Alexandra Aronov, <alexandraa@ti.com>
 *              Felix Rubinstein, <felixr@ti.com>
 *
 */


//#include "dda.h"
#include "iwlib.h"
#include "wcfglib.h"
#include "wcfg_ioctl.h"
#include "cfg_proc_if.h"
#include "dda_proc_fs_defs.h"

#define min(x,y) ((x)<(y) ? (x) : (y))
/*------------------------------------------------------------------*/
/*
 * Macro to handle errors when setting WE
 * Print a nice error message and exit...
 * We define them as macro so that "return" do the right thing.
 * The "do {...} while(0)" is a standard trick
 */




#define ERR_SET_EXT(rname, request) \
	fprintf(stderr, "Error for wireless request \"%s\" (%X) :\n", \
		rname, request)

#define ABORT_ARG_SIZE(rname, request, max) \
	do { \
		ERR_SET_EXT(rname, request); \
		fprintf(stderr, "    argument too big (max %d)\n", max); \
		return(-3); \
	} while(0)

#define ABORT_NULL_ARG(rname, request) \
	do { \
		ERR_SET_EXT(rname, request); \
		fprintf(stderr, "    argument is NULL\n"); \
		return(-3); \
	} while(0)

/*------------------------------------------------------------------*/
/*
 * Wrapper to push some Wireless Parameter in the driver
 * Use standard wrapper and add pretty error message if fail...
 */
#define IW_SET_EXT_ERR(ioctl_sock, ifname, request, wrq, rname) \
	do { \
	int ret; \
	if ((ret = iw_set_ext(ioctl_sock, ifname, request, wrq))) { \
		ERR_SET_EXT(rname, request); \
		fprintf(stderr, "    SET failed on device %-1.16s ; %s.\n", \
			ifname, strerror(errno)); \
		return ret; \
	} } while(0)


static int generic_ioctl_get_buf(char * buf, int buf_len, int cmd, char * ifname, int param1, int param2, char * mac);
static int read_priv_ioctls(int ioctl_sock,char *ifname,iwprivargs *priv);
static int ioctl_set_essid(const char *essid, int ioctl_sock, char *ifname);
static int ioctl_set_rts(const __s32 *rts, int ioctl_sock, char *ifname);
static int ioctl_set_frag(const __s32 *frag, int ioctl_sock, char *ifname);
static int ioctl_set_retry(const __s32 *retry, const int type, int ioctl_sock, char *ifname);
static int ioctl_set_txpow(tiap_param_txpower_t *txpow, int ioctl_sock, char *ifname);



static int ioctl_set_hwaddr(const __u8 *hwaddr, int ioctl_sock, const char *ifname)
{
	struct ifreq ifr;

	strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
	ifr.ifr_hwaddr.sa_family = ARPHRD_ETHER;
	memcpy(ifr.ifr_hwaddr.sa_data, hwaddr, ETH_ALEN);
		
	return ioctl(ioctl_sock, SIOCSIFHWADDR, &ifr);
}


static int ioctl_set_essid(const char *essid, int ioctl_sock, char *ifname)
{

	/* Check the size of what the user passed us to avoid
	 * buffer overflows */
	if (essid)
	{
		if(strlen(essid) > IW_ESSID_MAX_SIZE)
		  ABORT_ARG_SIZE(__func__, SIOCSIWESSID, IW_ESSID_MAX_SIZE);
		else
		{
			struct iwreq		wrq;

			wrq.u.essid.flags = 1;
			wrq.u.essid.pointer = (caddr_t) essid;
			wrq.u.essid.length = strlen(essid) + 1;
			
			IW_SET_EXT_ERR(ioctl_sock, ifname, SIOCSIWESSID, &wrq,__func__);
		}
	}
	else ABORT_NULL_ARG("Set ESSID", SIOCSIWESSID);
	
	return 0;

}

static int generic_ioctl_get_buf(char * buf, int buf_len, int cmd, char * ifname, int param1, int param2, char * mac) {
	int ioctl_sock, ret = 0;
	generic_ioctl_buf_t data;
	struct ifreq ifr;

	if ((ioctl_sock = iw_sockets_open()) < 0) {
		printf("%s : iw_sockets_open() failed - %s\n",  __func__, strerror(errno));
		ret = -1;
		goto exit;
	}

	if (!buf && !buf_len) {
		printf("%s: Both buf and buf_len must be valid (cmd=%d)\n", __func__, cmd);
		ret = -1;
		goto exit;
	}

	data.buf = buf;
	data.buf_len = buf_len;
	data.cmd = cmd;
	data.param1 = param1;
	data.param2 = param2;
	if (mac != NULL)
		memcpy(data.mac, mac, 6);

	strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
	ifr.ifr_data = (char *)&data;

	ret = ioctl(ioctl_sock, SIOCDEVPRIVATE, &ifr);	
	if (ret < 0)
		printf("%s : ioctl() failed - %s\n",  __FUNCTION__, strerror(errno));

 exit:
	close(ioctl_sock);

	return ret;
}


int ioctl_get_tx_info(int Qid, int QprintEn, char * buf, int buf_len, char * ifname) {
	return generic_ioctl_get_buf(buf, buf_len, TIAP_GENERIC_IOCTL_GET_TX_INFO, ifname, Qid, QprintEn, NULL);
}

int ioctl_get_tx_queue(int Qid, char * buf, int buf_len, char * ifname) {
	return generic_ioctl_get_buf(buf, buf_len, TIAP_GENERIC_IOCTL_GET_TX_QUEUE, ifname, Qid, 0, NULL);
}

int ioctl_get_challenge(int index, char * buf, int buf_len, char * ifname) {
	return generic_ioctl_get_buf(buf, buf_len, TIAP_GENERIC_IOCTL_TST_CHALLENGE, ifname, index, 0, NULL);
}

int ioctl_get_find_sta(char * mac, char * buf, int buf_len, char * ifname) {
	return generic_ioctl_get_buf(buf, buf_len, TIAP_GENERIC_IOCTL_TST_FINDSTA, ifname, 0, 0, mac);
}

int ioctl_get_insert_sta(char * mac, int wone_index, char * buf, int buf_len, char * ifname) {
	return generic_ioctl_get_buf(buf, buf_len, TIAP_GENERIC_IOCTL_TST_FIND_INS, ifname, wone_index, 0, mac);
}

int ioctl_get_eer(int addr, char * buf, int buf_len, char * ifname) {
	return generic_ioctl_get_buf(buf, buf_len, TIAP_GENERIC_IOCTL_GET_EER, ifname, addr, 0, NULL);
}
 
int ioctl_get_mac_reg_read(int addr, char * buf, int buf_len, char * ifname) {
	return generic_ioctl_get_buf(buf, buf_len, TIAP_GENERIC_IOCTL_GET_MAC_REGREAD, ifname, addr, 0, NULL);
}

int ioctl_get_phy_reg_read(int addr, char * buf, int buf_len, char * ifname) {
	return generic_ioctl_get_buf(buf, buf_len, TIAP_GENERIC_IOCTL_GET_PHY_REGREAD, ifname, addr, 0, NULL);
}

int ioctl_get_trace_print(int num, int from, char * buf, int buf_len, char * ifname) {
	return generic_ioctl_get_buf(buf, buf_len, TIAP_GENERIC_IOCTL_TRACE_PRINT, ifname, num, from, NULL);
}

int ioctl_get_wlan_params(int wone_index, char *buf, int buf_len, char *ifname){
	return generic_ioctl_get_buf(buf, buf_len, TIAP_GENERIC_IOCTL_WLAN_PARAMS, ifname, wone_index, 0, NULL);
}

int ioctl_set_burst_limit(__u32 burst_limit, int ioctl_sock, char *ifname)
{
	char dummy_buf;

	return generic_ioctl_get_buf(&dummy_buf, sizeof(dummy_buf), TIAP_GENERIC_IOCTL_SET_BURST_LIMIT, ifname, burst_limit, 0, NULL);

}

int ioctl_get_burst_limit(char *buf, int buf_len, int ioctl_sock, char *ifname)
{
	return generic_ioctl_get_buf(buf, buf_len, TIAP_GENERIC_IOCTL_GET_BURST_LIMIT, ifname, 0, 0, NULL);

}

int ioctl_set_gpio_owner(__u8 gpio_owner, int ioctl_sock, char *ifname)
{
#if defined(CONFIG_MIPS_AR7VWI)
	char dummy_buf;
	return generic_ioctl_get_buf(&dummy_buf, sizeof(dummy_buf), 
			TIAP_GENERIC_IOCTL_SET_GPIO_OWNER, ifname, gpio_owner, 0, NULL);
#else
	return 0;
#endif
}

#ifdef REPORT_DEBUG

int ioctl_get_logged_chars(char *buf, int buf_len, int ioctl_sock, char *ifname)
{
	return generic_ioctl_get_buf(buf, buf_len, TIAP_GENERIC_IOCTL_GET_LOGGED_CHARS, ifname, 0, 0, NULL);  
}


int ioctl_set_logged_chars(__u32 logged_chars, int ioctl_sock, char *ifname)
{
	char dummy_buf;
	
	return generic_ioctl_get_buf(&dummy_buf, sizeof(dummy_buf), TIAP_GENERIC_IOCTL_SET_LOGGED_CHARS, ifname, logged_chars, 0, NULL);
}

#endif

int ioctl_get_essid(char *essid, int ioctl_sock, char *ifname) {
	struct iwreq		wrq;

	if (essid)
	{
		wrq.u.essid.pointer = (caddr_t) essid;
		wrq.u.essid.length = IW_ESSID_MAX_SIZE + 1;
		iw_get_ext(ioctl_sock, ifname, SIOCGIWESSID, &wrq);
	}
	else ABORT_NULL_ARG("Get ESSID", SIOCGIWESSID);
	return 0;
	
}

static int ioctl_set_rts(const __s32 *rts, int ioctl_sock, char *ifname)
{
	 
	struct iwreq		wrq;
	  
	if (rts)
	{
		wrq.u.rts.value = *rts;			
		IW_SET_EXT_ERR(ioctl_sock, ifname, SIOCSIWRTS, &wrq, __func__);
	}
	else ABORT_NULL_ARG("Set RTS", SIOCSIWRTS);

	return 0;

}

int ioctl_get_rts(__s32 *rts,int ioctl_sock, char *ifname)
{
	struct iwreq		wrq;
	if (rts)
	{
		if (iw_get_ext(ioctl_sock, ifname, SIOCGIWRTS, &wrq) >= 0)
			*rts = wrq.u.rts.value;
	
	}
	else ABORT_NULL_ARG("Get RTS", SIOCGIWRTS);

	return 0;
}

static int ioctl_set_frag(const __s32 *frag, int ioctl_sock, char *ifname)
{
	 
	struct iwreq		wrq;

	if (frag)
	{
		
		wrq.u.frag.value = *frag;
		IW_SET_EXT_ERR(ioctl_sock, ifname, SIOCSIWFRAG, &wrq, __func__);

	}
	else ABORT_NULL_ARG("Set FRAG", SIOCSIWFRAG);

	return 0;

}

int ioctl_get_frag(__s32 *frag, int ioctl_sock, char *ifname)
{

	struct iwreq		wrq;
	if (frag)
	{
		if(iw_get_ext(ioctl_sock, ifname, SIOCGIWFRAG, &wrq) >= 0)
			*frag = wrq.u.frag.value;
    
	}
	else ABORT_NULL_ARG(__func__, SIOCGIWFRAG);

	return 0;
}

static int ioctl_set_retry(const __s32 *retry, const int type, int ioctl_sock, char *ifname)
{
	struct iwreq		wrq;

	if (retry)
	{
		
		wrq.u.retry.value = *retry;
		wrq.u.retry.flags = type;
		IW_SET_EXT_ERR(ioctl_sock, ifname, SIOCSIWRETRY, &wrq, __func__);

	}
	else ABORT_NULL_ARG(__func__, SIOCSIWRETRY);

	return 0;

}

int ioctl_get_retry(__s32 *retry, int type, int ioctl_sock, char *ifname)
{
	struct iwreq		wrq;

	if (retry)
	{
		wrq.u.retry.flags = type;
		if(iw_get_ext(ioctl_sock, ifname, SIOCGIWRETRY, &wrq) >= 0)
			*retry = wrq.u.retry.value;
	}
	else ABORT_NULL_ARG(__func__, SIOCGIWRETRY);

	return 0;

}


static int ioctl_set_txpow(tiap_param_txpower_t *txpow, int ioctl_sock, char *ifname)
{
	struct iwreq		wrq;
	
	memcpy (&(wrq.u.txpower), (struct iw_param *)txpow, sizeof (struct iw_param));

	IW_SET_EXT_ERR(ioctl_sock, ifname, SIOCSIWTXPOW, &wrq, __func__);

	return 0;
}


int ioctl_get_txpow(tiap_param_txpower_t *txpow, int ioctl_sock, char *ifname)
{
	struct iwreq		wrq;
	if (txpow)
	{
		memcpy(&(wrq.u.txpower), txpow, sizeof(tiap_param_txpower_t));
		if(iw_get_ext(ioctl_sock, ifname, SIOCGIWTXPOW, &wrq) >= 0)
			memcpy(txpow, &(wrq.u.txpower), sizeof(tiap_param_txpower_t));
	}
	else ABORT_NULL_ARG(__func__, SIOCGIWTXPOW);

	return 0;
}



int exec_priv_ioctl_set(const __u32 we_id,
			       void *param,
			       int param_size,
			       int ioctl_sock, char *ifname)
{
	int		num_ioctls,
			k,j,
			offset = 0,
			subioctl = 0;
	iwprivargs	priv_tbl[IW_MAX_PRIV_DEF];
	u_char		buffer[4096]	;
	struct iwreq	wrq;

	num_ioctls = read_priv_ioctls(ioctl_sock, ifname, priv_tbl);
	if ( num_ioctls <= 0)
		return -1;

	 /* Search the correct ioctl */
	k = -1;
	while((++k < num_ioctls) && (priv_tbl[k].cmd != we_id));
	
	/* If not found... */
	if(k == num_ioctls)
	{
		fprintf(stderr, "No such WE ID : %d\n", we_id);
		return(-1);
	}

	/* Watch out for sub-ioctls ! */

	if(priv_tbl[k].cmd < SIOCDEVPRIVATE)
	{

		j = -1;
		/* Find the matching *real* ioctl */
		 while((++j < num_ioctls) && ((priv_tbl[j].name[0] != '\0') 
					  || (priv_tbl[j].set_args != priv_tbl[k].set_args)));

		/* If not found... */
		if(j == num_ioctls)
		{
			fprintf(stderr, "Invalid private ioctl definition for : %d\n",we_id);
			return(-1);
		}

		/* Save sub-ioctl ID */
		subioctl = priv_tbl[k].cmd;
		/* Reserve one int (simplify alignement issues) */
		offset = sizeof(__u32);
		/* Use real ioctl definition from now on */
		k = j;
	
	}

	memcpy(buffer, param, param_size);
	
	strncpy(wrq.ifr_name, ifname, IFNAMSIZ);

	/* ThIs test is important. It defines how the driver
	 * will have to handle the data */
	
	  if( (priv_tbl[k].set_args & IW_PRIV_SIZE_FIXED) &&
	      (iw_get_priv_size(priv_tbl[k].set_args) + offset <= IFNAMSIZ))
	  {
		  /* First case : all SET args fit within wrq */
		  if(offset)
			wrq.u.mode = subioctl;
		  memcpy(wrq.u.name + offset, buffer, IFNAMSIZ - offset);
	  }
	  else
	  {
		  
		  /* Second case : arg won't fit in wrq */
		  wrq.u.data.pointer = (caddr_t) buffer;
		  wrq.u.data.length = param_size;
		  wrq.u.data.flags = subioctl;
	  }

	  /* Perform the private ioctl */
	  if(ioctl(ioctl_sock, priv_tbl[k].cmd, &wrq))
	  {
		fprintf(stderr, "Interface doesn't accept private ioctl...\n");
		fprintf(stderr, "(%X): %d: %s\n", priv_tbl[k].cmd, subioctl, strerror(errno));
		return(-1);
	  }


	return 0;
}

int exec_priv_ioctl_get(const __u32 we_id, /* private ioctl ID */
			       void *param,       /* ioctl parameter */
			       int param_size,	  /* parameter size */
			       int ioctl_sock,
			       char *ifname)
{
	int		num_ioctls,
			k,j, ret,
			offset = 0,
			subioctl = 0;
	iwprivargs	priv_tbl[IW_MAX_PRIV_DEF];
	u_char		buffer[4096]	;
	struct iwreq	wrq;

	num_ioctls = read_priv_ioctls(ioctl_sock, ifname, priv_tbl);
	if ( num_ioctls <= 0)
		return -1;

	/*for (k=0; k<num_ioctls; k++)
		printf("%X %s\n", priv_tbl[k].cmd, priv_tbl[k].name);*/
	 /* Search the correct ioctl */
	k = -1;
	while((++k < num_ioctls) && (priv_tbl[k].cmd != we_id ||
				     priv_tbl[k].get_args == 0));
	
	/* If not found... */
	if(k == num_ioctls)
	{
		fprintf(stderr, "No such WE ID : %d\n", we_id);
		return 1;
	}

	/* Watch out for sub-ioctls ! */
	if(priv_tbl[k].cmd < SIOCDEVPRIVATE)
	{

		j = -1;
		/* Find the matching *real* ioctl */
		 while((++j < num_ioctls) && ((priv_tbl[j].name[0] != '\0') 
					  || (priv_tbl[j].get_args != priv_tbl[k].get_args)));

		/* If not found... */
		if(j == num_ioctls)
		{
			fprintf(stderr, "Invalid private ioctl definition for : %d\n",we_id);
			return 1;
		}
	

		/* Save sub-ioctl ID */
		subioctl = priv_tbl[k].cmd;
		/* Reserve one int (simplify alignement issues) */
		offset = sizeof(__u32);
		/* Use real ioctl definition from now on */
		k = j;
	}
	strncpy(wrq.ifr_name, ifname, IFNAMSIZ);

	/* This test is important. It defines how the driver
	 * will have to handle the data */
	
	  if( (priv_tbl[k].get_args & IW_PRIV_SIZE_FIXED) &&
	      (iw_get_priv_size(priv_tbl[k].get_args) <= IFNAMSIZ))
	  {
		  /* First case : all GET args fit within wrq */
		  if(offset)
			wrq.u.mode = subioctl;
	  }
	  else
	  {
		  /* Second case : arg won't fit in wrq */
		  memcpy(buffer, param, min(sizeof(buffer),param_size));
		  wrq.u.data.pointer = (caddr_t) buffer;
		  wrq.u.data.flags = subioctl;
	  }

	  /* Perform the private ioctl */
	  if( (ret=ioctl(ioctl_sock, priv_tbl[k].cmd, &wrq)) )
	  {
		fprintf(stderr, "Interface doesn't accept private ioctl...\n");
		fprintf(stderr, "(%X): %s\n", priv_tbl[k].cmd, strerror(errno));
		return ret;
	  }


	   /* Check where is the returned data */
	  if((priv_tbl[k].get_args & IW_PRIV_SIZE_FIXED) &&
	       (iw_get_priv_size(priv_tbl[k].get_args) <= IFNAMSIZ))
	  {
		memcpy(buffer, wrq.u.name, IFNAMSIZ);
	  }
	  /* else the returned data is already in the buffer */

	  memcpy(param, buffer, param_size);
	
	 return 0;
}


/*
 * read the all the private ioctls of the given device into the table
 */
static int read_priv_ioctls( int ioctl_sock,   /* Socket */
				       char *ifname,     /* Dev name */
				       iwprivargs *priv) /* dest ioctls table */
{
  int		number;		/* Max of private ioctl */

  /* Read all the private ioctls */
  number = iw_get_priv_info(ioctl_sock, ifname, priv, IW_MAX_PRIV_DEF);

  /* Is there any ? */
  if(number <= 0)
    {
      /* Could skip this message ? */
      fprintf(stderr, "%-8.8s  no private ioctls.\n\n",ifname);
      return(-1);
    }

  return number;
}

int wcfg_ioctl_init(wcfgd *wcfg_data)
{
	struct wcfg_storage_t ldb = wcfg_data->storage.ldb;
	int ioctl_sock, ret = 0;
	__s32 replev;
	__s8 i;
	__u8 band;

	/* Create ioctls socket */
	if((ioctl_sock = iw_sockets_open()) < 0)
	{
		perror("socket");
		return(-1);
	}

	/* Call the ioctls */
	/* Attention: the order of ioctl calls matters !!! */


	#ifdef TIWLAN_DEBUG
	/* burst limit - can be configured only in the  DEBUG version */ 
	if ( (ret = ioctl_set_burst_limit(ldb.txop_limit, ioctl_sock, wcfg_data->dev_name)))
		return ret;
	#endif
	
	/* hardware address */
	if ( (ret = ioctl_set_hwaddr(ldb.hwaddr, ioctl_sock, wcfg_data->dev_name)))
		return ret;

	/* standard */
	if ( (ret = ioctl_set_essid(ldb.essid, ioctl_sock, wcfg_data->dev_name)))
		return ret;
	if ( (ret = ioctl_set_rts((__s32 *)&ldb.rts, ioctl_sock, wcfg_data->dev_name)))
		return ret;
	if ( (ret = ioctl_set_retry((__s32 *)&ldb.retry_long, TIAP_PARAM_RETRY_LONG, 
		ioctl_sock, wcfg_data->dev_name)))/*long*/
		return ret;
	if ( (ret = ioctl_set_retry((__s32 *)&ldb.retry_short, TIAP_PARAM_RETRY_SHORT, 
		ioctl_sock, wcfg_data->dev_name))) /*short*/
		return ret;
	if ( (ret = ioctl_set_retry((__s32 *)&ldb.lifetime_tx, TIAP_PARAM_LIFETIME_TX, 
		ioctl_sock, wcfg_data->dev_name)))/*tx*/
		return ret;
	if ( (ret = ioctl_set_retry((__s32 *)&ldb.lifetime_rx, TIAP_PARAM_LIFETIME_RX,
		ioctl_sock, wcfg_data->dev_name))) /*rx*/
		return ret;
	if ( (ret = ioctl_set_frag((__s32 *)&ldb.frag, ioctl_sock, wcfg_data->dev_name)))
		return ret;
	if ( (ret = ioctl_set_txpow((tiap_param_txpower_t *)&ldb.txpow_a, ioctl_sock, wcfg_data->dev_name)))
		return ret;
	if ( (ret = ioctl_set_txpow((tiap_param_txpower_t *)&ldb.txpow_g, ioctl_sock, wcfg_data->dev_name)))
		return ret;

	/* basic */

	if (ldb.replev == 0){
		i = 0;
		if( (ret = exec_priv_ioctl_set(TIAP_PARAM_REPLEV, &i, sizeof(i),
						ioctl_sock, wcfg_data->dev_name)))
			return ret;
	}
	else
	{
		for(i = 1; i < MAX_REPLEV - 1; i ++) 
		{
			replev = ldb.replev & (1 << i);
			if (replev)
			{
				if( (ret = exec_priv_ioctl_set(TIAP_PARAM_REPLEV, &i, 
					 sizeof(__s8), ioctl_sock, wcfg_data->dev_name)))
					return ret;
			}
		}
		
	}
	
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_PRIV_OPTION, (__u8 *)&ldb.priv_opt, 
			    sizeof(__u8), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_BEACON_PERIOD, (__u16 *)&ldb.beacon_period, 
			    sizeof(__u16), ioctl_sock, wcfg_data->dev_name)))
		return ret;
	
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_DTIM_PERIOD, (__u8 *)&ldb.dtim_period, 
			    sizeof(__u8), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_BSS_BRIDGE_STATE, (tiap_param_bss_br_state_t *)&ldb.bss_bridge_state, 
			    sizeof(tiap_param_bss_br_state_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_TIME_MODE, (__u8 *)&ldb.wone_timemod, 
			    sizeof(__u8), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_RATE_FALLBACK, (__u8 *)&ldb.rate_fallback, 
		    sizeof(__u8), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_NUM, (__u8 *)&ldb.wone_num, 
			    sizeof(__u8), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_RADIO_CAL, (__u32 *)&ldb.radio_cal, 
			    sizeof(__u32), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_RADIO_CAL_INT, (__u16 *)&ldb.cal_interval, 
			    sizeof(__u16), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_ENERGY_DETECT, (__u8 *)&ldb.energy_detect, 
			    sizeof(__u8), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_HIDDEN_SSID, (__u16 *)&ldb.hidden_ssid, 
			    sizeof(__u16), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_MULTI_DOMAIN_CAPAB, (__u8 *)&ldb.multi_domain_cap, 
			    sizeof(__u8), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_SPECTRUM_MGMT, (__u8 *)&ldb.spectrum_mgmt, 
			    sizeof(__u8), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_LEGACY_STA, (__u8 *)&ldb.legacy_sta, 
			    sizeof(__u8), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_LOCAL_POWER_CONS, (__s8 *)&ldb.local_pow_cons, 
			    sizeof(__s8), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_MAX_TX_POWER_LEVEL, (__u8 *)&ldb.max_txpow_lev, 
			    sizeof(__u8), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_COUNTRY_STR, (__s8 *)&ldb.country_str, 
			    TIAP_PARAM_COUNTRY_STRING_LEN, ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_OPER_STATE, (__u8 *)&ldb.oper_state, 
			    sizeof(__u8), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_SECURITY_TYPE,
			    &wcfg_data->storage.wpa.type.sec.type,
			    sizeof(wcfg_data->storage.wpa.type.sec.type),
			    ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_RADAR_DETECT_MODE, (__u8 *)&ldb.radar_detect,
					sizeof(__u8), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_AP_BRIDGE_MODE, (__u8 *)&ldb.bridge_mode,
					sizeof(__u8), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	/*complex*/

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_4X,
		    &ldb._4x_4x, sizeof(tiap_param_4x_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_4X,
		    &ldb._4x_fs, sizeof(tiap_param_4x_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_4X,
		    &ldb._4x_cat, sizeof(tiap_param_4x_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_4X,
		    &ldb._4x_pb, sizeof(tiap_param_4x_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_4X,
		    &ldb._4x_mm, sizeof(tiap_param_4x_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_AUTH_TYPE,
		    ldb.auth_tbl, sizeof(ldb.auth_tbl), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WD,
		    &ldb.wd_en, sizeof(tiap_param_wd_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WD,
		    &ldb.wd_rxfree, sizeof(tiap_param_wd_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WD,
		    &ldb.wd_beacons, sizeof(tiap_param_wd_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WD,
		    &ldb.wd_txstuck, sizeof(tiap_param_wd_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WD,
		    &ldb.wd_fcserrs, sizeof(tiap_param_wd_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WD,
		    &ldb.wd_aci, sizeof(tiap_param_wd_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;
	
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WD,
		    &ldb.wd_nb, sizeof(tiap_param_wd_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;
	
	{/*Initialize interference WD state machine parameters */
		tiap_param_interference_wd_t interf_wd_param;

		interf_wd_param.cmd 		= TIAP_PARAM_PD_LOW;
		interf_wd_param.value.val16 = ldb.pd_low_thresh;
		if( (ret = exec_priv_ioctl_set(TIAP_PARAM_INTERF_WD_CONFIG,&interf_wd_param,
			       sizeof(tiap_param_interference_wd_t), ioctl_sock, wcfg_data->dev_name)))
			return ret;

		
		interf_wd_param.cmd 		 = TIAP_PARAM_PD_HIGH;
		interf_wd_param.value.val16  = ldb.pd_high_thresh;
		if( (ret = exec_priv_ioctl_set(TIAP_PARAM_INTERF_WD_CONFIG,&interf_wd_param,
			       sizeof(tiap_param_interference_wd_t), ioctl_sock, wcfg_data->dev_name)))
			return ret;

		interf_wd_param.cmd 		= TIAP_PARAM_ACI_TIMEOUT;
		interf_wd_param.value.val32 = ldb.aci_state_timeout;
		if( (ret = exec_priv_ioctl_set(TIAP_PARAM_INTERF_WD_CONFIG,&interf_wd_param,
			       sizeof(tiap_param_interference_wd_t), ioctl_sock, wcfg_data->dev_name)))
			return ret;

		interf_wd_param.cmd 		= TIAP_PARAM_NB_TIMEOUT;
		interf_wd_param.value.val32 = ldb.nb_state_timeout;
		if( (ret = exec_priv_ioctl_set(TIAP_PARAM_INTERF_WD_CONFIG,&interf_wd_param,
			       sizeof(tiap_param_interference_wd_t), ioctl_sock, wcfg_data->dev_name)))
			return ret;

		interf_wd_param.cmd = TIAP_PARAM_ACI_RATE_THR;
		interf_wd_param.value.val32 = ldb.aci_rate_thresh;
		if( (ret = exec_priv_ioctl_set(TIAP_PARAM_INTERF_WD_CONFIG,&interf_wd_param,
			       sizeof(tiap_param_interference_wd_t), ioctl_sock, wcfg_data->dev_name)))
			return ret;

		interf_wd_param.cmd = TIAP_PARAM_ACI_EVENT_THR;
		interf_wd_param.value.val32 = ldb.aci_event_thresh;
		if( (ret = exec_priv_ioctl_set(TIAP_PARAM_INTERF_WD_CONFIG,&interf_wd_param,
			       sizeof(tiap_param_interference_wd_t), ioctl_sock, wcfg_data->dev_name)))
			return ret;

		interf_wd_param.cmd = TIAP_PARAM_MEDIUM_BUSY_THR;
		interf_wd_param.value.val32 = ldb.medium_busy_thresh;
		if( (ret = exec_priv_ioctl_set(TIAP_PARAM_INTERF_WD_CONFIG,&interf_wd_param,
			       sizeof(tiap_param_interference_wd_t), ioctl_sock, wcfg_data->dev_name)))
			return ret;

	}/*interference WD config*/

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_ACCESS_LIST,
		    &ldb.access_list, sizeof(tiap_param_access_list_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_MULTI_SSID,
		    &ldb.multi_ssid, sizeof(ldb.multi_ssid), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_CFG_POWER_TABLE,
		    &ldb.radio, sizeof(ldb.radio), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_CHANNEL_RANGE,
		    &ldb.ch_rang_a, sizeof(tiap_param_channel_rang), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_CHANNEL_RANGE,
		    &ldb.ch_rang_g, sizeof(tiap_param_channel_rang), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_VHCF,
		    &ldb.vhcf, sizeof(tiap_param_vhcf_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_PRIV_REG_DOMAIN,
		    &ldb.priv_regdomain, sizeof(ldb.priv_regdomain), ioctl_sock, wcfg_data->dev_name)))
		return ret;


	band = ldb.band[0].elem.band;
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_CURR_REG_DOMAIN,
		    &ldb.cur_regdomain[band], sizeof(ldb.cur_regdomain[band]), ioctl_sock, wcfg_data->dev_name)))
		return ret;


	/*wone 0*/
	
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.dot11_mode[0], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.oper_rate[0], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;
	
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.oper_add[0], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.oper_ofdm[0], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.ch[0], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.band[0], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;
	
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.pbcc[0], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.rate_adapt[0], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.pream_mode[0], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.slot_mode[0], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.erp_mode[0], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.erp_type[0], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.erp_ti[0], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;
	
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.cwmin_mode[0], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.usage_time[0], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	/*wone 1*/
	
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.dot11_mode[1], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.oper_rate[1], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;
	
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.oper_add[1], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.oper_ofdm[1], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.ch[1], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.band[1], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;
	
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.pbcc[1], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.rate_adapt[1], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.pream_mode[1], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.slot_mode[1], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.erp_mode[1], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.erp_type[1], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.erp_ti[1], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.cwmin_mode[1], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;
	if( (ret = exec_priv_ioctl_set(TIAP_PARAM_WONE_ELEM,
		    &ldb.usage_time[1], sizeof(tiap_param_wone_t), ioctl_sock, wcfg_data->dev_name)))
		return ret;

	if ( (ret = ioctl_set_gpio_owner(ldb.gpio_owner, ioctl_sock, wcfg_data->dev_name)))
		return ret;


	if (strlen(wcfg_data->fw_filename) > 0) {
		int fd, buf_len;
		char buf[MAX_FW_SIZE];

		fd = open(wcfg_data->fw_filename, O_RDONLY);
		if (fd < 0) {
			printf("Can not read FW file %s! (%s)", wcfg_data->fw_filename, strerror(errno));
			return -1;
		}
		
		buf_len = read(fd, buf, MAX_FW_SIZE);

		if (buf_len <= 0 || buf_len == MAX_FW_SIZE) {
			printf("Weird FW file size : %d!", buf_len);
			return -1;
		}
						 
		close(fd);

		if (generic_ioctl_get_buf(buf, buf_len, TIAP_GENERIC_IOCTL_UPLOAD_FW, 
					wcfg_data->dev_name, 0, 0, NULL) != 0)
			return -1;
	} else
		puts("NOT sending FW on init!");

	
/*goto go;
go:*/
	if ( (ret = exec_priv_ioctl_set(DDA_IOCTL_GO, NULL, 
			    0, ioctl_sock, wcfg_data->dev_name)))
		return ret;

	close (ioctl_sock);
	return ret;


}
