/*
 * WCFG         An implementation of the wcfglib library.
 *		See wcfg_lib companion document.
 *
 *
 * Version:     0.1.0
 *
 * Authors:     Alexandra Aronov, <alexandraa@ti.com>
 *              Felix Rubinstein, <felixr@ti.com>
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>

#include "wcfg_storage.h"
#include "wcfglib.h"

int set_dot11mode(struct wcfg_storage_t *, int dot11mode, int idx);

int wcfg_storage_read(struct storage *storage, const char *storage_path)
{
int fd, nread, expected;

	if( (fd = open(storage_path, O_RDONLY)) == -1 ) {
		fprintf(stderr, "Failed to open %s file: %s\n", storage_path,  strerror(errno));
		return 1;
	}

	expected = sizeof(*storage);
	if( (nread = read(fd, (void *)storage, expected)) == -1 ) {
		fprintf(stderr, "Failed to read %s: %s\n", WCFG_MTDBLOCK, strerror(errno));
		close(fd);
		return 1;
	}
	else if( nread != expected ) {
		fprintf(stderr, "%s: could not read all data (%d %d)\n", __func__, nread, expected);
		close(fd);
		return 1;
	}

	close(fd);
	return 0;
}


/* wcfg_storage_save: save storage in pstorage_path */
int wcfg_storage_write(const struct storage *storage, const char *storage_path)
{
int fd, nwritten, expected;

	if( (fd = open(storage_path, O_WRONLY)) == -1 ) {
		fprintf(stderr, "Failed to open %s: %s\n", storage_path, strerror(errno));
		return 1;
	}

	((struct storage *)storage)->magic = WCFG_MAGIC;
	expected = sizeof(*storage);
	if( (nwritten = write(fd, (const void *)storage, expected)) == -1 ) {
		fprintf(stderr, "Failed to write to %s: %s\n", storage_path, strerror(errno));
		close(fd);
		return 1;
	}
	else if( nwritten != expected ) {
		fprintf(stderr, "%s: could not save all data (%d %d)\n", __func__, nwritten, expected);
		close(fd);
		return 1;
	}

	close(fd);
	return 0;
}

int get_random_byte(__u8 *r)
{
	int fd;

	if( (fd = open("/dev/urandom", O_RDONLY)) == -1 ) {
		fprintf(stderr, "Failed to open /dev/urandom: %s\n", strerror(errno));
		return 1;
	}

	if( read(fd, r, sizeof(*r)) == -1 ) {
		fprintf(stderr, "Failed to read /dev/urandom: %s\n", strerror(errno));
		close(fd);
		return 1;
	}

	close(fd);
	return 0;
}

int read_hwaddr_from_bootloader(char *hwaddr)
{
	int fd, nread, i;
	char buf[2000], *p;

	if( (fd = open(WCFG_BOOTLOADER, O_RDONLY)) == -1 ) {
		fprintf(stderr, "Failed to open %s file: %s\n", WCFG_BOOTLOADER,  strerror(errno));
		return 1;
	}

	if( (nread = read(fd, (void *)buf, sizeof(buf))) == -1 ) {
		fprintf(stderr, "Failed to read %s: %s\n", WCFG_BOOTLOADER, strerror(errno));
		close(fd);
		return 1;
	}
	close(fd);


	if( !(p = strstr(buf, WCFG_WLAN_HWADDR)) ) {
		fprintf(stderr, "Boot-loader environment variable %s is not set, "
				"cannot configure wireless LAN driver hardware address\n",
				WCFG_WLAN_HWADDR);
		return 1;
	}

	if( !(p = strtok(p, " \t")) ) {
		fprintf(stderr, "Invalid format of %s environment variable\n", WCFG_WLAN_HWADDR);
		return 1;
	}

	for(i=0; i<ETH_ALEN; i++)
	{
		if( !(p = strtok(NULL, " \t:")) ) {
			fprintf(stderr, "Invalid format of %s environment variable\n", WCFG_WLAN_HWADDR);
			return 1;
		}
		sscanf(p, "%hhx", &hwaddr[i]);
	}
	
	return 0;
}


void wcfg_storage_setdefaults_ldb(struct wcfg_storage_t *ldb)
{
	if( read_hwaddr_from_bootloader(ldb->hwaddr) )
	{
		memset(ldb->hwaddr, 0, sizeof(ldb->hwaddr));
		/* first four octets belongs to TI */
		ldb->hwaddr[0] = 0x0;
		ldb->hwaddr[1] = 0x50;
		ldb->hwaddr[2] = 0xF1;
		ldb->hwaddr[3] = 0x12;
#if 0
		{
			__u8 r;
			if( get_random_byte(&r) ) r = 0;
			ldb->hwaddr[4] = r;
			if( get_random_byte(&r) ) r = 0;
			ldb->hwaddr[5] = r;
		}
#endif
	}

	ldb->gpio_owner = 1;
	
	ldb->ipaddr = inet_addr("192.168.1.1");
	ldb->netmask = inet_addr("255.255.255.0");
	ldb->default_gw = inet_addr("192.168.1.100");
	
	/*standard*/
	strncpy(ldb->essid, "TI", IW_ESSID_MAX_SIZE); ldb->essid[IW_ESSID_MAX_SIZE]='\0';
	ldb->rts = 2347;
	ldb->frag = 2346;
	ldb->retry_long = 4;
	ldb->retry_short = 7;
	ldb->lifetime_tx = 512; ldb->lifetime_rx = 512;
	
	ldb->txpow_a.hr = txpower_a;
	{
	__s8 txpower[TIAP_PARAM_NUM_TXPOW_LEVELS] = {11, 10, 9, 7, 5};
	memcpy((void *)&ldb->txpow_a.txpower, (void *)txpower, sizeof(ldb->txpow_a.txpower));
	}
	ldb->txpow_g.hr = txpower_g;
	{
	__s8 txpower[TIAP_PARAM_NUM_TXPOW_LEVELS] = {19, 18, 16, 15, 14};
	memcpy((void *)&ldb->txpow_g.txpower, (void *)txpower, sizeof(ldb->txpow_g.txpower));
	}
		
	/*private*/

	/*basic*/
	ldb->replev = 0;
	ldb->priv_opt = 0;
	ldb->beacon_period = 200;
	ldb->dtim_period = 2;
	ldb->bss_bridge_state = TIAP_PARAM_BSS_BR_ENABLE;
	ldb->rate_fallback = 0;
	ldb->radio_cal = 0x1;
	ldb->cal_interval = 300;
	ldb->energy_detect = 1;
	ldb->hidden_ssid = 0;
	ldb->multi_domain_cap = 0;
	ldb->spectrum_mgmt = 0;
	ldb->legacy_sta = 0;
	ldb->local_pow_cons = 0;
	ldb->max_txpow_lev = 1;

	memcpy(ldb->country_str, "US ", TIAP_PARAM_COUNTRY_STRING_LEN);

	ldb->oper_state = 1;
	ldb->wone_num = 1;
	ldb->wone_timemod = 1;
	ldb->radar_detect = 0;
	ldb->txop_limit = 3000;
	ldb->bridge_mode = 0;

	/*complex*/

	ldb->auth_tbl[0].algo = TIAP_PARAM_MIB_OPEN_SYSTEM;
	ldb->auth_tbl[0].enable = 1;
	ldb->auth_tbl[1].algo = TIAP_PARAM_MIB_SHARED_KEY;
	ldb->auth_tbl[1].enable = 0;
	
	ldb->wd_en.cmd = TIAP_PARAM_WD_ENABLE; ldb->wd_en.value.val16 = 1;
	ldb->wd_rxfree.cmd = TIAP_PARAM_WD_RXFREE; ldb->wd_rxfree.value.val16 = 0;
	ldb->wd_beacons.cmd = TIAP_PARAM_WD_BEACONS; ldb->wd_beacons.value.val16 = 3;
	ldb->wd_txstuck.cmd = TIAP_PARAM_WD_TXSTUCK; ldb->wd_txstuck.value.val16 = 5;
	ldb->wd_fcserrs.cmd = TIAP_PARAM_WD_FCSERR; ldb->wd_fcserrs.value.val16 = 1;
	ldb->wd_aci.cmd = TIAP_PARAM_WD_ACI; ldb->wd_aci.value.val8 = 1;
	ldb->wd_nb.cmd = TIAP_PARAM_WD_NB; ldb->wd_nb.value.val8 = 1;

	ldb->aci_state_timeout 	= 60; /* seconds */
	ldb->nb_state_timeout 	= 300; /* seconds */
	ldb->aci_rate_thresh 	= 50;
	ldb->aci_event_thresh   = 4;
	ldb->pd_low_thresh 		= 0x0090;
	ldb->pd_high_thresh		= 0x00D5;
	ldb->medium_busy_thresh = 75; /* percents */
		
	memset(&ldb->access_list, 0, sizeof(ldb->access_list));
	ldb->access_list.cmd = TIAP_PARAM_ACCESS_RESET;

	ldb->_4x_fs.cmd = TIAP_PARAM_4X_FEATSET; ldb->_4x_fs.value = 1;
	ldb->_4x_4x.cmd = TIAP_PARAM_4X_4X; ldb->_4x_4x.value = 0;
	ldb->_4x_cat.cmd = TIAP_PARAM_4X_CAT; ldb->_4x_cat.value = 0;
	ldb->_4x_pb.cmd = TIAP_PARAM_4X_PKTBURST; ldb->_4x_pb.value = 0;
	ldb->_4x_mm.cmd = TIAP_PARAM_4X_MIXMOD; ldb->_4x_mm.value = 0;
		
	memset(&ldb->multi_ssid, 0, sizeof(ldb->multi_ssid));
	ldb->multi_ssid.cmd = TIAP_PARAM_MULTI_SSID_RESET;
		
	ldb->radio.mask = 0;
	ldb->radio.tbl.type = TIAP_PARAM_PWRTBL;
	{
#if defined(CONFIG_MIPS_AR7WI) || defined(CONFIG_MIPS_AR7VWI) /* CRT radio */
	__s8 param[TIAP_PARAM_MAX_RADIO_PARAM_POW_TBL] = {0,2,3, 0, 0, 1, 2,-1,-1, 0, 0,-3,-3,-3,-3,-5,-5, -5, -5, -5};
#else
	__s8 param[TIAP_PARAM_MAX_RADIO_PARAM_POW_TBL] = {0,2,31,0,-3,-1, 1,-3,-6,-4,-2,-6,-9,-7,-5,-9,-12,-10,-8,-12};
#endif
	memcpy((void *)&ldb->radio.tbl.param, (void *)param, sizeof(ldb->radio.tbl.param));
	}
	
	memset(&ldb->ch_rang_a, 0, sizeof(ldb->ch_rang_a));
	ldb->ch_rang_a.type = TIAP_PARAM_RANG_A;
	memset(&ldb->ch_rang_g, 0, sizeof(ldb->ch_rang_g));
	ldb->ch_rang_g.type = TIAP_PARAM_RANG_G;
	
	memset(&ldb->vhcf, 0, sizeof(ldb->vhcf));

	ldb->priv_regdomain.band = BAND_DEFAULT;
	ldb->priv_regdomain.domain = 0;

	ldb->cur_regdomain[0].band = 0;
	ldb->cur_regdomain[0].type = 0x10;

	ldb->cur_regdomain[1].band = 1;
	ldb->cur_regdomain[1].type = 0x50;

	/*wone index 0*/
	ldb->ch[0].index = 0;
	ldb->ch[0].type = wone_ch;
	ldb->ch[0].elem.ch = TIAP_PARAM_DEF_CHANNEL;
	
	ldb->band[0].index = 0;
	ldb->band[0].type = wone_band;
	ldb->band[0].elem.band = 0;
	
	ldb->rate_adapt[0].index = 0;
	ldb->rate_adapt[0].type = wone_rate_adapt;
	ldb->rate_adapt[0].elem.rate_adapt = 1;

	ldb->erp_ti[0].index = 0;
	ldb->erp_ti[0].type = wone_erp_ti; 
	ldb->erp_ti[0].elem.erp_ti = 0; 

	ldb->dot11_mode[0].index = 0;
	ldb->dot11_mode[0].type = wone_dot11_mode;
	ldb->dot11_mode[0].elem.dot11_mode = 0;

	ldb->cwmin_mode[0].index = 0; 
	ldb->cwmin_mode[0].type = wone_cwmin_mode; 
	ldb->cwmin_mode[0].elem.cwmin_mode = 2;

	ldb->usage_time[0].index = 0;
	ldb->usage_time[0].type = wone_usage_time;
	ldb->usage_time[0].elem.usage_time = 20;


	set_dot11mode(ldb, TIAP_PARAM_D11MODE_MIXED, 0);
	
	/*wone index 1*/
	ldb->ch[1].index = 1;
	ldb->ch[1].type = wone_ch;
	ldb->ch[1].elem.ch = TIAP_PARAM_DEF_A_CHANNEL;
	
	ldb->band[1].index = 1;
	ldb->band[1].type = wone_band;
	ldb->band[1].elem.band = 1;

	ldb->rate_adapt[1].index = 1;
	ldb->rate_adapt[1].type = wone_rate_adapt;
	ldb->rate_adapt[1].elem.rate_adapt = 1;

	ldb->erp_ti[1].index = 1;
	ldb->erp_ti[1].type = wone_erp_ti; 
	ldb->erp_ti[1].elem.erp_ti = 0; 

	ldb->dot11_mode[1].index = 1;
	ldb->dot11_mode[1].type = wone_dot11_mode;
	ldb->dot11_mode[1].elem.dot11_mode = 4;

	ldb->cwmin_mode[1].index = 1;
	ldb->cwmin_mode[1].type = wone_cwmin_mode; 
	ldb->cwmin_mode[1].elem.cwmin_mode = 2; 

	ldb->usage_time[1].index = 1;
	ldb->usage_time[1].type = wone_usage_time;
	ldb->usage_time[1].elem.usage_time = 20;

	set_dot11mode(ldb, TIAP_PARAM_D11MODE_A, 1);
}


void wcfg_storage_setdefaults_wpa(tiap_wpa_auth_t *wpa)
{
	int i;

	
	wpa->type.sec.interval = 3600 * 1000;
	wpa->type.sec.wep_def_key_id = 0;
	
	for (i=0; i<TIAP_WPA_DEFS_KEY_NAL_LEN; i++)
	{
		wpa->type.sec.keys[i].len = TIAP_WPA_WEP_40_KEY_LEN;
		memset(wpa->type.sec.keys[i].key, 0 , TIAP_WPA_WEP_KEY_LEN_MAX);
	}

	/* since wep+wpa mode is not supported security type wep means that wpa is disabled */
	wpa->type.sec.type = TIAP_PARAM_SECURITY_WEP; 

	wpa->type.sec.wep_auth_type = TIAP_WPA_WEPAUTH_OPEN;
	wpa->type.sec.wep_privacy = 0;
	
	memcpy(wpa->type.radius.secret, "RADIUS-SECRET",TIAP_WPA_MAX_RADIUS_SECRET_LEN);
	wpa->type.radius.secret_len = strlen("RADIUS-SECRET");
	
	wpa->type.radius.port = 1812;
	
	wpa->type.radius.serv_ip_addr = inet_addr("192.168.100.3");
	wpa->type.psk.pass_phrase_mode = TIAP_WPA_PSK_PARAM_NONE;
	wpa->type.psk.psk_passwd_len = TIAP_WPA_MIN_PSK_PWD_LEN;
	memcpy(wpa->type.psk.psk_passwd, "PSK12345", TIAP_WPA_MIN_PSK_PWD_LEN);
	wpa->hr = tiap_wpa_auth_psk;
	wpa->type.sec.default_cipher = TIAP_WPA_CIPHER_WEP_40;
	
}


void wcfg_storage_setdefaults(struct storage *stor)
{
	wcfg_storage_setdefaults_wpa(&stor->wpa);
	wcfg_storage_setdefaults_ldb(&stor->ldb);
}
