/*
 * WCFG         An implementation of the wcfglib library.
 *		See wcfg_lib companion document.
 *
 *
 * Version:     0.1.0
 *
 * Authors:     Alexandra Aronov, <alexandraa@ti.com>
 *              Felix Rubinstein, <felixr@ti.com>
 *
 */



#include <stdio.h>
#include <unistd.h>
#include <libgen.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include <sys/wait.h>
#include <stdarg.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <linux/route.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <linux/if_bridge.h>

#include "iwlib.h"
#include "wcfglib.h"
#include "dda_wext_ioctl.h"
#include "dda_wpaauth_ioctl.h"
#include "wcfg_ioctl.h"
#include "wcfg_net.h"

static tiap_wpa_auth_t init_wpa; /* stores initial wpa parameters */

static void set_replev_ldb(wcfgd *wd, __u8 replev);
static void unset_replev_ldb(wcfgd *wd, __u8 replev);
static void disable_replev_ldb(wcfgd *wd);
static int set_radar_detect(wcfgd *wd, __u8 val);

static int write_wpapid(pid_t pid)
{
	int fd;
	char buf[30];

	if( (fd = open(WCFG_WPAPIDFILE, O_CREAT|O_WRONLY|O_TRUNC)) == -1 ) {
		perror("open: ");
		return 1;
	}

	sprintf(buf, "%d\n", pid);

	if( write(fd, buf, strlen(buf)) == -1 ) {
		perror("write: ");
		close(fd);
		return 1;
	}

	close(fd);
	
	return 0;
}

static int daemonize(void)
{
	int nullfd;

	fflush(NULL);
	if( (nullfd=open("/dev/null", O_RDWR) == -1) ) {
		perror("open: ");
		return 1;
	}

	if (dup2(nullfd,0) < 0 || dup2(nullfd,1) < 0 || dup2(nullfd,2) < 0) {
		perror("dup2: ");
		return 1;
	}

	if( setsid() == -1 ) {
		perror("setsid: ");
		return 1;
	}

	return 0;
}

static pid_t start_wpaauth(const char *path)
{
	pid_t pid = 0, expected_pid;
	int status;
	int i;

	if( !path ) return -1;

	if( !strlen(path) ) return -1;

	if( daemonize() ) return -1;

	if( (pid = fork()) == -1 ) {
		perror("In parent: ");
		return -1;
	}
	else if( !pid )
	{
		execl(path, basename((char *)path), NULL);
		fprintf(stderr, "Could not execute %s, reason: %s\n", path, strerror(errno));
		if( getppid() == 1 ) /*I'm an orphan*/
		{
			char failure_file[_POSIX_PATH_MAX*2];
			int fd;
			/*Lets touch the file, for debug purpose only*/
			strcpy(failure_file, WCFG_PATHPREFIX);
			strcat(failure_file, path);
			strcat(failure_file, ".failure");
			fprintf(stderr, "Internal error: failed to start %s process "
					"and this failure is unknown to the parent.\n"
					"Dumping %s file.\n", path, failure_file);
			if( (fd = open(failure_file, O_CREAT|O_TRUNC)) == -1 )
				perror("Unfortunate day");
			else close(fd);
		}
		exit(1);
	}

	if( write_wpapid(pid) )
		return -1;

/* We wait WCFG_MAXWAIT seconds, maybe less if signal arrives,
   to verify that the process is execl'ed successfully.
   The calling process, wants to know if it failed to execl'ed.
   Note: there is a slight chance that we still wouldn't know it,
   for example we return and the fork'ed process is failed to execl'ed.
   But at least we know that the executable exists.
 */
	for(i=0, expected_pid=pid; i<WCFG_MAXWAIT; i++)
	{
		printf("%s: wait for the child pid = %d\n", __func__, expected_pid); fflush(NULL);
		if( (pid = waitpid(expected_pid, &status, WNOHANG)) == -1 ) {
			perror("Failed in waiting for the child, reason: ");
			return -1;
		}
		else if( !pid ) /*child has not exited yet*/
			sleep(1);
		else if( expected_pid == pid )
		{
			if( WIFEXITED(status) ) {
				if( WEXITSTATUS(status) == 1 ) /*since child did exit(1)*/
					fprintf(stderr, "The child execl'ed process is exited prematurely %d\n",
							WEXITSTATUS(status));
				else fprintf(stderr, "Internal error %d\n",
						WEXITSTATUS(status));
				return -1;
			}
			else if( WIFSIGNALED(status) ) {
				fprintf(stderr, "The child execl'ed process is %d\n",
						WTERMSIG(status));
				return -1;
			}
		}
		else {
			fprintf(stderr, "Bug: returned pid is not of child %d %d\n", pid, expected_pid);
			return -1;
		}
	}

	return expected_pid;
}

static pid_t read_wpapid(void)
{
	int fd, nbytes;
	char buf[30];
	pid_t pid;

	if( (fd = open(WCFG_WPAPIDFILE, O_RDONLY)) == -1 ) {
		fprintf(stderr, "Failed to open %s wpa pid file: %s\n", WCFG_WPAPIDFILE, strerror(errno));
		return -1;
	}

	if( (nbytes = read(fd, buf, sizeof(buf))) == -1 ) {
		fprintf(stderr, "Failed to read %s wpa pid file: %s\n", WCFG_WPAPIDFILE, strerror(errno));
		return -1;
	}

	if( !nbytes ) {
		fprintf(stderr, "Failed to read %s wpa file: %s\n", WCFG_WPAPIDFILE, strerror(errno));
		return -1;
	}

	/*get wpa authenticator pid*/
	sscanf(buf, "%d", &pid);

	return pid;
}


pid_t pseudo_start(const char *path)
{
pid_t pid;

	fflush(NULL);
	if( (pid = fork()) == -1) {
		perror("fork: ");
		return -1;
	}
	else if( pid > 0 ) /*parent*/
	{
		int i, status;
		pid_t expected_pid;

		sleep(WCFG_MAXWAIT);
		for(i=0, expected_pid=pid; i<WCFG_MAXWAIT+1; i++) 
		{
			if( (pid = waitpid(expected_pid, &status, WNOHANG)) == -1 ) {
				perror("Failed in waiting for the child, reason: ");
				return -1;
			}
			else if( !pid ) /*child has not exited yet*/
				sleep(1);
			else if( expected_pid == pid )
				return read_wpapid();
		}
		return -1;
	}
	else if( !pid ) { /*child*/
		start_wpaauth(path); exit(0);
       	}
	else {
		printf("\n\n\n\n\n\n\nERROR!!!\n\n\n\n\n\n\n\n"); fflush(NULL);
		return -2;
	}

	return -3;
}

/*mod_apdriver: insmod or rmmod the module specifies by file argument*/
static int mod_apdriver(const char *mod, const char *file)
{
	pid_t pid;
	int status;
	
	if( !strlen(file) )
		return 1;

	if( (pid = fork()) == -1 ) {
		fprintf(stderr, "wcfglib: %s\n", strerror(errno));
		return -1;
	}
	else if( !pid ) { /*child*/
		execlp(mod, mod, file, NULL);
		/*want to cover all cases call execl here with a full path to mod*/
		exit(1);
	}

	if( waitpid(pid, &status, 0) == -1 ) {
		fprintf(stderr, "wcfglib: %s\n", strerror(errno));
		return 1;
	}
	else if( WIFEXITED(status) )
		return WEXITSTATUS(status); /*in case of success this one must return 0*/
	else if( WIFSIGNALED(status) )
		return WTERMSIG(status);

	return 1;
}


static int wcfg_read_cfg(int fd, wcfgd *wd)
{
	char delim[]="<> \t";
	int nbytes;
	char buf[_POSIX_NAME_MAX+2*IFNAMSIZ+50], *p = NULL;


	if( !wd ) return 1;

	if( (nbytes = read(fd, buf, sizeof(buf))) == -1 ) {
		fprintf(stderr, "Failed to read %s configuration file: %s\n",
				WCFG_CFGFILE, strerror(errno));
		return 1;
	}

	if( !nbytes ) {
		fprintf(stderr, "Failed to read %s configuration file: %s\n",
				WCFG_CFGFILE, strerror(errno));
		return 1;
	}

	if( !(p = strtok(buf, delim)) ) {
		fprintf(stderr, "Invalid file format\n");
		return 1;
	}

	/*get wpa authenticator pid*/
	sscanf(p, "%d", &wd->wpaauth_pid);

	if( !(p = strtok(NULL, delim)) ) {
		fprintf(stderr, "Invalid file format\n");
		close(fd);
		return 1;
	}

	/*get ap module name*/
	strncpy(wd->apmodule_name, p, _POSIX_NAME_MAX);
	wd->apmodule_name[_POSIX_NAME_MAX] = '\0';

	if( !(p = strtok(NULL, delim)) ) {
		fprintf(stderr, "Invalid file format\n");
		return 1;
	}
	
	/*get driver device name*/
	strncpy(wd->dev_name, p, IFNAMSIZ-1);
	wd->dev_name[IFNAMSIZ-1] = '\0';

	if( !(p = strtok(NULL, delim)) ) {
		fprintf(stderr, "Invalid file format\n");
		return 1;
	}

	/*get bridge device name*/
	if( !isspace(*p) && *p != '\0' ) {
		strncpy(wd->br_name, p, IFNAMSIZ-1);
		wd->br_name[IFNAMSIZ-1] = '\0';
	}

	printf("wcfglib: Found: wpa pid=%d, ap module name %s, device ifname %s, bridge ifname %s\n", 
			wd->wpaauth_pid, wd->apmodule_name, wd->dev_name,
			strlen(wd->br_name)?wd->br_name:"");

	return 0;
}


static int wcfg_read_initcfg(wcfginit *init)
{
	char delim[]="<> \t";
	int fd, nbytes;
	char buf[2*_POSIX_NAME_MAX+2*IFNAMSIZ+50], *p = NULL;


	if( !init ) return 1;

	if( (fd = open(WCFG_INITCFGFILE, O_RDONLY)) == -1 ) {
		fprintf(stderr, "Failed to open %s init configuration file: %s\n",
				WCFG_INITCFGFILE, strerror(errno));
		return 1;
	}

	if( (nbytes = read(fd, buf, sizeof(buf))) == -1 ) {
		fprintf(stderr, "Failed to read %s configuration file: %s\n",
				WCFG_INITCFGFILE, strerror(errno));
		return 1;
	}

	if( !nbytes ) {
		fprintf(stderr, "Failed to read %s configuration file: %s\n",
				WCFG_INITCFGFILE, strerror(errno));
		return 1;
	}

	if( !(p = strtok(buf, delim)) ) {
		fprintf(stderr, "Invalid file format\n");
		return 1;
	}

	/*get wpa authenticator executable*/
	sscanf(p, "%s", init->wpaauth_file);

	if( !(p = strtok(NULL, delim)) ) {
		fprintf(stderr, "Invalid file format\n");
		close(fd);
		return 1;
	}

	/*get ap module file*/
	strncpy(init->apmodule_file, p, _POSIX_NAME_MAX);
	init->apmodule_file[_POSIX_NAME_MAX] = '\0';

	if( !(p = strtok(NULL, delim)) ) {
		fprintf(stderr, "Invalid file format\n");
		return 1;
	}

	/*get driver device name*/
	strncpy(init->dev_name, p, IFNAMSIZ-1);
	init->dev_name[IFNAMSIZ-1] = '\0';

	if( !(p = strtok(NULL, delim)) ) {
		fprintf(stderr, "Invalid file format\n");
		return 1;
	}

	/*get bridge device name*/
	if( !isspace(*p) && *p != '\0' ) {
		strncpy(init->br_name, p, IFNAMSIZ-1);
		init->br_name[IFNAMSIZ-1] = '\0';
	}

	printf("wcfglib: Found: %s, %s, %s, %s\n", 
			init->wpaauth_file, init->apmodule_file, init->dev_name,
			strlen(init->br_name)?init->br_name:"");

	return 0;
}



static int wcfg_write_initcfg(wcfginit *init)
{
	int fd;
	char buf[1300], tmp[100];

	if( !init ) return 1;

	if( (fd = open(WCFG_INITCFGFILE, O_CREAT|O_WRONLY|O_TRUNC)) == -1 ) {
		perror("open: ");
		return 1;
	}

	sprintf(buf, "<%s> ", init->wpaauth_file);
	sprintf(tmp, "<%s> ", init->apmodule_file);
	strcat(buf, tmp);
	sprintf(tmp, "<%s>", init->dev_name);
	strcat(buf, tmp);
	if( strlen(init->br_name) ) {
		sprintf(tmp, " <%s>", init->br_name);
		strcat(buf, tmp);
	}
	strcat(buf, "\n");

	if( write(fd, buf, strlen(buf)) == -1 ) {
		perror("write: ");
		close(fd);
		return 1;
	}

	close(fd);
	return 0;
}


static int wcfg_write_cfg(wcfgd *wd)
{
	int fd;
	char buf[_POSIX_NAME_MAX+IFNAMSIZ+50], tmp[100];

	if( !wd ) return 1;

	if( (fd = open(WCFG_CFGFILE, O_CREAT|O_WRONLY|O_TRUNC)) == -1 ) {
		perror("open: ");
		return 1;
	}

	sprintf(buf, "<%d> ", wd->wpaauth_pid);
	sprintf(tmp, "<%s> ", wd->apmodule_name);
	strcat(buf, tmp);
	sprintf(tmp, "<%s>", wd->dev_name);
	strcat(buf, tmp);
	if( strlen(wd->br_name) ) {
		sprintf(tmp, " <%s>", wd->br_name);
		strcat(buf, tmp);
	}
	strcat(buf, "\n");

	if( write(fd, buf, strlen(buf)) == -1 ) {
		perror("write: ");
		close(fd);
		return 1;
	}

	close(fd);
	return 0;
}

static void kill_wpaauth(pid_t pid)
{
	if( !pid ) return;

	printf("wcfglib: Terminating wpa authenticator (pid=%d).\n", pid);
	if( kill(pid, SIGTERM) == -1 )
	{
		sleep(10);
		fprintf(stderr, "wcfglib: Wpa authenticator ignored SIGTERM signal, %s\n", 
				strerror(errno));
		if( kill(pid, SIGINT) == -1 ) 
		{
			fprintf(stderr, "wcfglib: Wpa authenticator ignored SIGINT signal, %s\n",
					strerror(errno));
trythis:
			kill(pid, SIGKILL);
			fprintf(stderr, "wcfglib: Sent SIGKILL signal to the wpa authenticator.\n");
			sleep(2);
		}
		fflush(stderr);
	}
	else printf("wcfglib: Successfully terminated the wpa authenticator.\n");
		sleep(10);

	if( kill(pid, SIGTERM) == -1 ) {
		if( errno == ESRCH )
			printf("\nwcfglib: Successfully terminated wpa authenticator.\n\n");
		else goto trythis;
	}

	fflush(stdout);
}

static int config_wpaauth(tiap_wpa_auth_t wpa_conf)
{
	int sock;
	struct sockaddr_un wpa_sockaddr;


	/* Create socket on which to send. */
	sock = socket(AF_UNIX, SOCK_DGRAM, 0);
	if (sock < 0) {
		perror("opening datagram socket");
		return -1;
	}

	/* Construct name of socket to send to. */
	wpa_sockaddr.sun_family = AF_UNIX;
	strcpy(wpa_sockaddr.sun_path, TIAP_SOCKADDR_WPAAUTH);
	/* Send message. */
	if (sendto(sock, &wpa_conf, sizeof(wpa_conf), 0,
				(struct sockaddr *)&wpa_sockaddr, sizeof(struct sockaddr_un)) < 0) {
		perror("sending wpa configuration");
		return -1;
	}
	close(sock);

	return 0;

}

static void deinit_process(wcfgd *wd, int stage)
{
	/*delete the configuration file*/
	unlink(WCFG_CFGFILE);
	unlink(WCFG_INITCFGFILE);
	unlink(WCFG_WPAPIDFILE);

	if( wd->storage.wpa.type.sec.type != TIAP_PARAM_SECURITY_WEP ||
		wd->storage.ldb.priv_opt == 1 )
		kill_wpaauth(wd->wpaauth_pid);

	if( stage > 1 ) 
	{
		if( tiap_wcfg_dev_set_flags(wd->dev_name, ~IFF_UP, 0) )
			fprintf(stderr, "wcglib: Failed to bring down %s interface\n", wd->dev_name);
		printf("wcfglib: Unloading the ap module.\n");
		if( mod_apdriver("rmmod", wd->apmodule_name) )
			fprintf(stderr, "wcfglib: Failed to unload the ap module.\n");
		else printf("wcfglib: Successfully unloaded the ap module.\n");
	}
}

/*remove it later*/
#if 0
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

static int myifconfig(const char *dev, const __u32 ipaddr)
{
	pid_t pid;
	int status;

	if( (pid = fork()) == -1 ) {
		perror("In parent: ");
		return -1;
	}
	else if( !pid ) { /*child*/
		struct in_addr in;
		in.s_addr = ipaddr;
		execlp("ifconfig", "ifconfig", dev, inet_ntoa(in), NULL);
		exit(1);
	}

	if( waitpid(pid, &status, 0) == -1 ) {
		fprintf(stderr, "%d: %s\n", __LINE__, strerror(errno));
		return 1;
	}
	else if( WIFEXITED(status) )
		return WEXITSTATUS(status); /*in case of success this one must return 0*/
	else if( WIFSIGNALED(status) )
		return WTERMSIG(status);

	return 0;
}
#endif



wcfgd *tiap_wcfg_init(wcfginit *init)
{
	wcfgd *wd = NULL;
	int fd, ret;
	char *p = NULL;
	struct stat s;

	/*allocate wcfg descriptor*/
	if( !(wd = malloc(sizeof(wcfgd))) )
		return NULL;

	memcpy(wd->fw_filename, init->fw_filename, sizeof(wd->fw_filename));

	/*store storage file*/
	strncpy(wd->storage_file, init->storage_file, _POSIX_NAME_MAX+_POSIX_PATH_MAX);
	wd->storage_file[_POSIX_NAME_MAX+_POSIX_PATH_MAX] = '\0';

	/*read storage */
	if( wcfg_storage_read(&wd->storage, wd->storage_file) ) {
		free(wd);
		return NULL;
	}

	/*verify storage integrity*/
	if( wd->storage.magic != WCFG_MAGIC ) {
		printf("\nwcfglib: Setting default parameters.\n");
		wd->storage.defaults = 0; /*once is enough*/
		wcfg_storage_setdefaults(&wd->storage);
	}

	if( wd->storage.defaults == 1 ) {
		wd->storage.defaults = 0;
		printf("\nwcfglib: Setting default parameters.\n");
	}

	/* save initial wpa parameters */
	init_wpa = wd->storage.wpa;


	if( (fd = open(WCFG_CFGFILE, O_RDONLY)) == -1 ) /*initialization case*/
	{	
		/* start wpa authenticator only if wep or wpa is enabled 
		   since wep+wpa mode is not supported security type wep means that wpa is disabled */
		if( wd->storage.wpa.type.sec.type != TIAP_PARAM_SECURITY_WEP ||
			wd->storage.ldb.priv_opt == 1 )
		{
			/*start wpa authenticator*/
			//if( (wd->wpaauth_pid = start_wpaauth(init->wpaauth_file)) == -1 ) {
			if( (wd->wpaauth_pid = pseudo_start(init->wpaauth_file)) == -1 ) {
				free(wd);
				printf("%s: %d\n", __func__, __LINE__);
				return NULL;
			}

			if( config_wpaauth(wd->storage.wpa) == -1 ) {
				fprintf(stderr, "wcfglib: Failed to configure wpa authenticator.\n");
				kill_wpaauth(wd->wpaauth_pid);
				free(wd);
				return NULL;
			}
		}
		else wd->wpaauth_pid = 0;

		/* Demiurg : check whether module exist */
		if (stat(init->apmodule_file, &s) != 0 || s.st_size <= 0) {
			fprintf(stderr, "wcfglib: Kernel module %s does not exist or invalid !\n", 
					init->apmodule_file);
			kill_wpaauth(wd->wpaauth_pid);
			free(wd);
			return NULL;
		}

		/*insmod ap driver*/
		if( mod_apdriver("insmod", init->apmodule_file) ) 
		{
			if( !strlen(init->apmodule_file) )
				fprintf(stderr, "wcfglib: Empty module name.\n");
			else fprintf(stderr, "wcfglib: Failed to insmod %s, terminating %s.\n",
					init->apmodule_file, init->wpaauth_file);
			deinit_process(wd, 1);
			free(wd);
			return NULL;
		}

		/*store ap module name*/
		strncpy(wd->apmodule_name, basename(init->apmodule_file), _POSIX_NAME_MAX);
		wd->apmodule_name[_POSIX_NAME_MAX] = '\0';
		if( !(p = strrchr(wd->apmodule_name, '.')) ) {
			fprintf(stderr, "wcfglib: Incorrect ap module name %s.\n", wd->apmodule_name);
			deinit_process(wd, 1);
			free(wd);
			printf("%s: %d\n", __func__, __LINE__);
			return NULL;
		}
		*p = '\0';

		/* store ap device name */
		strncpy(wd->dev_name, init->dev_name, IFNAMSIZ-1);
		wd->dev_name[IFNAMSIZ-1] = '\0';

		/* store bridge device name */
		strncpy(wd->br_name, init->br_name, IFNAMSIZ-1);
		wd->br_name[IFNAMSIZ-1] = '\0';

		if( wcfg_write_initcfg(init) )
		{
			fprintf(stderr, "wcfglib: Failed to write %s configuration file.\n", WCFG_INITCFGFILE);
			deinit_process(wd, 2);
			free(wd);
			printf("%s: %d\n", __func__, __LINE__);
			return NULL;
		}

		if( wcfg_write_cfg(wd) )
		{
			fprintf(stderr, "wcfglib: Failed to write %s configuration file.\n", WCFG_CFGFILE);
			deinit_process(wd, 2);
			free(wd);
			printf("%s: %d\n", __func__, __LINE__);
			return NULL;
		}

	
		if( (ret = wcfg_ioctl_init(wd)) ) 
		{
			fprintf(stderr, "wcfglib: Failed to initialize the wcfg library, error=%d\n", ret);
			deinit_process(wd, 2);
			free(wd);
			printf("%s: %d\n", __func__, __LINE__);
			return NULL;
		}

		if( config_ip_settings(wd) )
		{
			fprintf(stderr, "wcfglib: Failed to config IP settings\n");
			deinit_process(wd, 2);
			free(wd);
			printf("%s: %d\n", __func__, __LINE__);
			return NULL;

		}
	}
	else {
		if( (ret = wcfg_read_cfg(fd, wd)) ) {
			printf("%s: %d\n", __func__, __LINE__);
			return NULL;
		}
		close(fd);
	}

	return wd;
}

/* Free all library resources. */
int tiap_wcfg_light_deinit(const wcfgd *wd)
{
	
	if( !wd ) return 1;

	/*free wcfg descriptor*/
	free((void *)wd);

	return 0;
}

int tiap_wcfg_deinit(const wcfgd *wd)
{
	int ret;
	__u32 gw;
	struct in_addr in;

	if( !wd ) return 1;

	/*update persistent storage*/
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	/*kill the wpa authenticator process*/
	if( wd->wpaauth_pid > 1 )
		kill_wpaauth(wd->wpaauth_pid);
	
	if( tiap_wcfg_dev_set_flags(wd->dev_name, ~IFF_UP, 0) )
		fprintf(stderr, "wcglib: Failed to bring down %s interface\n", wd->dev_name);

	/*unload the ap driver*/
	if( mod_apdriver("rmmod", wd->apmodule_name) ) {
		fprintf(stderr, "wcglib: Failed to unload the ap module from the kernel\n");
		ret = 1;
	}

	/*delete the configuration file*/
	if( unlink(WCFG_CFGFILE) == -1 )
		return 1;

	if( wd->storage.wpa.type.sec.type != TIAP_PARAM_SECURITY_WEP ||
	    wd->storage.ldb.priv_opt == 1 )
		if( unlink(WCFG_WPAPIDFILE) == -1 )
			return 1;

	if( tiap_wcfg_get_default_gateway(&gw) ) {
		fprintf(stderr, "wcglib: Failed to get default route.\n");
		return 1;
	}

	/*delete default route*/
	in.s_addr = gw;
	if( tiap_wcfg_set_route(wd->br_name, 0, 0, gw, SIOCDELRT) )
		return 0;
	printf("wcglib: Deleted default route %s\n", inet_ntoa(in));
	fflush(stdout);

	/*free wcfg descriptor*/
	free((void *)wd);

	return 0;
}


/* 
 * Deinitialize and then initialize library.
 * Additionally, read all required configuration parameters so that
 * the next time library will be correctly initialized.
 */
#include <sys/reboot.h>

void tiap_wcfg_system_reboot() {
	sync();
	sleep(1);
	reboot(RB_ENABLE_CAD);
	sync();
	sleep(1);
	reboot(RB_AUTOBOOT);
}

wcfgd *tiap_wcfg_restart(const wcfgd *wd) 
{
	wcfginit init;

	memset(&init, 0, sizeof(init));
	strncpy(init.storage_file, WCFG_MTDBLOCK, sizeof(init.storage_file));
	init.storage_file[sizeof(init.storage_file)-1] = '\0';

	if( wcfg_read_initcfg(&init) )
		return NULL;
	
	if( tiap_wcfg_deinit(wd) )
		return NULL;

	memcpy(init.fw_filename, wd->fw_filename, sizeof(wd->fw_filename));

	return tiap_wcfg_init(&init);
}



/* The following functions update both local and physical storage.
   On success zero is returned, otherwise nonzero value.

   Note: no syncronization is used for simplicity reasons only.
 */


int tiap_wcfg_set_hwaddr(wcfgd *wd, const __u8 *hwaddr)
{
	memcpy(wd->storage.ldb.hwaddr, hwaddr, ETH_ALEN);

	/* Felix R.
	   The following code is verified and works.
	   It's commented since changing hw addr is not allowed at run time,
	   because of bssid issue.
	 */

	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_get_hwaddr(const wcfgd *wd, __u8 *hwaddr)
{
	int s;
	struct ifreq ifr;

	if( (s =  socket(AF_INET, SOCK_DGRAM, 0)) == -1 ) {
		fprintf(stderr, "%d: %s\n", __LINE__, strerror(errno));
		return 1;
	}

	strncpy(ifr.ifr_name, wd->dev_name, IFNAMSIZ);
	ifr.ifr_hwaddr.sa_family = ARPHRD_ETHER;

	if( ioctl(s, SIOCGIFHWADDR, &ifr) < 0) {
		fprintf(stderr, "%d: %s\n", __LINE__, strerror(errno));
		return 1;
	}

	memcpy(hwaddr, ifr.ifr_hwaddr.sa_data, ETH_ALEN);
	close(s);

	return 0;
}



/* Standard */

int tiap_wcfg_siwessid(wcfgd *wd, const char *essid, int update_storage)
{
	strncpy(wd->storage.ldb.essid, essid, IW_ESSID_MAX_SIZE);
	wd->storage.ldb.essid[IW_ESSID_MAX_SIZE] = '\0';

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_siwrts(wcfgd *wd, const __s32 rts, int update_storage)
{
	wd->storage.ldb.rts = rts;

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_siwfrag(wcfgd *wd, const __s32 frag, int update_storage)
{
	wd->storage.ldb.frag = frag;

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_siwretry(wcfgd *wd, const __u32 value, const int type, int update_storage)
{
	switch( type )
	{
		case TIAP_PARAM_RETRY_LONG:
			wd->storage.ldb.retry_long = (__u8)value;
			break;
		case TIAP_PARAM_RETRY_SHORT:
			wd->storage.ldb.retry_short = (__u8)value;
			break;
		case TIAP_PARAM_LIFETIME_TX:
			wd->storage.ldb.lifetime_tx = value;
			break;
		case TIAP_PARAM_LIFETIME_RX:
			wd->storage.ldb.lifetime_rx = value;
			break;
		default: return 1;
	}

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_siwtxpow(wcfgd *wd, const tiap_param_txpower_t *txpow, int update_storage)
{
	if( txpow->hr == txpower_a )
		memcpy(&wd->storage.ldb.txpow_a, (void *)txpow, sizeof(*txpow));
	else if( txpow->hr == txpower_g )
		memcpy(&wd->storage.ldb.txpow_g, (void *)txpow, sizeof(*txpow));

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}



/* The following functions read from local storage*/

/*Warning: essid must be at least IW_ESSID_MAX_SIZE+1 size long*/
int tiap_wcfg_giwessid(const wcfgd *wd, char *essid, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
	{
		strncpy(essid, wd->storage.ldb.essid, IW_ESSID_MAX_SIZE);
		essid[IW_ESSID_MAX_SIZE] = '\0';
	}
	else
	{

		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = ioctl_get_essid (essid, ioctl_sock, (char *)wd->dev_name);

		close(ioctl_sock);
	}

	return ret;
}

int tiap_wcfg_giwrts(const wcfgd *wd, __s32 *rts, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
	{
		*rts = wd->storage.ldb.rts;
	}
	else
	{

		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = ioctl_get_rts (rts, ioctl_sock, (char *)wd->dev_name);

		close(ioctl_sock);
	}

	return ret;
}

int tiap_wcfg_giwfrag(const wcfgd *wd, __s32 *frag, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
	{
		*frag = wd->storage.ldb.frag;
	}
	else
	{
		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = ioctl_get_frag (frag, ioctl_sock, (char *)wd->dev_name);

		close(ioctl_sock);
	}

	return ret;
}

int tiap_wcfg_giwretry(const wcfgd *wd, __u32 *value, int type, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
	{
		switch( type )
		{
			case TIAP_PARAM_RETRY_LONG:
				*(__u8 *)value = wd->storage.ldb.retry_long;
				break;
			case TIAP_PARAM_RETRY_SHORT:
				*(__u8 *)value = wd->storage.ldb.retry_short;
				break;
			case TIAP_PARAM_LIFETIME_TX:
				*value = wd->storage.ldb.lifetime_tx;
				break;
			case TIAP_PARAM_LIFETIME_RX:
				*value = wd->storage.ldb.lifetime_rx;
				break;
			default: return 1;
		}
	}
	else
	{


		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = ioctl_get_retry((__s32 *)value, type, ioctl_sock, (char *)wd->dev_name);

		close(ioctl_sock);
	}

	return ret;

}

int tiap_wcfg_giwtxpow(const wcfgd *wd, tiap_param_txpower_t *txpow, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
	{
		if( txpow->hr == txpower_a )
			memcpy(txpow, &wd->storage.ldb.txpow_a, sizeof(*txpow));
		else if( txpow->hr == txpower_g )
			memcpy(txpow, &wd->storage.ldb.txpow_g, sizeof(*txpow));
		else return 1;
	}
	else
	{

		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = ioctl_get_txpow (txpow, ioctl_sock, (char *)wd->dev_name);

		close(ioctl_sock);
	}

	return ret;

}



/* Complex */

int tiap_wcfg_set_basic(wcfgd *wd, const __u32 we_id, void *arg, int update_storage)
{
	int ret = 0, ioctl_sock;

	/* Create a channel to the NET kernel. */
	if((ioctl_sock = iw_sockets_open()) < 0)
	{
		perror("socket");
		return(-1);
	}

	switch( we_id )
	{
		case TIAP_PARAM_REPLEV:
			{
				__s8 replev = *(__s8 *)arg;

				if (replev > 0)
					set_replev_ldb(wd, replev);
				else if (replev < 0)
					    unset_replev_ldb(wd, replev * (-1));
				else
					disable_replev_ldb(wd);
				
				ret = exec_priv_ioctl_set(TIAP_PARAM_REPLEV, arg, sizeof(__s8),ioctl_sock, wd->dev_name);
			}
			break;
		case TIAP_PARAM_PRIV_OPTION:
			wd->storage.ldb.priv_opt = *(__u8 *)arg;
			break;
		case TIAP_PARAM_BEACON_PERIOD:
			wd->storage.ldb.beacon_period = *(__u16 *)arg;
			break;
		case TIAP_PARAM_DTIM_PERIOD:
			wd->storage.ldb.dtim_period = *(__u8 *)arg;
			break;
		case TIAP_PARAM_BSS_BRIDGE_STATE:
			wd->storage.ldb.bss_bridge_state = *(tiap_param_bss_br_state_t *)arg;
			ret = exec_priv_ioctl_set(TIAP_PARAM_BSS_BRIDGE_STATE, arg,
					sizeof(tiap_param_bss_br_state_t),ioctl_sock, wd->dev_name);
			break;
		case TIAP_PARAM_SECURITY_TYPE:
			wd->storage.wpa.type.sec.type = *(tiap_param_security_type_t *)arg;
			break;
		case TIAP_PARAM_WONE_TIME_MODE:
			wd->storage.ldb.wone_timemod = *(__u8 *)arg;
			break;
		case TIAP_PARAM_RATE_FALLBACK:
			wd->storage.ldb.rate_fallback = *(__u8 *)arg;
			break;
		case TIAP_PARAM_WONE_NUM:
			wd->storage.ldb.wone_num = *(__u8 *)arg;
			break;
		case TIAP_PARAM_RADIO_CAL:
			wd->storage.ldb.radio_cal = *(__u32 *)arg;
			break;
		case TIAP_PARAM_RADIO_CAL_INT:
			wd->storage.ldb.cal_interval = *(__u16 *)arg;
			break;
		case TIAP_PARAM_ENERGY_DETECT:
			wd->storage.ldb.energy_detect = *(__u8 *)arg;
			break;
		case TIAP_PARAM_HIDDEN_SSID:
			wd->storage.ldb.hidden_ssid = *(__u16 *)arg;
			break;
		case TIAP_PARAM_MULTI_DOMAIN_CAPAB:
			wd->storage.ldb.multi_domain_cap = *(__u8 *)arg;
			break;
		case TIAP_PARAM_SPECTRUM_MGMT:
			wd->storage.ldb.spectrum_mgmt = *(__u8 *)arg;
			break;
		case TIAP_PARAM_LEGACY_STA:
			wd->storage.ldb.legacy_sta = *(__u8 *)arg;
			break;
		case TIAP_PARAM_LOCAL_POWER_CONS:
			wd->storage.ldb.local_pow_cons = *(__s8 *)arg;
			break;
		case TIAP_PARAM_MAX_TX_POWER_LEVEL:
			wd->storage.ldb.max_txpow_lev = *(__u8 *)arg;
			break;
		case TIAP_PARAM_COUNTRY_STR:
			strncpy(wd->storage.ldb.country_str, (char *)arg, TIAP_PARAM_COUNTRY_STRING_LEN);
			break;
		case TIAP_PARAM_OPER_STATE:
			wd->storage.ldb.oper_state = *(__u8 *)arg;
			ret = exec_priv_ioctl_set(TIAP_PARAM_OPER_STATE, arg, sizeof(__u8),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_RADAR_DETECT_MODE:
			{
				__u8 radar_detect = *(__u8 *)arg;
				ret = set_radar_detect(wd, radar_detect);
			}
			break;
		case TIAP_PARAM_BURST_LIMIT:
			wd->storage.ldb.txop_limit = *(__u16 *)arg;
			break;
		case TIAP_PARAM_AP_BRIDGE_MODE:
			wd->storage.ldb.bridge_mode = *(__u8 *)arg;
			break;
		default:
			ret = 1;
	}

	if ( !ret && update_storage){	
		if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
			return 1;
	}

	close(ioctl_sock);

	return ret;
}

static void set_replev_ldb(wcfgd *wd, __u8 replev)
{
	wd->storage.ldb.replev |= 1 << replev;
}

static void unset_replev_ldb(wcfgd *wd, __u8 replev)
{

	wd->storage.ldb.replev &= ~(1 << replev);
}

static void disable_replev_ldb(wcfgd *wd)
{

	wd->storage.ldb.replev = 0;
}

int tiap_wcfg_get_basic(wcfgd *wd, const __u32 we_id, void *arg, int from_storage)
{
	int ret = 0, ioctl_sock;

	/* Create a channel to the NET kernel. */
	if((ioctl_sock = iw_sockets_open()) < 0)
	{
		perror("socket");
		return(-1);
	}

	switch( we_id )
	{
		case TIAP_PARAM_REPLEV:
			if (from_storage)
				*(__s32 *)arg = wd->storage.ldb.replev;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_REPLEV, arg,sizeof(__s32),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_PRIV_OPTION:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.priv_opt;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_PRIV_OPTION, arg,sizeof(wd->storage.ldb.priv_opt),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_BEACON_PERIOD:
			if (from_storage)
				*(__u16 *)arg = wd->storage.ldb.beacon_period;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_BEACON_PERIOD, arg,sizeof(wd->storage.ldb.beacon_period),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_DTIM_PERIOD:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.dtim_period;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_DTIM_PERIOD, arg,sizeof(wd->storage.ldb.dtim_period),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_BSS_BRIDGE_STATE:
			if (from_storage)
				*(tiap_param_bss_br_state_t *)arg = wd->storage.ldb.bss_bridge_state;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_BSS_BRIDGE_STATE, arg, sizeof(wd->storage.ldb.bss_bridge_state),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_SECURITY_TYPE:
			if (from_storage)
				*(tiap_param_security_type_t *)arg = wd->storage.wpa.type.sec.type;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_SECURITY_TYPE, arg, sizeof(wd->storage.wpa.type.sec.type),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_WONE_TIME_MODE:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.wone_timemod;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_WONE_TIME_MODE, arg, sizeof(wd->storage.ldb.wone_timemod),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_RATE_FALLBACK:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.rate_fallback;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_RATE_FALLBACK, arg, sizeof(wd->storage.ldb.rate_fallback),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_WONE_NUM:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.wone_num;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_WONE_NUM, arg, sizeof(wd->storage.ldb.wone_num),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_RADIO_CAL:
			if (from_storage)
				*(__u32 *)arg = wd->storage.ldb.radio_cal;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_RADIO_CAL, arg, sizeof(wd->storage.ldb.radio_cal),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_RADIO_CAL_INT:
			if (from_storage)
				*(__u16 *)arg = wd->storage.ldb.cal_interval;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_RADIO_CAL_INT, arg, sizeof(wd->storage.ldb.cal_interval),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_ENERGY_DETECT:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.energy_detect;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_ENERGY_DETECT, arg, sizeof(wd->storage.ldb.energy_detect),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_HIDDEN_SSID:
			if (from_storage)
				*(__u16 *)arg = wd->storage.ldb.hidden_ssid;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_HIDDEN_SSID, arg, sizeof(wd->storage.ldb.hidden_ssid),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_MULTI_DOMAIN_CAPAB:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.multi_domain_cap;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_MULTI_DOMAIN_CAPAB, arg, sizeof(wd->storage.ldb.multi_domain_cap),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_SPECTRUM_MGMT:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.spectrum_mgmt;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_SPECTRUM_MGMT, arg, sizeof(wd->storage.ldb.spectrum_mgmt),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_LEGACY_STA:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.legacy_sta;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_LEGACY_STA, arg, sizeof(wd->storage.ldb.legacy_sta),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_LOCAL_POWER_CONS:
			if (from_storage)
				*(__s8 *)arg = wd->storage.ldb.local_pow_cons;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_LOCAL_POWER_CONS, arg, sizeof(wd->storage.ldb.local_pow_cons),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_MAX_TX_POWER_LEVEL:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.max_txpow_lev;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_MAX_TX_POWER_LEVEL, arg, sizeof(wd->storage.ldb.max_txpow_lev),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_COUNTRY_STR:
			if (from_storage)
				strncpy((char *)arg, wd->storage.ldb.country_str, TIAP_PARAM_COUNTRY_STRING_LEN);
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_COUNTRY_STR, arg, sizeof(wd->storage.ldb.country_str),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_OPER_STATE:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.oper_state;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_OPER_STATE, arg, sizeof(wd->storage.ldb.oper_state),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_RADAR_DETECT_MODE:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.radar_detect;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_RADAR_DETECT_MODE, arg, sizeof(wd->storage.ldb.radar_detect),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_BURST_LIMIT:
			if (from_storage)
				*(__u16 *)arg = wd->storage.ldb.txop_limit;
			else
				ret = ioctl_get_burst_limit(arg, sizeof(arg), ioctl_sock, wd->dev_name);
			break;
		case TIAP_PARAM_AP_BRIDGE_MODE:
			if (from_storage)
				*(__u16 *)arg = wd->storage.ldb.bridge_mode;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_AP_BRIDGE_MODE, arg,
						sizeof(wd->storage.ldb.bridge_mode),
						ioctl_sock, wd->dev_name);
			break;
		default:
			ret = 1;
	}

	close(ioctl_sock);

	return ret;
}

int tiap_wcfg_set_auth_type(const wcfgd *wd, const tiap_param_auth_algo_t *tbl)
{
	memcpy(&wd->storage.ldb.auth_tbl, tbl, sizeof(wd->storage.ldb.auth_tbl));

	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_get_auth_type(const wcfgd *wd, tiap_param_auth_algo_t *tbl, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
	{
		memcpy(tbl, wd->storage.ldb.auth_tbl, sizeof(wd->storage.ldb.auth_tbl));
	}
	else
	{
		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = exec_priv_ioctl_get(TIAP_PARAM_AUTH_TYPE, tbl, sizeof(wd->storage.ldb.auth_tbl),ioctl_sock,(char *)wd->dev_name);

		close(ioctl_sock);
	}

	return ret;
}

int tiap_wcfg_set_wd(const wcfgd *wd, const tiap_param_wd_t *w, int update_storage)
{
	switch( w->cmd )
	{
		case TIAP_PARAM_WD_ENABLE:
			memcpy((void *)&wd->storage.ldb.wd_en, (void *)w, sizeof(tiap_param_wd_t));
			break;
		case TIAP_PARAM_WD_RXFREE:
			memcpy((void *)&wd->storage.ldb.wd_rxfree, (void *)w, sizeof(tiap_param_wd_t));
			break;
		case TIAP_PARAM_WD_BEACONS:
			memcpy((void *)&wd->storage.ldb.wd_beacons, (void *)w, sizeof(tiap_param_wd_t));
			break;
		case TIAP_PARAM_WD_TXSTUCK:
			memcpy((void *)&wd->storage.ldb.wd_txstuck, (void *)w, sizeof(tiap_param_wd_t));
			break;
		case TIAP_PARAM_WD_FCSERR:
			memcpy((void *)&wd->storage.ldb.wd_fcserrs, (void *)w, sizeof(tiap_param_wd_t));
			break;
		case TIAP_PARAM_WD_ACI:
			memcpy((void *)&wd->storage.ldb.wd_aci, (void *)w, sizeof(tiap_param_wd_t));
			break;
		case TIAP_PARAM_WD_NB:
			memcpy((void *)&wd->storage.ldb.wd_nb, (void *)w, sizeof(tiap_param_wd_t));
			break;
		default:
			return 1;
	}

	if (update_storage){
		if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
			return 1;
	}

	return 0;

}


int tiap_wcfg_get_wd(const wcfgd *wd, const tiap_param_wd_t *w, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
	{
		switch( w->cmd )
		{
			case TIAP_PARAM_WD_ENABLE:
				memcpy((void *)w, (void *)&wd->storage.ldb.wd_en, sizeof(tiap_param_wd_t));
				break;
			case TIAP_PARAM_WD_RXFREE:
				memcpy((void *)w, (void *)&wd->storage.ldb.wd_rxfree, sizeof(tiap_param_wd_t));
				break;
			case TIAP_PARAM_WD_BEACONS:
				memcpy((void *)w, (void *)&wd->storage.ldb.wd_beacons, sizeof(tiap_param_wd_t));
				break;
			case TIAP_PARAM_WD_TXSTUCK:
				memcpy((void *)w, (void *)&wd->storage.ldb.wd_txstuck, sizeof(tiap_param_wd_t));
				break;
			case TIAP_PARAM_WD_FCSERR:
				memcpy((void *)w, (void *)&wd->storage.ldb.wd_fcserrs, sizeof(tiap_param_wd_t));
				break;
			case TIAP_PARAM_WD_ACI:
				memcpy((void *)w, (void *)&wd->storage.ldb.wd_aci, sizeof(tiap_param_wd_t));
				break;
			case TIAP_PARAM_WD_NB:
				memcpy((void *)w, (void *)&wd->storage.ldb.wd_nb, sizeof(tiap_param_wd_t));
				break;	
		}
	}
	else
	{

		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}
		ret = exec_priv_ioctl_get(TIAP_PARAM_WD, (void *)w, sizeof(tiap_param_wd_t),ioctl_sock, (char *)wd->dev_name);

		close(ioctl_sock);
	}

	return ret;
}

int tiap_wcfg_get_interf_wd_config(const wcfgd *wd, tiap_param_interference_wd_t *param, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage) {
	
		switch(param->cmd)
		{
		case TIAP_PARAM_ACI_TIMEOUT:
			param->value.val32 = wd->storage.ldb.aci_state_timeout ;
			break;
		case TIAP_PARAM_NB_TIMEOUT:
			 param->value.val32 = wd->storage.ldb.nb_state_timeout;
			break;
		case TIAP_PARAM_ACI_RATE_THR:
			param->value.val32 = wd->storage.ldb.aci_rate_thresh;
			break;
		case TIAP_PARAM_ACI_EVENT_THR:
			param->value.val32 = wd->storage.ldb.aci_event_thresh;
			break;
		case TIAP_PARAM_PD_LOW:
			param->value.val16 = wd->storage.ldb.pd_low_thresh;
			break;
		case TIAP_PARAM_PD_HIGH:
			param->value.val16 = wd->storage.ldb.pd_high_thresh;
			break;
		case TIAP_PARAM_MEDIUM_BUSY_THR:
			param->value.val32 = wd->storage.ldb.medium_busy_thresh;
			break;
		}
	}
	else
	{
		 if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}
		ret = exec_priv_ioctl_get(TIAP_PARAM_INTERF_WD_CONFIG, (void *)param, sizeof(tiap_param_interference_wd_t),ioctl_sock, (char *)wd->dev_name);

		close(ioctl_sock);
	
	}
	return 0;
		
}

int tiap_wcfg_set_interf_wd_config(wcfgd *wd, tiap_param_interference_wd_t *param, int update_storage)
{
	switch(param->cmd)
	{
	case TIAP_PARAM_ACI_TIMEOUT:
		wd->storage.ldb.aci_state_timeout = param->value.val32;
		break;
	case TIAP_PARAM_NB_TIMEOUT:
		wd->storage.ldb.nb_state_timeout = param->value.val32;
		break;
	case TIAP_PARAM_ACI_RATE_THR:
		wd->storage.ldb.aci_rate_thresh = param->value.val32;
		break;
	case TIAP_PARAM_ACI_EVENT_THR:
		wd->storage.ldb.aci_event_thresh = param->value.val32;
		break;
	case TIAP_PARAM_PD_LOW:
		wd->storage.ldb.pd_low_thresh = param->value.val16;
		break;
	case TIAP_PARAM_PD_HIGH:
		wd->storage.ldb.pd_high_thresh = param->value.val16;
		break;
	case TIAP_PARAM_MEDIUM_BUSY_THR:
		wd->storage.ldb.medium_busy_thresh = param->value.val32;
		break;
	}

	if(update_storage){
		if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
			return 1;
	}

	return 0;
		
}

void shrink_table(void *tbl, int type, int idx, int tbl_len, int entry_size)
{
	__s8 (*a)[ETH_ALEN] = tbl;
	__s8 (*m)[IW_ESSID_MAX_SIZE+1] = tbl;

	while( idx < (tbl_len - 1) )
	{
		if( !type ) { /*access list*/
			a = tbl;
			memcpy(a[idx], a[idx+1], entry_size);
		}
		else if( type == 1 ) { /*multiple ssid*/
			m = tbl;
			memcpy(m[idx], m[idx+1], entry_size);
		}
		else printf("%s: invalid type = %d\n", __func__, type);
		idx++;
	}

	memset((!type ? a[tbl_len-1] : m[tbl_len-1]), 0, entry_size);
}



int tiap_wcfg_set_access_list(wcfgd *wd, const tiap_param_access_list_t *tbl)
{
	int ioctl_sock, ret = 0;

	if((ioctl_sock = iw_sockets_open()) < 0)
	{
		perror("socket");
		return(-1);
	}

	switch( tbl->cmd ) /* No matter what is the cmd value only two values are written
			   to NVRAM: TIAP_PARAM_ACCESS_LST (for all cmds except TIAP_PARAM_ACCESS_RESET)
			   and TIAP_PARAM_ACCESS_RESET*/ 
	{
		case TIAP_PARAM_ACCESS_SET_TYPE:
			wd->storage.ldb.access_list.enabled = tbl->enabled;
			wd->storage.ldb.access_list.type = tbl->type;
			wd->storage.ldb.access_list.cmd = TIAP_PARAM_ACCESS_LST;
			break;
		case TIAP_PARAM_ACCESS_LST:
			wd->storage.ldb.access_list.enabled = tbl->enabled;
			wd->storage.ldb.access_list.type = tbl->type;
			wd->storage.ldb.access_list.cmd = tbl->cmd;
			memcpy(wd->storage.ldb.access_list.list, tbl->list,
					sizeof(wd->storage.ldb.access_list.list));
			break;
		case TIAP_PARAM_ACCESS_ADD:
			wd->storage.ldb.access_list.enabled = tbl->enabled;
			ret = exec_priv_ioctl_set(TIAP_PARAM_ACCESS_LIST, 
					(void *)tbl,sizeof(wd->storage.ldb.access_list),
					ioctl_sock, (char *)wd->dev_name);
			if (!ret) /* ioctl succeeded which means that MAC was also checked for validity*/
			{
				/* then add the MAC to the access list in the storage */ 
				int i;
				char hwaddr[ETH_ALEN];

				memset(hwaddr, 0, ETH_ALEN);
				for(i=0; i<TIAP_PARAM_ACCESS_LIST_SIZE; i++)
				{
					/* if such MAC already exists we abort the setting*/
					if( !memcmp(wd->storage.ldb.access_list.list[i], tbl->list[0], ETH_ALEN) )
						return 0;
					
					if( !memcmp(wd->storage.ldb.access_list.list[i], hwaddr, ETH_ALEN) )
						break;
				}
				if( i == TIAP_PARAM_ACCESS_LIST_SIZE )
					return 1;
				memcpy(wd->storage.ldb.access_list.list[i], tbl->list[0], ETH_ALEN);
				wd->storage.ldb.access_list.cmd = TIAP_PARAM_ACCESS_LST;
			}

			
			break;

		case TIAP_PARAM_ACCESS_DEL_BY_INDEX:
			wd->storage.ldb.access_list.enabled = tbl->enabled;
			wd->storage.ldb.access_list.cmd = TIAP_PARAM_ACCESS_LST;

			shrink_table(wd->storage.ldb.access_list.list, 0,
					tbl->index-1, TIAP_PARAM_ACCESS_LIST_SIZE, ETH_ALEN);
			
			ret = exec_priv_ioctl_set(TIAP_PARAM_ACCESS_LIST, 
					(void *)tbl,sizeof(wd->storage.ldb.access_list),
					ioctl_sock, (char *)wd->dev_name);
			break;

		case TIAP_PARAM_ACCESS_DEL_BY_ADDR:
			wd->storage.ldb.access_list.enabled = tbl->enabled;
			wd->storage.ldb.access_list.cmd = TIAP_PARAM_ACCESS_LST;
			{
				int i;
				for(i=0; i<TIAP_PARAM_ACCESS_LIST_SIZE; i++)
					if( !memcmp(wd->storage.ldb.access_list.list[i], tbl->list[0], ETH_ALEN) )
						break;
				shrink_table(wd->storage.ldb.access_list.list, 0,
					i, TIAP_PARAM_ACCESS_LIST_SIZE, ETH_ALEN);
			}
			ret = exec_priv_ioctl_set(TIAP_PARAM_ACCESS_LIST, (void *)tbl,sizeof(wd->storage.ldb.access_list),ioctl_sock, (char *)wd->dev_name);
			wd->storage.ldb.access_list.cmd = TIAP_PARAM_ACCESS_LST;
			break;

		case TIAP_PARAM_ACCESS_RESET:
			if( tbl->enabled )
				return 1;
			memset(wd->storage.ldb.access_list.list, 0, sizeof(wd->storage.ldb.access_list.list));
			wd->storage.ldb.access_list.cmd = TIAP_PARAM_ACCESS_RESET;
			break;
	}
	
	if ( !ret ){	
		if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
			return 1;
	}

	close(ioctl_sock);

	return ret;
}


int tiap_wcfg_get_access_list(wcfgd *wd, tiap_param_access_list_t *tbl, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
	{
		memcpy(tbl, &wd->storage.ldb.access_list, sizeof(wd->storage.ldb.access_list));
	}
	else
	{	
	
		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = exec_priv_ioctl_get(TIAP_PARAM_ACCESS_LIST, tbl, sizeof(wd->storage.ldb.access_list),ioctl_sock,(char *)wd->dev_name);

		close(ioctl_sock);
	}

	return ret;
}

int tiap_wcfg_set_4x(wcfgd *wd, const tiap_param_4x_t *_4x, int idx, const int update_storage)
{
	switch( _4x->cmd )
	{
		case TIAP_PARAM_4X_FEATSET:
			memcpy((void *)&wd->storage.ldb._4x_fs, (void *)_4x, sizeof(wd->storage.ldb._4x_fs));
			/* when setting/resetting FeatureSet, the ERPProtectionTI must be set/reset too */
			((wcfgd *)wd)->storage.ldb.erp_ti[idx].elem.erp_ti = _4x->value;
			break;
		case TIAP_PARAM_4X_4X:
			if ( !memcmp( &wd->storage.ldb._4x_4x, _4x, sizeof(wd->storage.ldb._4x_4x) ))
			{
				/* if the given 4x value remained unchanged - do nothing */
				break;
			}
			else
			{
			memcpy((void *)&wd->storage.ldb._4x_4x, (void *)_4x, sizeof(wd->storage.ldb._4x_4x));
			if (_4x->value == 1)
			{
				((wcfgd *)wd)->storage.ldb.frag = TIAP_PARAM_MAX_FRAGMENTATION_TRESHOLD;
				((wcfgd *)wd)->storage.ldb.rts = TIAP_PARAM_RTS_TRESHOLD_MAX;
			}
			else if (_4x->value == 0)
			{

				((wcfgd *)wd)->storage.ldb.frag = TIAP_PARAM_FRAG_TRESH_DEFAULT;
				((wcfgd *)wd)->storage.ldb.rts = TIAP_PARAM_RTS_TRESHOLD_DEFAULT;
			}
			}

			break;
		case TIAP_PARAM_4X_CAT:
			memcpy((void *)&wd->storage.ldb._4x_cat, (void *)_4x, sizeof(wd->storage.ldb._4x_cat));
			break;
		case TIAP_PARAM_4X_PKTBURST:
			memcpy((void *)&wd->storage.ldb._4x_pb, (void *)_4x, sizeof(wd->storage.ldb._4x_pb));
			break;
		case TIAP_PARAM_4X_MIXMOD:
			memcpy((void *)&wd->storage.ldb._4x_mm, (void *)_4x, sizeof(wd->storage.ldb._4x_mm));
			break;
		default:
			return 1;
	}

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_get_4x(const wcfgd *wd, tiap_param_4x_t *_4x, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
	{
		switch( _4x->cmd )
		{
			case TIAP_PARAM_4X_FEATSET:
				memcpy((void *)_4x, (void *)&wd->storage.ldb._4x_fs, sizeof(wd->storage.ldb._4x_fs));
				break;
			case TIAP_PARAM_4X_4X:
				memcpy((void *)_4x, (void *)&wd->storage.ldb._4x_4x, sizeof(wd->storage.ldb._4x_4x));
				break;
			case TIAP_PARAM_4X_CAT:
				memcpy((void *)_4x, (void *)&wd->storage.ldb._4x_cat, sizeof(wd->storage.ldb._4x_cat));
				break;
			case TIAP_PARAM_4X_PKTBURST:
				memcpy((void *)_4x, (void *)&wd->storage.ldb._4x_pb, sizeof(wd->storage.ldb._4x_pb));
				break;
			case TIAP_PARAM_4X_MIXMOD:
				memcpy((void *)_4x, (void *)&wd->storage.ldb._4x_mm, sizeof(wd->storage.ldb._4x_mm));
				break;
		}
	}

	else
	{
		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = exec_priv_ioctl_get(TIAP_PARAM_4X, _4x, sizeof(tiap_param_4x_t),ioctl_sock,(char *)wd->dev_name);

		close(ioctl_sock);
	}

	return ret;
}

int tiap_wcfg_set_multi_ssid(wcfgd *wd, const tiap_param_multi_ssid_t *multi_ssid, int update_storage)
{

	switch( multi_ssid->cmd)
	{
		case TIAP_PARAM_MULTI_SSID_SET:
			memcpy((void *)&wd->storage.ldb.multi_ssid, (void *)multi_ssid,
				sizeof(wd->storage.ldb.multi_ssid));
			break;
		case TIAP_PARAM_MULTI_SSID_ADD:
			{
				int i;
				for (i=0; i<TIAP_PARAM_NUM_MULTIPLE_SSID; i++)
					if (wd->storage.ldb.multi_ssid.tbl.ssid[i][0] == '\0')
						break;
				if (i == TIAP_PARAM_NUM_MULTIPLE_SSID)
					return TIAP_PARAM_ERANGE;

				memcpy((char *)wd->storage.ldb.multi_ssid.tbl.ssid[i],
					multi_ssid->tbl.ssid[0],IW_ESSID_MAX_SIZE+1);
			}
			wd->storage.ldb.multi_ssid.cmd = TIAP_PARAM_MULTI_SSID_SET;
			break;

		case TIAP_PARAM_MULTI_SSID_DEL:
	
			shrink_table(wd->storage.ldb.multi_ssid.tbl.ssid, 1, multi_ssid->tbl.ssid_id,
					TIAP_PARAM_NUM_MULTIPLE_SSID, IW_ESSID_MAX_SIZE+1);
			wd->storage.ldb.multi_ssid.cmd = TIAP_PARAM_MULTI_SSID_SET;
			break;
		case TIAP_PARAM_MULTI_SSID_RESET:
			memset(&wd->storage.ldb.multi_ssid,0,sizeof(wd->storage.ldb.multi_ssid));
			break;
		default: return TIAP_PARAM_ERANGE;

	}


	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_get_multi_ssid(const wcfgd *wd, tiap_param_multi_ssid_t *multi_ssid, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
		memcpy((void *)multi_ssid, (void *)&wd->storage.ldb.multi_ssid, sizeof(wd->storage.ldb.multi_ssid));
	else
	{
		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}
		ret = exec_priv_ioctl_get(TIAP_PARAM_MULTI_SSID, (tiap_param_multi_ssid_t *)multi_ssid, sizeof(wd->storage.ldb.multi_ssid),ioctl_sock,(char *)wd->dev_name);

		close(ioctl_sock);
	}

	return ret;
}

int tiap_wcfg_set_cfg_pow_tbl(wcfgd *wd, const tiap_param_radio_t *radio, int update_storage)
{
	int ret = 0, ioctl_sock;

	if((ioctl_sock = iw_sockets_open()) < 0) {
		perror("socket");
		return ioctl_sock;
	}
	
	memcpy((void *)&wd->storage.ldb.radio, (const void *)radio, sizeof(wd->storage.ldb.radio));

	if (update_storage) {
		if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
			return 1;
	}
	
	ret = exec_priv_ioctl_set(TIAP_PARAM_CFG_POWER_TABLE, (tiap_param_radio_t *)radio,
			sizeof(*radio), ioctl_sock, (char *)wd->dev_name);

	return ret;
}

int tiap_wcfg_set_ch_range(const wcfgd *wd, const tiap_param_channel_rang *ch_rang)
{
	if( ch_rang->type == TIAP_PARAM_RANG_A )
		memcpy((void *)&wd->storage.ldb.ch_rang_a, ch_rang, sizeof(wd->storage.ldb.ch_rang_a));
	else if( ch_rang->type == TIAP_PARAM_RANG_G )
		memcpy((void *)&wd->storage.ldb.ch_rang_g, ch_rang, sizeof(wd->storage.ldb.ch_rang_g));
	else return 1;

	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_get_ch_range(const wcfgd *wd, tiap_param_channel_rang *ch_rang, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
	{
		if( ch_rang->type == TIAP_PARAM_RANG_A )
			memcpy(ch_rang, &wd->storage.ldb.ch_rang_a, sizeof(wd->storage.ldb.ch_rang_a));
		else if( ch_rang->type == TIAP_PARAM_RANG_G )
			memcpy(ch_rang, &wd->storage.ldb.ch_rang_g, sizeof(wd->storage.ldb.ch_rang_g));
	}
	else
	{

		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = exec_priv_ioctl_get(TIAP_PARAM_CHANNEL_RANGE, ch_rang, sizeof(tiap_param_channel_rang),ioctl_sock,(char *)wd->dev_name);

		close(ioctl_sock);
	}

	return ret;
}

int tiap_wcfg_set_vhcf(const wcfgd *wd, const tiap_param_vhcf_t *vhcf, const int update_storage)
{
	memcpy((void *)&wd->storage.ldb.vhcf, vhcf, sizeof(wd->storage.ldb.vhcf));

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_get_vhcf(const wcfgd *wd, tiap_param_vhcf_t *vhcf, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
	{
		memcpy((void *)vhcf, (void *)&wd->storage.ldb.vhcf, sizeof(wd->storage.ldb.vhcf));
	}
	else
	{
		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}
		ret = exec_priv_ioctl_get(TIAP_PARAM_VHCF, vhcf, sizeof(tiap_param_vhcf_t),ioctl_sock,(char *)wd->dev_name);
		close(ioctl_sock);
	}

	return ret;
}


int tiap_wcfg_set_wone(wcfgd *wd, const tiap_param_wone_t *wone, const int update_storage)
{
	unsigned int i = wone->index;

	if( i > TIAP_PARAM_WONE_DUAL_BAND - 1 ) return 1;

	switch( wone->type )
	{
		case wone_oper_rate:
			wd->storage.ldb.oper_rate[i].elem.b_legacy = wone->elem.b_legacy;
			break;
		case wone_oper_add:
			wd->storage.ldb.oper_add[i].elem.ofdm_legacy = wone->elem.ofdm_legacy;
			break;
		case wone_oper_ofdm:
			wd->storage.ldb.oper_ofdm[i].elem.ofdm_extend = wone->elem.ofdm_extend;
			break;
		case wone_ch:
			wd->storage.ldb.ch[i].elem.ch = wone->elem.ch;
			break;
		case wone_band:
			wd->storage.ldb.band[i].elem.band = wone->elem.band;
			break;
		case wone_pbcc:
			wd->storage.ldb.pbcc[i].elem.pbcc_mode = wone->elem.pbcc_mode;
			break;
		case wone_rate_adapt:
			wd->storage.ldb.rate_adapt[i].elem.rate_adapt = wone->elem.rate_adapt;
			break;
		case wone_pream_mode:
			wd->storage.ldb.pream_mode[i].elem.pream_mode = wone->elem.pream_mode;
			break;
		case wone_slot_mode:
			wd->storage.ldb.slot_mode[i].elem.slot_mode = wone->elem.slot_mode;
			break;
		case wone_erp_mode:
			wd->storage.ldb.erp_mode[i].elem.erp_mode = wone->elem.erp_mode;
			break;
		case wone_erp_type:
			wd->storage.ldb.erp_type[i].elem.erp_type = wone->elem.erp_type;
			break;
		case wone_erp_ti:
			wd->storage.ldb.dot11_mode[i].elem.erp_ti = wone->elem.erp_ti;
			break;
		case wone_dot11_mode:
			wd->storage.ldb.dot11_mode[i].elem.dot11_mode = wone->elem.dot11_mode;
			break;
		case wone_cwmin_mode:
			wd->storage.ldb.cwmin_mode[i].elem.cwmin_mode = wone->elem.cwmin_mode;
			break;
		case wone_usage_time:
			wd->storage.ldb.usage_time[i].elem.usage_time = wone->elem.usage_time;
			break;

		default:
			return 1;
	}

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_get_wone(const wcfgd *wd, tiap_param_wone_t *wone, int from_storage)
{
	int ret = 0, ioctl_sock;
	unsigned int i = wone->index;

	if( i > TIAP_PARAM_WONE_DUAL_BAND - 1 ) return 1;

	if (from_storage)
	{
		switch( wone->type )
		{
			case wone_oper_rate:
				wone->elem.b_legacy = wd->storage.ldb.oper_rate[i].elem.b_legacy;
				break;
			case wone_oper_add:
				wone->elem.ofdm_legacy = wd->storage.ldb.oper_add[i].elem.ofdm_legacy;
				break;
			case wone_oper_ofdm:
				wone->elem.ofdm_extend = wd->storage.ldb.oper_ofdm[i].elem.ofdm_extend;
				break;
			case wone_ch:
				wone->elem.ch = wd->storage.ldb.ch[i].elem.ch;
				break;
			case wone_band:
				wone->elem.band = wd->storage.ldb.band[i].elem.band;
				break;
			case wone_pbcc:
				wone->elem.pbcc_mode = wd->storage.ldb.pbcc[i].elem.pbcc_mode;
				break;
			case wone_rate_adapt:
				wone->elem.rate_adapt = wd->storage.ldb.rate_adapt[i].elem.rate_adapt;
				break;
			case wone_pream_mode:
				wone->elem.pream_mode = wd->storage.ldb.pream_mode[i].elem.pream_mode;
				break;
			case wone_slot_mode:
				wone->elem.slot_mode = wd->storage.ldb.slot_mode[i].elem.slot_mode;
				break;
			case wone_erp_mode:
				wone->elem.erp_mode = wd->storage.ldb.erp_mode[i].elem.erp_mode;
				break;
			case wone_erp_type:
				wone->elem.erp_type = wd->storage.ldb.erp_type[i].elem.erp_type;
				break;
			case wone_erp_ti:
				wone->elem.erp_ti = wd->storage.ldb.erp_ti[i].elem.erp_ti;
				break;
			case wone_dot11_mode:
				wone->elem.dot11_mode = wd->storage.ldb.dot11_mode[i].elem.dot11_mode;
				break;
			case wone_cwmin_mode:
				wone->elem.cwmin_mode = wd->storage.ldb.cwmin_mode[i].elem.cwmin_mode;
				break;
			case wone_usage_time:
				wone->elem.usage_time = wd->storage.ldb.usage_time[i].elem.usage_time;
				break;
		}
	}
	else
	{
		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = exec_priv_ioctl_get(TIAP_PARAM_WONE_ELEM, (void *)wone, sizeof(tiap_param_wone_t),
					ioctl_sock,(char *)wd->dev_name);

		close(ioctl_sock);
	}
	
	return ret;


}


void set_dot11mode(struct wcfg_storage_t *ldb, int dot11mode, int idx)
{
	switch( dot11mode )
	{
		case TIAP_PARAM_D11MODE_MIXED:

			ldb->oper_rate[idx].index = idx;
			ldb->oper_rate[idx].type = wone_oper_rate;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] =
				{ TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_BASIC_SUPPORTED,
					TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_BASIC_SUPPORTED,
					TIAP_PARAM_RATE_SUPPORTED };
					memcpy((void *)&ldb->oper_rate[idx].elem.b_legacy.rates, (void *)&rates,
							sizeof(ldb->oper_rate[idx].elem.b_legacy.rates));
			}
			ldb->oper_rate[idx].elem.b_legacy.len = 5;


			ldb->oper_add[idx].index = idx;
			ldb->oper_add[idx].type = wone_oper_add;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] =
				{ TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED,
					TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED };
					memcpy((void *)&ldb->oper_add[idx].elem.ofdm_legacy.rates, (void *)&rates,
							sizeof(ldb->oper_add[idx].elem.b_legacy.rates));
			}
			ldb->oper_add[idx].elem.ofdm_legacy.len = 0;

			ldb->oper_ofdm[idx].index = idx;
			ldb->oper_ofdm[idx].type = wone_oper_ofdm;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] = 
				{ TIAP_PARAM_RATE_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED,
					TIAP_PARAM_RATE_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED,
					TIAP_PARAM_RATE_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED,
					TIAP_PARAM_RATE_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED };
					memcpy((void *)&ldb->oper_ofdm[idx].elem.ofdm_extend.rates, (void *)&rates,
							sizeof(ldb->oper_ofdm[idx].elem.ofdm_extend.rates));
			}
			ldb->oper_ofdm[idx].elem.ofdm_extend.len = 8;

			ldb->pbcc[idx].index = idx;
			ldb->pbcc[idx].type = wone_pbcc;
			ldb->pbcc[idx].elem.pbcc_mode = 4;

			ldb->pream_mode[idx].index = idx;
			ldb->pream_mode[idx].type = wone_pream_mode;
			ldb->pream_mode[idx].elem.pream_mode = 2;

			ldb->slot_mode[idx].index = idx;
			ldb->slot_mode[idx].type = wone_slot_mode;
			ldb->slot_mode[idx].elem.slot_mode = 2;

			ldb->erp_mode[idx].index = idx;
			ldb->erp_mode[idx].type = wone_erp_mode;
			ldb->erp_mode[idx].elem.erp_mode = 3;

			ldb->erp_type[idx].index = idx;
			ldb->erp_type[idx].type = wone_erp_type;
			ldb->erp_type[idx].elem.erp_type = 1;

			break;

		case TIAP_PARAM_D11MODE_B:

			ldb->oper_rate[idx].index = idx;
			ldb->oper_rate[idx].type = wone_oper_rate;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] =
				{ TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_BASIC_SUPPORTED,
					TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_BASIC_SUPPORTED };
					memcpy((void *)&ldb->oper_rate[idx].elem.b_legacy.rates, (void *)&rates,
							sizeof(ldb->oper_rate[idx].elem.b_legacy.rates));
			}
			ldb->oper_rate[idx].elem.b_legacy.len = 4;

			ldb->oper_add[idx].index = idx;
			ldb->oper_add[idx].type = wone_oper_add;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] =
				{ TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED,
					TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED };
					memcpy((void *)&ldb->oper_add[idx].elem.ofdm_legacy.rates, (void *)&rates,
							sizeof(ldb->oper_add[idx].elem.b_legacy.rates));
			}
			ldb->oper_add[idx].elem.ofdm_legacy.len = 0;

			ldb->oper_ofdm[idx].index = idx;
			ldb->oper_ofdm[idx].type = wone_oper_ofdm;
			memset((void *)&ldb->oper_ofdm[idx].elem.ofdm_extend.rates, 0,
					sizeof(ldb->oper_ofdm[idx].elem.ofdm_extend.rates));
			ldb->oper_ofdm[idx].elem.ofdm_extend.len = 0;

			ldb->pbcc[idx].index = idx;
			ldb->pbcc[idx].type = wone_pbcc;
			ldb->pbcc[idx].elem.pbcc_mode = 0;

			ldb->pream_mode[idx].index = idx;
			ldb->pream_mode[idx].type = wone_pream_mode;
			ldb->pream_mode[idx].elem.pream_mode = 0;

			ldb->slot_mode[idx].index = idx;
			ldb->slot_mode[idx].type = wone_slot_mode;
			ldb->slot_mode[idx].elem.slot_mode = 1;

			ldb->erp_mode[idx].index = idx;
			ldb->erp_mode[idx].type = wone_erp_mode;
			ldb->erp_mode[idx].elem.erp_mode = 0;

			ldb->erp_type[idx].index = idx;
			ldb->erp_type[idx].type = wone_erp_type;
			ldb->erp_type[idx].elem.erp_type = 0;

			break;

		case TIAP_PARAM_D11MODE_B_PLUS:

			ldb->oper_rate[idx].index = idx;
			ldb->oper_rate[idx].type = wone_oper_rate;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] =
				{ TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_BASIC_SUPPORTED,
					TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_BASIC_SUPPORTED,
					TIAP_PARAM_RATE_SUPPORTED };
					memcpy((void *)&ldb->oper_rate[idx].elem.b_legacy.rates, (void *)&rates,
							sizeof(ldb->oper_rate[idx].elem.b_legacy.rates));
			}
			ldb->oper_rate[idx].elem.b_legacy.len = 5;

			ldb->oper_add[idx].index = idx;
			ldb->oper_add[idx].type = wone_oper_add;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] =
				{ TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED,
					TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED };
					memcpy((void *)&ldb->oper_add[idx].elem.ofdm_legacy.rates, (void *)&rates,
							sizeof(ldb->oper_add[idx].elem.b_legacy.rates));
			}
			ldb->oper_add[idx].elem.ofdm_legacy.len = 0;

			ldb->oper_ofdm[idx].index = idx;
			ldb->oper_ofdm[idx].type = wone_oper_ofdm;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] = 
				{ TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED,
					TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED };
					memcpy((void *)&ldb->oper_ofdm[idx].elem.ofdm_extend.rates, (void *)&rates,
							sizeof(ldb->oper_ofdm[idx].elem.ofdm_extend.rates));
			}
			ldb->oper_ofdm[idx].elem.ofdm_extend.len = 0;

			ldb->pbcc[idx].index = idx;
			ldb->pbcc[idx].type = wone_pbcc;
			ldb->pbcc[idx].elem.pbcc_mode = 4;

			ldb->pream_mode[idx].index = idx;
			ldb->pream_mode[idx].type = wone_pream_mode;
			ldb->pream_mode[idx].elem.pream_mode = 0;

			ldb->slot_mode[idx].index = idx;
			ldb->slot_mode[idx].type = wone_slot_mode;
			ldb->slot_mode[idx].elem.slot_mode = 0;

			ldb->erp_mode[idx].index = idx;
			ldb->erp_mode[idx].type = wone_erp_mode;
			ldb->erp_mode[idx].elem.erp_mode = 0;

			ldb->erp_type[idx].index = idx;
			ldb->erp_type[idx].type = wone_erp_type;
			ldb->erp_type[idx].elem.erp_type = 0;

			break;


		case TIAP_PARAM_D11MODE_OFDM:

			ldb->oper_rate[idx].index = idx;
			ldb->oper_rate[idx].type = wone_oper_rate;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] =
				{ TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_BASIC_SUPPORTED,
					TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_BASIC_SUPPORTED };
					memcpy((void *)&ldb->oper_rate[idx].elem.b_legacy.rates, (void *)&rates,
							sizeof(ldb->oper_rate[idx].elem.b_legacy.rates));
			}
			ldb->oper_rate[idx].elem.b_legacy.len = 4;

			ldb->oper_add[idx].index = idx;
			ldb->oper_add[idx].type = wone_oper_add;
			memset((void *)&ldb->oper_add[idx].elem.ofdm_legacy.rates, TIAP_PARAM_RATE_NOT_SUPPORTED,
					sizeof(ldb->oper_add[idx].elem.b_legacy.rates));
			ldb->oper_add[idx].elem.ofdm_legacy.len = 0;

			ldb->oper_ofdm[idx].index = idx;
			ldb->oper_ofdm[idx].type = wone_oper_ofdm;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] = 
				{ TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED,
					TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED,
					TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED,
					TIAP_PARAM_RATE_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED };
					memcpy((void *)&ldb->oper_ofdm[idx].elem.ofdm_extend.rates, (void *)&rates,
							sizeof(ldb->oper_ofdm[idx].elem.ofdm_extend.rates));
			}
			ldb->oper_ofdm[idx].elem.ofdm_extend.len = 8;

			ldb->pbcc[idx].index = idx;
			ldb->pbcc[idx].type = wone_pbcc;
			ldb->pbcc[idx].elem.pbcc_mode = 0;

			ldb->pream_mode[idx].index = idx;
			ldb->pream_mode[idx].type = wone_pream_mode;
			ldb->pream_mode[idx].elem.pream_mode = 2;

			ldb->slot_mode[idx].index = idx;
			ldb->slot_mode[idx].type = wone_slot_mode;
			ldb->slot_mode[idx].elem.slot_mode = 2;

			ldb->erp_mode[idx].index = idx;
			ldb->erp_mode[idx].type = wone_erp_mode;
			ldb->erp_mode[idx].elem.erp_mode = 3;

			ldb->erp_type[idx].index = idx;
			ldb->erp_type[idx].type = wone_erp_type;
			ldb->erp_type[idx].elem.erp_type = 1;

			break;


		case TIAP_PARAM_D11MODE_A:

			ldb->oper_rate[idx].index = idx;
			ldb->oper_rate[idx].type = wone_oper_rate;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] =
				{ TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED,
					TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED };
					memcpy((void *)&ldb->oper_rate[idx].elem.b_legacy.rates, (void *)&rates,
							sizeof(ldb->oper_rate[idx].elem.b_legacy.rates));
			}
			ldb->oper_rate[idx].elem.b_legacy.len = 0;

			ldb->oper_add[idx].index = idx;
			ldb->oper_add[idx].type = wone_oper_add;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] =
				{ TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED,
					TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED,
					TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED,
					TIAP_PARAM_RATE_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED };
					memcpy((void *)&ldb->oper_add[idx].elem.ofdm_legacy.rates, (void *)&rates,
							sizeof(ldb->oper_add[idx].elem.b_legacy.rates));
			}
			ldb->oper_add[idx].elem.ofdm_legacy.len = 8;

			ldb->oper_ofdm[idx].index = idx;
			ldb->oper_ofdm[idx].type = wone_oper_ofdm;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] = 
				{ TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED,
					TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED };
					memcpy((void *)&ldb->oper_ofdm[idx].elem.ofdm_extend.rates, (void *)&rates,
							sizeof(ldb->oper_ofdm[idx].elem.ofdm_extend.rates));
			}
			ldb->oper_ofdm[idx].elem.ofdm_extend.len = 0;

			ldb->pbcc[idx].index = idx;
			ldb->pbcc[idx].type = wone_pbcc;
			ldb->pbcc[idx].elem.pbcc_mode = 0;

			ldb->pream_mode[idx].index = idx;
			ldb->pream_mode[idx].type = wone_pream_mode;
			ldb->pream_mode[idx].elem.pream_mode = 0;

			ldb->slot_mode[idx].index = idx;
			ldb->slot_mode[idx].type = wone_slot_mode;
			ldb->slot_mode[idx].elem.slot_mode = 1;

			ldb->erp_mode[idx].index = idx;
			ldb->erp_mode[idx].type = wone_erp_mode;
			ldb->erp_mode[idx].elem.erp_mode = 0;

			ldb->erp_type[idx].index = idx;
			ldb->erp_type[idx].type = wone_erp_type;
			ldb->erp_type[idx].elem.erp_type = 1;

			break;

		default: return;
	}
}


int tiap_wcfg_set_dot11mode(wcfgd *wd, int dot11mode, int wone_index, int update_storage)
{
	
	set_dot11mode(&wd->storage.ldb, dot11mode, wone_index);
	wd->storage.ldb.dot11_mode[wone_index].elem.dot11_mode = dot11mode;
	
	if( dot11mode == TIAP_PARAM_D11MODE_A) {
            
		__u8 band;

		band = wd->storage.ldb.band[wone_index].elem.band;
		switch (wd->storage.ldb.cur_regdomain[band].type)
        {
            case TIAP_PARAM_RD_FCC:
			case TIAP_PARAM_RD_US:
				wd->storage.ldb.cur_regdomain[TIAP_PARAM_RADIO_BAND_5GHZ].band = TIAP_PARAM_RADIO_BAND_5GHZ;
                wd->storage.ldb.cur_regdomain[TIAP_PARAM_RADIO_BAND_5GHZ].type = TIAP_PARAM_RD_US;
                break;

            case TIAP_PARAM_RD_IC:
            case TIAP_PARAM_RD_ETSI:
            case TIAP_PARAM_RD_SPAIN:
            case TIAP_PARAM_RD_FRANCE:
            case TIAP_PARAM_RD_MKK:
            case TIAP_PARAM_RD_MKK1:
			case TIAP_PARAM_RD_WORLD:
				wd->storage.ldb.cur_regdomain[TIAP_PARAM_RADIO_BAND_5GHZ].band = TIAP_PARAM_RADIO_BAND_5GHZ;
                wd->storage.ldb.cur_regdomain[TIAP_PARAM_RADIO_BAND_5GHZ].type = TIAP_PARAM_RD_WORLD;
                break;
            default:
				break;
        }
    
		wd->storage.ldb.ch[wone_index].elem.ch = TIAP_PARAM_DEF_A_CHANNEL;
		wd->storage.ldb.band[wone_index].elem.band = TIAP_PARAM_RADIO_BAND_5GHZ;	
	}
	else {
		__u8 wone_nr, radar_detect;

		if (wd->storage.ldb.band[wone_index].elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
		{
			wd->storage.ldb.ch[wone_index].elem.ch = TIAP_PARAM_DEF_CHANNEL;
			wd->storage.ldb.band[wone_index].elem.band = TIAP_PARAM_RADIO_BAND_2_4GHZ;
		}
		/* in all modes except TIAP_PARAM_D11MODE_A (in single band)
		   radar detect should be disabled*/

		/* check if AP is in single band */
		if (tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, FROM_STORAGE))
			return 1;

		/* if it is - disable radar detect */
		if(wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
		{
			radar_detect = 0;
			if (tiap_wcfg_set_basic(wd, TIAP_PARAM_RADAR_DETECT_MODE, 
						&radar_detect, UPDATE_STORAGE)) /*only for you*/
				return 1;
			printf("wcfglib: Setting radar detect mode to Disabled.\n");
		}

	}

	if (update_storage)
	{
		if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
			return 1;
	}

	return 0;
}

int tiap_wcfg_get_dot11mode(wcfgd *wd, __u8 *dot11_mode, int wone_index, int from_storage)
{
	tiap_param_wone_t wone;
	int ret = 0;


	if (from_storage)
	{
		*dot11_mode = wd->storage.ldb.dot11_mode[wone_index].elem.dot11_mode;
	}
	else
	{
		int ioctl_sock;

		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		wone.type = wone_dot11_mode;
		wone.index = wone_index;

		ret = exec_priv_ioctl_get(TIAP_PARAM_WONE_ELEM, &wone, 
				sizeof(wone),ioctl_sock, wd->dev_name);

		*dot11_mode = wone.elem.dot11_mode;
		close(ioctl_sock);
	}

	return ret;  


}

int tiap_wcfg_config_wpaauth(const wcfgd *wd, const tiap_wpa_auth_t *wa)
{
	memcpy((void *)&wd->storage.wpa, wa, sizeof(wd->storage.wpa));

	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}



int tiap_wcfg_get_sta_table(wcfgd *wd, tiap_param_statbl_t *sta_tbl)
{
	int ioctl_sock, ret;

	if((ioctl_sock = iw_sockets_open()) < 0)
	{
		perror("socket");
		return(-1);
	}

	ret = exec_priv_ioctl_get(TIAP_PARAM_STA_TBL, sta_tbl, 
			sizeof(tiap_param_statbl_t)*TIAP_PARAM_MAX_STATIONS,ioctl_sock, wd->dev_name);

	close(ioctl_sock);

	return ret;  
}

int tiap_wcfg_set_wep_default_key_id(wcfgd *wd, __u8 key_id)
{
	wd->storage.wpa.type.sec.wep_def_key_id = key_id;

	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
	
	return 0;

}


void tiap_wcfg_get_wep_default_key_id(wcfgd *wd, __u8 *key_id)
{
	*key_id = wd->storage.wpa.type.sec.wep_def_key_id;
}
	

int tiap_wcfg_set_def_wep_key(wcfgd *wd, int key_idx, tiap_wpa_wep_key_t *wep_key)
{
	
	if( key_idx > TIAP_WPA_DEFS_KEY_NAL_LEN-1 )
		return 1;


	wd->storage.wpa.type.sec.keys[key_idx].len = wep_key->len;
	memcpy(wd->storage.wpa.type.sec.keys[key_idx].key, wep_key->key, wep_key->len);

	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
	
	return 0;
}

int tiap_wcfg_get_def_wep_key(wcfgd *wd, int key_idx, tiap_wpa_wep_key_t *wep_key)
{
	if( key_idx > TIAP_WPA_DEFS_KEY_NAL_LEN-1 )
		return 1;
	
	wep_key->len = wd->storage.wpa.type.sec.keys[key_idx].len;
	memcpy(wep_key->key, wd->storage.wpa.type.sec.keys[key_idx].key, wep_key->len);
	
	return 0;
}

void  print_mac(const char *mac)
{
	fprintf(stderr,"%02hhx:%02hhx:%02hhx:%02hhx:%02hhx:%02hhx\n", 
			     mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]); 
}


int tiap_wcfg_set_cur_regdomain(wcfgd *wd, tiap_param_cur_reg_domain_t *cur_regdomain,
		const int update_storage)
{
	int band;

	band = cur_regdomain->band;
	wd->storage.ldb.cur_regdomain[band] = *cur_regdomain;

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}


int tiap_wcfg_get_cur_regdomain(wcfgd *wd, tiap_param_cur_reg_domain_t *cur_regdomain, int from_storage)
{
	int ioctl_sock, ret = 0;

	if (from_storage){
		int band;
		
		band = cur_regdomain->band;
		*cur_regdomain = wd->storage.ldb.cur_regdomain[band]; 
	}
	else
	{
		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = exec_priv_ioctl_get(TIAP_PARAM_CURR_REG_DOMAIN, cur_regdomain, 
				sizeof(*cur_regdomain), ioctl_sock, wd->dev_name);

		close(ioctl_sock);
	}

	return ret;  

}


int tiap_wcfg_set_priv_regdomain(wcfgd *wd, tiap_param_priv_reg_domain_t *priv_regdomain,
		const int update_storage)
{
	wd->storage.ldb.priv_regdomain = *priv_regdomain;

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}


int tiap_wcfg_get_priv_regdomain(wcfgd *wd, tiap_param_priv_reg_domain_t *priv_regdomain, int from_storage)
{
	int ioctl_sock, ret = 0;

	if (from_storage)
		*priv_regdomain = wd->storage.ldb.priv_regdomain;
	else
	{
		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = exec_priv_ioctl_get(TIAP_PARAM_PRIV_REG_DOMAIN, priv_regdomain,
				sizeof(*priv_regdomain), ioctl_sock, wd->dev_name);

		close(ioctl_sock);
	}

	return ret;  

}

int tiap_wcfg_get_priv_net_stats(wcfgd *wd, tiap_param_net_stats *stats)
{
	int ioctl_sock, ret;

	if((ioctl_sock = iw_sockets_open()) < 0) {
		perror("socket");
		return(-1);
	}

	ret = exec_priv_ioctl_get(TIAP_PARAM_NET_STATS, stats, sizeof(*stats), ioctl_sock, wd->dev_name);
	close(ioctl_sock);

	return ret;
}

int tiap_wcfg_set_defaults(wcfgd *wd)
{
	wcfg_storage_setdefaults(&wd->storage);
	wd->storage.defaults = 1;

	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_set_psk_passphrase_mode(wcfgd *wd, int value)
{
	wd->storage.wpa.type.psk.pass_phrase_mode = value;
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;

}

void tiap_wcfg_get_psk_passphrase_mode(wcfgd *wd, int *value)
{
	*value = init_wpa.type.psk.pass_phrase_mode;
}


int tiap_wcfg_set_psk_passwd(wcfgd *wd, __s8 *passwd, __u8 passwd_len)
{
	wd->storage.wpa.type.psk.psk_passwd_len = passwd_len;
	memcpy(wd->storage.wpa.type.psk.psk_passwd, passwd, passwd_len);
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;

}

void tiap_wcfg_get_psk_passwd(wcfgd *wd, __s8 *passwd, __u8 *passwd_len)
{
	*passwd_len = init_wpa.type.psk.psk_passwd_len;
	memcpy(passwd, init_wpa.type.psk.psk_passwd, init_wpa.type.psk.psk_passwd_len);

}

int tiap_wcfg_set_default_cipher(wcfgd *wd, __u32 cipher)
{
	wd->storage.wpa.type.sec.default_cipher = cipher;
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;

}

void tiap_wcfg_get_default_cipher(wcfgd *wd, __u32 *cipher)
{
	*cipher = init_wpa.type.sec.default_cipher ;
}

int tiap_wcfg_set_gtk_interval(wcfgd *wd, __u32 intvl)
{
	wd->storage.wpa.type.sec.interval = intvl;
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;

}

void tiap_wcfg_get_gtk_interval(wcfgd *wd, __u32 *intvl)
{
	*intvl = init_wpa.type.sec.interval;
	
}

int tiap_wcfg_set_radius_port(wcfgd *wd, __u16 port)
{
	wd->storage.wpa.type.radius.port = port;
	
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
	
	return 0;
}

void tiap_wcfg_get_radius_port(wcfgd *wd, __u16 *port)
{
	*port = init_wpa.type.radius.port;
	
}
int tiap_wcfg_set_radius_ipaddr(wcfgd *wd, __u32 ipaddr)
{
	wd->storage.wpa.type.radius.serv_ip_addr = ipaddr;
	
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
	
	return 0;
}

void tiap_wcfg_get_radius_ipaddr(wcfgd *wd, __u32 *ipaddr)
{
	*ipaddr = init_wpa.type.radius.serv_ip_addr ;
}

void tiap_wcfg_get_radius_secret(wcfgd *wd, __s8 *secret, __u8 *len)
{
	//memcpy(wd->storage.wpa.type.radius.secret, ipaddr, TIAP_WPA_MAX_RADIUS_SECRET_LEN);
	*len = init_wpa.type.radius.secret_len;
	memcpy(secret, init_wpa.type.radius.secret, *len);
}

int tiap_wcfg_set_radius_secret(wcfgd *wd, __s8 *secret, __u8 len)
{
	if( len > TIAP_WPA_MAX_RADIUS_SECRET_LEN )
		return 1;

	wd->storage.wpa.type.radius.secret_len = len;
	memcpy(wd->storage.wpa.type.radius.secret, secret, len);
	
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
	
	return 0;
}


int tiap_wcfg_set_dual_ap(wcfgd *wd, __u8 wone_nr)
{

	if (wone_nr == wd->storage.ldb.wone_num)
	{
	      printf("AP stays in the same wone mode\n");
	      return 0;
	}

	if ( wone_nr < wd->storage.ldb.wone_num )
	{
		if( tiap_wcfg_set_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, UPDATE_STORAGE) )
			return 1;
	}
	else if (wone_nr > wd->storage.ldb.wone_num )
	{
		int beacon_period;

		if( tiap_wcfg_set_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !UPDATE_STORAGE) )
			return 1;

		if ( tiap_wcfg_set_dot11mode(wd, TIAP_PARAM_D11MODE_MIXED, 0, !UPDATE_STORAGE ) )
			return 1;
		if ( tiap_wcfg_set_dot11mode(wd, TIAP_PARAM_D11MODE_A, 1, !UPDATE_STORAGE ) )
			return 1;

		beacon_period = TIAP_PARAM_DEF_WONE_BEACON_PERIOD;

		if ( tiap_wcfg_set_basic(wd, TIAP_PARAM_BEACON_PERIOD,
					  &beacon_period, UPDATE_STORAGE) )
						
			return 1;

	}

	return 0;

}

static int set_radar_detect(wcfgd *wd, __u8 val)
{
	__u8 wone_nr, dot11_mode;

	if (val) /* enable */
	{
		/* Allow to enable radar detect only if AP works in TIAP_PARAM_D11MODE_A 
		   if AP works in dual band it is garanteered that one of two bands is TIAP_PARAM_D11MODE_A*/
		
		/* So we check if AP is in single band */
		if (tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, FROM_STORAGE))
			return 1;
		
		if(wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
		{
			if (tiap_wcfg_get_dot11mode(wd, &dot11_mode, 0, FROM_STORAGE))
				return 1;
			
			if (dot11_mode != TIAP_PARAM_D11MODE_A)
			{
				printf("Radar detect cannot be enabled while AP is not in A_Only mode\n");
				return 1;
			}

		}
	}
			

	/* in all other cases radar detect can be set */
	wd->storage.ldb.radar_detect = val;

	return 0;
}

int tiap_wcfg_set_gpio_owner(wcfgd *wd, __u8 val)
{
	wd->storage.ldb.gpio_owner = val;
	
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

