/* LICENSE_CODE JUNGO */

#include "auth1x_pktfil.h"
#include "port_ctl_types.h"
#include <kos/knet.h>
#include <kos/kos_chardev.h>

#define MAX_WAITING_PACKETS 20

typedef struct auth1x_pkt_t {
    struct auth1x_pkt_t *next;
    knet_buf_t *pkt_buf;
} auth1x_pkt_t;

/* 802.1x char device context, created per authenticated interface */
typedef struct mac_list_t {
    struct mac_list_t *next;
    knet_ether_addr_t mac;
} mac_list_t;

/* This is the char device context passed by its ops */
typedef struct auth1x_if_context_t {
    struct auth1x_if_context_t *next;
    auth1x_pkt_t *pkt_list;     /* list of eapol packets waiting to be read */
    kos_wait_queue_t waitq;	/* wait queue */
    knet_netdev_t *ifs;		/* interface to bind to */
    auth1x_mode_t mode;		/* authentication mode */
    mac_list_t *auth_clients;	/* MACs of authenticated clients */
    int use_count;		/* number of files open on this context */
    int pass_single_unauth;	/* pass next unauthorized packet to pkt_list */
} auth1x_if_context_t;

static knet_hook_ret_type_t auth1x_pktfil_rx_hook(knet_ip_hdr_t **ip_hdr,
    knet_netdev_t *dev, knet_buf_t **pkt_buf);
static knet_hook_ret_type_t auth1x_pktfil_tx_hook(knet_ip_hdr_t **ip_hdr,
    knet_netdev_t *dev, knet_buf_t **pkt_buf);

static void mac_list_del(auth1x_if_context_t *fdc);

#define AUTH1X_PKTFIL_CHECK_FDC(fdc) \
    do { \
        if (!(fdc)) \
        { \
	    kos_print(__FUNCTION__ ": no priv in context \n"); \
	    return -1; \
        } \
    } while(0)

static auth1x_if_context_t *auth1x_context_list;

static knet_ether_addr_t zero_mac = {{0,0,0,0,0,0}};

static int auth1x_pktfil_open(kos_chardev_t *context)
{
    context->priv = NULL;
    return 0;
}

static auth1x_if_context_t *create_fd_context(kos_chardev_t *context)
{
    auth1x_if_context_t *fdc;
    
    if (!(fdc = kos_alloc(sizeof(auth1x_if_context_t), KOS_ALLOC_KERNEL)))
    {
	kos_print(__FUNCTION__ ": kos_alloc failed\n");
	return NULL;
    }
    memset(fdc, 0, sizeof(auth1x_if_context_t));
    
    /* Add to global list */
    fdc->next = auth1x_context_list;
    auth1x_context_list = fdc;

    kos_waitq_init(&fdc->waitq);
    fdc->use_count = 1;
    context->priv = fdc;
    return fdc;
}

static void free_hanging_packets(auth1x_pkt_t **pkt_list)
{
    auth1x_pkt_t *dp = *pkt_list;

    while (dp)
    {
	*pkt_list = dp->next;
	if (dp->pkt_buf)
	    knet_buf_free(dp->pkt_buf);
	kos_free(dp);
	dp = *pkt_list;
    }
}

static int auth1x_pktfil_close(kos_chardev_t *context)
{
    auth1x_if_context_t *fdc = (auth1x_if_context_t *)context->priv;
    auth1x_if_context_t **f;

    if (!fdc)
	return 0;

    if (--fdc->use_count)
	return 0;
    
    if (fdc->ifs && fdc->ifs->netdev_name)
    {
	/* Delete device from rx hook */
	if (knet_buf_rx_hook_dev_del(auth1x_pktfil_rx_hook, 
	    fdc->ifs->netdev_name, KNET_PRI_AUTH1X) < 0)
	{
	    kos_print(__FUNCTION__ ": %s rx dev del failed\n", 
		fdc->ifs->netdev_name);
	}
	/* Delete device from tx hook */
	if (knet_buf_tx_hook_dev_del(auth1x_pktfil_tx_hook, 
	    fdc->ifs->netdev_name, KNET_PRI_AUTH1X) < 0)
	{
	    kos_print(__FUNCTION__ ": %s tx dev del failed\n", 
		fdc->ifs->netdev_name);
	}
	/* turn off promiscuous mode */
        if (fdc->mode.promisc)
	    knet_netdev_promisc_set(fdc->ifs, 0);
    }
    
    /* Free any packet still hanging... */
    free_hanging_packets(&fdc->pkt_list); 
    mac_list_del(fdc);

    /* Remove the context from the list */
    for (f = &auth1x_context_list; *f && *f != fdc; f = &(*f)->next);
    if (*f)
	*f = (*f)->next;
    else
	kos_print(__FUNCTION__ ": fdc not in list\n");

    kos_free(fdc);
    return 0;
}

static int auth1x_pktfil_read(kos_chardev_t *context, char *buf, int len)
{
    auth1x_if_context_t *fdc = (auth1x_if_context_t *)context->priv;
    auth1x_pkt_t *pkt;
    int data_read;
       
    AUTH1X_PKTFIL_CHECK_FDC(fdc);
     
    pkt = fdc->pkt_list;
 
    /* XXX: Sanity check, to be removed when code proves stable */
    if (!pkt)
    {
	kos_print(__FUNCTION__ ": pkt_list empty!\n");
	return -1;
    }

    /* prepare to read packet ethernet header */
    knet_buf_data_push(pkt->pkt_buf, KNET_ETHER_HDR_LEN);

    /* make sure len is big enough to accept the packet */
    if (pkt->pkt_buf->knet_pktlen > len)
    {
	kos_print(__FUNCTION__ ": packet buffer too small\n");
	data_read = -1;
    }
    else if (!(data_read = knet_buf_copy_to_user(pkt->pkt_buf, buf, 0,
	pkt->pkt_buf->knet_pktlen)))
    {
	kos_print(__FUNCTION__ ": knet_buf_copy_to_user failed\n");
	data_read = -1;
    }

    /* Delete the packet from pkt_list */
    fdc->pkt_list = pkt->next;
    knet_buf_free(pkt->pkt_buf);
    kos_free(pkt);

    return data_read;
}

static int auth1x_pktfil_write(kos_chardev_t *context, const char *buf, int len)
{
    auth1x_if_context_t *fdc = (auth1x_if_context_t *)context->priv;

    AUTH1X_PKTFIL_CHECK_FDC(fdc);
    
    /* Send buf to interface */
    if (knet_netdev_send(fdc->ifs, buf, len, 1) < 0)
    {
	kos_print(__FUNCTION__ ": knet_netdev_send failed\n");
	return -1;
    }

    return len;
}

static int is_mac_zero(knet_ether_addr_t *mac)
{
    return !memcmp(mac->ether_addr_octet, zero_mac.ether_addr_octet,
	KNET_ETHER_ADDR_LEN);
}

static void mac_list_del(auth1x_if_context_t *fdc)
{
    while (fdc->auth_clients)
    {
	mac_list_t *mac = fdc->auth_clients;

	fdc->auth_clients = mac->next;
	kos_free(mac);
    }
}

static mac_list_t **mac_find(auth1x_if_context_t *fdc, knet_ether_addr_t *mac)
{
    mac_list_t **mac_list = &fdc->auth_clients;

    /* find mac in MAC list */
    for (; *mac_list && memcmp((*mac_list)->mac.ether_addr_octet,
	mac->ether_addr_octet, KNET_ETHER_ADDR_LEN);
       	mac_list = &(*mac_list)->next);

    return mac_list;
}

static int mac_add(auth1x_if_context_t *fdc, knet_ether_addr_t *mac)
{
    mac_list_t **mac_list;
    
    if (is_mac_zero(mac))
    {
	kos_print(__FUNCTION__ ": Illegal MAC\n");
	return -1;
    }
    
    mac_list = mac_find(fdc, mac);

    /* if found - return */
    if (*mac_list)
	return 0;

    if (!(*mac_list = kos_alloc(sizeof(mac_list_t), KOS_ALLOC_KERNEL)))
    {
	kos_print(__FUNCTION__ ": kos_alloc failed\n");
	return -1;
    }
    (*mac_list)->next = NULL;
    memcpy((*mac_list)->mac.ether_addr_octet, mac->ether_addr_octet, 
	KNET_ETHER_ADDR_LEN);
    return 0;
}

static void mac_del(auth1x_if_context_t *fdc, knet_ether_addr_t *mac)
{
    mac_list_t **mac_list = mac_find(fdc, mac);
    mac_list_t *tmp;

    if (!*mac_list)
	return;

    tmp = *mac_list;
    *mac_list = (*mac_list)->next;
    kos_free(tmp);
}

static knet_ether_addr_t *mac_get_next(auth1x_if_context_t *fdc, 
    knet_ether_addr_t *mac)
{
    mac_list_t **mac_list;
    
    if (is_mac_zero(mac))
	return fdc->auth_clients ? &fdc->auth_clients->mac : &zero_mac;
	
    mac_list = mac_find(fdc, mac);

    return *mac_list && (*mac_list)->next ? &(*mac_list)->next->mac : &zero_mac;
}

static int auth1x_pktfil_ioctl(kos_chardev_t *context, unsigned int cmd,
    unsigned long arg)
{
    auth1x_dev_t dev;
    auth1x_mode_t mode;
    knet_ether_addr_t mac;
    auth1x_if_context_t *fdc = (auth1x_if_context_t *)context->priv;
    int ret = 0;
    kos_interrupt_flags_t flags = kos_enter_protected();

    if (cmd != SIOCSAUTH1XFILDEV && !fdc)
    {
	kos_print(__FUNCTION__ ": fd has not been initialized\n");
	ret = ENODEV;
	goto Exit;
    }

    switch (cmd)
    {
    case SIOCSAUTH1XFILDEV:
	if (kos_copy_from_user(&dev, (void *)arg, sizeof(dev)) < 0)
	{
	    kos_print(__FUNCTION__ ": kos_copy_from_user failed\n");
	    ret = EFAULT;
	    goto Exit;
	}
	
	/* Check if fdc exists fore this dev.name. If so use it otherwise 
	 * create a new one */
	for (fdc = auth1x_context_list; fdc; fdc = fdc->next)
	{
	    if (!strcmp(fdc->ifs->netdev_name, dev.name))
	    {
		context->priv = fdc;
		fdc->use_count++;
		break;
	    }
	}
        
	if (!fdc && !(fdc = create_fd_context(context)))
        {
	    ret = ENOMEM;
	    goto Exit;
	}

	if (fdc->ifs)
	    break;

	if (!(fdc->ifs = knet_netdev_get_by_name(dev.name)))
	{
	    kos_print(__FUNCTION__ ": knet_netdev_get_by_name failed\n");
	    ret = ENOENT;
	    goto Exit;
	}

	/* Add device to rx hook */
	if (knet_buf_rx_hook_dev_add(auth1x_pktfil_rx_hook, dev.name,
	    KNET_PRI_AUTH1X) < 0 ||
	    knet_buf_rx_hook_dev_set(auth1x_pktfil_rx_hook, 1,
	    KNET_PRI_AUTH1X) < 0)
	{
	    kos_print(__FUNCTION__ ": %s rx dev add failed\n", dev.name);
	    ret = ENODEV;
	    goto Exit;
	}
	/* Add device to tx hook */
	if (knet_buf_tx_hook_dev_add(auth1x_pktfil_tx_hook, dev.name,
	    KNET_PRI_AUTH1X) < 0 ||
	    knet_buf_tx_hook_dev_set(auth1x_pktfil_tx_hook, 1,
	    KNET_PRI_AUTH1X) < 0)
	{
	    kos_print(__FUNCTION__ ": %s tx dev add failed\n", dev.name);
	    ret = ENODEV;
	    goto Exit;
	}
	break;
    case SIOCSAUTH1XSETMODE:
	if (kos_copy_from_user(&mode, (void *)arg, sizeof(mode)) < 0)
	{
	    kos_print(__FUNCTION__ ": kos_copy_from_user failed\n");
	    ret = EFAULT;
	    goto Exit;
	}

        if (fdc->mode.auth_ctl != mode.auth_ctl)
	    mac_list_del(fdc);

        if (fdc->mode.promisc != mode.promisc)
	    knet_netdev_promisc_set(fdc->ifs, mode.promisc);

	fdc->mode = mode;
	break;
    case SIOCGAUTH1XGETMODE:
	if (kos_copy_to_user((void *)arg, &fdc->mode, sizeof(mode)) < 0)
	{
	    kos_print(__FUNCTION__ ": kos_copy_to_user failed\n");
	    ret = EFAULT;
	    goto Exit;
	}
	break;
    case SIOCSAUTH1XAUTHMAC:
	if (kos_copy_from_user(&mac, (void *)arg, sizeof(mac)) < 0)
	{
	    kos_print(__FUNCTION__ ": kos_copy_from_user failed\n");
	    ret = EFAULT;
	    goto Exit;
	}

	if (mac_add(fdc, &mac))
	{
	    ret = EFAULT;
	    goto Exit;
	}
	break;
    case SIOCSAUTH1XUNAUTHMAC:
	if (kos_copy_from_user(&mac, (void *)arg, 
	    sizeof(knet_ether_addr_t)) < 0)
	{
	    kos_print(__FUNCTION__ ": kos_copy_from_user failed\n");
	    ret = EFAULT;
	    goto Exit;
	}
	
	mac_del(fdc, &mac); 
	break;
    case SIOCGAUTH1XGETNEXTMAC:
	if (kos_copy_from_user(&mac, (void *)arg, 
	    sizeof(knet_ether_addr_t)) < 0)
	{
	    kos_print(__FUNCTION__ ": kos_copy_from_user failed\n");
	    ret = EFAULT;
	    goto Exit;
	}
	
	if (kos_copy_to_user((void *)arg, mac_get_next(fdc, &mac),
	    sizeof(knet_ether_addr_t)) < 0)
	{
	    kos_print(__FUNCTION__ ": kos_copy_to_user failed\n");
	    ret = EFAULT;
	    goto Exit;
	}
	break;
    case SIOCGAUTH1XGETUSECOUNT:
	if (kos_copy_to_user((void *)arg, &fdc->use_count, sizeof(int)) < 0)
	{
	    kos_print(__FUNCTION__ ": kos_copy_to_user failed\n");
	    ret = EFAULT;
	    goto Exit;
	}
	break;
    case SIOCSAUTH1XSETPASSSINGLEUNAUTH:
	if (kos_copy_from_user(&fdc->pass_single_unauth, (void *)arg, 
	    sizeof(int)) < 0)
	{
	    kos_print(__FUNCTION__ ": kos_copy_from_user failed\n");
	    ret = EFAULT;
	    goto Exit;
	}
	break;
    case SIOCGAUTH1XGETPASSSINGLEUNAUTH:
	if (kos_copy_to_user((void *)arg, &fdc->pass_single_unauth, 
	    sizeof(int)) < 0)
	{
	    kos_print(__FUNCTION__ ": kos_copy_to_user failed\n");
	    ret = EFAULT;
	    goto Exit;
	}
	break;
   default:
	ret = EINVAL;
    }
Exit:
    kos_exit_protected(flags);
    return KOS_ERRNO(ret);
}

static unsigned int auth1x_pktfil_select(kos_chardev_t *context,
    kos_poll_node_t poll_node)
{
    auth1x_if_context_t *fdc = (auth1x_if_context_t *)context->priv;

    AUTH1X_PKTFIL_CHECK_FDC(fdc);

    kos_poll_wait(KOS_CHARDEV_GET_FILP(context), &fdc->waitq, poll_node);
    
    /* Read:  Only if there are packets on pkt_list
     * Write: Allways
     */
    return kos_select(poll_node, (int)fdc->pkt_list, 1);
}

static int auth1x_pktfil_unselect(kos_chardev_t *context,
    kos_poll_node_t poll_node)
{
    auth1x_if_context_t *fdc = (auth1x_if_context_t *)context->priv;

    AUTH1X_PKTFIL_CHECK_FDC(fdc);
     
    return kos_poll_node_del(&fdc->waitq, poll_node);
}

static void packet_add(auth1x_if_context_t *fdc, auth1x_pkt_t **pkt_list, 
    knet_buf_t **pkt_buf)
{
    auth1x_pkt_t *pkt, **pkt_list_end;
    int pkt_list_count = 0;

    /* Add the packet to the pkt_list */
    for (pkt_list_end = pkt_list; *pkt_list_end;
	pkt_list_end = &(*pkt_list_end)->next);
    {
	pkt_list_count++;
    }

    if (pkt_list_count >= MAX_WAITING_PACKETS)
	return; /* too many waiting packets in the list */

    if (!(pkt = kos_alloc(sizeof(auth1x_pkt_t), KOS_ALLOC_ATOMIC)))
    {
	kos_print(__FUNCTION__ ": kos_alloc for auth1x_pkt failed\n");
	return;
    }

    pkt->next = NULL;

    /* Always clone, so that the original will not be freed by the read
     * operation of the waking fd until we finish. */
    pkt->pkt_buf = knet_buf_clone(*pkt_buf, KNET_DONTWAIT);

    *pkt_list_end = pkt;

    /* Throttling: Wake up the waitq for reading */
    kos_wake_up_int_read(&fdc->waitq);
}

static inline int is_mac_broadcast(u8 *ether_host)
{
    static knet_ether_addr_t broadcast_mac = {{0xff,0xff,0xff,0xff,0xff,0xff}};

    return !memcmp(broadcast_mac.ether_addr_octet, ether_host, 
	KNET_ETHER_ADDR_LEN);
}

static inline int is_mac_multicast(u8 *ether_host)
{
    /* multicast MAC is from 01:00:5e:00:00:00 to 01:00:5e:7f:ff:ff */
    return ether_host[0] == 0x01 && ether_host[1] == 0x00 &&
       	ether_host[2] == 0x5e && !(ether_host[3] & 0x80);
}

static int is_mac_auth(auth1x_if_context_t *fdc, u8 *ether_host)
{
    return (int)*(mac_find(fdc, (knet_ether_addr_t *)ether_host));
}

static knet_hook_ret_type_t auth1x_pktfil_rx_hook(knet_ip_hdr_t **ip_hdr,
    knet_netdev_t *dev, knet_buf_t **pkt_buf)
{
    auth1x_if_context_t *fdc;
    knet_eth_hdr_t *eth_hdr;
    
    /* Find the fdc this packet was destined for */
    for (fdc = auth1x_context_list;
	fdc && strcmp(fdc->ifs->netdev_name, dev->netdev_name);
	fdc = fdc->next);

    if (!fdc)
	return KNET_PKT_NOT_HANDLED;

    eth_hdr = (knet_eth_hdr_t *)(*pkt_buf)->knet_data - 1;
    /* If Packet is EAPOL_TYPE add to packet list and return handled */
    if (ntohs(eth_hdr->ether_type) == EAPOL_TYPE)
	packet_add(fdc, &fdc->pkt_list, pkt_buf);
    /* Else, pass packets that adheres to authorization */
    else if (fdc->mode.auth_ctl == ACPC_FORCE_AUTHORIZED ||
	(fdc->mode.auth_ctl == ACPC_AUTO &&
	is_mac_auth(fdc, eth_hdr->ether_shost)))
    {
	return KNET_PKT_NOT_HANDLED;
    } 
    else if (fdc->pass_single_unauth &&
	memcmp(eth_hdr->ether_shost, KNET_LLADDR(dev), KNET_ETHER_ADDR_LEN))
    {
	fdc->pass_single_unauth = 0;
	packet_add(fdc, &fdc->pkt_list, pkt_buf);
    }

    knet_buf_free(*pkt_buf);
    return KNET_PKT_HANDLED;
}

static knet_hook_ret_type_t auth1x_pktfil_tx_hook(knet_ip_hdr_t **ip_hdr,
    knet_netdev_t *dev, knet_buf_t **pkt_buf)
{
    auth1x_if_context_t *fdc;
    /* In TX, data points to link layer header */
    knet_eth_hdr_t *eh = (knet_eth_hdr_t *)((*pkt_buf)->knet_data);
    
    /* If Packet is EAPOL_TYPE - allow transmission */
    if (ntohs(eh->ether_type) == EAPOL_TYPE)
	return KNET_PKT_NOT_HANDLED;

    /* Find the fdc this packet is destined for */
    for (fdc = auth1x_context_list;
	fdc && strcmp(fdc->ifs->netdev_name, dev->netdev_name);
	fdc = fdc->next);

    /* Check that packet adheres to authorization */
    if (!fdc || fdc->mode.dir == ACD_IN || 
	fdc->mode.auth_ctl == ACPC_FORCE_AUTHORIZED || 
	(fdc->mode.auth_ctl == ACPC_AUTO && 
	(is_mac_auth(fdc, eh->ether_dhost) ||
	is_mac_multicast(eh->ether_dhost) ||
	is_mac_broadcast(eh->ether_dhost))))
    {
	return KNET_PKT_NOT_HANDLED;
    }

    knet_buf_free(*pkt_buf);
    return KNET_PKT_HANDLED;
}

static const kos_reg_chardev_t auth1x_pktfil_reg_cd = {
    type:     KOS_CDT_AUTH1X,
    open:     auth1x_pktfil_open,
    close:    auth1x_pktfil_close,
    read:     auth1x_pktfil_read,
    write:    auth1x_pktfil_write,
    ioctl:    auth1x_pktfil_ioctl,
    select:   auth1x_pktfil_select,
    unselect: auth1x_pktfil_unselect,
};

/* Uninitialize device driver, called by user at shutdown time */
void auth1x_pktfil_uninit(void)
{
    knet_buf_rx_hook_dev_clear(auth1x_pktfil_rx_hook, KNET_PRI_AUTH1X);
    knet_buf_tx_hook_dev_clear(auth1x_pktfil_tx_hook, KNET_PRI_AUTH1X);
    knet_buf_rx_hook_unregister(auth1x_pktfil_rx_hook, KNET_PRI_AUTH1X);
    knet_buf_tx_hook_unregister(auth1x_pktfil_tx_hook, KNET_PRI_AUTH1X);
 
    if (kos_chardev_unregister((kos_reg_chardev_t *)&auth1x_pktfil_reg_cd) < 0)
	kos_print(__FUNCTION__ ": kos_chardev_unregister failed\n");
}

/* Initialize device driver, called by user at init time */
int auth1x_pktfil_init(void)
{
    if (kos_chardev_register((kos_reg_chardev_t *)&auth1x_pktfil_reg_cd) < 0)
    {
	kos_print(__FUNCTION__ ": kos_chardev_register failed\n");
	return -1;
    }
    /* Register knet_rx hook */
    if (knet_buf_rx_hook_register(auth1x_pktfil_rx_hook, KNET_PRI_AUTH1X) < 0)
    {
	kos_print(__FUNCTION__ ": knet_buf_rx_hook_register failed\n");
	goto Error;
    }
    /* Register knet_tx hook */
    if (knet_buf_tx_hook_register(auth1x_pktfil_tx_hook, KNET_PRI_AUTH1X) < 0)
    {
	kos_print(__FUNCTION__ ": knet_buf_tx_hook_register failed\n");
	goto Error;
    }
    /* set rx hook filter by device list */
    if (knet_buf_rx_hook_dev_set(auth1x_pktfil_rx_hook, 1, KNET_PRI_AUTH1X) < 0)
    {
	kos_print(__FUNCTION__ ": knet_buf_rx_hook_dev_set failed\n");
	goto Error;
    }
    /* set tx hook filter by device list */
    if (knet_buf_tx_hook_dev_set(auth1x_pktfil_tx_hook, 1, KNET_PRI_AUTH1X) < 0)
    {
	kos_print(__FUNCTION__ ": knet_buf_tx_hook_dev_set failed\n");
	goto Error;
    }

    return 0;
Error:
    auth1x_pktfil_uninit();
    return -1;
}

#ifdef MODULE
int init_module(void)
{
    if (auth1x_pktfil_init() < 0)
	return -1;
    return 0;
}

void cleanup_module(void)
{
    auth1x_pktfil_uninit();
}
#endif
