/* LICENSE_CODE JUNGO */

/* 802.1x port authentication implementation */
#define OS_INCLUDE_STD

#include "rg_error.h"

#include <os_includes.h>
#include <event.h>
#include <util/rg_error.h>
#include <util/memory.h>
#include <util/conv.h>
#include "auth1x.h"
#include "auth1x_conn.h"
#include "port_ctl.h"
#include "eap.h"
#include "eapol.h"
#include "rg_error.h"



/* max allowed number of concurrent authorized clients per device */
#define MAX_DEV_AUTH_CONNECTIONS 100
#define PASS_SINGLE_UNAUTH_PERIOD (10 * MS_SECOND)
#define AUTH1X_LOG_PREFIX "auth1x: "


/* helpers used in connection state machines */
#ifdef DEBUG
#include <syscalls.h>
#define _SM_MOVE_PRINT(sm_prefix, new_state, quiet) \
    if (!quiet) \
    { \
	rg_error(LERR, "%s: state machine %s moved to %s (st %p)\n", \
	    log_time(), #sm_prefix, #new_state, st); \
    }
#else
#define _SM_MOVE_PRINT(sm_prefix, new_state, quiet)
#endif

#define SM_DEC(x) do { if ((x)) (x)--; } while (0)

#define SM_INC(x) do { if (++(x) > 255) (x) = 0; } while (0)

#define SM_DEFS \
    auth1x_conn_t *conn = o; \
    auth1x_state_machines_t *st = &conn->st; \
    int changed = 0; \
    if (conn->closing) \
	return 0;

#define _SM_MOVE_STATE(sm_prefix, new_state, quiet) \
    do { \
	st->sm_prefix ## _state = new_state; \
	st->sm_prefix ## _did_state_action = 0; \
	changed = 1; \
	_SM_MOVE_PRINT(sm_prefix, new_state, quiet) \
    } while (0)

#define SM_MOVE_STATE_QUIET(sm_prefix, new_state) \
    _SM_MOVE_STATE(sm_prefix, new_state, 1)

#define SM_MOVE_STATE(sm_prefix, new_state) \
    _SM_MOVE_STATE(sm_prefix, new_state, 0)

#define SM_IF_NOT_DID_ACTION(sm_prefix) \
    for (;!st->sm_prefix ## _did_state_action; \
	changed = 1, st->sm_prefix ## _did_state_action = 1)

/* helpers for global state machines */
#define SM_GLOB_DEFS \
    auth1x_port_t *st = o; \
    int changed = 0;

#define CODE2STR_ENTRY(x) { ET_ ## x, #x }
static code2str_t eapol_type2str[] = {
    CODE2STR_ENTRY(EAP),
    CODE2STR_ENTRY(START),
    CODE2STR_ENTRY(LOGOFF),
    CODE2STR_ENTRY(KEY),
    CODE2STR_ENTRY(ENCAPSULATED_ASF_ALERT),
    {-1}
};

/* Use for calling state machines, errmsg is format and args for error report */
#define STATE_MACHINE_DO_STATE_REPORT(st, context, errmsg...) \
    do \
    { \
	if (state_machine_do_state(st, context)) \
	    rg_error(LERR, AUTH1X_LOG_PREFIX "%s", errmsg); \
    } \
    while (0)

#define GLOBAL_STATE_MACHINE_DO_STATE(dev,errmsg...) \
    STATE_MACHINE_DO_STATE_REPORT(eapol_glob_state_machines,dev,errmsg)

#ifdef CONFIG_RG_8021X_MD5
extern eap_method_t eap_md5_methods;
#endif
#ifdef CONFIG_RG_8021X_TLS
extern eap_method_t eap_tls_methods;
#endif
#ifdef CONFIG_RG_8021X_TTLS
extern eap_method_t eap_ttls_methods;
#endif
#ifdef CONFIG_RG_RADIUS
extern eap_method_t eap_radius_methods;
#endif

static eap_method_t *eap_methods[] = {
#ifdef CONFIG_RG_8021X_MD5
    &eap_md5_methods,
#endif
#ifdef CONFIG_RG_8021X_TLS
    &eap_tls_methods,
#endif
#ifdef CONFIG_RG_8021X_TTLS
    &eap_ttls_methods,
#endif
#ifdef CONFIG_RG_RADIUS
    &eap_radius_methods,
#endif
    NULL
};

#ifdef DEBUG
static char *log_time(void)
{
    static char buf[20];
    struct timeval tv;
    int len;

    sys_get_time(&tv);
    len = strftime(buf, sizeof(buf), "%T", localtime(&tv.tv_sec));
    snprintf(buf+len, sizeof(buf)-len, ".%.3d", (int)tv.tv_usec/1000);
    return buf;
}
#endif

/* General state machine implementation */

/* Move to another state, if needed, and perform new state actions. Return 0
 * if no state change was made, 1 if state change was made, -1 on error */
typedef int (*state_machine_t)(void *o);

/* EAPoL state machine */

typedef enum {
    PTS_TICK = 1,
    PTS_ONE_SECOND = 2,
} auth1x_port_timers_state_t;

typedef enum {
    PS_INITIALIZE = 1,
    PS_DISCONNECTED = 2,
    PS_CONNECTING = 3,
    PS_AUTHENTICATING = 4,
    PS_AUTHENTICATED = 5,
    PS_ABORTING = 6,
    PS_HELD = 7,
    PS_FORCE_AUTH = 8,
    PS_FORCE_UNAUTH = 9,
} auth1x_pae_state_t;

typedef enum {
    RTS_INITIALIZE = 1,
    RTS_REAUTHENTICATE = 2,
} auth1x_reauthentication_timer_state_t;

typedef enum {
    BAS_REQUEST = 1,
    BAS_RESPONSE = 2,
    BAS_SUCCESS = 3,
    BAS_FAIL = 4,
    BAS_TIMEOUT = 5,
    BAS_IDLE = 6,
    BAS_INITIALIZE = 7,
} auth1x_backend_authentication_state_t;

typedef enum {
    CDS_FORCE_BOTH = 1,
    CDS_IN_OR_BOTH = 2,
} auth1x_controlled_directions_state_t;

/* context of all state machines */
typedef struct {
    /* timers from 802.1X-2001 section 8.5.2.1 */
    /* all timers are in seconds; they expire when reaching zero */
    int auth_while;
    int a_while;
    int held_while;
    int quiet_while;
    int re_auth_when;
    int start_when;
    int tx_when;

    /* global vars from 802.1X-2001 section 8.5.2.2 */
    int auth_abort;
    int auth_fail;
    int auth_start;
    int auth_timeout;
    int auth_success;
    int current_id;
    auth1x_auth_controlled_port_control_t port_mode;
    int is_port_authorized;
    int re_authenticate;
    int receive_id;
    int supp_status;

    /* Port Timer state machine vars from 802.1X-2001 section 8.5.3.1.1 */
    auth1x_port_timers_state_t pts_state;
    int pts_did_state_action;

    /* Authenticator PAE state machine vars from 802.1X-2001 section 8.5.4 */
    auth1x_pae_state_t pae_state;
    int pae_did_state_action;
    int re_auth_count;

    /* Reauthentication Timer state machine vars from 802.1X-2001 
     * section 8.5.7 */
    auth1x_reauthentication_timer_state_t rts_state;
    int rts_did_state_action;

    /* Backend Authentication state machine vars from 802.1X-2001 
     * section 8.5.8.1.1 */
    auth1x_backend_authentication_state_t bas_state;
    int bas_did_state_action;
    int req_count;

} auth1x_state_machines_t;

/* The following vars are set by sources external to state machines */
typedef struct {
    /* global vars from 802.1X-2001 section 8.5.2.2 */
    /* note: this variable has also global version */
    int initialize; /* set to non-zero to initialize all state machines */

    /* Port Timer state machine vars from 802.1X-2001 section 8.5.3.1.1 */
    int tick; /* set to non-zero once per second */

    /* Authenticator PAE state machine vars from 802.1X-2001 section 8.5.4 */
    int eap_logoff; /* set to non-zero if EAPOL-Logoff packet received */
    int eap_start; /* set to non-zero if EAPOL-Start packet received */
    int rx_resp_id; /* set to non-zero if EAPOL containing EAP
		     * Response/Identity received */

    /* Backend Authentication state machine vars from 802.1X-2001 
     * section 8.5.8.1.1 */
    int rx_resp; /* set to non-zero if EAPOL containing EAP Response other than 
                  * Identity received */;
    int a_success; /* set to non-zero if EAP method returned auth success */
    int a_fail; /* set to non-zero if EAP method returned auth failure */
    int a_req; /* set to non-zero if EAP method wishes to send EAP Request */
    int id_from_server; /* value of the Identifier field in the most recent
                         * EAP Success, EAP Failure or EAP Request from the
			 * EAP method */
} auth1x_ext_vars_t;

/* eapol connection context, one per client throughout session */
struct auth1x_conn_t {
    struct auth1x_conn_t *next;
    auth1x_port_t *auth1x_port;
    int closing; /* if != 0 - connection is being closed */
    auth1x_state_machines_t st;
    auth1x_ext_vars_t ext_vars;
    ether_mac_t mac; /* mac of authenticated/authenticating supplicant */
    eap_pkt_t *method_2_supp; /* eap packet received from eap method to be
			       * sent to the supplicant */
    eap_pkt_t *supp_2_method; /* eap packet received from the supplicant */
    eap_method_t *eap_method; /* EAP method for the client */
    void *eap_method_context;
    auth1x_conn_cb_t *cb; /* used by WPA */
    void *context; /* used for cb */
    int no_auth; /* if != 0 no authentication starts on this connection.
		  * used for opening connection for WPA. */
    int is_last_rx_eapol_encrypted;
    u8 *pmk; /* Master key provided by autentication method or NULL if 
	      * not used */
    int pmk_len; /* Master key length */
};

/* auth1x_port_t is the authentication context for a device */
struct auth1x_port_t {
    auth1x_port_cb_t *cb;
    void *context;
    ether_mac_t mac; /* mac of interface */
    int is_wireless_mode;
    int packet_blocking_enabled;
    int dev_mtu;
    auth1x_params_t params;
    auth1x_glob_ext_vars_t glob_ext_vars;
    auth1x_pc_ops_t *ops; /* port control functionns */
    auth1x_pc_t *auth1x_pc; /* descriptor for receiving/sending EAPOL
			     * packets and for controlling the interface */
    auth1x_conn_t *conn_list; /* all currently running connections */

    /* global vars from 802.1X-2001 section 8.5.2.2 */
    auth1x_auth_controlled_port_control_t port_mode;

    /* Authenticator PAE state machine vars from 802.1X-2001 section 8.5.4 */
    auth1x_pae_state_t pae_state;
    int pae_did_state_action;

    /* Controlled Directions state machine vars from 802.1X-2001 
     * section 8.5.9.1 */
    auth1x_controlled_directions_state_t cds_state;
    int cds_did_state_action;
    auth1x_admin_controlled_direction_t oper_controlled_directions;
};

/* connection state machine functions */
static int sm_port_timers(void *o);
static int sm_pae(void *o);
static int sm_reauthentication_timer(void *o);
static int sm_backend_authentication(void *o);

/* global state machine functions */
static int sm_controlled_directions(void *o);
static int sm_global_pae(void *o);

static state_machine_t eapol_conn_state_machines[] = {
    sm_port_timers,
    sm_pae,
    sm_reauthentication_timer,
    sm_backend_authentication,
    NULL
};

static state_machine_t eapol_glob_state_machines[] = {
    sm_controlled_directions,
    sm_global_pae,
    NULL
};

/* Creates an EAP packet with the specified code, type and data. 
 * Returns static EAP packet. */
static eap_pkt_t *eap_pkt_create(int eap_id, eap_code_t code, eap_type_t type,
    u8 *data, int data_len)
{
    static eap_pkt_t pkt;

    pkt.hdr.code = code;
    pkt.hdr.id = eap_id;
    pkt.hdr.len = htons(sizeof(eap_pkt_hdr_t) + data_len + (type!=EAP_NONE));
    pkt.type = type;
    pkt.data = data;

    return &pkt;
}

/* duplicate EAP packet */
static eap_pkt_t *eap_pkt_dup(eap_pkt_t *pkt)
{
    int len;
    eap_pkt_t *dup = malloc_l(sizeof(eap_pkt_t));

    if (!dup)
	return NULL;

    memcpy(dup, pkt, sizeof(eap_pkt_t));

    if (!pkt->data)
	return dup;

    len = ntohs(pkt->hdr.len) - sizeof(eap_pkt_hdr_t) - 1;
    dup->data = malloc_l(len);
    if (!dup->data)
    {
	free(dup);
	return NULL;
    }
    memcpy(dup->data, pkt->data, len);

    return dup;
}

/* free EAP packet */
static void eap_pkt_free(eap_pkt_t *pkt)
{
    if (!pkt)
	return;
    nfree(pkt->data);
    free(pkt);
}

static void auth1x_conn_clean(auth1x_conn_t *c)
{
    if (c->eap_method)
	c->eap_method->close(c->eap_method_context);
    c->eap_method = NULL;
    c->eap_method_context = NULL;
    eap_pkt_free(c->method_2_supp);
    c->method_2_supp = NULL;
    c->supp_2_method = NULL;
    nfree(c->pmk);
    c->pmk = NULL;
}

static void auth1x_conn_remove(auth1x_conn_t *conn)
{
    auth1x_conn_t **c;
    auth1x_port_t *auth1x_port = conn->auth1x_port;

    /* remove from the list */
    for (c = &auth1x_port->conn_list; *c && *c != conn; c = &(*c)->next);
    if (*c)
	*c = (*c)->next;

    auth1x_conn_clean(conn);
    free(conn);
}

/* use for calling a function from auth1x_port_t port control ops */
#define PC_OPS_CALL(dev,func,args...) \
    (dev)->ops->func(dev->auth1x_pc, args)

/* use when function might be NULL */
#define PC_OPS_CALL_SAFE(dev,func,args...) \
    ((dev)->ops->func ? PC_OPS_CALL(dev, func, args) : 0)

/* use for access auth1x_port_t port control ops using a connection */
#define CONN_PC_OPS_CALL(conn,func,args...) \
    PC_OPS_CALL_SAFE((conn)->auth1x_port, func, args)

static int auth1x_port_mode_set(auth1x_port_t *auth1x_port)
{
    auth1x_mode_t auth1x_mode = { 
	auth1x_port->oper_controlled_directions,
	auth1x_port->packet_blocking_enabled ? 
	    auth1x_port->glob_ext_vars.port_control :
	    ACPC_FORCE_AUTHORIZED,
	!auth1x_port->is_wireless_mode
    };
    return PC_OPS_CALL_SAFE(auth1x_port, mode_set, &auth1x_mode);
}

/* Send an EAPoL packet */
void auth1x_conn_eapol_pkt_send(auth1x_conn_t *c, eapol_pkt_t *pkt, int encrypt)
{
	int i;
    int pkt_len = sizeof(eapol_pkt_t) - 1 + ntohs(pkt->pkt_body_len);

#if 0 
	printf("\nUSER eapol len = %d",pkt_len );
	for(i = 0;i < pkt_len;i++)
	{
		
		printf("0x%x ",pkt->pkt_body[i]);
		if(!(i & 0x7))
			printf("\n");
	}

	fflush(NULL);
	
	usleep(100);
#endif
	
    if (pkt->pkt_type == ET_KEY && !c->auth1x_port->params.key_tx_enabled)
	return;

    if (CONN_PC_OPS_CALL(c, send_eapol, pkt, pkt_len, encrypt) != pkt_len)
    {
	rg_error(LERR, "%sCould not send EAPoL packet type %s: %m", 
	    AUTH1X_LOG_PREFIX, code2str(eapol_type2str, pkt->pkt_type));
    }
}

/* Contructs and send an EAPoL packet of EAP type, encapsulating the specified
 * EAP packet */
static void eapol_pkt_send_eap(auth1x_conn_t *c, eap_pkt_t *eap_pkt)
{
    eapol_pkt_t *pkt;
    int eap_data_len = ntohs(eap_pkt->hdr.len) - sizeof(eap_pkt->hdr);

    /* send only if we have supplicant mac */
    if (is_mac_zero(&c->mac))
	return;

    pkt = zalloc_l(sizeof(eapol_pkt_t)-1+sizeof(eap_pkt->hdr)+eap_data_len);
    if (!pkt)
    {
	rg_error(LERR, "%sCould not allocate EAPoL packet: %m", 
	    AUTH1X_LOG_PREFIX);
	return;
    }
    memcpy(&pkt->dest, c->mac, sizeof(pkt->dest));
    memcpy(&pkt->src, c->auth1x_port->mac, sizeof(pkt->src));
    pkt->ether_type = htons(EAPOL_TYPE);
    pkt->proto_ver = EAPOL_VER;
    pkt->pkt_type = ET_EAP;
    pkt->pkt_body_len = eap_pkt->hdr.len; /* both in network order */
    memcpy(pkt->pkt_body, &eap_pkt->hdr, sizeof(eap_pkt->hdr));
    if (eap_data_len > 0)
    {
	
	printf("USER:epa data len = %d\n",eap_data_len);
	pkt->pkt_body[sizeof(eap_pkt->hdr)] = eap_pkt->type;
	if (eap_data_len > 1)
	{
	    memcpy(pkt->pkt_body + sizeof(eap_pkt->hdr) + 1, eap_pkt->data,
		eap_data_len - 1);
	}
    }
    auth1x_conn_eapol_pkt_send(c, pkt, c->is_last_rx_eapol_encrypted);
    free(pkt);
}

static int method_find(auth1x_conn_t *conn)
{
    /* if this is EAP Response/Identity - find matching eap method */
    eap_method_id_t method_id;
    eap_method_t **method;
    void *params;
    u8 *identity = conn->supp_2_method->data;
    int identity_len = ntohs(conn->supp_2_method->hdr.len) -
	sizeof(eap_pkt_hdr_t) - 1; /* the -1 is needed since eap_pkt_hdr_t
				    * definition contains the first data byte
				    */

    /* search in database */
    method_id = conn->auth1x_port->cb->client_get(identity,
	identity_len, &params);

    /* set eap method */
    for (method = eap_methods; *method && (*method)->id != method_id;
	method++);

    /* if not found - discard packet */
    if (!*method)
    {
	rg_error(LWARN, "%sClient %.*s not found", AUTH1X_LOG_PREFIX,
	    identity_len, identity);
	return 1;
    }

    /* clean previous method, if any */
    if (conn->eap_method)
    {
	rg_error(LWARN, "%sClient %.*s closed method", AUTH1X_LOG_PREFIX,
	    identity_len, identity);
	conn->eap_method->close(conn->eap_method_context);
	conn->eap_method = NULL;
    }

    conn->eap_method_context = (*method)->open(params, conn);
    if (!conn->eap_method_context)
    {
	rg_error(LWARN, "%sCould not init method for %.*s",
	    AUTH1X_LOG_PREFIX, identity_len, identity);
	return 1;
    }

    conn->eap_method = *method;
    return 0;
}

/* authenticator PAE state machine functions from 802.1X-2001 section
 * 8.5.4.1.3 */

/* constructs EAPOL packet with EAP failure packet, with EAP identifier
 * eap_id, and sends it to the Supplicant */
static void tx_canned_fail(auth1x_conn_t *c, int eap_id)
{
    eapol_pkt_send_eap(c, 
	eap_pkt_create(eap_id, EAP_FAILURE, EAP_NONE, NULL, 0));
}

/* constructs EAPOL packet with EAP success packet, with EAP identifier
 * eap_id, and sends it to the Supplicant */
static void tx_canned_success(auth1x_conn_t *c, int eap_id)
{
    eapol_pkt_send_eap(c, 
	eap_pkt_create(eap_id, EAP_SUCCESS, EAP_NONE, NULL, 0));
}

/* constructs EAPOL packet with EAP Request/Identity packet, with EAP
 * identifier eap_id, and sends it to the Supplicant */
static void tx_req_id(auth1x_conn_t *c, int eap_id)
{
    eapol_pkt_send_eap(c, 
	eap_pkt_create(eap_id, EAP_REQUEST, EAP_IDENTITY, NULL, 0));
}

/* Backend Authentication state machine functions from 802.1X-2001 section
 * 8.5.8.1.3 */

/* constructs EAPOL packet with EAP Request packet, with EAP
 * identifier eap_id, and sends it to the Supplicant. the EAP content is the
 * last received eap packet, which was set in c->method_2_supp */
static void tx_req(auth1x_conn_t *c, int eap_id)
{
    c->method_2_supp->hdr.id = eap_id;
    eapol_pkt_send_eap(c, c->method_2_supp);
}

/* notify the eap method of the last received EAP/response or EAP/Identity
 * packet from the supplicant */
static void send_resp_to_server(auth1x_conn_t *c)
{
    /* notify eap method */
    if (c->eap_method)
	c->eap_method->supp_to_method(c->eap_method_context, c->supp_2_method);
}

static void abort_auth(auth1x_conn_t *c)
{
    /* release all used resources */
    auth1x_conn_clean(c);
}

/* act on the key provided from supplicant */
static void process_key(auth1x_conn_t *c, eapol_pkt_t *eapol_pkt, 
    int is_encrypted)
{
    if (c->ext_vars.initialize ||
	!c->auth1x_port->glob_ext_vars.port_enabled ||
	!c->cb || 
	!c->cb->eapol_key_rcv)
    {
	return;
    }

    c->is_last_rx_eapol_encrypted = is_encrypted;

    c->cb->eapol_key_rcv(c->context, eapol_pkt, is_encrypted);
}

/* General notes on state machines:
 * - Global transitions are checked first, since they have priority over
 *   normal stat transitions.
 * - Global transitions which depend on external variable (initialize,
 *   port_enabled) or on conditions which will not change in their state
 *   should return 0 otherwise state_machine_do_state() will never finish.
 */
static int sm_port_timers(void *o)
{
    SM_DEFS;

    /* check global transition first */
    if (conn->ext_vars.initialize)
    {
	SM_MOVE_STATE_QUIET(pts, PTS_ONE_SECOND);
	return 0;
    }

    switch (st->pts_state)
    {
    case PTS_TICK:
	SM_DEC(st->a_while);
	SM_DEC(st->auth_while);
	SM_DEC(st->held_while);
	SM_DEC(st->quiet_while);
	SM_DEC(st->re_auth_when);
	SM_DEC(st->start_when);
	SM_DEC(st->tx_when);
	conn->ext_vars.tick = 0;
	SM_MOVE_STATE_QUIET(pts, PTS_ONE_SECOND);
	break;
    case PTS_ONE_SECOND:
	if (conn->ext_vars.tick)
	    SM_MOVE_STATE_QUIET(pts, PTS_TICK);
    }
    return changed;
}

static void call_auth_end_cb(void *o)
{
    auth1x_conn_t *conn = (auth1x_conn_t *)o;
    
    /* conn->pmk may be NULL which means no PMK available */
    conn->cb->auth_end(conn->context, conn, conn->st.is_port_authorized, 
	conn->pmk, conn->pmk_len);
}

static void auth_end_notify(auth1x_conn_t *conn)
{
    if (conn->st.is_port_authorized)
    {
	rg_error(LDEBUG, "802.1x: Station %s authenticated", 
	    mactoa(&conn->mac));
    }
    else
    {
	rg_error(LDEBUG, "802.1x: Station %s failed to authenticate", 
	    mactoa(&conn->mac));
    }

    if (!conn->cb || !conn->cb->auth_end)
	return;
	
    event_timer_set(0, call_auth_end_cb, conn);
}

/* Handle connection logoff - in wireless mode, remove the connection,
 * otherwise move state machine to disconnected state */
#define PAE_HANDLE_LOGOFF do { \
    conn->closing = 1; \
    if (conn->auth1x_port->is_wireless_mode) \
        return CONN_PC_OPS_CALL(conn, mac_unauth, &conn->mac) ? -1 : 0; \
    SM_MOVE_STATE(pae, PS_DISCONNECTED); \
    } while(0)

static int sm_pae(void *o)
{
    SM_DEFS;

    /* check global transition first */
    if ((conn->auth1x_port->glob_ext_vars.port_control == ACPC_AUTO && 
	st->port_mode != conn->auth1x_port->glob_ext_vars.port_control)
	|| conn->ext_vars.initialize ||
	!conn->auth1x_port->glob_ext_vars.port_enabled ||
	conn->auth1x_port->glob_ext_vars.initialize)
    {
	SM_MOVE_STATE(pae, PS_INITIALIZE);
	if (conn->ext_vars.initialize ||
	    !conn->auth1x_port->glob_ext_vars.port_enabled ||
	    conn->auth1x_port->glob_ext_vars.initialize)
	{
	    return 0;
	}
    }
    else if (st->port_mode != 
	conn->auth1x_port->glob_ext_vars.port_control)
    {
	if (conn->auth1x_port->glob_ext_vars.port_control == 
	    ACPC_FORCE_UNAUTHORIZED)
	{
	    SM_MOVE_STATE(pae, PS_FORCE_UNAUTH);
	}
	else
	{
	    /* conn->auth1x_port->ext_vars.port_control ==
	     * ACPC_FORCE_AUTHORIZED */
	    SM_MOVE_STATE(pae, PS_FORCE_AUTH);
	}
    }

    switch (st->pae_state)
    {
    case PS_INITIALIZE:
	st->current_id = 0;
	/* actual port settings will be done globally for all interface
	 * connections */
	st->port_mode = ACPC_AUTO; 
	SM_MOVE_STATE(pae, PS_DISCONNECTED);
	break;
    case PS_DISCONNECTED:
	if (CONN_PC_OPS_CALL(conn, mac_unauth, &conn->mac))
	    return -1;
	st->is_port_authorized = 0;
	conn->ext_vars.eap_logoff = 0;
	st->re_auth_count = 0;
	/* XXX: SPEC diversion: In the spec (which was written for wired mode),
	 * tx_canned_fail() should be called. This is not done in wireless
	 * mode, since the client interprets this as authentication failure,
	 * and waits some time before next try.
	 */
	if (!conn->auth1x_port->is_wireless_mode)
	    tx_canned_fail(conn, st->current_id);
	SM_INC(st->current_id);
	SM_MOVE_STATE(pae, PS_CONNECTING);
	break;
    case PS_CONNECTING:
	SM_IF_NOT_DID_ACTION(pae)
	{
	    conn->ext_vars.eap_start = 0;
	    st->re_authenticate = 0;
	    st->tx_when = conn->auth1x_port->params.tx_period;
	    conn->ext_vars.rx_resp_id = 0;
	    tx_req_id(conn, st->current_id);
	    SM_INC(st->re_auth_count);
	}
	if (conn->ext_vars.eap_logoff || 
	    st->re_auth_count > conn->auth1x_port->params.re_auth_max)
	{
	    PAE_HANDLE_LOGOFF;
	}
	else if (!st->tx_when || conn->ext_vars.eap_start ||
	    st->re_authenticate)
	{
	    SM_MOVE_STATE(pae, PS_CONNECTING);
	}
	else if (conn->ext_vars.rx_resp_id)
	{
	    if (!method_find(conn))
		SM_MOVE_STATE(pae, PS_AUTHENTICATING);
	}
	break;
    case PS_AUTHENTICATING:
	SM_IF_NOT_DID_ACTION(pae)
	{
	    method_find(conn);
	    st->auth_success = 0;
	    st->auth_fail = 0;
	    st->auth_timeout = 0;
	    st->auth_start = 1;
	}
	if (st->auth_success)
	    SM_MOVE_STATE(pae, PS_AUTHENTICATED);
	else if (st->auth_fail)
	    SM_MOVE_STATE(pae, PS_HELD);
	else if (st->re_authenticate || conn->ext_vars.eap_start ||
	    conn->ext_vars.eap_logoff || st->auth_timeout)
	{
	    SM_MOVE_STATE(pae, PS_ABORTING);
	}
	break;
    case PS_AUTHENTICATED:
	SM_IF_NOT_DID_ACTION(pae)
	{
	    if (CONN_PC_OPS_CALL(conn, mac_auth, &conn->mac))
		return -1;
	    st->is_port_authorized = 1;

	    /* Notify authentication succeed */
	    auth_end_notify(conn);
	    st->re_auth_count = 0;
	    SM_INC(st->current_id);
	}
	if (st->re_authenticate || conn->ext_vars.eap_start)
	    SM_MOVE_STATE(pae, PS_CONNECTING);
	else if (conn->ext_vars.eap_logoff)
	    PAE_HANDLE_LOGOFF;
	break;
    case PS_ABORTING:
	SM_IF_NOT_DID_ACTION(pae)
	{
	    st->auth_abort = 1;
	    SM_INC(st->current_id);
	}
	if (!st->auth_abort)
	{
	    if (conn->ext_vars.eap_logoff)
		PAE_HANDLE_LOGOFF;
	    else
		SM_MOVE_STATE(pae, PS_CONNECTING);
	}
	break;
    case PS_HELD:
	SM_IF_NOT_DID_ACTION(pae)
	{
	    if (CONN_PC_OPS_CALL(conn, mac_unauth, &conn->mac))
		return -1;
	    st->is_port_authorized = 0;

	    /* Notify authentication failed */
	    auth_end_notify(conn);
	    st->quiet_while = conn->auth1x_port->params.quiet_period;
	    conn->ext_vars.eap_logoff = 0;
	    SM_INC(st->current_id);
	}
	if (!st->quiet_while)
	    SM_MOVE_STATE(pae, PS_CONNECTING);
	break;
    case PS_FORCE_AUTH:
	SM_IF_NOT_DID_ACTION(pae)
	{
	    /* no need to actually authorize client, since the global
	     * interface settings will do it for all clients */
	    st->is_port_authorized = 1;
	    /* actual port settings will be done globally for all interface
	     * connections */
	    st->port_mode = ACPC_FORCE_AUTHORIZED;
	    conn->ext_vars.eap_start = 0;
	    tx_canned_success(conn, st->current_id);
	    SM_INC(st->current_id);
	}
	if (conn->ext_vars.eap_start)
	    SM_MOVE_STATE(pae, PS_FORCE_AUTH);
	break;
    case PS_FORCE_UNAUTH:
	SM_IF_NOT_DID_ACTION(pae)
	{
	    /* no need to actually unauthorize client, since the global
	     * interface settings will do it for all clients */
	    st->is_port_authorized = 0;
	    /* actual port settings will be done globally for all interface
	     * connections */
	    st->port_mode = ACPC_FORCE_UNAUTHORIZED;
	    conn->ext_vars.eap_start = 0;
	    tx_canned_fail(conn, st->current_id);
	    SM_INC(st->current_id);
	}
	if (conn->ext_vars.eap_start)
	    SM_MOVE_STATE(pae, PS_FORCE_UNAUTH);
	break;
    }
    return changed;
}

static int sm_global_pae(void *o)
{
    SM_GLOB_DEFS;

    /* check global transition first */
    if (st->glob_ext_vars.initialize || !st->glob_ext_vars.port_enabled)
    {
	SM_MOVE_STATE(pae, PS_INITIALIZE);
	return 0;
    }
    else if (st->port_mode != st->glob_ext_vars.port_control)
    {
	switch (st->glob_ext_vars.port_control)
	{
	case ACPC_FORCE_UNAUTHORIZED:
	    SM_MOVE_STATE(pae, PS_FORCE_UNAUTH);
	    break;
	case ACPC_FORCE_AUTHORIZED:
	    SM_MOVE_STATE(pae, PS_FORCE_AUTH);
	    break;
	default:
	    break;
	}
    }

    switch (st->pae_state)
    {
    case PS_INITIALIZE:
	SM_IF_NOT_DID_ACTION(pae)
	{
	    st->port_mode = ACPC_AUTO;
	    if (auth1x_port_mode_set(st))
		return -1;
	}
	break;
    case PS_FORCE_AUTH:
	SM_IF_NOT_DID_ACTION(pae)
	{
	    st->port_mode = ACPC_FORCE_AUTHORIZED;
	    if (auth1x_port_mode_set(st))
		return -1;
	}
	break;
    case PS_FORCE_UNAUTH:
	SM_IF_NOT_DID_ACTION(pae)
	{
	    st->port_mode = ACPC_FORCE_UNAUTHORIZED;
	    if (auth1x_port_mode_set(st))
		return -1;
	}
	break;
    default: /* keep compiler quiet */
	break;
    }
    return changed;
}

static int sm_reauthentication_timer(void *o)
{
    SM_DEFS;

    /* check global transition first */
    if (conn->ext_vars.initialize ||
	conn->auth1x_port->glob_ext_vars.port_control != ACPC_AUTO ||
	!st->is_port_authorized || !conn->auth1x_port->params.re_auth_enabled)
    {
	SM_MOVE_STATE_QUIET(rts, RTS_INITIALIZE);
	return 0;
    }

    switch (st->rts_state)
    {
    case RTS_INITIALIZE:
	SM_IF_NOT_DID_ACTION(rts)
	    st->re_auth_when = conn->auth1x_port->params.re_auth_period;
	if (!st->re_auth_when)
	    SM_MOVE_STATE(rts, RTS_REAUTHENTICATE);
	break;
    case RTS_REAUTHENTICATE:
	SM_IF_NOT_DID_ACTION(rts)
	    st->re_authenticate = 1;
	SM_MOVE_STATE(rts, RTS_INITIALIZE);
	break;
    }
    return changed;
}

static int sm_backend_authentication(void *o)
{
    SM_DEFS;

    /* check global transition first */
    if (conn->ext_vars.initialize ||
	conn->auth1x_port->glob_ext_vars.port_control != ACPC_AUTO ||
	st->auth_abort)
    {
	SM_MOVE_STATE(bas, BAS_INITIALIZE);
	if (conn->ext_vars.initialize ||
	    conn->auth1x_port->glob_ext_vars.port_control != ACPC_AUTO)
	{
	    return 0;
	}
    }

    switch (st->bas_state)
    {
    case BAS_REQUEST:
	SM_IF_NOT_DID_ACTION(bas)
	{
	    st->current_id = conn->ext_vars.id_from_server;
	    tx_req(conn, st->current_id);
	    st->a_while = conn->auth1x_port->params.supp_timeout;
	    st->req_count++;
	}
	if (conn->ext_vars.rx_resp)
	    SM_MOVE_STATE(bas, BAS_RESPONSE);
	else if (!st->a_while)
	{
	    if (st->req_count >= conn->auth1x_port->params.max_req)
		SM_MOVE_STATE(bas, BAS_TIMEOUT);
	    else
		SM_MOVE_STATE(bas, BAS_REQUEST);
	}
	break;
    case BAS_RESPONSE:
	SM_IF_NOT_DID_ACTION(bas)
	{
	    conn->ext_vars.a_req = 0;
	    conn->ext_vars.a_success = 0;
	    st->auth_timeout = 0;
	    conn->ext_vars.rx_resp = 0;
	    conn->ext_vars.a_fail = 0;
	    st->a_while = conn->auth1x_port->params.server_timeout;
	    st->req_count = 0;
	    send_resp_to_server(conn);
	}
	if (conn->ext_vars.a_success)
	    SM_MOVE_STATE(bas, BAS_SUCCESS);
	else if (conn->ext_vars.a_fail)
	    SM_MOVE_STATE(bas, BAS_FAIL);
	else if (conn->ext_vars.a_req)
	    SM_MOVE_STATE(bas, BAS_REQUEST);
	else if (!st->a_while)
	    SM_MOVE_STATE(bas, BAS_TIMEOUT);
	break;
    case BAS_SUCCESS:
	SM_IF_NOT_DID_ACTION(bas)
	{
	    st->current_id = conn->ext_vars.id_from_server;
	    tx_canned_success(conn, st->current_id);
	    st->auth_success = 1;
	}
	SM_MOVE_STATE(bas, BAS_IDLE);
	break;
    case BAS_FAIL:
	SM_IF_NOT_DID_ACTION(bas)
	{
	    st->current_id = conn->ext_vars.id_from_server;
	    tx_canned_fail(conn, st->current_id);
	    st->auth_fail = 1;
	}
	SM_MOVE_STATE(bas, BAS_IDLE);
	break;
    case BAS_TIMEOUT:
	rg_error(LWARN, "802.1x: Backend authentication timeout");
	SM_IF_NOT_DID_ACTION(bas)
	{
	    if (!st->is_port_authorized)
		tx_canned_fail(conn, st->current_id);
	    st->auth_timeout = 1;
	}
	SM_MOVE_STATE(bas, BAS_IDLE);
	break;
    case BAS_IDLE:
	SM_IF_NOT_DID_ACTION(bas)
	{
	    st->auth_start = 0;
	    st->req_count = 0;
	}
	if (st->auth_start)
	    SM_MOVE_STATE(bas, BAS_RESPONSE);
	break;
    case BAS_INITIALIZE:
	SM_IF_NOT_DID_ACTION(bas)
	{
	    abort_auth(conn);
	    st->auth_abort = 0;
	}
	SM_MOVE_STATE(bas, BAS_IDLE);
	break;
    }
    return changed;
}

/* this state machine is global per interface */
static int sm_controlled_directions(void *o)
{
    SM_GLOB_DEFS;

    /* check global transition first */
    if (st->glob_ext_vars.initialize)
    {
	SM_MOVE_STATE(cds, CDS_IN_OR_BOTH);
	return 0;
    }

    switch (st->cds_state)
    {
    case CDS_FORCE_BOTH:
	SM_IF_NOT_DID_ACTION(cds)
	{
	    st->oper_controlled_directions = ACD_BOTH;
	    if (auth1x_port_mode_set(st))
		return -1;
	}
	if (st->glob_ext_vars.port_enabled &&
	    !st->glob_ext_vars.bridge_detected)
	{
	    SM_MOVE_STATE(cds, CDS_IN_OR_BOTH);
	}
	break;
    case CDS_IN_OR_BOTH:
	SM_IF_NOT_DID_ACTION(cds)
	{
	    st->oper_controlled_directions = 
		st->glob_ext_vars.admin_controlled_directions;
	    if (auth1x_port_mode_set(st))
		return -1;
	}
	if (st->oper_controlled_directions != 
		st->glob_ext_vars.admin_controlled_directions)
	{
	    SM_MOVE_STATE(cds, CDS_IN_OR_BOTH);
	}
	else if (!st->glob_ext_vars.port_enabled ||
	    st->glob_ext_vars.bridge_detected)
	{
	    SM_MOVE_STATE(cds, CDS_FORCE_BOTH);
	}
	break;
    }
    return changed;
}

/* Move all state machines to their next state. state_machines is NULL
 * terminated. Return 0 on success, 1 otherwise */
int state_machine_do_state(state_machine_t *state_machines, void *o)
{
    state_machine_t *sm;
    int changed;

    /* call do_state repeatedly on all state machines until none of them
     * has changed */
    do {
	changed = 0;
	for (sm = state_machines; *sm; sm++)
	{
	    int ret = (*sm)(o);

	    if (ret < 0)
		return 1;
	    changed |= ret;
	}
    } while (changed);

    return 0;
}

/* find connection in auth1x_port conn_list. */
static auth1x_conn_t *auth1x_conn_find(auth1x_port_t *auth1x_port,
    ether_mac_t *supplicant_addr)
{
    auth1x_conn_t *conn = auth1x_port->conn_list;

    for (; conn && memcmp(&conn->mac, supplicant_addr, sizeof(conn->mac));
	conn=conn->next);

    return conn;
}

static int auth1x_conn_get_num(auth1x_port_t *auth1x_port)
{
    auth1x_conn_t *conn;
    int count = 0;

    for (conn = auth1x_port->conn_list; conn; conn = conn->next, count++);

    return count;
}

eap_method_id_t eap_client_get(void *context, u8 *id, int id_len,
    void **eap_method_params)
{
    auth1x_conn_t *conn = context;

    return conn->auth1x_port->cb->client_get(id, id_len,
	eap_method_params);
}

static void conn_state_machine_do_state(auth1x_conn_t *conn, char *errmsg)
{
    if (conn->no_auth)
	return;

    STATE_MACHINE_DO_STATE_REPORT(eapol_conn_state_machines, conn, errmsg);
}

/* Main receive packet handler; called whenever EAPOL packet is received in
 * the attached interface or directly from test program. */
static void auth1x_pkt_handler(void *ctx, void *buf, int ret, int is_encrypted)
{
    auth1x_port_t *auth1x_port = (auth1x_port_t *)ctx;
    eapol_pkt_t *pkt = buf;
    auth1x_conn_t *conn;
    int is_eap = 0;
    eap_pkt_t eap_pkt;
    int pkt_body_len;

    if (ret < 0)
    {
	rg_error(LWARN, "%sError in reading packet: %m", AUTH1X_LOG_PREFIX);
	return;
    }

    if (ret < sizeof(eapol_pkt_t)-1)
    {
	rg_error(LWARN, "%sPacket too short (%d)", AUTH1X_LOG_PREFIX, ret);
	return;
    }

    rg_error(LDEBUG, "802.1x: Received an EAPOL packet from %s", 
	mactoa(&pkt->src));
    
    conn = auth1x_conn_find(auth1x_port, &pkt->src);

    /* If this is a non-EAPoL packet - convert it to EAPoL-Start
     * message. This should happen whenever the unauthorized client sends a
     * non-EAPoL packet and the pass_single_unauth flag is set. The flag is set
     * every PASS_SINGLE_UNAUTH_PERIOD. */
    if (pkt->ether_type != htons(EAPOL_TYPE))
    {
	/* if we have connection - authentication is now taking place. Don't
	 * restart it - just ignore packet.
	 */
	if (conn)
	    return;

	/* fake EAPoL-Start */
	pkt->ether_type = htons(EAPOL_TYPE);
	pkt->proto_ver = EAPOL_VER;
	pkt->pkt_type = ET_START;
	pkt->pkt_body_len = 0;
	ret = sizeof(eapol_pkt_t)-1;
    }

    if (pkt->pkt_type == ET_START && auth1x_port->cb->eapol_start_received)
	auth1x_port->cb->eapol_start_received(auth1x_port->context, &pkt->src);

    if (!conn && pkt->pkt_type == ET_START)
    {
	/* Test to see if there is an association for this mac */
	if (auth1x_port->cb->new_assoc)
	    auth1x_port->cb->new_assoc(auth1x_port->context, &pkt->src);

	conn = auth1x_conn_find(auth1x_port, &pkt->src);
	if (!conn)
	    return;
    }
    else
    {
	if (!conn)
	    return;

	/* verify correct packet length according to len field. Packet may not
	 * be fully available, if the read length is too small */
	if (ret < sizeof(eapol_pkt_t)-1 || 
	    ret < sizeof(eapol_pkt_t)-1+ntohs(pkt->pkt_body_len))
	{
	    rg_error(LWARN, "%sIncorrect EAPoL packet length (%d)",
		AUTH1X_LOG_PREFIX, ret);
	    return;
	}
	pkt_body_len = ntohs(pkt->pkt_body_len);

	if (pkt->proto_ver != EAPOL_VER)
	{
	    rg_error(LWARN, "%sIncorrect EAPoL packet version (%d)",
		AUTH1X_LOG_PREFIX, pkt->proto_ver);
	    return;
	}

	/* if this is EAP encapsulated - verify correct length according to EAP
	 * len */
	if (pkt->pkt_type == ET_EAP)
	{
	    int eap_data_len = pkt_body_len - sizeof(eap_pkt_hdr_t);

	    if (eap_data_len < 0 ||
		pkt_body_len < ntohs(((eap_pkt_hdr_t *)pkt->pkt_body)->len))
	    {
		rg_error(LWARN, "%sIncorrect EAPoL/EAP packet length (%d)",
		    AUTH1X_LOG_PREFIX, pkt_body_len);
		return;
	    }
	    is_eap = 1;
	    memcpy(&eap_pkt.hdr, pkt->pkt_body, sizeof(eap_pkt.hdr));
	    if (eap_data_len)
		eap_pkt.type = pkt->pkt_body[sizeof(eap_pkt.hdr)];
	    eap_pkt.data = eap_data_len > 1 ?
		pkt->pkt_body + sizeof(eap_pkt.hdr) + 1 : NULL;
	}

	/* if found and packet is a EAPOL-EAP and EAP Identifier differs - 
	 * ignore packet */
	if (is_eap && eap_pkt.hdr.id != conn->st.current_id)
	{
	    rg_error(LWARN, "%sIncorrect EAPoL/EAP packet id (%d)",
		AUTH1X_LOG_PREFIX, ret);
	    return;
	}

	/* save input packet */
	if (is_eap)
	    conn->supp_2_method = &eap_pkt;

	/* update the eapol connection */
	conn->ext_vars.eap_logoff = pkt->pkt_type == ET_LOGOFF && !pkt_body_len;
	conn->ext_vars.eap_start = pkt->pkt_type == ET_START && !pkt_body_len;
	conn->ext_vars.rx_resp_id = is_eap && eap_pkt.hdr.code == EAP_RESPONSE
	    && ntohs(eap_pkt.hdr.len) > sizeof(eap_pkt.hdr) &&
	    eap_pkt.type == EAP_IDENTITY;
	conn->ext_vars.rx_resp = is_eap && eap_pkt.hdr.code == EAP_RESPONSE &&
	    !conn->ext_vars.rx_resp_id;

	/* 802.1X doesn't handle EAPOL-Key packets, it just passes it on to
	 * whoever registered a callback for this purpose */
	if (pkt->pkt_type == ET_KEY)
	{
	    process_key(conn, pkt, is_encrypted);
	    return;
	}
    }

    /* notify the eapol connection */
    conn_state_machine_do_state(conn,
	"Error updating connection state machines");
    
    conn->ext_vars.eap_logoff = 0;
    conn->ext_vars.eap_start = 0;
    conn->ext_vars.rx_resp_id = 0;
    conn->ext_vars.rx_resp = 0;

    if (conn->closing)
    {
	/* use callback to notify the port that this connection should be 
	 * closed */
	if (conn->cb->conn_term)
	    conn->cb->conn_term(conn->context, conn);
    }
}

/* 1 second timer callback - update state machine */
static void one_second_cb(void *o)
{
    auth1x_port_t *auth1x_port = (auth1x_port_t *)o;
    auth1x_conn_t *conn, *next;

    /* update global state machines */
    GLOBAL_STATE_MACHINE_DO_STATE(auth1x_port,
	"Error updating 802.1x tick in global state machines");

    /* update all 802.1x connections with new tick */
    for (conn=auth1x_port->conn_list; conn; conn=next)
    {
	next = conn->next;
	conn->ext_vars.tick = 1;
	conn_state_machine_do_state(conn,
	    "Error updating 802.1x tick in connection state machines");

	if (conn->closing)
	{
	    /* use callback to notify the port that this connection should be 
	     * closed */
	    if (conn->cb->conn_term)
		conn->cb->conn_term(conn->context, conn);
	}
    }
}

/* Periodically allow a single unauthorized packet to reach main callback. This
 * allows authentication start from OpenRG, without waiting for EAPoL-Start
 * message. */
static void set_pass_single_unauth(void *o)
{
    auth1x_port_t *auth1x_port = (auth1x_port_t *)o;

    if (PC_OPS_CALL_SAFE(auth1x_port, pass_single_unauth_set, 1))
    {
	rg_error(LERR, "%sError setting pass_single_unauth", 
		AUTH1X_LOG_PREFIX);
    }
}

/* open a new connection without stating authentication */
auth1x_conn_t *auth1x_conn_open(auth1x_port_t *auth1x_port,
    ether_mac_t *supplicant_addr, auth1x_conn_cb_t *cb, void *context)
{
    auth1x_conn_t *conn;

    if (auth1x_conn_get_num(auth1x_port) >= MAX_DEV_AUTH_CONNECTIONS)
    {
	rg_error(LERR, "%sMax connection limit reached", AUTH1X_LOG_PREFIX);
	return NULL;
    }

    if (!(conn = zalloc_l(sizeof(*conn))))
	return NULL;

    conn->next = auth1x_port->conn_list;
    auth1x_port->conn_list = conn;
    conn->auth1x_port = auth1x_port;

    /* save source mac */
    memcpy(&conn->mac, supplicant_addr, sizeof(conn->mac));

    conn->cb = cb;
    conn->context = context;
    conn->no_auth = 1;
    conn->ext_vars.eap_start = 1;

    /* initialize the key state machines */
    conn->ext_vars.initialize = 1;
    conn_state_machine_do_state(conn,
	"Error in state initialization of connection state machine");
    conn->ext_vars.initialize = 0;

    return conn;
}

/* Terminates the authentication session. auth1x_conn is the value returned
 * from auth1x_conn_open() */
void auth1x_conn_close(auth1x_conn_t *auth1x_conn)
{
    /* Notify state machines connection is loging off */
    auth1x_conn->ext_vars.eap_logoff = 1;
    conn_state_machine_do_state(auth1x_conn,
	"Error notifing state machines that connection is logging off");

    if (!auth1x_conn->closing)
    {
	rg_error(LDEBUG, "%sWarning: Unclean connection removal",
	    AUTH1X_LOG_PREFIX);
    }
    
    event_timer_del(call_auth_end_cb, auth1x_conn);
    
    /* Remove from device's list */
    auth1x_conn_remove(auth1x_conn);
}

/* auth1x_port_start() is called on each device that should require
 * 802.1x authentication. returns context on success, NULL otherwise.
 */
auth1x_port_t *auth1x_port_open(auth1x_port_params_t *params,
    auth1x_port_cb_t *cb, auth1x_pc_ops_t *port_ctl_ops, void *context)
{
    auth1x_mode_t mode_promisc = { ACD_BOTH, ACPC_FORCE_UNAUTHORIZED, 1 };
    
    /* allocate device authentication context */
    auth1x_port_t *auth1x_port = zalloc_l(sizeof(auth1x_port_t));
    if (!auth1x_port)
	return NULL;

    /* set callback */
    auth1x_port->cb = cb;
    auth1x_port->context = context;

    /* set port control ops */
    auth1x_port->ops = port_ctl_ops;

    /* open port control on the device (and switch mode to auth) */
    auth1x_port->auth1x_pc =
	auth1x_port->ops->open(params->dev_id, auth1x_pkt_handler, auth1x_port);

    if (!auth1x_port->auth1x_pc)
    {
		rg_error(LERR, "%sCould not open Port Control", AUTH1X_LOG_PREFIX);
		goto Error;
    }

    /* on wired mode - switch to promiscuous mode */
    if (!params->is_wireless && 
	PC_OPS_CALL_SAFE(auth1x_port, mode_set, &mode_promisc))
    {
	rg_error(LERR, "%sCould not set Promiscuous mode", AUTH1X_LOG_PREFIX);
	goto Error;
    }

    /* set params and init state maachin */
    auth1x_port->is_wireless_mode = params->is_wireless;
    auth1x_port->packet_blocking_enabled = params->packet_blocking_enabled;
    auth1x_port_reconf(auth1x_port, params);

    /* register a 1 second timer callback */
    event_timer_set_period(1*MS_SECOND, one_second_cb, auth1x_port);

    if (!auth1x_port->is_wireless_mode)
    {
	event_timer_set_period(PASS_SINGLE_UNAUTH_PERIOD,
	    set_pass_single_unauth, auth1x_port);
	set_pass_single_unauth(auth1x_port);
    }

    return auth1x_port;

Error:
    auth1x_port_close(auth1x_port);
    return NULL;
}

/* Called on each device that should require no 802.1x authentication */
void auth1x_port_close(auth1x_port_t *auth1x_port)
{
    auth1x_conn_t *conn;

    /* unregister a 1 second and pass_single_unauth timer callbacks */
    event_timer_del(one_second_cb, auth1x_port);
    event_timer_del(set_pass_single_unauth, auth1x_port);

    /* release all 802.1x connections */
    while ((conn = auth1x_port->conn_list))
    {
	auth1x_port->conn_list = conn->next;
	auth1x_conn_clean(conn);
	free(conn);
    }

    /* close port control descriptor, thereby unregistering eapol packet
     * callback, switching port to force authorized and setting
     * non-promiscuous mode (if set) */
    if (auth1x_port->auth1x_pc)
	auth1x_port->ops->close(auth1x_port->auth1x_pc);

    /* free device authentication context */
    free(auth1x_port);
}

void auth1x_port_reconf(auth1x_port_t *auth1x_port,
    auth1x_port_params_t *params)
{
    memcpy(&auth1x_port->mac, &params->mac,
	sizeof(auth1x_port->mac));
    auth1x_port->dev_mtu = params->mtu;
    auth1x_port->params = params->params;
    auth1x_port->glob_ext_vars = params->glob_ext_vars;

    /* init state machines */
    one_second_cb(auth1x_port);
    auth1x_port->glob_ext_vars.initialize = 0;
    one_second_cb(auth1x_port);
}

/* Function called by authentication methods to send EAPoL packets to
 * Supplicant. */
void eap_method_to_supp(void *context, eap_pkt_t *eap_pkt)
{
    auth1x_conn_t *conn = context;
    
    eap_pkt_free(conn->method_2_supp);
    conn->method_2_supp = eap_pkt_dup(eap_pkt);
    if (!conn->method_2_supp)
    {
	rg_error(LERR, "%sCould not copy EAP packet", AUTH1X_LOG_PREFIX);
	return;
    }
    conn->ext_vars.a_success = eap_pkt->hdr.code == EAP_SUCCESS;
    conn->ext_vars.a_fail = eap_pkt->hdr.code == EAP_FAILURE;
    conn->ext_vars.a_req = eap_pkt->hdr.code == EAP_REQUEST;
    conn->ext_vars.id_from_server = eap_pkt->hdr.id;
}

/* Update state machines */
void eap_method_sm_update(void *context)
{
    auth1x_conn_t *conn = context;
    
    conn_state_machine_do_state(conn, 
	"Error updating 802.1x connection state machines");
}

/* Function that will be called by the eap method to get current max EAP packet
 * length, as derrived from device MTU. It is used to decide if fragmentation
 * is needed. */
int eap_max_len_get(void *context)
{
    auth1x_conn_t *conn = context;

    return conn->auth1x_port->dev_mtu - sizeof(eapol_pkt_t) - 1;
}

/* Copy master key and save it on the connection */
void eap_method_master_key(void *context, u8 *key, int key_len)
{
    auth1x_conn_t *conn = context;

    /* Remove old key */
    nfree(conn->pmk);
    conn->pmk = NULL;
    conn->pmk_len = 0;
    
    /* Allocate new buffer. */
    /* XXX: use bufdup_l when available */
    if (!key || !key_len || !(conn->pmk = malloc_l(key_len)))
	return;
    
    memcpy(conn->pmk, key, key_len);
    conn->pmk_len = key_len;
}

/* start authentication on an existing connection */
void auth1x_conn_auth_start(auth1x_conn_t *conn)
{
    conn->no_auth = 0;
    /* initialize the state machine */
    conn->ext_vars.initialize = 1;
    conn_state_machine_do_state(conn,
	"Error in state initialization of connection state machine");
    conn->ext_vars.initialize = 0;
}

/* Get next auth1x_conn_t in conn_list, NULL means get first */
auth1x_conn_t *auth1x_conn_get_next(auth1x_port_t *port, 
    auth1x_conn_t *auth1x_conn)
{
    if (auth1x_conn)
	return auth1x_conn->next;
    
    return port->conn_list;
}

/* Get pmk of auth1x_conn_t */
void auth1x_conn_pmk_get(auth1x_conn_t *auth1x_conn, u8 **pmk, int *pmk_len)
{
    *pmk = auth1x_conn->pmk;
    *pmk_len = auth1x_conn->pmk_len;
}

/* Get mac of auth1x_conn_t supplicant */
ether_mac_t *auth1x_conn_mac_get(auth1x_conn_t *auth1x_conn)
{
    return &auth1x_conn->mac;
}

/* Get mac of auth1x_port_t authenticator */
ether_mac_t *auth1x_port_mac_get(auth1x_port_t *auth1x_port)
{
    return &auth1x_port->mac;
}
