/* LICENSE_CODE JUNGO */

#include "rg_error.h"
#include <util/memory.h>
#include <os_libc_extra.h>
#include "eap.h"
#include "eap_radius.h"

/* Forwards */
void eap_radius_packet_recv(void *context, radius_pkt_t *pkt);

/* Open a RADIUS eap session */
void *eap_radius_open(void *params, void *context)
{
    eap_method_radius_params_t *p = (eap_method_radius_params_t *) params;
    eap_radius_t *eap_radius;
    radius_client_t *radius_client = p->radius_client;
    
    if (!radius_client)
	return NULL;

    if (!(eap_radius = (eap_radius_t *) zalloc_l(sizeof(eap_radius_t))))
	return NULL;

    eap_radius->context = context;
    eap_radius->identity_len = p->identity_len;
    eap_radius->identity = zalloc_l(eap_radius->identity_len);
    if (!eap_radius->identity)
	goto Error;
    memcpy(eap_radius->identity, p->identity, eap_radius->identity_len);
    
    /* Create RADIUS eap session */
    eap_radius->session = radius_client_session_open(radius_client, 
	eap_radius_packet_recv, eap_radius);
    
    if (!eap_radius->session)
	goto Error;

    return eap_radius;
Error:
    nfree(eap_radius->identity);
    free(eap_radius);
    return NULL;
}

/* Close a RADIUS eap session */
void eap_radius_close(void *eap_context)
{
    eap_radius_t *eap_radius = eap_context;

    radius_client_session_close(eap_radius->session);
    nfree(eap_radius->identity);
    free(eap_radius);
}

/* Create a RADIUS Packet encapsulating EAP packet */
radius_pkt_t *eap_radius_packet_create(eap_radius_t *eap_radius, 
    eap_pkt_t *eap_pkt)
{
    radius_pkt_t *radius_pkt;
    u8 *buf = NULL;
    u8 *buf_ptr;
    u16 buf_len = ntohs(eap_pkt->hdr.len);
    u32 framed_mtu;
    u32 nas_port_type;
    
    eap_radius->recv_id = eap_pkt->hdr.id; 
    /* Build RADIUS packet */
    radius_pkt = radius_packet_create();
    if (!radius_pkt)
	goto Error;
    
    /* Add User-Name attribute */
    if (radius_packet_attr_add(radius_pkt, PW_USER_NAME, eap_radius->identity, 
	eap_radius->identity_len))
    {
	goto Error;
    }

    /* Add Framed MTU attribute */
    framed_mtu = htonl(eap_max_len_get(eap_radius->context));
    
    if (radius_packet_attr_add(radius_pkt, PW_FRAMED_MTU, (u8 *)&framed_mtu,
	sizeof(framed_mtu)))
    {
	goto Error;
    }

    /* Add NAS-Port-Type attribute */
    nas_port_type = htonl(19);
    
    if (radius_packet_attr_add(radius_pkt, PW_NAS_PORT_TYPE, 
	(u8 *)&nas_port_type, sizeof(nas_port_type)))
    {
	goto Error;
    }

    /* Add EAP message attributes by splitting it into 253 byte chunks */
    buf = (u8 *) zalloc_l(buf_len);
    if (!buf)
	goto Error;
    memcpy(buf, (u8 *) &eap_pkt->hdr, sizeof(eap_pkt_hdr_t));
    buf[sizeof(eap_pkt_hdr_t)] = eap_pkt->type;
    if (eap_pkt->data && buf_len > sizeof(eap_pkt_hdr_t)+1)
    {
	memcpy(buf+sizeof(eap_pkt_hdr_t)+1, eap_pkt->data, 
	    buf_len-sizeof(eap_pkt_hdr_t)-1);
    }
    buf_ptr = buf;
    while (buf_len > 0)
    {
	u8 len = MIN(AUTH_STRING_LEN, buf_len);

	if (radius_packet_attr_add(radius_pkt, PW_EAP_MESSAGE, buf_ptr, len))
	    goto Error;
	buf_ptr += len;
	buf_len -= len;
    }
    
    nfree(buf);
    return radius_pkt;

Error:
    nfree(buf);
    radius_packet_free(radius_pkt);
    return NULL;
}

void eap_radius_supp_to_method(void *context, eap_pkt_t *resp)
{
    radius_pkt_t *radius_pkt;
    eap_radius_t *eap_radius = context;
    
    /* Create a RADIUS Access-Request Packet encapsulating the EAP packet */
    radius_pkt = eap_radius_packet_create(eap_radius, resp);
    if (!radius_pkt)
	return;

    /* Send the RADIUS packet via RADIUS task */
    radius_client_packet_send(eap_radius->session, radius_pkt);

    /* Free RADIUS packet */
    radius_packet_free(radius_pkt);
}

/* Extract and build an EAP packet from a RADIUS packet. 0 -Success */
static int eap_radius_packet_eap_extract(radius_pkt_t *pkt, eap_pkt_t *eap_pkt)
{
    radius_pkt_attr_t *attr;
    u8 *data_curr;
    int curr_len;
    int slice_len;

    attr = radius_packet_attr_get(pkt, PW_EAP_MESSAGE, NULL);
    if (!attr || attr->len-2 < (sizeof(eap_pkt_hdr_t)))
	return -1;

    eap_pkt->hdr = *(eap_pkt_hdr_t *) attr->value;
    if (ntohs(eap_pkt->hdr.len) == sizeof(eap_pkt_hdr_t))
	return 0;
    eap_pkt->type = *(u8 *)(attr->value + sizeof(eap_pkt_hdr_t));
    curr_len = ntohs(eap_pkt->hdr.len)-sizeof(eap_pkt_hdr_t)-1;
    if (curr_len  <= 0)
	return curr_len ? -1 : 0;

    eap_pkt->data = (u8 *)zalloc_l(curr_len);
    if (!eap_pkt->data)
	return -1;

    data_curr = eap_pkt->data;
    slice_len = attr->len-2-sizeof(eap_pkt_hdr_t)-1;
    if (slice_len > curr_len)
	goto Error;
    memcpy(data_curr, attr->value+sizeof(eap_pkt_hdr_t)+1, slice_len);
    curr_len -= slice_len;
    data_curr += slice_len;

    while ((attr=radius_packet_attr_get(pkt, PW_EAP_MESSAGE, attr)))
    {
	slice_len = attr->len - 2;
	if (slice_len > curr_len)
	    goto Error;
	memcpy(data_curr, attr->value, slice_len);
	curr_len -= slice_len;
	data_curr += slice_len;
    }

    if (curr_len)
	goto Error;

    return 0;
Error:
    nfree(eap_pkt->data);
    eap_pkt->data = NULL;
    return -1;
}

/* Get master key, if exist */
static void eap_radius_packet_master_key_extract(eap_radius_t *eap_radius, 
    radius_pkt_t *pkt)
{
    u8 *master_key = NULL;
    int master_key_len;

    if (radius_packet_ms_mppe_key_get(eap_radius->session, pkt, 
	&master_key, &master_key_len))
    {
	goto Exit;
    }
    
    eap_method_master_key(eap_radius->context, master_key, master_key_len);
    
Exit:
    nfree(master_key);
}

/* Receive radius eap packet from server callback */
void eap_radius_packet_recv(void *context, radius_pkt_t *pkt)
{
    eap_radius_t *eap_radius = context;
    eap_pkt_t eap_pkt;
    
    MZERO(eap_pkt);

    rg_error(LDEBUG, "802.1x: Received a RADIUS packet");

    switch (radius_packet_code_get(pkt)) {
    case ACCESS_ACCEPT:
	eap_radius_packet_master_key_extract(eap_radius, pkt);
	break;
    case ACCESS_REJECT:
    case ACCESS_CHALLENGE:
	break;
    default:
	goto Error;
    }
    
    if (!eap_radius_packet_eap_extract(pkt, &eap_pkt) &&
	(eap_pkt.hdr.code != EAP_SUCCESS || 
	radius_packet_code_get(pkt) == ACCESS_ACCEPT))
    {
	goto Exit;
    }
	
Error:
    nfree(eap_pkt.data);
    MZERO(eap_pkt);
    eap_pkt.hdr.id = (eap_radius->recv_id + 1) & 0xff;
    eap_pkt.hdr.code = EAP_FAILURE;
Exit:
    eap_method_to_supp(eap_radius->context, &eap_pkt);
    eap_method_sm_update(eap_radius->context);
    nfree(eap_pkt.data);
}

eap_method_t eap_radius_methods = {
    EMI_RADIUS,
    eap_radius_open,
    eap_radius_close,
    eap_radius_supp_to_method,
};

