/* LICENSE_CODE JUNGO */

#define OS_INCLUDE_IOCTL
#include <os_includes.h>
#include <misc_funcs.h>
#include <esyscall.h>
#include <rg_def.h>
#include <net_utils.h>
#include <kos/knet.h>
#include <kos/kos_chardev.h>
#include <syscalls.h>

#ifdef CONFIG_RG_TARGET_VXWORKS
#include <etherLib.h>
#endif

#include "auth1x_pktfil.h"
#include "port_ctl.h"

struct auth1x_pc_t {
    int fd;
    char devname[IFNAMSIZ+1];
    auth1x_pkt_handler_t pkt_handler;
    void *context;
};

/* hooks for wireless driver controls */
int (*auth1x_pc_mode_set_hook)(char *ifname, auth1x_mode_t *mode);
int (*auth1x_pc_mac_auth_hook)(char *ifname, ether_mac_t *mac);
int (*auth1x_pc_mac_unauth_hook)(char *ifname, ether_mac_t *mac);

static void auth1x_rcv_cb(int fd, void *data)
{
    u8 buf[MAX_ETH_RAW_FRAME_SIZE];
    int res;
    auth1x_pc_t *pc = data;

    /* read packet */
    res = read(fd, buf, sizeof(buf));
    if (res < 0)
    {
	rg_error(LERR, "Error reading packet: %m\n");
	return;
    }

    pc->pkt_handler(pc->context, buf, res, 0);
}

/* Open a file to control the named device. Returns file descriptor if
 * successful, -1 otherwise. */
int auth1x_pc_init(char *name)
{
    int fd;
    
    if ((fd = sys_rg_chrdev_open(KOS_CDT_AUTH1X, O_RDWR)) < 0)
	return rg_error(LERR, "Can't open auth1x_pktfil");
    
    if (ioctl(fd, SIOCSAUTH1XFILDEV, name) < 0)
    {
	close(fd);
	return rg_error(LERR, "ioctl SIOCSAUTH1XFILDEV failed");
    }
    return fd;
}

/* Close device control file */
void auth1x_pc_uninit(int fd)
{
    if (fd >= 0)
    {
	event_fd_clr(fd, EFD_READ);
	close(fd);
    }
}

static void auth1x_pc_close(auth1x_pc_t *t)
{
    auth1x_pc_uninit(t->fd);
    nfree(t);
}

static auth1x_pc_t *auth1x_pc_open(void *dev_id,
    auth1x_pkt_handler_t auth1x_pkt_handler, void *context)
{
    auth1x_pc_t *t = zalloc_l(sizeof(auth1x_pc_t));

    if (!t)
	return NULL;
    
    strncpyz(t->devname, (char *)dev_id, sizeof(t->devname));
    t->pkt_handler = auth1x_pkt_handler;
    t->context = context;

    t->fd = auth1x_pc_init(t->devname);
    if (t->fd < 0)
	goto Error;
    
     /* register eapol packet callback */
    if (auth1x_pkt_handler)
	event_fd_set(t->fd, EFD_READ, auth1x_rcv_cb, t);
    return t;
Error:
    auth1x_pc_close(t);
    return NULL;
}

/* send the specified EAPoL packet through the device. Packet should contain
 * ethernet header. Returns number of bytes sent, -1 on error. */
static int auth1x_pc_send_eapol(auth1x_pc_t *t, void *buf, int len, int encrypt)
{
    return write(t->fd, buf, len);
}

/* Sets mode for the device. Returns 0 on success, 1 otherwise */
static int auth1x_pc_mode_set(auth1x_pc_t *t, auth1x_mode_t *mode)
{
    if (ioctl(t->fd, SIOCSAUTH1XSETMODE, mode) < 0)
    {
	rg_error(LERR, "ioctl SIOCSAUTH1XSETMODE failed");
	return 1;
    }
    if (auth1x_pc_mode_set_hook && 
	auth1x_pc_mode_set_hook(t->devname, mode))
    {
	rg_error(LERR, "auth1x_pc_mode_set_hook() failed");
	return 1;
    }
    return 0;
}

/* Gets mode for the device. Returns 0 on success, 1 otherwise */
static int auth1x_pc_mode_get(auth1x_pc_t *t, auth1x_mode_t *mode)
{
    if (ioctl(t->fd, SIOCGAUTH1XGETMODE, mode) < 0)
    {
	rg_error(LERR, "ioctl SIOCGAUTH1XGETMODE failed");
	return 1;
    }
    return 0;
}

/* Authorize mac on the device. Returns 0 on success, 1 otherwise */
static int auth1x_pc_mac_auth(auth1x_pc_t *t, ether_mac_t *mac)
{
    if (ioctl(t->fd, SIOCSAUTH1XAUTHMAC, mac) < 0)
    {
	rg_error(LERR, "ioctl SIOCSAUTH1XAUTHMAC failed");
	return 1;
    }
    if (auth1x_pc_mac_auth_hook && 
	auth1x_pc_mac_auth_hook(t->devname, mac))
    {
	rg_error(LERR, "auth1x_pc_mac_auth_hook() failed");
	return 1;
    }
    return 0;
}

/* Unauthorize mac on the device. Returns 0 on success, 1 otherwise */
static int auth1x_pc_mac_unauth(auth1x_pc_t *t, ether_mac_t *mac)
{
    if (ioctl(t->fd, SIOCSAUTH1XUNAUTHMAC, mac) < 0)
    {
	rg_error(LERR, "ioctl SIOCSAUTH1XUNAUTHMAC failed");
	return 1;
    }
    if (auth1x_pc_mac_unauth_hook && 
	auth1x_pc_mac_unauth_hook(t->devname, mac))
    {
	rg_error(LERR, "auth1x_pc_mac_unauth_hook() failed");
	return 1;
    }
    return 0;
}

/* Returns in *mac, next authorized mac. If on input the mac is all zeros then
 * return the first mac in the list. A mac of all zeros on output represents
 * end of list. Returns 0 on success, 1 otherwise */
static int auth1x_pc_mac_get_next(auth1x_pc_t *t, ether_mac_t *mac)
{
    if (ioctl(t->fd, SIOCGAUTH1XGETNEXTMAC, mac) < 0)
    {
	rg_error(LERR, "ioctl SIOCGAUTH1XGETNEXTMAC failed");
	return 1;
    }
    return 0;
}

/* Returns in *use_count the use_count of device. Used to find if any other
 * user has already running 802.1X on the device. Returns 0 on success, 1
 * otherwise */
static int auth1x_pc_use_count_get(auth1x_pc_t *t, int *use_count)
{
    if (ioctl(t->fd, SIOCGAUTH1XGETUSECOUNT, use_count) < 0)
    {
	rg_error(LERR, "ioctl SIOCGAUTH1XGETUSECOUNT failed");
	return 1;
    }
    return 0;
}

/* Sets pass_single_unauth value of the device. Returns 0 on success, 1
 * otherwise */
static int auth1x_pc_pass_single_unauth_set(auth1x_pc_t *t,
    int pass_single_unauth)
{
    if (ioctl(t->fd, SIOCSAUTH1XSETPASSSINGLEUNAUTH, &pass_single_unauth) < 0)
    {
	rg_error(LERR, "ioctl SIOCSAUTH1XSETPASSSINGLEUNAUTH failed");
	return 1;
    }
    return 0;
}

/* Gets pass_single_unauth value of the device. Returns 0 on success, 1
 * otherwise */
static int auth1x_pc_pass_single_unauth_get(auth1x_pc_t *t,
    int *pass_single_unauth)
{
    if (ioctl(t->fd, SIOCGAUTH1XGETPASSSINGLEUNAUTH, pass_single_unauth) < 0)
    {
	rg_error(LERR, "ioctl SIOCGAUTH1XGETPASSSINGLEUNAUTH failed");
	return 1;
    }
    return 0;
}

/* file ops registered when starting 802.1x authentication on a device */
auth1x_pc_ops_t auth1x_pc_ops = {
    open: auth1x_pc_open,
    close: auth1x_pc_close,
    send_eapol: auth1x_pc_send_eapol,
    mode_set: auth1x_pc_mode_set,
    mac_auth: auth1x_pc_mac_auth,
    mac_unauth: auth1x_pc_mac_unauth,
    pass_single_unauth_set: auth1x_pc_pass_single_unauth_set,
    pass_single_unauth_get: auth1x_pc_pass_single_unauth_get,
    mode_get: auth1x_pc_mode_get,
    mac_get_next: auth1x_pc_mac_get_next,
    use_count_get: auth1x_pc_use_count_get,
};

