/* LICENSE_CODE JUNGO */


#include "rg_error.h"
#include <util/random.h>
#include <event.h>
#include <syscalls.h>
#include <openssl/hmac.h>
#include <openssl/rc4.h>
#include "wep_port.h"
#include "auth1x_conn.h"
#include "misc_funcs.h"

/* wep port context */
struct wep_port_t {
    auth1x_port_t *auth1x_port;
    wep_keys_t wep_keys;
    wep_port_cb_t *cb;
    void *ctx;
};

/* EAPoL Key descriptor packet */
#define RC4_KEY_DESC_TYPE 1

#define KEY_DESC_REPLAY_COUNT_LEN 8
#define KEY_DESC_IV_LEN 16
#define KEY_DESC_SIGNATURE_LEN 16

typedef struct {
    u8 type;
    u16 length;
    u8 replay_count[KEY_DESC_REPLAY_COUNT_LEN];
    u8 iv[KEY_DESC_IV_LEN];
    u8 index;
    u8 signature[KEY_DESC_SIGNATURE_LEN];
    u8 data[1];
} __attribute__ ((packed)) eapol_key_desc_t;

#define SEC_IN_70_YEARS 2208988800UL

eapol_pkt_t *eap_rekey_pkt_create(ether_mac_t *supp_addr, 
    ether_mac_t *auth_addr, u8 *key, int key_len, int key_index, int unicast,
    u8 *pmk, int pmk_len)
{
    int key_desc_len;
    eapol_pkt_t *eapol_pkt;
    eapol_key_desc_t *key_desc;
    int eapol_pkt_len;
    u8 signature[KEY_DESC_SIGNATURE_LEN];
    u8 rc4_enc_key[KEY_DESC_IV_LEN+32];
    RC4_KEY rc4_key;
    u32 ntp_seconds, ntp_useconds;
    struct timeval tv;

    key_desc_len = sizeof(eapol_key_desc_t) - 1;
    if (key)
	key_desc_len += key_len;

    eapol_pkt_len =  sizeof(eapol_pkt_t) + key_desc_len;
    eapol_pkt = (eapol_pkt_t *) malloc_l(eapol_pkt_len);
    if (!eapol_pkt)
	return NULL;

    /* EAPOL packet header */
    memcpy(eapol_pkt->dest, supp_addr, sizeof(ether_mac_t));
    memcpy(eapol_pkt->src, auth_addr, sizeof(ether_mac_t));
    eapol_pkt->ether_type = htons(EAPOL_TYPE);
    eapol_pkt->proto_ver = EAPOL_VER;
    eapol_pkt->pkt_type = ET_KEY;
    eapol_pkt->pkt_body_len = htons(key_desc_len); 

    /* EAPOL Key header */
    key_desc = (eapol_key_desc_t *)eapol_pkt->pkt_body;
    key_desc->type = RC4_KEY_DESC_TYPE;
    key_desc->length = htons(key_len);
    
    /* Set replay count to time in ntp */
    sys_get_time(&tv);
    ntp_seconds = htonl(tv.tv_sec + SEC_IN_70_YEARS);
    ntp_useconds = htonl(4295 * tv.tv_usec - (tv.tv_usec >> 5) -
	(tv.tv_usec >> 9));
    memcpy(key_desc->replay_count, &ntp_seconds, 4);
    /* XXX Temporary fix to overcome a problem in TI's AP which causes both the
     * unicast and broadcast EAPOL keys to contain the same replay counter. Will
     * be fixed before merge to dev
     */
    if (unicast)
	ntp_useconds++; /* XXX fix */
    memcpy(key_desc->replay_count+4, &ntp_useconds, 4);

    /* Generate random vector for IV */
    random_buf_get(key_desc->iv, KEY_DESC_IV_LEN);
    
    MZERO(key_desc->signature);
    
    key_desc->index = key_index | (unicast ? 1 << 7 : 0);

    if (key)
    {
	/* Encrypt key using rc4 */
	memcpy(rc4_enc_key, key_desc->iv, KEY_DESC_IV_LEN);
	memcpy(rc4_enc_key+KEY_DESC_IV_LEN, pmk, 32);
	RC4_set_key(&rc4_key, KEY_DESC_IV_LEN+32, rc4_enc_key);
	RC4(&rc4_key, key_len, key, key_desc->data);
    }

    /* sign packet using HMAC_md5 */
    HMAC(EVP_md5(), pmk+32, 32, &eapol_pkt->proto_ver,
	key_desc_len + 4, /* for the eapol headers */
	signature, NULL);
    memcpy(key_desc->signature, signature, KEY_DESC_SIGNATURE_LEN); 
    
    return eapol_pkt;
}

/* Send a single key to a single supplicant (connection) */
static void wep_port_conn_send_single_key(wep_port_t *t, auth1x_conn_t *conn,
    int key_id, int unicast)
{
    u8 *pmk = NULL;
    int pmk_len;
    eapol_pkt_t *eapol_pkt;

    auth1x_conn_pmk_get(conn, &pmk, &pmk_len);
    if (!pmk)
	return;

    if (!t->wep_keys.key[key_id].data)
	return;

    eapol_pkt = eap_rekey_pkt_create(auth1x_conn_mac_get(conn),
	auth1x_port_mac_get(t->auth1x_port), t->wep_keys.key[key_id].data, 
	t->wep_keys.key[key_id].sz, key_id, unicast, pmk, pmk_len);

    if (!eapol_pkt)
	return;

    auth1x_conn_eapol_pkt_send(conn, eapol_pkt, 0);

    free(eapol_pkt);
}

/* Send all keys to a single supplicant (connection) */
static void wep_port_conn_send_keys(wep_port_t *t, auth1x_conn_t *conn)
{
    /* Send one broadcast key and one unicast key. */
    wep_port_conn_send_single_key(t, conn, 0, 0);
    wep_port_conn_send_single_key(t, conn, t->wep_keys.tx_key, 1);
}

/* Send a single key to all supplicants (connections) */
static void wep_port_send_key(wep_port_t *t, int key_id)
{
    auth1x_conn_t *conn = NULL;
    
    while ((conn = auth1x_conn_get_next(t->auth1x_port, conn)))
	wep_port_conn_send_single_key(t, conn, key_id, 1);
}

/* On timeout update current key */
static void wep_port_rekey_timeout(void *context)
{
    wep_port_t *t = (wep_port_t *)context;
    int rekey_key;

    /* increment the current key rekey the last one used. */
    t->wep_keys.tx_key++;
    if (t->wep_keys.tx_key == 4)
	t->wep_keys.tx_key = 1;

    rekey_key = t->wep_keys.tx_key;

    random_buf_get(t->wep_keys.key[rekey_key].data,
	t->wep_keys.key[rekey_key].sz);
    
    /* write new key to ap */
    t->cb->wlan_ap_set_wep_key(t->ctx, rekey_key, 
	t->wep_keys.key[rekey_key].data, t->wep_keys.key[rekey_key].sz);

    /* Send dynamic key to all supplicants */
    wep_port_send_key(t, rekey_key);

    if (t->wep_keys.type == WEP_TYPE_DYNAMIC)
	t->cb->wlan_ap_set_tx_wep_key(t->ctx,t->wep_keys.tx_key);
}

static void wep_port_assoc_auth_end(void *context, auth1x_conn_t *conn,
    int is_authenticated, u8 *pmk, int pmk_len)
{
    wep_port_t *t = (wep_port_t *)context;

    if (is_authenticated)
    {
	rg_error(LINFO, "802.1x: Station %s authenticated successfully",
	    mactoa(auth1x_conn_mac_get(conn)));
	/* We always send keys to the station, even if we're in fixed mode.
	 * it can't hurt.
	 */
	wep_port_conn_send_keys(t, conn);
    }
    else
    {
	rg_error(LWARN, "802.1x: Station %s failed authentication",
	    mactoa(auth1x_conn_mac_get(conn)));
    }

}

static void wep_port_conn_term(void *context, auth1x_conn_t *conn)
{
    rg_error(LDEBUG,"802.1x: WEP connection to station %s closed", 
	mactoa(auth1x_conn_mac_get(conn)));
    auth1x_conn_close(conn);
}

auth1x_conn_cb_t auth1x_conn_cb = {
    NULL, /* EAPOL key received. */
    wep_port_assoc_auth_end,
    wep_port_conn_term
};

static void wep_port_new_assoc(void *ctx, ether_mac_t *src)
{
    wep_port_t *t = (wep_port_t *)ctx;
    auth1x_conn_t *conn;

    conn = auth1x_conn_open(t->auth1x_port, src, &auth1x_conn_cb, t);
    if (!conn)
	return;

    /* Start authenticating */
    rg_error(LDEBUG,"802.1x: starting authentication" );
    auth1x_conn_auth_start(conn);
}

wep_port_t *wep_port_open(auth1x_port_params_t *params, wep_port_cb_t *cb,
    auth1x_pc_ops_t *port_ctl_ops, void *ctx)
{
    wep_port_t *t;
    static auth1x_port_cb_t auth1x_port_cb;
    int i;

    auth1x_port_cb.client_get = cb->client_get;
    auth1x_port_cb.new_assoc = wep_port_new_assoc;

    t = zalloc_l(sizeof(wep_port_t));
    if (!t)
	return NULL;

    t->cb = cb;
    t->ctx = ctx;
    t->auth1x_port = auth1x_port_open(params, &auth1x_port_cb, port_ctl_ops, t);
    if (!t->auth1x_port)
    {
	free(t);
	return NULL;
    }

    /* Don't try to set WEP keys if wired */
    if (!params->is_wireless)
	goto Exit;

    t->wep_keys = params->wep_keys;

    /* Set all keys to ap, NULL keys are not set (DISABLED and DYNAMIC) */
    switch (t->wep_keys.type)
    {
    case WEP_TYPE_DISABLED:
    case WEP_TYPE_FIXED:
	goto Exit;
    case WEP_TYPE_MIXED:
	rg_error(LDEBUG, "802.1x: Installing random keys to AP");

	for (i = 1; i < WEP_KEYS_NUM; i++)
	{
	    random_buf_get(t->wep_keys.key[i].data, t->wep_keys.key[i].sz);
	    t->cb->wlan_ap_set_wep_key(t->ctx, i, t->wep_keys.key[i].data, 
		t->wep_keys.key[i].sz);
	}

	t->cb->wlan_ap_set_wep_key(t->ctx, 0, t->wep_keys.key[0].data, 
	    t->wep_keys.key[0].sz);
	/* Set 0 key as active key */
	t->cb->wlan_ap_set_tx_wep_key(t->ctx, 0);
	break;
    case WEP_TYPE_DYNAMIC:
	rg_error(LDEBUG, "802.1x: Installing random keys to AP");

	for (i = 0; i < WEP_KEYS_NUM; i++)
	{
	    random_buf_get(t->wep_keys.key[i].data, t->wep_keys.key[i].sz);
	    t->cb->wlan_ap_set_wep_key(t->ctx, i, t->wep_keys.key[i].data, 
		t->wep_keys.key[i].sz);
	}
	/* Set 1 key as active key */
	t->cb->wlan_ap_set_tx_wep_key(t->ctx, 1);
	break;
    }

    t->wep_keys.tx_key = 1;

    /* Start Key update timer */
    event_timer_set_period(t->wep_keys.timeout, wep_port_rekey_timeout, t);

Exit:
    return t;
}

void wep_port_close(wep_port_t *t)
{
    event_timer_del(wep_port_rekey_timeout, t);
    auth1x_port_close(t->auth1x_port);
    free(t);
}

void wep_port_reconf(wep_port_t *t, auth1x_port_params_t *params)
{
    auth1x_port_reconf(t->auth1x_port, params);
}

wep_keys_t *wep_port_keys_get(wep_port_t *t)
{
    static wep_keys_t wep_keys;

    if (!t)
	return &wep_keys;

    return &t->wep_keys;
}

