 /***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																									*/
/*		MODULE:	bssbridge.c	      														*/
/*   PURPOSE:	Bssbridge functions.	      			 								*/
/*																									*/
/***************************************************************************/
#include <stdarg.h>
#include "wpaIfc.h"
#include "utils.h"
#include <wpa.h>
#include "public_security_keys.h"

#include "stdio.h"
#include "dda_wpa.h"
#include "linux_ipc_user.h"

/*-----------------------------------------------------------*/
/*         Local Prototypes									 */
/*-----------------------------------------------------------*/

static void set_default_key(ap_dev_t *t,         
							int index,            
							wpa_cipher_t cipher, 
							wpa_temporal_key_t *temporal_key); 
	
static void set_default_key_id(	ap_dev_t *t,
								int index);   

static void remove_default_key(	ap_dev_t *t, 
								int index, 
								wpa_cipher_t cipher);   

static void set_mapping_key(ap_dev_t *t,         
							ether_mac_t mac,    
							wpa_cipher_t cipher, 
							wpa_temporal_key_t *temporal_key);            

static void remove_mapping_key(	ap_dev_t *t,       
								ether_mac_t mac, 
								wpa_cipher_t cipher); 

void deauth_sta(ap_dev_t *t,     
				ether_mac_t mac); 
 
void disassoc_sta(	ap_dev_t *t,      
					ether_mac_t mac); 

static void	set_ap_wpa_info_elem(	ap_dev_t *t,       
									u8 *wpa_info_elem, 
									int len);          

static u64 get_tsc(ap_dev_t *t);


static void block_multicast_traffic(ap_dev_t *t, 
							 int block);  

static void block_unicast_traffic(ap_dev_t *t,
						   int block,
						   ether_mac_t mac); 

static int get_max_num_of_mapping_keys(ap_dev_t *t); 

/*-----------------------------------------------------------*/
/*         Implementation									 */
/*-----------------------------------------------------------*/
/****************************************************************************
 *                      wdrv_wpaApDevInitCallbacks()
 ****************************************************************************
 * DESCRIPTION:	Bssbridge module init function	
 * 
 * INPUTS:	
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
STATUS wdrv_wpaApDevInitCallbacks(ap_dev_ops_t *deviceCBs)
{
	memset(deviceCBs, 0, sizeof(ap_dev_ops_t));
	
	deviceCBs->set_default_key		= set_default_key;
	deviceCBs->set_default_key_id	= set_default_key_id;
	deviceCBs->remove_default_key	= remove_default_key;
	deviceCBs->set_mapping_key		= set_mapping_key;
	deviceCBs->remove_mapping_key	= remove_mapping_key;
	deviceCBs->deauth_sta			= deauth_sta;
	deviceCBs->disassoc_sta			= disassoc_sta;
	deviceCBs->set_ap_wpa_info_elem = set_ap_wpa_info_elem;
	deviceCBs->get_tsc				= get_tsc;
	deviceCBs->block_multicast_traffic	= block_multicast_traffic;
	deviceCBs->block_unicast_traffic	= block_unicast_traffic;
	deviceCBs->get_max_num_of_mapping_keys	= get_max_num_of_mapping_keys;
	return OK;
}

/* Sets a default key */
static void set_default_key(ap_dev_t *t,         /* object context */
					 int index,            /* index in default key array */
					 wpa_cipher_t cipher, /* type of key */
					 wpa_temporal_key_t *temporal_key)            /* key data */

{
	wpaMessage_T	msg;
	int rc;

/*	printf("WPA MODULE:setting default key for index %d.\n",index);*/

	msg.type = SET_DEFAULT_KEY;
	msg.content.setDefaultKey.temporal_key = *temporal_key;
	msg.content.setDefaultKey.cipher = cipher;
	msg.content.setDefaultKey.index = index;

	rc = ipc_connect_buf_write(wdrv_wpaGetListenSock(),(void *)&msg, sizeof(wpaMessage_T));
	if (rc == -1)
	{
		printf("set_default_key:error in 'ipc_connect_buf_write'. Error Code = %d\n", rc);
		return;
	}
}



/* Sets the index for the default key to be used */
static void set_default_key_id(ap_dev_t *t, /* object context */
							   int index)   /* Index of default key that should be used */
{
	wpaMessage_T	msg;
	int rc;

/*	printf("WPA MODULE:setting default key id for index = %d.\n",index);*/


	msg.type = SET_DEFAULT_KEY_ID;
	msg.content.setDefaultKeyId.index = index;

	rc = ipc_connect_buf_write(wdrv_wpaGetListenSock(),(void *)&msg, sizeof(wpaMessage_T));
	if (rc == -1)
	{
		printf("set_default_key_id:error in 'ipc_connect_buf_write'. Error Code = %d\n", rc);
		return;
	}
}
	

/* Removes a default key */
static void remove_default_key(	ap_dev_t *t, 
								int index, 
								wpa_cipher_t cipher)
{
	wpaMessage_T	msg;
	int rc;

/*	printf("WPA MODULE:remove default key for index = %d.\n",index);*/

	msg.type = REMOVE_DEFAULT_KEY;
	msg.content.removeDefaultKey.index = index;
	msg.content.removeDefaultKey.cipher = cipher;


	rc = ipc_connect_buf_write(wdrv_wpaGetListenSock(),(void *)&msg, sizeof(wpaMessage_T));
	if (rc == -1)
	{
		printf("remove_default_key:error in 'ipc_connect_buf_write'. Error Code = %d\n", rc);
		return;
	}
}


/* Set a mapping key */
static void set_mapping_key(ap_dev_t *t,         
							ether_mac_t mac,    
							wpa_cipher_t cipher, 
							wpa_temporal_key_t *temporal_key)            
{
	wpaMessage_T	msg;
	int rc;
/*	char *tmp;*/

/*	printf("WPA MODULE:set mapping key.\n");

	fflush(NULL);*/

	msg.type = SET_MAP_KEY;
	memcpy(msg.content.setMapKey.mac, mac,MAC_ADDR_LEN);
	msg.content.setMapKey.cipher = cipher;
	memcpy(&(msg.content.setMapKey.temporal_key),temporal_key,sizeof(wpa_temporal_key_t));
#if 0
	printf("USERKey =\n"); 
	tmp = (char*)temporal_key->tkip.tkip_key;
	printf("\nKEY : ");
	for (i = 0;i<8;i++)
	{
		printf("0x%x,",tmp[i]);
	}
	tmp = (char*)temporal_key->tkip.tx_mic_key;
	printf("\nUSERTX :");
	for (i = 0;i<8;i++)
	{
		printf("0x%x,",tmp[i]);
	}
	tmp = (char*)temporal_key->tkip.rx_mic_key;
	printf("\nUSERRX :");
	for (i = 0;i<8;i++)
	{
		printf("0x%x,",tmp[i]);
	}

	printf("\nUSER MAC = %x,%x,%x,%x,%x,%x",msg.content.setMapKey.mac[0],
										  msg.content.setMapKey.mac[1],
										  msg.content.setMapKey.mac[2],
										  msg.content.setMapKey.mac[3],
										  msg.content.setMapKey.mac[4],
										  msg.content.setMapKey.mac[5]);

	printf("\nUSER cipher = %d\n",msg.content.setMapKey.cipher);fflush(NULL);
	
#endif
	
	rc = ipc_connect_buf_write(wdrv_wpaGetListenSock(),(void *)&msg, sizeof(wpaMessage_T));
	if (rc == -1)
	{
		printf("set_mapping_key:error in 'ipc_connect_buf_write'. Error Code = %d\n", rc);
		return;
	}

/*	usleep(100);*/
}


/* Removes a mapping key */
static void remove_mapping_key(	ap_dev_t *t,       
								ether_mac_t mac, 
								wpa_cipher_t cipher) 
{
	wpaMessage_T	msg;
	int rc;

/*	printf("WPA MODULE:remove mapping key.\n");*/

	msg.type = REMOVE_MAP_KEY;
	memcpy(msg.content.removeMapKey.mac, mac,MAC_ADDR_LEN);
	msg.content.removeMapKey.cipher = cipher;

	rc = ipc_connect_buf_write(wdrv_wpaGetListenSock(),(void *)&msg, sizeof(wpaMessage_T));
	if (rc == -1)
	{
		printf("remove_mapping_key:error in 'ipc_connect_buf_write'. Error Code = %d\n", rc);
		return;
	}
}



/* Deauthenticates a station */
void deauth_sta(ap_dev_t *t,     
				ether_mac_t mac) 
{
	wpaMessage_T	msg;
	int rc;

/*	printf("WPA MODULE:deauth station.\n");*/

	msg.type = DEAUTH_STA;
	memcpy(msg.content.deAssAuthSta.mac, mac,MAC_ADDR_LEN);

	rc = ipc_connect_buf_write(wdrv_wpaGetListenSock(),(void *)&msg, sizeof(wpaMessage_T));
	if (rc == -1)
	{
		printf("deauth_sta:error in 'ipc_connect_buf_write'. Error Code = %d\n", rc);
		return;
	}
}


/* Disassociates a station */
void disassoc_sta(	ap_dev_t *t,      
					ether_mac_t mac) 
{
	wpaMessage_T	msg;
	int rc;

/*	printf("WPA MODULE:disass station.\n");*/

	msg.type = DISASS_STA;
	memcpy(msg.content.deAssAuthSta.mac, mac,MAC_ADDR_LEN);

	rc = ipc_connect_buf_write(wdrv_wpaGetListenSock(),(void *)&msg, sizeof(wpaMessage_T));
	if (rc == -1)
	{
		printf("disassoc_sta:error in 'ipc_connect_buf_write'. Error Code = %d\n", rc);
		return;
	}
}
 
/* Sets the AP's WPA information element to be included in beacons */
static void set_ap_wpa_info_elem(ap_dev_t *t,       /* object context */
						  u8 *wpa_info_elem, /* WPA information element the AP should use */
						  int len)          /* length of WPA information element */
{
	/* TODO:check why its empty*/
}


/* Get the TKIP sequence counter */
static u64 get_tsc(ap_dev_t *t)
{
	/* we set it to zero, because no body uses this value. */
	return 0;
}

static void block_multicast_traffic(ap_dev_t *t, 
							 int block)
{
	wpaMessage_T	msg;
	int rc;

/*	printf("WPA MODULE:block multy traffic with flag %d.\n",block);*/

	msg.type = BLOCK_MULTYCAST;
	msg.content.blockMcast.block = block;

	rc = ipc_connect_buf_write(wdrv_wpaGetListenSock(),(void *)&msg, sizeof(wpaMessage_T));
	if (rc == -1)
	{
		printf("block_multicast_traffic:error in 'ipc_connect_buf_write'. Error Code = %d\n", rc);
		return;
	}
}

static void block_unicast_traffic(ap_dev_t *t,
						   int block,
						   ether_mac_t mac)
{
	wpaMessage_T	msg;
	int rc;

/*	printf("WPA MODULE:block unicast traffic with flag %d.\n",block);

	fflush(NULL);*/
	
	msg.type = BLOCK_UNICAST;
	msg.content.blockMcast.block = block;
	memcpy(msg.content.blockUcast.mac, mac,MAC_ADDR_LEN);


	rc = ipc_connect_buf_write(wdrv_wpaGetListenSock(),(void *)&msg, sizeof(wpaMessage_T));
	if (rc == -1)
	{
		printf("block_unicast_traffic:error in 'ipc_connect_buf_write'. Error Code = %d\n", rc);
		return;
	}
}

static int get_max_num_of_mapping_keys(ap_dev_t *t)
{
	return MAX_STATIONS;
}

int  wpa_restart(void)
{
	return 0;
}


