/* LICENSE_CODE JUNGO */

#ifndef _ESTREAM_H_
#define _ESTREAM_H_

#include "event.h"

typedef struct estream_t estream_t;
typedef struct estream_stack_t estream_stack_t;
typedef int (*estream_connect_func_t)(estream_t *estream, int connected);
typedef int (*estream_func_t)(estream_t *estream, u8 *data, int len);
typedef int (*estream_filter_func_t)(estream_t *estream, void *data, int len);

typedef enum {
    ESTREAM_NONE = 0x0,
    ESTREAM_BLOCK = 0x1,
    ESTREAM_LINE = 0x2,
    ESTREAM_PARTIAL = 0x4,
    ESTREAM_FOREVER = 0x8,
} estream_in_mode_t;

struct estream_stack_t {
    estream_stack_t *next;
    void *data;
    free_t data_free_func;
    int state;
    estream_func_t func;
};

typedef enum {
    ESTREAM_CLOSE_FD = 0x1,
    ESTREAM_SET_DEF_TIMEOUT = 0x2,
    ESTREAM_CONNECTED = 0x4,
    ESTREAM_NOIP = 0x8,
    ESTREAM_SOCKOPT_SET = 0x10,
    ESERVER_INDEPENDENT_STREAMS = 0x20,
    ESTREAM_SSL = 0x40,
} estream_flags_t;

typedef struct estream_ssl_t estream_ssl_t;

struct estream_t {
    /* fd of the socket */
    int sock;
    /* is this estream being closed */
    int closing;

    /* context for user - can be saved in stack */
    void *data;
    /* callback for end of data validity - signal the user and free data if 
     * needed. Callback should not close the estream.
     */
    free_t data_free_func;

    /* an int for user's use, to know in what stage of the protocol
     * he is now
     */
    int state;

    /* call back functions */
    estream_func_t func;
    estream_func_t except_func;
    estream_func_t timeout_func;
    estream_connect_func_t connect_func;

    /* data filters */
    estream_filter_func_t read_filter;
    estream_filter_func_t write_filter;

    /* timeout for operations in ms */
    int timeout;
    /* have we sent the connect request */
    int start_connect;
    /* have we already connected */
    int connected;
    /* is this an accept(server) or connect(client) socket */
    int is_accept;
    
    /* incoming buffer for read */
    u8 *in_buf;
    int in_req; /* size requested by user */
    int in_recv; /* Amount of data pending for the user in the buffers. */
    int in_alloc; /* size of in_buf */
    int in_max_buf_size;
    estream_in_mode_t in_mode;

    /* outgoing buffer for write */
    u8 *out_buf;
    int out_len;
    int out_sent;
    int out_alloc;

    /* to speed things up, this can be set to avoid repetetive scheduling */
    int dont_update;
    
    /* mode of operation */
    estream_flags_t flags;

    /* src and dst addresses of this connection */
    u32 peer_ip;
    u16 peer_port;
    u32 local_ip;
    u16 local_port;

    /*  the expiration time of the last timeout set */
    double last_timeout_expire;

    int efd_read_set;
    int efd_write_set;
    int et_read_set;
    int changed;

    /* stack of callbacks */
    estream_stack_t *stack;

    /* SSL support */
    estream_ssl_t *ssl;
};

extern int (*estream_ssl_open_func)(estream_t *e, void *ssl_ctx);

#define ESTREAM_DEF_TIMEOUT 3 * MS_SECOND
#define ESTREAM_DEF_BUF_SIZE 1460
#define ESTREAM_DEF_MAX_BUF_SIZE 102400
#define ESTREAM_MAX_LINE 1024

typedef struct eserver_t {
    /* socket to listen */
    int sock;
    /* is this server being closed */
    int closing;
    
    void *data;
    /* list of streams opened on this server */
    struct eserver_stream_t *streams;
    /* function to call for new connections */
    estream_connect_func_t func;
    estream_flags_t flags;
    /* port this server is listening to */
    u16 port;
} eserver_t;

/* Check if connection established ok */
int sock_valid(int sock);
/* Accept incoming sockets */
int sock_accept(int sock);
/* Open a socket to remote host */
int sock_socket(int type, u32 src_ip, u16 src_port);
/* Set default socket options */
int sock_set_sockopt(int sock, estream_flags_t flags);

/* Open server socket using sock/port number and set callback */
eserver_t *eserver_open(int sock, estream_flags_t flags, 
    estream_connect_func_t func, void *data);
eserver_t *eserver_open_port(u16 port, estream_flags_t flags,
    estream_connect_func_t func, void *data);
eserver_t *eserver_open_addr(u32 addr, u16 port, estream_flags_t flags,
    estream_connect_func_t func, void *data);

/* Close server and free memory immediately */
void eserver_close(eserver_t *eserver);
/* Close only estreams selected by the filter function */
void eserver_estreams_close(eserver_t *eserver, estream_filter_func_t filter,
    void *data);

/* Allocate estream (internal) */
estream_t *estream_open(void);
/* Close socket immediately/scheduled */
void estream_close(estream_t *estream);
/* Cancel socket read request */
void estream_read_abort(estream_t *estream);
/* Read len bytes from socket */
int estream_read(estream_t *estream, int len);
/* Call user on every incoming data buffer, and call again when the complete
 * len bytes have arrived.
 * if len==-1, then call on each packet.
 */
int estream_read_partial(estream_t *estream, int len);
/* Call user on every incoming data */
int estream_read_forever(estream_t *estream, int max_buf_size);
/* Call user on each line of input */
int estream_read_line(estream_t *estream);
/* Clears all data waiting in the input buffer */
void estream_clear_input(estream_t *estream);
/* Acknowledge the recived data buffer, and remove it from the estream buffer
 * Only relevant for estream_read_forever.
 */
int estream_read_ack(estream_t *estream, int len);
/* Write len bytes from buf to socket. */
int estream_write(estream_t *estream, void *buf, int len);
int estream_printf(estream_t *estream, char *fmt, ...);
/* Accept socket connection and set callback */
estream_t *estream_accept(int sock, estream_flags_t flags,
    estream_connect_func_t func);
/* Connect via TCP using available socket and set callback */
estream_t *estream_connect_tcp(estream_flags_t flags,
    estream_connect_func_t func, u32 ip, u16 port);
/* Connect via TCP using available socket and set callback (with source address
 * selection option)
 */
estream_t *estream_connect_tcp_full(int sock, estream_flags_t flags,
    estream_connect_func_t func, u32 ip, u16 port, u32 src_ip, u16 src_port);
/* Connect a socket to file descriptor and set callback */
estream_t *estream_connect_fd(int sock, estream_flags_t flags,
    estream_connect_func_t func);

/* Set timeout period for next operation in ms and the timeout callback */
void estream_set_timeout(estream_t *estream, int timeout,
    estream_func_t timeout_func);
/* Set new socket data and free handler (previous value is freed) */
void estream_set_data(estream_t *estream, void *data,
    free_t data_free_func);
/* Get internal data */
void *estream_get_data(estream_t *estream);
/* Get connection status for existing socket */
int estream_is_connected(estream_t *estream);
/* Set callback function for next event */
void estream_set_func(estream_t *estream, estream_func_t func);
/* Set filter functions */
void estream_set_read_filter(estream_t *estream, estream_filter_func_t func);
void estream_set_write_filter(estream_t *estream, estream_filter_func_t func);
/* Get/Set state value */
void estream_set_state(estream_t *estream, int state);
int estream_get_state(estream_t *estream);
/* Get number of bytes in read request */
size_t estream_get_read_req(estream_t *estream);
/* save and retrieve stack.
 * Note that estream_push does not cancel pending events for the estream, so you
 * might want to cancel requrests before pushing.
 */
int estream_push(estream_t *estream);
int estream_pop(estream_t *estream);

#endif

