/* LICENSE_CODE JUNGO */

/* General RG defenitions */
#ifndef _RG_DEFINES_H_
#define _RG_DEFINES_H_

/* Needed for IFNAMSIZ and in_addr in ipsec_conn_t */
#include <net/if.h>
#include <netinet/in.h>
#include <conv.h>

/* Ethernet over ATM control parameters */
typedef enum {
    ATM_ENCAPSULATION_VC = 1,
    ATM_ENCAPSULATION_LLC = 2,
    ATM_ENCAPSULATION_VC_HDLC = 3,
} atm_encapsulation_t;

typedef enum {
    ATM_ROUTE_BRIDGE = 1,
    ATM_ROUTE_ROUTED = 2,
} atm_route_t;

typedef enum {
    IPSEC_TRANS_TUNNELING = 1,
    IPSEC_TRANS_TRANSPORT = 2,
} ipsec_trans_type_t;

typedef enum {
    IPSEC_AUTH_SHA1 = 0x1,
    IPSEC_AUTH_MD5 = 0x2,
} ipsec_auth_type_t;

typedef enum {
    IPSEC_PROTO_AH = 0x1,
    IPSEC_PROTO_ESP = 0x2,
} ipsec_proto_t;

typedef enum {
    IPSEC_ENC_NULL = 0x1,
    IPSEC_ENC_DES = 0x2,
    IPSEC_ENC_3DES = 0x4,
} ipsec_enc_type_t;

typedef enum {
    IPSEC_KX_AUTO = 1,
    IPSEC_KX_MANUAL = 2,
} ipsec_kx_type_t;

typedef enum {
    IPSEC_AUTH_TYPE_SHARED = 0,
    IPSEC_AUTH_TYPE_RSA = 1,
    IPSEC_AUTH_TYPE_CERT = 2,
} ipsec_peer_auth_type_t;

typedef struct {
    int cert_id; /* Certificate id. */
    void *cert_data; /* Certificate content. */
    int cert_data_size; /* Certificate content size. */
    void *cert_private; /* Private key content. */
    int cert_private_size; /* Private key content size. */
    /* Connection's IDs. Used for certificate authentication only. */
    char *local_id, *peer_id;
} ipsec_conn_cert_t;

/* A U T O M A T I C   P A R A M E T E R S 
 * These parameters are used when defining an automatic connection
 */

typedef struct {
    /* how many rekey negotiation attempts should be done, before we stop
     * trying to initiate the automatic connection
     * 0 - Infinite
     * n - try n times
     */
    int negotiation_attmpts;

    /* Rekey parameters.
     * ike_lifetime and ipsec_lifetime must be greater than
     * rekeymargin*(100+rekeyfuzz)/100
     */
    /* Life time for IKE encryption key; MAIN mode (in seconds). */
    int ike_lifetime;
    /* Life time for IPSec encryption key; QUICK mode (in seconds). */
    int ipsec_lifetime;
    /* How long before connection expiry should attempts to negotiate a
     * replacement begin (in seconds).
     */
    int rekey_margin;
     /* maximum  percentage  by  which  rekeymargin  should be randomly
      * increased to randomize rekeying intervals (in percents).
      */
    int rekey_fuzz;

    /* Perfect Forwarding Security - This is an enhancement to the IKE protocol
     * in which the re-keying is speeded up - with some small compromise in
     * security. 
     * 0 - Disabled (default)
     * 1 - Enabled
     */
    int is_pfs;
    ipsec_peer_auth_type_t auth_type;

    union {
	/* Peer's RSASIGKEY or shared secret */
	char *key;
	/* OpenRG's certificate */
	ipsec_conn_cert_t *cert;
    } secure;

    /* Propositions - combination bits */
    int main_auth;
    int main_encrypt;
    int main_grp;
    int quick_auth;
    int quick_encrypt;
    int quick_hash;
} ipsec_conn_auto_t;

/* M A N U A L   P A R A M E T E R S
 * These parameters are used when defining a manual connection
 */
typedef struct {
    /* SPI numbers used to identify the connection for both sides, the SPI is
     * a number between 0x100 and 0xFFFF_FFFF.
     */
    unsigned int local_spi, remote_spi;
    
    ipsec_proto_t protocol;
    /* auth is common for IPSEC_PROTO_AH and IPSEC_PROTO_ESP */
    ipsec_auth_type_t auth;
    /* encrypt is used only for IPSEC_PROTO_ESP */
    ipsec_enc_type_t encrypt;

    /* A string of the form 0x12aab.... 128-bit long for md5 and 160-bit long
     * for sha1. It is common for IPSEC_PROTO_AH and IPSEC_PROTO_ESP.
     */
    char *local_auth_key;
    char *remote_auth_key;

    /* Encryption key for ESP. A string of the form 0x12aab....
     * 192-bit long for 3des, 64-bit for des and empty for null.
     */
    char *local_encrypt_key;
    char *remote_encrypt_key;
} ipsec_conn_manual_t;

typedef enum {
    IPSEC_GRP_1 = 0x1,
    IPSEC_GRP_2 = 0x2,
    IPSEC_GRP_5 = 0x4,
} ipsec_group_descr_t;

typedef struct ipsec_conn_t {
    /* A *unique* connection name (e.g. "kylie-meir", "here-to-there") */
    char conn_name[IFNAMSIZ];
   
    struct in_addr remote_ip;
    /* Definition of LAN subnet that will be connected via this connection
     * for Host-to-Host leave these fields 0.0.0.0
     * for Subnet-to-Subnet
     * fill (e.g. subnet=192.168.71.0; netmask=255.255.255.0)
     */
    struct in_addr local_wan_ip;
    struct in_addr local_lan_subnet;
    struct in_addr local_lan_netmask;
    struct in_addr local_nexthop;
    struct in_addr remote_lan_subnet;
    struct in_addr remote_lan_netmask;

    ipsec_trans_type_t trans_type;

    /* 0 - Disable IP Compression
     * 1 - Enable IP Compression
     * Why is compression important in the context of encryption? this is
     * important because encrypted information can't be compressed, and
     * therefore compression must be done prior to encryption.
     */
    int is_compressed;

    /* 0 - Manual key exchange
     * 1 - Automatic key exchange
     */
    ipsec_kx_type_t type;
    union {
	ipsec_conn_auto_t automatic;
	ipsec_conn_manual_t manual;
    } settings;
} ipsec_conn_t;

/* XXX from cablehome/evt_rpt/evt_rpt.h */

/* The following enum provides mapping between internal event names and their
 * corresponding entry number in 'events' table. Since values in this enum
 * function as indices to the events table, they MUST be subsequent, starting
 * from 0
 */
typedef enum {
    /* DHCP errors before provisioning complete */
    EVT_ID_INIT_CDC_NO_OFFER = 0,
    EVT_ID_INIT_CDC_NO_RESPONSE = 1,
    EVT_ID_INIT_CDC_NOT_SUPPORTED = 2,
    EVT_ID_INIT_CDC_RESPONSE_NOT_COMPLETE = 3,
    EVT_ID_INIT_CDC_WAN_IP_OBTAINING = 4,
    /* TOD errors before provisioning complete */
    EVT_ID_INIT_TOD_NO_RESPONSE = 5,
    EVT_ID_INIT_TOD_INVALID_DATA = 6,
    /* TFTP errors before provisioning complete */
    EVT_ID_INIT_TFTP_NO_RESPONSE = 7,
    EVT_ID_INIT_TFTP_NO_CONFIG = 8,
    EVT_ID_INIT_TFTP_OUT_OF_ORDER = 9,
    EVT_ID_INIT_TFTP_SHA1_FAILURE = 10,
    EVT_ID_INIT_TFTP_RETRIES_EXCEEDED = 11,
    /* TFTP success */
    EVT_ID_INIT_TFTP_SUCCESS = 12,
    /* TLV parsing */
    EVT_ID_INIT_TLV_UNRECOGNIZED_OID = 13,
    EVT_ID_INIT_TLV_UNKNOWN_TLV = 14,
    EVT_ID_INIT_TLV_INVALID_FORMAT = 15,
    /* Provisioning */
    EVT_ID_INIT_PROVISIONING_COMPLETE = 16,
    /* DHCP failure after provisioning complete */
    EVT_ID_DHCP_CDC_RENEW_SENT_NO_RESPONSE = 17,
    EVT_ID_DHCP_CDC_REBIND_SENT_NO_RESPONSE = 18,
    EVT_ID_DHCP_CDC_RENEW_SENT_INVALID_OPTION = 19,
    EVT_ID_DHCP_CDC_REBIND_SENT_INVALID_OPTION = 20,
    /* TOD failure after provisioning complete */
    EVT_ID_TOD_AFTER_PROVISIONING_NO_RESPONSE = 21,
    EVT_ID_TOD_AFTER_PROVISIONING_INVALID_DATA = 22,
    /* Verification of code file */
    EVT_ID_SW_UPGRADE_IMPROPER_CODE_FILE_CONTROLS = 23,
    EVT_ID_SW_UPGRADE_MANUFACTURER_CVC_FAILURE = 24,
    EVT_ID_SW_UPGRADE_MANUFACTURER_CVS_FAILURE = 25,
    EVT_ID_SW_UPGRADE_COSIGNER_CVC_FAILURE = 26,
    EVT_ID_SW_UPGRADE_COSIGNER_CVS_FAILURE = 27,
    /* Verification of CVC */
    EVT_ID_SW_UPGRADE_CONFIG_CVC_FORMAT = 28,
    EVT_ID_SW_UPGRADE_CONFIG_CVC_VALIDATION = 29,
    EVT_ID_SW_UPGRADE_SNMP_CVC_FORMAT = 30,
    EVT_ID_SW_UPGRADE_SNMP_CVC_VALIDATION = 31,
    /* CDP events */
    EVT_ID_CDP_LAN_IP_ALLOCATION = 32,
    EVT_ID_CDP_POOL_EXHAUSTED = 33,
    /* CSP events */
    EVT_ID_CSP_HACKER_THRESHOLD_EXCEEDED = 34,
    EVT_ID_CSP_TYPE1 = 35,
    EVT_ID_CSP_TYPE2 = 36,
    EVT_ID_CSP_CONFIG_CHANGED = 37,
    EVT_ID_CSP_TFTP_NO_RESPONSE = 38,
    EVT_ID_CSP_TFTP_FILE_NO_FOUND = 39,
    EVT_ID_CSP_INVALID_FILE = 40,
    EVT_ID_CSP_SHA1_FAILURE = 41,
    EVT_ID_CSP_TFTP_RETRIES_EXCEEDED = 42,
    EVT_ID_CSP_TFTP_SUCCESS = 43,
    /* CAP events */
    EVT_ID_CAP_CNAT_MAPPING = 44,
    EVT_ID_CAP_CNAPT_MAPPING = 45,
    /* CTP events */
    EVT_ID_CTP_CONNECTION_SPEED_SUCCESS = 46,
    EVT_ID_CTP_CONNECTION_SPEED_TIMEOUT = 47,
    EVT_ID_CTP_CONNECTION_SPEED_ABORTED = 48,
    EVT_ID_CTP_PING_SUCCESS = 49,
    EVT_ID_CTP_PING_TIMEOUT = 50,
    EVT_ID_CTP_PING_ABORTED = 51,
    /* SW upgrade init */
    EVT_ID_SW_UPGRADE_INIT_NMS = 52,
    EVT_ID_SW_UPGRADE_INIT_CONFIG = 53,
    /* SW upgrade general failure */
    EVT_ID_SW_UPGRADE_RETRIES_EXCEEDED = 54,
    EVT_ID_SW_UPGRADE_SERVER_NOT_FOUND = 55,
    EVT_ID_SW_UPGRADE_FILE_NOT_FOUND = 56,
    EVT_ID_SW_UPGRADE_TFTP_RETRIES_EXCEEDED = 57,
    EVT_ID_SW_UPGRADE_INCOMPATIBLE_FILE = 58,
    EVT_ID_SW_UPGRADE_FILE_CORRUPTION = 59,
    EVT_ID_SW_UPGRADE_POWER_FAILURE = 60,
    /* SW upgrade success */
    EVT_ID_SW_UPGRADE_SUCCESS_NMS = 61,
    EVT_ID_SW_UPGRADE_SUCCESS_CONFIG = 62,
} evt_key_t;

/* XXX this file is from rmt-update/ch_upg.h */

/* This enumaration type describes the available administrator status. */
typedef enum {
    RMT_UPG_FROM_MGT = 1,
    RMT_UPG_ALLOW_PROV_UPG = 2,
    RMT_UPG_IGNORE_PROV_UPG = 3,
} ch_rmt_upg_admin_status_t;

typedef enum {
    TOD_P_TIME = 1, /* Time Protocol, RFC 868 */
    TOD_P_SNTP = 2, /* SNTP ver 1/2/3/4, RFC 2030 */
} tod_proto_t;

typedef enum {
    WL_MACFILTER_OFF = 1,
    WL_MACFILTER_ALLOW = 2,
    WL_MACFILTER_DENY = 3,
} wl_macfilter_t;

typedef enum {
    WL_AUTH_OPEN = 0,
    WL_AUTH_SHARED = 1,
} wl_auth_t;

/* X509 */
typedef enum {
    X509_OPENRG = 1, 
    X509_CA = 2,
} x509_cert_owner_t;

typedef enum {
    DHCP_MODE_DISABLED = 0,
    DHCP_MODE_SERVER = 1,
    DHCP_MODE_RELAY = 2,
} dhcp_mode_t;

typedef enum { 
    RULE_ACTION_EMPTY = 0,
    RULE_ACTION_ACCEPT = 1,
    RULE_ACTION_BLOCK = 2,
} rule_action_t;

typedef enum {
    PPP_COMP_REJECT = 0,
    PPP_COMP_ALLOW = 1,
    PPP_COMP_REQUIRE = 2,
} ppp_comp_opt_t;

typedef enum {
    DOT11_MODE_MIXED = 1,
    DOT11_MODE_G_ONLY = 2,
    DOT11_MODE_B_ONLY = 3,
} dot11_mode_t;

extern code2str_t dot11_mode_t_str[];

#endif
