/* LICENSE_CODE JUNGO */

#ifndef _SET_H_
#define _SET_H_

#include <netinet/in.h>

/* The set library uses the zalloc_e functions for allocations and will exit
 * on error. Any error returned from these functions is a programmatic error
 * such as giving a NULL value.
 */

typedef struct set_t {
    char *value;
    struct set_t *next;
    struct set_t *son;
} set_t;

/* Add/create */

/* Alloc a new set with requested value */
set_t *set_alloc(char *value);

/* Add path to current set, return set at end of path or NULL if fail. 
 * Path is '/' seperated, with '\\' as escape character
 */
set_t **set_set_path(set_t **set, char *path);

/* Add a new set, as last son of the given set. Return pointer to the newly
 * created set or NULL if fails. If a set that has the same value already
 * exists - returns it
 */
set_t **set_set(set_t **set, char *value);

/* Add path, and new set as path's son, with 0/1 value */
set_t **set_set_path_flag(set_t **set, char *path, int flag);

/* Add path, and new set as path's son, with value */
set_t **set_set_path_value(set_t **set, char *path, char *value);

/* Add path, and new set as path's son, with in_addr value */
set_t **set_set_path_ip(set_t **set, char *path, struct in_addr ip);

/* Add path, and new set as path's son, with int value */
set_t **set_set_path_int(set_t **set, char *path, int value);

/* Get data value at the end of path starting from set. */
char *set_get_path_str(set_t **set, char *path);
char *set_get_path_strz(set_t **set, char *path);
int set_get_path_int(set_t **set, char *path);
int set_get_path_flag(set_t **set, char *path);
struct in_addr set_get_path_ip(set_t **set, char *path);

/* Replace the set with new_set (set_free then set_dup), returns pointer to the
 * new_set 
 */
set_t **set_replace(set_t **set, set_t *new_set);

/* Compare two sets (deep). Return 0 if trees are euqal or -1 otherwise */
int set_compare(set_t **seta, set_t **setb);

/* Join a new tree to existing tree. If there is a son with the same value as
 * the new tree root, replace it. The new tree must be root with no 'brothers'
 * to avoid structure corruption
 */
set_t **set_join(set_t **root, set_t *set);

/* Add a new set as son of given set, with index (list) logic.
 * If index is negative and equals:
 *   SET_IDX_ADD_AUTO: The added index will be the highest existing index + 1.
 *   SET_IDX_ADD_FREE: The added index will be the first free index. If
 *                     there are no "holes" then it will be last index + 1.
 * If index is 0 or positive:
 *   It will add the set at the index requested, pushing all existing indexes
 *   after it, stopping at the first "hole".
 * Return pointer to index set, or NULL if failed
 */
set_t **set_index_add(set_t **set, int index);

/* Remove a set, with index (list) logic.
 * If index is 0 or positive it will pull back all the indexes after the
 *   deleted index. Return 0 on success or -1 on failure (index not found
 *   or allocation error)
 */
int set_index_del(set_t **set, int index);

/* Re-index the indexes from 0 till (n-1). Just overwrites the indexes,
 * without checking their previous value. Return last index or -1 if fails
 * (set has no sons)
 */
int set_index_reindex(set_t **set);

/* Change the value of a set, return NULL if fails */
set_t **set_set_str(set_t **set, char *name);
set_t **set_set_int(set_t **set, int i);
set_t **set_set_hex(set_t **set, int i);

/* Duplicate */

/* Allocate a new set, and duplicate org_set and its sons, return pointer to
 * the new set or NULL if fails
 */ 
set_t *set_dup(set_t **org_set);

/* Allocate a new set, and duplicate the org_set without sons
 * (shallow copy)
 */
set_t *set_dup_one(set_t **org_set);

/* Delete */

/* Free set and all its sons. Set the pointer pointing to the set to
 * NULL. Can be called with set NULL pointer (does nothing). If the
 * deleted set is in a list (has next) then the list is updated
 */
void set_free(set_t **set);

/* Free only the given set, without updating the parent or list.
 * Use this function carfully, since if used on a complex tree it
 * might corrupt the data structure
 */
void set_free_one(set_t *set);

/* Print the tree structure (for debugging purposes) */
void set_print_tree(set_t **set);

/* Search */

/* Get requested set, return pointer to the found set or NULL if the
 * set does not exist
 */
set_t **set_get(set_t **set, char *path);

/* Perform a search from current set over all set->sons for the sequence 
 * name=value. Return pointer to the set containing name set (and starting
 * from "name", see example) (or NULL if not found)
 * Name can be in format of path: set_get_byname(set, "name/first", "shay")
 */
set_t **set_get_byname(set_t **set, char *name, char *value);

/* Return the number of son sets, count starts from 1, return 0 if list 
 * empty
 */
int set_get_nsons(set_t **set);

/* Return the son in location n, where 0 is the first son. If n is too large
 * then return NULL
 */
set_t **set_get_son(set_t **set, int n);

/* Get the next of a set. Return NULL if set is NULL or if there is no next */
set_t **set_get_next(set_t **set);

/* Get value of set */
char *set_get_value(set_t **set);
int set_get_int(set_t **set); /* return as int. on error - return zero */

/* Create an array of integers of size "len" under "set" whose members are
 * contained in the formatted string "value".
 * Format is - "12, 234, 453, 234", where the delimiter (excluding space) must
 * be immediately after the number.
 */
void set_set_int_arr(set_t **set, char *value, int len);

/* Get member of array of integers. Return NULL if set does not contain son
 * number "index".
 */
int set_arr_get_int(set_t **set, int index);

/* Return value of data-set under set (set->son->value). If the set does
 * not contain a data-set - return NULL
 */
char *set_son_get_str(set_t **set); /* if empty return NULL */
char *set_son_get_strz(set_t **set); /* if empty return "" */
int set_son_get_int(set_t **set); /* return int */
int set_son_get_hex(set_t **set); /* return hex */

/* Serialization */

/* Create a flat buffer from the given SDB set (root). User needs to free
 * memory. The sets in buffer are in human readable format and contain \n and
 * tabs. BEWARE: When creating the buffer manually, don't use special
 * characters: c>126 || c<32 || c=='(' || c==')' || c=='&' || c==';')
 */
char *set_write_alloc(set_t **set);

/* Does the same as set_write_alloc(), only adds '\r' before each '\n' */
char *set_write_alloc_dos(set_t **set);

/* Expand the given buffer into an SDB, return pointer to new tree */
set_t *set_read(char *buf);

/* Merge 'to' and 'from' into 'to'.
 * If there were conflicts on merge, returns -1, otherwise 0.
 */
int set_merge(set_t **to, set_t **from);

/* Convert bin to hex store */
void set_set_path_bin(set_t **set, char *path, int len, char *bin);

/* Calculate needed len for binary data */
int set_get_path_bin_len(set_t **set, char *path);

/* Get hex data and convert to binary.
 * Function returns pointer to bin
 */
char *set_get_path_bin_buf(set_t **set, char *path, char *bin);

/* Obscure value and store to path */
int set_set_path_obscured(set_t **set, char *path, char *value);

/* Read value from path and unonscure it.
 * NOTE. Function allocates returned buffer. Caller need to call free().
 */
char *set_get_path_unobscured(set_t **set, char *path);

/* Some defines */

#define SET_IDX_ADD_AUTO (-1)
#define SET_IDX_ADD_FREE (-2)

#define SPECIAL_CHAR_TO_AMP_HEX 1
#define SPECIAL_AMP_HEX_TO_CHAR 2

#define SPECIAL_CHAR_SIZE 3

#ifdef SET_DEBUG
#define DPRINT_SET(set, set_name) \
{ \
    if (!set || !*set) \
    { \
	printf("NULL (%s) %s:%d\n", __FILE__, __FUNCTION__, __LINE__); \
	exit(1); \
    } \
    printf("%s: (*set)->([value=%s] [son=0x%p] [next=0x%p]) \n",\
	set_name, (*set)->value, (*set)->son, (*set)->next); \
}

#define DPRINTF(arg...) printf(arg...)

#define ASSERT(ptr) \
{ \
     if (!ptr) \
    { \
	printf("NULL (%s) %s:%d\n", __FILE__, __FUNCTION__, __LINE__); \
	exit(1); \
    } \
}

#define ASSERT_P2P(ptr_to_ptr) \
{ \
    ASSERT(ptr_to_ptr); \
    ASSERT(*ptr_to_ptr); \
}
#endif
     
#endif

