/* LICENSE_CODE JUNGO */

#ifndef _WPA_H_
#define _WPA_H_

#include <net_utils.h>
#include <802.1x/auth1x.h>

#define WPA_NUM_DEFAULT_KEYS        4

#define WPA_MIC_KEY_LEN             8
#define WPA_TKIP_ENCRYPTION_KEY_LEN 16 
#define WPA_TKIP_KEY_LEN            WPA_TKIP_ENCRYPTION_KEY_LEN + 2 * \
    WPA_MIC_KEY_LEN
#define WPA_WEP_40_KEY_LEN          5
#define WPA_WEP_104_KEY_LEN         13
#define WPA_WEP_232_KEY_LEN         29
#define WPA_MAX_WEP_KEY_LEN         WPA_WEP_232_KEY_LEN
#define WPA_PSK_KEY_LEN             32 
#define WPA_TSC_LEN                 6

/* Station configuration types */ 
typedef enum {
    WPA_STA_TYPE_WPA        = 0x1, /* WPA station */ 
    WPA_STA_TYPE_WEP_LEGACY = 0x2, /* WEP station using fixed keys */ 
    WPA_STA_TYPE_WEP_8021X  = 0x4, /* WEP station supporting 802.1X 
                                    * authentication and key exchange 
                                    */ 
} wpa_sta_type_t;

#ifndef wpa_cipher_t_defined
#define wpa_cipher_t_defined
/* Pairwise and group cipher suites */ 
typedef enum {
    WPA_CIPHER_TKIP    = 0x1,
    WPA_CIPHER_WEP_40  = 0x2,
    WPA_CIPHER_WEP_104 = 0x4,
    WPA_CIPHER_WEP_232 = 0x8,
    WPA_CIPHER_WRAP    = 0x16,
    WPA_CIPHER_CCMP    = 0x32,
} wpa_cipher_t;
#endif

/* Used to specify how the PSK is passed as a parameter to wpa_port */
typedef enum {
    WPA_PSK_PARAM_NONE  = 0,
    WPA_PSK_PARAM_HEX   = 1,
    WPA_PSK_PARAM_ASCII = 2,
} wpa_psk_param_t;
        
typedef struct { 
    wpa_cipher_t cipher; /* Can be WPA_CIPHER_WEP_40, WPA_CIPHER_WEP_104 or
                          * WPA_CIPHER_WEP_232
                          */
    u8 data[WPA_MAX_WEP_KEY_LEN]; 
} wpa_wep_key_t;

typedef struct { 
    u8 data[WPA_PSK_KEY_LEN];
} wpa_psk_t;

/* Temporal key is the part of the PTK and GTK which is fed to the data 
 * encryption engine. It is used differently depending on the cipher suite in
 * use 
 */
typedef union {

    u8 tk[16][2];
   
    struct {
        u8 tkip_key[WPA_TKIP_ENCRYPTION_KEY_LEN];
        u8 tx_mic_key[WPA_MIC_KEY_LEN];
        u8 rx_mic_key[WPA_MIC_KEY_LEN];
    } tkip;
    
    struct {
        u8 wep_key[WPA_MAX_WEP_KEY_LEN];
    } wep;

} wpa_temporal_key_t;

/* Access point device interface */

typedef void *ap_dev_t;

typedef struct {
    /* Called when the AP receives an association request from a station to
     * query whether the station should be allowed to associate. Returns 0 if
     * station can be associated, a non-zero reason code otherwise 
     */
    int (*sta_assoc_request)(
        void *context,      /* callback context */
        u8 *info_elem,      /* WPA information element of station, may be NULL 
                             * if station didn't include an information element
                             * in its association request
                             */
        int info_elem_len); /* length of information element */
    
    /* Called when a new station associates to the AP */
    void (*sta_assoc_done)(
        void *context,      /* callback context */
        ether_mac_t mac,    /* MAC address of associated station */
        u8 *info_elem,      /* WPA information element of station, may be NULL 
                             * if station didn't include an information element
                             * in its association request
                             */
        int info_elem_len); /* length of information element */

    /* Called when a station disassociates from the AP */
    void (*sta_disassoc)(
        void *context,    /* callback context */
        ether_mac_t mac); /* MAC address of dissasociated station */
 
    /* Called when a MIC failure is detected in a packet protected with a
     * unicast key */ 
    void (*unicast_mic_failure)(
        void *context,    /* callback context */
        ether_mac_t mac); /* MAC address of the station from which the bad 
                             packet was received */

    /* Called when a first data packet is received from a station since it's
     * associated */
    void (*first_data_packet_received)(
        void *context,    /* callback context */
        int is_eapol,     /* 1 if the it's an EAPOL packet, 0 otherwise */
        ether_mac_t mac); /* Packet source MAC address */

} ap_dev_cbs_t;

/* Opeartions provided by the underlying access point driver */
typedef struct {
    /* Allocates and initializes an ap_dev_t object */
    ap_dev_t *(*open)(
        void *dev_id,     /* identifier of device to be used */
        ap_dev_cbs_t *cb, /* callbacks to use */
        void *context);   /* context to be passed when calling callbacks */

    /* Closes a wpa_ap_if_t object */
    void (*close)(ap_dev_t *t);

    /* Sets a default key */
    void (*set_default_key)(
        ap_dev_t *t,                       /* object context */
        int index,                         /* index in default key array */
        wpa_cipher_t cipher,               /* type of key */
        wpa_temporal_key_t *temporal_key); /* key data */

    /* Sets the index for the default key to be used */
    void (*set_default_key_id)(
        ap_dev_t *t, /* object context */
        int index);  /* index of default key that should be used */

    /* Removes a default key */
    void (*remove_default_key)(
        ap_dev_t *t,          /* object context */
        int index,            /* index in default key array */
        wpa_cipher_t cipher); /* type of key */
    
    /* Sets a mapping key */
    void (*set_mapping_key)(
        ap_dev_t *t,                       /* object context */
        ether_mac_t mac,                   /* MAC address of station for which
                                            * this key should be used
                                            */
        wpa_cipher_t cipher,               /* type of key */
        wpa_temporal_key_t *temporal_key); /* key data */

    /* Removes a mapping key */
    void (*remove_mapping_key)(
        ap_dev_t *t,          /* object context */
        ether_mac_t mac,      /* MAC address of station whose key is to be 
                               * removed */
        wpa_cipher_t cipher); /* type of key */
    
    /* Disassociates a station */
    void (*disassoc_sta)(
        ap_dev_t *t,      /* object context */
        ether_mac_t mac); /* MAC address of station to disassociate */

    /* Deauthenticates a station */
    void (*deauth_sta)(
        ap_dev_t *t,      /* object context */
        ether_mac_t mac); /* MAC address of station to deauthenticate */
 
    /* Sets the AP's WPA information element to be included in beacons */
    void (*set_ap_wpa_info_elem)(
        ap_dev_t *t,       /* object context */
        u8 *wpa_info_elem, /* WPA information element the AP should use */
        int len);          /* length of WPA information element */

    /* Gets the TKIP sequence counter */
    u64 (*get_tsc)(
        ap_dev_t *t); /* object context */

    /* Blocks/unblocks all outgoing multicast and broadcast traffic */
    void (*block_multicast_traffic)(
        ap_dev_t *t, /* object context */
        int block);  /* 1 to enable blocking, 0 to disable it */

    /* Blocks/unblocks all outgoing non-EAPOL unicast traffic destined to a 
     * specific station
     */
    void (*block_unicast_traffic)(
        ap_dev_t *t,      /* object context */
        int block,        /* 1 to enable blocking, 0 to disable it */
        ether_mac_t mac); /* MAC address of station to which destined traffic 
                           * should be blocked 
                           */

    /* Gets the maximum number of mapping keys that can be installed 
     * simultaneously
     */
    int (*get_max_num_of_mapping_keys)(
               ap_dev_t *t); /* object context */

} ap_dev_ops_t;

/* Configurable parameters of the wpa_port_t object */
typedef struct {
    
    /* The PSK may be configured by entering it directly or by entering an 
     * ASCII pass-phrase from which it's generated
     */
    wpa_psk_param_t psk_param;
    union {
        char *ascii;
        wpa_psk_t hex;
    } psk;
    
    /* Type of WEP cipher to use for keys distributed to WEP-802.1X stations */
    wpa_cipher_t rekeying_wep_cipher;

    /* Types of stations allowed to associate to the AP */
    wpa_sta_type_t allowed_sta_types;

    /* Time between group key updates (in milliseconds) */
    u32 gtk_update_interval;

    /* Preconfigured keys to use with WEP-legacy stations */
    wpa_wep_key_t wep_keys[WPA_NUM_DEFAULT_KEYS];

    /* Index of TX key to use with WEP-legacy stations */
    int wep_key_index;

    /* AP's SSID */
    u8 *ssid;
    int ssid_len;

    /* Points to something that identifies the underlying device */
    void *dev_id;      
                      
    /* Operations of underlying device */
    ap_dev_ops_t *ap_dev_ops; 

    /* Points to the auth1x_port object that the wpa_port object will use */
    auth1x_port_t *auth1x_port;

    /* MAC address of AP */
    ether_mac_t mac;
} wpa_port_params_t; 

typedef struct wpa_port_t wpa_port_t;

/* Allocates and initializes a wpa_port_t object. */
wpa_port_t *wpa_port_open(wpa_port_params_t *params);

/* Closes a wpa_port_t object. */
void wpa_port_close(wpa_port_t *t);

/* Reconfigures a wpa_port_t object. Returns 1 if the reconfiguration requires
 * reopening the port, 0 otherwise
 */
int wpa_port_reconf(wpa_port_t *t, wpa_port_params_t *params);

#endif
