/* LICENSE_CODE JUNGO */

#include <rg_config.h>
#include <kos/knet.h>
#include <kos/knet_utils.h>
#include <kos/knet_if_ext.h>
#ifdef CONFIG_RG_TARGET_VXWORKS
#include <util/vx_net_fixup.h>
#endif

typedef struct knet_dev_list_t {
    struct knet_dev_list_t *next;
    knet_netdev_t *dev;
} knet_dev_list_t;

typedef struct knet_hook_list_t {
    struct knet_hook_list_t *next;
    int dev_test;
    knet_dev_list_t *dev_list;
    knet_hook_handler_t knet_hook;
} knet_hook_list_t;

typedef struct {
    knet_pri_t priority;
    knet_hook_list_t *hooks;
} knet_pri_hooks_t;

static knet_pri_hooks_t knet_rx_hooks[KNET_NUM_HOOK_TYPES] = {
    { KNET_PRI_COUNT, NULL },
    { KNET_PRI_AUTH1X, NULL },
    { KNET_PRI_MULTI_WAN, NULL },
    { KNET_PRI_FIREWALL, NULL },
    { KNET_PRI_BRIDGE, NULL },
    { KNET_PRI_FASTPATH, NULL },
    { KNET_PRI_DHCP, NULL },
    { KNET_PRI_ROUTE, NULL },
};

static knet_pri_hooks_t knet_tx_hooks[KNET_NUM_HOOK_TYPES] = {
    { KNET_PRI_ROUTE, NULL },
    { KNET_PRI_BRIDGE, NULL },
    { KNET_PRI_MULTI_WAN, NULL },
    { KNET_PRI_FIREWALL, NULL },
    { KNET_PRI_AUTH1X, NULL },
    { KNET_PRI_COUNT, NULL },
};

/* net_rx_hook is called from within net_bh() in Linux.
 * net_tx_hook is called from within dev_queue_xmit() in Linux.
 * In VxWorks both are called from within the network device driver.
 */


#ifndef CONFIG_RG_TARGET_LINUX
knet_hook_t net_rx_hook;
knet_hook_t net_tx_hook;
#endif

char *knet_ntoa(in_addr_t addr)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return in_ntoa(addr.s_addr);
#else
    /* We cannot use the VX implementation for inet_ntoa because it allocates
     * a buffer every time it is called, and no one frees it.
     */

    static char ip_str[INET_ADDR_LEN];
    
    inet_ntoa_b(addr, ip_str);
    return ip_str;
#endif
}

/* Assumes address is in network byte order */
int knet_aton(char *cp, struct in_addr *inp)
{
#ifdef CONFIG_RG_TARGET_LINUX
    unsigned long addr = 0;
    unsigned int val;
    int i, j;

    for (i = 0; i < 4 && *cp; i++) 
    {
	addr <<= 8;
	val = 0;
	for (j=0; *cp != '\0' && *cp != '.'; cp++, j++) 
	{
	    if (!is_digit(*cp) || j>2)
		return -1;
	    val *= 10;
	    val += *cp - '0';
	}
	if (val>255)
	    return -1;
	addr |= val;
	if (*cp)
	    cp++;
    }
    inp->s_addr = (htonl(addr));
    return 0;
#else
    return inet_aton(cp, inp);
#endif
}

#ifdef CONFIG_RG_TARGET_LINUX
/* Copy buf to a new allocated buf which has bytes_added more room.
 * Returns a pointer to the new buf.
 * Code is based on skb_realloc_headroom(...), but we want more space in the
 * tail. */
static knet_buf_t *knet_skbuff_realloc(knet_buf_t *skb, int bytes_added)
{
    struct sk_buff *new_skb = NULL;
    int headroom = skb_headroom(skb);

#ifdef CONFIG_RG_OS_LINUX_22
    unsigned long offset;

    /* Allocate the copy buffer */
    new_skb = alloc_skb(skb->truesize + bytes_added, GFP_ATOMIC);
    if(!new_skb)
	return NULL;
    /* copy data */
    memcpy(new_skb->head, skb->head, skb->len + headroom);
    skb_reserve(new_skb, headroom);
    /* Shift between the two data areas in bytes */
    offset = new_skb->data - skb->data;
    /* Set the tail pointer and length */
    skb_put(new_skb,skb->len);
    new_skb->list = NULL;
    new_skb->sk = NULL;
    new_skb->priority = skb->priority;
    new_skb->protocol = skb->protocol;
    new_skb->dev = skb->dev;
    new_skb->dst = dst_clone(skb->dst);
    new_skb->h.raw = skb->h.raw ? skb->h.raw+offset : 0;
    new_skb->nh.raw = skb->nh.raw+offset;
    new_skb->mac.raw = skb->mac.raw+offset;
    memcpy(new_skb->cb, skb->cb, sizeof(skb->cb));
    new_skb->used = skb->used;
    new_skb->is_clone = 0;
    atomic_set(&new_skb->users, 1);
    new_skb->pkt_type = skb->pkt_type;
    new_skb->stamp = skb->stamp;
    new_skb->destructor = NULL;
#ifdef CONFIG_ARCH_IXP22X
    new_skb->vx2lin_destructor = 0;
#endif
    new_skb->net_pool.context = NULL;
    new_skb->net_pool.free_func = NULL;
    new_skb->security = skb->security;
#ifdef CONFIG_IP_FIREWALL
    new_skb->fwmark = skb->fwmark;
#endif
#else
    new_skb = skb_copy_expand(skb, headroom, bytes_added, GFP_ATOMIC);
    memcpy(new_skb->head, skb->head, headroom);
#endif
    return new_skb;
}
#endif

/* Place holders to save old values */
static knet_hook_t save_net_rx_hook;
static knet_hook_t save_net_tx_hook;

static void knet_hook_table_uninit(int is_rx);
    
static knet_pri_hooks_t *knet_pri_hooks_find(int is_rx, knet_pri_t priority)
{
    knet_pri_hooks_t *pri_hooks_table = is_rx ? knet_rx_hooks : knet_tx_hooks;
    int i;

    for (i = 0; i < KNET_NUM_HOOK_TYPES; i++)
    {
	if (pri_hooks_table[i].priority == priority)
	    return &pri_hooks_table[i];
    }

    return NULL;
}

static int knet_buf_hook_register(knet_hook_handler_t handler,
    knet_pri_t priority, int is_rx)
{
    knet_pri_hooks_t *pri_hooks;
    knet_hook_list_t **hook_list_ptr, *new_hook = NULL;
    kos_interrupt_flags_t flags;

    if (!(pri_hooks = knet_pri_hooks_find(is_rx, priority)))
	KOS_PANIC(__FUNCTION__ ": Wrong priority");

    if (!(new_hook = kos_alloc(sizeof(knet_hook_list_t), KOS_ALLOC_KERNEL)))
	KOS_PANIC(__FUNCTION__ ": Failed memory allocation");
        
    new_hook->next = NULL;
    new_hook->knet_hook = handler;
    new_hook->dev_test = 0;
    new_hook->dev_list = NULL;
    
    flags = kos_enter_protected();
    
    for (hook_list_ptr = &pri_hooks->hooks; *hook_list_ptr;
	hook_list_ptr = &(*hook_list_ptr)->next);

    *hook_list_ptr = new_hook;
    
    kos_exit_protected(flags);

    kos_mod_count_inc();

    return 0;
}

static knet_hook_list_t **knet_hook_find(knet_hook_list_t **head, 
	knet_hook_handler_t handler)
{
    knet_hook_list_t **hook_list_ptr = head;

    for (; *hook_list_ptr && (*hook_list_ptr)->knet_hook != handler;
	    hook_list_ptr = &(*hook_list_ptr)->next);

    return hook_list_ptr;
}

static void knet_buf_hook_unregister(knet_hook_handler_t handler,
    knet_pri_t priority, int is_rx)
{
    knet_pri_hooks_t *pri_hooks;
    knet_hook_list_t **hook_list_ptr, *tmp;
    kos_interrupt_flags_t flags;

    if (!(pri_hooks = knet_pri_hooks_find(is_rx, priority)))
	return;

    flags = kos_enter_protected();
    hook_list_ptr = knet_hook_find(&pri_hooks->hooks, handler);

    if (!*hook_list_ptr)
    {
	kos_exit_protected(flags);
	return;
    }

    tmp = *hook_list_ptr;
    *hook_list_ptr = (*hook_list_ptr)->next;
    kos_exit_protected(flags);
    kos_free(tmp);

    kos_mod_count_dec();
}

int knet_buf_rx_hook_register(knet_hook_handler_t rx_handler,
    knet_pri_t priority)
{
    return knet_buf_hook_register(rx_handler, priority, 1);
}

int knet_buf_tx_hook_register(knet_hook_handler_t tx_handler,
    knet_pri_t priority)
{
    return knet_buf_hook_register(tx_handler, priority, 0);
}

void knet_buf_rx_hook_unregister(knet_hook_handler_t rx_handler,
    knet_pri_t priority)
{
    knet_buf_hook_unregister(rx_handler, priority, 1);
}

void knet_buf_tx_hook_unregister(knet_hook_handler_t tx_handler,
    knet_pri_t priority)
{
    knet_buf_hook_unregister(tx_handler, priority, 0);
}

static knet_hook_list_t *pri_hook_get(knet_hook_handler_t handler,
    knet_pri_t priority, int is_rx)
{
    knet_pri_hooks_t *pri_hooks;

    if (!(pri_hooks = knet_pri_hooks_find(is_rx, priority)))
	return NULL;

    return *knet_hook_find(&pri_hooks->hooks, handler);
}

int knet_buf_hook_dev_add(knet_hook_handler_t handler, char *dev,
	knet_pri_t priority, int is_rx)
{
    knet_hook_list_t *hook_list_ptr;
    knet_dev_list_t **dev_list, *new_dev;
    knet_netdev_t *netdev;

    hook_list_ptr = pri_hook_get(handler, priority, is_rx);
    if (!hook_list_ptr)
	return -1;

    netdev = knet_netdev_get_by_name(dev);
    if (!netdev)
	return -1;
    
    for (dev_list = &hook_list_ptr->dev_list;
	 *dev_list && (*dev_list)->dev != netdev; 
	 dev_list = &(*dev_list)->next);
    
    if (*dev_list)
	return -1;

    if (!(new_dev = kos_alloc(sizeof(knet_dev_list_t), KOS_ALLOC_KERNEL)))
	return -1;

    new_dev->dev = netdev;
    new_dev->next = NULL;
    *dev_list = new_dev;
    return 0;
}

int knet_buf_hook_dev_del(knet_hook_handler_t handler, char *dev,
    knet_pri_t priority, int is_rx)
{
    knet_hook_list_t *hook_list_ptr;
    knet_dev_list_t **dev_list, *del_dev;
    knet_netdev_t *netdev;

    hook_list_ptr = pri_hook_get(handler, priority, is_rx);
    if (!hook_list_ptr)
	return -1;

    netdev = knet_netdev_get_by_name(dev);
    if (!netdev)
	return -1;
    
    for (dev_list = &hook_list_ptr->dev_list;
	*dev_list && (*dev_list)->dev != netdev; 
	dev_list = &(*dev_list)->next);
    
    if (!*dev_list)
	return -1;

    del_dev = *dev_list;
    *dev_list = (*dev_list)->next;
    kos_free(del_dev);
    
    return 0;
}

int knet_buf_hook_dev_clear(knet_hook_handler_t handler, knet_pri_t priority,
    int is_rx)
{
    knet_hook_list_t *hook_list_ptr;
    knet_dev_list_t *dev_list;

    hook_list_ptr = pri_hook_get(handler, priority, is_rx);
    if (!hook_list_ptr)
	return -1;

    hook_list_ptr->dev_test = 0;
    dev_list = hook_list_ptr->dev_list;
    hook_list_ptr->dev_list = NULL;

    while (dev_list)
    {
	knet_dev_list_t *tmp = dev_list;

	dev_list = tmp->next;
	kos_free(tmp);
    }

    return 0;
}

int knet_buf_hook_dev_set(knet_hook_handler_t handler, int val,
    knet_pri_t priority, int is_rx)
{
    knet_hook_list_t *hook_list_ptr;

    hook_list_ptr = pri_hook_get(handler, priority, is_rx);
    if (!hook_list_ptr)
	return -1;

    hook_list_ptr->dev_test = val;
    return 0;
}

int knet_buf_rx_hook_dev_add(knet_hook_handler_t handler,
    char *dev, knet_pri_t priority)
{
    return knet_buf_hook_dev_add(handler, dev, priority, 1);
}

int knet_buf_tx_hook_dev_add(knet_hook_handler_t handler, char *dev,
    knet_pri_t priority)
{
    return knet_buf_hook_dev_add(handler, dev, priority, 0);
}

int knet_buf_rx_hook_dev_del(knet_hook_handler_t handler,
    char *dev, knet_pri_t priority)
{
    return knet_buf_hook_dev_del(handler, dev, priority, 1);
}

int knet_buf_tx_hook_dev_del(knet_hook_handler_t handler, char *dev,
    knet_pri_t priority)
{
    return knet_buf_hook_dev_del(handler, dev, priority, 0);
}

int knet_buf_rx_hook_dev_clear(knet_hook_handler_t handler, knet_pri_t priority)
{
    return knet_buf_hook_dev_clear(handler, priority, 1);
}

int knet_buf_tx_hook_dev_clear(knet_hook_handler_t handler, knet_pri_t priority)
{
    return knet_buf_hook_dev_clear(handler, priority, 0);
}

int knet_buf_rx_hook_dev_set(knet_hook_handler_t handler, int val,
    knet_pri_t priority)
{
    return knet_buf_hook_dev_set(handler, val, priority, 1);
}

int knet_buf_tx_hook_dev_set(knet_hook_handler_t handler, int val,
    knet_pri_t priority)
{
    return knet_buf_hook_dev_set(handler, val, priority, 0);
}

static knet_hook_ret_type_t knet_buf_handler(knet_ip_hdr_t *iph, 
    knet_netdev_t *dev, knet_buf_t **kbuf, int is_rx)
{
    knet_pri_hooks_t *pri_hooks_table = is_rx ? knet_rx_hooks : knet_tx_hooks;
    knet_hook_list_t *hook_list_item;
    knet_hook_ret_type_t hook_ret;
    int i;
    kos_interrupt_flags_t flags;

    /* XXX: Very unclean, but the following might be needed for hooks that
     * assume knet_rcvif is set (which is not always true, especially in
     * RX on VxWorks).
     */
    (*kbuf)->knet_rcvif = dev;

    flags = kos_enter_protected();
    for (i = 0; i < KNET_NUM_HOOK_TYPES; i++)
    {
	hook_ret = KNET_PKT_NOT_HANDLED;

	for (hook_list_item = pri_hooks_table[i].hooks; hook_list_item;
	    hook_list_item = hook_list_item->next)
	{
	    if (hook_list_item->dev_test)
	    {
		/* Check if device is in action list */
		knet_dev_list_t *dev_item = hook_list_item->dev_list;

		for (; dev_item; dev_item = dev_item->next)
		{
		    if (dev_item->dev == dev)
			break;
		}
		if (!dev_item)
		    continue;
	    }

	    hook_ret = hook_list_item->knet_hook(&iph, dev, kbuf);
	    /* Don't continue in this priority if handled or handled for
	     * this priority.
	     */
	    if (hook_ret != KNET_PKT_NOT_HANDLED)
		break;

	    /* XXX Bridge and CHWAN change (*kbuf)->knet_rcvif, and still 
	     * return KNET_PKT_NOT_HANDLED - Thus, we must update our pointer.
	     */
	    dev = (*kbuf)->knet_rcvif;
	}
	if (hook_ret == KNET_PKT_HANDLED)
	    break;
    }
    kos_exit_protected(flags);
    
    if (hook_ret == KNET_PKT_HANDLED)
	return KNET_PKT_HANDLED;
    return KNET_PKT_NOT_HANDLED;
}

static knet_hook_ret_type_t knet_buf_rx(knet_ip_hdr_t *iph,
    knet_netdev_t *dev, knet_buf_t **kbuf)
{
#ifdef CONFIG_RG_TARGET_VXWORKS
    /* VxWorks drivers do not bother setting iph to really point to IP header.
     * We set it here under the assumption that knet_data points to IP header.
     */
    iph = (knet_ip_hdr_t *)(*kbuf)->knet_data;
#endif
    return knet_buf_handler(iph, dev, kbuf, 1);
}

static knet_hook_ret_type_t knet_buf_tx(knet_ip_hdr_t *iph,
    knet_netdev_t *dev, knet_buf_t **kbuf)
{
#ifdef CONFIG_RG_TARGET_VXWORKS
    /* VxWorks drivers do not bother setting iph to really point to IP header.
     * We set it here under the assumption that knet_data points to Ethernet
     * header.
     */
    iph = (knet_ip_hdr_t *)((*kbuf)->knet_data + dev->knet_netdev_hw_hdrlen);
#endif
    return knet_buf_handler(iph, dev, kbuf, 0);
}

/* register knet_buf_rx and knet_buf_tx */
int knet_init(void)
{
    save_net_rx_hook = net_rx_hook;
    save_net_tx_hook = net_tx_hook;
    net_rx_hook = knet_buf_rx;
    net_tx_hook = knet_buf_tx;
    knet_buf_rx_hook_register(knet_stats_count_rx, KNET_PRI_COUNT);
    knet_buf_tx_hook_register(knet_stats_count_tx, KNET_PRI_COUNT);
    /* Decrease the MOD_USE_COUNT when registering hooks internally */
    kos_mod_count_dec();
    kos_mod_count_dec();
    knet_if_ext_init();
    return 0;
}

/* unregister knet_buf_rx and knet_buf_tx */
void knet_uninit(void)
{
    net_rx_hook = save_net_rx_hook;
    net_tx_hook = save_net_tx_hook;
    
    knet_hook_table_uninit(0);
    knet_hook_table_uninit(1);
    knet_if_ext_uninit();
}

static void knet_hook_table_uninit(int is_rx)
{
    int i;
    knet_hook_list_t *tmp;
    knet_pri_hooks_t *pri_hooks_table = is_rx ? knet_rx_hooks : knet_tx_hooks;
    
    for (i = 0; i < KNET_NUM_HOOK_TYPES; i++)
    {
	while (pri_hooks_table[i].hooks)
	{
	    tmp = pri_hooks_table[i].hooks;
	    pri_hooks_table[i].hooks = pri_hooks_table[i].hooks->next;
	    kos_free(tmp);
	}
    }
}

u32 knet_get_u32(void *p)
{
    u32 result;

    memcpy(&result, p, sizeof(u32));
    return result;
}

void knet_pkt_type_set(knet_buf_t *buf, int val)
{
#ifdef CONFIG_RG_TARGET_LINUX
    buf->knet_pkt_type = val;
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    switch (val) 
    {
    case KNET_MULTICAST:
    case KNET_BROADCAST:
	buf->knet_pkt_type = (buf->knet_pkt_type &
		~(KNET_MULTICAST | KNET_BROADCAST)) | val;
	break;
    }
#endif
}

knet_buf_t *knet_buf_alloc(int size, int flags)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return dev_alloc_skb(size);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    int vx_flags = M_DONTWAIT;
    knet_buf_t *m = NULL;
    if (flags & KNET_WAIT)
	vx_flags = M_WAIT;
    /* bestFit equals to TRUE for this call, which means that the cluster
     * of the same size or greater will be granted, or NULL will be returned.
     */
    m = netTupleGet(_pNetDpool, size, vx_flags, MT_DATA, TRUE);
    if (m && flags & KNET_FIRST_IN_CHAIN)
	m->m_flags |= M_PKTHDR;
    return m;
#endif
}

void knet_buf_free(knet_buf_t *buf)
{
#ifdef CONFIG_RG_TARGET_LINUX
    kfree_skb(buf);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    m_freem(buf);
#endif
}

struct in_addr knet_buf_ip_get(knet_buf_t *buf, int is_src)
{
    struct in_addr ip;
    knet_ip_hdr_t *iph;
    knet_eth_hdr_t *eh;
    u8 *nh;
    u16 proto;

    eh = knet_mtod(buf, knet_eth_hdr_t*);
    proto = ntohs(eh->ether_type);
    nh = buf->knet_data + KNET_ETHER_HDR_LEN;

    ip.s_addr = 0;

    switch (proto)
    {
    case KNET_ETH_TYPE_ARP:
    case KNET_ETH_TYPE_RARP:
	nh += is_src ? KNET_ARP_SIP_OFFSET : KNET_ARP_DIP_OFFSET;
	memcpy(&ip, nh, sizeof(struct in_addr));
	break;
    case KNET_ETH_TYPE_IP:
	iph = (knet_ip_hdr_t *)nh;
	if (is_src)
	    ip = iph->ip_src;
	else
	    ip = iph->ip_dst;
	break;
    }

    return ip;
}

knet_buf_t *knet_pullup(knet_buf_t *buf, int size)
{
#ifdef CONFIG_RG_TARGET_LINUX
    if (size <= buf->knet_buflen)
	return buf;
    knet_buf_free(buf);
    return 0;
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    return m_pullup(buf, size);
#endif
}

knet_buf_t *knet_buf_cow(knet_buf_t *buf)
{
#ifdef CONFIG_RG_TARGET_LINUX
    /* if buf is shared - clone it */
    buf=skb_share_check(buf, GFP_ATOMIC);
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0))
    return skb_cow(buf, skb_headroom(buf));
#else
    if (skb_cow(buf, skb_headroom(buf)))
    {
	knet_buf_free(buf);
	return NULL;
    }
    return buf;
#endif
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    knet_buf_t *ret_buf;

    /* cluster is not shared, return as is */
    if (buf->m_extRefCnt == 1)
	return buf;

    /* create private copy of buffer */
    ret_buf = knet_buf_dup(buf, KNET_DONTWAIT);

    /* free old buffer */
    knet_buf_free(buf);
    return ret_buf;
#endif
}

int knet_buf_copydata(knet_buf_t *buf, u8 *dst, int offset, int len)
{
    if (len + offset > buf->knet_pktlen)
	return -1;
#ifdef CONFIG_RG_TARGET_LINUX
    memcpy(dst, &buf->knet_data[offset], len);
    
    return len;
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    return netMblkOffsetToBufCopy(buf, offset, dst, len, NULL);
#endif
}

int knet_buf_copy_to_user(knet_buf_t *buf, u8 *dst, int offset, int len)
{
#ifdef CONFIG_RG_TARGET_LINUX
    if (copy_to_user(dst, &buf->knet_data[offset], len))
	return 0;
    return len;
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    return netMblkOffsetToBufCopy(buf, offset, dst, len, NULL);
#endif
}

u8 *knet_buf_data_push(knet_buf_t *buf, u32 len)    
{
#ifdef CONFIG_RG_TARGET_LINUX
    return skb_push(buf, len);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    if (M_LEADINGSPACE(buf) >= len)
    {
	buf->knet_data -= len;
	buf->knet_buflen += len;
	if (knet_is_first(buf))
	    buf->knet_pktlen += len;
    }
    else
    {
	KOS_PANIC("knet_buf_data_push: not enough space to push %d bytes",
		len);
    }
    return buf->knet_data;
#endif
}

knet_buf_t *knet_buf_cow_align(knet_buf_t *buf)
{
#ifdef CONFIG_RG_TARGET_LINUX
    knet_buf_t *result;
    u32 data_pos = (u32) buf->knet_data;
    int data_shift = data_pos - (data_pos & ~0xf);
    
    knet_buf_data_push(buf, data_shift); /* buf must have enough headroom */
    result = knet_buf_cow(buf);
    if (buf != result)
	knet_buf_data_pull(buf, data_shift);
    if (result)
	knet_buf_data_pull(result, data_shift);
    return result;
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    /* knet_buf_cow() for VxWorks does require aligned data,
     * it copies the whole cluster from it's aligend beginning */
    return knet_buf_cow(buf);
#endif
}

void knet_buf_data_adj(knet_buf_t *buf, int len)    
{
#ifdef CONFIG_RG_TARGET_LINUX
    if (len<0)
	skb_trim(buf, buf->len+len);
    if (len>0)
	skb_pull(buf, len);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    m_adj(buf, len);
#endif
}

u32 knet_buf_copyback(knet_buf_t **pbuf, u8 *src, u32 offset, u32 len)
{
#ifdef CONFIG_RG_TARGET_LINUX
/* Implemented under the assumption that skb data is linear! (linux-2.2) */ 

    knet_buf_t *buf = *pbuf;
    /* Calculate buffer sizes */
    unsigned char *new_tail = buf->data + offset + len;
    int inc = new_tail - buf->tail;
    int extra_len = inc - skb_tailroom(buf);

    /* Increase buffer, if needed */
    if (extra_len > 0)
    {
	knet_buf_t *tmp = knet_skbuff_realloc(buf, extra_len);
	if (!tmp)
	    return 0;

	knet_buf_free(buf);
	buf = *pbuf = tmp;
    }
   
    /* Increase data area, if needed */
    if (inc > 0)
	skb_put(buf, inc);

    /* Copy buffer */
    memcpy(buf->data + offset, src, len);
    
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    int mlen;
    knet_buf_t *m = *pbuf, *n;
    int totlen = 0, save_len = len;
    int ret = -1;

    /* XXX: this code was taken from BSD, we're not sure its fully working in
     * case of long length buffers.
     */
    if (!*pbuf || !knet_is_first(*pbuf))
	return 0;

    /* find the mbuf node in the chain that contains the offset position, 
     * extend the chain if needed
     */
    for (; m->knet_nextbuf && offset > (mlen = m->knet_buflen); 
	m = m->knet_nextbuf)
    {
	offset -= mlen;
	totlen += mlen;
    }

    if (offset > m->knet_buflen)
    {
	n = netMblkGet(_pNetDpool, M_DONTWAIT, m->m_type);
	if (!n)
	    goto Exit;
	if (OK != netMblkClGet(_pNetDpool, n, len + offset, M_DONTWAIT, TRUE))
	    goto Exit;
	n->knet_buflen = len + offset;
	m = m->knet_nextbuf = n;
    }
    
    /* copy the data to the chain, starting at the node found in the previous
     * loop, extending chain if needed.
     */
    while (len > 0)
    {
	mlen = min(m->knet_buflen - offset, len);
	memcpy(knet_mtod(m, u8*) + offset, src, mlen);
	src += mlen;
	len -= mlen;
	mlen += offset;
	offset = 0;
	totlen += mlen;
	if (!len)
	    break;
	if (!m->knet_nextbuf)
	{
	    n = netMblkGet(_pNetDpool, M_DONTWAIT, m->m_type);
	    if (!n)
		goto Exit;
	    if (OK != netMblkClGet(_pNetDpool, n, len, M_DONTWAIT, TRUE))
		goto Exit;
	    n->knet_buflen = len;
	    m->knet_nextbuf = n;
	}
	m = m->knet_nextbuf;
    }
    
    ret = 0;
    len = save_len;
    
Exit:
    if ((*pbuf)->m_pkthdr.len < totlen)
	(*pbuf)->m_pkthdr.len = totlen;
    if (ret < 0)
	return 0;
#endif

    return len;
}

knet_buf_t *knet_buf_clone(knet_buf_t *m, int flags)
{
#ifdef CONFIG_RG_TARGET_LINUX
    int linux_flags = GFP_ATOMIC;
    if (flags & KNET_WAIT)
	linux_flags = GFP_KERNEL;
    return skb_clone(m, linux_flags);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    int vx_flags = M_DONTWAIT;
    if (flags & KNET_WAIT)
	vx_flags = M_WAIT;
    return m_copym(m, 0, M_COPYALL, vx_flags);
#endif
}

knet_buf_t *knet_buf_dup(knet_buf_t *m, int flags)
{
#ifdef CONFIG_RG_TARGET_LINUX
    int linux_flags = GFP_ATOMIC;
    if (flags & KNET_WAIT)
	linux_flags = GFP_KERNEL;
    return skb_copy(m, linux_flags);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    knet_buf_t *ret_buf = NULL, *tail_buf = NULL, *cur_buf = NULL;
    int len;

    while (m)
    {
	len = m->knet_buflen; /* current buffer length */
	
	/* Get block from pool and attach a cluster to the block.
	 * We get the size we really need and not the size of the 
	 * original to optimize pool usage */
	if (!(cur_buf = knet_buf_alloc(len + M_LEADINGSPACE(m),
	    flags | (ret_buf ? 0 : KNET_FIRST_IN_CHAIN))))
	{
	    goto Error;
	}

	cur_buf->knet_buflen = len;
	cur_buf->knet_nextbuf = NULL;

	/* copy data to cluster from start of buffer to end of actual data */
	memcpy(cur_buf->knet_head, m->knet_head, len + M_LEADINGSPACE(m));

	/* Set data pointer of copy at the same offset as in original */
	(u8 *)cur_buf->knet_data += M_LEADINGSPACE(m);
	
	/* Set data relevant only for first block in chain */
	if (!ret_buf)
	{
	    cur_buf->knet_rcvif = m->knet_rcvif;
	    cur_buf->knet_pktlen = m->knet_pktlen; /* total length of packet */
	    ret_buf = cur_buf;
	}

	/* Add cur_buf to list */
	if (tail_buf)
	    tail_buf->knet_nextbuf = cur_buf;
	tail_buf = cur_buf;

	m = m->knet_nextbuf;
    }
    return ret_buf;
Error:
    /* Free data already allocated */
    if (ret_buf)
	knet_buf_free(ret_buf);
    return NULL;
#endif
}

int knet_buf_merge(knet_buf_t **buf, knet_buf_t *buf2)
{
#ifdef CONFIG_RG_TARGET_LINUX
    /* We copy ALL the data, from head to tail so that we won't lose the MAC
     * headers that reside after the data pointer.
     */
    knet_buf_t *resbuf = buf2;
    knet_buf_t *buf1 = *buf;
    int size = buf1->tail - buf1->head;
    int headroom = skb_headroom(buf2);
    
    /* If we don't have enough space, allocate it! */
    if (headroom < size)
    {
	resbuf = skb_realloc_headroom(buf2, size);
	if (!resbuf)
	    return -1;
	kfree_skb(buf2);
    }

    /* Place the first buffer at the start of the second */
    skb_push(resbuf, size);
    memcpy(resbuf->data, buf1->head, size);

    /* Update packet data fields */
    resbuf->mac.raw = resbuf->h.raw = resbuf->nh.raw = resbuf->data;

    resbuf->mac.raw += buf1->mac.raw - buf1->head;
    resbuf->h.raw += buf1->h.raw - buf1->head;
    resbuf->nh.raw += buf1->nh.raw - buf1->head;

    /* Make sure skb->data points to IP layer and not MAC layer */
    skb_pull(resbuf, resbuf->nh.raw - resbuf->data);
    
    kfree_skb(*buf);
    *buf = resbuf;
    return 0;
    
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    /* This is the OpenBSD inspiration. Seems to work in VxWorks. */
    m_cat(*buf, buf2);
    return 0;
#endif
}

u8 *knet_buf_data_put(knet_buf_t *buf, u32 len)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return skb_put(buf, len);
#elif defined (CONFIG_RG_TARGET_VXWORKS)
    int plen = buf->knet_pktlen;
    
    buf->knet_pktlen += len;
    buf->knet_buflen += len;
    return buf->knet_data + plen;
#endif
}

void knet_buf_data_reserve(knet_buf_t *buf, u32 len)
{
#ifdef CONFIG_RG_TARGET_LINUX
    skb_reserve(buf, len);
#elif defined (CONFIG_RG_TARGET_VXWORKS)
    /* not required for VxWorks. */
#endif
}

u8 *knet_buf_data_pull(knet_buf_t *buf, u32 len)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return skb_pull(buf, len);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    /* len is always positive, thus m_adj works exactly as skb_pull. */
    if (len > buf->knet_pktlen)
	return NULL;
    m_adj(buf, len);
    return buf->knet_data;
#endif
}

u16 knet_ip_compute_csum(void *buf, int buf_hdrlen)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return ip_compute_csum(buf, buf_hdrlen);
#else
    int len = buf_hdrlen;
    u32 sum; 
    u16 *addr = (u16 *)buf;

    for (sum = 0; len > 1; len -= 2)
	sum += *addr++;
    /* mop up an odd byte, if necessary */
    if (len == 1)
	sum += *(u8 *)addr;

    /* add back carry outs from top 16 bits to low 16 bits
     */
    sum = (sum >> 16) + (sum & 0xffff);	/* add hi 16 to low 16 */
    sum += (sum >> 16);	/* add carry */

    return (u16)(~sum);
#endif
}

u16 knet_fix_csum_16(u16 csum, u16 old, u16 new, u8 is_udp)
{
    u32 l;

    if (is_udp && !csum)
	return 0;
    l = csum + old - new;
    l = (l >> 16) + (l & 0xffff);
    l = l & 0xffff;
    if (is_udp && !l)
	return 0xffff;
    return l;
}

int knet_netdev_register(knet_netdev_t *dev)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return register_netdev(dev);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    ether_attach(dev, dev->if_unit, dev->if_name, dev->if_init, dev->if_ioctl,
	dev->if_output, dev->if_reset);
    netdev_addr_init(dev);
    return 0;
#endif
}

void knet_netdev_unregister(knet_netdev_t *dev)
{
    kos_interrupt_flags_t flags = kos_save_flags_cli();
#ifdef CONFIG_RG_TARGET_LINUX
    unregister_netdev(dev);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
extern void if_dettach(struct ifnet *);
    if_dettach(dev);
    device_detach_vx_fix(dev);
#endif
    kos_restore_flags(flags);
}

knet_netdev_t *knet_netdev_get_by_name(char *name)
{
#ifdef CONFIG_RG_TARGET_LINUX
#ifdef KNET_LINUX_24
    return __dev_get_by_name(name);
#else
    return dev_get(name);
#endif
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    return ifunit(name);
#endif
}

knet_netdev_t *knet_netdev_get_by_index(int if_idx)
{
#ifdef CONFIG_RG_TARGET_LINUX
#ifdef KNET_LINUX_24
    return __dev_get_by_index(if_idx);
#else
    return dev_get_by_index(if_idx);
#endif
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    knet_netdev_t *dev;

    for (dev = ifnet; dev && dev->if_index != if_idx; dev = dev->if_next);
    return dev;
#endif
}

int knet_netdev_promisc_set(knet_netdev_t *dev, int flag)
{
#ifdef CONFIG_RG_TARGET_LINUX
    dev_change_flags(dev, flag ? dev->flags | IFF_PROMISC :
	dev->flags & ~IFF_PROMISC);
    return 0;

#elif defined(CONFIG_RG_TARGET_VXWORKS)
    struct ifreq ifr;
    int is_promisc, was_promisc = dev->if_pcount > 0;

    dev->if_pcount += flag ? 1 : -1;
    is_promisc = dev->if_pcount > 0;
    if (is_promisc == was_promisc)
	return 0;

    if (is_promisc)
	dev->if_flags |= IFF_PROMISC;
    else
	dev->if_flags &= ~IFF_PROMISC;

    ifr.ifr_flags = dev->if_flags;
    return dev->if_ioctl(dev, SIOCSIFFLAGS, (caddr_t)&ifr);
#endif
}

int knet_netdev_send(knet_netdev_t *dev, const char *data, int len,
    int is_data_from_user)
{
#ifdef CONFIG_RG_TARGET_LINUX
    struct sk_buff *skb = dev_alloc_skb(len + KNET_ETHER_ALIGN_FIX);
    if (!skb)
	return -1;

    skb_reserve(skb, KNET_ETHER_ALIGN_FIX);
    if (is_data_from_user)
    {
	if (copy_from_user(skb->data, data, len))
	{
	    kfree_skb(skb);
	    return -1;
	}
    }
    else 
	memcpy(skb->data, data, len);
    
    skb_put(skb, len);
    skb->dev = dev;
    skb->mac.ethernet = (struct ethhdr *)skb->knet_data;
    skb->protocol = skb->mac.ethernet->h_proto;
    skb->nh.iph = (struct iphdr *)(skb->knet_data + KNET_ETHER_HDR_LEN);

    return knet_netdev_queue_xmit(skb);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    return etherOutput(dev, (struct ether_header *)data,
	(char *)data + KNET_ETHER_HDR_LEN, len - KNET_ETHER_HDR_LEN);
#endif
}

int knet_netdev_receive(knet_buf_t *buf)
{
    /* Pull the ETH header towards IP stack */
    knet_buf_data_pull(buf, KNET_ETHER_HDR_LEN);    
#ifdef CONFIG_RG_TARGET_LINUX
#ifdef KNET_LINUX_24
    return netif_rx(buf);
#else
    netif_rx(buf);
    return 0;
#endif
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    if (net_rx_hook && net_rx_hook((knet_ip_hdr_t *)NULL, buf->knet_rcvif,
	&buf) != KNET_PKT_NOT_HANDLED)
    {
	return -1;
    }
    
    ether_input(buf->knet_rcvif,
	(struct ether_header *)(buf->knet_data - KNET_ETHER_HDR_LEN), buf);
    return 0;
#endif
}

void knet_netdev_fields_init(knet_netdev_t *dev)
{
#ifdef CONFIG_RG_TARGET_LINUX
    ether_setup(dev);
    memset(dev->dev_addr, 0, KNET_ETHER_ADDR_LEN);
    /* XXX Statistics not implemented yet */
    dev->get_stats = NULL;
    dev->set_multicast_list = NULL;
    dev->accept_fastpath = NULL;
    dev->tx_queue_len = 0;
#elif defined (CONFIG_RG_TARGET_VXWORKS)
    dev->if_output = ether_output;
#endif
}

knet_netdev_t *knet_netdev_zalloc(void)
{
    knet_netdev_t *tmp;
    int alloc_size = sizeof(KNET_IFDEF(knet_netdev_t, struct arpcom));
    
    tmp = (knet_netdev_t *)kos_alloc(alloc_size, KOS_ALLOC_KERNEL);
    return tmp ? (knet_netdev_t *)memset(tmp, 0, alloc_size) : NULL;
}

int knet_netdev_queue_stopped(knet_netdev_t *dev)
{
#ifdef CONFIG_RG_TARGET_LINUX
#ifdef KNET_LINUX_24
    return netif_queue_stopped(dev);
#else
    return dev->tbusy;
#endif
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    /* XXX: Check if this really does the job. */
    return dev->netdev_flags & IFF_OACTIVE;
#endif
}

#ifdef CONFIG_RG_TARGET_VXWORKS
void knet_netdev_stop_queue(knet_netdev_t *dev)
{
    knet_buf_t *buf;
    kos_interrupt_flags_t fl = kos_splnet();
    
    dev->netdev_flags |= IFF_OACTIVE;
    while ((buf = knet_netdev_dequeue(dev)))
	knet_buf_free(buf);

    kos_splx(fl);
}

void knet_netdev_start_queue(knet_netdev_t *dev)
{
    /* XXX: Check if this really does the job. */
    dev->netdev_flags &= ~IFF_OACTIVE;
}
#endif

int knet_netdev_queue_xmit(knet_buf_t *buf)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return dev_queue_xmit(buf);

#elif defined(CONFIG_RG_TARGET_VXWORKS)
    struct ifnet *dev = buf->m_pkthdr.rcvif;
    struct ifqueue *txq = &dev->if_snd;
    kos_interrupt_flags_t fl = kos_enter_protected();

    if (IF_QFULL(txq))
    {
	IF_DROP(txq);
	knet_buf_free(buf);
	return -1;
    }
    
    IF_ENQUEUE(txq, buf);
    kos_exit_protected(fl);
    
    if (dev->if_start && !knet_netdev_queue_stopped(dev))
	dev->if_start(dev);
    
    return 0;
#endif
}

#ifdef CONFIG_RG_TARGET_VXWORKS
/* dequeue packet from the device queue */
knet_buf_t *knet_netdev_dequeue(knet_netdev_t *dev)
{
    knet_buf_t *buf;
    kos_interrupt_flags_t flags = kos_save_flags_cli();
    
    IF_DEQUEUE(&dev->if_snd, buf);
    kos_restore_flags(flags);

    return buf;
}

int knet_netdev_separate_name(char *buf, char *name)
{
    int i;

    for (i = 0; name[i]; i++)
    {
	if (isdigit((int)name[i]))
	    break;
    }
    strncpy(buf, name, i);
    buf[i] = 0;
    return name[i] ? atoi(name + i) : 0;
}
#endif

void knet_netdev_copy_name(knet_netdev_t *dev, char *name)
{
#ifdef KNET_LINUX_24
    strcpy(dev->name, name);
#else
    char buf[IFNAMSIZ];
    int len;
#ifdef CONFIG_RG_TARGET_VXWORKS
    int index = knet_netdev_separate_name(buf, name);
#else
    strcpy(buf, name);
#endif
    
    len = strlen(buf) + 1;
    if (len > IFNAMSIZ)
	KOS_PANIC("knet: Device name too long!\n");

    dev->netdev_name = kos_alloc(len, KOS_ALLOC_KERNEL);
    if (!dev->netdev_name)
	KOS_PANIC("knet: Can't allocate device name!\n");
    memcpy(dev->netdev_name, buf, len);
#ifdef CONFIG_RG_TARGET_VXWORKS
    dev->if_unit = index;
#endif
#endif
}

void knet_netdev_free_name(knet_netdev_t *dev)
{
#ifndef KNET_LINUX_24
    if (dev->netdev_name)
	kos_free(dev->netdev_name);
    dev->netdev_name = NULL;
#endif
}

char *knet_netdev_name_get(knet_netdev_t *dev)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return dev->netdev_name;

#elif defined(CONFIG_RG_TARGET_VXWORKS)
    static char name[IFNAMSIZ];

    sprintf(name, "%s%hd", dev->netdev_name, dev->if_unit);
    return name;
#endif
}

struct in_ifaddr *knet_netdev_on_addrlist_loop(knet_netdev_t *dev,
    knet_addrlist_loop_cb_t cb, void *arg)
{
#ifdef CONFIG_RG_TARGET_LINUX
    struct in_device *in_dev = (struct in_device *)dev->ip_ptr;
    struct in_ifaddr *ifap;
    
    if (!in_dev)
	return NULL;

    for (ifap = in_dev->ifa_list; ifap; ifap = ifap->ifa_next)
    {
	if (cb(ifap, arg))
	    return ifap;
    }
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    struct ifaddr *ifap;

    for (ifap = dev->if_addrlist; ifap; ifap = ifap->ifa_next)
    {
	if (ifap->ifa_addr->sa_family == AF_INET &&
	    cb((struct in_ifaddr *)ifap, arg))
	{
	    return (struct in_ifaddr *)ifap;
	}
    }
#endif

    return NULL;
}

static int knet_netdev_ip_get_cb(struct in_ifaddr *addr, void *arg)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return !strcmp((char *)arg, addr->ifa_label);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    return addr->ia_addr.sin_addr.s_addr;
#endif
}

struct in_addr knet_netdev_ip_get(knet_netdev_t *dev, char *ifa_name)
{
    struct in_addr dev_ip;
    struct in_ifaddr *addr = knet_netdev_on_addrlist_loop(dev,
	knet_netdev_ip_get_cb, (void *)ifa_name);

    dev_ip.s_addr = 0;

    if (!addr)
	return dev_ip;

#ifdef CONFIG_RG_TARGET_LINUX
    dev_ip.s_addr = addr->ifa_local;
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    dev_ip = addr->ia_addr.sin_addr;
#endif
    
    return dev_ip;
}

static int knet_netdev_has_ip_cb(struct in_ifaddr *addr, void *arg)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return addr->ifa_local;
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    return knet_netdev_ip_get_cb(addr, NULL);
#endif
}

int knet_netdev_has_ip(knet_netdev_t *dev)
{
    return (int)knet_netdev_on_addrlist_loop(dev, knet_netdev_has_ip_cb, NULL);
}

static int knet_netdev_ip_cmp_cb(struct in_ifaddr *addr, void *arg)
{
    struct in_addr ip = *(struct in_addr *)arg;
    
#ifdef CONFIG_RG_TARGET_LINUX
    return addr->ifa_local == ip.s_addr;
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    return addr->ia_addr.sin_addr.s_addr == ip.s_addr;
#endif
}

int knet_is_devs_ip(knet_netdev_t *dev, struct in_addr ip)
{
    return (int)knet_netdev_on_addrlist_loop(dev, knet_netdev_ip_cmp_cb,
	(void *)&ip);
}

knet_netdev_t *knet_netdev_by_ip_get(struct in_addr ip)
{
    knet_netdev_t *dev;
    
    for (dev = KNET_NETDEV_HEAD; dev; dev = dev->netdev_next)
    {
	if (knet_netdev_on_addrlist_loop(dev, knet_netdev_ip_cmp_cb,
	    (void *)&ip))
	{
	    return dev;
	}
    }

    return NULL;
}

#ifdef CONFIG_RG_TARGET_VXWORKS
int knet_route_table_loop(short flags, knet_route_iter_cb_t cb, void *cb_arg)
{
    struct radix_node *base, *next;
    register struct radix_node *rn;
    /*
     * This gets complicated because we may delete the node
     * while applying the callback function to it, so we need to calculate
     * the successor node in advance. */

    /* Set rn to point to the head of the internet family routing table */
    if (!rt_tables[AF_INET] || !(rn = rt_tables[AF_INET]->rnh_treetop))
	return 0;

    /* Go left until finding the first leave. rn_b has always a negative
     * value for leaves */
    while (rn->rn_b >= 0)
	rn = rn->rn_l;
    do
    {
	base = rn;
	/* If at right child go back up, otherwise, go right */
	while (rn->rn_p->rn_r == rn && !(rn->rn_flags & RNF_ROOT))
	    rn = rn->rn_p;
	/* Find the next *leaf* since next node might vanish, too */
	for (rn = rn->rn_p->rn_r; rn->rn_b >= 0;)
	    rn = rn->rn_l;
	next = rn;
	/* Process leaves */
	while ((rn = base))
	{
	    base = rn->rn_dupedkey;
	    if (!(rn->rn_flags & RNF_ROOT))
	    {
		struct rtentry *rt = (struct rtentry *)rn;
		
		if ((rt->rt_flags & flags) == flags)
		{
		    if (cb(rt, cb_arg))
			return 1;
		}
	    }
	}
	rn = next;
    }
    while (!(rn->rn_flags & RNF_ROOT));

    return 0;
}
#endif
