/* LICENSE_CODE JUNGO */

/* Utility functions for knet_buf manipulation */

#include <kos/kos.h>
#include <kos/knet_utils.h>
#include "knet.h"

static int _knet_buf_find(knet_buf_t *buf, int start_offset, int end_offset, 
    u8 *what_to_find, u32 what_to_find_len, int is_case_sensetive)
{
    int found;
    u8 c; 

    end_offset = MIN(end_offset, buf->knet_pktlen - 1);
    for (found=0; found<what_to_find_len && start_offset <= end_offset && 
	knet_buf_copydata(buf, &c, start_offset, 1)==1;
	found++, start_offset++)
    {
	if ((is_case_sensetive && c!=what_to_find[found]) || 
	    to_lower(c)!=to_lower(what_to_find[found]))
	{
	    found = -1;
	}
    }
    return found==what_to_find_len ? start_offset-what_to_find_len : -1;
}

int knet_buf_find(knet_buf_t *buf, int start_offset, int end_offset, 
    u8 *what_to_find, u32 what_to_find_len)
{
    return _knet_buf_find(buf, start_offset, end_offset, what_to_find, 
	what_to_find_len, 1);
}

int knet_buf_find_case(knet_buf_t *buf, int start_offset, int end_offset, 
    u8 *what_to_find, u32 what_to_find_len)
{
    return _knet_buf_find(buf, start_offset, end_offset, what_to_find, 
	what_to_find_len, 0);
}

/* Replaces the segment of length 'old_segment_len' found at 
 * 'offset' of 'buf' with the segment of length 'new_segment_len'
 * pointed to by 'new_segment'. Buffer length will be adjusted if necessary. 
 * Returns 0 is successful, -1 otherwise. */
int knet_buf_replace(knet_buf_t **bufp, u32 offset, u32 old_segment_len, 
    u8 *new_segment, u32 new_segment_len)
{
    /* This implementation is supposed to work both on Linux and VxWorks */

    knet_buf_t *buf = *bufp;
    int inc = new_segment_len - old_segment_len; 
    u8 *moved_segment;
    u32 moved_segment_len = buf->knet_pktlen - offset - old_segment_len;
    u32 number_of_bytes_copied;

    /* If there is moving to do */
    if ((inc != 0) && (moved_segment_len > 0)) 
    {
	/* We're replacing a segment in the middle of the mbuf chain
	 * with a new segment which is not the same length. That
	 * means that we have to move the part that comes after the
	 * segment. */

	/* Allocate buffer to hold the data of the moved segment */
	moved_segment = kos_alloc(moved_segment_len, KOS_ALLOC_ATOMIC);

	if (!moved_segment)
	    return -1;

	/* Copy moved segment */
	knet_buf_copydata(buf, moved_segment, offset + old_segment_len, 
	    moved_segment_len);

	/* Reduce buffer if necessary */
	if (inc < 0)
	    knet_buf_data_adj(buf, inc);

	/* Put moved segment in the right place */
	number_of_bytes_copied = knet_buf_copyback(bufp, moved_segment, offset +
	    new_segment_len, moved_segment_len);

	kos_free(moved_segment);

	if (number_of_bytes_copied != moved_segment_len)
	    return -1;
    }

    /* Insert new segment, the buffer will be extended if necessary by 
     * knet_buf_copyback() */
    number_of_bytes_copied = knet_buf_copyback(bufp, new_segment, offset, 
	new_segment_len);

    if (number_of_bytes_copied != new_segment_len)
	return -1;

    return 0;
}

int knet_buf_ncmp(knet_buf_t *knet_buf, int offset, u8 *buf, u32 len)
{
    return knet_buf_find(knet_buf, offset, offset+len-1, buf, len)==-1;
}

int knet_buf_case_ncmp(knet_buf_t *knet_buf, u32 offset, u8 *buf, u32 len)
{
    return knet_buf_find_case(knet_buf, offset, offset+len-1, buf, len)==-1;
}
    
int knet_buf_pbrk(knet_buf_t *buf, u32 offset, char *delims)
{
    char c; 

    for (;knet_buf_copydata(buf, &c, offset, 1)==1; offset++)
    {
	if (strchr(delims, c))
	    return offset;
    }
    return -1;
}

int knet_buf_uint_read(knet_buf_t *knet_buf, u32 offset, int *bytes_read)
{
    int res = 0;
    char c;

    for (*bytes_read=0; 
	knet_buf_copydata(knet_buf, &c, offset + *bytes_read, 1)==1 &&
	is_digit(c); (*bytes_read)++)
    {
	res *= 10;
	res += c - '0';
    }
    return res;
}

int knet_buf_aton(knet_buf_t *buf, int start_offset, int end_offset,
    in_addr_t *addr)
{
    char addr_buf[DOTTED_IP_LEN];

    if (end_offset - start_offset > DOTTED_IP_LEN - 1)
	return -1;
    knet_buf_copydata(buf, addr_buf, start_offset, end_offset-start_offset);
    addr_buf[end_offset - start_offset] = 0;
    return knet_aton(addr_buf, addr);
}

/* count outgoing non-unicast device statistics */
knet_hook_ret_type_t knet_stats_count_tx(knet_ip_hdr_t **ip, knet_netdev_t *dev,
    knet_buf_t **kbuf)
{
#if defined(CONFIG_RG_OS_LINUX)
    struct net_device_stats *dev_stats;
    static char bcast_mac[] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};

    if (dev->netdev_ift != KNET_IFT_ETHER)
	return KNET_PKT_NOT_HANDLED; /* non-ethernet device */
    if (!dev->get_stats || !(dev_stats = dev->get_stats(dev)))
	return KNET_PKT_NOT_HANDLED; /* device doesn't keep statistics */

    (*kbuf)->mac.raw = (*kbuf)->knet_data;
    if (!((*kbuf)->mac.ethernet->h_dest[0] & 1))
	return KNET_PKT_NOT_HANDLED; /* unicast packet */
    if (!memcmp((*kbuf)->mac.ethernet->h_dest, dev->broadcast, dev->addr_len) ||
	!memcmp((*kbuf)->mac.ethernet->h_dest, bcast_mac, sizeof(bcast_mac)))
    {
	dev_stats->tx_bcasts++;
    }
    else
	dev_stats->tx_mcasts++;
#elif defined(CONFIG_RG_OS_VXWORKS)
/* XXX: knet_stats_count_tx not implemented yet */
#endif
    return KNET_PKT_NOT_HANDLED;
}

/* count non-unicast device statistics
 * multicast packets are supposed to be counted by the device but some devices
 * don't count it */
knet_hook_ret_type_t knet_stats_count_rx(knet_ip_hdr_t **ip, knet_netdev_t *dev,
    knet_buf_t **kbuf)
{
#if defined(CONFIG_RG_OS_LINUX)
    struct net_device_stats *dev_stats;

    if (!dev->get_stats || !(dev_stats = dev->get_stats(dev)))
	return KNET_PKT_NOT_HANDLED; /* device doesn't keep statistics */
    switch ((*kbuf)->pkt_type)
    {
    case PACKET_BROADCAST:
	dev_stats->rx_bcasts++;
	break;
    case PACKET_MULTICAST:
	dev_stats->rx_mcasts++;
	break;
    }
#elif defined(CONFIG_RG_OS_VXWORKS)
/* XXX: knet_stats_count_rx not implemented yet */
#endif
    return KNET_PKT_NOT_HANDLED;
}

#ifdef DEBUG_PRINT_BUFFER

#define NIPQUAD(addr) \
    ((unsigned char *)&addr)[0], \
    ((unsigned char *)&addr)[1], \
    ((unsigned char *)&addr)[2], \
    ((unsigned char *)&addr)[3]

static u16 knet_print_ip(knet_ip_hdr_t *ip, char *out)
{
    u32 addr;
    u16 off = ntohs(ip->ip_off);

    sprintf(out+strlen(out), "IP (");
    sprintf(out+strlen(out), "hl: %d, ver: %d, tos: %d, ", ip->ip_hl<<2,
	ip->ip_v, ip->ip_tos);
    sprintf(out+strlen(out), "len: %d, id: %x, ", ntohs(ip->ip_len),
	ntohs(ip->ip_id));
    sprintf(out+strlen(out), "off: %x %d%s%s, ", off, off&KNET_IP_OFFMASK,
	off&IP_MF ? " MF": "", off&IP_DF ? " DF" : "");
    sprintf(out+strlen(out), "ttl: %d, prot: %d, ", ip->ip_ttl, ip->ip_p);
    sprintf(out+strlen(out), "csum: %x, ",
	ntohs(ip->ip_sum)); /* XXX: Calculate check and print it */
    addr = ntohl(ip->ip_src.s_addr);
    sprintf(out+strlen(out), "src: %d.%d.%d.%d, ", NIPQUAD(addr));
    addr = ntohl(ip->ip_dst.s_addr);
    sprintf(out+strlen(out), "dst: %d.%d.%d.%d", NIPQUAD(addr));
    sprintf(out+strlen(out), ")");

    return ntohs(ip->ip_len);
}

static int knet_print_tcp(knet_tcp_hdr_t *tcp, char *out)
{
    static struct flag {
	char *flag;
	u8 value;
    } flags[] = {
	{"S", TH_SYN},
	{"F", TH_FIN},
	{"R", TH_RST},
	{"A", TH_ACK},
	{"U", TH_URG},
	{"P", TH_PUSH},
	{"E", TH_ECN},
	{"C", TH_CWR},
    };
    int i;
    u16 sport = ntohs(tcp->th_sport);
    u16 dport = ntohs(tcp->th_dport);

    sprintf(out+strlen(out), "TCP %p (sport: %d, dport: %d, ", tcp, sport,
	dport);
    sprintf(out+strlen(out), "seq: %x, ack: %x, res: %x, off: %d, ", 
	ntohl(tcp->th_seq), ntohl(tcp->th_ack), tcp->th_res, tcp->th_off<<2);
    sprintf(out+strlen(out), "flags: ");
    for (i=0; i < sizeof(flags)/sizeof(struct flag); i++)
    {
	if (flags[i].value & tcp->th_flags)
	    sprintf(out+strlen(out), "%s", flags[i].flag);
    }
    sprintf(out+strlen(out), ", winsize: %d, urg: %d, ", ntohs(tcp->th_win),
	tcp->th_urp);
    /* XXX: calculate csum and print it */
    sprintf(out+strlen(out), "csum: %x)", ntohs(tcp->th_sum));
    
    return 0;
}

static int knet_print_udp(knet_udp_hdr_t *udp, char *out)
{
    u16 sport = ntohs(udp->uh_sport);
    u16 dport = ntohs(udp->uh_dport);

    sprintf(out+strlen(out), "UDP (sport: %d, dport: %d, ", sport, dport);
    sprintf(out+strlen(out), "len: %d, csum: %x)", ntohs(udp->uh_ulen),
	ntohs(udp->uh_sum));

    return 0;
}

static int knet_print_icmp(knet_icmp_hdr_t *icmp, char *out)
{
    sprintf(out+strlen(out), "ICMP (type: %d, code: %d, cksum: %d, id: %d, "
	"seq: %d)", icmp->icmp_type, icmp->icmp_code, icmp->icmp_cksum,
	icmp->icmp_hun.ih_idseq.icd_id, icmp->icmp_hun.ih_idseq.icd_seq);

    return 0;
}


void knet_print_buffer(char *prefix, knet_buf_t *buf)
{
    int skip = 1;
    knet_ip_hdr_t *ip;
    int hlen;
    int plen = 0;
    char *next;
    u16 proto;
    static char out[1024]; 
    char tmp[80];
#ifdef CONFIG_RG_TARGET_VXWORKS
    knet_eth_hdr_t *eth_hdr = 
	(knet_eth_hdr_t *)(buf->knet_data - KNET_ETHER_HDR_LEN);
#endif

#ifdef CONFIG_RG_TARGET_LINUX
    ip = (knet_ip_hdr_t *)buf->nh.iph;
#else
    /* We assume that IP header is at the start */
    ip = knet_mtod(buf, knet_ip_hdr_t *);
#endif
    if (!ip)
    {
	kos_print(__FUNCTION__ ": IP is null\n");
	return;
    }
    
    hlen = ip->ip_hl << 2;
    next = ((char *)ip) + hlen;

#ifdef CONFIG_RG_TARGET_LINUX
    proto = buf->protocol;
#else
    proto = eth_hdr->ether_type;
#endif
    
    switch (ntohs(proto))
    {
    case 0: /* for bootp */
    case KNET_ETH_TYPE_IP:
	strcpy(out, prefix);
	plen = knet_print_ip(ip, out);
	strcat(out, " ");
	switch (ip->ip_p)
	{
	case IPPROTO_TCP:
	    skip = knet_print_tcp((knet_tcp_hdr_t *)next, out);
	    break;
	case IPPROTO_UDP:
	    skip = knet_print_udp((knet_udp_hdr_t *)next, out); 
	    break;
	case IPPROTO_ICMP:
	    skip = knet_print_icmp((knet_icmp_hdr_t *)next, out); 
	    break;
	default: 
	    sprintf(tmp, "Unknown IP protocol (%d)", ip->ip_p); 
	    strcat(out, tmp);
	}
	break;

    case KNET_ETH_TYPE_ARP:
	strcpy(out, "ARP");
	break;
    case KNET_ETH_TYPE_RARP:
	strcpy(out, "RARP");
	break;
		
    case KNET_ETH_TYPE_PPP_DISC:
	strcpy(out, "PPP DISC");
	break;
    case KNET_ETH_TYPE_PPP_SESSION:
	strcpy(out, "PPP SESSION");
	break;

    default:
	sprintf(out, "Unknown protocol (%d)", ntohs(proto));
    }
    /* No printing of output */
    if (skip)
	return;

    if (plen != buf->knet_pktlen)
    {
	sprintf(out+strlen(out), " LENERR (plen=%d, buflen=%d)", plen,
	    buf->knet_pktlen);
    }

    kos_print("%s\n", out);
}
#endif

