/* LICENSE_CODE JUNGO */

#include <kos/kos.h>
#include <kos/knet.h>
#include <kos/kos_chardev.h>
#include <stdarg.h>
#ifdef CONFIG_RG_TARGET_VXWORKS
#include <netBufLib.h>
#endif


#ifdef CONFIG_RG_TARGET_LINUX
#include <kos/knet.h>
#include <kos/knet_utils.h>
#include <kos/knet_if_ext.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/version.h>
#endif

#include "kos_chardev.h"

static int knet_initialized;
static int kos_chardev_initialized;

#ifdef CONFIG_RG_TARGET_VXWORKS

static int kos_mem_initialized;

struct single_pool_t;

typedef struct master_pool_t {
    size_t buf_size;
    int prefered_nbufs; /* In a single pool */
    int total_bufs; /* In all pools that belong to this master */
    int total_used;
    int max_used;
    struct single_pool_t *head;
    /* An ISR might do many allocs/frees in a row in a protected section.
     * This means that the adjust function might be queued many times
     * before the first instance acts. We want it to work only once.
     */
    int is_adjust_initiated;
} master_pool_t;

typedef struct single_pool_t {
    struct single_pool_t *next;
    int nbufs; /* Total number of buffers in this pool */
    int used; /* Number of used buffers in this pool */
    master_pool_t *master_pool; /* Pointer back to the master pool */
    u8 *buffers;
} single_pool_t;

static master_pool_t kos_mem_pools[] = {
    { 64, 128, 0, 0, 0, NULL, 0 },
    { 128, 256, 0, 0, 0, NULL, 0 },
    { 256, 256, 0, 0, 0, NULL, 0 },
    { 512, 64, 0, 0, 0, NULL, 0 },
};

#define MAX_MEM_FOR_POOL (1024 * 1024)

static int is_mem_available(size_t size_needed)
{
    int i;
    size_t total_used = 0;
    
    for (i = 0; i < NELEMENTS(kos_mem_pools); i++)
    {
	total_used += kos_mem_pools[i].total_bufs *
	    (kos_mem_pools[i].buf_size + sizeof(single_pool_t *));
    }
    
    return total_used + size_needed < MAX_MEM_FOR_POOL;
}

#define MIN_BUFFERS 16
static int single_pool_add(master_pool_t *master, int can_alloc_less)
{
    single_pool_t **s, *new_pool;
    int i, nbufs = master->prefered_nbufs, ret = -1;
    kos_interrupt_flags_t flags = kos_save_flags_cli();
    
    /* The extra space is used both as a back pointer to the single pool and
     * as an indication (if zero) that this buffer is available for allocation.
     */
    size_t one_buff_alloc_size = master->buf_size + sizeof(single_pool_t *);
    
    for (s = &master->head; *s; s = &(*s)->next);
    
    new_pool = (single_pool_t *)malloc(sizeof(*new_pool));
    if (!new_pool)
	goto Error;
    memset(new_pool, 0, sizeof(*new_pool));

    do
    {
	new_pool->buffers = is_mem_available(nbufs * one_buff_alloc_size) ? 
	    (u8 *)malloc(nbufs * one_buff_alloc_size) : NULL;
	if (new_pool->buffers)
	    break;
	nbufs /= 2;
    } while (can_alloc_less && nbufs >= MIN_BUFFERS);
    
    if (!new_pool->buffers)
	goto Error;

    new_pool->nbufs = nbufs;
    new_pool->master_pool = master;
    master->total_bufs += nbufs;
    *s = new_pool;
    
    /* Zero all back pointers to indicate all buffers are free */
    for (i = 0; i < nbufs; i++)
	*(single_pool_t **)(new_pool->buffers + i * one_buff_alloc_size) = 0;

    ret = 0;
    goto Exit;

Error:
    if (new_pool)
	free(new_pool);

Exit:
    kos_restore_flags(flags);
    return ret;
}

static void single_pool_del(single_pool_t **single_pool)
{
    single_pool_t *s;
    master_pool_t *master;
    kos_interrupt_flags_t flags = kos_save_flags_cli();

    if (!single_pool || !(s = *single_pool))
	goto Exit;
    
    master = s->master_pool;
    master->total_bufs -= s->nbufs;
    master->total_used -= s->used;
    free(s->buffers);
    *single_pool = s->next;
    free(s);

Exit:
    kos_restore_flags(flags);
}

/* Free the memory allocated for the memory pool. */
static void kos_mem_pool_uninit(void)
{
    int i;
    single_pool_t **s;

    for (i = 0; i < NELEMENTS(kos_mem_pools); i++)
    {
	s = &kos_mem_pools[i].head;
	while (*s)
	    single_pool_del(s);
    }
}

static int kos_mem_pool_init(void)
{
    int i;

    for (i = 0; i < NELEMENTS(kos_mem_pools); i++)
    {
	if (single_pool_add(&kos_mem_pools[i], 0))
	    goto Error;
    }

    return 0;

Error:
    kos_mem_pool_uninit();
    return ENOMEM;
}

static void kos_mem_pool_adjust(unsigned long param)
{
    master_pool_t *master = (master_pool_t *)param;
    kos_interrupt_flags_t flags = kos_save_flags_cli();
    single_pool_t **s = &master->head;

    if (master->total_used > master->total_bufs * 0.75)
    {
	single_pool_add(master, 1);
	goto Exit; /* If enlarged no need to check shrink */
    }

    if (master->total_bufs < 2 * master->total_used)
	goto Exit;
    
    /* Try to delete all completely unused single pools so that after the
     * process used will be less than 0.75*total for this master.
     */
    while (*s)
    {
	int new_total;

	new_total = master->total_bufs - (*s)->nbufs;
	
	if ((*s)->used || !new_total || master->total_used > new_total * 0.75)
	{
	    s = &(*s)->next;
	    continue;
	}
	
	single_pool_del(s);
    }

Exit:
    master->is_adjust_initiated = 0;
    kos_restore_flags(flags);
}

static void *kos_vx_alloc_protected(size_t size)
{
    int i;
    kos_interrupt_flags_t flags = kos_save_flags_cli();
    void *ptr = NULL;

    for (i = 0; i < NELEMENTS(kos_mem_pools); i++)
    {
	single_pool_t *s;
	int j;
	
	if (kos_mem_pools[i].buf_size < size ||
	    kos_mem_pools[i].total_used == kos_mem_pools[i].total_bufs)
	{
	    continue;
	}
	
	for (s = kos_mem_pools[i].head; s && s->nbufs == s->used; s = s->next);
	
	if (!s)
	    KOS_PANIC("KOS Memory pool inconsistent - exiting 1\n");

	for (j = 0; j < s->nbufs; j++)
	{
	    u8 *buff = s->buffers +
		j * (kos_mem_pools[i].buf_size + sizeof(single_pool_t *));
	    
	    if (*(single_pool_t **)buff)
		continue;
	    
	    *(single_pool_t **)buff = s;
	    s->used++;
	    kos_mem_pools[i].total_used++;
	    ptr = (void *)(buff + sizeof(single_pool_t *));
	    if (kos_mem_pools[i].total_used > kos_mem_pools[i].max_used)
		kos_mem_pools[i].max_used = kos_mem_pools[i].total_used;
	    goto Exit;
	}
	
	KOS_PANIC("KOS Memory pool inconsistent - exiting\n");
    }

Exit:
    if (!ptr)
	kos_print("Error allocating %d bytes\n", size);

    if (ptr && !kos_mem_pools[i].is_adjust_initiated)
    {
	kos_mem_pools[i].is_adjust_initiated = 1;
	netJobAdd(kos_mem_pool_adjust, (unsigned long)&kos_mem_pools[i],
	    0, 0, 0, 0);
    }
    kos_restore_flags(flags);
    return ptr;
}

static void kos_vx_free_protected(void *ptr)
{
    single_pool_t *s, **sp;
    kos_interrupt_flags_t flags = kos_save_flags_cli();

    sp = (single_pool_t **)((u8 *)ptr - sizeof(single_pool_t *));
    s = *sp;
    if (!s)
	KOS_PANIC("Trying to free an alreay freed buffer - exiting\n");
    *sp = 0;
    s->used--;
    s->master_pool->total_used--;
    if (!s->master_pool->is_adjust_initiated)
    {
	s->master_pool->is_adjust_initiated = 1;
	netJobAdd(kos_mem_pool_adjust, (unsigned long)s->master_pool,
	    0, 0, 0, 0);
    }
    kos_restore_flags(flags);
}

static int is_protected_alloc(void *ptr)
{
    int i;
    single_pool_t *s;
    
    for (i = 0; i < NELEMENTS(kos_mem_pools); i++)
    {
	for (s = kos_mem_pools[i].head; s; s = s->next)
	{
	    size_t alloc_space = s->nbufs * 
		(kos_mem_pools[i].buf_size + sizeof(single_pool_t *));

	    /* Do not check (ptr == s->buffers) because first buffer passed to
	     * user is s->buffers + sizeof(single_pool_t *)
	     */
	    if ((u8 *)ptr > s->buffers && (u8 *)ptr < s->buffers + alloc_space)
		return 1;
	}
    }

    return 0;
}

void kos_mem_pool_stats(void)
{
    int i;
    
    printf("size\ttotal\tfree\tmax used\n");
    printf("----\t-----\t----\t--------\n");

    for (i = 0; i < NELEMENTS(kos_mem_pools); i++)
    {
	printf("%d\t%d\t%d\t%d\n", kos_mem_pools[i].buf_size,
	    kos_mem_pools[i].total_bufs, kos_mem_pools[i].total_bufs -
	    kos_mem_pools[i].total_used, kos_mem_pools[i].max_used);
    }
}
#endif

int kos_init(void)
{
    int ret = 0;

#ifdef CONFIG_RG_TARGET_VXWORKS
    if ((ret = kos_mem_pool_init()))
    {
	kos_print("Error allocating memory pool\n");
	return ret;
    }
    kos_mem_initialized = 1;
#endif
    
    if ((ret = kos_chardev_init()))
	goto Exit;

    kos_chardev_initialized = 1;
	
    if ((ret = knet_init()))
	goto Exit;

    knet_initialized = 1;
    
    return 0;
    
Exit:
   kos_uninit();
   return ret;
}

void kos_uninit(void)
{
    if (knet_initialized)
	knet_uninit();

    if (kos_chardev_initialized)
	kos_chardev_uninit();
    
#ifdef CONFIG_RG_TARGET_VXWORKS
    if (kos_mem_initialized)
	kos_mem_pool_uninit();
#endif
}

#ifdef MODULE
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
    module_init(kos_init);
    module_exit(kos_uninit);
#else
int init_module(void)
{
    return kos_init();
}

void cleanup_module(void)
{
    kos_uninit();
}
#endif
#endif /* MODULE */

u32 kos_copy_from_user(void *to, const void *from, u32 n)
{
#ifdef CONFIG_RG_TARGET_LINUX
    int err = copy_from_user(to, from, n);
    return err && err!=n ? err : 0;
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    memcpy(to, from, n); 
    return 0; /* Though it seems copy_from_user returns n, it actually returns 
	       * zero on success (careful with the assembly).
	       */
#endif
}

u32 kos_copy_to_user(void *to, const void *from, u32 n)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return copy_to_user(to, from, n);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    memcpy(to, from, n); 
    return 0;
#endif
}

void *__kos_alloc(size_t size, int flags, char *func)
{
#ifdef CONFIG_RG_TARGET_LINUX
    int linux_flags = GFP_KERNEL;
    if (flags == KOS_ALLOC_ATOMIC)
	linux_flags = GFP_ATOMIC;
    if (kos_in_interrupt() && linux_flags != GFP_ATOMIC)
	kos_print("%s: alloc not atomic in interrupt\n", func);
    return kmalloc(size, linux_flags);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    if ((flags == KOS_ALLOC_ATOMIC) || (flags == KOS_ALLOC_PROTECTED))
	return kos_vx_alloc_protected(size);
    if (kos_in_interrupt())
	kos_print("%s: alloc not atomic in interrupt\n", func);
    return malloc(size);
#endif
}

void kos_free(void *ptr)
{
#ifdef CONFIG_RG_TARGET_LINUX
    kfree(ptr);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    if (is_protected_alloc(ptr))
	kos_vx_free_protected(ptr);
    else
	free(ptr);
#endif
}

int kos_timer_init(kos_timer_t *t)
{
#ifdef CONFIG_RG_TARGET_LINUX
    init_timer(t);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    t->expires = 0;
    t->id = wdCreate();
    if (!t->id)
	return -1;
#endif
    return 0;
}

int kos_timer_uninit(kos_timer_t *t)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return 0;
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    return wdDelete(t->id);
#endif
}

int kos_timer_start(kos_timer_t *t, int expires, kos_timer_callback_t cb, 
    int data)
{
#ifdef CONFIG_RG_TARGET_LINUX
    t->expires = expires + jiffies;
    t->data = data;
    t->function = cb;
    add_timer(t);
    return 0;    
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    t->expires = kos_ticks_get() + expires;
    if (OK == wdStart(t->id, expires, cb, data))
	return OK;
    t->expires = 0;
    return ERROR;
#endif
}

int kos_timer_stop(kos_timer_t *t)
{
#ifdef CONFIG_RG_TARGET_LINUX
    del_timer(t);
    t->expires = 0;
    return 0;    
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    t->expires = 0;
    return wdCancel(t->id);
#endif
}

int kos_timer_active(kos_timer_t *t)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return t->expires > jiffies;
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    return t->id && t->expires > kos_ticks_get();
#endif
}

int kos_timer_mod(kos_timer_t *t, int expires, kos_timer_callback_t cb, 
    int data)
{
#ifdef CONFIG_RG_TARGET_LINUX
#if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0) )
    return mod_timer(t, expires + jiffies);
#else
    mod_timer(t, expires + jiffies); /* void function */
    return 1;
#endif
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    return wdStart(t->id, expires, cb, data);
#endif
}

void kos_gettime(kos_timeval_t *t)
{
#ifdef CONFIG_RG_TARGET_LINUX
    do_gettimeofday(t);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    struct timespec tt;
    clock_gettime(CLOCK_REALTIME, &tt);
    t->tv_sec = tt.tv_sec;
    t->tv_usec = tt.tv_nsec / 1000;
#endif    
}

#if defined(CONFIG_RG_TARGET_VXWORKS)
void kos_print(char *fmt, ...)
{
    va_list va;
    void *args[MAX_LOG_ARGS];
    char *p;
    int i;

    va_start(va, fmt); 
    memset(args, 0, sizeof(args));
    /* count arguments, and get them from stack */
    for (p = fmt, i=0; i < MAX_LOG_ARGS && (p = strchr(p, '%')) && *(p+1); 
	p += 2, i++)
    {
	if (*(p+1)=='%') /* %% means print % */
	    continue;
	args[i] = va_arg(va, void *);
    }
    va_end(va);
#if MAX_LOG_ARGS < 6
#error VxWorks logMsg requires not less than 6 arguments
#endif
    logMsg(fmt, (int)args[0], (int)args[1], (int)args[2], (int)args[3], 
	(int)args[4], (int)args[5]);
}
#endif

kos_interrupt_flags_t kos_save_flags_cli(void)
{
#ifdef CONFIG_RG_TARGET_LINUX
    kos_interrupt_flags_t flags;
    save_flags(flags);
    cli();
    return flags;
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    return intLock();    
#endif
}

void kos_restore_flags(kos_interrupt_flags_t flags)
{
#ifdef CONFIG_RG_TARGET_LINUX
    restore_flags(flags);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    intUnlock(flags);
#endif
}

kos_interrupt_flags_t kos_enter_protected(void)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return 0;
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    return kos_save_flags_cli();
#endif
}

void kos_exit_protected(kos_interrupt_flags_t flags)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return;
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    kos_restore_flags(flags);
#endif
}

int kos_in_interrupt(void)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return in_interrupt();
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    return intContext();
#endif
}

void kos_atomic_set(volatile kos_atomic_t *v, int i)
{
#ifdef CONFIG_RG_TARGET_LINUX
    atomic_set(v, i);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    kos_interrupt_flags_t flags = kos_save_flags_cli();
    v->counter = i;
    kos_restore_flags(flags);
#endif
}

int kos_atomic_read(volatile kos_atomic_t *v)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return atomic_read(v);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    return v->counter;
#endif
}

void kos_atomic_inc(volatile kos_atomic_t *v)
{
#ifdef CONFIG_RG_TARGET_LINUX
    atomic_inc(v);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    kos_interrupt_flags_t flags = kos_save_flags_cli();
    v->counter++;
    kos_restore_flags(flags);
#endif
}

void kos_atomic_dec(volatile kos_atomic_t *v)
{
#ifdef CONFIG_RG_TARGET_LINUX
    atomic_dec(v);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    kos_interrupt_flags_t flags = kos_save_flags_cli();
    v->counter--;
    kos_restore_flags(flags);
#endif
}

int kos_random(void)
{
#ifdef CONFIG_RG_TARGET_LINUX
    int n;
    get_random_bytes(&n, sizeof(int));
    return n;
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    return rand();
#endif
}

void kos_pool_init(struct kos_pool *pool, unsigned int size)
{
    pool->pool_size_max = -1;
    kos_atomic_set(&pool->pool_size, 0);
    pool->alloc_size = size;
}

void *kos_pool_get(struct kos_pool *pool)
{
    void *ptr;
    
    if (pool->pool_size_max != -1 && 
	kos_atomic_read(&pool->pool_size) >= pool->pool_size_max)
    {
	return NULL;
    }

    ptr = kos_alloc(pool->alloc_size, KOS_ALLOC_ATOMIC);
    if (ptr)
	kos_atomic_inc(&pool->pool_size);
    return ptr;
}

void kos_pool_put(struct kos_pool *pool, void *p)
{
    kos_free(p);
    kos_atomic_dec(&pool->pool_size);
}

void kos_pool_max_set(struct kos_pool *pool, int sz)
{
    pool->pool_size_max = sz;
}

int kos_pool_max_get(struct kos_pool *pool)
{
    return pool->pool_size_max;
}

void kos_mod_count_inc(void)
{
#ifdef MODULE
    MOD_INC_USE_COUNT;
#endif
}

void kos_mod_count_dec(void)
{
#ifdef MODULE
    MOD_DEC_USE_COUNT;
#endif
}

void kos_wake_up_interruptible(kos_wait_queue_t *q)
{
#ifdef CONFIG_RG_TARGET_LINUX
#if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0) )
    wake_up_interruptible(*q);
#else
    wake_up_interruptible(q);
#endif
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    selWakeupAll(q, SELREAD);
    selWakeupAll(q, SELWRITE);
#endif
}

void kos_waitq_init(kos_wait_queue_t *waitq)
{
#ifdef CONFIG_RG_TARGET_LINUX
    *waitq = NULL;
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    selWakeupListInit(waitq);
#endif
}

unsigned int kos_select(kos_poll_node_t poll_node, int read, int write)
{
#ifdef CONFIG_RG_TARGET_LINUX
    unsigned int mask = 0;
    if (read)
	mask |= (POLLIN | POLLRDNORM);
    if (write)
	mask |= (POLLOUT | POLLWRNORM);
    return mask;
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    if (read && (selWakeupType(poll_node) == SELREAD))
	selWakeup(poll_node);
    if (write && (selWakeupType(poll_node) == SELWRITE))
	selWakeup(poll_node);
    return 0;
#endif
}

int kos_poll_node_del(kos_wait_queue_t *waitq, kos_poll_node_t poll_node)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return 0;
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    return selNodeDelete(waitq, poll_node);
#endif
}

unsigned long kos_simple_strtoul(char *cp, char **endp, unsigned int base)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return simple_strtoul(cp, endp, base);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    unsigned long result = 0, value;

    if (!base)
    {
	base = 10;
	if (*cp == '0')
	{
	    base = 8;
	    cp++;
	    if ((*cp == 'x') && isxdigit((int)cp[1]))
	    {
		cp++;
		base = 16;
	    }
	}
    }
    while (isxdigit((int)*cp) && (value = isdigit((int)*cp) ? *cp - '0' : 
	(islower((int)*cp) ? toupper(*cp) : *cp) - 'A' + 10) < base)
    {
	result = result * base + value;
	cp++;
    }
    if (endp)
	*endp = (char *)cp;
    return result;
#endif
}

#ifdef CONFIG_RG_TARGET_LINUX
/* kos.c */
EXPORT_SYMBOL(kos_copy_from_user);
EXPORT_SYMBOL(kos_copy_to_user);
EXPORT_SYMBOL(__kos_alloc);
EXPORT_SYMBOL(kos_free);
EXPORT_SYMBOL(kos_timer_init);
EXPORT_SYMBOL(kos_timer_uninit);
EXPORT_SYMBOL(kos_timer_start);
EXPORT_SYMBOL(kos_timer_stop);
EXPORT_SYMBOL(kos_timer_active);
EXPORT_SYMBOL(kos_timer_mod);
EXPORT_SYMBOL(kos_gettime);
EXPORT_SYMBOL(kos_save_flags_cli);
EXPORT_SYMBOL(kos_restore_flags);
EXPORT_SYMBOL(kos_enter_protected);
EXPORT_SYMBOL(kos_exit_protected);
EXPORT_SYMBOL(kos_in_interrupt);
EXPORT_SYMBOL(kos_atomic_set);
EXPORT_SYMBOL(kos_atomic_read);
EXPORT_SYMBOL(kos_atomic_inc);
EXPORT_SYMBOL(kos_atomic_dec);
EXPORT_SYMBOL(kos_random);
EXPORT_SYMBOL(kos_pool_init);
EXPORT_SYMBOL(kos_pool_get);
EXPORT_SYMBOL(kos_pool_put);
EXPORT_SYMBOL(kos_pool_max_set);
EXPORT_SYMBOL(kos_pool_max_get);
EXPORT_SYMBOL(kos_wake_up_interruptible);
EXPORT_SYMBOL(kos_waitq_init);
EXPORT_SYMBOL(kos_select);
EXPORT_SYMBOL(kos_poll_node_del);
EXPORT_SYMBOL(kos_simple_strtoul);
/* knet.h */
EXPORT_SYMBOL(knet_ntoa);
EXPORT_SYMBOL(knet_aton);
EXPORT_SYMBOL(knet_get_u32);
EXPORT_SYMBOL(knet_pkt_type_set);
EXPORT_SYMBOL(knet_buf_alloc);
EXPORT_SYMBOL(knet_buf_free);
EXPORT_SYMBOL(knet_buf_ip_get);
EXPORT_SYMBOL(knet_pullup);
EXPORT_SYMBOL(knet_buf_cow);
EXPORT_SYMBOL(knet_buf_copydata);
EXPORT_SYMBOL(knet_buf_copy_to_user);
EXPORT_SYMBOL(knet_buf_data_push);
EXPORT_SYMBOL(knet_buf_data_pull);
EXPORT_SYMBOL(knet_buf_cow_align);
EXPORT_SYMBOL(knet_buf_data_adj);
EXPORT_SYMBOL(knet_buf_copyback);
EXPORT_SYMBOL(knet_buf_clone);
EXPORT_SYMBOL(knet_buf_dup);
EXPORT_SYMBOL(knet_buf_merge);
EXPORT_SYMBOL(knet_buf_data_put);
EXPORT_SYMBOL(knet_buf_data_reserve);
EXPORT_SYMBOL(knet_ip_compute_csum);
EXPORT_SYMBOL(knet_fix_csum_16);
EXPORT_SYMBOL(knet_netdev_register);
EXPORT_SYMBOL(knet_netdev_unregister);
EXPORT_SYMBOL(knet_netdev_get_by_name);
EXPORT_SYMBOL(knet_netdev_get_by_index);
EXPORT_SYMBOL(knet_netdev_promisc_set);
EXPORT_SYMBOL(knet_netdev_send);
EXPORT_SYMBOL(knet_netdev_receive);
EXPORT_SYMBOL(knet_netdev_fields_init);
EXPORT_SYMBOL(knet_netdev_zalloc);
EXPORT_SYMBOL(knet_netdev_queue_stopped);
EXPORT_SYMBOL(knet_netdev_queue_xmit);
EXPORT_SYMBOL(knet_netdev_copy_name);
EXPORT_SYMBOL(knet_netdev_free_name);
EXPORT_SYMBOL(knet_netdev_name_get);
EXPORT_SYMBOL(knet_netdev_ip_get);
EXPORT_SYMBOL(knet_is_devs_ip);
EXPORT_SYMBOL(knet_netdev_has_ip);
EXPORT_SYMBOL(knet_buf_rx_hook_register);
EXPORT_SYMBOL(knet_buf_tx_hook_register);
EXPORT_SYMBOL(knet_buf_rx_hook_unregister);
EXPORT_SYMBOL(knet_buf_tx_hook_unregister);
EXPORT_SYMBOL(knet_buf_hook_dev_add);
EXPORT_SYMBOL(knet_buf_rx_hook_dev_add);
EXPORT_SYMBOL(knet_buf_tx_hook_dev_add);
EXPORT_SYMBOL(knet_buf_hook_dev_del);
EXPORT_SYMBOL(knet_buf_rx_hook_dev_del);
EXPORT_SYMBOL(knet_buf_tx_hook_dev_del);
EXPORT_SYMBOL(knet_buf_hook_dev_clear);
EXPORT_SYMBOL(knet_buf_rx_hook_dev_clear);
EXPORT_SYMBOL(knet_buf_tx_hook_dev_clear);
EXPORT_SYMBOL(knet_buf_hook_dev_set);
EXPORT_SYMBOL(knet_buf_rx_hook_dev_set);
EXPORT_SYMBOL(knet_buf_tx_hook_dev_set);
/* knet_util.h */
EXPORT_SYMBOL(knet_buf_find_case);
EXPORT_SYMBOL(knet_buf_ncmp);	
EXPORT_SYMBOL(knet_buf_case_ncmp);	
EXPORT_SYMBOL(knet_buf_pbrk);
EXPORT_SYMBOL(knet_buf_uint_read);
EXPORT_SYMBOL(knet_buf_aton);
EXPORT_SYMBOL(knet_buf_find);
EXPORT_SYMBOL(knet_buf_replace);
EXPORT_SYMBOL(kos_chardev_register);
EXPORT_SYMBOL(kos_chardev_unregister);
#ifdef DEBUG_PRINT_BUFFER
EXPORT_SYMBOL(knet_print_buffer);
#endif
/* knet_if_ext.c */
EXPORT_SYMBOL(knet_if_ext_get);
EXPORT_SYMBOL(knet_if_ext_free);
#endif

