/* LICENSE_CODE JUNGO */

#include <kos/kos.h>
#include <kos/kos_chardev.h>
#include <kos/knet.h>


#ifdef CONFIG_RG_TARGET_VXWORKS
#include <selectLib.h>
#include <iosLib.h>
#elif defined(CONFIG_RG_TARGET_LINUX)
#include <linux/poll.h>
#ifdef KNET_LINUX_24
#include <linux/module.h>
#endif
#endif

/* Registered devices list */
kos_reg_chardev_t *reg_chardev_list = NULL;

int kos_chardev_register(kos_reg_chardev_t *dev)
{
    kos_reg_chardev_t **r;

    for (r = &reg_chardev_list; *r; r = &(*r)->next)
    {
	if ((*r)->type == dev->type)
	    return -EEXIST;
    }

    dev->next = NULL;
    *r = dev;

    return 0;
}

int kos_chardev_unregister(kos_reg_chardev_t *dev)
{
    kos_reg_chardev_t **r;

    for (r = &reg_chardev_list; *r; r = &(*r)->next)
    {
	if (*r == dev)
	{
	    *r = (*r)->next;
	    return 0;
	}
    }

    kos_print(__FUNCTION__" dev not found\n");
    return -ENOENT;
}

/* The open call must be followed immediately by a RG_IOCTL_SIOCSETRGCHRDEVTYPE
 * ioctl, as a single transaction. Otherwise, the char dev will be in an
 * inconsistent state (i.e. calling open and close MUST NOT be done, call
 * open, ioctl, close).
 */
#ifdef CONFIG_RG_TARGET_VXWORKS
static int kos_chardev_open(DEV_HDR *dev_hdr, char *unused, int mode)
#else
static int kos_chardev_open(struct inode *inode, struct file *file)
#endif
{
    kos_chardev_t *dev;

    if (!(dev = kos_alloc(sizeof(kos_chardev_t), KOS_ALLOC_KERNEL)))
    {
	kos_print(__FUNCTION__" kos_alloc failed\n");
	return -1;
    }

    memset(dev, 0, sizeof(kos_chardev_t));

    /* We can't call a device specific open() because we dont know which
     * device it is yet. dev->open() and the increment of use_count will be
     * called from the RG_IOCTL_SIOCSETRGCHRDEVTYPE ioctl handler.
     */
     
#ifdef CONFIG_RG_TARGET_VXWORKS
    return (int)dev;
#else
    file->private_data = dev;
    return 0;
#endif
}

#define KOS_CD_CALL_OP(dev, func, no_func_ret, params...) \
    (dev->reg_dev ? \
    (dev->reg_dev->func ? dev->reg_dev->func(params) : no_func_ret) : \
    -EBADF)

#ifdef CONFIG_RG_TARGET_VXWORKS
static int kos_chardev_close(kos_chardev_t *dev)
#else
static int kos_chardev_close(struct inode *inode, struct file *file)
#endif
{
    int ret;
#ifdef CONFIG_RG_TARGET_LINUX
    kos_chardev_t *dev = (kos_chardev_t *)file->private_data;
#endif

    if (!dev->reg_dev)
	return -EBADF;

    kos_atomic_dec(&dev->reg_dev->use_count);

    ret = KOS_CD_CALL_OP(dev, close, 0, dev);
    kos_free(dev);
    return ret;
}

#ifdef CONFIG_RG_TARGET_VXWORKS
static int kos_chardev_read(kos_chardev_t *dev, char *buf, int len)
#else
static ssize_t kos_chardev_read(struct file *file, char *buf, size_t len,
    loff_t *pos)
#endif
{
#ifdef CONFIG_RG_TARGET_LINUX
    kos_chardev_t *dev = (kos_chardev_t *)file->private_data;
#endif

    return KOS_CD_CALL_OP(dev, read, KOS_ERRNO(EINVAL), dev, buf, len);
}

#ifdef CONFIG_RG_TARGET_VXWORKS
static int kos_chardev_write(kos_chardev_t *dev, char *buf, int len)
#else
static ssize_t kos_chardev_write(struct file *file, const char *buf, size_t len,
    loff_t *pos)
#endif
{
#ifdef CONFIG_RG_TARGET_LINUX
    kos_chardev_t *dev = (kos_chardev_t *)file->private_data;
#endif

    return KOS_CD_CALL_OP(dev, write, KOS_ERRNO(EINVAL), dev, buf, len);
}

#ifdef CONFIG_RG_TARGET_VXWORKS
static int kos_chardev_ioctl(kos_chardev_t *dev, int cmd, int arg)
#else
static int kos_chardev_ioctl(struct inode *inode, struct file *file,
    unsigned int cmd, unsigned long arg)
#endif
{
    kos_reg_chardev_t *reg_dev;
#ifdef CONFIG_RG_TARGET_LINUX
    kos_chardev_t *dev = (kos_chardev_t *)file->private_data;
#else
    SEL_WAKEUP_NODE *wakeup_node;
#endif

    if (cmd != RG_IOCTL_SIOCSETRGCHRDEVTYPE && !dev->reg_dev)
	return KOS_ERRNO(EBADF);
	
    switch (cmd)
    {
    case RG_IOCTL_SIOCSETRGCHRDEVTYPE:
	for (reg_dev = reg_chardev_list; reg_dev && reg_dev->type != arg;
	    reg_dev = reg_dev->next);
	
	if (!reg_dev)
	{
	    kos_print(__FUNCTION__": type %d not found\n", (int)arg);
	    return KOS_ERRNO(EINVAL);
	}

	kos_atomic_inc(&reg_dev->use_count);
	dev->reg_dev = reg_dev;
#ifdef CONFIG_RG_TARGET_LINUX
	dev->filp = file;
#endif
	return KOS_CD_CALL_OP(dev, open, 0, dev);
#ifdef CONFIG_RG_TARGET_VXWORKS
    case FIOSELECT:
	wakeup_node = (SEL_WAKEUP_NODE *)arg;
	return KOS_CD_CALL_OP(dev, select, 0, dev, wakeup_node);
	
    case FIOUNSELECT:
	wakeup_node = (SEL_WAKEUP_NODE *)arg;
	return KOS_CD_CALL_OP(dev, unselect, 0, dev, wakeup_node);
	break;
#endif
    default:
	return KOS_CD_CALL_OP(dev, ioctl, KOS_ERRNO(EINVAL), dev, cmd, arg);
    }

    return 0;
}

#ifdef CONFIG_RG_TARGET_LINUX
static unsigned int kos_chardev_select(struct file *file,
    struct poll_table_struct *pts)
{
    kos_chardev_t *dev = (kos_chardev_t *)file->private_data;

    return KOS_CD_CALL_OP(dev, select, 0, dev, pts);
}
#endif

#ifdef CONFIG_RG_TARGET_VXWORKS
static int kos_cd_drv_num;
static DEV_HDR kos_cd_hdr;
#else 
static struct file_operations kos_chardev_fops = {
#ifdef KNET_LINUX_24
    THIS_MODULE,
#endif
    /* Note: There are more fops. implement them if you need ;-) */
    read:	kos_chardev_read,
    write: 	kos_chardev_write,
    poll:	kos_chardev_select,
    ioctl:	kos_chardev_ioctl,
    open:	kos_chardev_open,
    release:	kos_chardev_close,
};
#endif

int kos_chardev_init(void)
{
    /* Register kos_chardev with the relevant OS */
#ifdef CONFIG_RG_TARGET_VXWORKS
    kos_cd_drv_num = iosDrvInstall(0, 0, kos_chardev_open, kos_chardev_close,
	kos_chardev_read, kos_chardev_write, kos_chardev_ioctl);    
    
    if (kos_cd_drv_num == ERROR)
    {
	kos_print(__FUNCTION__ ": iosDrvInstall failed\n");
	return ERROR;
    }

    if (iosDevAdd(&kos_cd_hdr, "/dev/"OPENRG_CHRDEV_NAME, kos_cd_drv_num) ==
	ERROR)
    {
	kos_print(__FUNCTION__ ": iosDevAdd failed\n");
	return ERROR;
    }
#else
    if (register_chrdev(OPENRG_CHRDEV_MAJOR, OPENRG_CHRDEV_NAME,
	&kos_chardev_fops))
    {
	kos_print(__FUNCTION__": register_chrdev failed\n");
	return -EIO;
    }
#endif
    return 0;
}

int kos_chardev_uninit(void)
{
    if (reg_chardev_list)
	return -1;
    
    /* Unregister kos_chardev */
#ifdef CONFIG_RG_TARGET_VXWORKS
    if (iosDrvRemove(kos_cd_drv_num, TRUE) == ERROR)
    {
	kos_print(__FUNCTION__ ": iosDrvRemove failed\n");
    	return -1;
    }
#else
    unregister_chrdev(OPENRG_CHRDEV_MAJOR, OPENRG_CHRDEV_NAME);
#endif

    return 0;
}
