/** \file mic.c
 * \brief MIC calculation functions.
 *
 * This file includes all implimentation of the MIC API and 
 * internal functions.
 *
 * \see mic.h
*/

/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */
/*		MODULE:	mic.c							 		   				   */
/*     PURPOSE:	ICV calculation algorithm (CRC32)						   */
/*																	 	   */
/***************************************************************************/

#include "wpaIfc.h"
#include "mic.h"
#include "utils.h"
/*#include "pform_types_user.h"*/

#ifndef __linux__ // why the heck we define these macros twice !!??
#define USE_ENDIAN_SWAP 1
#endif

#define LO_BYTE(x)			((x) & 0xff)		
#define HI_BYTE(x)			(((x) >> 8) & 0xff)
#define LO_WORD(x)			((x) & 0xffff)		
#define HI_WORD(x)			(((x) >> 16) & 0xffff)

#ifdef USE_ENDIAN_SWAP
#define ENDIAN_HANDLE_WORD(x)	((LO_BYTE(x) << 8) | HI_BYTE(x))
#define ENDIAN_HANDLE_LONG(x) ((ENDIAN_HANDLE_WORD(LO_WORD(x)) << 16) | (ENDIAN_HANDLE_WORD(HI_WORD(x))))
#else
#define ENDIAN_HANDLE_WORD(x) (x)
#define ENDIAN_HANDLE_LONG(x) (x)
#endif


#define ROL32( A, n ) \
 ( ((A) << (n)) | ( ((A) >> (32-(n)))  & ( (1UL << (n)) - 1 ) ) )

#define ROR32( A, n ) ROL32( (A), 32-(n) )

/* Local functions definitions */
static UINT32 michael_getUInt32(UINT8 *p);
static void michael_putUInt32(UINT8 *p, UINT32 val);


void mic_init(micHandle_t *pHandle)
{
	memset(pHandle->micValue, 0, MIC_LEN);
	memset(pHandle->micKey, 0, MICHAEL_KEY_LEN);
	memset(pHandle->scratchpad, 0, 4);

	pHandle->micDataLength = 0;
}


void mic_setKey(micHandle_t *pHandle, UINT8 *pKey)
{
	memcpy(pHandle->micKey, pKey, MICHAEL_KEY_LEN);
	memcpy(pHandle->micValue, pKey, MICHAEL_KEY_LEN);
}

STATUS mic_addBuffer(micHandle_t *pHandle, UINT8 *pBuffer, UINT32 bufferLen)
{
    UINT32      *pData, *pEnd;
    UINT8       *pLeftOver;
    UINT8       residue, endResidue;
    UINT32      R, L, i;
/*	UINT8		endian1[4], endian2[4];	*/	


    if (pBuffer == NULL)
    {
        return NOK;
    }

	/* calculate scratchpad size */
    residue = pHandle->micDataLength % 4;

    /* calculate remaining bytes after buffer calculation */
    endResidue = (bufferLen + residue) % 4;

    /* start of data */
    pData = (UINT32*)pBuffer;

    /* end of data */
    pEnd = (UINT32*)(pBuffer + (bufferLen - endResidue));

    /* get Left and right MIC keys */
    L = michael_getUInt32(&(pHandle->micValue[0]));
    R = michael_getUInt32(&(pHandle->micValue[4]));

	
	/* Fill scratchpad */
    pLeftOver = pBuffer;

    if (residue > 0)
    {
        /* Fill scratchpad */
        for (i = residue; i < 4; i++)
        {
            *(UINT32*)(pHandle->scratchpad) |= *pLeftOver << (8*i);
            pLeftOver++;
        }

        /* calculate MIC on scratchpad */
        L ^= *(UINT32*)(pHandle->scratchpad);
        R ^= ROL32( L, 17 );
        L += R;
        R ^= ((L & 0xff00ff00) >> 8) | ((L & 0x00ff00ff) << 8);
        L += R;
        R ^= ROL32( L, 3 );
        L += R;
        R ^= ROR32( L, 2 );
        L += R;

        *(UINT32*)(pHandle->scratchpad) = 0;
        /* start of buffer data (without scratchpad filling) */
        pData = (UINT32*)pLeftOver;
  	

	}

	/* calculate MIC on buffer data */
    while ( pData < pEnd )
    {
		L ^= ENDIAN_HANDLE_LONG(*pData);

        R ^= ROL32( L, 17 );
        L += R;
        R ^= ((L & 0xff00ff00) >> 8) | ((L & 0x00ff00ff) << 8);
        L += R;
        R ^= ROL32( L, 3 );
        L += R;
        R ^= ROR32( L, 2 );
        L += R;

        pData++;
    }

	/* fill scratchpad with remaining bytes */
    pLeftOver = (UINT8*)pEnd;
    for (i = 0; i < endResidue; i++)
    {
        *(UINT32*)(pHandle->scratchpad) |= *pLeftOver << (8*i);
        pLeftOver++;
    }

    /* update right an left MIC values */
    michael_putUInt32(&(pHandle->micValue[0]), L);
    michael_putUInt32(&(pHandle->micValue[4]), R);
 
    /* update MIC data size */
    pHandle->micDataLength += bufferLen;

    return OK;
}

STATUS mic_getMic(micHandle_t *pHandle, UINT8* pMicValue)
{
    UINT8       paddingBuffer[8] = {0x5a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
    UINT16      bufferLen;

    if (pMicValue == NULL)
    {
        return NOK;
    } 

    /* padding buffer len */
    bufferLen = 8 - (pHandle->micDataLength % 4);

    // Append the minimum padding
    mic_addBuffer(pHandle, paddingBuffer, bufferLen);

    /* copy MIC value to result buffer */
    memcpy(pMicValue, pHandle->micValue, MIC_LEN);


    return OK;
}

static UINT32 michael_getUInt32(UINT8 *p)
{
    UINT32      res = 0;
    int         i;

    for (i = 0; i < 4; i++)
    {
        res |= (*p++) << (8*i);
    }

    return res;
}


static void michael_putUInt32(UINT8 *p, UINT32 val)
{
    int i;

    for (i = 0; i < 4; i++)
    {
        *p++ = (UINT8)(val & 0xff);
        val >>= 8;
    }
}