/* JUNGO_COPYRIGHT_1 */

#include "rg_error.h"
#include <rg_memory.h>
#include <util/random.h>
#include <esyscall.h>
#include <syscalls.h>
#include "radiusc.h"
#include "radiusct.h"
#include "rad_md5.h"

#define RADIUS_BUF_MAX 4096

/* forwards */
static void radius_client_packet_recv(int ret, void *o);

typedef enum {
    RADIUS_SESSION_START = 0,
    RADIUS_SESSION_ONGOING = 1,
} session_state_t;

#pragma pack(1)
/* RADIUS packet header */
typedef struct {
    u8 code;
    u8 id;
    u16 len; /* of the entire radius packet, including code, id, len, 
	      * authenticator and attributes */
    u8 authenticator[RADIUS_AUTHENTICATOR_LEN];
} __attribute__ ((packed)) radius_pkt_hdr_t;

/* RADIUS packet */
struct radius_pkt_t {
    radius_pkt_hdr_t hdr;
    radius_pkt_attr_t *attr; /* 0 or more */
};

struct radius_client_t {
    server_list_t *servers;
    u8 last_id; /* Last ID sent to RADIUS */
    radius_client_session_t *sessions; /* List of current sessions */
    int sock; /* Listening socket */
    u8 rcv_pkt_buf[RADIUS_BUF_MAX];   /* Receive buffer */
    int rcv_pkt_sz;   /* Received packet size */
    struct sockaddr rcv_pkt_addr;
    int rcv_pkt_addr_len;
};

struct radius_client_session_t {
    void *context; 
    radius_client_session_t *next;
    radius_client_t *client; /* if NULL - session is disabled */
    radius_pkt_recv_cb_t pkt_recv_cb;
    server_list_t *server; /* Current server */
    session_state_t state;
    u8 identifier;  /* Identifier field of RADIUS packet */
    u8 authenticator[RADIUS_AUTHENTICATOR_LEN]; /* Authenticator of last RADIUS 
						 * packet sent */
    u8 last_server_state[AUTH_STRING_LEN];
    u8 last_server_state_len;
    struct sockaddr send_pkt_addr;
    int send_pkt_addr_len;
};
#pragma pack()

static server_list_t *server_list_dup(server_list_t *server_list)
{
    server_list_t *dup_list = NULL, **dup_server;

    for (dup_server = &dup_list; server_list; 
	server_list = server_list->next, dup_server = &(*dup_server)->next)
    {
	*dup_server = (server_list_t *) zalloc_l(sizeof(server_list_t));
	if (!*dup_server)
	    break;
	**dup_server = *server_list;
    }

    return dup_list;
    
}

void server_list_free(server_list_t *server_list)
{
    while (server_list)
    {
	server_list_t *tmp = server_list;
	server_list = server_list->next;
	nfree(tmp);
    }
}

radius_client_t *radius_client_open(radius_params_t *params)
{
    radius_client_t *client;

    client = (radius_client_t *) zalloc_l(sizeof(radius_client_t));
    if (!client)
	return NULL;

    client->servers = server_list_dup(params->servers);

    /* Start listening on radius client port */ 
    client->sock = sock_socket(SOCK_DGRAM, htonl(INADDR_ANY), 
	htons(RADIUS_PORT));
    if (client->sock == -1)
	goto Error;
    
    if (sys_sock_set_nonblocking(client->sock) < 0)
	goto Error;

    client->rcv_pkt_addr_len = sizeof(client->rcv_pkt_addr);

    erecvfrom(client->sock, client->rcv_pkt_buf, sizeof(client->rcv_pkt_buf), 0,
	&client->rcv_pkt_addr, &client->rcv_pkt_addr_len, 
	(esyscall_event_t)radius_client_packet_recv, client, 0.0);
    
    return client;
Error:
    server_list_free(client->servers);
    if (client->sock >= 0)
	close(client->sock);
    free(client);
    return NULL;
}

void radius_client_close(radius_client_t *client)
{
    void *buf;
    radius_client_session_t *session;

    /* Stop Listening */
    esyscall_cancel(client->sock, ERECVFROM, NULL);

    /* Cancel all pending sends */
    while (!esyscall_cancel(client->sock, ESENDTO, &buf))
	free(buf);

    /* Remove radius_client from all radius client sessions. The sessions
     * should still be closed by each session's opener using the
     * radius_client_session_close. */
    for (session = client->sessions; session; session = session->next)
	session->client = NULL;

    server_list_free(client->servers);

    free(client);
}

/* Open a RADIUS session */
radius_client_session_t *radius_client_session_open(radius_client_t *client, 
    radius_pkt_recv_cb_t pkt_recv_cb, void *context)

{
    radius_client_session_t *session;
    
    session =
	(radius_client_session_t *)zalloc_l(sizeof(radius_client_session_t));
    if(!session)
	return NULL;

    session->client = client;
    session->pkt_recv_cb = pkt_recv_cb;
    session->context = context;
    
    /* Add session to task session list */
    session->next = client->sessions;
    client->sessions = session;
    
    return session;
}

/* Close a RADIUS session */
void radius_client_session_close(radius_client_session_t *session)
{
   /* Remove session from session list if exists */
   if( session != NULL)
   {
      if (session->client)
      {
         radius_client_session_t **sess = &session->client->sessions;
         
         for (; *sess; sess = &(*sess)->next)
         {
            if (*sess == session)
            {
               *sess = (*sess)->next;
               break;
            }
         }
      }
      free(session);
   }
}

static u32 nas_ip_address_get(radius_client_session_t *session)
{
    route_t *route =  route_to_host_get(session->server->ip);

    if (!route)
	return NULL;

    return get_if_ip(route->interface);
}

static int radius_client_packet_ser(u8 *buf, radius_pkt_t *pkt, int buf_len)
{
    radius_pkt_attr_t *attr;
    
    memcpy(buf, &pkt->hdr, sizeof(radius_pkt_hdr_t));
    buf += sizeof(radius_pkt_hdr_t);
    buf_len -= sizeof(radius_pkt_hdr_t);

    for (attr = pkt->attr; attr; attr = attr->next)
    {
	if (buf_len < attr->len)
	    return -1;
	*buf = attr->type;
	*(buf+1) = attr->len;
	memcpy(buf+2, attr->value, attr->len-2);
	buf += attr->len;
	buf_len -= attr->len;
    }
   
    return 0;
}

/* Set server socket address */
static void serv2sock_set(radius_client_session_t *session)
{
   if( session != NULL)
   {
      struct sockaddr_in *addr = (struct sockaddr_in *) &session->send_pkt_addr;
      u16 port;
      
      MZERO(*addr);
      addr->sin_family = AF_INET;
      
      port = htons(session->server->port);
      memcpy(&(addr->sin_port), &port, sizeof(port));
      
      memcpy( &(addr->sin_addr), &(session->server->ip), sizeof(session->server->ip));
      /* addr->sin_addr = session->server->ip; */
      session->send_pkt_addr_len = sizeof(struct sockaddr_in);
   }
}

/* Free send buffer */
void send_buf_free_cb(int ret, void *buf)
{
    if (ret < 0)
	rg_error(LWARN, "radius: Failed sending packet to RADIUS server");
    
    free(buf);
}
    
/* Send a RADIUS packet */
void radius_client_packet_send(radius_client_session_t *session, 
    radius_pkt_t *pkt)
{
    u8 *buf;
    radius_pkt_attr_t *attr;
    radius_client_t *client = session->client;
    u32 nas_ip;
    u16 pkt_len;
    u8 authenticator[RADIUS_AUTHENTICATOR_LEN];
    
    /* If Session isn't in session_list don't send packet */
    if (!client || !client->servers)
	return;
    
    /* Calculate RADIUS packet Identifier and add to packet */
    session->client->last_id++;
    pkt->hdr.id = session->identifier = session->client->last_id;

    /* Generate random packet authenticator vector */
    if (random_buf_get(session->authenticator, RADIUS_AUTHENTICATOR_LEN))
    {
	/* If random function fails change authenticator anyway */
	session->authenticator[session->client->last_id%
	    RADIUS_AUTHENTICATOR_LEN] += session->client->last_id;
    }
    memcpy(pkt->hdr.authenticator, session->authenticator, 
	RADIUS_AUTHENTICATOR_LEN);
    
    /* Add State attribute (if received from server) */
    if (session->last_server_state_len)
    {
	radius_packet_attr_add(pkt, PW_STATE, session->last_server_state, 
	    session->last_server_state_len);
    }
    
    /* Find server to send to. Initially one */
    session->server = client->servers;
	
    /* Add NAS-IP-Address attribute */
    if ((nas_ip = nas_ip_address_get(session)))
    {
	radius_packet_attr_add(pkt, PW_NAS_IP_ADDRESS, (u8 *)&nas_ip, 
	    sizeof(u32));
    }
    /* Add Message-Authenticator attribute (HMAC-MD5) */
    MZERO(authenticator);
    radius_packet_attr_add(pkt, PW_MESSAGE_AUTHENTICATOR, authenticator, 
	RADIUS_AUTHENTICATOR_LEN);

    pkt_len = sizeof(radius_pkt_hdr_t);
    for (attr = pkt->attr; attr; attr = attr->next)
	pkt_len += attr->len;
    
    /* Check that packet is not longer than RADIUS_BUF_MAX */
    if (pkt_len > RADIUS_BUF_MAX)
    {
	rg_error(LERR, "RADIUS packet len = %d > %d", pkt_len, 
	    RADIUS_BUF_MAX);
	return;
    }
    
    pkt->hdr.len = htons(pkt_len);

    buf = (u8 *) zalloc_l(pkt_len); 
    if (!buf)
	return;

    radius_client_packet_ser(buf, pkt, pkt_len); 

    /* Calculate HMAC-MD5 checksum of packet - message Authenticator */
    hmac_md5(buf, pkt_len, session->server->secret, 
	session->server->secret_len, authenticator);

    memcpy(buf+pkt_len-RADIUS_AUTHENTICATOR_LEN, authenticator, 
	RADIUS_AUTHENTICATOR_LEN);
    
    serv2sock_set(session);
  
    rg_error(LDEBUG, "radius: Sending RADIUS packet to server at %s",
	inet_ntoa(session->server->ip));
    
    /* Send RADIUS packet to server according to fallback server and retry
     * policy. Initially only one server and no retries */
    esendto(client->sock, buf, pkt_len, 0, &session->send_pkt_addr, 
	session->send_pkt_addr_len, send_buf_free_cb, buf, 0.0);
}

/* Authenticate a received RADIUS packet */
static int radius_packet_authenticate(radius_client_t *client, 
    radius_client_session_t *session)
{
    int ret = -1;
    u8 auth_calc[RADIUS_AUTHENTICATOR_LEN];
    MD5_CTX	context;

    MD5_Init(&context);
    MD5_Update(&context, client->rcv_pkt_buf, 4);
    MD5_Update(&context, session->authenticator, RADIUS_AUTHENTICATOR_LEN);
    MD5_Update(&context, client->rcv_pkt_buf+4+RADIUS_AUTHENTICATOR_LEN,
	client->rcv_pkt_sz-4-RADIUS_AUTHENTICATOR_LEN);
    MD5_Update(&context, session->server->secret, session->server->secret_len);
    MD5_Final(auth_calc, &context);

    ret = memcmp(auth_calc, client->rcv_pkt_buf+4, RADIUS_AUTHENTICATOR_LEN);

    if (ret)
    {
	rg_error(LWARN, "radius: RADIUS authentication failed, ID = %d",
	    session->identifier);
    }

    return ret;
}

/* Receive incoming RADIUS packet */
static void radius_client_packet_recv(int ret, void *o)
{
    radius_client_t *client = (radius_client_t *) o;
    radius_client_session_t *session;
    radius_pkt_t *pkt;
    radius_pkt_attr_t **attr;
    u8 *buf;
    int buf_len;
    
    client->rcv_pkt_sz = ret;
    if (client->rcv_pkt_sz < sizeof(radius_pkt_hdr_t))
	return;

    pkt = (radius_pkt_t *) zalloc_l(sizeof(radius_pkt_t));
    if (!pkt)
	return;
    
    /* Get RADIUS packet header */
    pkt->hdr = *(radius_pkt_hdr_t *) client->rcv_pkt_buf;

    /* Get RADIUS session by packet ID */
    for (session = client->sessions; session; session = session->next)
    {
	if (session->identifier == pkt->hdr.id)
	    break;
    }

    if (!session)
	goto Exit;
    
    /* Authenticate packet */
    if (radius_packet_authenticate(client, session))
	goto Exit;
	
     /* IT: */
    pkt->hdr.len = htons(pkt->hdr.len);


   /* Get RADIUS packet attributes */
    buf = client->rcv_pkt_buf + sizeof(radius_pkt_hdr_t);
    buf_len = client->rcv_pkt_sz - sizeof(radius_pkt_hdr_t);
    attr = &(pkt->attr);
    while (buf_len > 2)
    {
	u8 type = *buf;
	u8 len = *(buf+1);

	if (buf_len < len)
	    break;
	if (radius_packet_attr_add(pkt, type, buf+2, len-2)) 
	    break;
	/* Save State if exists */
	if (type == PW_STATE)
	{
	    session->last_server_state_len = len-2;
	    memcpy(session->last_server_state, buf+2, len-2);
	}
	buf += len;
	buf_len -= len;
    }
    
    session->pkt_recv_cb(session->context, pkt);

Exit:
    radius_packet_free(pkt);
    erecvfrom(client->sock, client->rcv_pkt_buf, sizeof(client->rcv_pkt_buf),
	0, &client->rcv_pkt_addr, &client->rcv_pkt_addr_len, 
	(esyscall_event_t)radius_client_packet_recv, client, 0.0);
}

/* Toolkit for building RADIUS packets */
/* Create an RADIUS empty packet */
radius_pkt_t *radius_packet_create(void)
{
    /* Allocate a new RADIUS packet */
    radius_pkt_t *pkt = (radius_pkt_t *) zalloc_l(sizeof(radius_pkt_t));

    if (!pkt)
	goto Exit;
    
    pkt->hdr.code = ACCESS_REQUEST;
    
Exit:
    return pkt;
}

/* Free a RADIUS packet */
void radius_packet_free(radius_pkt_t *pkt)
{
   while (pkt->attr)
   {
       radius_pkt_attr_t *tmp_attr;

       tmp_attr = pkt->attr;
       pkt->attr = pkt->attr->next;
       free(tmp_attr);
   }
   free(pkt);
}

/* Add an attribute to an existing packet. Adds the attribute at the end
 * of the packet. value_len is length of attribute without type & length */
int radius_packet_attr_add(radius_pkt_t *pkt, int type, u8 *value, 
    u8 value_len)
{
    radius_pkt_attr_t **attr;

    if (value_len > AUTH_STRING_LEN)
	return -1;
    /* Add attribute to packet */
    for (attr = &(pkt->attr); *attr; attr = &(*attr)->next);

    *attr = (radius_pkt_attr_t *) zalloc_l(sizeof(radius_pkt_attr_t));
    if (!*attr)
	return -1;

    (*attr)->type = type;
    (*attr)->len = value_len+2;
    memcpy((*attr)->value, value, value_len);
    /* Return 0 if successful */
    return 0;
}
   
/* Get next attribute of the defined type, NULL means first attribute */
radius_pkt_attr_t *radius_packet_attr_get(radius_pkt_t *pkt, int type, 
    radius_pkt_attr_t *last_attr)
{
    radius_pkt_attr_t *attr;

    for (attr = last_attr?last_attr->next:pkt->attr; 
	attr && attr->type != type; attr = attr->next);
    
    return attr;
}

/* Get code of RADIUS packet */
radius_codes_t radius_packet_code_get(radius_pkt_t *pkt)
{
    return pkt->hdr.code;
}

/* Decode encoded key from MS-MPPE-Recv-Key and MS-MPPE-Send-Key 
 * 0 - success */
static int radius_packet_mppe_key_decode(radius_client_session_t *session, 
    radius_pkt_vendor_t *vendor, u8 **key, int *key_len)
{
    u8 salt[2];
    u8 *b = NULL;
    u8 *p = NULL;
    int enc_len;
    int i;

    enc_len = vendor->len - 4;
    if (enc_len <= 0 || enc_len % 16)
	return -1;

    memcpy(&salt, vendor->data, 2);
    if (!salt[0] && !salt[1])
    {
	p = vendor->data + 2;
	goto Exit;
    }

    b = (u8 *) zalloc_l(enc_len);
    p = (u8 *) zalloc_l(enc_len);
    if (!b || !p)
	goto Error;
    
    for (i=0; i<enc_len/16; i++)
    {
	MD5_CTX	context;
	int j;

	MD5_Init(&context);
	MD5_Update(&context, session->server->secret, 
	    session->server->secret_len);
	if (i)
	    MD5_Update(&context, vendor->data+2+16*(i-1), 16);
	else 
	{
	    MD5_Update(&context, session->authenticator, 
		RADIUS_AUTHENTICATOR_LEN);
	    MD5_Update(&context, salt, 2); 
	}
	MD5_Final(b+16*i, &context);
	
	for (j=0; j<16; j++)
	    p[16*i+j] = b[16*i+j] ^ *(vendor->data+2+16*i+j);

    }
Exit:
    if (*p > enc_len-1)
	goto Error;
    *key_len = *p;
    *key = (u8 *) zalloc_l(*key_len);
    if (!*key)
	goto Error;
    memcpy(*key, p + 1, *key_len);

    /* If salt is zero p was not allocated and points to vendor->data+2 */
    if (salt[0] || salt[1])
	nfree(p);
    nfree(b);
    return 0;
Error:
    nfree(b);
    nfree(p);
    return -1;
}

/* Get key from MS-MPPE-Recv-Key and MS-MPPE-Send-Key Microsoft Vendor 
 * attributes, 0 - Success */
int radius_packet_ms_mppe_key_get(radius_client_session_t *session, 
    radius_pkt_t *pkt, u8 **master_key, int *master_key_len)
{
    radius_pkt_attr_t *attr = NULL;
    radius_pkt_vendor_t *vendor;
    u8 *recv_key = NULL;
    int recv_key_len;
    u8 *send_key = NULL;
    int send_key_len;
    int ret = -1;
   
    while ((attr = radius_packet_attr_get(pkt, PW_VENDOR_SPECIFIC, attr)))
    {
	if (attr->len < 2 + sizeof(radius_pkt_vendor_t) + 2)
	    continue;

	vendor = (radius_pkt_vendor_t *) attr->value;
	if (vendor->id != htonl(PW_VENDOR_MICROSOFT))
	    continue;

	switch (vendor->type)
	{
	case PW_MS_MPPE_RECV_KEY:
	    if (recv_key)
		goto Error;
	    radius_packet_mppe_key_decode(session, vendor, &recv_key, 
		&recv_key_len);
	    break;
	case PW_MS_MPPE_SEND_KEY:
	    if (send_key)
		goto Error;
	    radius_packet_mppe_key_decode(session, vendor, &send_key, 
		&send_key_len);
	    break;
	default:
	    continue;
	}
    }

    if (!recv_key || !send_key)
	goto Error;

    *master_key_len = recv_key_len+send_key_len;
    *master_key = (u8 *) zalloc_l(*master_key_len);
    if (!*master_key)
	goto Error;

    memcpy(*master_key, recv_key, recv_key_len);
    memcpy(*master_key+recv_key_len, send_key, send_key_len); 

    ret = 0;
Error:
    nfree(recv_key);
    nfree(send_key);
    return ret;
}
