/* LICENSE_CODE JUNGO */

#include "rg_error.h"
#include "event.h"
#include "esyscall.h"
#include "misc_funcs.h"
#include <errno.h>

#define rg_error(level,str,args...) printf(str,##args)


#ifdef CONFIG_RG_TARGET_BSD
#include "vx_net_fixup.h"
#endif

#define OP2MODE(op) ((op)&ERECVALL ? EFD_READ : EFD_WRITE)
#define ESC2MODE(esc) OP2MODE((esc)->op)
#define ESC2FUNC(esc) ((esc)->op&ERECVALL ? esyscall_recv_et : esyscall_send_et)

typedef struct esyscall_op_t {
    struct esyscall_op_t *next;
    int op;
    
    esyscall_event_t cb;
    void *o;
    double ms_timeout;
    int timed_out; /* if != 0 - this is timeout event */
    
    /* fields corresponing to standard send/recv prototypes */
    int fd;
    struct msghdr *hdr;
    void *buf;
    int len;
    int flags;
    struct sockaddr *othr;
    int *othr_len;
    int dst_len;
    /* standard prototypes end */

#ifdef CONFIG_RG_TARGET_BSD
    void *ext_context; /* Context needed for canceling the fixup timer */
#endif
} esyscall_op_t;

typedef void (*esyscall_cb_t)(int, esyscall_op_t *);

static esyscall_op_t *esyscall_pending_ops;

static void esyscall_cancel_(esyscall_op_t *esc);

/* return 1 if both fd and mode direction (send/receive) are the same, 0
 * otherwise
 */
static __inline__ int is_same_fd_mode(esyscall_op_t *esc1, esyscall_op_t *esc2)
{
    return esc1->fd == esc2->fd && 
	/* note: the ! converts the integer value of (esc->op & ERECVALL)
	 * to boolean (i.e. both modes are receive or both are send)
	 */
	!(esc1->op & ERECVALL) == !(esc2->op & ERECVALL);
}
    
static void esyscall_et_end(esyscall_op_t *esc, int ret)
{
    esyscall_event_t cb;
    void *o;
    int errno_save = errno;
    
    /* error handling. On timeout we must not return since we must cancel the
     * event and call the callback function. Also we dont want to write another
     * message to the log since it has been written in esyscall_timeout().
     */
    if (!esc->timed_out && ret<0)
    {
	/* on EAGAIN/INTR/WOULDBLOCK we want to stick around for another go */
	if (errno==EAGAIN || errno==EINTR || errno==EWOULDBLOCK)
	    return;
	/* Connection Refused error is logged as debug, since this is
	 * (usually) client problem. This error occurrs occasionally when a
	 * client loads a web page with many references.
	 */
	rg_error_f(errno==ECONNREFUSED ? LDEBUG : LERR,
	    "fd %d op %d: failed operation: %m",
	    esc->fd, esc->op);
    }
    cb = esc->cb;
    o = esc->o;
    /* clean up */
    esyscall_cancel_(esc);
    /* call back to the user */
    if (cb)
    {
	errno = errno_save;
	cb(ret, o);
    }
}

static void esyscall_send_et(int fd, esyscall_op_t *esc)
{
    int rc = 0;

    if (esc->op & ESEND)
	rc = send(esc->fd, esc->buf, esc->len, esc->flags);
    else if (esc->op & ESENDTO)
    {
	rc = sendto(esc->fd, esc->buf, esc->len, esc->flags, esc->othr,
	    esc->dst_len);
    }
    else /* ESENDMSG */
	rc = sendmsg(esc->fd, esc->hdr, esc->flags);

    esyscall_et_end(esc, rc);
}

static void esyscall_recv_et(int fd, esyscall_op_t *esc)
{
    int rc = 0;

    /* actually try to read */
    if (esc->op & ERECVMSG)
	rc = recvmsg(esc->fd, esc->hdr, esc->flags);
    else
    {
	rc = recvfrom(esc->fd, esc->buf, esc->len, esc->flags, esc->othr,
	    esc->othr_len);
    }

    esyscall_et_end(esc, rc);
}

static void esyscall_timeout(esyscall_op_t *esc)
{
    esyscall_cb_t event = ESC2FUNC(esc);

    rg_error_f(LDEBUG, "on fd %d op %d\n", esc->fd, esc->op);

    esc->timed_out = 1;

    /* time's up - give it a last try */
    event(esc->fd, esc);
}

static void esyscall_start_op(esyscall_op_t *esc)
{
    event_fd_set(esc->fd, ESC2MODE(esc), (efd_func_t)ESC2FUNC(esc), esc);
}

static void wait_for_ready_and_start(esyscall_op_t *esc)
{
#ifdef CONFIG_RG_TARGET_BSD
    int mode = ESC2MODE(esc);

    /* We fix if it is a send operation and the destination is known */
    if (mode == EFD_WRITE && esc->othr && esc->othr->sa_family == AF_INET)
    {
	esc->ext_context = send_wait_for_ready_and_start_vx(esc->othr, 
	    (void *)esc, (etimer_func_t)esyscall_start_op);
    }
    else
#endif
	esyscall_start_op(esc);
}

static void esyscall_cancel_(esyscall_op_t *esc)
{
    esyscall_op_t **cur;
    int mode;

    /* search the list for esc */
    for (cur=&esyscall_pending_ops; *cur && *cur != esc; cur=&(*cur)->next);

    if (!*cur)
	return;
    
    *cur = (*cur)->next;
    event_timer_del((etimer_func_t)esyscall_timeout, esc);
    mode = ESC2MODE(esc);
    event_fd_clr(esc->fd, mode);
#ifdef CONFIG_RG_TARGET_BSD
    if (esc->ext_context)
	send_wait_for_ready_cancel_vx((void *)esc->ext_context);
#endif

    /* Search the list for another entry with the same fd and mode. This may be
     * located before the found entry in case calling from function
     * syscall_cancel_by_cb()
     */
    for (cur=&esyscall_pending_ops; *cur && !is_same_fd_mode(*cur, esc);
	cur=&(*cur)->next);

    if (*cur)
	wait_for_ready_and_start(*cur);

    free(esc);
}

static void esyscall_op_init(esyscall_op_t *esc, int s, int op,
    struct msghdr *hdr, void *buf, int len, unsigned int flags,
    struct sockaddr *othr, int *fromlen, int tolen, esyscall_event_t cb,
    void *o, double ms_timeout)
{
    /* initialize new request */
    esc->fd = s;
    esc->op = op;
    esc->hdr = hdr;
    esc->buf = buf;
    esc->len = len;
    esc->flags = flags;
    /* force non-blocking */
    esc->flags |= MSG_DONTWAIT;
    esc->othr = othr;
    esc->othr_len = fromlen;
    esc->dst_len = tolen;
    esc->cb = cb;
    esc->o = o;
    esc->ms_timeout = ms_timeout;
}

static void esyscall_op_set(esyscall_op_t *esc)
{
    esyscall_op_t **end;
    int same_fd_mode_found = 0;

    /* goto list end, and also find if there is another entry for the
     * same fd and mode.
     */
    for (end=&esyscall_pending_ops; *end; end=&(*end)->next)
    {
	if (*end != esc && is_same_fd_mode(*end, esc))
	    same_fd_mode_found = 1;
    }

    /* put in end of local pending list */
    esc->next = NULL;
    *end = esc;

    /* schedule timeout */
    if (esc->ms_timeout)
	event_timer_set(esc->ms_timeout, (etimer_func_t)esyscall_timeout, esc);

    if (!same_fd_mode_found)
	wait_for_ready_and_start(esc);
}

/* exported api
 * ============
 */

void erecv(int s, void *buf, int len, unsigned int flags, esyscall_event_t cb,
    void *o, double ms_timeout)
{
    esyscall_op_t *esc;

    esc = (esyscall_op_t *)zalloc_l(sizeof(esyscall_op_t));
    if (!esc)
	return;
    esyscall_op_init(esc, s, ERECV, NULL, buf, len, flags, NULL, NULL, 0, cb, o,
	ms_timeout);
    esyscall_op_set(esc);
}


void erecvfrom(int s, void *buf, int len, unsigned int flags,
    struct sockaddr *from, int *fromlen, esyscall_event_t cb, void *o,
    double ms_timeout)
{
    esyscall_op_t *esc;

    esc = (esyscall_op_t *)zalloc_l(sizeof(esyscall_op_t));
    if (!esc)
	return;
    esyscall_op_init(esc, s, ERECVFROM, NULL, buf, len, flags, from, fromlen, 0,
	cb, o, ms_timeout);
    esyscall_op_set(esc);
}

void esend(int s, const void *msg, int len, unsigned int flags,
    esyscall_event_t cb, void *o, double ms_timeout)
{
    esyscall_op_t *esc;

    esc = (esyscall_op_t *)zalloc_l(sizeof(esyscall_op_t));
    if (!esc)
	return;
    esyscall_op_init(esc, s, ESEND, NULL, (void *)msg, len, flags, NULL, NULL,
	0, cb, o, ms_timeout);
    esyscall_op_set(esc);
}

void esendto(int s, const void *msg, int len, unsigned int flags,
    const struct sockaddr *to, int tolen, esyscall_event_t cb, void *o,
    double ms_timeout)
{
    esyscall_op_t *esc;

    esc = (esyscall_op_t *)zalloc_l(sizeof(esyscall_op_t));
    if (!esc)
	return;
    esyscall_op_init(esc, s, ESENDTO, NULL, (void *)msg, len, flags,
	(struct sockaddr *)to, NULL, tolen, cb, o, ms_timeout);
    esyscall_op_set(esc);
}


void erecvmsg(int s, struct msghdr *msg, unsigned int flags,
    esyscall_event_t cb, void *o, double ms_timeout)
{
    esyscall_op_t *esc;

    esc = (esyscall_op_t *)zalloc_l(sizeof(esyscall_op_t));
    if (!esc)
	return;
    esyscall_op_init(esc, s, ERECVMSG, msg, NULL, 0, flags, NULL, NULL, 0, cb,
	o, ms_timeout);
    esyscall_op_set(esc);
}

void esendmsg(int s, const struct msghdr *msg, unsigned int flags,
    esyscall_event_t cb, void *o, double ms_timeout)
{
    esyscall_op_t *esc;

    esc = (esyscall_op_t *)zalloc_l(sizeof(esyscall_op_t));
    if (!esc)
	return;
    esyscall_op_init(esc, s, ESENDMSG, (struct msghdr *)msg, NULL, 0, flags,
	NULL, NULL, 0, cb, o, ms_timeout);
    esyscall_op_set(esc);
}
 
int esyscall_cancel(int fd, int op, void **ret_ctx)
{
    esyscall_op_t *cur;
    
    for (cur = esyscall_pending_ops;
	cur && !(fd==cur->fd && op&cur->op); cur = cur->next);
    if (cur)
    {
	if (ret_ctx)
	    *ret_ctx = cur->o;
	esyscall_cancel_(cur);

	return 0;
    }

    return -1;
}

int esyscall_cancel_by_cb(esyscall_event_t cb, void *o)
{
    esyscall_op_t *cur;
    
    for (cur = esyscall_pending_ops;
	cur && !(cb==cur->cb && o==cur->o); cur = cur->next);
    if (cur)
    {
	esyscall_cancel_(cur);
	return 0;
    }
    return -1;
}

