/* LICENSE_CODE JUNGO */

#define OS_INCLUDE_STD
#define OS_INCLUDE_SELECT
#define OS_INCLUDE_TIME
#define OS_INCLUDE_SIGNAL

#include "rg_error.h"

#include <os_includes.h>
#include <os_libc_extra.h>
#include "event.h"
#include "misc_funcs.h"
#include "syscalls.h"

#define rg_error(level,str,args...) printf(str,##args)
#define rg_perror(level,str,args...) printf(str,##args)

// general time conversion functions

#define MIN_TICK_RESOLUTION 100

void ms2tv(struct timeval *tv, double ms)
{
    tv->tv_sec = (unsigned long)(ms/1000);
    tv->tv_usec = (unsigned long)((ms - tv->tv_sec*1000) * 1000);
}

double tv2ms(struct timeval *tv)
{
    return ((double) (unsigned long) tv->tv_usec)/1000
	+ ((double) (unsigned long) tv->tv_sec)*1000;
}

static int exact_timers;

void event_timer_set_exact(int enable)
{
    exact_timers = enable;
}

static clock_t ticks_per_second;

double get_time_from_boot(void)
{
    if (exact_timers)
    {
	struct timeval tv;

	return tv2ms(sys_get_time(&tv));
    }
    return ((double)sys_get_ticks_from_boot()) / ticks_per_second * 1000;
}

// event strctures definitions

typedef struct event_timer_t {
    struct event_timer_t *next;
    int stamp;
    double expire;
    double period;
    etimer_func_t func;
    void *data;
} event_timer_t;
typedef struct event_signal_t {
    struct event_signal_t *next;
    int sig;
    int called;
    esignal_func_t func;
    void *data;
} event_signal_t;
typedef struct event_fd_t {
    struct event_fd_t *next;
    int fd;
    int type;
    int suspend;
    int stamp;
    efd_func_t func;
    void *data;
} event_fd_t;

#ifndef CONFIG_RG_TARGET_VXWORKS
typedef struct event_sigchild_t {
    struct event_sigchild_t *next;
    pid_t pid;
    esigchild_func_t func;
    void *data;
    int exited; /* Filled by the signal handler when this PID has exited. */
    int status; /* Exit status */
} event_sigchild_t;
#endif

// variables used for operation of the event loop

fd_set event_readfds, event_writefds, event_exceptfds;
event_timer_t *event_timer_queue;
int event_loop_stamp;
double event_loop_now;
time_t event_loop_date_now;
int event_loop_quit;
event_fd_t *event_fd_list;
int event_fd_nset;
int event_fd_max;
event_signal_t *event_signal_list;
int event_signal_called;

#ifndef CONFIG_RG_TARGET_VXWORKS
event_sigchild_t *event_sigchild_list;
#endif

// event functions

double event_get_loop_now(void)
{
    return event_loop_now;
}

time_t event_loop_date_now_get(void)
{
    return event_loop_date_now;
}

static void event_timer_add(event_timer_t *et)
{
    event_timer_t **search;
    for (search = &event_timer_queue; *search; search = &(*search)->next)
    {
	if ((*search)->expire>et->expire)
	    break;
    }
    et->next = *search;
    *search = et;
}

/* WARNING : We are using now get_time_from_boot instead of gettimeofday, which
 * was a lot more precise. Order of events that where set to fire around the
 * same time is not gauranteed.
 */
static void event_timer_set_full(double ms, etimer_func_t func, void *data,
    int period)
{
    event_timer_t *et;
    double now;

    event_timer_del(func, data);
    now = get_time_from_boot();
    et = zalloc_e(sizeof(event_timer_t));
    et->func = func;
    et->data = data;
    et->period = period ? ms : 0;
    et->expire = now + ms;
    et->stamp = event_loop_stamp;
    event_timer_add(et); 
}

void event_timer_set_period(double ms, etimer_func_t func, void *data)
{
    event_timer_set_full(ms, func, data, 1);
}

void event_timer_set(double ms, etimer_func_t func, void *data)
{
    event_timer_set_full(ms, func, data, 0);
}

static event_timer_t **event_timer_get(etimer_func_t func, void *data)
{
    event_timer_t **et;
    for (et = &event_timer_queue; *et; et = &(*et)->next)
    {
	if ((*et)->func==func && (*et)->data==data)
	    return et;
    }
    return NULL;
}

double event_timer_left(etimer_func_t func, void *data)
{
    event_timer_t **et;
    
    if (!(et = event_timer_get(func, data)))
	return -1;

    return (*et)->expire - event_get_loop_now();
}

int event_timer_exist(etimer_func_t func, void *data)
{
    return event_timer_get(func, data) ? 1 : 0;
}

void event_timer_del(etimer_func_t func, void *data)
{
    event_timer_t **et, *et_del;
    
    et = event_timer_get(func, data);
    if (!et)
	return;
    et_del = *et;
    *et = et_del->next;
    free(et_del);
}

static void event_call_expiered_timers(void)
{
    event_timer_t **et, *et_cur, et_save;
    
    while (1)
    {
	// get the first timer from the queue, while
	// ignoring those that where added in this loop (according
	// to the stamp).
	// we want to delay handling events scheduled in this loop,
	// (only to handle them in the next loop).
	for (et = &event_timer_queue; *et; et = &(*et)->next)
	{
	    if ((*et)->expire<=event_loop_now && (*et)->stamp!=event_loop_stamp)
		break;
	}
	if (!*et)
	    break;
	
	// remove timer from queue
	et_cur = *et;
	*et = (*et)->next;

	et_save = *et_cur;

	// reschedule if needed
	if (et_cur->period)
	{
	    // re-sched the timer with the next id, so we will not 
	    // execute the same periodic timer twice in this loop
	    et_cur->stamp = event_loop_stamp;
	    et_cur->expire = event_loop_now + et_cur->period;
	    event_timer_add(et_cur);
	}
	else
	    free(et_cur);
	et_save.func(et_save.data);
	if (event_loop_quit)
	    break;
    }
}

static fd_set *event_fds_choose(int type, fd_set *readfds, fd_set *writefds,
    fd_set *exceptfds)
{
    return type==EFD_READ ? readfds :
	type==EFD_WRITE ? writefds :
	type==EFD_EXCEPT ? exceptfds : NULL;
}

static event_fd_t **event_fd_get_p(int fd, int type)
{
    event_fd_t **efd;
    // FIXME: scaning a linked list is a performance problem - we 
    // should locate this faster (with some kind of hash table)
    for (efd = &event_fd_list; *efd; efd = &(*efd)->next)
    {
	if ((*efd)->fd==fd && (*efd)->type>=type)
	    return efd;
    }
    return NULL;
}

static event_fd_t *event_fd_get(int fd, int type)
{
    event_fd_t **efd = event_fd_get_p(fd, type);
    return efd && (*efd)->fd==fd && (*efd)->type==type ? *efd : NULL; 
}

static event_fd_t **event_fd_get_next(int fd, int type)
{
    event_fd_t **efd;
    // FIXME: scaning a linked list is a performance problem - we 
    // should locate this faster (with some kind of hash table)
    for (efd = &event_fd_list; *efd; efd = &(*efd)->next)
    {
	if ((*efd)->fd>fd || ((*efd)->fd==fd && (*efd)->type>=type))
	    break;
    }
    return efd;
}

#ifndef CONFIG_RG_TARGET_VXWORKS
void event_sigchild_set(pid_t pid, esigchild_func_t func, void *data)
{
    event_sigchild_t *echild = zalloc_e(sizeof(event_sigchild_t));
    echild->pid = pid;
    echild->func = func;
    echild->data = data;

    echild->next = event_sigchild_list;
    event_sigchild_list = echild;
}

/* pid is either the pid we are waiting on, or -1 for whatever pid.
 * When pid == -1 the fucntion is non-blocking and rips all children.
 * When pid is a valid pid, we block until the child exists.
 * For blocking mode (pid > 0), return value 0 indicate that the process with 
 * the requested pid exited and its exit status saved in ret_status.
 */
static int event_sigchild_cb_do(pid_t pid, int *ret_status)
{
    pid_t new_pid;
    int status;
    event_sigchild_t *e;

    while ((new_pid = waitpid(pid, &status, pid == -1 ? WNOHANG : 0)))
    {
	if (new_pid == -1)
	{
	    /* Resume waiting if wait was interupted. */
	    if (errno == EINTR)
		continue;
	    /* ECHILD should not happen but does anyway on some systems. */
	    if (errno != ECHILD)
	    {
		rg_error(LERR, "Error waiting on child: %s.", 
		    strerror(errno));
	    }
	    break;
	}

	/* No child died according to the man (in reality, you get -1 with
	 * ECHILD), see above.
	 */
	if (!new_pid)
	    return -1;

	for (e = event_sigchild_list; e && e->pid != new_pid; e = e->next);
	if (!e)
	{
	    if (pid == -1)
	    {
		rg_error(LWARN, "Process %d exited. No action was taken", 
		    new_pid);
	    }
	}
	else
	{
	    e->exited = 1;
	    e->status = status;
	}
	/* The one we're looking for (in a blocking manner) was found,
	 * so we can now exit.
	 */
	if (new_pid == pid)
	{
	    if (ret_status)
		*ret_status = status;
	    return 0;
	}
    }

    return -1;
}

void event_sigchild_disable(void)
{
    sigset_t set;
    
    sigemptyset(&set);
    sigaddset(&set, SIGCHLD);

    if (sigprocmask(SIG_BLOCK, &set, NULL))
	rg_error(LPANIC, "Error setting masking for SIGCHLD signal");
}

void event_sigchild_enable(void)
{
    sigset_t set;
    
    sigemptyset(&set);
    sigaddset(&set, SIGCHLD);
    
    if (sigprocmask(SIG_UNBLOCK, &set, NULL))
	rg_error(LPANIC, "Error setting masking for SIGCHLD signal");
}
   
/* Blocking wait for a specific child. Return value 0 indicate we've waited
 * on the requested pid and its status is saved on ret_status.
 * Should be called between event_sigchild_disable() and 
 * event_sigchild_enable().
 */
int event_wait_for_child(pid_t pid, int *ret_status)
{
    /* External interface for waiting to a singal child in a block mode */
    if (pid < 0)
	return -1;

    return event_sigchild_cb_do(pid, ret_status);
}

void event_sigchild_del(pid_t pid, int kill_proc)
{
    event_sigchild_t **e, *tmp;

    for (e = &event_sigchild_list; *e && (*e)->pid != pid; e = &(*e)->next);

    if (*e)
    {
	tmp = *e;
	*e = (*e)->next;
	free(tmp);
    }
    if (!kill_proc)
	return;

    event_sigchild_disable();
   
    kill(pid, SIGKILL);

    event_sigchild_cb_do(pid, NULL);

    event_sigchild_enable();
}

/* SIGCHLD signal handler. */
static void event_sigchild_cb(int sig, void *data)
{
    event_sigchild_cb_do(-1, NULL);
}

static void event_call_sigchild_handlers(void)
{
    event_sigchild_t **e = &event_sigchild_list;

    while (*e)
    {
	if ((*e)->exited)
	{
	    event_sigchild_t *tmp;
	    tmp = *e;
	    *e = (*e)->next;
	    tmp->func(tmp->pid, tmp->data, tmp->status);
	    free(tmp);
	}
	else
	    e = &(*e)->next;
    }
}
#endif

static void event_signal_cb(int sig)
{
    event_signal_t *es;
    for (es = event_signal_list; es; es = es->next)
    {
	if (es->sig==sig)
	    es->called++;
    }
    event_signal_called = 1;
}

static void event_call_signals(void)
{
    event_signal_t *es;
    if (!event_signal_called)
	return;
    event_signal_called = 0;
    for (es = event_signal_list; es; es = es->next)
    {
	if (es->called)
	{
	    es->called = 0;
	    es->func(es->sig, es->data);
	}
    }
}

void event_signal_set_flags(int sig, esignal_func_t func, void *data,
    int flags)
{
    event_signal_t *es;
    struct sigaction action;

    event_signal_del(sig);
    es = zalloc_e(sizeof(event_signal_t));
    es->func = func;
    es->data = data;
    es->sig = sig;
    es->next = event_signal_list;
    event_signal_list = es;

    action.sa_handler = event_signal_cb;
    sigemptyset(&action.sa_mask);
    action.sa_flags = flags;

    sigaction(sig, &action, NULL);
}

void event_signal_set(int sig, esignal_func_t func, void *data)
{
    event_signal_set_flags(sig, func, data, 0);
}

void event_signal_del(int sig)
{
    event_signal_t **es, *es_del;
    struct sigaction action;

    for (es = &event_signal_list; *es && (*es)->sig!=sig;
	es = &(*es)->next);
    if (!*es)
	return;

    action.sa_handler = SIG_DFL;
    sigemptyset(&action.sa_mask);
    action.sa_flags = 0;
    
    sigaction((*es)->sig, &action, NULL);

    es_del = *es;
    *es = es_del->next;
    free(es_del);
}

// performs a complete scan of all expiered timers,
// and waits for fd events
// returns:
// 0 - normal termination
// 1 - terminate main wait - no more threads to wait on
// negative - error
static int event_loop_single(void)
{
    struct timeval timeout;
    fd_set readfds, writefds, exceptfds;
    int n;
    event_fd_t **scan_efd;
    int fds_copy_bytes;

    event_call_signals();
#ifndef CONFIG_RG_TARGET_VXWORKS
    event_call_sigchild_handlers();
#endif

    // the stamp allows us to diffrentiate between the first time we handle
    // a timer in the loop, and the second time we see it's periodic
    // resched
    event_loop_stamp++;
    event_loop_now = get_time_from_boot();
    event_loop_date_now = time(NULL);
	
    event_call_expiered_timers();
    if (exact_timers)
	memset(&timeout, 0, sizeof(timeout));
    else if (event_timer_queue)
    {
	double next_timeout = event_timer_queue->expire - event_loop_now;
	if (next_timeout<0)
	    next_timeout = 0;
	ms2tv(&timeout, next_timeout);
    }
    
    // no more events or fds to wait on, our work is done -
    // terminate the event_loop()
    if (!event_timer_queue && !event_fd_nset)
	return 1;
    
    // the fd_sets are copied each time since select() modifys them.
    // since they can be quite big, we only copy part of them:
    // first we count the bytes (devide by 8 and add 1).
    // then we round up to 4 (sizeof(int)) so memcpy will work faster
    fds_copy_bytes = MIN((event_fd_max / 8 + 1 + 4) & ~3, sizeof(fd_set));
    memcpy(&readfds, &event_readfds, fds_copy_bytes);
    memcpy(&writefds, &event_writefds, fds_copy_bytes);
    memcpy(&exceptfds, &event_exceptfds, fds_copy_bytes);

    // wait for events of fds
    n = select(event_fd_max + 1, &readfds, &writefds, &exceptfds,
	event_timer_queue ? &timeout : NULL);

    if (n<0)
    {
	if (errno==EINTR)
	    return 0;
	return rg_perror(LERR, "event_loop_single: select()");
    }
    if (!n)
	return 0;

    // scan the fd events, and call the handlers
    event_fd_max = 0;
    for (scan_efd = &event_fd_list; *scan_efd;)
    {
	event_fd_t *efd, efd_tmp;
	fd_set *pfds;

	efd = *scan_efd;
	// move to next fd event
	scan_efd = &(*scan_efd)->next;

	if (efd->fd>event_fd_max)
	    event_fd_max = efd->fd;
	if (efd->suspend)
	    continue;
	
	pfds = event_fds_choose(efd->type, &readfds, &writefds, &exceptfds);
	if (!FD_ISSET(efd->fd, pfds))
	    continue;
	if (efd->stamp==event_loop_stamp)
	    continue;
	
	// save event information, since the handler might modify it
	efd_tmp = *efd;
	efd->func(efd->fd, efd->data);
	if (event_loop_quit)
	    return 0;

	// get this event again (or the next one after it).
	// we need to do this since the handler might have been destroyed.
	scan_efd = event_fd_get_next(efd_tmp.fd, efd_tmp.type+1);
    }
    return 0;
}

void event_loop_init(void)
{
    event_timer_queue = NULL;
    event_fd_list = NULL;
    FD_ZERO(&event_readfds);
    FD_ZERO(&event_writefds);
    FD_ZERO(&event_exceptfds);
    event_loop_quit = 0;
    event_fd_nset = 0;
    if (!(ticks_per_second = sys_get_clock_rate()))
	rg_error(LEXIT, "error getting ticks per second");
    if (ticks_per_second < MIN_TICK_RESOLUTION)
    {
	rg_error(LWARN, "ticks per second lower than "
	    "minimum resolution required!\n"
	    "Event fire time may not be exact.");
    }
    event_loop_now = get_time_from_boot();
    event_loop_date_now = time(NULL);
    event_signal_list = NULL;
    event_signal_called = 0;

#ifndef CONFIG_RG_TARGET_VXWORKS
    event_sigchild_list = NULL;

    event_signal_set_flags(SIGCHLD, event_sigchild_cb, NULL,
	SA_NOCLDSTOP | SA_NODEFER);
#endif
}

static void event_timer_queue_free(event_timer_t *et)
{
    while (et)
    {
	event_timer_t *next = et->next;
	free(et);
	et = next;
    }
}

static void event_fd_list_free(event_fd_t *efd)
{
    while (efd)
    {
	event_fd_t *next = efd->next;
	free(efd);
	efd = next;
    }
}

static void event_signal_list_free(event_signal_t *es)
{
    while (es)
    {
	event_signal_t *next = es->next;
	free(es);
	es = next;
    }
}

void event_loop_uninit(void)
{
    event_timer_queue_free(event_timer_queue);
    event_fd_list_free(event_fd_list);
    event_signal_list_free(event_signal_list);
}

int event_loop(void)
{
    int rc = 0;
    for (;!event_loop_quit;)
    {
	// exit on negative (error) or 1 (normal exit)
	if ((rc = event_loop_single()))
	    break;
    }
    return rc;
}

void event_fd_set(int fd, int type, efd_func_t func, void *data)
{
    event_fd_t *efd, **add_point;
    fd_set *pfds;

    if (!type)
	return;
    
    // handle multiple types in one call
    if (type!=EFD_READ && type!=EFD_WRITE && type!=EFD_EXCEPT)
    {
	if (type & EFD_READ)
	    event_fd_set(fd, EFD_READ, func, data);
	if (type & EFD_WRITE)
	    event_fd_set(fd, EFD_WRITE, func, data);
	if (type & EFD_EXCEPT)
	    event_fd_set(fd, EFD_EXCEPT, func, data);
	return;
    }
    
    // check if the event already exists
    efd = event_fd_get(fd, type);
    if (!efd)
    {
	// it does not exist - add it
	add_point = event_fd_get_next(fd, type);
	efd = zalloc_e(sizeof(event_fd_t));
	efd->fd = fd;
	efd->type = type;
	efd->next = *add_point;
	*add_point = efd;
    }

    // set callback info
    efd->func = func;
    efd->data = data;
    efd->stamp = event_loop_stamp;
    
    // set the bit in the fd_set for select()
    pfds = event_fds_choose(efd->type, &event_readfds, &event_writefds,
	&event_exceptfds);
    if (!FD_ISSET(efd->fd, pfds))
    {
	FD_SET(efd->fd, pfds);
	event_fd_max = MAX(event_fd_max, efd->fd);
	event_fd_nset++;
    }
}

void event_fd_clr(int fd, int type)
{
    event_fd_t **efd, *next_efd;
    fd_set *pfds;

    if (!type)
	return;
    // handle multiple types in one call
    if (type!=EFD_READ && type!=EFD_WRITE && type!=EFD_EXCEPT)
    {
	if (type & EFD_READ)
	    event_fd_clr(fd, EFD_READ);
	if (type & EFD_WRITE)
	    event_fd_clr(fd, EFD_WRITE);
	if (type & EFD_EXCEPT)
	    event_fd_clr(fd, EFD_EXCEPT);
	return;
    }
    efd = event_fd_get_p(fd, type);
    if (!efd)
	return;

    // clear the bit in the fd_set for select()
    pfds = event_fds_choose((*efd)->type, &event_readfds, &event_writefds,
	&event_exceptfds);
    if (FD_ISSET((*efd)->fd, pfds))
    {
	FD_CLR((*efd)->fd, pfds);
	event_fd_nset--;
    }

    // remove it from the list
    next_efd = (*efd)->next;
    free(*efd);
    *efd = next_efd;
}

// returns negative on error, 0 if fd not ready, positive if fd is ready
int event_fd_ready(int fd, int type)
{
    fd_set fds;
    struct timeval tv;

    MZERO(tv);
    FD_ZERO(&fds);
    FD_SET(fd, &fds);
    
    return select(fd + 1,
	type==EFD_READ ? &fds : NULL,
	type==EFD_WRITE ? &fds : NULL, 
	type==EFD_EXCEPT ? &fds : NULL, 
	&tv);
}


