/* LICENSE_CODE JUNGO */

#define OS_INCLUDE_NETDB
#define OS_INCLUDE_STD
#define OS_INCLUDE_IOCTL
#define OS_INCLUDE_SOCKET
#define OS_INCLUDE_INET
#define OS_INCLUDE_TIME
#define OS_INCLUDE_IF_ETHER
#define OS_INCLUDE_SOCKIOS

#define rg_error(level,str,args...) printf(str,##args)


#include <os_includes.h>
#include <rg_def.h>
#include <net_utils.h>
#include <errno.h>
#include <syscalls.h>
#include <util/rg_error.h>
#include "memory.h"
#include "print.h"
#include "str.h"
#include "strconv.h"

ether_mac_t bcast_mac = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};

#define PROC_NET_DEV_LINE_SIZE	512

int sockets_open(int domain, int type, int protocol)
{
    int fd = socket(domain, type, protocol);

    if (fd<0)
	rg_error(LERR, "sockets_open: no usable address was found.");
    return fd;
}

int get_if_data(char *iface, struct ifreq *ifr, int cmd)
{
    int sk_fd = -1;
    int rc = 0;
  
    MZERO(*ifr);
    strcpy(ifr->ifr_name, iface);
    ifr->ifr_addr.sa_family = AF_INET;
   
    if ((sk_fd = sockets_open(AF_INET, SOCK_DGRAM, 0))<0)
	return rg_error(LWARN, "get_if_data: failed sockets_open");

    if (ioctl(sk_fd, cmd, ifr)<0)
    {
	rg_error(LWARN, "get_if_data: failed ioctl(%s): %s", iface,
	    strerror(errno));
	rc = -1;
    }
   
    close(sk_fd);
    return rc;
}

u32 get_if_ip(char *iface)
{
    struct ifreq ifr;

    if (get_if_data(iface, &ifr, SIOCGIFADDR)<0)
	return 0;
    return ((struct sockaddr_in *) &ifr.ifr_addr)->sin_addr.s_addr;
}

u32 get_if_broadcast(char *iface)
{
    struct ifreq ifr;

    if (get_if_data(iface, &ifr, SIOCGIFBRDADDR)<0)
	return 0;
    return ((struct sockaddr_in *) &ifr.ifr_broadaddr)->sin_addr.s_addr;
}

int get_if_mac(char *iface, ether_mac_t *mac)
{
    return get_if_hw_addr(iface, mac, NULL);
}

u32 invert_ip_addr(u32 ip_addr)
{
    return (ip_addr & 0x000000FF) << 24 | (ip_addr & 0x0000FF00) << 8 |
        (ip_addr & 0x00FF0000) >> 8 | (ip_addr & 0xFF000000) >> 24;
}

#ifndef CONFIG_RG_MT_UNIT_TEST
int is_route_exist(char *dev, u32 ip, u32 netmask, u32 gw, int metric)
{
    route_t *head, *cur;
    int found = 0, name_len;
    char *colon;

    /* calc real dev name without aliases */
    colon = strchr(dev, ':');
    name_len = colon ? colon-dev : strlen(dev);
   
    for (head = cur = route_table_alloc(); cur; cur = cur->next)
    {
	/* look in route table for a route with exact properties like current
	 * (route_set). If found it was applied */
	if (cur->dest.s_addr==(ip&netmask) && cur->gateway.s_addr==gw &&
	    cur->netmask.s_addr==netmask && cur->metric==metric &&
	    !strncmp(cur->interface, dev, name_len))
	{
	    found = 1;
	    break;
	}
    }
    route_list_free(&head);

    return found;
}

/* Change the MTU setting of a device */
static int mtu_change(char *dev, int mtu)
{
    int s, ret = -1;
    struct ifreq ifr;

    if ((s = sockets_open(AF_INET, SOCK_DGRAM, 0)) < 0)
	return rg_error_f(LWARN, "Failed to open socket");

    MZERO(ifr);
    memcpy(ifr.ifr_name, dev, strlen(dev));

    /* The following is actually for VxWorks, but does no harm in Linux
     * because we included net/if.h in net_utils.h. In any case struct
     * ifreq is actually a union, so it does not matter how we call the
     * field.
     */
#ifndef ifr_mtu
#define ifr_mtu ifr_metric
#endif

    ifr.ifr_mtu = mtu;
    if (ioctl(s, SIOCSIFMTU, &ifr) < 0)
    {
	rg_error(LWARN, "Set MTU failed: %m");
	goto Exit;
    }

    ret = 0;

Exit:
    close(s);
    return ret;
}

int ifconfig(char *dev, int alias_index, u32 ip_addr, u32 net_mask, u32 remote,
    ifconfig_flag_t flags, int metric, int mtu)
{
    int is_alias = alias_index >= 0;
    u32 bcast = 0;

#if defined(CONFIG_IXP) && defined(CONFIG_RG_TARGET_VXWORKS)
    /* XXX: A temporary hack. Setting the IP of cs0 under VxWorks will kill it
     */
    if (!strcmp(dev, "cs0"))
    {
	rg_error(LWARN, "Unable to set IP of device cs0");
	return 0;
    }
#endif

    if (!ip_addr)
    {
	net_mask = 0;
	bcast = 0;
	if (is_alias)
	    flags = IFCFG_SDOWN;
    }
    else
	bcast = ip_addr | ~net_mask;
 
    if (is_alias)
    {
	dev = sys_alias_name_get(dev, alias_index);
	/* In Linux sys_alias_configure() does nothing and returns
	 * ALIAS_CONF_RET_CONTINUE, so the regular code is executed.
	 * In VxWorks sys_alias_configure() does all the work for aliases and
	 * returns ALIAS_CONF_RET_SUCCESS or ALIAS_CONF_RET_ERROR,
	 * which means immediate exit from this function.
	 */
	switch (sys_alias_configure(dev, ip_addr, net_mask, remote, flags))
	{
	case ALIAS_CONF_RET_ERROR:
	   return rg_error(LERR, "Error configuring an alias for %s\n", dev); 
	case ALIAS_CONF_RET_SUCCESS:
	   return 0;
	case ALIAS_CONF_RET_CONTINUE:
	   break;
	}
    }

    /* Cannot bring an IP alias up without an IP. Setting an IP to the alias 
     * (by the next ioctl) brings it UP.
     */
    if (!(is_alias && (flags & IFCFG_SUP)) && (sys_if_flags_set(dev, flags)<0))
	return rg_error(LWARN, "Error setting device %s flags: %m\n", dev);

#ifdef CONFIG_RG_TARGET_VXWORKS
     /* In VxWorks, setting the mask should come BEFORE setting the IP 
      * address. */
    if ((flags & IFCFG_SNETMASK) && sys_if_mask_set(dev, net_mask) < 0)
	return rg_error(LWARN, "Failed to set device %s netmask: %m\n", dev);
#endif

    if (flags & (IFCFG_SUP | IFCFG_SIPADDR))
    {
	if (sys_if_addr_set(dev, ip_addr) < 0)
	{
	    return rg_error(LWARN, "Error setting device %s ip address: %m\n",
		dev);
	}
    }

#ifdef CONFIG_RG_TARGET_LINUX
     /* In Linux, setting the mask should come AFTER setting the IP address. */
    if ((flags & IFCFG_SNETMASK) && sys_if_mask_set(dev, net_mask) < 0)
	return rg_error(LWARN, "Failed to set device %s netmask: %m\n", dev);
#endif
    
    if (!(flags & IFCFG_SUP))
	return  0;

    if (!is_alias)
    {
	if (sys_if_ext_set(dev, (flags & IFCFG_IS_EXT) != 0) < 0)
	    rg_error(LWARN, "ifconfig: ERROR sys_if_ext_set for %s: %m", dev);
    }

    if (remote && sys_if_dst_addr_set(dev, remote) < 0)
	return rg_error(LWARN, "Failed to set remote %s ip address: %m\n", dev);

    if ((flags & IFCFG_SIPADDR) && (flags & IFCFG_SNETMASK) &&
	sys_if_bcast_addr_set(dev, bcast) < 0)
    {
	return rg_error(LWARN,
	    "Failed to set device %s broadcast address: %m\n", dev);
    }
    
    if (ip_addr && metric && !is_route_exist(dev, ip_addr, net_mask, 0, metric))
    {
	/* Metric fixup - remove automatic route and add it again with device's
	 * metric */
	route(ip_addr, 0, net_mask, dev, RTF_UP, 0, 0);
	route(ip_addr, 0, net_mask, dev, RTF_UP, metric, 1);
    }

    if (mtu && mtu_change(dev, mtu))
    {
	return rg_error(LWARN,
	    "Failed to set device %s mtu: %m\n", dev);
    }

    return 0;
}
#endif

int is_ip_format(char *ip_addr)
{
    int ip0, ip1, ip2, ip3;

    return ip_addr && 
	sscanf(ip_addr, "%d.%d.%d.%d", &ip0, &ip1, &ip2, &ip3) == 4;
}

int subnet_compare(u32 ip1, u32 netmask1, u32 ip2, u32 netmask2)
{
    if ((ip2 & netmask2 & netmask1) != (ip1 & netmask1 & netmask2))
	return -1;
    if (ntohl(netmask1) > ntohl(netmask2))
	return 2;
    if (ntohl(netmask1) < ntohl(netmask2))
	return 1;
    return 0;
}

/* Pay attention that if 'cp' contains a valid IP and a suffix afterwards,
 * then rg_inet_aton() returns with success. For example, "192.168.71.13 xxx"
 * will cause rg_inet_aton() to return with success
 */
int rg_inet_aton(char *cp, struct in_addr *inp)
{
    if (!cp || !is_ip_format(cp) || !inet_aton(cp, inp))
    {
	inp->s_addr = 0;
	return 0;
    }
    return 1;
}

/* The function finds what is the netmask according to the IP.
 * This function only handles class A,B and C netmasks, as the rest are
 * currently irelevant.
 */
u32 ip_class_netmask(u32 ip)
{
    /* For case of class D multicast or class E resereved, which are illegal */
    u32 mask = 0;

    /* Class A address */
    if ((ip & htonl(0x80000000)) == 0)
	mask = htonl(0xff000000);
    /* Class B address */
    else if ((ip & htonl(0xc0000000)) == htonl(0x80000000))
	mask = htonl(0xffff0000);
    /* Class C address */
    else if ((ip & htonl(0xe0000000)) == htonl(0xc0000000))
	mask = htonl(0xffffff00);

    return mask;
}

int is_mac_zero(ether_mac_t *mac)
{
    return !((*mac)[0]|(*mac)[1]|(*mac)[2]|(*mac)[3]|(*mac)[4]|(*mac)[5]);
}

char *mactoa(ether_mac_t *mac)
{
    static char mac_str[18];

    sprintf(mac_str, "%02x:%02x:%02x:%02x:%02x:%02x", (*mac)[0], (*mac)[1],
       (*mac)[2], (*mac)[3], (*mac)[4], (*mac)[5]);
    return mac_str;
}

ether_mac_t *atomac(char *mac_str)
{
    static ether_mac_t mac;
    int mac_tmp[6];
    int i;

    MZERO(mac);
    if (!mac_str)
    {
	rg_error_f(LWARN, "invalid MAC address");
	return NULL;
    }
    if (sscanf(mac_str, "%02x:%02x:%02x:%02x:%02x:%02x", &mac_tmp[0],
	&mac_tmp[1], &mac_tmp[2], &mac_tmp[3], &mac_tmp[4], &mac_tmp[5]) != 6)
    {
	rg_error_f(LWARN, "error translating a string to MAC "
	    "address: %s", mac_str);
	return NULL;
    }
    
    for (i=0; i<6; i++)
	mac[i] = (u8)mac_tmp[i];
    return &mac;
}

/* Return the termination point of a string holding host name */
static char *find_host_str_termination(char *str)
{
    char *t = str;

    /* host must begin with alpha numeric character */
    if (!isalnum(*t))
	return t;
    /* host may include letters, numbers, or '-', up to max length */
    for (t++; (isalnum(*t) || *t=='-') && t-str<MAX_HOST_NAME_LEN; t++);
    /* host must end with alpha numeric, not a '-' */
    if (*(t-1)=='-')
	return t-1;
    /* t points to termination point */
    return t;
}

/* Check validity of the hostname */
int is_valid_host_name(char *host, char *end)
{
    char *term = find_host_str_termination(host);
    if (!end)
	end = host + strlen(host);
    /* a valid edit box host is a non empty host */
    return term!=host && term==end;
}

/* A domain is a series of hosts seperated by dots */
int is_valid_domain_name(char *val, char *endval)
{
    char *next;
    if (!endval)
	endval = val + strlen(val);

    /* Length must be less than 256 */
    if (endval==val || endval-val>MAX_DOMAIN_NAME_LEN)
	return 0;

    while (1)
    {
	next = find_host_str_termination(val);
	if (next==val)
	    return 0;
	if (next >= endval)
	    break;
	if (*next != '.')
	    return 0;
	val = next+1;
    }
    return 1;
}

char *url_encode(char *url)
{
    str_t *encoded_url;
    char tmp_buf[4], *ret;

    encoded_url = str_alloc();
    for (; *url; url++)
    {
	/* The following cases are not escaped */
	if (*url == '-' ||
	    *url == '_' ||
	    *url == '.' ||
	    *url == '*' ||
	    *url == '@' ||
	    isalnum((int)*url))
	{
	    sprintf(tmp_buf, "%c", *url);
	}
	else if (*url == ' ')
	    sprintf(tmp_buf, "%c", '+');
	else
	    sprintf(tmp_buf, "%%%02X", *url); /* default: escape */
	str_cat(encoded_url, tmp_buf);
    }

    ret = strdup_e(encoded_url->str);
    str_free(encoded_url);
    return ret;
}

int parse_url(char *url, parsed_url_t *h)
{
    int ret = 0;
    char *query, *host, *port, *path, *after_transport, *shtrudel, *tmp_url;
    
    MZERO(*h);

    tmp_url = strdup_e(url);
    /* We use case insensitive comparison because according to RFC 1738 (URL
     * definition): "For resiliency, programs interpreting URLs should treat
     * upper case letters as equivalent to lower case in scheme names (e.g.,
     * allow "HTTP" as well as "http")."
     */
    if (!strncasecmp(tmp_url, HTTP_PREFIX, sizeof(HTTP_PREFIX) - 1))
    {
	h->port = 80;
	after_transport = tmp_url + sizeof(HTTP_PREFIX) - 1;
	h->transport = URL_HTTP;
    }
    else if (!strncasecmp(tmp_url, HTTPS_PREFIX, sizeof(HTTPS_PREFIX) - 1))
    {
	h->port = 443;
	after_transport = tmp_url + sizeof(HTTPS_PREFIX) - 1;
	h->transport = URL_HTTPS;
    }
    else if (!strncasecmp(tmp_url, FTP_PREFIX, sizeof(FTP_PREFIX) - 1))
    {
	h->port = 21;
	after_transport = tmp_url + sizeof(FTP_PREFIX) - 1;
	h->transport = URL_FTP;
    }
    else if (!strncasecmp(tmp_url, TFTP_PREFIX, sizeof(TFTP_PREFIX) - 1))
    {
	h->port = 69;
	after_transport = tmp_url + sizeof(TFTP_PREFIX) - 1;
	h->transport = URL_TFTP;
    }
    else
    {
	ret = rg_error(LERR, "Not a HTTP, HTTPS, FTP or TFTP url: %s", url);
	goto Exit;
    }

    /* If the transport is HTTP, then extract query, if any */
    if (h->transport == URL_HTTP || h->transport == URL_HTTPS)
    {
	query = strchr(after_transport, '?');
	if (query)
	{
	    *query = 0;
	    h->query = strdup_e(query + 1);
	}
    }
 
    host = after_transport;

    /* Extract path, if any */
    path = strchr(host, '/');
    if (path)
    {
	*path = 0;
	h->path = strdup_e(path + 1);
    }
    else if (h->transport == URL_FTP || h->transport == URL_TFTP)
    {
	ret = rg_error(LERR, "The URL path must not be empty for FTP/TFTP");
	goto Exit;
    }

    /* Extract username and password, if any */
    shtrudel = strrchr(after_transport, '@');
    if (shtrudel)
    {
	char *colon;

	*shtrudel = 0;
	host = shtrudel + 1;

	colon = strchr(after_transport, ':');
	if (colon)
	{
	    *colon = 0;
	    h->pass = strdup_e(colon + 1);
	}
	h->user = strdup_e(after_transport);
    }

    /* Extract port, if any */
    port = strchr(host, ':');
    if (port)
    {
	*port = 0;
	h->port = atoi(port + 1);
	h->has_port = 1; /* Indicate that the URL contains a port number */
    }
    h->host = strdup_e(host);

Exit:
    if (ret)
	parsed_url_uninit(h);
    free(tmp_url);
    return ret;
}

void parsed_url_uninit(parsed_url_t *h)
{
    nfree(h->user);
    nfree(h->pass);
    nfree(h->host);
    nfree(h->path);
    nfree(h->query);
}

int netmask_to_bitcount(u32 netmask)
{
    u32 bit_mask, bit_count, mask;

    mask = ntohl(netmask);
    for (bit_count=0, bit_mask=0x80000000;
        mask&bit_mask;
	bit_count++, bit_mask >>= 1);
    return bit_count;
}

u32 bitcount_to_netmask(int bitcount)
{
    u32 mask = 0;
    int i;
   
    for (i=0; i < 32; i++)
    {
	mask <<= 1;
	if (i < bitcount)
	    mask++;
    }
    return htonl(mask);
}

int is_netmask_valid(u32 netmask)
{
    u32 bit_mask, mask;

    mask = ntohl(netmask);
    /* make sure it starts with a sequence of '1' */
    for (bit_mask=0x80000000; mask&bit_mask; bit_mask >>= 1);
    /* then check that it has '0' until the end */
    for (; bit_mask && !(mask&bit_mask); bit_mask >>= 1);
    return !bit_mask;
}

int is_ip_in_subnet(u32 ip, u32 netmask, u32 subnet)
{
    return netmask && (ip & netmask)==subnet;
}

void ip_range_start_get(struct in_addr ip, struct in_addr netmask,
    struct in_addr *start_ip)
{
    start_ip->s_addr = htonl(ntohl(ip.s_addr & netmask.s_addr)+1);
}

void ip_range_end_get(struct in_addr ip, struct in_addr netmask,
    struct in_addr *end_ip)
{
    end_ip->s_addr = htonl(ntohl(ip.s_addr & netmask.s_addr) |
	(ntohl(~netmask.s_addr)-1));
}

void route_list_free(route_t **head)
{
    route_t *tmp;

    while (*head)
    {
	tmp = *head;
	*head = (*head)->next;
	free(tmp);
    }
}

route_t *route_to_host_get(struct in_addr host)
{
    int min_metric=-1;
    route_t *head, *cur, *found = NULL;
    static route_t stat_route;

    for (head = cur = route_table_alloc(); cur; cur = cur->next)
    {
	if (!found && !cur->dest.s_addr && cur->gateway.s_addr) /* Gateway */
	{
	    found = cur;
	    break;
	}
	/* Destination - look for matching route with minimum metric */
	if ((min_metric==-1 || cur->metric<min_metric) &&
	    cur->netmask.s_addr && is_ip_in_subnet(host.s_addr,
	    cur->netmask.s_addr, cur->dest.s_addr&cur->netmask.s_addr))
	{
	    found = cur;
	    min_metric = cur->metric;
	}
    }

    if (found)
	memcpy(&stat_route, found, sizeof(route_t));
    route_list_free(&head);
    return found ? &stat_route : NULL;
}

int is_bcast_mac(ether_mac_t mac)
{
    return !memcmp(mac, bcast_mac, sizeof(mac));
}
