/* LICENSE_CODE JUNGO */

#include <ctype.h>
#include "str.h"
#define OS_INCLUDE_STDARG
#define OS_INCLUDE_STD
#define OS_INCLUDE_STRING
#define OS_INCLUDE_PARAM
#include <os_includes.h>
#include <os_libc_extra.h>
#include "memory.h"
#include <rg_def.h>
#include <misc_funcs.h>

#define MAX_STR_LEN 64*1024
#define DEF_STR_LEN 64

static void str_size_alloc(str_t *s, int sz)
{
    sz = MAX(sz, DEF_STR_LEN);
    if (s->alloc_size < sz)
    {
	s->str = (char *)realloc_e(s->str, sz);
	s->alloc_size = sz;
    }
}

static void str_reset(str_t *s)
{
    str_size_alloc(s, 1);
    s->str[0] = 0;
}
    

str_t *str_alloc(void)
{
    str_t *s = zalloc_e(sizeof(str_t));
    
    str_reset(s);
    return s;
}

void str_free(str_t *s)
{
    nfree(s->str);
    free(s);
}

void str_cpy(str_t *dest, char *src)
{
    str_size_alloc(dest, strlen(src) + 1);
    strcpy(dest->str, src);
}

int str_vsprintf(str_t *s, char *fmt, va_list ap)
{
    int rc;

    str_reset(s);
    /* New version of vsnprintf return the number of characters that whould
     * have been written if the buffer was big enough, excluding the trailing
     * NULL.
     */
    rc = vsnprintf(s->str, s->alloc_size, fmt, ap);

    if (rc < 0 || rc > s->alloc_size - 1)
    {
	/* Not enough space in the default string, realloc and redo. */

	if (rc + 1 > MAX_STR_LEN)
	    return 0; /* String too long. */
	
	str_size_alloc(s, rc + 1);

	rc = vsnprintf(s->str, s->alloc_size, fmt, ap);
	if (rc < 0 || rc > s->alloc_size - 1)
	    goto Error;
    }

    return strlen(s->str);

Error:
    /* Error - free some memory. */
    str_reset(s);
    return -1;
}

int str_sprintf(str_t *s, char *fmt, ...)
{
    va_list arg;
    int rc;

    va_start(arg, fmt);
    rc = str_vsprintf(s, fmt, arg);
    va_end(arg);

    return rc;
}

void str_cat(str_t *s, char *new_str)
{
    str_size_alloc(s, strlen(s->str) + strlen(new_str) + 1);
    strcat(s->str, new_str);
}

char *find_first_of(char *s, char *chars)
{
    char *where;

    for (; *s; s++)
    {
	where = strchr(chars, *s);

	if (where)
	    return s;
    }
    return NULL;
}

split_t *split_alloc(void)
{
    split_t *split = zalloc_e(sizeof(split_t));
    
    split->str = str_alloc();
    return split;
}

void split_free(split_t *split)
{
    str_free(split->str);
    nfree(split);
}

split_t *str_split(char *input, char *characters)
{
    char *pos = input, *old_pos = input;
    split_t *splitted = 0, **current;
    unsigned int length = strlen(input);

    current = &splitted;
    while (*pos)
    {
	pos = find_first_of(old_pos, characters);
	if (!pos)
	    pos = input + length;
	if (pos > old_pos) /* Skip empty splits. */
	{
	    *current = split_alloc();
	    (*current)->str->str = (char *)realloc_e((*current)->str->str,
	        pos - old_pos + 1);
	    strncpy((*current)->str->str, old_pos, pos - old_pos);
	    (*current)->str->str[pos - old_pos] = '\0';
	    (*current)->str->alloc_size = pos - old_pos + 1;
	    current = &(*current)->next;
	}
	old_pos = pos + 1;
    }
    return splitted;
}

char *trim_spaces(char *str, u8 how)
{
    int start = 0, end = strlen(str);

    if (!end)
	return str;
    
    if (how & TRIM_SPACES_LEADING)
	for (; start < end && isspace(str[start]); start++);

    if (how & TRIM_SPACES_TRAILING)
	for (; start < end && isspace(str[end - 1]); end--);

    if (start < end && start)
	memmove(str, str + start, end - start);

    str[end - start] = 0;
    return str;
}

char *remove_end_line(char *str)
{
    int i = strlen(str);

    if (str[i] == '\n')
    {
	if (i && str[i-1] == '\r')
	    i--;
	str[i] = 0;
    }
    return str;
}

char *strcpy_line(char *dest, const char *src)
{
    char *d = dest;

    while (*src && *src != '\n')
        *(dest++) = *(src++);
    *dest = '\n';
    return d;
}

size_t strlen_line(const char *str)
{
    register char *ptr = (char *)str;

    while (*ptr && *ptr != '\n')
        ptr++;
    return (ptr - str);
}

char *strtolower(char *str)
{
    int i;
    
    for (i=0; str[i]; i++)
	str[i] = tolower(str[i]);
    return str;
}

char *strncpyz(char *dest, char const *src, size_t size)
{
    if (!size--)
	return dest;
    strncpy(dest, src, size);
    dest[size] = 0; /* Make sure the string is null terminated */
    return dest;
}

/* in strncat, size is the maximum number of additional chars added
 * to dest. in strncatz_total the size is the size of dest buffer, which
 * is the maximum total size after concatanation src into dest.
 */
char *strncatz_total(char *dest, char const *src, size_t size)
{
    size_t dst_len = strlen(dest);

    if (!size-- || dst_len>=size)
	return dest;
    strncpy(dest+dst_len, src, size-dst_len);
    dest[size] = 0;
    return dest;
}

int nstrcmp(char *str1, char *str2)
{
    if (!str1 || !str2)
	return -1;
    return strcmp(str1, str2);
}

void extract_pty_ip(char *ptycmd, char *buf, int bufsize)
{
    char *start, *end;

    for (start=ptycmd; *start && !isdigit(*start); start++);
    for (end=start; *end && !isspace(*end); end++);
    strncpyz(buf, start, bufsize<end-start+1 ? bufsize : end-start+1);
}

