/* JUNGO_COPYRIGHT_1 */

#define OS_INCLUDE_STD
#define OS_INCLUDE_STDARG
#define OS_INCLUDE_IOCTL
#define OS_INCLUDE_SOCKET
#define OS_INCLUDE_INET
#include <os_includes.h>
#include <estream.h>
#include <misc_funcs.h>
#include <net_utils.h>
#include <kos/kos.h>
#include <kos/knet.h>
#include <syscalls.h>
#include "vx_net_fixup.h"
#include <m2Lib.h>

typedef struct m2_entry_t {
    M2_INTERFACETBL m2_e;
    struct m2_entry_t *next;
} m2_entry_t;

static m2_entry_t *m2_entry_head;

/* in VxWorks there is a problem that after many TCP connections,
 * select() stops returning on new connections.
 * It resumes returning only after a TCP timeout period (around 30
 * seconds). The problem is a bug in VxWorks kernel, that when a socket is
 * closed from user-mode - close(fd) - and there is still data to send, 
 * the VxWorks kernel needs to still keep the TCP session information 
 * in the TCP session table, until the data is sent. The bug is that
 * when the data has been sent, it does not free the session, so 
 * very quickly the TCP session table fills up.
 *
 * The delayed close fixup polls the status of TCP connection, and only when
 * it reaches the state	TCPS_FIN_WAIT_2	(have closed, fin is acked), we call
 * close(fd). To disable further communications and immediate release of TCP
 * control blocks, we set options to SO_LINGER with l_linger 0.
 *
 * NOTE: For future releases of VxWorks, we should:
 * 1. Verify whether the bug is fixed, to remove the fixup
 * 2. Verify that the private internal structures are still there and have
 *    the same types (VxWorks documentation does not encourage to use data
 *    defined in xxxP.h - private headers).
 */

#include <netinet/tcp_var.h>
#include <netinet/tcp_fsm.h>
#include <netinet/in_pcb.h>
#include <private/iosLibP.h>
#include <net/socketvar.h>
#include <net/if_types.h>

static struct socket *fd_to_socket(int fd)
{
    FD_ENTRY *pfd;
    
    /* skip first three fd: stdin, stdout, stderr */
    if (fd<3)
	return NULL;
    pfd = fdTable + fd - 3;    

    /* check that its valid */
    if (!pfd->inuse)
	return NULL;
    if (strcmp(pfd->name, "(socket)"))
	return NULL;

    return (struct socket *) pfd->value;
}

static void socket_close_immediate(int fd)
{

    struct linger l;
    /* make close() discard any un-sent data and close immediately */
    MZERO(l);
    l.l_onoff = 1;
    l.l_linger = 0;
    setsockopt(fd, SOL_SOCKET, SO_LINGER, (char *) &l, sizeof(l));
    close(fd);
}

#define TCP_CLOSE_TIMEOUT 30*1000 /* TCP_TTL = 30 in netinet/tcp_timer.h */
#define TCP_CLOSE_POLL_MS 50

static void socket_close_poll(void *data);

static void socket_close_timeout(void *data)
{
    int fd = (int)data;
    socket_close_immediate(fd);
    event_timer_del(socket_close_poll, data);
}

static int socket_get_tcp_fsm(int fd)
{
    struct socket *so;
    struct inpcb *ip;
    struct tcpcb *tp;

    so = fd_to_socket(fd);
    if (!so)
	return -1;
    ip = (struct inpcb *) so->so_pcb;
    if (!ip)
	return -1;
    tp = (struct tcpcb *) ip->inp_ppcb;
    if (!tp)
	return -1;
    return tp->t_state;
}

static int socket_test_and_close(int fd)
{
    int fsm;
    
    fsm = socket_get_tcp_fsm(fd);
    
    if (fsm>=0 && fsm<=TCPS_FIN_WAIT_2)
	return -1;
    
    socket_close_immediate(fd);
    return 0;
}

static void socket_close_poll(void *data)
{
    int fd = (int)data;

    if (socket_test_and_close(fd))
	return;
    
    event_timer_del(socket_close_poll, data);
    event_timer_del(socket_close_timeout, data);
}

void socket_close_vx(int fd)
{
    struct socket *s;

    /* start nice close of socket */
    s = fd_to_socket(fd);
    if (!s)
	rg_error_f(LPANIC, "Invalid socket %d", fd);
    
    sodisconnect(s);
    
    /* check we can close immediately */
    if (!socket_test_and_close(fd))
	return;

    /* if cannot close now, start polling */
    event_timer_set(TCP_CLOSE_TIMEOUT, socket_close_timeout, (void *)fd);
    event_timer_set_period(TCP_CLOSE_POLL_MS, socket_close_poll, (void *)fd);
}

#define ARPRESOLVE_TIMEOUT 5000
#define ARPRESOLVE_POLL_MS 50

typedef struct {
    void *o;
    etimer_func_t cb;
    struct in_addr dst;
    struct ifnet *dev;
} resolve_context_t;

/* Return non-zero if destination MAC is already resolved, zero otherwise */
static int arp_test_and_resolve(struct in_addr dst, struct ifnet *dev)
{
    struct sockaddr_in sin;
    u8 mac[6];

    sin.sin_len = sizeof(sin);
    sin.sin_family = AF_INET;
    sin.sin_addr = dst;

    /* For loopback device we don't need arp resolving. */
    if (IFT_LOOP == dev->if_type)
	return 1;

    return arpresolve((struct arpcom *)dev, NULL, NULL,
	(struct sockaddr *)&sin, mac);
}

#include <netinet/in_var.h>

typedef struct {
    struct rtentry *rt;
    struct in_addr dst;
} rt_result_t;

static int search_rtentry_cb(struct rtentry *rt, void *arg)
{
    struct sockaddr_in *dst = (struct sockaddr_in *)rt_key(rt);
    struct sockaddr_in *mask = (struct sockaddr_in *)rt_mask(rt);
    u32 saved_mask, new_mask;

    rt_result_t *res = (rt_result_t *)arg;

    new_mask = mask ? mask->sin_addr.s_addr : 0xffffffff;
    
    /* Check that this route can qualify as a candidate for this destination */
    if (dst->sin_addr.s_addr != (res->dst.s_addr & new_mask))
	return 0;

    if (!res->rt)
    {
	res->rt = rt;
	return 0;
    }
   
    saved_mask = rt_mask(res->rt) ?
	((struct sockaddr_in *)rt_mask(res->rt))->sin_addr.s_addr : 0xffffffff;
    
    /* Check which route (the new one or the saved one) is more specific - 
     * i.e for which route, mask has more bits set.
     */
    if (new_mask & (new_mask ^ saved_mask))
	res->rt = rt;
    
    return 0;
}

/* Find the real destination of the packet (different in case it is not the
 * same subnet - dest is a gateway) and the device on which to send the arp
 * request.
 */
static int calc_real_dest_and_dev(struct sockaddr *dst,
    struct in_addr *real_dst, struct ifnet **dev)
{
    rt_result_t rt_res;
    struct rtentry *rt;
    kos_interrupt_flags_t flags;
    
    MZERO(rt_res);
    memcpy(&(rt_res.dst),
		   &(((struct sockaddr_in *)dst)->sin_addr),
		   sizeof(rt_res.dst));

    flags = kos_save_flags_cli();
    
    knet_route_table_loop(RTF_UP, search_rtentry_cb, (void *)&rt_res);

    kos_restore_flags(flags);
    
    if (!rt_res.rt)
	   return -1;

    rt = rt_res.rt;

    *dev = rt->rt_ifp;
   if (rt->rt_flags & RTF_GATEWAY) {
       /*
	   *real_dst = ((struct sockaddr_in *)rt->rt_gateway)->sin_addr;
      */
       memcpy( real_dst,
          &(((struct sockaddr_in *)rt->rt_gateway)->sin_addr),
          sizeof(struct in_addr));
    } else {
       /*
	   *real_dst = ((struct sockaddr_in *)dst)->sin_addr;
      */
       memcpy( real_dst,
          &(((struct sockaddr_in *)dst)->sin_addr),
          sizeof(struct in_addr));
    }    
    return 0;
}

static void arpresolve_timeout(void *data);

static void arpresolve_poll(void *data)
{
    resolve_context_t *c = (resolve_context_t *)data;

    if (!arp_test_and_resolve(c->dst, c->dev))
      return;

    event_timer_del(arpresolve_poll, data);
    event_timer_del(arpresolve_timeout, data);
    c->cb(c->o);
}

static void arpresolve_timeout(void *data)
{
    resolve_context_t *c = (resolve_context_t *)data;
    
    event_timer_del(arpresolve_poll, data);
    c->cb(c->o);
}

/* Cancel is always called whether we successfully sent, a timeout (of any
 * kind) has occured, or in case of a user requested cancel. This is why our
 * context is freed only here.
 */
void send_wait_for_ready_cancel_vx(void *data)
{
    event_timer_del(arpresolve_poll, data);
    event_timer_del(arpresolve_timeout, data);
    free(data);
}

void *send_wait_for_ready_and_start_vx(struct sockaddr *dst, void *context,
	etimer_func_t cb)
{
    struct in_addr real_dst;
    struct ifnet *dev;
    resolve_context_t *c;

    dst->sa_len = sizeof(struct sockaddr_in);

    /* In case we don't find a route to destination or destination is already
     * resolved, we don't bother setting timer - just try to continue with
     * sending.
     */
    if (calc_real_dest_and_dev(dst, &real_dst, &dev) ||
	arp_test_and_resolve(real_dst, dev))
    {
	cb(context);
	return NULL;
    }

    /* If still unresolved, start polling */
    c = (resolve_context_t *)zalloc_l(sizeof(*c));
    c->cb = cb;
    c->o = context;
    c->dst = real_dst;
    c->dev = dev;
    
    event_timer_set(ARPRESOLVE_TIMEOUT, arpresolve_timeout, (void *)c);
    event_timer_set_period(ARPRESOLVE_POLL_MS, arpresolve_poll, (void *)c);
    return (void *)c;
}

static int device_rt_do_delete(struct rtentry *rt, void *arg)
{
    int rc;
    struct ifnet *ifp = (struct ifnet *)arg;
    struct sockaddr *dst = (struct sockaddr *)rt_key(rt);
    struct sockaddr *gw = (struct sockaddr *)rt->rt_gateway;
    struct sockaddr *mask = (struct sockaddr *)rt_mask(rt);
    
    if (rt->rt_ifp != ifp) /* Continue scan */
	return 0;

    rc = rtrequest(RTM_DELETE, dst, gw, mask, rt->rt_flags, NULL);

    if (rc)
	rg_error_f(LERR, "Error deleteing routing rule rc=%d\n", rc);

    /* This return stops the loop on current routing table */
    return 1;
}

static void device_rt_delete(struct ifnet *ifp)
{
    /* Run until the routing table doesn't contain routing rule pointing to
     * the ifp
     */
    while (knet_route_table_loop(RTF_UP, device_rt_do_delete, ifp));
}

static void device_sockets_close(struct ifnet *ifp)
{
    int fd;
    struct socket *sock;
    struct rtentry *rt;
    struct inpcb *t_inpcb;
    
    for (fd = 3; fd < maxFiles; fd++)
    {
	if (!(sock = fd_to_socket(fd)))
	    continue;

	if (!(t_inpcb = (struct inpcb *)sock->so_pcb))
	    continue;
	
	if (!(rt = t_inpcb->inp_route.ro_rt))
	    continue;

	if (rt->rt_ifp == ifp)
	{
	    /* Detach socket from routing rule */
	    t_inpcb->inp_route.ro_rt = NULL;
	    close(fd);
	}
    }
}

/* Re-create mibII device information, upon the exist devices */
static void device_m2_update(struct ifnet *ifp)
{
    m2_entry_t **e, *e_t;
    M2_INTERFACETBL vx_m2e;

    memset(&vx_m2e, 0, sizeof(M2_INTERFACETBL));
 
    /* Build information list */
    for (e = &m2_entry_head; m2IfTblEntryGet(M2_NEXT_VALUE, &vx_m2e) == OK;
	e = &(*e)->next)
    {
	(*e) = malloc(sizeof(m2_entry_t));
	if (!*e)
	    rg_error_f(LPANIC, "Failed alloc m2_entry_t");

	(*e)->m2_e = vx_m2e;
	(*e)->next = NULL;
    }

    /* Delete current m2 information */
    m2IfDelete();
    m2IfInit(NULL, NULL);

    /* Restore original m2 list */
    while (m2_entry_head)
    {
	/* Don't Write back the removed ifp */
	if (strcmp(ifp->if_name, m2_entry_head->m2_e.ifDescr))
	{
	    if (m2IfTblEntrySet(&m2_entry_head->m2_e) != OK)
		rg_perror(LERR, "Failed set m2IfTblEntry");
	}
	e_t = m2_entry_head;
	m2_entry_head = m2_entry_head->next;
	free(e_t);
    }
}

void device_detach_vx_fix(struct ifnet *ifp)
{
    device_sockets_close(ifp);
    device_rt_delete(ifp);
    device_m2_update(ifp);
}

int netdev_addr_init(struct ifnet *ifp)
{
    char ifname[IFNAMSIZ];
    int rc;
    
    sprintf(ifname, "%s%d", ifp->if_name, ifp->if_unit);
    rc = sys_if_addr_set(ifname, 0);
    m2IpInit(0);

    return rc;
}

