/* LICENSE_CODE JUNGO */

#include "rg_error.h"
#include <event.h>
#include <rg_memory.h>
#include <openssl/rc4.h>
#include <rg_error.h>
#include <802.1x/wep_port.h>
#include <kos/knet.h>
#include "wpa_key_utils.h"
#include "wpa_port.h"
#include "wpa_assoc.h"
#include "wpa_log.h"
#include "wpa_common.h"
#include "pform_types_user.h"

#define HANDSHAKE_MAX_RETRIES 3

/* Time to wait for a station response for an EAPOL packet before retrying */
#define HANDSHAKE_TIMEOUT 2000 /* ms */

/* If we're not getting any data from station within this period since the
 * station associates, we assume it's a WEP-legacy station
 */
#define WPA_WAIT_FOR_FIRST_PACKET_TIMEOUT 2000 /* ms */

#define KEY_DESC_TYPE 254

/* Key info bitmask */
#define KEY_INFO_VERSION_1_BIT 0x0001
#define KEY_INFO_PAIRWISE_BIT  0x0008
#define KEY_INFO_INSTALL_BIT   0x0040
#define KEY_INFO_ACK_BIT       0x0080
#define KEY_INFO_MIC_BIT       0x0100
#define KEY_INFO_SECURE_BIT    0x0200
#define KEY_INFO_ERROR_BIT     0x0400
#define KEY_INFO_REQUEST_BIT   0x0800

#define KEY_INFO_INDEX_BITS 4 /* position of bits from right */

#define SET_BIT(info, mask) ((info) |= (mask))
#define TEST_BITS(info, mask) (((info) & (mask)) == (mask))
#undef MIN
#define MIN(a, b) ((a)<(b)?(a):(b))

#define IV_FIELD_LEN  16
#define ID_FIELD_LEN  8

#define TSC_MASK 0x00FFFFFF /* 6 least significant bytes of the TSC are used as 
			     * RSC
			     */

/* Tests if the key descriptor indicates a MIC failure report */
#define IS_MIC_FAILURE_REPORT(key_desc) \
    TEST_BITS(key_desc->info, htons(KEY_INFO_REQUEST_BIT | \
    KEY_INFO_ERROR_BIT | KEY_INFO_MIC_BIT))

/* Tests if the key descriptor indicates a request for a 4-way-handshake/group 
 * key update
 */
#define IS_KEY_REQUEST(key_desc) \
    (TEST_BITS(key_desc->info, htons(KEY_INFO_REQUEST_BIT)) && \
    !TEST_BITS(key_desc->info, htons(KEY_INFO_ERROR_BIT)) && \
    !TEST_BITS(key_desc->info, htons(KEY_INFO_MIC_BIT)))

/* A WPA EAPOL-key key descriptor */
typedef struct {
    u8 type;
    u16 info;
    u16 len;
    u64 replay_count;
    wpa_nonce_t nonce;
    u8 iv[IV_FIELD_LEN];
    u64 rsc;
    u8 id[ID_FIELD_LEN];
    wpa_eapol_mic_t mic;
    u16 data_len;
    u8 data[1];
} __attribute__ ((packed)) eapol_key_desc_t;
 
typedef enum {
    WPA_START_PTK = 0,
    WPA_NEGOTIATING_PTK = 1,
    WPA_START_GTK = 2,
} eapol_key_type_t;

BOOL	micFailureReportFromStation = TRUE;

static void calc_mic_on_eapol_pkt(wpa_assoc_t *assoc, eapol_pkt_t *eapol_pkt, 
    wpa_eapol_mic_t *mic)
{
    wpa_key_util_hmac_md5((u8 *)mic, (u8 *)&eapol_pkt->proto_ver, 
	ntohs(eapol_pkt->pkt_body_len) + 4,
	(u8 *)&assoc->ptk.eapol_key_mic_key, sizeof(wpa_eapol_mic_t));
}

/* Create an EAPOL-key packet */
static eapol_pkt_t *create_eapol_key_pkt(wpa_assoc_t *assoc, int key_info_bits, 
    int key_index, u64 rsc, int key_len, wpa_nonce_t *nonce, u8 *iv, 
    u8 *key_data, int key_data_len)
{
    eapol_pkt_t *eapol_pkt;
    eapol_key_desc_t *key_desc;
    int eapol_pkt_len;
    int key_desc_len;
    RC4_KEY rc4_key;
    u8 rc4_key_data[32];
    u8 dummy[256];

    key_desc_len = sizeof(eapol_key_desc_t) + key_data_len - 1;
    eapol_pkt_len = sizeof(eapol_pkt_t) + key_desc_len - 1;
    eapol_pkt = (eapol_pkt_t *)zalloc_l(eapol_pkt_len);
    if (!eapol_pkt)
	return NULL;

    /* EAPOL packet header */
    memcpy(eapol_pkt->dest, &assoc->sta_mac, sizeof(ether_mac_t));
    memcpy(eapol_pkt->src, &assoc->port->params.mac, sizeof(ether_mac_t));
    eapol_pkt->ether_type = htons(EAPOL_TYPE);
    eapol_pkt->proto_ver = EAPOL_VER;
    eapol_pkt->pkt_type = ET_KEY;
    eapol_pkt->pkt_body_len = htons(key_desc_len); 

    /* Key descriptor header */
    key_desc = (eapol_key_desc_t *)eapol_pkt->pkt_body;
    key_desc->type = KEY_DESC_TYPE;

    key_desc->info = key_info_bits | KEY_INFO_VERSION_1_BIT;
    if (key_index)
	key_desc->info |= (key_index << KEY_INFO_INDEX_BITS);
    key_desc->info = htons(key_desc->info);

    /* Note that key_len and key_data_len have different meanings. key_len is
     * the length of the key that should be eventually installed to station/AP
     * while key_data_len is the length of the data actually included in this
     * message.
     */
    key_desc->len = htons(key_len); 

    /* Increment replay counter and put it in packet */
    assoc->ap_to_sta_replay_count++;
    key_desc->replay_count = knet_htonll(assoc->ap_to_sta_replay_count);

    memcpy(&key_desc->nonce, nonce, sizeof(key_desc->nonce));

    key_desc->rsc = rsc;

    if (iv)
        memcpy(&key_desc->iv, iv, sizeof(key_desc->iv));

    key_desc->data_len = htons(key_data_len);
    
    /* In EAPOL-key messages which are part of the GTK update, the key is stored
     * encrypted with RC4
     */
    if (!(key_info_bits & KEY_INFO_PAIRWISE_BIT))
    {
	/* RC4 key = Key IV || EAPOL-Key encryption key */
	memcpy(rc4_key_data, key_desc->iv, sizeof(key_desc->iv));
	memcpy(&(rc4_key_data[sizeof(key_desc->iv)]), 
	    &assoc->ptk.eapol_key_encryption_key,
	    sizeof(assoc->ptk.eapol_key_encryption_key));

	/* Initialize RC4 key */
	RC4_set_key(&rc4_key, sizeof(rc4_key_data), rc4_key_data);

	/* Skip first 256 bytes of key stream (spec says so) */
	RC4(&rc4_key, sizeof(dummy), dummy, dummy);

	/* Encrypt GTK */
	RC4(&rc4_key, key_data_len, (u8 *)&assoc->port->gtk, key_desc->data);
    }
    else
    {
	/* Copy key data to packet */
	if (key_data_len > 0)
	    memcpy(key_desc->data, key_data, key_data_len);
    }

    /* Calculate MIC if it should be included in the packet */
    if (key_info_bits & KEY_INFO_MIC_BIT)
	calc_mic_on_eapol_pkt(assoc, eapol_pkt, &key_desc->mic);
    
    return eapol_pkt;	
}

/* Check whether a replay counter contained in a key descriptor is valid */
static int replay_verify(wpa_assoc_t *assoc, eapol_key_desc_t *key_desc)
{
   int result = 1;

   /* The replay counter is verified differently for replys and requests */
   if (TEST_BITS(key_desc->info, htons(KEY_INFO_REQUEST_BIT)))
   {
       /* For request EAPOL-Key messages, replay counter must be monotonic */
       if (!assoc->sta_to_ap_replay_count || 
	   knet_ntohll(key_desc->replay_count) > assoc->sta_to_ap_replay_count)
       {
	   result = 0;

	   /* Remember the last replay counter we received */
	   assoc->sta_to_ap_replay_count = knet_ntohll(key_desc->replay_count);
       }
   }
   else
   {
       /* For reply EAPOL-Key messages, the replay counter in the received 
	* EAPOL-Key must be identical to the replay counter in the last 
	* EAPOL-Key we sent
	*/
       if (assoc->ap_to_sta_replay_count == knet_ntohll(key_desc->replay_count))
	   result = 0;
   }
   
   if (result)
   {
       WPA_DEBUG("Invalid replay counter in message received from station %s", 
	   mactoa(&assoc->sta_mac)); 
   }

   return result;
}

static int mic_check(wpa_assoc_t *assoc, eapol_pkt_t *eapol_pkt)
{
/*	int i;*/
	
    wpa_eapol_mic_t mic_recv, mic_calc;
    eapol_key_desc_t *key_desc = (eapol_key_desc_t *)eapol_pkt->pkt_body;
    int result;

    memcpy(&mic_recv, &key_desc->mic, sizeof(wpa_eapol_mic_t));
    
    /* MIC should be calculated on packet when its MIC field is zero */
    MZERO(key_desc->mic);
    
    calc_mic_on_eapol_pkt(assoc, eapol_pkt, &mic_calc);

    result = memcmp(&mic_recv, &mic_calc, sizeof(wpa_eapol_mic_t));

#if 0
	printf("\nrec mic ");
	for(i = 0;i<sizeof(wpa_eapol_mic_t);i++)
	{
		printf("%02x ", mic_recv.data[i]);
		if(!(i & 0x7))
			printf("\n");
	}

	printf("\n @@ calc mic ");
	for(i = 0;i<sizeof(wpa_eapol_mic_t);i++)
	{
		printf("%02x ", mic_calc.data[i]);
		if(!(i & 0x7))
			printf("\n");
	}

	usleep(1000);
#endif

    if (result)
	rg_error(LWARN, "wpa: Bad MIC detected in EAPOL-Key packet");

    return result;
}

static void ptk_neg_timeout_cb(void *context);
static void gtk_neg_timeout_cb(void *context);

/* Sends an EAPOL-key packet */
static void send_eapol_key_pkt(wpa_assoc_t *assoc, 
    eapol_pkt_t *eapol_pkt, int is_ptk_neg_pkt)
{
    etimer_func_t timeout_cb = is_ptk_neg_pkt ? ptk_neg_timeout_cb : 
	gtk_neg_timeout_cb;
    
    event_timer_set(HANDSHAKE_TIMEOUT, timeout_cb, assoc);
    
    /* Send EAPOL packet. If a pairwise key is installed, use it to encrypt 
     * EAPOL-Key packet, otherwise send it unencrypted.
     */
    auth1x_conn_eapol_pkt_send(assoc->auth1x_conn, eapol_pkt, 
	assoc->pairwise_cipher ? 1 : 0);

    free(eapol_pkt);
}

/* PTK (4 way handshake) negotiation uses the following states:
 * WPA_START_MSG1, WPA_WAIT_MSG2, WPA_SEND_MSG3, WPA_WAIT_MSG4, WPA_DONE */
static void wpa_assoc_ptk_neg(wpa_assoc_t *assoc, eapol_pkt_t *in_eapol_pkt)
{
    eapol_key_desc_t *key_desc = NULL;
    eapol_pkt_t *out_eapol_pkt;
   
    /* eapol_pkt points to a received EAPOL packet when the function is called
     * in response to a received packet, or NULL when the function is called to
     * initiate a new 4-way handshake or to retry sending an EAPOL packet 
     */
    if (in_eapol_pkt)
    {
	key_desc = (eapol_key_desc_t *)in_eapol_pkt->pkt_body;
	
	/* Verify replay counter, ignore packet if invalid */
	if (replay_verify(assoc, key_desc))
	    return;

	/* In case the last message we sent was message 1, the message we just
	 * received is supposed to be message 2. We must check the MIC on the
	 * received message before moving to the next state, but before we can
	 * check the MIC we must calculate the PTK
	 */
	if (assoc->ptk_state == WPA_START_MSG1)
	{
	    /* Derive PTK */
	    wpa_key_utils_calc_ptk(&assoc->ptk, &assoc->pmk, &assoc->nonce, 
		&key_desc->nonce, assoc->port->params.mac, assoc->sta_mac);
	}
	
	/* Ignore packet if it has a bad MIC */
	if (mic_check(assoc, in_eapol_pkt))
	    return;

	event_timer_del(ptk_neg_timeout_cb, assoc);

	/* Check if the station reported of a unicast key MIC failure */
	if (IS_MIC_FAILURE_REPORT(key_desc))
	{
	    rg_error(LWARN, "wpa: Station %s reports of unicast key MIC "
		"failure", mactoa(&assoc->sta_mac));

	    /* Tell the port to handle the MIC failure */
		if (micFailureReportFromStation)
		    wpa_port_mic_failure(assoc->port, assoc);
	    return;
	}

	/* Check if the station requested a 4-way handshake */
	if (IS_KEY_REQUEST(key_desc))
	{
	    WPA_DEBUG("4-way handshake request received from station %s",
		mactoa(&assoc->sta_mac));

	    /* Initiate a new 4-way handshake */
	    wpa_assoc_start_ptk_neg(assoc);
	    return;
	}

	assoc->ptk_state++;
	assoc->ptk_neg_retries_left = HANDSHAKE_MAX_RETRIES;
    }


    switch (assoc->ptk_state) 
    {
    case WPA_START_MSG1:
	/* Generate ANonce */
	wpa_port_generate_nonce(assoc->port, &assoc->nonce);


	/* Create new EAPOL packet */
	out_eapol_pkt = create_eapol_key_pkt(assoc, KEY_INFO_PAIRWISE_BIT | 
	    KEY_INFO_ACK_BIT, 0, 0, WPA_TKIP_KEY_LEN, &assoc->nonce, 0, NULL, 
	    0);

	if (!out_eapol_pkt)
	    goto Error;

/*	WPA_DEBUG("Sending message 1 of 4-way handshake");*/

	send_eapol_key_pkt(assoc, out_eapol_pkt, 1);

	break;

    case WPA_WAIT_MSG2:
/*	WPA_DEBUG("Received message 2 of 4-way handshake");*/

	/* Information element received in packet must be identical to the one
	 * received in the station association request 
	 */
	if (memcmp(assoc->sta_info_elem, key_desc->data, 
	    MIN(wpa_info_elem_get_len(assoc->sta_info_elem), 
	    ntohs(key_desc->data_len)))) 
	{
	    rg_error(LWARN, "wpa: Information element in message 2 of "
		"4-way handshake doesn't match one in station's association "
		"request.\nExpected (length=%d):\n%s\nReceived "
		"(length=%d):\n%s", 
		wpa_info_elem_get_len(assoc->sta_info_elem), 
		wpa_log_format_buf((u8 *)assoc->sta_info_elem, 
		wpa_info_elem_get_len(assoc->sta_info_elem)),
		ntohs(key_desc->data_len),
		wpa_log_format_buf(key_desc->data, key_desc->data_len));

	    goto Error;
	}

	assoc->ptk_state++;

    /* Fall Through */
    /* WPA_WAIT_MSG2 and WPA_SEND_MSG3 are two separate states because we should
     * be able to resend message 3 (on retries) without doing the parsing on
     * message 2 (in the retransmission case, message 2 would be NULL)
     */
    case WPA_SEND_MSG3:
	/* Create new EAPOL packet */
	out_eapol_pkt = create_eapol_key_pkt(assoc, KEY_INFO_PAIRWISE_BIT | 
	    KEY_INFO_INSTALL_BIT | KEY_INFO_ACK_BIT | KEY_INFO_MIC_BIT, 0, 0,
	    WPA_TKIP_KEY_LEN, &assoc->nonce, 0, (u8 *)assoc->port->ap_info_elem,
	    wpa_info_elem_get_len(assoc->port->ap_info_elem));

	if (!out_eapol_pkt)
	    goto Error;
	
/*	WPA_DEBUG("Sending message 3 of 4-way handshake");*/
	/* Send EAPOL packet */
	send_eapol_key_pkt(assoc, out_eapol_pkt, 1);

	break;

    case WPA_WAIT_MSG4:	
/*	WPA_DEBUG("Received message 4 of 4-way handshake");*/
	
	WPA_DEBUG("Installing pairwise key for station %s",
		mactoa(&assoc->sta_mac));
	WPA_DEBUG("TKIP encryption key=\n%s",
	    wpa_log_format_buf((u8 *)&assoc->ptk.temporal_key.tkip.tkip_key, 
	    sizeof(assoc->ptk.temporal_key.tkip.tkip_key)));
	WPA_DEBUG("TX MIC key=\n%s",
	    wpa_log_format_buf((u8 *)&assoc->ptk.temporal_key.tkip.tx_mic_key, 
	    sizeof(assoc->ptk.temporal_key.tkip.tx_mic_key)));
	WPA_DEBUG("RX MIC key=\n%s",
	    wpa_log_format_buf((u8 *)&assoc->ptk.temporal_key.tkip.rx_mic_key, 
	    sizeof(assoc->ptk.temporal_key.tkip.rx_mic_key)));

	
	/* Install pairwise key to AP */
	assoc->port->params.ap_dev_ops->set_mapping_key(assoc->port->ap_dev, 
	    assoc->sta_mac, WPA_CIPHER_TKIP, &assoc->ptk.temporal_key);

	assoc->pairwise_cipher = WPA_CIPHER_TKIP;

	/* Once a pairwise key is installed to the AP, we can stop blocking
	 * traffic going to the station
	 */
	assoc->port->params.ap_dev_ops->block_unicast_traffic(
	    assoc->port->ap_dev, 0, assoc->sta_mac);

	assoc->ptk_state = WPA_DONE;

	/* Notify port that the 4-way handshake with the station succeeded */
	wpa_port_ptk_neg_end(assoc->port, assoc, 1);

	break;

    /* XXX Is this case really needed? */
    case WPA_DONE:
	break;
    }

    return;

Error:
    /* Notify port that the 4-way handshake failed */
    wpa_port_ptk_neg_end(assoc->port, assoc, 0);
}

/* GTK negotiation uses the following states:
 * WPA_START_MSG1, WPA_WAIT_MSG2, WPA_DONE */
static void wpa_assoc_gtk_neg(wpa_assoc_t *assoc, eapol_pkt_t *in_eapol_pkt)
{
    eapol_key_desc_t *key_desc;
    u8 iv[IV_FIELD_LEN];
    u64 rsc = 0;
    eapol_pkt_t *out_eapol_pkt;

    /* eapol_pkt points to a received EAPOL packet when the function is called
     * in response to a received packet, or NULL when the function is called to
     * initiate a new group key update or to retry sending an EAPOL packet
     */
    if (in_eapol_pkt)
    {
	key_desc = (eapol_key_desc_t *)in_eapol_pkt->pkt_body;

	/* Ignore message if replay counter is invalid */
	if (replay_verify(assoc, key_desc))
	    return;

	/* Ignore packet if it has a bad MIC */
	if (mic_check(assoc, in_eapol_pkt))
	    return;

	event_timer_del(gtk_neg_timeout_cb, assoc);

	/* Check if the station reported of a group key MIC failure */
	if (IS_MIC_FAILURE_REPORT(key_desc))
	{
	    rg_error(LWARN, "wpa: Station %s reports of group key MIC failure", 
		mactoa(&assoc->sta_mac));

	    /* Tell the port to handle the MIC failure */
		if (micFailureReportFromStation)
		    wpa_port_mic_failure(assoc->port, assoc);
	    return;
	}
	
	/* Check if the station requested a group key update */
	if (IS_KEY_REQUEST(key_desc))
	{
	    WPA_DEBUG("Group key update request received from station %s", 
	        mactoa(&assoc->sta_mac));

	    wpa_port_gtk_update_request(assoc->port, assoc);
	    return;
	}

	assoc->gtk_state++;
	assoc->gtk_neg_retries_left = HANDSHAKE_MAX_RETRIES;
    }
 
    switch (assoc->gtk_state) 
    {
    case WPA_START_MSG1:
	/* RSC field should contain TSC for TKIP or 0 for WEP */
	if (assoc->port->multicast_cipher == WPA_CIPHER_TKIP)
	{
	    /* XXX This is splitted into 2 lines because of B7579 */
	    rsc = assoc->port->params.ap_dev_ops->get_tsc(assoc->port->ap_dev) &
		TSC_MASK;
	    rsc = knet_cpu_to_le64(rsc);
	}

    	/* For the key IV we use the lower 16 bytes of the key counter. Our key
     	 * counter is always big endian so we take its last 16 bytes
	 */
	memcpy(iv, &assoc->port->key_counter.data[16], sizeof(iv));
	
	/* Create EAPOL packet */
	out_eapol_pkt = create_eapol_key_pkt(assoc, KEY_INFO_ACK_BIT | 
	    KEY_INFO_MIC_BIT | KEY_INFO_SECURE_BIT, 
	    assoc->port->current_gtk_index, rsc, 
	    wpa_key_utils_get_key_len(assoc->port->multicast_cipher),
	    &assoc->port->gnonce, iv, (u8 *)&assoc->port->gtk, 
	    wpa_key_utils_get_key_len(assoc->port->multicast_cipher));
	
	if (!out_eapol_pkt)
	    goto Error;

	/* Once we sent the GTK to the station, it potentially has the key. We
	 * intentionally update this flag as early as now and not when the 
	 * station acknowledges receiving the key, to handle the case where
	 * a hostile station interested in sniffing the network may request us 
	 * for a key and not acknowledge receiving in order to confuse us.
	 */
	assoc->has_gtk = 1;
	
/*	WPA_DEBUG("Sending message 1 of group key update");*/

	/* Send EAPOL packet */
	send_eapol_key_pkt(assoc, out_eapol_pkt, 0);
	break;

    case WPA_WAIT_MSG2:
/*	WPA_DEBUG("Received message 2 of group key update");*/

	assoc->gtk_state = WPA_DONE;

	/* Notify port that group key update with this station succeeded */
	wpa_port_gtk_neg_end(assoc->port, assoc, 1);
	break;

    /* XXX Is this case really needed? */
    case WPA_DONE:
	break;
	
    default:
	/* Should never reach here */
	break;
    }

    return;

Error:
    /* Notify port the the group key update with this station failed */
    wpa_port_gtk_neg_end(assoc->port, assoc, 0);
    return;
}

static void send_pairwise_key_to_8021x_sta(wpa_assoc_t *assoc)
{
    eapol_pkt_t *eapol_pkt;
    u8 *pmk = NULL;
    int pmk_len;
      
/*    WPA_DEBUG("Sending pairwise key to %s in 802.1X EAPOL-Key", 
	mactoa(&assoc->sta_mac));*/

    /* Get the PMK that has been created as result of the 802.1X authentication
     */
    auth1x_conn_pmk_get(assoc->auth1x_conn, &pmk, &pmk_len);
	
    /* Create an EAPOL-Key packet that only specifies the length of the key, 
     * the station will use the PMK as its unicast key
     */
    eapol_pkt = eap_rekey_pkt_create(&assoc->sta_mac, &assoc->port->params.mac,
	NULL, 
	wpa_key_utils_get_key_len(assoc->port->params.rekeying_wep_cipher), 0,
	1, pmk, 0);

    if (!eapol_pkt)
    {
	/* Notify port that we failed to send the key */
	wpa_port_ptk_neg_end(assoc->port, assoc, 0);
	return;
    }
  
    WPA_DEBUG("Installing %s pairwise key for station %s. Key=\n%s",
	wpa_log_cipher_to_str(assoc->port->params.rekeying_wep_cipher), 
	mactoa(&assoc->sta_mac), wpa_log_format_buf(pmk,
	wpa_key_utils_get_key_len(assoc->port->params.rekeying_wep_cipher)));
    
	/* sleep so that the EAPOL frame would come out unencrypted */
	sleep(1); 

    assoc->port->params.ap_dev_ops->set_mapping_key(assoc->port->ap_dev, 
	assoc->sta_mac, assoc->port->params.rekeying_wep_cipher, 
	(wpa_temporal_key_t *)pmk);

    assoc->pairwise_cipher = assoc->port->params.rekeying_wep_cipher;

    /* Unicast traffic to station can now be encrypted using the unicast key, so
     * we can stop blocking it
     */
    assoc->port->params.ap_dev_ops->block_unicast_traffic(assoc->port->ap_dev, 
	0, assoc->sta_mac);
   
    /* Send EAPOL-Key packet */
    auth1x_conn_eapol_pkt_send(assoc->auth1x_conn, eapol_pkt, 0);

    free(eapol_pkt);

    /* Notify port that the key was sent successfully */
    wpa_port_ptk_neg_end(assoc->port, assoc, 1);
}

/* Starts PTK (4 way handshake) negotiation */
void wpa_assoc_start_ptk_neg(wpa_assoc_t *assoc)
{
    if (!assoc->pmk_valid)
    {
	wpa_port_ptk_neg_end(assoc->port, assoc, 0);
	return;
    }
   
    /* Use 4-way handshake to send unicast key to WPA stations or a single
     * EAPOL-Key to send it to WEP-802.1X stations 
     */
    if (assoc->sta_type == WPA_STA_TYPE_WPA)
    {
	assoc->ptk_state = WPA_START_MSG1;
	assoc->ptk_neg_retries_left = HANDSHAKE_MAX_RETRIES;

	wpa_assoc_ptk_neg(assoc, NULL);
    }
    else
	send_pairwise_key_to_8021x_sta(assoc);
}

/* Sends GTK in a legacy 802.1X EAPOL-Key message */
static void send_group_key_to_8021x_sta(wpa_assoc_t *assoc)
{
    eapol_pkt_t *eapol_pkt;
    u8 *pmk = NULL;
    int pmk_len;
      
    WPA_DEBUG("Sending group key to %s in 802.1X EAPOL-Key", 
	mactoa(&assoc->sta_mac));
   
    /* Get the PMK that has been created as result of the 802.1X authentication
     */
    auth1x_conn_pmk_get(assoc->auth1x_conn, &pmk, &pmk_len);
    if (!pmk)
	return;

    /* Create an EAPOL-Key packet containing the GTK */
    eapol_pkt = eap_rekey_pkt_create(&assoc->sta_mac, &assoc->port->params.mac, 
	(u8 *)&assoc->port->gtk, 
	wpa_key_utils_get_key_len(assoc->port->multicast_cipher), 
	assoc->port->current_gtk_index, 0, pmk, 0);

    if (!eapol_pkt)
    {
	/* Notify port that we failed to send the group key */
	wpa_port_gtk_neg_end(assoc->port, assoc, 0);

	return;
    }

    /* Send EAPOL-Key packet */
    auth1x_conn_eapol_pkt_send(assoc->auth1x_conn, eapol_pkt, 0);

    free(eapol_pkt);

    /* We sent the key to the station, we'll never know if it got it or not
     * (because there is no ack defined in the protocol) but we have to assume 
     * that it potentially has the key
     */
    assoc->has_gtk = 1;

    /* Notify port the the group key update with this station succeeded */
    wpa_port_gtk_neg_end(assoc->port, assoc, 1);
}

/* Starts GTK negotiation */
void wpa_assoc_start_gtk_neg(wpa_assoc_t *assoc)
{
    /* We send the GTK to WPA stations by doing group key update and to 
     * WEP-802.1X stations by using a single EAPOL-Key message
     */
    if (assoc->sta_type == WPA_STA_TYPE_WPA)
    {
	assoc->gtk_state = WPA_START_MSG1;
	assoc->gtk_neg_retries_left = HANDSHAKE_MAX_RETRIES;
	wpa_assoc_gtk_neg(assoc, NULL);
    }
    else
	/* We want the completion function called from within
	 * send_group_key_to_8021x_sta to be called only after the current
	 * execution thread finishes
	 */
	event_timer_set(0, (etimer_func_t)send_group_key_to_8021x_sta, assoc);
}

/* Receive and process EAPoL-key  */
static void wpa_assoc_eapol_key_rcv(void *context, eapol_pkt_t *eapol_pkt,
    int is_encrypted)
{
    wpa_assoc_t *assoc = context;

    /* Check if this EAPOL-Key is part of 4-way handshake or group key update 
     * and pass it to the correct state machine
     */
    if (TEST_BITS(((eapol_key_desc_t *)eapol_pkt->pkt_body)->info, 
	ntohs(KEY_INFO_PAIRWISE_BIT)))
    {
	wpa_assoc_ptk_neg(assoc, eapol_pkt);
    }
    else
	wpa_assoc_gtk_neg(assoc, eapol_pkt);
}

/* 802.1x authentication end callback */
static void wpa_assoc_auth1x_end(void *context, auth1x_conn_t *conn, 
    int is_authenticated, u8 *pmk, int pmk_len)
{
    wpa_assoc_t *assoc = context;

    if (!is_authenticated)
	goto Exit;

    if (pmk && pmk_len >= sizeof(wpa_pmk_t))
    {
	memcpy(&assoc->pmk, pmk, sizeof(wpa_pmk_t));
	assoc->pmk_valid = 1;
    }
    else
    {
	rg_error(LWARN, "wpa: 802.1X authentication produced a PMK which is too"
	    "small: %d bytes, expected at least %d bytes", pmk_len, 
	    sizeof(wpa_pmk_t));

	is_authenticated = 0;
    }

Exit:
    wpa_port_8021x_auth_end(assoc->port, assoc, is_authenticated); 
}

/* Called when the station type is detected to update the association object and
 * the port counters
 */
void wpa_assoc_set_sta_type(wpa_assoc_t *assoc, wpa_sta_type_t sta_type)
{
    /* In some cases we assume the station is of a certain type and then change
     * the type when we find out we were wrong, so update the relevant counter
     */
    if (assoc->sta_type)
	wpa_port_update_sta_counter(assoc->port, sta_type, -1);
    
    assoc->sta_type = sta_type;

    /* Count station by its new type */
    wpa_port_update_sta_counter(assoc->port, sta_type, 1);
}

/* Called when no data is received from a station since it associated for a
 * defined timeout
 */
static void wait_for_first_packet_timeout_cb(void *context)
{
    wpa_assoc_t *assoc = (wpa_assoc_t *)context;
    
    WPA_DEBUG("Didn't receive data from station %s. It's assumed to be "
	"WEP-legacy station", mactoa(&assoc->sta_mac));
    
    /* Nothing has been received from the station yet, we assume it's a legacy
     * station (a 802.1X station would have sent EAPOL-start packet by now)
     */
    wpa_assoc_set_sta_type(assoc, WPA_STA_TYPE_WEP_LEGACY);
   
    /* Now that we know the station type, let the port do type-specific handling
     * for the station
     */
    wpa_port_sta_type_detected(assoc->port, assoc);
}

/* Called when we found out whether a certain WEP station is legacy or 802.1X */
void wpa_assoc_sta_type_detected(wpa_assoc_t *assoc)
{
    WPA_DEBUG("Station %s detected to be %s", mactoa(&assoc->sta_mac), 
	wpa_log_sta_type_to_str(assoc->sta_type));
    
    if (assoc->sta_type == WPA_STA_TYPE_WEP_LEGACY)
    {
	if (assoc->auth1x_conn)
	{
	    /* There is no point in trying to do 802.1X authentication with a
	     * non-802.1X station so close 802.1X connection
	     */
	    auth1x_conn_close(assoc->auth1x_conn);
	    assoc->auth1x_conn = NULL;
	}

	WPA_DEBUG("Unblocking unicast traffic to station %s", 
	    mactoa(&assoc->sta_mac));
	
	/* No authentication is required with legacy stations, so we can stop 
	 * blocking unicast traffic to the station now
	 */
	assoc->port->params.ap_dev_ops->block_unicast_traffic(
	    assoc->port->ap_dev, 0, assoc->sta_mac);
    }

    /* We just found out the station type so we no longer need the timeout */
    event_timer_del(wait_for_first_packet_timeout_cb, assoc);
}

static void ptk_neg_timeout_cb(void *context)
{
    wpa_assoc_t *assoc = context;

    WPA_DEBUG("Timeout waiting for reply from station");
    
    /* Retry sending the last packet */
    if (assoc->ptk_neg_retries_left--)
    {
	WPA_DEBUG("Retrying to send EAPOL-Key message");

	wpa_assoc_ptk_neg(assoc, NULL);

	return;
    }

    /* Number of retries exceeded, notify port the 4-way handshake failed */
    wpa_port_ptk_neg_end(assoc->port, assoc, 0);
}


/* Called when a station doesn't respond to one of our messages */
static void gtk_neg_timeout_cb(void *context)
{
    wpa_assoc_t *assoc = context;

    WPA_DEBUG("Timeout waiting for reply from station");
    
    /* Retry sending the last packet */
    if (assoc->gtk_neg_retries_left--)
    {
	WPA_DEBUG("Retrying to send EAPOL-Key message");

	wpa_assoc_gtk_neg(assoc, NULL);

	return;
    }

    /* Number of retries exceeded, notify port that the group key update
     * failed
     */
    wpa_port_gtk_neg_end(assoc->port, assoc, 0);
}

/* Callbacks that the 802.1X package will use to notify us of important events
 */
static auth1x_conn_cb_t auth1x_conn_cb = {
    wpa_assoc_eapol_key_rcv,
    wpa_assoc_auth1x_end,
};

/* Opens a new association */
wpa_assoc_t *wpa_assoc_open(wpa_port_t *port, ether_mac_t sta_mac, 
    wpa_info_elem_t *sta_info_elem)
{
    wpa_assoc_t *assoc = (wpa_assoc_t *)zalloc_l(sizeof(wpa_assoc_t));

    if (!assoc)
	return NULL;
    
    /* Fill new association context */
    assoc->port = port;
    memcpy(assoc->sta_mac, sta_mac, sizeof(ether_mac_t));

    /* Try to detect station type */
    if (sta_info_elem)
    {
	/* This station has an information element so it's a WPA station */
	wpa_assoc_set_sta_type(assoc, WPA_STA_TYPE_WPA);
    }
    /* If WEP-legacy stations are allowed and WEP-802.1X are not allowed, we
     * will assume that this non-WPA station is a WEP-legacy station instead
     * of waiting for the first packet coming from it and telling the 
     * station type by it. If we didn't use this assumption we would have to
     * block traffic to this station for up to 2 seconds.
     */
    else if ((assoc->port->params.allowed_sta_types & WPA_STA_TYPE_WEP_LEGACY) 
	&& !(assoc->port->params.allowed_sta_types & WPA_STA_TYPE_WEP_8021X))
    {
	wpa_assoc_set_sta_type(assoc, WPA_STA_TYPE_WEP_LEGACY);
    }
    /* If both WEP-legacy and WEP-802.1X stations are allowed, we still have
     * no way to tell whether it's a 802.1X station or a legacy station so 
     * we wait for the first packet it sends us. If no packet arrives within
     * 2 seconds, we assume it's a legacy station because a 802.1X is 
     * expected to send us an EAPOL packet right after association. Traffic
     * going to this station is blocked until we detect its type.
     */ 
    else
    {
	event_timer_set(WPA_WAIT_FOR_FIRST_PACKET_TIMEOUT,
	    wait_for_first_packet_timeout_cb, assoc);
    }

    /* Do stuff which is relevant only for WPA stations */
    if (assoc->sta_type == WPA_STA_TYPE_WPA)
    {
	/* Save station's information element */
	assoc->sta_info_elem = zalloc_l(wpa_info_elem_get_len(sta_info_elem));
	if (!assoc->sta_info_elem)
	    goto Error;

	memcpy(assoc->sta_info_elem, sta_info_elem, 
	    wpa_info_elem_get_len(sta_info_elem));

	/* If we're working in PSK mode, set PMK to be the PSK */
	if (assoc->port->params.psk_param != WPA_PSK_PARAM_NONE)
	{
	    memcpy(&assoc->pmk, &assoc->port->psk, sizeof(assoc->pmk));

	    assoc->pmk_valid = 1;

	    WPA_DEBUG("Using PSK as PMK, PMK =\n%s", 
		wpa_log_format_buf((u8 *)&assoc->pmk, sizeof(assoc->pmk)));
	}
    }

    /* Open a 802.1X connection for types of stations that need it. This 
     * includes also stations which we don't know their type yet, because they 
     * might be WEP-802.1X stations, so we must be prepared to handle incoming
     * EAPOL-Start messages from those stations.
     */
    if (!assoc->sta_type || (assoc->sta_type & (WPA_STA_TYPE_WPA | 
	WPA_STA_TYPE_WEP_8021X)))
    {
	assoc->auth1x_conn = auth1x_conn_open(assoc->port->params.auth1x_port, 
	    &assoc->sta_mac, &auth1x_conn_cb, assoc);
	if (!assoc->auth1x_conn)
	    goto Error;

	/* Unless this is a WPA-PSK station, we also have to be prepared for the
	 * station starting 802.1X authentication
	 */
	if (!((assoc->sta_type == WPA_STA_TYPE_WPA) && 
	    (assoc->port->params.psk_param != WPA_PSK_PARAM_NONE)))
	{
	    auth1x_conn_auth_start(assoc->auth1x_conn);
	}
    }
    
    return assoc;

Error:
    if (assoc->auth1x_conn)
	auth1x_conn_close(assoc->auth1x_conn);
    
    if (assoc->sta_info_elem)
	wpa_info_elem_close(assoc->sta_info_elem);

    event_timer_del(wait_for_first_packet_timeout_cb, assoc);
    
    nfree(assoc);

    return NULL;
}

/* Disassociates a station */
void wpa_assoc_disassoc(wpa_assoc_t *assoc)
{
    WPA_DEBUG("Disassociating station %s", mactoa(&assoc->sta_mac));
    
    /* Disassociate station */
    assoc->port->params.ap_dev_ops->disassoc_sta(assoc->port->ap_dev, 
	assoc->sta_mac);
}

/* Deauthenticates a station */
void wpa_assoc_deauth(wpa_assoc_t *assoc)
{
    WPA_DEBUG("Deauthenticating station %s", mactoa(&assoc->sta_mac));
    
    /* Deauthenticate station */
    assoc->port->params.ap_dev_ops->deauth_sta(assoc->port->ap_dev, 
	assoc->sta_mac);
}

/* Cleans up and frees an association object */
void wpa_assoc_close(wpa_assoc_t *assoc)
{
    WPA_DEBUG("Removing pairwise key of station %s", mactoa(&assoc->sta_mac)); 
    
    /* Remove PTK from AP */
    if (assoc->pairwise_cipher)
    {
	assoc->port->params.ap_dev_ops->remove_mapping_key(assoc->port->ap_dev,
	    assoc->sta_mac, assoc->pairwise_cipher);

	assoc->pairwise_cipher = 0;
    }

    event_timer_del(ptk_neg_timeout_cb, assoc);
    event_timer_del(gtk_neg_timeout_cb, assoc);
    event_timer_del(wait_for_first_packet_timeout_cb, assoc);

    if (assoc->auth1x_conn)
	auth1x_conn_close(assoc->auth1x_conn);

    if (assoc->sta_info_elem)
	wpa_info_elem_close(assoc->sta_info_elem);
	
    /*free(assoc);*/
}

void wpa_assoc_free(wpa_assoc_t *assoc)
{
	free(assoc);
}

int wpa_assoc_is_in_gtk_update(wpa_assoc_t *assoc)
{
    /* Only WPA stations may be in the process of a GTK update */
    if (assoc->sta_type != WPA_STA_TYPE_WPA)
	return 0;
    
    return assoc->gtk_state == WPA_START_MSG1;
}

void wdrv_wpaSetMicFailureReportFromStation(BOOL value)
{
	micFailureReportFromStation = value;
}

BOOL wdrv_wpaGetMicFailureReportFromStation(void)
{
	return micFailureReportFromStation;
}