/* LICENSE_CODE JUNGO */

/* This code is adapted from IEEE standard 802.11i/D3.0 */

#include <openssl/sha.h>
#include <openssl/rand.h>
#include <openssl/md5.h>
#include <rg_memory.h>
#include <time.h>
#include <util/conv.h>
#include "wpa_key_utils.h"

#ifndef MIN
#define MIN(a, b) (((a) < (b)) ? (a) : (b))
#endif

/* This macro used for concatenation of memory blocks.
 * Difference from generic memcpy - return value is pointer to next free
 * byte after concatenation
 */
#define MEMCPY(dst, src, size) \
   memcpy((void *)(dst), (void *)(src), (size)) + (size)

/* HMAC-SHA1 is the EAPOL-Key encryption algorithm used to protect the
 * distributed GTK
 */
void hmac_sha1(u8 *digest, u8 *text, int text_len, u8 *key, int key_len)
{
    SHA_CTX context;
    u8 k_ipad[64]; /* inner padding - key XORd with ipad */
    u8 k_opad[64]; /* outer padding - key XORd with opad */
    int i;
    u8 *key_dup;
    
    key_dup = malloc_l(key_len);
    if (!key_dup)
	return;
	    
    memcpy(key_dup, key, key_len);
    
    /* if key is longer than 64 bytes reset it to key=SHA1(key) */
    if (key_len > 64)
    {
	SHA_CTX tctx;

	SHA1_Init(&tctx);
	SHA1_Update(&tctx, key_dup, key_len);
	SHA1_Final(key_dup, &tctx);

	key_len = 20;
    }

    /* the HMAC_SHA1 transform looks like:
     *
     * SHA1(K XOR opad, SHA1(K XOR ipad, text))
     *
     * where K is an n byte key
     * ipad is the byte 0x36 repeated 64 times
     * opad is the byte 0x5c repeated 64 times
     * and text is the data being protected
     */

    /* start out by storing key in pads */
    MZERO(k_ipad);
    MZERO(k_opad);
    memcpy(k_ipad, key_dup, key_len);
    memcpy(k_opad, key_dup, key_len);

    /* XOR key with ipad and opad values */
    for (i = 0; i < 64; i++)
    {
	k_ipad[i] ^= 0x36;
	k_opad[i] ^= 0x5c;
    }

    /* perform inner SHA1*/
    SHA1_Init(&context); /* init context for 1st pass */
    SHA1_Update(&context, k_ipad, 64); /* start with inner pad */
    SHA1_Update(&context, text, text_len); /* then text of datagram */
    SHA1_Final(digest, &context); /* finish up 1st pass */

    /* perform outer SHA1 */
    SHA1_Init(&context); /* init context for 2nd pass */
    SHA1_Update(&context, k_opad, 64); /* start with outer pad */
    SHA1_Update(&context, digest, 20); /* then results of 1st hash */
    SHA1_Final(digest, &context); /* finish up 2nd pass */

    free(key_dup);
}

/* Pseudo-random function for generate cryptographic quality random numbers.
 * Length of output is in octets rather than bits since length is always a 
 * multiple of 8.
 * Output array is organized so first N octets starting from 0 contains prf 
 * output.
 * Supported inputs are 16, 32, 48, 64
 */
void prf(u8 *key, int key_len, u8 *prefix, int prefix_len, u8 *data, 
    int data_len, u8 *output, int len)
{
    int i;
    u8 input[1024]; /* concatenated input */
    u8 buff[20]; /* output array to allow for sha1 overflow */
    int currentindex = 0;
    int total_len;
    
    memcpy(input, prefix, prefix_len);
    input[prefix_len] = 0; /* single byte 0 */
    memcpy(&input[prefix_len+1], data, data_len);
    total_len = prefix_len + 1 + data_len;
    input[total_len] = 0; /* single byte count, starts at 0 */
    total_len++;
    for (i = 0; i < (len+19)/20; i++)
    {
	hmac_sha1(buff, input, total_len, key, key_len);
	memcpy(&output[currentindex], buff, MIN(20, len - currentindex));
	currentindex += 20; /* next concatenation location */
	input[total_len-1]++; /* increment octet count */
    }
}

static void digest_hash(u8 *output, char *password, u8 *ssid, int ssid_len,
    int iterations, int count)
{
    u8 digest[36], digest1[SHA_DIGEST_LENGTH];
    int i, j;

    /* U1 = PRF(P, S || int(i)) */
    memcpy(digest, ssid, ssid_len);
    digest[ssid_len] = (u8)((count>>24) & 0xff);
    digest[ssid_len+1] = (u8)((count>>16) & 0xff);
    digest[ssid_len+2] = (u8)((count>>8) & 0xff);
    digest[ssid_len+3] = (u8)(count & 0xff);
    hmac_sha1(digest1, digest, ssid_len+4, password, strlen(password));

    /* output = U1 */
    memcpy(output, digest1, SHA_DIGEST_LENGTH);

    for (i = 1; i < iterations; i++)
    {
	/* Un = PRF(P, Un-1) */
	hmac_sha1(digest, digest1, SHA_DIGEST_LENGTH, password, 
	    strlen(password));
	memcpy(digest1, digest, SHA_DIGEST_LENGTH);

	/* output = output xor Un */
	for (j = 0; j < SHA_DIGEST_LENGTH; j++)
	    output[j] ^= digest[j];
    }
}

/* Calculates a pre-shared key from a pass-phrase.
 * Source code adapted from Annex F.8 of IEEE Standard 802.11i/D3.0
 */
void wpa_key_utils_calc_psk(wpa_pmk_t *psk, char *pass_phrase, u8 *ssid,
    int ssid_len)
{
    u8 buff[SHA_DIGEST_LENGTH];

    digest_hash((u8 *)psk, pass_phrase, ssid, ssid_len, 4096, 1);
    digest_hash(buff, pass_phrase, ssid, ssid_len, 4096, 2);
    memcpy((u8 *)psk + SHA_DIGEST_LENGTH, buff, sizeof(*psk) - 
	SHA_DIGEST_LENGTH);
}

/* Initializes a key counter.
 * Implemented according to clause 8.5.7 of IEEE Standard 802.11i/D3.0
 */  
#define SEC_IN_70_YEARS 2208988800UL;

void wpa_key_utils_init_key_counter(wpa_nonce_t *key_counter,
    ether_mac_t local_addr)
{
    wpa_nonce_t random_number;
    u8 data[sizeof(ether_mac_t) + 8];
    unsigned long ntp_seconds;
    
    /* data is concatenation of MAC address and current time in NTP format
     * sizeof(ether_mac_t) first bytes - MAC address (input parameter),
     * four next bytes - time in seconds from 1900 year,
     * four last bytes - must be fractional seconds, in our case - random value
     */
    memcpy(data, local_addr, sizeof(ether_mac_t));
    ntp_seconds = time(NULL) + SEC_IN_70_YEARS;
    memcpy(&data[sizeof(ether_mac_t)], &ntp_seconds, sizeof(ntp_seconds));
    RAND_bytes(&data[sizeof(ether_mac_t) + 4], 4);
    RAND_bytes((u8 *)&random_number, sizeof(random_number));
    
    prf((u8 *)&random_number, sizeof(random_number), "Init Counter", 12, data,
	sizeof(data), (u8 *)key_counter, sizeof(*key_counter));
}

/* Compare two memory blocks and puts to dst less block, then larger block.
 * Return value - pointer to next free byte after concatenation
 */
static u8 *wpa_key_utils_minmax_concat(u8 *dst, u8 *src1, u8 *src2, int size)
{
    u8 *current_pos = dst;
    u8 *min_ptr, *max_ptr;

    if (memcmp(src1, src2, size)<0)
    {
	min_ptr = src1;
	max_ptr = src2;
    }
    else
    {
	min_ptr = src2;
	max_ptr = src1;
    }
    
    current_pos = MEMCPY(current_pos, min_ptr, size);
    current_pos = MEMCPY(current_pos, max_ptr, size);
    return current_pos;
}

/* Generates a Pairwise Transient Key.
 * Implemented according to section 8.5.1.2 of standard IEEE Std 802.11i/D3.0
 */
void wpa_key_utils_calc_ptk(wpa_ptk_t *ptk, wpa_pmk_t *pmk, 
    wpa_nonce_t *anonce, wpa_nonce_t *snonce, ether_mac_t authenticator_addr,
    ether_mac_t supplicant_addr)
{
    u8 data[sizeof(ether_mac_t)*2 + sizeof(wpa_nonce_t)*2];
    u8 *current_pos = data;

    current_pos = wpa_key_utils_minmax_concat(current_pos, authenticator_addr,
	supplicant_addr, sizeof(ether_mac_t));
    
    wpa_key_utils_minmax_concat(current_pos, (u8 *)anonce, (u8 *)snonce,
	sizeof(wpa_nonce_t));

    prf((u8 *)pmk, sizeof(*pmk), "Pairwise key expansion", 22, data,
	sizeof(data), (u8 *)ptk, sizeof(*ptk));
}

/* Calculates a Group Transient Key. 
 * Implemented according to section 8.5.1.3 of standard IEEE Std 802.11i/D3.0
 */
void wpa_key_utils_calc_gtk(wpa_gtk_t *gtk, wpa_gmk_t *gmk, wpa_nonce_t *gnonce,
    ether_mac_t authenticator_addr)
{
    u8 data[sizeof(ether_mac_t) + sizeof(wpa_nonce_t)];
    u8 *current_pos = data;
    
    current_pos = MEMCPY(current_pos, authenticator_addr, sizeof(ether_mac_t));
    memcpy(current_pos, gnonce, sizeof(wpa_nonce_t));

    prf((u8 *)gmk, sizeof(*gmk), "Group key expansion", 19, data, sizeof(data),
	(u8 *)gtk, sizeof(*gtk));
}

/* Increment of any value, represented as array of bytes */
void wpa_key_utils_inc(u8 *value, int size)
{
    while (size--)
    {
	value[size]++;
	if (value[size])
	    return;
    }
}

/* Calculate HMAC-MD5 over a buffer 
 * Source code adapted from Annex F.3 of IEEE Standard 802.11i/D3.0
 */
void wpa_key_util_hmac_md5(u8 *digest, u8 *data, int data_len, u8 *key,
    int key_len)
{
    MD5_CTX context;
    u8 k_ipad[65]; /* inner padding - key XORd with ipad */
    u8 k_opad[65]; /* outer padding - key XORd with opad */
    int i;
    u8 shorted_key[16]; /* used in case very long key */

    /* if key is longer than 64 bytes reset it to key=MD5(key) */
    if (key_len > 64)
    {
	MD5_CTX tctx;

	MD5_Init(&tctx);
	MD5_Update(&tctx, key, key_len);
	MD5_Final(shorted_key, &tctx);

	key = shorted_key;
	key_len = 16;
    }

    /* the HMAC_MD5 transform looks like:
     *
     *   MD5(K XOR opad, MD5(K XOR ipad, date))
     *
     * where K is an n byte key
     * ipad is the byte 0x36 repeated 64 times
     * opad is the byte 0x5c repeated 64 times
     * and text is the data being protected
     */

    /* start out by storing key in pads */
    MZERO(k_ipad);
    MZERO(k_opad);
    memcpy(k_ipad, key, key_len);
    memcpy(k_opad, key, key_len);

    /* XOR key with ipad and opad values */
    for (i = 0; i < 64; i++)
    {
	k_ipad[i] ^= 0x36;
	k_opad[i] ^= 0x5c;
    }

    /* perform inner MD5 */
    MD5_Init(&context); /* init context for 1st pass */
    MD5_Update(&context, k_ipad, 64); /* start with inner pad */
    MD5_Update(&context, data, data_len); /* then text of datagram */
    MD5_Final((void *)digest, &context); /* finish up 1st pass */

    /* perform outer MD5 */
    MD5_Init(&context); /* init context for 2nd pass */
    MD5_Update(&context, k_opad, 64);
    
    /* start with outer pad */
    MD5_Update(&context, digest, 16);
    
    /* then results of 1st hash */
    MD5_Final((void *)digest, &context); /* finish up 2nd pass */
}

int wpa_key_utils_get_key_len(wpa_cipher_t cipher)
{
    code2code_t cipher2key_len[] = {
	{WPA_CIPHER_TKIP, WPA_TKIP_KEY_LEN},
	{WPA_CIPHER_WEP_40, WPA_WEP_40_KEY_LEN},
	{WPA_CIPHER_WEP_104, WPA_WEP_104_KEY_LEN},
	{WPA_CIPHER_WEP_232, WPA_WEP_232_KEY_LEN},
	{-1, 0}
    };
    return code2code(cipher2key_len, cipher);
}
