/* LICENSE_CODE JUNGO */

#include "rg_error.h"
#include <net_utils.h>
#include <event.h>
#include <util/random.h>
#include <rg_memory.h>
#include <rg_error.h>
#include <syscalls.h>
#include "wpa_port.h"
#include "wpa_info_elem.h"
#include "wpa_common.h"
#include "wpa_assoc.h"
#include "wpa_info_elem.h"
#include "wpa_key_utils.h"
#include "wpa_log.h"

#define MIC_FAILURE_DELAY_MS 60000 /* 60 sec */
#define MIC_COUNTERMEASURES_PERIOD_MS 60000 /* 60 sec */

#define PORT_IN_WPA_PSK_MODE(t) (t->params.psk_param != WPA_PSK_PARAM_NONE)

void wpa_port_update_sta_counter(wpa_port_t *t, wpa_sta_type_t sta_type, 
    int delta)
{
    switch(sta_type)
    {
	case WPA_STA_TYPE_WPA:
	    t->num_of_wpa_stas += delta;
	    break;

	case WPA_STA_TYPE_WEP_LEGACY:
	    t->num_of_legacy_stas += delta;
	    break;

	case WPA_STA_TYPE_WEP_8021X:
	    t->num_of_8021x_stas += delta;
	    break;

	default:
	    break;
    }
}

static wpa_assoc_t *find_assoc(wpa_port_t *t, ether_mac_t mac)
{
    wpa_assoc_t *assoc;

    for (assoc = t->assoc_list; assoc; assoc = assoc->next)
    {
	if (!memcmp(assoc->sta_mac, mac, sizeof(ether_mac_t)))
	    break;
    }

    return assoc;
}

static wpa_cipher_t select_multicast_cipher(wpa_port_t *t)
{
    wpa_cipher_t multicast_cipher;
    
    /* If WEP-legacy stations are allowed, we must use the cipher of the 
     * preconfigured key as our multicast cipher
     */
    if (t->params.allowed_sta_types & WPA_STA_TYPE_WEP_LEGACY)
    {
	multicast_cipher = t->params.wep_keys[t->params.wep_key_index].cipher;
    }
    /* If there are WEP-802.1X stations, we must use the WEP cipher configured
     * by the user for multicast */ 
    else if ((t->params.allowed_sta_types & WPA_STA_TYPE_WEP_8021X))
    	multicast_cipher = t->params.rekeying_wep_cipher;
    /* Not configured for backward compatibility, we can use TKIP */
    else
	multicast_cipher = WPA_CIPHER_TKIP;

    return multicast_cipher;
}

static int dup_params(wpa_port_params_t *dst_params, 
    wpa_port_params_t *src_params)
{
    dst_params->allowed_sta_types = src_params->allowed_sta_types;
    dst_params->rekeying_wep_cipher = src_params->rekeying_wep_cipher;
    dst_params->gtk_update_interval = src_params->gtk_update_interval;
    memcpy(dst_params->wep_keys, src_params->wep_keys, 
	sizeof(dst_params->wep_keys));
    dst_params->wep_key_index = src_params->wep_key_index;
    dst_params->ssid_len = src_params->ssid_len;
    dst_params->dev_id = src_params->dev_id;
    dst_params->auth1x_port = src_params->auth1x_port;
    memcpy(dst_params->mac, src_params->mac, sizeof(ether_mac_t));

    /* Copy PSK */
    dst_params->psk_param = src_params->psk_param;

    switch (src_params->psk_param)
    {
    case WPA_PSK_PARAM_ASCII:
	dst_params->psk.ascii = strdup_l(src_params->psk.ascii);

	if (!dst_params->psk.ascii)
	    return -1;
	break;

    case WPA_PSK_PARAM_HEX:
	memcpy(&dst_params->psk.hex, &src_params->psk.hex, sizeof(wpa_psk_t));

	break;

    default:
	break;
    }

    /* Copy SSID */
    dst_params->ssid = malloc_l(src_params->ssid_len);

  
    if (!dst_params->ssid)
	goto Error;
   
    memcpy(dst_params->ssid, src_params->ssid, dst_params->ssid_len);

 	/* Copy ap_dev_ops */
    dst_params->ap_dev_ops = malloc_l(sizeof(ap_dev_ops_t));

    if (!dst_params->ap_dev_ops)
	goto Error;
   
    memcpy(dst_params->ap_dev_ops, src_params->ap_dev_ops, 
	sizeof(ap_dev_ops_t)); 

    return 0;

Error:

    if (src_params->psk_param == WPA_PSK_PARAM_ASCII)
	nfree(dst_params->psk.ascii);
    nfree(dst_params->ssid);
    return -1;
}

static void free_params(wpa_port_t *t)
{
    if (t->params.psk_param == WPA_PSK_PARAM_ASCII)
	nfree(t->params.psk.ascii);
    
    nfree(t->params.ssid);
    nfree(t->params.ap_dev_ops);
}

static void generate_gtk(wpa_port_t *t)
{
    wpa_gmk_t gmk;
    
    WPA_DEBUG("Generating GTK");

    /* Generate a random master key */
    random_buf_get((unsigned char *)&gmk, sizeof(gmk));
       
    /* Generate GNonce */
    wpa_port_generate_nonce(t, &t->gnonce);

    /* Generate GTK */
    wpa_key_utils_calc_gtk(&t->gtk, &gmk, &t->gnonce, t->params.mac);
}

static void gtk_update_timer_cb(void *t);

static void install_single_preconfigured_key(wpa_port_t *t, int index)
{
    WPA_DEBUG("Installing preconfigured WEP key in index %d. Key=\n%s", index, 
	wpa_log_format_buf((u8 *)t->params.wep_keys[index].data,
	wpa_key_utils_get_key_len(t->params.wep_keys[index].cipher)));
	
    t->params.ap_dev_ops->set_default_key(t->ap_dev, index, 
	t->params.wep_keys[index].cipher, 
	(wpa_temporal_key_t *)t->params.wep_keys[index].data); 
}

static void install_preconfigured_keys(wpa_port_t *t, 
    int delay_current_gtk_installation)
{
    int i;

    /* Install all preconfigured keys. In some cases we don't want the current
     * group key to be replaced right away because this will cause a situation
     * where we're using the new key as our group key and our connected stations
     * still use the old key. To prevent this we'll first send the stations
     * another group key and only once all the stations have switched to it,
     * we'll replace the old group key
     */
    for (i = 0; i < WPA_NUM_DEFAULT_KEYS; i++)
    {
	if (!delay_current_gtk_installation || i != t->current_gtk_index)
	    install_single_preconfigured_key(t, i);
    }

    if (delay_current_gtk_installation)
    {
	WPA_DEBUG("Delaying installation of key in index %d", 
	    t->current_gtk_index);
	
	/* Take note that we still have a preconfigured key to install when
	 * the next GTK update is over
	 */
	t->delayed_preconfigured_key_install = t->current_gtk_index;
    }
}

static void schedule_next_gtk_update(wpa_port_t *t, int ms_to_wait)
{
    /* We must not update the GTK if a legacy station is connected */
    if (!t->num_of_legacy_stas)
	event_timer_set(ms_to_wait, gtk_update_timer_cb, t);
}

static void install_gtk(wpa_port_t *t)
{
    WPA_DEBUG("Installing a %s GTK in index %d:\n%s", 
	wpa_log_cipher_to_str(t->multicast_cipher), t->current_gtk_index,
	wpa_log_format_buf((u8 *)&t->gtk, 
	wpa_key_utils_get_key_len(t->multicast_cipher)));

    /* Install GTK as a default key */
    t->params.ap_dev_ops->set_default_key(t->ap_dev, t->current_gtk_index, 
	t->multicast_cipher, &t->gtk);

    /* Set the default key ID */
    t->params.ap_dev_ops->set_default_key_id(t->ap_dev, t->current_gtk_index);

    /* We may have a preconfigured key whose installation was delayed. Now is
     * the time to install it.
     */
    if (t->delayed_preconfigured_key_install)
    {
	install_single_preconfigured_key(t, 
	    t->delayed_preconfigured_key_install);

	t->delayed_preconfigured_key_install = 0;
    }

    /* We can now decrypt our mutlicast/broadcast traffic so we can stop
     * blocking it
     */
    t->params.ap_dev_ops->block_multicast_traffic(t->ap_dev, 0);
    
    if (t->params.gtk_update_interval)
	schedule_next_gtk_update(t, t->params.gtk_update_interval);
}

static void update_gtk(wpa_port_t *t)
{
    wpa_assoc_t *assoc = t->assoc_list;

    WPA_DEBUG("Updating GTK");

    /* If we're still waiting for some stations to acknowledge the last GTK 
     * update, postpone the requested GTK update for later
     */
    if (t->gtk_update_in_progress)
    {
	WPA_DEBUG("Postponing GTK update because a previous GTK update is "
	    "still in progress");

	t->delayed_gtk_update = 1;
	
	return;
    }

    /* Remove GTK key update callback */
    event_timer_del(gtk_update_timer_cb, t);

    /* When there is no need to support legacy stations, the key can be
     * dynamically generated and the index can be swapped on every update.
     * When legacy stations should be supported we must always use the 
     * preconfigured key and index for our group key
     */
    if (t->params.allowed_sta_types != WPA_STA_TYPE_WEP_LEGACY && 
	!t->num_of_legacy_stas)
    {
	/* Swap group key index */
	t->current_gtk_index = t->current_gtk_index == 1 ? 2 : 1;

	/* Generate a new GTK */
	generate_gtk(t);
    }
    else
    {
	/* Use preconfigured key and index for our next group key */
	t->current_gtk_index = t->params.wep_key_index;
	
	memcpy(&t->gtk, t->params.wep_keys[t->params.wep_key_index].data, 
	    sizeof(t->params.wep_keys[t->params.wep_key_index].data));
    }
    
    /* Start group key update for all stations that support it */
    for (assoc = t->assoc_list; assoc; assoc = assoc->next)
    {
	/* Only send group key updates to station that already has the old group
	 * key
	 */
	if (!assoc->has_gtk)
	    continue;

	/* Send the new GTK to this station */
	wpa_assoc_start_gtk_neg(assoc);

	t->gtk_update_in_progress = 1;
    }

    /* If we didn't start a GTK update with any station, we don't have to wait
     * for acknowledgments, we can install the new GTK right away
     */
    if (!t->gtk_update_in_progress)
	install_gtk(t);
}

static void gtk_update_timer_cb(void *context)
{
    wpa_port_t *t = (wpa_port_t *)context;

    WPA_DEBUG("GTK expired");
    
    /* Generate a new GTK and send new GTK to stations */
    update_gtk(t);
}

static int remove_assoc(wpa_port_t *t, ether_mac_t mac)
{
    wpa_assoc_t **assoc = &t->assoc_list;
    wpa_assoc_t *removed_assoc;

    /* Look for an association with the specified MAC */
    while ((*assoc) && memcmp(mac, (*assoc)->sta_mac, sizeof(ether_mac_t)))
	assoc = &((*assoc)->next);

    /* No such association */
    if (!(*assoc))
	return 1;

    /* Remove association from list */
    removed_assoc = *assoc;
    *assoc = (*assoc)->next;

    /* Update station counter to reflect the fact that the station is leaving */
    wpa_port_update_sta_counter(t, removed_assoc->sta_type, -1);
    
    /* Close association */
    wpa_assoc_close(removed_assoc);

	return 0;
}

/* Called when it's time to stop the countermeasures and return to normal
 * operation mode
 */
void countermeasures_timer_cb(void *context)
{
    wpa_port_t *t = (wpa_port_t *)context;

    t->countermeasures_invoked = 0;
}

static void test_for_end_of_gtk_update(wpa_port_t *t)
{
    wpa_assoc_t *assoc;
  
    /* If we already know that there's no GTK update in progress, there's no
     * need to go on
     */
    if (!t->gtk_update_in_progress)
	return;

    /* Go over all stations and check if there are still stations that are in
     * the middle of a GTK update
     */
    t->gtk_update_in_progress = 0;

    for (assoc = t->assoc_list; assoc; assoc = assoc->next)
    {
	if (wpa_assoc_is_in_gtk_update(assoc))
	{
	    t->gtk_update_in_progress = 1;
	    break;
	}
    }

    /* If we found out that the GTK update is over, install the new GTK */
    if (!t->gtk_update_in_progress)
    {
	install_gtk(t);

	/* In case that a GTK update was issued while the last GTK update was in
	 * progress, the second GTK was delayed and should take place now
	 */
	if (t->delayed_gtk_update)
	{
	    t->delayed_gtk_update = 0;

	    WPA_DEBUG("Doing postponed GTK update");
	    
	    update_gtk(t);
	}
    }
}

static void handle_sta_disassoc(wpa_port_t *t, wpa_assoc_t *assoc)
{
    int sta_has_gtk;
    wpa_sta_type_t sta_type;

    /* Save some information about the station before removing it */
    sta_has_gtk = assoc->has_gtk;
    sta_type = assoc->sta_type;

    /* Remove station's assoication object from list */

		/* Demiurg : moved this function outside of handle_sta_disconnect and after wpa_Assoc_disassoc to solve
			 problem with free */
	remove_assoc(t, assoc->sta_mac);

    /* If we were waiting for this station to acknowledge receiving the GTK,
     * because of it disassociating, we're never going to get this 
     * acknowledgement. In case it was the last station we're waiting to get 
     * acknowledgment from, we can install the new GTK now.
     */
    test_for_end_of_gtk_update(t);

    /* Update group key in 2 cases:
     * 1. If the leaving station has the current GTK, so it will no longer be 
     *    able to decrypt multicast/broadcast traffic. This is unless WEP-legacy
     *    stations are connected, in which case GTK update is disabled.
     * 2. The leaving station was the last legacy station connected and 802.1X
     *    and/or WPA stations are allowed. This means that GTK update was 
     *    disabled to support WEP-legacy stations, and can now be enabled again.
     */
    if (!t->num_of_legacy_stas && (sta_has_gtk || 
	(sta_type == WPA_STA_TYPE_WEP_LEGACY && 
	(t->params.allowed_sta_types & 
	(WPA_STA_TYPE_WPA | WPA_STA_TYPE_WEP_8021X)))))
    {
	/* Don't do the GTK update immediately since we might be called from a
	 * loop disassociating a number of stations and we don't want to start a
	 * separate GTK update for each disassociating station
	 */ 
	schedule_next_gtk_update(t, 0);
    }
}

static void disassoc_sta(wpa_port_t *t, wpa_assoc_t *assoc) {
	wpa_assoc_t *assocRemove = assoc;

    handle_sta_disassoc(t, assoc);
    wpa_assoc_disassoc(assoc);
	/*remove_assoc(t, assoc->sta_mac);*/
	wpa_assoc_free(assocRemove);
}

static void deauth_sta(wpa_port_t *t, wpa_assoc_t *assoc) {
	wpa_assoc_t *assocRemove = assoc;

    handle_sta_disassoc(t, assoc);
    wpa_assoc_deauth(assoc);
	/*remove_assoc(t, assoc->sta_mac);*/
	wpa_assoc_free(assocRemove);
}

void wpa_port_close(wpa_port_t *t)
{
	wpa_assoc_t *temp;

    /* Remove WPA information element from beacons to indicate that WPA is 
     * disabled
     */
    t->params.ap_dev_ops->set_ap_wpa_info_elem(t->ap_dev, NULL, 0);
    
    /* Deauthenticate all stations and free all association objects */
    while (t->assoc_list)
    {
	wpa_assoc_deauth(t->assoc_list);
	temp = t->assoc_list;
	if(remove_assoc(t, t->assoc_list->sta_mac) == 0)
		wpa_assoc_free(temp);
    }
    
    /* Free information element */
    wpa_info_elem_close(t->ap_info_elem);
    
    /* Close AP device */
    t->params.ap_dev_ops->close(t->ap_dev);

    /* Free parameters */
    free_params(t);

    /* Remove callbacks */
    event_timer_del(gtk_update_timer_cb, t);
    event_timer_del(countermeasures_timer_cb, t);
    
    free(t);
}

/* Inspects station's IE and returns 0 (to allow station to associate) if
 * station type is supported according to AP's configuration. Returns a reason 
 * code as defined in the IEEE 802.11i otherwise
 */
static int sta_assoc_request_cb(void *context, u8 *info_elem_buf, 
    int info_elem_buf_len)
{
    wpa_cipher_t multicast_cipher, unicast_ciphers;
    wpa_auth_suite_t auth_suites;
    wpa_info_elem_t *info_elem = (wpa_info_elem_t *)info_elem_buf;
    wpa_port_t *t = (wpa_port_t *)context;

    WPA_DEBUG("Station is requesting to associate");

    /* If MIC countermeasures are invoked due to 2 MIC failure within one 
     * minute and if this is a WPA station, reject it
     */
    if (t->countermeasures_invoked && info_elem)
    {
	rg_error(LWARN, "wpa: Rejecting station since no WPA associations are "
	    "currently allowed due to a recent MIC failure");
	goto Reject;
    }

    /* We can tell by the presence or absence of an information element if
     * this is a WPA station or not. Reject station if its type is not allowed
     */
    if ((!info_elem &&
	!(t->params.allowed_sta_types & (WPA_STA_TYPE_WEP_LEGACY |
	WPA_STA_TYPE_WEP_8021X))) || (info_elem && !(t->params.allowed_sta_types
	& WPA_STA_TYPE_WPA)))
    {
	rg_error(LWARN, "wpa: Rejecting station since AP is configured not "
	    "allow %sWPA stations", info_elem ? "" : "non-");

	goto Reject;
    }

    /* If this is a WPA station, we still have to check its information element,
     * otherwise we're done here
     */
    if (!info_elem)
	goto Accept;

    /* Parse station's information element */
    if (wpa_info_elem_parse(info_elem, &multicast_cipher, &unicast_ciphers,
	&auth_suites, info_elem_buf_len))
    {
	rg_error(LWARN, "wpa: Rejecting station since it has a "
	    "malformed information element");
	goto Reject;
    }

    /* Reject station if it doesn't support our authentication suite */
    if ((PORT_IN_WPA_PSK_MODE(t) && !(auth_suites & WPA_AUTH_PSK)) || 
	(!PORT_IN_WPA_PSK_MODE(t) && !(auth_suites & WPA_AUTH_8021X)))
    {
	rg_error(LWARN, "wpa: Rejecting station since it can't use AP's "
	    "authentication suite");
	goto Reject;
    }
   
    /* Reject station if it can't use TKIP as a unicast cipher */
    if (!(unicast_ciphers & WPA_CIPHER_TKIP))
    {
	rg_error(LWARN, "wpa: Rejecting station since it can't use TKIP as "
	    "unicast cipher");
	goto Reject;
    }

    /* Reject station if it can't use our current multicast cipher */
    if (!(multicast_cipher && t->multicast_cipher))
    {
	rg_error(LWARN, "wpa: Rejecting station since it can't use AP's "
	    "multicast cipher");
	goto Reject;
    }
	    
Accept:
    WPA_DEBUG("Accepting station");
    return 0;

Reject:
    /* We currently always return 1 as a reason code for rejection */
    return 1;
}

/* Called when a station's association process is complete by the underlying
 * layer
 */
static void sta_assoc_done_cb(void *context, ether_mac_t mac, u8 *info_elem_buf,
    int info_elem_buf_len)
{
    wpa_assoc_t *assoc;
    wpa_port_t *t = (wpa_port_t *)context;
    wpa_info_elem_t *info_elem = (wpa_info_elem_t *)info_elem_buf;
	wpa_assoc_t **assocRemove = &t->assoc_list;

    WPA_DEBUG("Station %s associated", mactoa((ether_mac_t *)mac));

    /* Start blocking data traffic going to this station, the blocking will be
     * removed later, once the station authenticates or once we detect that 
     * it's a legacy station incapable of authenticating
     */
    t->params.ap_dev_ops->block_unicast_traffic(t->ap_dev, 1, mac);

    /* If association object for this station already exists, remove it before
     * we add it again
     */

    while ((*assocRemove) && memcmp(mac, (*assocRemove)->sta_mac, sizeof(ether_mac_t)))
	assocRemove = &((*assocRemove)->next);

	if(*assocRemove)
	{
		if(remove_assoc(t, mac) == 0)
			wpa_assoc_free(*assocRemove);
	}
   
    /* Create a new association object */
    assoc = wpa_assoc_open(t, mac, info_elem);
    if (!assoc)
    {
	rg_error(LWARN, "wpa: Can't allocate an association object for "
	    "station");
	    
	return;
    }

    /* Add association to list */
    assoc->next = t->assoc_list;
    t->assoc_list = assoc;

    /* In some cases we can know the station type at this stage, so we do
     * type-specific handling now, otherwise this handling will have to be done
     * later when we'll detect the station type
     */
    if (assoc->sta_type)
	wpa_port_sta_type_detected(t, assoc);
    
    /* XXX Maybe this should move to wpa_assoc_open? */
    /* If this is a WPA station and we're in PSK mode, this is the time to start
     * 4-way-handshake
     */
    if (assoc->sta_type == WPA_STA_TYPE_WPA && PORT_IN_WPA_PSK_MODE(t))
	wpa_assoc_start_ptk_neg(assoc);
}

void wpa_port_8021x_auth_end(wpa_port_t *t, wpa_assoc_t *assoc, int success)
{
    if (!success)
    {
	rg_error(LWARN, "wpa: Failed to authenticate station %s",
	    mactoa(&assoc->sta_mac));

	/* Deauthenticate station */
	deauth_sta(t, assoc);

	return;
    }

    WPA_DEBUG("Station %s authenticated", mactoa(&assoc->sta_mac));
   
    /* Send station a pairwise key */
    wpa_assoc_start_ptk_neg(assoc);
}

void wpa_port_ptk_neg_end(wpa_port_t *t, wpa_assoc_t *assoc, int success)
{
    if (!success)
    {
	if (assoc->sta_type == WPA_STA_TYPE_WPA)
	{
	    WPA_DEBUG("Failed doing 4-way handshake with station %s", 
		mactoa(&assoc->sta_mac));
	}
	else
	{
	    WPA_DEBUG("Failed sending pairwise key to station %s",
		mactoa(&assoc->sta_mac));
	}
		
	/* Deauthenticate station */
	deauth_sta(t, assoc);

	return;
    }

    if (assoc->sta_type == WPA_STA_TYPE_WPA)
    {
	WPA_DEBUG("4-way handshake with station %s completed successfully", 
	    mactoa(&assoc->sta_mac));
    }
    else
    {
	WPA_DEBUG("Pairwise key sent to station %s successfully", 
	    mactoa(&assoc->sta_mac));
    }
    
    /* Start group key update for the station */
    wpa_assoc_start_gtk_neg(assoc);
}

void wpa_port_gtk_neg_end(wpa_port_t *t, wpa_assoc_t *assoc, int success)
{
    if (!success)
    {
	/* Deauthenticate station */
	deauth_sta(t, assoc);
	
	return;
    }
    
    /* This station have just acknowledged receiving the GTK. Check if it's the
     * last station we were waiting to acknowledge. In which case, we can
     * install the new GTK now.
     */
    test_for_end_of_gtk_update(t);
}

/* Called when the underlying layer detects that a station disassociated */
static void sta_disassoc_cb(void *context, ether_mac_t mac)
{
    wpa_port_t *t = (wpa_port_t *)context;
    wpa_assoc_t *assoc, *assocRemove;

    WPA_DEBUG("Station %s disassociated", mactoa((ether_mac_t *)mac));
    
    assoc = find_assoc(t, mac);
    if (!assoc)
	return;
    
	assocRemove = assoc;
    handle_sta_disassoc(t, assoc);
	/*remove_assoc(t, assoc->sta_mac);*/
	wpa_assoc_free(assocRemove);
}

/* Called when a MIC failure is detected in a packet protected with a unicast
 * key
 */ 
static void unicast_mic_failure_cb(void *context, ether_mac_t mac)
{
    wpa_port_t *t = (wpa_port_t *)context;
    wpa_assoc_t *assoc = find_assoc(t, mac);

    if (!assoc)
    {
	rg_error(LWARN, "wpa: MIC failure detected in a packet received from "
	    "an unknown station %s", mactoa((ether_mac_t *)&mac));
	return;
    }

    rg_error(LWARN, "wpa: MIC failure detected in a packet received from "
	"station %s", mactoa(&assoc->sta_mac));

    wpa_port_mic_failure(t, assoc);
}

/* Called when a first data packet is received from a station, judging by the
 * type of this packet we can tell whether a WEP station supports 802.1X
 */
static void first_data_packet_received_cb(void *context, int is_eapol, 
    ether_mac_t mac)
{
    wpa_port_t *t = (wpa_port_t *)context;
    wpa_assoc_t *assoc = find_assoc(t, mac);

    WPA_DEBUG("First packet received from station %s is %sEAPOL",
	mactoa((ether_mac_t *)mac), is_eapol ? "" : "non-");
    
    if (!assoc)
	return;

    /* Station type is unknown, try to figure it out according to the type of
     * the first packet
     */
    if (!assoc->sta_type)
    {
	/* 802.1X stations send EAPOL packets as their first packets while 
	 * legacy stations don't send EAPOL packets at all
	 */
	wpa_assoc_set_sta_type(assoc, is_eapol ? WPA_STA_TYPE_WEP_8021X :
	    WPA_STA_TYPE_WEP_LEGACY);
    }
    /* Handle a special case where we assumed that the station was a WEP-legacy
     * station (to avoid to 2 sec delay) but it sends an EAPOL-Start message so
     * it turns out that it's actually a 802.1X station
     */
#if 0
	/* Ths following code had been removed in order to support legacy station that sends EAPOL-Start 
			as their first data packet */
    else if ((assoc->sta_type == WPA_STA_TYPE_WEP_LEGACY) && is_eapol)
	wpa_assoc_set_sta_type(assoc, WPA_STA_TYPE_WEP_8021X);
    /* We already know the type of this station so we already did the 
     * type-specific handling for it, no need to go on
     */
#endif
    else
	{
		if (assoc->sta_type == WPA_STA_TYPE_WPA)
			WPA_DEBUG("wpa: First packet received station type is WPA\n");
		else if (assoc->sta_type == WPA_STA_TYPE_WEP_LEGACY)
			WPA_DEBUG("wpa: First packet received station type is WEP-Legacy\n");
		else if (assoc->sta_type == WPA_STA_TYPE_WEP_8021X)
			WPA_DEBUG("wpa: First packet received station type is WEP-802.1X\n");
		else
			WPA_DEBUG("wpa: First packet received station type is Unknown\n");
		return;
	}

    /* Now that we know the station type, do type-specific handling for the 
     * station
     */
    wpa_port_sta_type_detected(t, assoc); 
}

/* Callbacks to be called by the underlying layer */
static ap_dev_cbs_t ap_dev_cbs = {
    sta_assoc_request_cb,
    sta_assoc_done_cb,
    sta_disassoc_cb,
    unicast_mic_failure_cb,
    first_data_packet_received_cb,
};

/* Uses the key counter to generate a new Nonce */
void wpa_port_generate_nonce(wpa_port_t *t, wpa_nonce_t *nonce)
{
    memcpy(nonce, &t->key_counter, sizeof(wpa_nonce_t));
    wpa_key_utils_inc((u8 *)&t->key_counter, sizeof(t->key_counter));
}

wpa_port_t *wpa_port_open(wpa_port_params_t *params)
{
    wpa_port_t *t;
    int info_elem_len = 0;

    /* Create new wpa_port_t object */
	t = (wpa_port_t *)zalloc_l(sizeof(wpa_port_t));
    if (!t)
	return NULL;

    /* Save parameters */
    if (dup_params(&t->params, params))
	goto Error;

    /* Initialize key counter */
    wpa_key_utils_init_key_counter(&t->key_counter, t->params.mac);
    
    switch (params->psk_param)
    {
    /* If PSK is provided as ASCII, convert it to hex */
    case WPA_PSK_PARAM_ASCII:
	wpa_key_utils_calc_psk(&t->psk, t->params.psk.ascii, t->params.ssid, 
	    t->params.ssid_len);
	break;

    /* If PSK is provided as hex, simply copy it */	
    case WPA_PSK_PARAM_HEX:
	memcpy(&t->psk, &t->params.psk.hex, sizeof(wpa_psk_t));
	break;

    default:
	break;
    }


    /* Open AP device */
    t->ap_dev = t->params.ap_dev_ops->open(t->params.dev_id, &ap_dev_cbs, 
	(void *)t);
    if (!t->ap_dev)
	goto Error;


    t->current_gtk_index = 1;
    
    /* If only WEP-legacy stations are allowed, we can install preconfigured 
     * keys right now, if other types of stations are allowed we'll install the
     * preconfigured keys only when the first legacy station joins
     */
    if (t->params.allowed_sta_types == WPA_STA_TYPE_WEP_LEGACY)
	install_preconfigured_keys(t, 0);


    t->multicast_cipher = select_multicast_cipher(t);

    WPA_DEBUG("Going to use %s as multicast cipher", 
	wpa_log_cipher_to_str(t->multicast_cipher));

    /* If WPA stations are supported, we're going to need an information element
     */
    if (t->params.allowed_sta_types & WPA_STA_TYPE_WPA)
    {
	/* Build information element */
	t->ap_info_elem = wpa_info_elem_open(t->multicast_cipher, 
	    WPA_CIPHER_TKIP, PORT_IN_WPA_PSK_MODE(t) ? WPA_AUTH_PSK : 
	    WPA_AUTH_8021X);
	if (!t->ap_info_elem)
	    goto Error;

	info_elem_len = wpa_info_elem_get_len(t->ap_info_elem);
    }

    /* Tell the AP to use our information element. If WPA stations are not
     * allowed we install a NULL information element so that the underlying
     * layer will know not to include an information element in its beacon
     */
    t->params.ap_dev_ops->set_ap_wpa_info_elem(t->ap_dev, (u8 *)t->ap_info_elem,
	info_elem_len);

    /* Initialize a group key and install it */
    update_gtk(t);

    return t;

Error:
    if (t->ap_info_elem)
	wpa_info_elem_close(t->ap_info_elem);

    if (t->ap_dev)
	t->params.ap_dev_ops->close(t->ap_dev);

    free_params(t);
	    
    free(t);
    return NULL;
}

/* Called when a station requests a GTK update */
void wpa_port_gtk_update_request(wpa_port_t *t, wpa_assoc_t *assoc)
{
    update_gtk(t);
}

/* Called when a MIC failure occurs */
void wpa_port_mic_failure(wpa_port_t *t, wpa_assoc_t *assoc)
{
    struct timeval current_time;
    wpa_assoc_t *next_assoc;
    
    sys_get_time(&current_time);

    /* If this is the second MIC failure within the last 60 seconds, disconnect 
     * all stations and invoke countermeasures
     */
    if (tv2ms(&t->last_mic_failure_time) && ((tv2ms(&current_time) - 
	tv2ms(&t->last_mic_failure_time)) < MIC_FAILURE_DELAY_MS))
    {
	rg_error(LWARN, "wpa: Second MIC failure within 60 seconds. "
	    "Disassociating all WPA stations and invoking countermeasures");	
	
	/* Disassociate all WPA stations */
	for (assoc = t->assoc_list; assoc; assoc = next_assoc)
	{
	    /* It's important to save assoc->next now, because when disassociate
	     * the station, assoc is freed 
	     */	   
	    next_assoc = assoc->next;

	    if (assoc->sta_type == WPA_STA_TYPE_WPA)
		disassoc_sta(t, assoc);
	}

	/* Invoke countermeasures */
	t->countermeasures_invoked = 1;

	/* The countermeasures are only invoked for a limited period of time, so
	 * set a timer that will stop the countermeasures when it's due
	 */
	event_timer_set(MIC_COUNTERMEASURES_PERIOD_MS, countermeasures_timer_cb,
	    t);
    }

    t->last_mic_failure_time = current_time;
}

/* Called whenever we manage to detect whether a specific station is 802.1X or
 * WEP-legacy
 */
void wpa_port_sta_type_detected(wpa_port_t *t, wpa_assoc_t *assoc)
{
    /* Now that we know the station type, we might have to disassociate it if 
     * it's not allowed according to the configuration
     */
    if (!(assoc->sta_type & t->params.allowed_sta_types))
    {
	WPA_DEBUG("Deauthenticating station %s because %s stations are not "
	    "allowed", mactoa(&assoc->sta_mac), 
	    wpa_log_sta_type_to_str(assoc->sta_type));

	goto Reject;
    } 

    /* Deauthenticate station if it's of a type that uses mapping keys and no
     * more mapping keys can be installed
     */
    if ((assoc->sta_type != WPA_STA_TYPE_WEP_LEGACY) && (t->num_of_wpa_stas + 
	t->num_of_8021x_stas > 
	t->params.ap_dev_ops->get_max_num_of_mapping_keys(t->ap_dev)))
    {
	WPA_DEBUG("Deauthenticating station %s because all available mapping" 
	    "keys are in use", mactoa(&assoc->sta_mac));

	goto Reject;
    }
    
    /* Let the association object do type-specific handling */
    wpa_assoc_sta_type_detected(assoc);
	
    if (assoc->sta_type != WPA_STA_TYPE_WEP_LEGACY)
	return;

    /* If there were no legacy stations associated before this one joined, 
     * then we're probably using a dynamically generated GTK, we have to 
     * start using the preconfigured key as our GTK because the legacy 
     * station doesn't support key distribution. 
     * If the only allowed station type is WEP-legacy there is no need to do
     * anything, we've already installed the preconfigured keys when the 
     * port was opened.
     */
    if (t->params.allowed_sta_types == WPA_STA_TYPE_WEP_LEGACY ||
	t->num_of_legacy_stas > 1)
    {
	return;
    }

    install_preconfigured_keys(t, 1);
    
    WPA_DEBUG("Distributing preconfigured key");

    /* Start using the key preconfigured to be the TX key as our GTK and send it
     * to the connected stations
     */
    update_gtk(t);

    return;

Reject:
    deauth_sta(t, assoc);
}

int wpa_port_reconf(wpa_port_t *t, wpa_port_params_t *params)
{
    int gtk_update_needed = 0;

    /* Changing most of the parameters requires reopnening the port, check if
     * any of those parameters has changed compared to our current configuration
     */
    if ((t->params.psk_param != params->psk_param) ||
	((t->params.psk_param == WPA_PSK_PARAM_ASCII) && 
	strcmp(t->params.psk.ascii, params->psk.ascii)) ||
	((t->params.psk_param == WPA_PSK_PARAM_HEX) && 
	memcmp(&t->params.psk.hex, &params->psk.hex, 
	sizeof(t->params.psk.hex))))
    {
	goto ReopenNeeded;
    }

    if (t->params.rekeying_wep_cipher != params->rekeying_wep_cipher)
	goto ReopenNeeded;

    if (t->params.allowed_sta_types != params->allowed_sta_types)
	goto ReopenNeeded;

    if ((t->params.ssid_len != params->ssid_len) ||
	memcmp(t->params.ssid, params->ssid, t->params.ssid_len))
    {
	goto ReopenNeeded;
    }

    if (t->params.dev_id != params->dev_id)
	goto ReopenNeeded;

    if (memcmp(t->params.ap_dev_ops, params->ap_dev_ops, 
	sizeof(*t->params.ap_dev_ops)))
    {
	goto ReopenNeeded;
    }

    if (memcmp(t->params.mac, params->mac, sizeof(t->params.mac)))
	goto ReopenNeeded;

    /* Handle change in configuration of group key update interval */
    if (params->gtk_update_interval != t->params.gtk_update_interval)
    {
	WPA_DEBUG("Reconfiguring group key update interval");

	/* Save parameter */
	t->params.gtk_update_interval = params->gtk_update_interval;

	/* Cancel next scheduled GTK update */
	event_timer_del(gtk_update_timer_cb, t);

	/* Schedule a new GTK update according to the new configuration */
	if (t->params.gtk_update_interval)
	    schedule_next_gtk_update(t, t->params.gtk_update_interval);
    }

    /* Handle change in WEP keys */
    if (memcmp(params->wep_keys, t->params.wep_keys, sizeof(params->wep_keys)))
    {
	WPA_DEBUG("Reconfiguring WEP keys");

	/* Save parameter */
	memcpy(t->params.wep_keys, params->wep_keys, 
	    sizeof(t->params.wep_keys));

	/* This change has an immediate effect only if there are legacy stations
	 * connected or if we're in legacy-only mode, otherwise the
	 * preconfigured keys are currently not in use anyway
	 */
	if (t->num_of_legacy_stas || t->params.allowed_sta_types == 
	    WPA_STA_TYPE_WEP_LEGACY)
	{
	    install_preconfigured_keys(t, 1);

	    gtk_update_needed = 1;
	}
    }

    /* Handle change in configuration of WEP key index */
    if (params->wep_key_index != t->params.wep_key_index)
    {
	WPA_DEBUG("Reconfiguring WEP key index");

	/* Save parameter */
	t->params.wep_key_index = params->wep_key_index;

	/* This change has an immediate effect only if we currently have legacy
	 * stations connected or if we're in legacy-only mode. These are the
	 * only cases in which we use the WEP key index.
	 * When legacy stations are connected, the group key index is the
	 * preconfigured key index, before we start using the new key index
	 * we have to let all the stations know about this change
	 */
	gtk_update_needed = t->num_of_legacy_stas ||
	    t->params.allowed_sta_types == WPA_STA_TYPE_WEP_LEGACY;
    }

    /* If one of the changes may have affected our group key, we need to notify
     * WPA and WEP-802.1X stations about the new group key
     */
    if (gtk_update_needed)
    {
	/* One of the configuration changes might have changed the multicast
	 * cipher we're supposed to use. Since we've already advertised our
	 * existing multicast cipher in our information element and we have no
	 * way of telling the connected station of such a change, this
	 * reconfiguration requires reopening the port
	 */
	if ((t->params.allowed_sta_types & WPA_STA_TYPE_WPA) &&
	    select_multicast_cipher(t) != t->multicast_cipher)
	{
	    goto ReopenNeeded;
	}

	update_gtk(t);
    }

    return 0;

ReopenNeeded:
    WPA_DEBUG("Reconfiguration requires reopening port");
    return 1;
}

