/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************/
/*																			*/
/*		MODULE:		drvDbHash.c												*/
/*		PURPOSE:	Hash Implementation And station database. This file 	*/
/*					holds all the information regarding the stations		*/
/****************************************************************************/
#include "pform_types.h"
#include "pal.h"
#include "drvDbHash.h"


/*-----------------------------------------------------------*/
/*         Local Variables                                   */
/*-----------------------------------------------------------*/
static stationParams_T stationParams[MAX_STATIONS];

/* The staTable holds the station. Each entry describes a station. The table is not sorted */
static staHashEntry_T	staTable[MAX_STATIONS];

/* The hash table is used to find fast a station index in the database. The station index is known by all 
	the modules of the software and its easier to use it than the station mac address 
	The hash table has less entries than the station table, but every entry is a linked list.
	Station that are equal in the last 6 bits of their mac addresses are mapped to the same hash entry. */
static staHashEntry_T	*hashTable[HASH_ENTRIES_NUM];


static UINT16	numStations;

static UINT16	numStationsNotPBCCCapabale;		/* implemented only if working in PBCC mode and the PBCC algorithm is static.*/
static BOOL		currentPBCCSupportMode;

static UINT16	numStationsNotShortPreambleCapabale;
static BOOL		currentShortPreambleSupport;

static UINT16	numStationsNotSlotTimeCapabale;
static BOOL		currentShortSlotTimeSupport;

static UINT16	numOfNonERPStations=0;
static UINT16   numOfERPstations=0;
static BOOL		currentERPProtectionMode =FALSE;
static BOOL     currentERPNonPresent=FALSE;

static PAL_spinLockHandle drvDbHashLock;

/*-----------------------------------------------------------*/
/*         Local Functions                                   */
/*-----------------------------------------------------------*/
static STATUS	resetParams(void); 
static UINT16 convertDOT11RatesToAcxBitWiseFormat(UINT8 *rates, UINT8 ratesLen);
static void setStationAPRates(UINT16 index, dot11_supRates_T *rates, dot11_extSupRates_T *extRates,
                              UINT8 *apAllRates, UINT8 apAllRatesLen);

/*************************************************************************
 *                        wdrv_dbHashInit                                    *
 *************************************************************************
DESCRIPTION: This function initializes the hash module.                 
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      OK : Initialization succesfull.
             NOK: Initialization unsuccesfull.                                                                                                                                             
************************************************************************/
STATUS wdrv_dbHashInit(void)
{
	int rc;

	rc = resetParams();

	if (rc != OK)
		return NOK;

	PAL_spinLock_init(&drvDbHashLock);

	return OK;
}

/*************************************************************************
 *                        resetParams                                    *
 *************************************************************************
DESCRIPTION: This function reset the hash module.                 
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      OK : reset succesfull.
             NOK: reset unsuccesfull.                                                                                                                                             
************************************************************************/
static STATUS resetParams(void)
{
	int i;
	UINT8 PbccAlgorithmMode;
	UINT8 ERPProtectionMode;
	UINT8 PreambleMode;
	UINT8 SlotTimeMode;

	/* reset station table */
	for (i = 0; i < MAX_STATIONS; i++)
	{
		PAL_memSet(staTable[i].macAddr, 0, MAC_ADDR_LEN);
		PAL_memSet(staTable[i].ssid, 0, WHAL_SSID_STRING_LEN);
		staTable[i].nextSta = NULL;
		staTable[i].prevSta = NULL;
		staTable[i].free = TRUE;
		staTable[i].index = i;

		/* reset the station parameters table */
		PAL_memSet(&stationParams[i], 0, sizeof(stationParams_T));
	}

	/* reset hash table */
	for (i = 0; i < HASH_ENTRIES_NUM; i++)
		hashTable[i] = NULL;

	numStations = 0;
	
	/* Init PBCC */
	numStationsNotPBCCCapabale = 0;
	wdrv_dbGetPBCCAlgorithmMode(0,&PbccAlgorithmMode);
	if (PbccAlgorithmMode != PBCC_ALGO_DISABLED)
	{
		currentPBCCSupportMode = TRUE;
	}
	else
	{
		currentPBCCSupportMode = FALSE;
	}

	/* Init Short Preamble */
	numStationsNotShortPreambleCapabale = 0;
	wdrv_dbGetPreambleMode(0,&PreambleMode);
	if (PreambleMode == PREAMBLE_MODE_LONG)
	{
		currentShortPreambleSupport = FALSE;
	}
	else
	{
		currentShortPreambleSupport = TRUE;
	}

	/* Init Short Slot Time */
	numStationsNotSlotTimeCapabale = 0;
	wdrv_dbGetSlotTimeMode(0,&SlotTimeMode);
	if (SlotTimeMode == SLOTTIME_MODE_DISABLED)
	{
		currentShortSlotTimeSupport = FALSE;
	}
	else
	{
		currentShortSlotTimeSupport = TRUE;
	}

	/* Init ERP Protection */
	numOfNonERPStations = 0;
	numOfERPstations = 0;
	wdrv_dbGetERPProtectionMode(0,&ERPProtectionMode);
	if (ERPProtectionMode == PROTECTION_MODE_ENABLED)
	{
		currentERPProtectionMode = TRUE;
	}
	else
	{
		currentERPProtectionMode = FALSE;
	}

	return OK;
}

/*************************************************************************
 *                        wdrv_dbGetStationsNumForERP                    *
 *************************************************************************
DESCRIPTION: This function returns number of nonERP and ERP stations
			 associated with AP.				  
                                                                                                   
INPUT:       None

OUTPUT:      nonERPStaNum, ERPStaNum

RETURN:      None                                                                                                                                             
************************************************************************/
void wdrv_dbGetStationsNumForERP(UINT16 *nonERPStaNum, UINT16 *ERPStaNum)
{
	*nonERPStaNum = numOfNonERPStations;
	*ERPStaNum    = numOfERPstations;
}

/*************************************************************************
 *                        wdrv_dbFindStaIndex                                    *
 *************************************************************************
DESCRIPTION: This function finds a station index in the hash table.                 
                                                                                                   
INPUT:       The station mac address

OUTPUT:      The station index

RETURN:      OK : station found.
             NOK: station not found.                                                                                                                                             
************************************************************************/
STATUS wdrv_dbFindStaIndex(char *mac, UINT16 *idx)
{
	staHashEntry_T	*hashPtr;
	UINT8	hashValue = HASH_FUNC(mac);
	unsigned long flags;


	hashPtr = hashTable[hashValue];	/* get the station hash value base on the mac */
	PAL_intrLock(flags); /* START OF CRITICAL SECTION */	

	while (hashPtr != NULL)
	{
		if( MACCMP( hashPtr->macAddr, mac))		/* compare the mac addresses */
		{
			*idx = hashPtr->index;
		   	PAL_intrUnLock(flags); /* END OF CRITICAL SECTION */
			if(REPORT_LEVEL(RL_DB))
			{
				wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Station found. Index = %d", *idx);
				PRINT_MAC(RL_DB, PRI_INFORMATION, mac);
			}
			return OK;
		}
		hashPtr = hashPtr->nextSta;
	}	
	
	PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */

	if(REPORT_LEVEL(RL_DB))
	{
		wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Station not found.");
		PRINT_MAC(RL_DB, PRI_INFORMATION, mac);
	}
	return NOK;
}


/*************************************************************************
 *                        wdrv_dbIsTableIndexExists                      *
 *************************************************************************
DESCRIPTION: This function checks whether a station index is found in
             the hash table.                 
                                                                                                   
INPUT:       A station index

OUTPUT:      None.

RETURN:      TRUE : station free.
             FALSE: station not free (not found).
************************************************************************/
BOOL wdrv_dbIsTableIndexExists(UINT16 index)
{
    if (index >= MAX_STATIONS)
        return FALSE;
    return (!(staTable[index].free));
}


/*************************************************************************
 *                        wdrv_dbFindAndInsertStaIndex                   *
 *************************************************************************
DESCRIPTION: This function finds a station index in the hash table. If the station not found, 
				the function inserts the station into the hase table & station table.                  
                                                                                                   
INPUT:       The station mac address

OUTPUT:      The station index

RETURN:      OK : station found or station inserted successfully.
             NOK: station not found and insertion failed.                                                                                                                                             
************************************************************************/
STATUS wdrv_dbFindAndInsertStaIndex(char *mac, UINT8 woneIndex, UINT16 *idx)
{
	int i;
	staHashEntry_T	*hashPtr;
	UINT8	hashValue = HASH_FUNC(mac); 
	unsigned long flags;

	PAL_intrLock(flags); /* START OF CRITICAL SECTION */
	/* first search for the station */
	hashPtr = hashTable[hashValue];

	while (hashPtr != NULL)
	{
		if( MACCMP( hashPtr->macAddr, mac))
		{
			*idx = hashPtr->index;
		   	PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */

			if(REPORT_LEVEL(RL_DB))
			{
				wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Station found. Index = %d", *idx);
				PRINT_MAC(RL_DB, PRI_INFORMATION, mac);
			}
            stationParams[*idx].WoneIdentifierIndex = woneIndex;
			return OK;
		}
		hashPtr = hashPtr->nextSta;
	}

	/* station not found, now try to insert the station */
	if (numStations == MAX_STATIONS)	/* no place for the station, table is full */
	{
	   	PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
		if(REPORT_LEVEL(RL_DB))
		{
			wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Insertion failed1, table is full,");
			PRINT_MAC(RL_DB, PRI_INFORMATION, mac);
		}

		return NOK;
	}

	numStations++;
	
	/* first find a free index */
	for (i = 0; i < MAX_STATIONS; i++)
	{
		if (staTable[i].free == TRUE)
			break;
	}

	if(i == MAX_STATIONS)
	{
		wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Insertion failed2, table is full, numStations = %d",numStations);
		PRINT_MAC(RL_DB, PRI_INFORMATION, mac);
		return NOK;
	}

	hashPtr = &staTable[i];
	
	/* fill the entry with the station mac */
	hashPtr->free = FALSE;
	PAL_memCopy(hashPtr->macAddr, mac, MAC_ADDR_LEN);

	/* index to be returned to the caller */
	*idx = hashPtr->index;
    /*          set wone index */
    stationParams[*idx].WoneIdentifierIndex = woneIndex;
	/* now concatenate the station entry in the linked list */
	if (hashTable[hashValue] == NULL)	/* if linked list is empty*/
	{
		hashTable[hashValue] = hashPtr;

	   	PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
		if(REPORT_LEVEL(RL_DB))
		{
			wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Insertion station. Index = %d", *idx);
			PRINT_MAC(RL_DB, PRI_INFORMATION, mac);
		}

		return OK;
	}

	hashPtr->nextSta = hashTable[hashValue];	/* if linked list is not empty, insert as first entry */
	(hashPtr->nextSta)->prevSta = hashPtr;
	hashTable[hashValue] = hashPtr;
	
   	PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
	if(REPORT_LEVEL(RL_DB))
	{
		wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Insertion succeeded. Index = %d", *idx);
		PRINT_MAC(RL_DB, PRI_INFORMATION, mac);
	}

	return OK;
}

/*************************************************************************
 *                        wdrv_dbRemoveStaIndex				             *
 *************************************************************************
DESCRIPTION: This function removes a station from the hash table & station table.       
				It also resets the station parameters.
                                                                                                   
INPUT:       The station index

OUTPUT:      

RETURN:                                                                                                                                                   
************************************************************************/
void wdrv_dbRemoveStaIndex(UINT16 idx)
{
	UINT8	hashValue; 
	staHashEntry_T	*hashPtr;
	unsigned long flags;
	
	if (idx >= MAX_STATIONS)
	{
		if(REPORT_LEVEL(RL_DB))
			wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Remove station failed, index is out of range.\n");
		return;
	}

	hashPtr = &staTable[idx];
	PAL_intrLock(flags); /* START OF CRITICAL SECTION */

	if (hashPtr->free == TRUE)		/* entry is not in use, station does not exist in the tables */
	{	
	   	PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
		if(REPORT_LEVEL(RL_DB))
			wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Remove station failed, index is free already.\n");
		return;
	}

	numStations--;

	/* now try to remove the station entry from the linked list */
	if (hashPtr->prevSta == NULL)		/* first entry in the linked list */
	{
		hashValue = HASH_FUNC(hashPtr->macAddr);

		if (hashPtr->nextSta == NULL)	/* entry is also the last one */	
			hashTable[hashValue] = NULL;

		else							/* entry is not the last one */	
		{
			(hashPtr->nextSta)->prevSta = NULL;
			hashTable[hashValue] = hashPtr->nextSta;
			hashPtr->nextSta = NULL;
		}
	}
	
	else								/* not the first entry in the linked list */
	{
		(hashPtr->prevSta)->nextSta = hashPtr->nextSta;
		
		if (hashPtr->nextSta != NULL)
		{
			(hashPtr->nextSta)->prevSta = hashPtr->prevSta;
			hashPtr->nextSta = NULL;
		}
		hashPtr->prevSta = NULL;
	}

	/* now update rest of the entry structure */
	PAL_memSet(hashPtr->macAddr, 0, MAC_ADDR_LEN);
	PAL_memSet(hashPtr->ssid, 0, WHAL_SSID_STRING_LEN);
        hashPtr->ssidLength = 0;
	hashPtr->free = TRUE;
   	PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */

	if(REPORT_LEVEL(RL_DB))
		wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Remove station secceeded.\n");

	return;
}

int wdrv_dbGetNumStationsByState(void)
{
		int    i,n;
		UINT32 state;
		
		for (i=n=0; i<MAX_STATIONS; i++)
		{
				wdrv_mgmtGetState(i, &state);
				if (state != STATE_IDLE)
					n++;
		}

		return n;
}

/* called by the console */
int wdrv_dbPrintNumOfStations(char *buf)
{
	return sprintf(buf,"Num of stations:%d\n", wdrv_dbGetNumStationsByState() /*numStations*/);
}

/* print the stations table, called by the console */
int  wdrv_dbPrintStaTable(char *buf)
{
	UINT32 i,   state , len;
	char stateString[MAX_PRINT_STRING];
	BOOL TIcap;
    UINT8 band = 0;
    UINT8 channel = 0;
    UINT8 wone_index =0;  
	UINT16 acxActiveRate;

	len = 0;
	len += sprintf(buf + len ,"Stations table\n");
	len += sprintf(buf + len ,"Number of Stations:%d\n", wdrv_dbGetNumStationsByState() /*numStations*/);

	len += sprintf(buf + len ,"Index		Mac		State	  ActiveRate	ActiveRateAcx	PBCC  Privacy  TIoui  Cannel Band \n");
		
	PAL_spinLock(&drvDbHashLock);

	for (i = 0; i < MAX_STATIONS; i++)
	{

		wdrv_mgmtGetState(i, &state);
		if (state != STATE_IDLE)
		{
			len += sprintf(buf + len ," [%02d]   ", i);
			
			len += sprintf(buf + len ,"%02X-%02X-%02X-%02X-%02X-%02X  ",
							(UINT8)staTable[i].macAddr[0], (UINT8)staTable[i].macAddr[1],
							(UINT8)staTable[i].macAddr[2], (UINT8)staTable[i].macAddr[3],
							(UINT8)staTable[i].macAddr[4], (UINT8)staTable[i].macAddr[5]);

			memset(stateString, 0, MAX_PRINT_STRING);
			wdrv_mgmtGetStateString(i, stateString);
			len += sprintf(buf + len ,"%s ", stateString);

			acxActiveRate = tx_getStaActiveRate(i);			
			len += sprintf(buf + len ,"	0x%X	", convertAcxRateToDOT11Rate(acxActiveRate));			
			len += sprintf(buf + len ,"	0x%X	", acxActiveRate);

			if ((stationParams[i].capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK)
				len += sprintf(buf + len ,"%d	", 1);
			else
				len += sprintf(buf + len ,"%d	", 0);
			PAL_memSet(stateString, 0, MAX_PRINT_STRING);

			if ((stationParams[i].capabilities >> CAP_PRIVACY_SHIFT) & CAP_PRIVACY_MASK)
				len += sprintf(buf + len ,"%d	", 1);
			else
				len += sprintf(buf + len ,"%d	", 0);

			wdrv_dbGetStationTIcapEnable(i, &TIcap);
			len += sprintf(buf + len ,"%d	", TIcap);
			
            wdrv_dbGetStationWoneIndex(i, &wone_index);
	        if (wone_index ==  0xff)
            {
                 len += sprintf(buf + len ,"ERROR   ERROR\n\n");
            }
            else
            {
                wdrv_dbGetChannel(wone_index,&channel);
                len += sprintf(buf + len ,"%d   ", channel);

                wdrv_dbGetBand(wone_index,&band);

                if (band == RADIO_BAND_5GHZ)
                    len += sprintf(buf + len ,"A\n\n");
                else
                    len += sprintf(buf + len ,"G\n\n");  
            }    
		}
	}

	PAL_spinUnLock(&drvDbHashLock); 

	return len;
}

void wdrv_dbPrintStationCounters(void)
{

    int i,state;
    UINT32 rxCounter =0 ;

    printf("Index		Mac		Rx counter \n");                   
    
    for (i = 0; i < MAX_STATIONS; i++)
    {
        wdrv_mgmtGetState(i, &state);
        if (state != STATE_IDLE)
        {
            printf(" [%02d]   ", i);

            printf("%02X-%02X-%02X-%02X-%02X-%02X  ", staTable[i].macAddr[0], 
                   staTable[i].macAddr[1], 
                   staTable[i].macAddr[2], 
                   staTable[i].macAddr[3], 
                   staTable[i].macAddr[4], 
                   staTable[i].macAddr[5]);

            wdrv_dbGetStationRxPacketCounter(i,&rxCounter);
            printf("%d\n",rxCounter);
        }
    }
}


/* print the stations rates table */
int  wdrv_dbPrintRatesTable(char *buf)
{
	int i, j, state, empty=TRUE;
	UINT16 rateWithFallback;
	int len = 0;
	
	len += sprintf(buf + len ,"\nStations Rates table\n");

	for (i = 0; i < MAX_STATIONS; i++)
	{
		wdrv_mgmtGetState(i, &state);
		if ((state == STATE_ASSOCIATED) || (state == STATE_AUTHORIZED))
		{
			empty = FALSE;

			len += sprintf(buf + len ,"Index		Mac		PBCC	ActiveRate    currentRateAcxWithFallback\n");

			len += sprintf(buf + len ," [%02d]   ", i);
			
			len += sprintf(buf + len ,"%02X-%02X-%02X-%02X-%02X-%02X  ", (UINT8)staTable[i].macAddr[0], (UINT8)staTable[i].macAddr[1], (UINT8)staTable[i].macAddr[2], (UINT8)staTable[i].macAddr[3], (UINT8)staTable[i].macAddr[4], (UINT8)staTable[i].macAddr[5]);

			if ((stationParams[i].capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK)
				len += sprintf(buf + len ,"	%d	  ", 1);
			else
				len += sprintf(buf + len ,"	%d	  ", 0);

			len += wdrv_printRateInMbpsUnit(stationParams[i].activeRate,buf);

			wdrv_dbGetCurrentActiveRateWithFallback(i, &rateWithFallback);
			len += sprintf(buf + len ,"                    0x%x", rateWithFallback);
			len += sprintf(buf + len ,"\n"); 

			len += sprintf(buf + len ,"Rates =	"); 
			for (j = 0; j < stationParams[i].ratesLen; j++)
			{
				len += sprintf(buf + len ,"  "); 
				len += wdrv_printRateInMbpsUnit(stationParams[i].rates[j],buf);
			}
			for (j = 0; j < stationParams[i].extRatesLen; j++)
			{
				len += sprintf(buf + len ,"  "); 
				len += wdrv_printRateInMbpsUnit(stationParams[i].extRates[j],buf);
			}
			len += sprintf(buf + len ,"\n"); 

			/* DEBUG */
			len += sprintf(buf + len ,"APRates =	"); 
			for (j = 0; j < stationParams[i].apRatesLen; j++)
			{
				len += sprintf(buf + len ,"  "); 
				len +=wdrv_printRateInMbpsUnit(stationParams[i].apRates[j],buf+len);
			}
			len += sprintf(buf + len ,"\n"); 
			len += sprintf(buf + len ,"APExtRates =	"); 
			for (j = 0; j < stationParams[i].apExtRatesLen; j++)
			{
				len += sprintf(buf + len ,"  "); 
				len +=wdrv_printRateInMbpsUnit(stationParams[i].apExtRates[j],buf+len);
			}
			len += sprintf(buf + len ,"\n"); 
			/* DEBUG */
		}
	}

	if (empty)
	{
		len += sprintf(buf + len ,"The table is empty (no station is associated)\n");
	}
	return len;
}

/* print the hash table, called by the console */
int  wdrv_dbPrintHashTable(char *buf)
{
	int i;
	int len = 0;
	
	len += sprintf(buf + len ,"Hash table\n");

	for (i = 0; i < HASH_ENTRIES_NUM; i++)
	{
		
		len += sprintf(buf + len ,"[%d]", i);
		if (hashTable[i] != NULL)
			len += sprintf(buf + len ,"Entry Address:0x%x\n\n", (unsigned int)hashTable[i]);
		else
			len += sprintf(buf + len ,"Entry Address:NULL\n\n");
	}

	return len;
}


/* receives an index, returns the station mac address */
void wdrv_dbGetStaMacAddress(UINT16 index, char *macAddressPtr)
{
	PAL_memCopy(macAddressPtr, staTable[index].macAddr, MAC_ADDR_LEN);
}

/* print the stations table, called by the console */
int wdrv_dbPrintFourXTable(char *buf)
{
	int len = 0;
	int i, state;
	char stateString[MAX_PRINT_STRING];

	len += sprintf(buf + len,"4X table\n");
	len += sprintf(buf + len,"Number of Stations:%d\n", wdrv_dbGetNumStationsByState() /*numStations*/);

	len += sprintf(buf + len,"Index		Mac		State	  ActiveRate	ActiveRateAcx	PBCC  fourXConcatEnable fourXActive\n");
		
	for (i = 0; i < MAX_STATIONS; i++)
	{

		wdrv_mgmtGetState(i, &state);
		if (state != STATE_IDLE)
		{
			len += sprintf(buf + len," [%02d]   ", i);
			
			len += sprintf(buf + len,"%02X-%02X-%02X-%02X-%02X-%02X  ", (UINT8)staTable[i].macAddr[0], (UINT8)staTable[i].macAddr[1], (UINT8)staTable[i].macAddr[2], (UINT8)staTable[i].macAddr[3], (UINT8)staTable[i].macAddr[4], (UINT8)staTable[i].macAddr[5]);

			PAL_memSet(stateString, 0, MAX_PRINT_STRING);
			wdrv_mgmtGetStateString(i, stateString);
			len += sprintf(buf + len,"%s ", stateString);

			len += sprintf(buf + len,"	0x%X	", stationParams[i].activeRate);
			len += sprintf(buf + len,"	0x%X	", stationParams[i].activeRateAcx);

			if ((stationParams[i].capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK)
				len += sprintf(buf + len,"%d	", 1);
			else
				len += sprintf(buf + len,"%d	", 0);

			if (stationParams[i].fourX_cap.concatEn == TRUE)
				len += sprintf(buf + len,"%d	", 1);
			else
				len += sprintf(buf + len,"%d	", 0);

			if (stationParams[i].fourXActive == TRUE)
				len += sprintf(buf + len,"%d	\n", 1);
			else
				len += sprintf(buf + len,"%d	\n", 0);

		}
	}
	return len;

}


/* Attach the station's SSID as part of the station's details entry */
STATUS wdrv_dbSetStaSsidByIndex(UINT16 index, char *ssid, int SsidLen)
{
    staHashEntry_T	*hashPtr;
    
    if (index >= MAX_STATIONS)
    {
        if(REPORT_LEVEL(RL_DB))
            wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Setting SSID failed, index is out of range.\n");
        return ERROR;
    }

    if (SsidLen > WHAL_SSID_STRING_LEN) 
    {
        if(REPORT_LEVEL(RL_DB))
            wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Setting SSID failed, too long!\n");
        return ERROR;
    }

    hashPtr = &staTable[index];
    PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */

    if (hashPtr->free == TRUE)      /* entry is not in used, station does not exist in the tables */
    {
        PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
        if(REPORT_LEVEL(RL_DB))
            wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Setting SSID failed, index is free..!?\n");
        return ERROR;
    }

	PAL_memSet(hashPtr->ssid, 0, WHAL_SSID_STRING_LEN);
	PAL_memCopy(hashPtr->ssid, ssid, SsidLen);
    hashPtr->ssidLength = SsidLen;
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */

	if(REPORT_LEVEL(RL_DB))
		wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Adding SSID to station table secceeded.\n");

	return OK;
}


/* Get the station's associated SSID given its index */
void wdrv_dbGetStaSsidByIndex(UINT16 index, char *ssid, int *SsidLen)
{
	PAL_memCopy(ssid, staTable[index].ssid, staTable[index].ssidLength);
    *SsidLen = staTable[index].ssidLength;
}
 

/*************************************************************************
 *                        wdrv_dbIsRateSupported						 *
 *************************************************************************
DESCRIPTION: This function checks if rateAcx is supported
             by station[index]
                                                                                                   
INPUT:       index - the station index
			 rateAcx - the rate to check if station supports it

OUTPUT:      

RETURN:      TRUE  - rateAcx is supported in station[index]
             FALSE - rateAcx is not supported in station[index]
************************************************************************/
BOOL wdrv_dbIsRateSupported(UINT16 index, UINT16 rateAcx)
{
	if (stationParams[index].bitWiseSupRates & rateAcx)
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}


/* convert a list of rates to a bitwise ACX format */
static UINT16 convertDOT11RatesToAcxBitWiseFormat(UINT8 *rates, UINT8 ratesLen)
{
int i;
UINT16 bitWiseRates=0;

	for (i=0; i<ratesLen; i++)
	{
		switch (rates[i]&0x7F)
		{
			case DOT11_RATE_1:
				bitWiseRates |= ACX_RATE_1;
				break;

			case DOT11_RATE_2:
				bitWiseRates |= ACX_RATE_2;
				break;

			case DOT11_RATE_5_5:
				bitWiseRates |= ACX_RATE_5_5;
				break;

			case DOT11_RATE_6:
				bitWiseRates |= ACX_RATE_6;
				break;

			case DOT11_RATE_9:
				bitWiseRates |= ACX_RATE_9;
				break;

			case DOT11_RATE_11:
				bitWiseRates |= ACX_RATE_11;
				break;

			case DOT11_RATE_12:
				bitWiseRates |= ACX_RATE_12;
				break;

			case DOT11_RATE_18:
				bitWiseRates |= ACX_RATE_18;
				break;

			case DOT11_RATE_22:
				bitWiseRates |= ACX_RATE_22;
				break;

			case DOT11_RATE_24:
				bitWiseRates |= ACX_RATE_24;
				break;

			case DOT11_RATE_33:
				/* Do nothing as 33Mbps is not supported by the AP (1130) */
				break;

			case DOT11_RATE_36:
				bitWiseRates |= ACX_RATE_36;
				break;

			case DOT11_RATE_48:
				bitWiseRates |= ACX_RATE_48;
				break;

			case DOT11_RATE_54:
				bitWiseRates |= ACX_RATE_54;
				break;

			default:
				wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: FATAL ERROR, Invalid station active rate.\n");
		}
	}

	return bitWiseRates;
}


/* convert an ACX rate to a DOT11 rate format */
UINT8 convertAcxRateToDOT11Rate(UINT16 rateAcx)
{
UINT8 rate=0;

	switch (rateAcx & ACX_RATE_MASK)
	{
		case ACX_RATE_1:
			rate = DOT11_RATE_1;
			break;

		case ACX_RATE_2:
			rate = DOT11_RATE_2;
			break;

		case ACX_RATE_5_5:
			rate = DOT11_RATE_5_5;
			break;

		case ACX_RATE_6:
			rate = DOT11_RATE_6;
			break;

		case ACX_RATE_9:
			rate = DOT11_RATE_9;
			break;

		case ACX_RATE_11:
			rate = DOT11_RATE_11;
			break;

		case ACX_RATE_12:
			rate = DOT11_RATE_12;
			break;

		case ACX_RATE_18:
			rate = DOT11_RATE_18;
			break;

		case ACX_RATE_22:
			rate = DOT11_RATE_22;
			break;

		case ACX_RATE_24:
			rate = DOT11_RATE_24;
			break;

		case ACX_RATE_36:
			rate = DOT11_RATE_36;
			break;

		case ACX_RATE_48:
			rate = DOT11_RATE_48;
			break;

		case ACX_RATE_54:
			rate = DOT11_RATE_54;
			break;

		default:
			wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: FATAL ERROR, Invalid station active rate.\n");
	}

	return rate;
}


static void setStationAPRates(UINT16 index, dot11_supRates_T *rates, dot11_extSupRates_T *extRates,
                              UINT8 *apAllRates, UINT8 apAllRatesLen)
{
int i, j, length;

	/* rates & extRates are the Station's supported rates as */
	/* was transmitted in the (re)Association request IEs    */
	/* check each one of them if supported by the AP and set */
	/* the station's database apRates & apExtRates arrays    */
	/* Save the BasicRate bit as is supported by the AP as   */
	/* these arrays are used in the (re)association response */

	length = 0;
	for (i=0; i<rates->hdr.eleLen; i++)
	{
		for (j=0; j<apAllRatesLen; j++)
		{
			if ((rates->rates[i] & 0x7F) == (apAllRates[j] & 0x7F))
			{
				stationParams[index].apRates[length++] = apAllRates[j];
			}
		}
	}
	stationParams[index].apRatesLen = length;

	/* If extended rates doesn't exist, set length to zero and exit */
	if (extRates == NULL)
	{
		stationParams[index].apExtRatesLen = 0;
		return;
	}

	length = 0;
	for (i=0; i<extRates->hdr.eleLen; i++)
	{
		for (j=0; j<apAllRatesLen; j++)
		{
			if ((extRates->rates[i] & 0x7F) == (apAllRates[j] & 0x7F))
			{
				stationParams[index].apExtRates[length++] = apAllRates[j];
			}
		}
	}
	stationParams[index].apExtRatesLen = length;
}


/*************************************************************************
 *                        wdrv_dbSetStationRates						 *
 *************************************************************************
DESCRIPTION: This function  sets the station transmit parameters in the station params table.
                                                                                                   
INPUT:       The station index
			 The station transmit parameters

OUTPUT:      

RETURN:                                                                                                                                                   
************************************************************************/
void wdrv_dbSetStationRates(UINT16 index, dot11_supRates_T *rates, dot11_extSupRates_T *extRates, BOOL fourXSupport)
{
	int     i;
	UINT8	apAllRates[DOT11_SUPPORTED_RATES_MAX+DOT11_EXT_SUPPORTED_RATES_MAX];
	UINT8	apAllRatesLen;
	UINT16  bitWiseRates;
	UINT8	activeRate;
	UINT16	activeRateAcx = 0;
	BOOL	Pbcc;
	unsigned long flags;

	/* Find all AP's supported rates */
	wdrv_dbGetStationConfigOperationalAllRateSet(0,apAllRates, &apAllRatesLen);
	bitWiseRates = convertDOT11RatesToAcxBitWiseFormat(apAllRates, apAllRatesLen);
	
	/* Set all station's supported rates in bit wise format */
	stationParams[index].bitWiseSupRates = convertDOT11RatesToAcxBitWiseFormat( \
		rates->rates, rates->hdr.eleLen);
	
	/* Add also the extended supported rates if exist */
	if (extRates != NULL)
	{
		stationParams[index].bitWiseSupRates |= convertDOT11RatesToAcxBitWiseFormat( \
			extRates->rates, extRates->hdr.eleLen);
	}

	/* Find the station active rate whice is the highest rate supported */
	/* by both the AP and the station */
	/* The left most bit in the bit wise rates format is the highest */
	bitWiseRates &= stationParams[index].bitWiseSupRates;

	for (i=0; i<ACX_RATE_NUM; i++)
	{
		if ((bitWiseRates >> i) & 0x1)
		{
			activeRateAcx = (1 << i);
		}
	}

	activeRate = convertAcxRateToDOT11Rate(activeRateAcx);

	/* Set the station database rates that are both supported by the AP and the station */
	setStationAPRates(index, rates, extRates, apAllRates, apAllRatesLen);

	wdrv_dbGetCurrentPBCCSupportMode(&Pbcc);

	/* Enable PBCC mode if all the following meets: */
	/* 1) AP supports PBCC      */
	/* 2) Station supports PBCC */
	/* 3) 5.5, 11 or 22 Mbps only   */
	if (((stationParams[index].capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK) && (Pbcc))
	{
		if ((activeRate == DOT11_RATE_5_5) || (activeRate == DOT11_RATE_11) || (activeRate == DOT11_RATE_22))
		{
			activeRateAcx |= ACX_RATE_PBCC;
		}
	}
	else
	{
		if(activeRate == DOT11_RATE_22)
		{
			activeRateAcx = ACX_RATE_11;
			activeRate = DOT11_RATE_11;
		}
	}

	/* We set the four X enable feature only for rates greater than 11 */
	if(activeRate >= DOT11_RATE_11)
	{
		wdrv_dbSetStationConcatSupport(index, fourXSupport);
	}

	PAL_intrLock(flags); /* START OF CRITICAL SECTION */

	stationParams[index].ratesLen = rates->hdr.eleLen;
	PAL_memCopy(stationParams[index].rates, rates->rates, rates->hdr.eleLen);
	
	if (extRates != NULL)
	{
		stationParams[index].extRatesLen = extRates->hdr.eleLen;
		PAL_memCopy(stationParams[index].extRates, extRates->rates, extRates->hdr.eleLen);
	}
	else
	{
		stationParams[index].extRatesLen = 0;
	}

	stationParams[index].activeRate = activeRate;
	stationParams[index].activeRateAcx = activeRateAcx;

	/* updated every transmit, init to zero */
	stationParams[index].currentActiveRateWithFallback = 0;

	/* Config the rate adaptation table in the Tx module. */
	wdrv_txRateAdapSetRate( index, activeRateAcx);

   	PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *                        wdrv_dbSetStationRsnIe						 *
 *************************************************************************
DESCRIPTION: This function  sets the station transmit parameters in the station params table.
                                                                                                   
INPUT:       The station index
			 The station transmit parameters

OUTPUT:      

RETURN:                                                                                                                                                   
************************************************************************/
void wdrv_dbSetStationRsnIe(UINT16 index, dot11_rsne_T *rsnInfoEle)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	PAL_memSet(&stationParams[index].rsnIe, 0, sizeof(dot11_rsne_T));
	if (rsnInfoEle != NULL)
	{
		PAL_memCopy(&stationParams[index].rsnIe, rsnInfoEle, rsnInfoEle->hdr.eleLen + 2);
	}
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

void wdrv_dbGetStationRsnIe(UINT16 index, dot11_rsne_T *rsnInfoEle)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	PAL_memSet(rsnInfoEle, 0, sizeof(dot11_rsne_T));
	PAL_memCopy(rsnInfoEle, &stationParams[index].rsnIe, stationParams[index].rsnIe.hdr.eleLen + 2);
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

/* receives the station index, returns the station active rate */
void wdrv_dbGetActiveRateAcx(UINT16 index, UINT8 *activeRate)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	*activeRate = stationParams[index].activeRateAcx;
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

/* receives the station index, returns the station active rate */
void wdrv_dbGetActiveRate(UINT16 index, UINT8 *activeRate)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	*activeRate = stationParams[index].activeRate;
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

/* receives the station index, returns the station rates used in (re)associate response */
void wdrv_dbGetStationAPRates(UINT16 index, char *value, UINT8 *len)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	*len = stationParams[index].apRatesLen;
	PAL_memCopy(value, stationParams[index].apRates, stationParams[index].apRatesLen);
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

/* receives the station index, returns the station extended rates used in (re)associate response */
void wdrv_dbGetStationAPExtRates(UINT16 index, char *value, UINT8 *len)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	*len = stationParams[index].apExtRatesLen;
	PAL_memCopy(value, stationParams[index].apExtRates, stationParams[index].apExtRatesLen);
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

/* receives the station index, returns the station rates in ACX format */
void wdrv_dbGetStationAcxBitWiseRates(UINT16 index, UINT16 *acxBitWiseRates)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	*acxBitWiseRates = stationParams[index].bitWiseSupRates;
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

/* update the station currentActiveRateWithFallback in the station params table */
void wdrv_dbSetCurrentActiveRateWithFallback(UINT16 index, UINT16 *currentActiveRateWithFallback)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	stationParams[index].currentActiveRateWithFallback = *currentActiveRateWithFallback;
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

/* receives the station index, returns the station currentActiveRateWithFallback */
void wdrv_dbGetCurrentActiveRateWithFallback(UINT16 index, UINT16 *currentActiveRateWithFallback)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	*currentActiveRateWithFallback = stationParams[index].currentActiveRateWithFallback;
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

/* update the station listen interval in the station params table */
void wdrv_dbSetListenInterval(UINT16 index, UINT16 *listenInterval)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	stationParams[index].listenInterval = *listenInterval;
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

/* receives the station index, returns the station listen interval */
void wdrv_dbGetListenInterval(UINT16 index, UINT16 *listenInterval)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	*listenInterval = stationParams[index].listenInterval;
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

/* update the TI protection support in the station params table */
void wdrv_dbSetTIprotectionSupport(UINT16 index, BOOL *support)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	stationParams[index].TIprotectionSupported = *support;
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

/* receives the station index, returns the station TI protection support */
void wdrv_dbGetTIprotectionSupport(UINT16 index, BOOL *support)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	*support = stationParams[index].TIprotectionSupported;
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

/* update the FeatureSet in the station params table */
void wdrv_dbSetCommonFeatureSet(UINT16 index, UINT8 *set)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	stationParams[index].CommonFeatureSet = *set;
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}
 
/* receives the station index, returns the Common FeatureSet for AP-STA*/
void wdrv_dbGetCommonFeatureSet(UINT16 index, UINT8 *set)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	*set = stationParams[index].CommonFeatureSet;
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}


void    wdrv_dbSetStationWoneIndex(UINT16 index, UINT8 woneIndex)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	stationParams[index].WoneIdentifierIndex = woneIndex;
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}


void    wdrv_dbGetStationWoneIndex(UINT16 index, UINT8 *woneIndex)
{      
   *woneIndex = 0xff;

   if(index < MAX_STATIONS)
   {
      PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
      
      *woneIndex = stationParams[index].WoneIdentifierIndex;
	  if(staTable[index].free == TRUE)
	  {
		  printf("wdrv_dbGetStationWoneIndex: This STA entry was released: %d !!!\n",index); 
	  }      
      PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
      if(*woneIndex >= MAX_WONE_NET)
	  {
		  printf("wdrv_dbGetStationWoneIndex: wrong woneIndex = %d\n",*woneIndex);
         *woneIndex = 0xff;
	  }
      
   }
}


void    wdrv_dbGetStationRxPacketCounter(UINT16 index, UINT32 *unicastCounter)
{

	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */   
    *unicastCounter = stationParams[index].rxPacketCounter;
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}


void    wdrv_dbGetStationTxPacketCounter(UINT16 index, UINT32 *unicastCounter)
{
    *unicastCounter = stationParams[index].txPacketCounter;
}


void    wdrv_dbResetStationRxPacketCounter(UINT16 index)
{
    PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */

    stationParams[index].rxPacketCounter =0;
    
    PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */

}


void    wdrv_dbResetStationTxPacketCounter(UINT16 index)
{
    PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */

    stationParams[index].txPacketCounter =0;
    
    PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}


/* update the station capabilities in the station params table */
void wdrv_dbSetStationCapabilities(UINT16 index, UINT16 *capabilities)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	stationParams[index].capabilities = *capabilities;
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

/* Returns the station capabilities */
void wdrv_dbGetStationCapabilities(UINT16 index, UINT16 *capabilities)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	*capabilities = stationParams[index].capabilities;
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

/* reset the station params */
void wdrv_dbResetStationParams(UINT16 index)
{
	/* power save clean */
	wdrv_powerSaveCleanStaList(index, stationParams[index].WoneIdentifierIndex);
	
/*	wdrv_rxTerminateCollection(index);*/

	PAL_memSet(&stationParams[index], 0, sizeof(stationParams_T));

	if(REPORT_LEVEL(RL_DB))
		wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Reseting station params.\n");
	/* TODO: aging callbacks of other modules */
}

void wdrv_dbGetNumOfStationNotPBCCCapable(UINT16 *num)
{
	*num = numStationsNotPBCCCapabale;
}

void wdrv_dbGetCurrentPBCCSupportMode(BOOL *support)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	*support = currentPBCCSupportMode;
    PAL_spinUnLock(&drvDbHashLock);
}

void wdrv_dbSetCurrentPBCCSupportMode(BOOL support)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	currentPBCCSupportMode = support;
    PAL_spinUnLock(&drvDbHashLock);
}

void wdrv_dbGetNumOfStationNotShortPreambleCapable(UINT16 *num)
{
	*num = numStationsNotShortPreambleCapabale;
}

void wdrv_dbGetCurrentShortPreambleSupport(BOOL *support)
{
	*support = currentShortPreambleSupport;
}

void wdrv_dbGetNumOfStationNotSlotTimeCapable(UINT16 *num)
{
	*num = numStationsNotSlotTimeCapabale;
}

void wdrv_dbGetCurrentShortSlotTimeSupport(BOOL *support)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	*support = currentShortSlotTimeSupport;
    PAL_spinUnLock(&drvDbHashLock);
}

void wdrv_dbSetCurrentShortSlotTimeSupport(BOOL support)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	currentShortSlotTimeSupport = support;
    PAL_spinUnLock(&drvDbHashLock);
}

void wdrv_dbGetCurrentERPProtectionMode(BOOL *support)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	*support = currentERPProtectionMode;
    PAL_spinUnLock(&drvDbHashLock);
}

void wdrv_dbSetCurrentERPProtectionMode(BOOL support)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	currentERPProtectionMode = support;
    PAL_spinUnLock(&drvDbHashLock);
}
void  wdrv_dbGetCurrentERPNonPresent(BOOL *value)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	*value = currentERPNonPresent;
    PAL_spinUnLock(&drvDbHashLock);
}

void  wdrv_dbSetCurrentERPNonPresent(BOOL value)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */ 
    currentERPNonPresent = value;
    PAL_spinUnLock(&drvDbHashLock);
}

void wdrv_dbSetStationConcatSupport(UINT16 index, BOOL support)
{
	BOOL  enableConcat;

	wdrv_dbGetConcatEnable(&enableConcat);

	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */

	if ((enableConcat == TRUE) && (support == TRUE))
		stationParams[index].fourX_cap.concatEn = TRUE;
	else
		stationParams[index].fourX_cap.concatEn = FALSE;

   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

void wdrv_dbGetStationConcatSupport(UINT16 index, BOOL *mode)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	*mode = stationParams[index].fourX_cap.concatEn;
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

void wdrv_dbSetStationConcatMaxSize(UINT16 index, UINT16 size)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	stationParams[index].ConcatMaxSize = size;
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

void wdrv_dbGetStationConcatMaxSize(UINT16 index, UINT16 *size)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	*size = stationParams[index].ConcatMaxSize;
   	/* if the station cipher is TKIP, we need to keep place for the MIC and extended IV. */
	if (stationParams[index].cipherSuiteType == CIPHER_TKIP)
		*size = *size - MIC_LEN - 4;
	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

void wdrv_dbSetStationAckEmulSupport(UINT16 index, BOOL support)
{
	BOOL  enableAckEmul;

	wdrv_dbGetAckEmulEnable(&enableAckEmul);

	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */

	if ((enableAckEmul == TRUE) && (support == TRUE))
		stationParams[index].fourX_cap.ackEmulEn = TRUE;
	else
		stationParams[index].fourX_cap.ackEmulEn = FALSE;

   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

void wdrv_dbGetStationAckEmul(UINT16 index, BOOL *mode)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	*mode = stationParams[index].fourX_cap.ackEmulEn;
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

/* update the station capabilities in the station params table */
void wdrv_dbSetStationTIcapEnable(UINT16 index, BOOL support)
{	
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */

	if (support == TRUE)
		stationParams[index].TIcapEn = TRUE;
	else
		stationParams[index].TIcapEn = FALSE;

   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

/* update the station capabilities in the station params table */
void wdrv_dbSetStationFourXActive(UINT16 index, BOOL fourXActive)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */

    if( stationParams[index].fourX_cap.concatEn == FALSE )
        stationParams[index].fourXActive = FALSE;
    else
	    stationParams[index].fourXActive = fourXActive;

   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

/* Returns the station capabilities */
void wdrv_dbGetStationTIcapEnable(UINT16 index, BOOL *mode)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	*mode = stationParams[index].TIcapEn;
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

/* Returns the station capabilities */
void wdrv_dbGetStationFourXActive(UINT16 index, BOOL *mode)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
	*mode = stationParams[index].fourXActive;
   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

/* Disable PBCC support in station params table */
static void disableStationPBCCSupport(UINT16 index)
{
	if ((stationParams[index].activeRate == DOT11_RATE_5_5) ||
		(stationParams[index].activeRate == DOT11_RATE_11) ||
		(stationParams[index].activeRate == DOT11_RATE_22))
	{
		/* Disable PBCC mode for 5.5M, 11M & 22M rates */
		stationParams[index].activeRateAcx &= (~ACX_RATE_PBCC);

		/* Downgrade 22M to 11M as 22M can't work without PBCC */
		if (stationParams[index].activeRate == DOT11_RATE_22)
		{
			stationParams[index].activeRateAcx &= (~ACX_RATE_22);
			stationParams[index].activeRateAcx |= ACX_RATE_11;
		}

		wdrv_txRateAdapSetRate(index, stationParams[index].activeRateAcx);
	}

	#if 0
	if (stationParams[i].activeRate == 0x2C)
	{		/* if station active rate is 22, */
		stationParams[i].activeRateAcx = 110;		/* downgrade it to 11, no pbcc */
		wdrv_txRateAdapSetRate( i, 110);
	}
	else if ((stationParams[i].activeRate == 0x16) || (stationParams[i].activeRate == 0x0B))
	{		/* if the rate is pbcc encoded */
		stationParams[i].activeRateAcx &= 0x7F;		/* decode the rate as no pbcc */
		wdrv_txRateAdapSetRate( i,stationParams[i].activeRateAcx);
	}
	#endif
}

/* Enable PBCC support in station params table */
static void enableStationPBCCSupport(UINT16 index)
{
	if ((stationParams[index].activeRate == DOT11_RATE_5_5) ||
		(stationParams[index].activeRate == DOT11_RATE_11) ||
		(stationParams[index].activeRate == DOT11_RATE_22))
	{
		/* Enable PBCC mode for 5.5M, 11M & 22M rates */
		stationParams[index].activeRateAcx |= ACX_RATE_PBCC;

		/* Upgrade back to 22M, since PBCC enabled now */
		if (stationParams[index].activeRate == DOT11_RATE_22)
		{
			stationParams[index].activeRateAcx &= (~ACX_RATE_11);
			stationParams[index].activeRateAcx &= (~ACX_RATE_5_5);
			stationParams[index].activeRateAcx |= ACX_RATE_22;
		}

		wdrv_txRateAdapSetRate(index, stationParams[index].activeRateAcx);
	}

	#if 0
	if (stationParams[i].activeRate == 0x2C)
	{		/* if station active rate is 22, */
		stationParams[i].activeRateAcx = 220;		/* upgrade acx encoded rate to 22, pbcc */
		wdrv_txRateAdapSetRate( i,/*Rate*/220);
	}

	else if ((stationParams[i].activeRate == 0x0B) || (stationParams[i].activeRate == 0x16))
	{		/* if the rate should be pbcc encoded */
		stationParams[i].activeRateAcx |= ACX_RATE_PBCC;		/* encode the rate as pbcc */
		wdrv_txRateAdapSetRate( i,stationParams[i].activeRateAcx);
	}
	#endif
}


/*************************************************************************
 *                        wdrv_dbPerformPBCCAlgorithmStatic					 *
 *************************************************************************
DESCRIPTION: This function  implement the PBCC static algorithm as following:
			1. Upon station association: 
				a. If the station supports pbcc and pbcc current support is TRUE:
					1. We do nothing
				b. If the station supports pbcc and pbcc current support is FALSE:
					1. We change its active rate (only the one which is configured to the ACX) to a non pbcc one.
				c. If the station does not support pbcc and it is the first non-pbcc station to associate:
					1. We set the rates of all pbcc stations to non pbcc rates,
					2. We set the beacon's pbcc bit to FALSE.
					3. We set the current pbcc suport mode to FALSE
				d. If the station does not support pbcc and it is not the first non-pbcc station to associate:
					1. We do nothing
			2. Upon station dis association: 
				a. If the station supports pbcc and pbcc current support is TRUE:
					1. We do nothing
				b. If the station supports pbcc and pbcc current support is FALSE:
					1. We do nothing
				c. If the station does not support pbcc and it is the last non-pbcc station to disassociate:
					1. We set the rates of all pbcc stations to pbcc rates,
					2. We set the beacon's pbcc bit to TRUE.
					3. We set the current pbcc suport mode to TRUE
				d. If the station does not support pbcc and it is not the last non-pbcc station to associate:
					1. We do nothing
                                                                                                   
INPUT:       The station index
			 The station action (associate, disassociate)

OUTPUT:      

RETURN:                                                                                                                                                   
************************************************************************/
void wdrv_dbPerformPBCCAlgorithmStatic(UINT16 index, algo_action_t action)
{
	BOOL   PBCCdynamicActive;
	
	if (action == ASSOCIATE)
	{
		if (!((stationParams[index].capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK))  		/* station does not support PBCC */
		{
			numStationsNotPBCCCapabale++;
			
			/* if pbcc mode have been changed  */
			if ((numStationsNotPBCCCapabale) == 1)
			{
				if(REPORT_LEVEL(RL_DYNAMIC_ALG))
					printf("going to set PBCC3: %d\n",0);

				/* now, for stations that are PBCC, set active rate to non PBCC. */
				wdrv_dbSetPBCCStationsSupport(FALSE);
	
				whal_apiSetPbcc(0,0);
				currentPBCCSupportMode = FALSE;
			}
		}
		else	/* station supports PBCC */
		{
			if (currentPBCCSupportMode == FALSE)
			{	
				disableStationPBCCSupport(index);
			}
		}
	}

	else /* DISASSOCIATES */
	{
		if (!((stationParams[index].capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK)) 		/* station does not support PBCC*/
		{
			numStationsNotPBCCCapabale--;
			
			/* if there are no more notPBCCcap STAs and there is no external notPBCCcap BSSs
			   we can turn the PBCC on */
			if (numStationsNotPBCCCapabale == 0)
			{
				wdrv_mgmtIsPBCCexternalActive(&PBCCdynamicActive);
				if(PBCCdynamicActive == FALSE)
				{
					if(REPORT_LEVEL(RL_DYNAMIC_ALG))
						printf("going to set PBCC4: %d\n",1);

					/* now, for stations that are PBCC, set active rate to PBCC. */
					wdrv_dbSetPBCCStationsSupport(TRUE);
	
					whal_apiSetPbcc(0,1);
					currentPBCCSupportMode = TRUE;
				}
			}
		}
	}
}


/*************************************************************************
 *                        wdrv_dbPerformPreambleAlgorithmStatic					 *
 *************************************************************************
DESCRIPTION: This function implement the Preamble static algorithm.
             It is called only if AP supports Short Preamble

                                                                                                   
INPUT:       index - the station index
			 action - the station action (associate or disassociate)

OUTPUT:      

RETURN:                                                                                                                                                   
************************************************************************/
void wdrv_dbPerformPreambleAlgorithmStatic(UINT16 index, algo_action_t action)
{
	int stationShortPreambleSupport;
	
	stationShortPreambleSupport = (stationParams[index].capabilities >> CAP_PREAMBLE_SHIFT) & CAP_PREAMBLE_MASK;

	if (action == ASSOCIATE)
	{
		if (!stationShortPreambleSupport)
		{
            PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
			numStationsNotShortPreambleCapabale++;
			
			/* if first station not capable */
			if (numStationsNotShortPreambleCapabale == 1)
			{
               currentShortPreambleSupport = FALSE;
               PAL_spinUnLock(&drvDbHashLock);
			   whal_apiSetERPLongPreambleOnly(0, 1);
			}
            else 
               PAL_spinUnLock(&drvDbHashLock);
		}
	}

	else /* DISASSOCIATES */
	{
		if (!stationShortPreambleSupport)
		{
            PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */
			numStationsNotShortPreambleCapabale--;
			
			/* if no more stations that are not capable */
			if (numStationsNotShortPreambleCapabale == 0)
			{
                currentShortPreambleSupport = TRUE;
                PAL_spinUnLock(&drvDbHashLock);
				whal_apiSetERPLongPreambleOnly(0, 0);
			}
            else
                PAL_spinUnLock(&drvDbHashLock);
		}
	}
}


/*************************************************************************
 *                        wdrv_dbPerformSlotTimeAlgorithmStatic					 *
 *************************************************************************
DESCRIPTION: This function implement the Slot Time static algorithm.
             It is called only if AP supports short slot time

                                                                                                   
INPUT:       index - the station index
			 action - the station action (associate or disassociate)

OUTPUT:      

RETURN:                                                                                                                                                   
************************************************************************/
void wdrv_dbPerformSlotTimeAlgorithmStatic(UINT16 index, algo_action_t action)
{
	int stationShortSlotTimeSupport;
	
	stationShortSlotTimeSupport = (stationParams[index].capabilities >> CAP_SLOTTIME_SHIFT) & CAP_SLOTTIME_MASK;

	if (action == ASSOCIATE)
	{
		if (!stationShortSlotTimeSupport)
		{
			numStationsNotSlotTimeCapabale++;
			
			/* if first station not capable */
			if (numStationsNotSlotTimeCapabale == 1)
			{
				wdrv_mgmtShortSlotTimeSM(INPUT_SLOTTIME_B_STA_IN_BSS);
			}
		}
	}

	else /* DISASSOCIATES */
	{
		if (!stationShortSlotTimeSupport)
		{
			numStationsNotSlotTimeCapabale--;
			
			/* if no more stations that are not capable */
			if (numStationsNotSlotTimeCapabale == 0)
			{
				wdrv_mgmtShortSlotTimeSM(INPUT_SLOTTIME_G_STA_ONLY);
			}
		}
	}
}


/*************************************************************************
 *                        wdrv_dbPerformERPProtectionAlgorithmStatic					 *
 *************************************************************************
DESCRIPTION: This function implement the Slot Time static algorithm.
             It is called only if AP supports short slot time
			 (slot time capability is true in NVRAM)

                                                                                                   
INPUT:       index - the station index
			 action - the station action (associate or disassociate)

OUTPUT:      

RETURN:                                                                                                                                                   
************************************************************************/
void wdrv_dbPerformERPProtectionAlgorithmStatic(UINT16 index, algo_action_t action)
{
	BOOL nonERPStation;
	BOOL ERPdynamicActive;

	
	if (wdrv_dbIsRateSupported(index, ACX_RATE_ALL_OFDM_MASK))
	{
		/* Station supports OFDM so it is not a nonERP station */
		nonERPStation = FALSE;
	}
	else
	{
		nonERPStation = TRUE;
	}

	if (action == ASSOCIATE)
	{
		if (nonERPStation)
		{
			numOfNonERPStations++;
			
			/* if first nonERP station */
			if (numOfNonERPStations == 1)
			{
				if(REPORT_LEVEL(RL_DYNAMIC_ALG))
					wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_INFORMATION, "WDRV_MGMT: going to set ERP1: 1,1\n");
				
				whal_apiSetERPProtection(0,1,1);
			}
		}
		else
			numOfERPstations++;
	}

	else /* DISASSOCIATES */
	{
		if (nonERPStation)
		{
			numOfNonERPStations--;
			
			/* if no more nonERP stations */
			if (numOfNonERPStations == 0)
			{				
				wdrv_mgmtIsERPexternalActive(&ERPdynamicActive);
				if(ERPdynamicActive)
				{				
					if(REPORT_LEVEL(RL_DYNAMIC_ALG))
						wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_INFORMATION, "WDRV_MGMT: going to set ERP2: 0,1\n");
					
					whal_apiSetERPProtection(0,0,1);
				}
				else
				{
					if(REPORT_LEVEL(RL_DYNAMIC_ALG))
						wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_INFORMATION, "WDRV_MGMT: going to set ERP3: 0,0\n");

					whal_apiSetERPProtection(0,0,0);
				}				
			}
		}
		else
			numOfERPstations--;
	}
}


/*************************************************************************
 *                        wdrv_dbSetPBCCStationsSupport			 		*
 *************************************************************************
DESCRIPTION: This function sets the pbcc support of station according to the input mode.
			Called by the dynamic or static pbcc algorithm function.
			1. If mode is TRUE, set the rates of all pbcc station to pbcc rates.
			2. If mode is FALSE, set the rates of all pbcc station to non-pbcc rates.
                                                                                                   
INPUT:       The required pbcc mode

OUTPUT:      

RETURN:                                                                                                                                                   
************************************************************************/
void wdrv_dbSetPBCCStationsSupport(BOOL mode)
{
	int i, state;

	if (mode == TRUE)
	{
		for (i = 0; i < MAX_STATIONS; i++)
		{
			wdrv_mgmtGetState(i, &state);

			if (((stationParams[i].capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK) && ((state == STATE_ASSOCIATED) || (state == STATE_AUTHORIZED)))
			{	
				enableStationPBCCSupport(i);
			}
		}
	}

	else
	{
		for (i = 0; i < MAX_STATIONS; i++)
		{
			wdrv_mgmtGetState(i, &state);

			if (((stationParams[i].capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK)	&& ((state == STATE_ASSOCIATED) || (state == STATE_AUTHORIZED))) 
			{	
				disableStationPBCCSupport(i);
			}
		}
	}
}


/*************************************************************************
 *                        wdrv_dbIsStationPBCCCapable			 		*
 *************************************************************************
DESCRIPTION: Returns whether the STA is PBCC capable or not
                                                                                                   
INPUT:       STA index

OUTPUT:      

RETURN:  PBCC cap/not cap (TRUE/FALSE)                                                                                                                                                 
************************************************************************/
BOOL wdrv_dbIsStationPBCCCapable(UINT16 index)
{
	if ((stationParams[index].capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK)
		return TRUE;
	else
		return FALSE;
}

/*************************************************************************
 *                        wdrv_dbIsStationShortSlotTimeCapable	 		*
 *************************************************************************
DESCRIPTION: Returns whether the STA is short slot time capable or not
                                                                                                   
INPUT:       STA index

OUTPUT:      

RETURN:  Short Slot Time cap/not cap (TRUE/FALSE)                                                                                                                                                 
************************************************************************/
BOOL wdrv_dbIsStationShortSlotTimeCapable(UINT16 index)
{
	if((stationParams[index].capabilities >> CAP_SLOTTIME_SHIFT) & CAP_SLOTTIME_MASK)
		return TRUE;
	else
		return FALSE;
}

/*************************************************************************
 *                        wdrv_dbIsStationERP	 		*
 *************************************************************************
DESCRIPTION: Returns whether the STA is an ERP or NonERP
                                                                                                   
INPUT:       STA index

OUTPUT:      

RETURN: ERP/NonERP (TRUE/FALSE)                                                                                                                                                  
************************************************************************/
BOOL wdrv_dbIsStationERP(UINT16 index)
{
	if(wdrv_dbIsRateSupported(index, ACX_RATE_ALL_OFDM_MASK))
		return TRUE;
	else
		return FALSE;
}

/*************************************************************************
 *                        wdrv_dbPrintStaRsnIeTable            	 		*
 *************************************************************************
DESCRIPTION: 
                                                                                                   
INPUT:      

OUTPUT:      

RETURN:                                                                                                                                             
************************************************************************/
void wdrv_dbPrintStaRsnIeTable(void)
{
   int i, state;
   char stateString[MAX_PRINT_STRING];

   printf("Stations RSN IE table\n");
   printf("Number of Stations:%d\n", wdrv_dbGetNumStationsByState() /*numStations*/);

   for (i = 0; i < MAX_STATIONS; i++)
   {

      wdrv_mgmtGetState(i, &state);
      if (state != STATE_IDLE)
      {
         printf(" [%02d]   ", i);

         printf("%02X-%02X-%02X-%02X-%02X-%02X  ", staTable[i].macAddr[0], staTable[i].macAddr[1], staTable[i].macAddr[2], staTable[i].macAddr[3], staTable[i].macAddr[4], staTable[i].macAddr[5]);

         PAL_memSet(stateString, 0, MAX_PRINT_STRING);
         wdrv_mgmtGetStateString(i, stateString);
         printf("%s ", stateString);

         if (stationParams[i].generateTxMicFailure == TRUE)
            printf("	generateTxMicFailure for this station is TRUE\n" );
         else
            printf("	generateTxMicFailure for this station is FALSE\n" );

         if (stationParams[i].cipherSuiteType == CIPHER_NONE)
            printf("	NONE\n" );
         else if (stationParams[i].cipherSuiteType == CIPHER_WEP)
            printf("	WEP\n" );
         else if (stationParams[i].cipherSuiteType == CIPHER_TKIP)
            printf("	TKIP\n" );
         else
            printf("	Invalid\n" );

         printf("RX MIC Key:	0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X\n", stationParams[i].rxMicKey[0], stationParams[i].rxMicKey[1], stationParams[i].rxMicKey[2], stationParams[i].rxMicKey[3], stationParams[i].rxMicKey[4], stationParams[i].rxMicKey[5], stationParams[i].rxMicKey[6], stationParams[i].rxMicKey[7]);
         printf("TX MIC Key:	0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X\n", stationParams[i].txMicKey[0], stationParams[i].txMicKey[1], stationParams[i].txMicKey[2], stationParams[i].txMicKey[3], stationParams[i].txMicKey[4], stationParams[i].txMicKey[5], stationParams[i].txMicKey[6], stationParams[i].txMicKey[7]);

         printf("\n\n");
      }
   }
}



/*************************************************************************
 * Util function to print rate in MBPS units 
 *************************************************************************
DESCRIPTION: 
                                                                                                   
INPUT:      

OUTPUT:      

RETURN:                                                                                                                                             
************************************************************************/
int wdrv_printRateInMbpsUnit(UINT8 rate ,char *buf)
{
	UINT32 rate1, rate2, rate3;
	BOOL basic;
	int len =0;

	if (rate)
	{
		rate1 = rate;
		basic = rate1 & 0x80;
		rate1 &= 0x7f;
		rate1 *= 500;
		rate2 = rate1/1000;
		rate3 = rate1%1000;

		if (rate3)
			len +=sprintf(buf +len,"%d.%dMbps", rate2, 5);
		else
			len +=sprintf(buf +len,"%dMbps", rate2);

		if (basic)
			len +=sprintf(buf +len,"B");
	}

	return len;
}

/*************************************************************************
 *                     wdrv_dbSetStationSuiteType            	 
 *************************************************************************
DESCRIPTION: 
                                                                                                   
INPUT:      

OUTPUT:      

RETURN:                                                                                                                                                 
*************************************************************************/
void wdrv_dbSetStationSuiteType(UINT16 index, UINT8 cipher)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */

	stationParams[index].cipherSuiteType = cipher;

   	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}


/*************************************************************************
 *                     wdrv_dbSetStationEncryptionKey            	 
 *************************************************************************
DESCRIPTION: 
                                                                                                   
INPUT:      

OUTPUT:      

RETURN:                                                                                                                                                 
*************************************************************************/
void wdrv_dbSetStationEncryptionKey(UINT16 index, UINT16 keySize, UINT8 *keyValue)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */

	if (keyValue == NULL)
	{
		PAL_memSet(stationParams[index].encryptionKey, 0, WPA_MAX_WEP_KEY_LEN);	
		stationParams[index].encryptionKeyLen = 0;
	}
	else
	{
		PAL_memCopy(stationParams[index].encryptionKey, keyValue, keySize);	
		stationParams[index].encryptionKeyLen = keySize;
	}

	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}


/*************************************************************************
 *                     wdrv_dbGetStationEncryptionKey            	 
 *************************************************************************
DESCRIPTION: 
                                                                                                   
INPUT:      

OUTPUT:      

RETURN:                                                                                                                                                 
*************************************************************************/
void wdrv_dbGetStationEncryptionKey(UINT16 index, UINT16 *keySize, UINT8 *keyValue)
{
	PAL_memCopy(keyValue, stationParams[index].encryptionKey ,stationParams[index].encryptionKeyLen);
    *keySize = stationParams[index].encryptionKeyLen;
}


/*************************************************************************
 *                     wdrv_dbSetStationTxMicKey            	 
 *************************************************************************
DESCRIPTION: 
                                                                                                   
INPUT:      

OUTPUT:      

RETURN:                                                                                                                                                 
*************************************************************************/
void wdrv_dbSetStationTxMicKey(UINT16 index, UINT8 *keyValue)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */

	if (keyValue == NULL)
		PAL_memSet(stationParams[index].txMicKey, 0, MICHAEL_KEY_LEN);	
	else
		PAL_memCopy(stationParams[index].txMicKey, keyValue, MICHAEL_KEY_LEN);	

	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}


/*************************************************************************
 *                     wdrv_dbGetStationSuiteType            	 
 *************************************************************************
DESCRIPTION: 
                                                                                                   
INPUT:      

OUTPUT:      

RETURN:                                                                                                                                                 
*************************************************************************/
void wdrv_dbSetStationRxMicKey(UINT16 index, UINT8 *keyValue)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */

	if (keyValue == NULL)
		PAL_memSet(stationParams[index].rxMicKey, 0, MICHAEL_KEY_LEN);	
	else
		PAL_memCopy(stationParams[index].rxMicKey, keyValue, MICHAEL_KEY_LEN);	

	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *                                 	 
 *************************************************************************
DESCRIPTION: 
                                                                                                   
INPUT:      

OUTPUT:      

RETURN:                                                                                                                                                 
*************************************************************************/
void wdrv_dbGetStationTxMicKey(UINT16 index, UINT8 *keyValue)
{
	if (stationParams[index].generateTxMicFailure == TRUE)
	{
		PAL_memSet(keyValue, 1, MICHAEL_KEY_LEN);	
		stationParams[index].generateTxMicFailure = FALSE;
		printf("Generating a Tx MIC failure for station in index: %d\n", index);
	}
	else
		PAL_memCopy(keyValue, stationParams[index].txMicKey, MICHAEL_KEY_LEN);	
}

/*************************************************************************
 *                                 	 
 *************************************************************************
DESCRIPTION: 
                                                                                                   
INPUT:      

OUTPUT:      

RETURN:                                                                                                                                                 
*************************************************************************/
void wdrv_dbGetStationRxMicKey(UINT16 index, UINT8 *keyValue)
{
   PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */

   PAL_memCopy(keyValue, stationParams[index].rxMicKey, MICHAEL_KEY_LEN);
   
   PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
}


/*************************************************************************
 *                                 	 
 *************************************************************************
DESCRIPTION: 
                                                                                                   
INPUT:      

OUTPUT:      

RETURN:                                                                                                                                                 
*************************************************************************/
BOOL wdrv_dbIsDataFirstFromStation(UINT16 index)
{
	PAL_spinLock(&drvDbHashLock); /* START OF CRITICAL SECTION */

	if (stationParams[index].dataReceivedFromStation == FALSE)	
	{
		stationParams[index].dataReceivedFromStation = TRUE;
		PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
		return TRUE;
	}

	PAL_spinUnLock(&drvDbHashLock);  /* END OF CRITICAL SECTION */
	return FALSE;
}


/*************************************************************************
 *                       wdrv_dbGenerateTxMicFailure          	 
 *************************************************************************
DESCRIPTION: 
                                                                                                   
INPUT:      

OUTPUT:      

RETURN:                                                                                                                                                 
*************************************************************************/
void wdrv_dbGenerateTxMicFailure(char *mac)
{
   UINT16 index;
   int   rc;

   rc = wdrv_dbFindStaIndex(mac, &index);

   if (rc != OK)
      wdrv_utilsReport(RL_DB, PRI_INFORMATION,"wdrv_dbGenerateTxMicFailure: station not found\n");
   else
      stationParams[index].generateTxMicFailure = TRUE;
}

inline void wdrv_dbIncreaseStationRxPacketCounter(UINT16 index)
{
     stationParams[index].rxPacketCounter++;
}

inline void wdrv_dbIncreaseStationTxPacketCounter(UINT16 index)
{
    stationParams[index].txPacketCounter++;
}

inline void wdrv_dbGetStationSuiteType(UINT16 index, UINT8 *cipher)
{
	*cipher = stationParams[index].cipherSuiteType;
}

inline void wdrv_dbSetWpaReq(UINT16 index, UINT32 reason)
{
	stationParams[index].wpaReqReason = reason;
}

inline void wdrv_dbGetWpaReq(UINT16 index, UINT32 *reason)
{
	*reason = stationParams[index].wpaReqReason;
}

inline UINT16 wdrv_dbGetNumStations(void)
{
	return numStations;
}

