/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*                                                                          */
/*		MODULE:     drvDbRegDomain.c                                        */
/*		PURPOSE:    AP database for 802.11d 802.11h.                        */
/*                  This file holds the database of the configuration       */
/*					tables, global & local variables and interfaces.        */
/*                                                                          */
/***************************************************************************/
#include "drvDbRegDomain.h"
#include "drvDbRegDomainDefs.h"
#include "drvDbMain.h"

/*-----------------------------------------------------------*/
/*         Definitions                                       */
/*-----------------------------------------------------------*/

#define NUM_OF_REGULATORY_DOMAINS_A 2
#define NUM_OF_REGULATORY_DOMAINS_B 8   

#define NUM_OF_RANGE_ELEMENTS_US    2
#define NUM_OF_RANGE_ELEMENTS_WORLD 3


typedef struct {    /* This is a storage for a list of Channel Range elements */
    IntegerRegulatoryDomains_e  regulatoryDomain;
    UINT8                       numOfRanges;
    tChannelRangeElementStruct  *pListElements;
} tChannelRangeTableElement;

/*-----------------------------------------------------------*/
/*         Local Variables                                   */
/*-----------------------------------------------------------*/

/*----------------------------------------------------------------------------
	The following variables are intialized from the nvram. When changing the nvram
	during run time, these variables do not changed, in order to keep running in the 
	same configuration. After restart, this variables will be updated again from the nvram
	with the new configuration.
----------------------------------------------------------------------------*/

static UINT8     dot11MultiDomainCapabilityEnable;   /* disabling d ==> disabling h */
static UINT8     dot11SpectrumManagementRequired;    /* enabling h ==> enabling d */
static UINT8     whalEnableLegacyStations;
static UINT8     dot11CurrentRegDomain;
static INT8      dot11LocalPowerConstraint;
static UINT8     whalMaxTxPowerLevel;                        
static UINT8     txPowerLevels_a[NUM_OF_TX_POWER_LEVELS];      /* levels of dBm for band A*/
static UINT8     txPowerLevels_g[NUM_OF_TX_POWER_LEVELS];      /* levels of dBm for band G*/
static char      dot11CountryString[COUNTRY_STRING_LENGTH];
static UINT8     dot11PrivateRegulatoryDomain;
static UINT8     numOfRanges_802_11_a;
static tChannelRangeElementStruct privateRangeElements_a[MAX_NUM_OF_RANGES_A];
static UINT8					  numOfRanges_802_11_b;
static tChannelRangeElementStruct privateRangeElements_b[MAX_NUM_OF_RANGES_B];

/* 802.11d Allowed Channel Tables - hardcoded values */
static tChannelRangeTableElement  RangeTable_a[NUM_OF_REGULATORY_DOMAINS_A];
static tChannelRangeTableElement  RangeTable_b[NUM_OF_REGULATORY_DOMAINS_B];


/*-----------------------------------------------------------*/
/*         Local Functions                                   */
/*-----------------------------------------------------------*/
static void	wdrv_dbInitRangeTables(void);	

/*************************************************************************
 *                        wdrv_dbRegDomainInit                                    *
 *************************************************************************
DESCRIPTION: This function initializes the AP's main database:
			1. Reset the MIB tables.                 
			2. Load the database variables with the values from the nvram.
			3. Fixes the AP's defaulte active rate.
			4. Builds information elements based on the database. Thius elements
				are sent in management messges.
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      OK : Initialization succesfull.
             NOK: Initialization unsuccesfull.                                                                                                                                             
************************************************************************/
INIT_RET_CODE wdrv_dbRegDomainInit(RegDomainInitStrcut *pInitStruct , UINT8 band)
{
	int index, level;
    UINT8  privateRegDomain;
    wlan_bandType bandType;

    /* init 802.11 d & h params */
	dot11MultiDomainCapabilityEnable = pInitStruct->dot11MultiDomainCapabilityEnable;
	
	printf("\n802.11d is ");
    if (dot11MultiDomainCapabilityEnable) 
    {
        printf("** Enabled **\n");
    }
    else
        printf("disabled.\n");

	dot11SpectrumManagementRequired = pInitStruct->dot11SpectrumManagementRequired;
	whalEnableLegacyStations        = pInitStruct->whalEnableLegacyStations;
	
	printf("802.11h is ");
    if (dot11SpectrumManagementRequired) 
    {
        printf("** Enabled **\n");
        if (whalEnableLegacyStations) 
        {
            printf("Legacy Stations are ** Enabled **\n");
        }
    }
    else
        printf("disabled.\n");


	bandType = (band == RADIO_BAND_2_4GHZ) ? WLAN_BAND_B : WLAN_BAND_A;
    dot11CurrentRegDomain = pInitStruct->dot11CurrentRegDomain;

	/* Check here: if Regulatory domain is not one of the list - use default */
    switch (dot11CurrentRegDomain)
    { 
        case REG_DOMAIN_US:
        case REG_DOMAIN_WORLD:
            if (bandType == WLAN_BAND_A) 
                break;
            else    /* band B contains these value - check if it comes from private or set to default */
            {
				privateRegDomain = pInitStruct->privateRegDomain;
                if (dot11CurrentRegDomain != privateRegDomain)
                {
                    if (bandType == WLAN_BAND_B)
                    {
						if (dot11CurrentRegDomain == REG_DOMAIN_US)
						{
							printf("Setting to default (FCC)\n");
							dot11CurrentRegDomain = REG_DOMAIN_FCC;
						} else
						{
							printf("Setting to default (ETSI)\n");
							dot11CurrentRegDomain = REG_DOMAIN_ETSI;
						}
                    }
                    else
                    {
                        printf("Unknown Regulatory Domain (0x%X), setting to default (US)\n",dot11CurrentRegDomain);
                        dot11CurrentRegDomain = REG_DOMAIN_US;
                    }
                }
            }
            break;    
        case REG_DOMAIN_FCC:
        case REG_DOMAIN_ETSI:
        case REG_DOMAIN_IC:
        case REG_DOMAIN_SPAIN:
        case REG_DOMAIN_FRANCE:
        case REG_DOMAIN_MKK:
        case REG_DOMAIN_MKK1:
            if (bandType == WLAN_BAND_B) 
                break;
            /* or else (A).. continue to default */
        default:
			privateRegDomain = pInitStruct->privateRegDomain;
            if (dot11CurrentRegDomain != privateRegDomain)
            {
                if (bandType == WLAN_BAND_B)
                {
                    printf("Unknown Regulatory Domain (0x%X), setting to default (FCC)\n",dot11CurrentRegDomain);
                    dot11CurrentRegDomain = REG_DOMAIN_FCC;
                }
                else
                {
					if (dot11CurrentRegDomain == REG_DOMAIN_FCC)
					{
						dot11CurrentRegDomain = REG_DOMAIN_US;
						printf("Setting to default (US)\n");
	
					} else
					{
						dot11CurrentRegDomain = REG_DOMAIN_WORLD;
						printf("Setting to default (WORD)\n");
	
					}
                }
            }
            break;
    }

	dot11LocalPowerConstraint = pInitStruct->dot11LocalPowerConstraint;
    whalMaxTxPowerLevel       = pInitStruct->whalMaxTxPowerLevel;
	

	if ((whalMaxTxPowerLevel < 1) || (whalMaxTxPowerLevel > 5)) 
    {
	    printf("Error Max Tx PowerLevel value : %d.\n",whalMaxTxPowerLevel); 
        printf("The value  should be between 1..5\n");
        return DDC_INIT_CFG_RANGE_ERROR;
    }
    
	for (level=1; level<=5; level++)
    {
		txPowerLevels_a[level-1] = pInitStruct->txPowerLevels_a[level-1];
		txPowerLevels_g[level-1] = pInitStruct->txPowerLevels_g[level-1];
	}
    
    PAL_memCopy(dot11CountryString,pInitStruct->dot11CountryString,COUNTRY_STRING_LENGTH);

	if (dot11MultiDomainCapabilityEnable) 
    {
        printf("Country String is: '%c%c%c'\n",
               dot11CountryString[0],
               dot11CountryString[1],
               dot11CountryString[2]);
    }


	dot11PrivateRegulatoryDomain = 	pInitStruct->dot11PrivateRegulatoryDomain;
	numOfRanges_802_11_a   = pInitStruct->numOfRanges_802_11_a;

    for (index=1; index<=numOfRanges_802_11_a;index++)
	{
		privateRangeElements_a[index-1] = pInitStruct->privateRangeElements_a[index-1];
	}
	numOfRanges_802_11_b = pInitStruct->numOfRanges_802_11_b;
	
	for (index=1; index<=numOfRanges_802_11_b;index++)
    {
		privateRangeElements_b[index-1] = pInitStruct->privateRangeElements_b[index-1];
	}
    
	wdrv_dbInitRangeTables();

    return DDC_INIT_SUCCESS;
}

/*************************************************************************
 *						wdrv_dbRegDomainUnload                           *
 *************************************************************************
DESCRIPTION: Unloads reg domain sub module
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbRegDomainUnload(void)
{
	if(RangeTable_a[0].pListElements)
		PAL_memFree(0,(void *)RangeTable_a[0].pListElements,0);	
	if(RangeTable_b[0].pListElements)
		PAL_memFree(0,(void *)RangeTable_b[0].pListElements,0);
	if(RangeTable_b[1].pListElements)
		PAL_memFree(0,(void *)RangeTable_b[1].pListElements,0);
	if(RangeTable_b[2].pListElements)
		PAL_memFree(0,(void *)RangeTable_b[2].pListElements,0);
	if(RangeTable_b[3].pListElements)
		PAL_memFree(0,(void *)RangeTable_b[3].pListElements,0);
	if(RangeTable_b[4].pListElements)
		PAL_memFree(0,(void *)RangeTable_b[4].pListElements,0);
	if(RangeTable_b[5].pListElements)
		PAL_memFree(0,(void *)RangeTable_b[5].pListElements,0);
	if(RangeTable_b[6].pListElements)
		PAL_memFree(0,(void *)RangeTable_b[6].pListElements,0);
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetdot11MultiDomainCapabilityStatus(UINT8 *value)
{
    *value = dot11MultiDomainCapabilityEnable;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetdot11SpectrumManagementRequiredStatus(UINT8 *value)
{
    *value = dot11SpectrumManagementRequired;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetLegacyStatusEnableStatus(UINT8 *value)
{
    *value = whalEnableLegacyStations;
}
/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
int wdrv_dbGetdot11CurrentRegDomain(UINT8 band, UINT8 *pRegDomain)
{
    *pRegDomain = dot11CurrentRegDomain;
    
    if(band == WLAN_BAND_A)  
    {
        switch (*pRegDomain) 
        {
            case REG_DOMAIN_FCC:
            case REG_DOMAIN_US:
                *pRegDomain = REG_DOMAIN_US;
                break;

            case REG_DOMAIN_IC:
            case REG_DOMAIN_ETSI:
            case REG_DOMAIN_SPAIN:
            case REG_DOMAIN_FRANCE:
            case REG_DOMAIN_MKK:
            case REG_DOMAIN_MKK1:
            case REG_DOMAIN_WORLD:
                *pRegDomain = REG_DOMAIN_WORLD;
                break;
            default:
				break;
        }
    }
	return OK;
}


/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetdot11LocalPowerConstraint(INT8 *value)
{
    *value = dot11LocalPowerConstraint;
}
/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetMaxTxPowerLevel(INT8 *pMaxTxPowerLevel)
{
    *pMaxTxPowerLevel = whalMaxTxPowerLevel;
}
/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetTxPowerLevels(UINT32 band, UINT32 level, INT8 *pPowerLevel)
{
    if ((band != RADIO_BAND_2_4GHZ) && (band != RADIO_BAND_5GHZ))
    {
        *pPowerLevel = 0xff;    /* bad value */
        return;
    }
    if (level > NUM_OF_TX_POWER_LEVELS)
    {
        *pPowerLevel = 0xff;    /* bad value */
        return;
    }
    if (band == RADIO_BAND_2_4GHZ)
        *pPowerLevel = txPowerLevels_g[level-1];
    else
        *pPowerLevel = txPowerLevels_a[level-1];
}
/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetdot11CountryString(char* pCountryString)
{
    PAL_memCopy(pCountryString,dot11CountryString,COUNTRY_STRING_LENGTH);
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPrivateRegulatoryDomain(UINT8 *pRegDomain)
{
    *pRegDomain = dot11PrivateRegulatoryDomain;
}
/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetNumOfPrivateRanges(wlan_bandType bandType, UINT8 *pNumOfRanges)
{
    if (bandType == WLAN_BAND_A) 
        *pNumOfRanges = numOfRanges_802_11_a;
    else
        *pNumOfRanges = numOfRanges_802_11_b;
}


/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
STATUS wdrv_dbGetPrivateRangeElement(wlan_bandType bandType,
                                     UINT8 RangeIndex,
                                     tChannelRangeElementStruct *pRangeElementStruct)
{
    if (RangeIndex == 0)    /* index is a positive number */
        return ERROR;

    switch (bandType) 
    {
        case WLAN_BAND_A:
            if (RangeIndex > numOfRanges_802_11_a) 
                return ERROR;
            PAL_memCopy(pRangeElementStruct,&privateRangeElements_a[RangeIndex-1],sizeof(tChannelRangeElementStruct));
            break;

        case WLAN_BAND_B:
            if (RangeIndex > numOfRanges_802_11_b) 
                return ERROR;
            PAL_memCopy(pRangeElementStruct,&privateRangeElements_b[RangeIndex-1],sizeof(tChannelRangeElementStruct));
            break;
    }
    return OK;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
STATUS wdrv_dbGetRangeElement(wlan_bandType bandType, 
                              IntegerRegulatoryDomains_e RegDomain, 
                              UINT8         RangeIndex,
                              tChannelRangeElementStruct *pRangeElementStruct)
{
    if (RangeIndex == 0)    /* index is a positive number */
        return ERROR;

    switch (bandType) 
    {
        case WLAN_BAND_A:
            switch (RegDomain) 
            {
                case REG_DOMAIN_US:
                    if (RangeIndex > NUM_OF_RANGE_ELEMENTS_US)
                        return ERROR;
                    PAL_memCopy(pRangeElementStruct,
                           &RangeTable_a[0].pListElements[RangeIndex-1],
                           sizeof(tChannelRangeElementStruct));
                    break;

                case REG_DOMAIN_WORLD:
                    if (RangeIndex > NUM_OF_RANGE_ELEMENTS_WORLD)
                        return ERROR;
                    PAL_memCopy(pRangeElementStruct,
                           &RangeTable_a[1].pListElements[RangeIndex-1],
                           sizeof(tChannelRangeElementStruct));
                    break;

                default:
                    return ERROR;
            }
            break;
    
        case WLAN_BAND_B:
            switch (RegDomain) 
            {
                case REG_DOMAIN_FCC:
                    if (RangeIndex > 1)
                        return ERROR;
                    PAL_memCopy(pRangeElementStruct,
                           &RangeTable_b[0].pListElements[RangeIndex-1],
                           sizeof(tChannelRangeElementStruct));
                    break;

                case REG_DOMAIN_IC:
                    if (RangeIndex > 1)
                        return ERROR;
                    PAL_memCopy(pRangeElementStruct,
                           &RangeTable_b[1].pListElements[RangeIndex-1],
                           sizeof(tChannelRangeElementStruct));
                    break;

                case REG_DOMAIN_ETSI:
                    if (RangeIndex > 1)
                        return ERROR;
                    PAL_memCopy(pRangeElementStruct,
                           &RangeTable_b[2].pListElements[RangeIndex-1],
                           sizeof(tChannelRangeElementStruct));
                    break;

                case REG_DOMAIN_SPAIN:
                    if (RangeIndex > 1)
                        return ERROR;
                    PAL_memCopy(pRangeElementStruct,
                           &RangeTable_b[3].pListElements[RangeIndex-1],
                           sizeof(tChannelRangeElementStruct));
                    break;

                case REG_DOMAIN_FRANCE:
                    if (RangeIndex > 1)
                        return ERROR;
                    PAL_memCopy(pRangeElementStruct,
                           &RangeTable_b[4].pListElements[RangeIndex-1],
                           sizeof(tChannelRangeElementStruct));
                    break;

                case REG_DOMAIN_MKK:
                    if (RangeIndex > 1)
                        return ERROR;
                    PAL_memCopy(pRangeElementStruct,
                           &RangeTable_b[5].pListElements[RangeIndex-1],
                           sizeof(tChannelRangeElementStruct));
                    break;

                case REG_DOMAIN_MKK1:
                    if (RangeIndex > 2)
                        return ERROR;
                    PAL_memCopy(pRangeElementStruct,
                           &RangeTable_b[6].pListElements[RangeIndex-1],
                           sizeof(tChannelRangeElementStruct));
                    break;

                default:
                    return ERROR;
            }
            
            break;
    
        default:
            return ERROR;
    }
    return OK;
}


/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
UINT32 wdrv_dbGetNumberOfDomainElements(wlan_bandType bandType,
                                        IntegerRegulatoryDomains_e RegDomain)
{
    switch (bandType) 
    {
        case WLAN_BAND_A:
            switch (RegDomain) 
            {
                case REG_DOMAIN_US:
                    return NUM_OF_RANGE_ELEMENTS_US;

                case REG_DOMAIN_WORLD:
                    return NUM_OF_RANGE_ELEMENTS_WORLD;

                default:
                    break;
            }
            break;
    
        case WLAN_BAND_B:
            switch (RegDomain) 
            {
                case REG_DOMAIN_FCC:
                    return 1;

                case REG_DOMAIN_IC:
                    return 1;

                case REG_DOMAIN_ETSI:
                    return 1;

                case REG_DOMAIN_SPAIN:
                    return 1;

                case REG_DOMAIN_FRANCE:
                    return 1;

                case REG_DOMAIN_MKK:
                    return 1;

                case REG_DOMAIN_MKK1:
                    return 2;

                default:
                    break;
            }
            break;
    
        default:
            break;
    }
    return 0;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
STATUS wdrv_dbGetRegDomainInformation(wlan_bandType bandType,
                                      UINT8 *currentRegDomain, 
                                      UINT8 *numOfRanges,
                                      UINT8 *isPrivateDomain)
{
    *isPrivateDomain = FALSE;
    *currentRegDomain = dot11CurrentRegDomain;
    switch (bandType) 
    {
        case WLAN_BAND_A:
            switch (*currentRegDomain) 
            {
                case REG_DOMAIN_FCC:
                case REG_DOMAIN_US:
                    *currentRegDomain = REG_DOMAIN_US;
                    *numOfRanges = NUM_OF_RANGE_ELEMENTS_US;
                    break;

                case REG_DOMAIN_IC:
                case REG_DOMAIN_ETSI:
                case REG_DOMAIN_SPAIN:
                case REG_DOMAIN_FRANCE:
                case REG_DOMAIN_MKK:
                case REG_DOMAIN_MKK1:
                case REG_DOMAIN_WORLD:
                    *currentRegDomain = REG_DOMAIN_WORLD;
                    *numOfRanges = NUM_OF_RANGE_ELEMENTS_WORLD;
                    break;

                default:
                    *numOfRanges = numOfRanges_802_11_a;
                    *isPrivateDomain = TRUE;
                    break;
            }
            break;
    
        case WLAN_BAND_B:
            switch (*currentRegDomain) 
            {
                case REG_DOMAIN_FCC:
                    *numOfRanges = 1;
                    break;

                case REG_DOMAIN_IC:
                    *numOfRanges = 1;
                    break;

                case REG_DOMAIN_ETSI:
                    *numOfRanges = 1;
                    break;

                case REG_DOMAIN_SPAIN:
                    *numOfRanges = 1;
                    break;

                case REG_DOMAIN_FRANCE:
                    *numOfRanges = 1;
                    break;

                case REG_DOMAIN_MKK:
                    *numOfRanges = 1;
                    break;

                case REG_DOMAIN_MKK1:
                    *numOfRanges = 2;
                    break;

                default:
                    *numOfRanges = numOfRanges_802_11_b;
                    *isPrivateDomain = TRUE;
                    break;
            }
            break;
    
        default:
            return ERROR;
    }
    return OK;

}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
BOOL wdrv_dbIsStdRegDomain(UINT8 regDomain2Check)
{
    switch (regDomain2Check) 
    {
        case REG_DOMAIN_FCC:
        case REG_DOMAIN_IC:
        case REG_DOMAIN_ETSI:
        case REG_DOMAIN_SPAIN:
        case REG_DOMAIN_FRANCE:
        case REG_DOMAIN_MKK:
        case REG_DOMAIN_MKK1:
        case REG_DOMAIN_US:
        case REG_DOMAIN_WORLD:
            return TRUE;
    }
    return FALSE;
}
 

/* This routine fills the channels information in the given buffer with the given length    */
/* It returns ERROR if the buffer is too short (e.g. channel information was enlarged       */
/* In this case, the buffer should be enlarged hardcoded.                                   */
/* The information is organized according to the 802.11d SPEC requirements. The length      */
/* returns the actual buffer size that was filled with channel information.                 */
/* The channel information has a triplet structure, as follow:                              */
/* 1. 1st channel number                                                                    */
/* 2. # of channels                                                                         */
/* 3. max transmit Power Level                                                              */
/* The information is taken according to the current band & current regulatory domain       */
STATUS wdrv_dbFillChannelsInformation(wlan_bandType bandType, UINT8 *pBuffer, UINT8 *length)
{
    UINT32 size = 0;
    UINT8  currentRegDomain, numOfRanges, isPrivateDomain;
    int index;
    tChannelRangeElementStruct rangeElementStruct;
    UINT8 step, ch;

    wdrv_dbGetRegDomainInformation(bandType,&currentRegDomain,&numOfRanges,&isPrivateDomain);

    for (index = 1; index <= numOfRanges; index++) 
    {
        if (isPrivateDomain) 
            wdrv_dbGetPrivateRangeElement(bandType,index,&rangeElementStruct);
        else
            wdrv_dbGetRangeElement(bandType,
                                   (IntegerRegulatoryDomains_e)currentRegDomain,
                                   index,
                                   &rangeElementStruct);
        step = rangeElementStruct.stepOpt & 0xF;
        if (step > 1) 
        {
            for (ch=rangeElementStruct.startCh; ch <= rangeElementStruct.stopCh; ch+=step) 
            {   /* each channel is considered as one consecutive channel (step=1) */
                if (size+3 > *length)       /* bound checking */
                    return ERROR;
                pBuffer[size] = ch;
                size++;
                pBuffer[size] = 1;
                size++;
                pBuffer[size] = rangeElementStruct.powerLimit;
                size++;
            }
        }
        else
        {
            if (size+3 > *length)       /* bound checking */
                return ERROR;
            pBuffer[size] = rangeElementStruct.startCh;
            size++;
            /* no. of channels = last-first+1 */
            pBuffer[size] = rangeElementStruct.stopCh - rangeElementStruct.startCh + 1;
            size++;
            pBuffer[size] = rangeElementStruct.powerLimit;
            size++;
        }
    }

    *length = size;
    return OK;
}


/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION: 
    This routine tests if the channel is 
	allowed according to the band and the current regulatory domain 
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
BOOL IsChannelAllowed(UINT8 woneIndex, UINT8 channel)
{   
    UINT8 band, currentRegDomain, numOfRanges, isPrivateDomain, index;
    wlan_bandType bandType;
    tChannelRangeElementStruct rangeElementStruct;
    UINT8 step, ch;

	wdrv_dbGetBand(woneIndex, &band);

    bandType = (band == RADIO_BAND_2_4GHZ) ? WLAN_BAND_B : WLAN_BAND_A;
    
    wdrv_dbGetRegDomainInformation(bandType,&currentRegDomain,&numOfRanges,&isPrivateDomain);

    for (index = 1; index <= numOfRanges; index++) 
    {
        if (isPrivateDomain) 
            wdrv_dbGetPrivateRangeElement(bandType,index,&rangeElementStruct);
        else
            wdrv_dbGetRangeElement(bandType,
                                   (IntegerRegulatoryDomains_e)currentRegDomain,
                                   index,
                                   &rangeElementStruct);
        step = rangeElementStruct.stepOpt & 0xF;
        for (ch =  rangeElementStruct.startCh;
             ch <= rangeElementStruct.stopCh;
             ch += step) 
        {
            if (ch == channel)
                return TRUE;
        }
    }
    return FALSE;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
STATUS wdrv_dbRegDomainGetDefaultChannel(UINT8 regDomain,UINT8 *pChannel)
{
    switch (regDomain)
    {
        case REG_DOMAIN_FCC:
            *pChannel = 6;
			break;
        case REG_DOMAIN_IC:
            *pChannel = 1;
			break;
        case REG_DOMAIN_ETSI:
            *pChannel = 1;
			break;
        case REG_DOMAIN_SPAIN:
            *pChannel = 10;
			break;
        case REG_DOMAIN_FRANCE:
            *pChannel = 10;
			break;
        case REG_DOMAIN_MKK:
            *pChannel = 14;
			break;
        case REG_DOMAIN_MKK1:
            *pChannel = 1;
			break;
        case REG_DOMAIN_US:
            *pChannel = 36;
			break;
        case REG_DOMAIN_WORLD:
            *pChannel = 36;
			break;
		default:
			/* unrecognized RegDoamin */
			return NOK;
    }

    /* Note: Private Reg Domain is not handled */
    return OK;   
}



/*------------------------------ static routines -----------------------*/
/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
static void	wdrv_dbInitRangeTables(void)
{
    /* set hardcoded values into Range tables */
    /*              - A - band                */
#if 0   /* until the channels in the FW will be updated */
    RangeTable_a[0].regulatoryDomain = REG_DOMAIN_US;
    RangeTable_a[0].numOfRanges = NUM_OF_RANGE_ELEMENTS_US;
    RangeTable_a[0].pListElements = (tChannelRangeElementStruct *)PAL_memAlloc(0,sizeof(tChannelRangeElementStruct)*3,0,0);
    RangeTable_a[0].pListElements[0].startCh = 36;
    RangeTable_a[0].pListElements[0].stopCh = 48;
    RangeTable_a[0].pListElements[0].stepOpt = 4;
    RangeTable_a[0].pListElements[0].powerLimit = 16;
    RangeTable_a[0].pListElements[1].startCh = 52;
    RangeTable_a[0].pListElements[1].stopCh = 64;
    RangeTable_a[0].pListElements[1].stepOpt = 4;
    RangeTable_a[0].pListElements[1].powerLimit = 23;
    RangeTable_a[0].pListElements[2].startCh = 149;
    RangeTable_a[0].pListElements[2].stopCh = 161;
    RangeTable_a[0].pListElements[2].stepOpt = 4;
    RangeTable_a[0].pListElements[2].powerLimit = 29;

    RangeTable_a[1].regulatoryDomain = REG_DOMAIN_WORLD;
    RangeTable_a[1].numOfRanges = NUM_OF_RANGE_ELEMENTS_WORLD;
    RangeTable_a[1].pListElements = (tChannelRangeElementStruct *)PAL_memAlloc(0,sizeof(tChannelRangeElementStruct)*2,0,0);
    RangeTable_a[1].pListElements[0].startCh = 36;
    RangeTable_a[1].pListElements[0].stopCh  = 64;
    RangeTable_a[1].pListElements[0].stepOpt = 4;
    RangeTable_a[1].pListElements[0].powerLimit = 23;
    RangeTable_a[1].pListElements[1].startCh = 100;
    RangeTable_a[1].pListElements[1].stopCh  = 140;
    RangeTable_a[1].pListElements[1].stepOpt = 4;
    RangeTable_a[1].pListElements[1].powerLimit = 30;
#else

    RangeTable_a[0].regulatoryDomain = REG_DOMAIN_US;
    RangeTable_a[0].numOfRanges = NUM_OF_RANGE_ELEMENTS_US;
    if (!(RangeTable_a[0].pListElements = 
	(tChannelRangeElementStruct *)PAL_memAlloc(0,sizeof(tChannelRangeElementStruct)*2,0,0)))
	    return;
    RangeTable_a[0].pListElements[0].startCh = REG_DOMAIN_US_START_CH_0;
    RangeTable_a[0].pListElements[0].stopCh = REG_DOMAIN_US_STOP_CH_0;
    RangeTable_a[0].pListElements[0].stepOpt = REG_DOMAIN_US_STEP_OPT_0;
    RangeTable_a[0].pListElements[0].powerLimit = REG_DOMAIN_US_POWER_LIMIT_CH_0;
    RangeTable_a[0].pListElements[1].startCh = REG_DOMAIN_US_START_CH_1;
    RangeTable_a[0].pListElements[1].stopCh = REG_DOMAIN_US_STOP_CH_1;
    RangeTable_a[0].pListElements[1].stepOpt = REG_DOMAIN_US_STEP_OPT_1;
    RangeTable_a[0].pListElements[1].powerLimit = REG_DOMAIN_US_POWER_LIMIT_CH_1;

    RangeTable_a[1].regulatoryDomain = REG_DOMAIN_WORLD;
    RangeTable_a[1].numOfRanges = NUM_OF_RANGE_ELEMENTS_WORLD;
    if( !(RangeTable_a[1].pListElements = 
	(tChannelRangeElementStruct *)PAL_memAlloc(0,sizeof(tChannelRangeElementStruct)*3,0,0)))
	    return;
    RangeTable_a[1].pListElements[0].startCh = REG_DOMAIN_WORLD_START_CH_0;
    RangeTable_a[1].pListElements[0].stopCh  = REG_DOMAIN_WORLD_STOP_CH_0;
    RangeTable_a[1].pListElements[0].stepOpt = REG_DOMAIN_WORLD_STEP_OPT_0;
    RangeTable_a[1].pListElements[0].powerLimit = REG_DOMAIN_WORLD_POWER_LIMIT_CH_0;
    RangeTable_a[1].pListElements[1].startCh = REG_DOMAIN_WORLD_START_CH_1;
    RangeTable_a[1].pListElements[1].stopCh  = REG_DOMAIN_WORLD_STOP_CH_1;
    RangeTable_a[1].pListElements[1].stepOpt = REG_DOMAIN_WORLD_STEP_OPT_1;
    RangeTable_a[1].pListElements[1].powerLimit = REG_DOMAIN_WORLD_POWER_LIMIT_CH_1;
    RangeTable_a[1].pListElements[2].startCh = REG_DOMAIN_WORLD_START_CH_2;
    RangeTable_a[1].pListElements[2].stopCh = REG_DOMAIN_WORLD_STOP_CH_2;
    RangeTable_a[1].pListElements[2].stepOpt = REG_DOMAIN_WORLD_STEP_OPT_2;
    RangeTable_a[1].pListElements[2].powerLimit = REG_DOMAIN_WORLD_POWER_LIMIT_CH_2;

#endif

    /*              - G - band              */
    RangeTable_b[0].regulatoryDomain = REG_DOMAIN_FCC;
    RangeTable_b[0].numOfRanges = 1;
    if (!(RangeTable_b[0].pListElements =
	(tChannelRangeElementStruct *)PAL_memAlloc(0,sizeof(tChannelRangeElementStruct)*1,0,0)))
	    return;
    RangeTable_b[0].pListElements[0].startCh = REG_DOMAIN_FCC_START_CH_0;
    RangeTable_b[0].pListElements[0].stopCh  = REG_DOMAIN_FCC_STOP_CH_0;
    RangeTable_b[0].pListElements[0].stepOpt = REG_DOMAIN_FCC_STEP_OPT_0;
    RangeTable_b[0].pListElements[0].powerLimit = REG_DOMAIN_FCC_POWER_LIMIT_CH_0;

    RangeTable_b[1].regulatoryDomain = REG_DOMAIN_IC;
    RangeTable_b[1].numOfRanges = 1;
    if (!(RangeTable_b[1].pListElements =
	(tChannelRangeElementStruct *)PAL_memAlloc(0,sizeof(tChannelRangeElementStruct)*1,0,0)))
	    return;
    RangeTable_b[1].pListElements[0].startCh = REG_DOMAIN_IC_START_CH_0;
    RangeTable_b[1].pListElements[0].stopCh  = REG_DOMAIN_IC_STOP_CH_0;
    RangeTable_b[1].pListElements[0].stepOpt = REG_DOMAIN_IC_STEP_OPT_0;
    RangeTable_b[1].pListElements[0].powerLimit = REG_DOMAIN_IC_POWER_LIMIT_CH_0;

    RangeTable_b[2].regulatoryDomain = REG_DOMAIN_ETSI;
    RangeTable_b[2].numOfRanges = 1;
    if (!(RangeTable_b[2].pListElements =
	(tChannelRangeElementStruct *)PAL_memAlloc(0,sizeof(tChannelRangeElementStruct)*1,0,0)))
	    return;
    RangeTable_b[2].pListElements[0].startCh = REG_DOMAIN_ETSI_START_CH_0;
    RangeTable_b[2].pListElements[0].stopCh  = REG_DOMAIN_ETSI_STOP_CH_0;
    RangeTable_b[2].pListElements[0].stepOpt = REG_DOMAIN_ETSI_STEP_OPT_0;
    RangeTable_b[2].pListElements[0].powerLimit = REG_DOMAIN_ETSI_POWER_LIMIT_CH_0;

    RangeTable_b[3].regulatoryDomain = REG_DOMAIN_SPAIN;
    RangeTable_b[3].numOfRanges = 1;
    if (!(RangeTable_b[3].pListElements =
	(tChannelRangeElementStruct *)PAL_memAlloc(0,sizeof(tChannelRangeElementStruct)*1,0,0)))
	    return;
    RangeTable_b[3].pListElements[0].startCh = REG_DOMAIN_SPAIN_START_CH_0;
    RangeTable_b[3].pListElements[0].stopCh  = REG_DOMAIN_SPAIN_STOP_CH_0;
    RangeTable_b[3].pListElements[0].stepOpt = REG_DOMAIN_SPAIN_STEP_OPT_0;
    RangeTable_b[3].pListElements[0].powerLimit = REG_DOMAIN_SPAIN_POWER_LIMIT_CH_0;

    RangeTable_b[4].regulatoryDomain = REG_DOMAIN_FRANCE;
    RangeTable_b[4].numOfRanges = 1;
    if (!(RangeTable_b[4].pListElements =
	(tChannelRangeElementStruct *)PAL_memAlloc(0,sizeof(tChannelRangeElementStruct)*1,0,0)))
	    return;
    RangeTable_b[4].pListElements[0].startCh = REG_DOMAIN_FRANCE_START_CH_0;
    RangeTable_b[4].pListElements[0].stopCh  = REG_DOMAIN_FRANCE_STOP_CH_0;
    RangeTable_b[4].pListElements[0].stepOpt = REG_DOMAIN_FRANCE_STEP_OPT_0;
    RangeTable_b[4].pListElements[0].powerLimit = REG_DOMAIN_FRANCE_POWER_LIMIT_CH_0;

    RangeTable_b[5].regulatoryDomain = REG_DOMAIN_MKK;
    RangeTable_b[5].numOfRanges = 1;
    if (!(RangeTable_b[5].pListElements =
	(tChannelRangeElementStruct *)PAL_memAlloc(0,sizeof(tChannelRangeElementStruct)*1,0,0)))
	    return;
    RangeTable_b[5].pListElements[0].startCh = REG_DOMAIN_MKK_START_CH_0;
    RangeTable_b[5].pListElements[0].stopCh = REG_DOMAIN_MKK_STOP_CH_0;
    RangeTable_b[5].pListElements[0].stepOpt = REG_DOMAIN_MKK_STEP_OPT_0 | NO_OFDM_OPT;
    RangeTable_b[5].pListElements[0].powerLimit = REG_DOMAIN_MKK_POWER_LIMIT_CH_0;

    RangeTable_b[6].regulatoryDomain = REG_DOMAIN_MKK1;
    RangeTable_b[6].numOfRanges = 2;
    if (!(RangeTable_b[6].pListElements =
	(tChannelRangeElementStruct *)PAL_memAlloc(0,sizeof(tChannelRangeElementStruct)*2,0,0)))
	    return;
    RangeTable_b[6].pListElements[0].startCh = REG_DOMAIN_MKK1_START_CH_0;
    RangeTable_b[6].pListElements[0].stopCh  = REG_DOMAIN_MKK1_STOP_CH_0;
    RangeTable_b[6].pListElements[0].stepOpt = REG_DOMAIN_MKK1_STEP_OPT_0;
    RangeTable_b[6].pListElements[0].powerLimit = REG_DOMAIN_MKK1_POWER_LIMIT_CH_0;
    RangeTable_b[6].pListElements[1].startCh = REG_DOMAIN_MKK1_START_CH_1;
    RangeTable_b[6].pListElements[1].stopCh  = REG_DOMAIN_MKK1_STOP_CH_1;
    RangeTable_b[6].pListElements[1].stepOpt = REG_DOMAIN_MKK1_STEP_OPT_1 | NO_OFDM_OPT;
    RangeTable_b[6].pListElements[1].powerLimit = REG_DOMAIN_MKK1_POWER_LIMIT_CH_1;

}
