/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************/
/*																			*/
/*		MODULE:		mgmtParser.c											*/
/*		PURPOSE:	Management parser object								*/
/*																			*/
/****************************************************************************/
#include "mgmtSm.h"
#include "mainsm.h"
#include "memMngr.h"
#include "mgmtParser.h"
#include "802_11Defs.h"
#include "mgmtSwap.h"
#include "mgmtSm.h"
#include "drvDbHash.h"
#include "drvDbMain.h"
#include "utils.h"
#include "tx.h"
#include "pal.h"

#define EV_MESSAGE_LEN          128

/* 4x Protocol Version Field macros */
#define PROTOCOL_VERSION_OK(protoVerSTA, protoVerAP) (protoVerSTA&0xF0) == (protoVerAP&0xF0)
#define FEATURE_SET_OK(FeatureSetSTA, f_set) (unsigned char)(FeatureSetSTA&0xF) <= f_set

/* information elements macros */
#define ELE_ID(x)	x->eleId
#define ELE_LEN(x)	x->eleLen

#if SM_DEBUG
UINT32 parserIndexError;
#endif

/* Comparing OUI, DOT11_OUI_LEN = 3 */
#define CMPOUIN(a,b)  ( XORUST1((a),(b),0) | XORUST1((a),(b),1) | XORUST1((a),(b),2))
#define CMPOUI(a,b)   ( CMPOUIN((a),(b)) == 0)

/*-----------------------------------------------------------*/
/*         Global Functions                                   */
/*-----------------------------------------------------------*/

/*-----------------------------------------------------------*/
/*         Local Variables                                   */
/*-----------------------------------------------------------*/
/* de-fragmentation buffer */
static dot11_mgmtFrame_T		deFragMmpdu;

struct  
{
	UINT32 totalCount;
	UINT32 authCount;
	UINT32 deauthCount;
	UINT32 assocCount;
	UINT32 reassocCount;
	UINT32 disassocCount;
} mgmtCounters;

/*-----------------------------------------------------------*/
/*         Local Functions                                   */
/*-----------------------------------------------------------*/
static STATUS	mmpduCheckValidity(mem_MSDU_T *msduPtr);
static STATUS	mmpduCheckHeader(dot11_mgmtHeader_T *headerPtr, UINT16 index, BOOL *wepOn);

static STATUS	authParse(char *bodyPtr, UINT32 totalLen, UINT32 *protoEvent, dot11_auth_T *authPtr);
static STATUS	deAuthParse(char *bodyPtr, UINT32 totalLen, UINT32 *protoEvent, dot11_deAuth_T *deAuthPtr);
static STATUS	assocReqParse(char *bodyPtr, UINT32 totalLen, UINT32 *protoEvent, dot11_assocReq_T *assocReqPtr);
static STATUS	reAssocReqParse(char *bodyPtr, UINT32 totalLen, UINT32 *protoEvent, dot11_reAssocReq_T *reAssocReqPtr);
static STATUS	DisAssocParse(char *bodyPtr, UINT32 totalLen, UINT32 *protoEvent, dot11_disAssoc_T *disAssocPtr);

/*************************************************************************
 *                        wdrv_mgmtMmpduParse                              *
 *************************************************************************
DESCRIPTION: This function parses a mgmt frame                 
                                                                                                   
INPUT:      msduPtr -MSDU pointer
			index - Station index

OUTPUT:		protoEvent - Protocol event corresponding to the message subtype
			protoPtr - pointer to the structure that contains the infromation received in the frame.

RETURN:     OK - parsing successful
			NOK - parsing failed

************************************************************************/
STATUS wdrv_mgmtMmpduParse(mem_MSDU_T *msduPtr, UINT16 index, UINT32 *protoEvent, void *protoPtr)
{
	int			rc;
	dot11_mgmtFrame_T *mmpduPtr;
	UINT32		totalLen;
	BOOL	wepOn;

	rc = mmpduCheckValidity(msduPtr);
	if (rc != OK)
		return NOK;
	
	if (wdrv_mgmtLogEnable())
		wdrv_mgmtPrintFrame(msduPtr);

	/* If fragmented, perform de-fragmentation before continue parsing */
	if (wlan_memMngrMsduIsFragmented(msduPtr))
	{
		wlan_memMngrMsduReassemble(msduPtr, (char *)&deFragMmpdu);
		mmpduPtr = &deFragMmpdu; 
	}
	else
		mmpduPtr = (dot11_mgmtFrame_T *)(msduPtr->firstBDPtr->data + msduPtr->firstBDPtr->dataOffset); 

	/* swap */
	wdrv_mgmtSwap(mmpduPtr);
	
	totalLen = msduPtr->dataLen - sizeof(dot11_mgmtHeader_T);

	/* verify that the header is OK */
	rc = mmpduCheckHeader(&mmpduPtr->hdr, index, &wepOn);
	if (rc != OK)
		return NOK;

	
	/* continue parsing according to the frame sub type */
	switch (mmpduPtr->hdr.fc  & DOT11_FC_SUB_MASK)
	{
		case DOT11_FC_AUTH:
			rc = authParse((char *)&mmpduPtr->body, totalLen, protoEvent, (dot11_auth_T *)protoPtr);
			if (rc != OK)
				return NOK;
			((dot11_auth_T *)protoPtr)->wepOn = wepOn;	/* used in auth#3 frames in the system check phase. */
			mgmtCounters.authCount++;
			mgmtCounters.totalCount++;
			return OK;
			break;

		case DOT11_FC_DEAUTH:
			rc = deAuthParse((char *)&mmpduPtr->body, totalLen, protoEvent, (dot11_deAuth_T *)protoPtr);
			if (rc != OK)
				return NOK;
			mgmtCounters.deauthCount++;
			mgmtCounters.totalCount++;
			return OK;
			break;

		case DOT11_FC_ASSOC_REQ:
			rc = assocReqParse((char *)&mmpduPtr->body, totalLen, protoEvent, (dot11_assocReq_T *)protoPtr);
			if (rc != OK)
				return NOK;
			mgmtCounters.assocCount++;
			mgmtCounters.totalCount++;
			return OK;
			break;

		case DOT11_FC_REASSOC_REQ:
			rc = reAssocReqParse((char *)&mmpduPtr->body, totalLen, protoEvent, (dot11_reAssocReq_T *)protoPtr);
			if (rc != OK)
				return NOK;
			mgmtCounters.reassocCount++;
			mgmtCounters.totalCount++;
			return OK;
			break;

		case DOT11_FC_DISASSOC:
			rc = DisAssocParse((char *)&mmpduPtr->body, totalLen, protoEvent, (dot11_disAssoc_T *)protoPtr);
			if (rc != OK)
				return NOK;
			mgmtCounters.disassocCount++;
			mgmtCounters.totalCount++;
			return OK;
			break;

		default:
			if(REPORT_LEVEL(RL_MGMT)) 
				wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error filtering mmpdu, unexpected frame sub type = %d\n", mmpduPtr->hdr.fc  & DOT11_FC_SUB_MASK);
			return NOK;
			break;
	}
	
}

/*************************************************************************
 *                        mmpduCheckHeader                              *
 *************************************************************************
DESCRIPTION: This function verify a mgmt frame header                 
                                                                                                   
INPUT:      Header pointer
			Station index

OUTPUT:      

RETURN:		OK - header ok
			NOK - header not ok	

************************************************************************/
static STATUS mmpduCheckHeader(dot11_mgmtHeader_T *headerPtr, UINT16 index, BOOL *wepOn)
{
	UINT16	 protocolVersion;
	char	mac[MAC_ADDR_LEN];
	
	/* TODO: maybe the next 4 checking need to be removed.*/
	/* check protocol version */
	if ((protocolVersion = headerPtr->fc & DOT11_FC_PROT_VERSION_MASK) != DOT11_FC_PROT_VERSION)
	{
		if(REPORT_LEVEL(RL_MGMT)) 
			wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error filtering mmpdu, bad protocol version = %d\n", protocolVersion);

		return NOK;
	}

	/* verify that dst MAC is equal to AP MAC */
	wdrv_dbGetOperationMacAddress(mac);
	
	if (MACNCMP( mac, headerPtr->DA.addr))
	{
      /* mac != headerPtr->DA.addr */
		if(REPORT_LEVEL(RL_MGMT)) 
			wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error filtering mmpdu, dst MAC address error.\n");

		return NOK;
	}

	/* verify that src MAC is equal to table MAC */
	wdrv_dbGetStaMacAddress(index, mac);

	if (MACNCMP( mac, headerPtr->SA.addr))
	{
      /* mac != headerPtr->SA.addr */
		if(REPORT_LEVEL(RL_MGMT)) 
			wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error filtering mmpdu, source MAC address error\n");
#if SM_DEBUG
		parserIndexError++;
#endif

		return NOK;
	}

	if IS_WEP_ON(headerPtr->fc)
		*wepOn = TRUE;
	else
		*wepOn = FALSE;

	if(REPORT_LEVEL(RL_MGMT)) 
		wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Header OK\n");

	return OK;
}

/*************************************************************************
 *                        authParse			                             *
 *************************************************************************
DESCRIPTION: This function parses an authentication frame                 
                                                                                                   
INPUT:      bodyPtr - pointer to the frame body
			totalLen - frame data total len

OUTPUT:		protoEvent - protocol event corresponds to the frame subtype
			authPtr - a structure contsins the information received in the frame.    

RETURN:     OK - parsing successful
			NOK - parsing failed

************************************************************************/
static STATUS authParse(char *bodyPtr, UINT32 totalLen, UINT32 *protoEvent, dot11_auth_T *authPtr)
{
	dot11_eleHdr_T	*eleHdrPtr;
	UINT32				currLen = 0;

	/* fixed fields first */
	(char *)(authPtr->algoNumber) = bodyPtr;	
	bodyPtr += FIX_FIELD_LEN;
	if((currLen += FIX_FIELD_LEN) > totalLen)
		return NOK;

	(char *)(authPtr->transSeqNumber) = bodyPtr;	
	if ((*(authPtr->transSeqNumber) != 1) && (*(authPtr->transSeqNumber) != 3))
	{
		if(REPORT_LEVEL(RL_MGMT)) 
			wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error parsing auth, bad sequence number = %d\n",	*(authPtr->transSeqNumber));
		return NOK;
	}
	bodyPtr += FIX_FIELD_LEN;
	if((currLen += FIX_FIELD_LEN) > totalLen)
		return NOK;
	
	/* jump over the status */
	bodyPtr += FIX_FIELD_LEN;
	if((currLen += FIX_FIELD_LEN) > totalLen)
		return NOK;

	if(*(authPtr->transSeqNumber) == 1)
	{
		*protoEvent = EV_11_AUTH_1;
		return OK;
	}

	/* now information elements */
	(char *)eleHdrPtr = bodyPtr;
	
	if (ELE_ID(eleHdrPtr) != DOT11_CHALLENGE_TEXT_ELE_ID)
	{
		if(REPORT_LEVEL(RL_MGMT)) 
			wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error parsing auth, Unexpected eleId = %d\n", ELE_ID(eleHdrPtr));
		return NOK;
	}
	
	if (ELE_LEN(eleHdrPtr) != DOT11_CHALLENGE_TEXT_MAX)
	{
		if(REPORT_LEVEL(RL_MGMT)) 
			wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error parsing auth, Invalid eleLen = %d, eleId = %d\n", ELE_LEN(eleHdrPtr), ELE_ID(eleHdrPtr));
		return NOK;
	}
	
	(dot11_eleHdr_T *)(authPtr->challenge) = eleHdrPtr;	
	if((currLen += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T)) > totalLen)
		return NOK;
	*protoEvent = EV_11_AUTH_3;

	return OK;
}

/*************************************************************************
 *                        deAuthParse			                             *
 *************************************************************************
DESCRIPTION: This function parses a DEauthentication frame                 
                                                                                                   
INPUT:      bodyPtr - pointer to the frame body
			totalLen - frame data total len

OUTPUT:		 protoEvent - protocol event corresponds to the frame subtype
			 deAuthPtr - a structure contsins the information received in the frame.    

RETURN:		OK - parsing successful
			NOK -parsing failed	

************************************************************************/
static STATUS deAuthParse(char *bodyPtr, UINT32 totalLen, UINT32 *protoEvent, dot11_deAuth_T *deAuthPtr)
{
	UINT32				currLen = 0;

	(char *)(deAuthPtr->reason) = bodyPtr;
	*protoEvent = EV_11_DE_AUTH;
	if((currLen += FIX_FIELD_LEN) > totalLen)
		return NOK;

	return OK;
}

/*************************************************************************
 *                        assocReqParse			                         *
 *************************************************************************
DESCRIPTION: This function parses an association request frame                 
                                                                                                   
INPUT:      bodyPtr - pointer to the frame body
			totalLen - frame data total len

OUTPUT:		 protoEvent - protocol event corresponds to the frame subtype
			 assocReqPtr - a structure contsins the information received in the frame.    

RETURN:		OK - parsing successful
			NOK -parsing failed	

************************************************************************/
static STATUS assocReqParse(char *bodyPtr, UINT32 totalLen, UINT32 *protoEvent, dot11_assocReq_T *assocReqPtr)
{
	UINT32 currLen = 0;
	dot11_eleHdr_T *eleHdrPtr;
	dot11_TIcap_T apTIcap;
	char *capPtr;
	UINT8 MyFeatureSet;
	int  curr;
	UINT16 len,cap_type;
	BOOL  FourXen;
    UINT16 staMaxConcatSize,apMaxConcatSize;
	char				msOui[3] = {0x00, 0x50, 0xF2};
	
	/* fixed fields first */
	(char *)(assocReqPtr->capabilities) = bodyPtr;
	bodyPtr += FIX_FIELD_LEN;
	if((currLen += FIX_FIELD_LEN) > totalLen)
		return NOK;

	(char *)(assocReqPtr->listenInterval) = bodyPtr;
	bodyPtr += FIX_FIELD_LEN;
	if((currLen += FIX_FIELD_LEN) > totalLen)
		return NOK;
	

	/* Must be here, in order to reset this varlable. */
	/* It may be still TRUE or FALSE from the parsing of the previous frame */
	assocReqPtr->rsnIe = NULL;
	assocReqPtr->fourXSupport = 0;
	assocReqPtr->TIprotectionSupport = FALSE;
	assocReqPtr->TIouiSupport = FALSE;
	wdrv_dbGetFeatureSet(&MyFeatureSet);
	assocReqPtr->CommonFeatureSet = MyFeatureSet;
	assocReqPtr->ConcatMaxSize = (UINT16)wdrv_txGetMaxConcatSize();

	/* Init all IE pointers to NULL */
	assocReqPtr->SSID = NULL;
	assocReqPtr->supportedRates = NULL;
	assocReqPtr->extSupportedRates = NULL;


	/* now information elements */
	(char *)eleHdrPtr = bodyPtr;

	while (currLen < totalLen /*+ sizeof(dot11_eleHdr_T) <= totalLen*/)
	{
		switch (ELE_ID(eleHdrPtr))
		{
			case DOT11_SSID_ELE_ID:
				if (ELE_LEN(eleHdrPtr) > DOT11_SSID_MAX)
				{
					if(REPORT_LEVEL(RL_MGMT)) 
						wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error parsing assoc req, Invalid eleLen = %d, eleId = %d\n", ELE_LEN(eleHdrPtr), ELE_ID(eleHdrPtr));
					return NOK;
				}
				(dot11_eleHdr_T *)(assocReqPtr->SSID) = eleHdrPtr;
				currLen += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				(char *)eleHdrPtr += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				break;

			case DOT11_SUPPORTED_RATES_ELE_ID:
				if (ELE_LEN(eleHdrPtr) > DOT11_SUPPORTED_RATES_MAX)
				{
					if(REPORT_LEVEL(RL_MGMT)) 
						wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error parsing assoc req, Invalid eleLen = %d, eleId = %d\n", ELE_LEN(eleHdrPtr), ELE_ID(eleHdrPtr));
					return NOK;
				}
				(dot11_eleHdr_T *)(assocReqPtr->supportedRates) = eleHdrPtr;
				currLen += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				(char *)eleHdrPtr += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				break;

			case DOT11_EXT_SUPPORTED_RATES_ELE_ID:
				if (ELE_LEN(eleHdrPtr) > DOT11_EXT_SUPPORTED_RATES_MAX)
				{
					if(REPORT_LEVEL(RL_MGMT)) 
						wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error parsing assoc req, Invalid eleLen = %d, eleId = %d\n", ELE_LEN(eleHdrPtr), ELE_ID(eleHdrPtr));
					return NOK;
				}
				(dot11_eleHdr_T *)(assocReqPtr->extSupportedRates) = eleHdrPtr;
				currLen += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				(char *)eleHdrPtr += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				break;

		case DOT11_TI_CAP_ELE_ID:
				if (ELE_LEN(eleHdrPtr) > DOT11_TI_CAP_MAX_LEN)
				{
					if(REPORT_LEVEL(RL_MGMT)) 
						wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error parsing assoc req, Invalid eleLen = %d, eleId = %d\n", ELE_LEN(eleHdrPtr), ELE_ID(eleHdrPtr));
					return NOK;
				}

				/* Now, if the info element is with TI OUI and currently supported version, */
				/* set TIouiSupport to TRUE. Otherwise, set FALSE */
				wdrv_dbGetTIInfoElement(apTIcap.cap, &(apTIcap.hdr.eleLen));

				capPtr = (char *)eleHdrPtr;
				capPtr += 2;

				/* compare OUI field only */
				if( CMPOUI( apTIcap.cap, capPtr))
				{
					/* check the common supported feature set */
					if((capPtr[DOT11_OUI_LEN] & 0xF) <= MyFeatureSet)
						assocReqPtr->CommonFeatureSet = capPtr[DOT11_OUI_LEN] & 0xF;
					else /* if STA sends FeatureSet greater than AP's, ignore it */
					{
						currLen += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
						(char *)eleHdrPtr += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
						break;
					}

					if(assocReqPtr->CommonFeatureSet == 0)
					{
						wdrv_dbGet4xEnable(&FourXen);
						if(FourXen)
						{							
							assocReqPtr->TIouiSupport = TRUE;
							assocReqPtr->fourXSupport |= FOURX_SUPPORT_CONCAT_MASK;
							assocReqPtr->fourXSupport |= FOURX_SUPPORT_CW_MIN_MASK;
							assocReqPtr->fourXSupport |= FOURX_SUPPORT_ACK_EMUL_MASK;
						}
					}
					else /* may be 4x and Trick Packet support should be checked here */
						assocReqPtr->TIouiSupport = TRUE;


					/* now parse the capabilities */
					if(assocReqPtr->CommonFeatureSet > 0)
					{
						curr = DOT11_OUI_LEN+1;

						while(curr < ELE_LEN(eleHdrPtr))
						{
							cap_type = (UINT16) (capPtr[curr] | (capPtr[curr+1] << 8));
							switch(cap_type)
							{
							case TI_CAP_4X_CONCATENATION:							
								assocReqPtr->fourXSupport |= FOURX_SUPPORT_CONCAT_MASK;
                                len = (UINT16) (capPtr[curr+sizeof(UINT16)] | (capPtr[curr+sizeof(UINT16)+1] << 8));
                                apMaxConcatSize = (UINT16)wdrv_txGetMaxConcatSize();
                                PAL_memCopy(&staMaxConcatSize,&capPtr[curr+sizeof(UINT16)+sizeof(UINT16)],sizeof(UINT16));
                                staMaxConcatSize = ENDIAN_HANDLE_WORD(staMaxConcatSize);
                                if(staMaxConcatSize <= apMaxConcatSize)
                                    assocReqPtr->ConcatMaxSize = staMaxConcatSize;
								else
									assocReqPtr->ConcatMaxSize = apMaxConcatSize;
                                curr += sizeof(UINT16) /* type field */ + sizeof(UINT16) /* len field */ + len /* val field */;

								break;
							case TI_CAP_4X_CONT_WINDOW:
								/* currently unsupported */
                                len = (UINT16) (capPtr[curr+sizeof(UINT16)] | (capPtr[curr+sizeof(UINT16)+1] << 8));
                                curr += sizeof(UINT16) /* type field */ + sizeof(UINT16) /* len field */ + len /* val field */;
								break;
							case TI_CAP_4X_TCP_ACK_EMUL:
								assocReqPtr->fourXSupport |= FOURX_SUPPORT_ACK_EMUL_MASK;
                                len = (UINT16) (capPtr[curr+sizeof(UINT16)] | (capPtr[curr+sizeof(UINT16)+1] << 8));
                                curr += sizeof(UINT16) /* type field */ + sizeof(UINT16) /* len field */ + len /* val field */;
								break;
							case TI_CAP_TRICK_PACKET_ERP:
								assocReqPtr->TIprotectionSupport = TRUE;
                                len = (UINT16) (capPtr[curr+sizeof(UINT16)] | (capPtr[curr+sizeof(UINT16)+1] << 8));
                                curr += sizeof(UINT16) /* type field */ + sizeof(UINT16) /* len field */ + len /* val field */;
								break;
							default:
								if(REPORT_LEVEL(RL_MGMT))
									printf("unrecognized capability in TI IE: %d\n",cap_type);
								return NOK;
							}
						}
					}
				}				

				else if (CMPOUI(capPtr, msOui))		/* WPA IE */
				{
					if (ELE_LEN(eleHdrPtr) > DOT11_RSNE_MAX)
					{
						if(REPORT_LEVEL(RL_MGMT)) 
							wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error parsing assoc req, Invalid eleLen = %d, eleId = %d\n", ELE_LEN(eleHdrPtr), ELE_ID(eleHdrPtr));
						return NOK;
					}
					(dot11_eleHdr_T *)(assocReqPtr->rsnIe) = eleHdrPtr;
				}

			currLen += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
			(char *)eleHdrPtr += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
			break;

			default:
				/* if IE is not recognized, just print them but do not fail */
				if(REPORT_LEVEL(RL_MGMT)) 
					wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error parsing assoc req, Unexpected eleId = %d\n", ELE_ID(eleHdrPtr));
            /* skip the unrecognized element */
			   currLen += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
			   (char *)eleHdrPtr += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);

				break;
		}

		if (currLen > totalLen)
		{
			return NOK;
		}
	}

	/* Check if received the required information elements */
	/* note that extSupportedRates is not a require. */
	if ((assocReqPtr->SSID == NULL) || (assocReqPtr->supportedRates == NULL))
	{
		return NOK;
	}

	*protoEvent = EV_11_ASSOC_REQ;
	return OK;
}

/*************************************************************************
 *                        reAssocReqParse			                         *
 *************************************************************************
DESCRIPTION: This function parses a Reassociation request frame                 
                                                                                                   
INPUT:      bodyPtr - pointer to the frame body
			totalLen - frame data total len

OUTPUT:		 protoEvent - protocol event corresponds to the frame subtype
			 reAssocReqPtr - a structure contsins the information received in the frame.    

RETURN:		OK - parsing successful
			NOK -parsing failed	

************************************************************************/
static STATUS reAssocReqParse(char *bodyPtr, UINT32 totalLen, UINT32 *protoEvent, dot11_reAssocReq_T *reAssocReqPtr)
{
	UINT32 currLen = 0;
	dot11_eleHdr_T *eleHdrPtr;
	dot11_TIcap_T apTIcap;
	char *capPtr;
	UINT8 MyFeatureSet;
	int  curr;
	UINT16 len,cap_type;
	BOOL  FourXen;
    UINT16 staMaxConcatSize,apMaxConcatSize;
	char				msOui[3] = {0x00, 0x50, 0xF2};

	/* fixed fields first */
	(char *)(reAssocReqPtr->capabilities) = bodyPtr;
	bodyPtr += FIX_FIELD_LEN;
	if((currLen += FIX_FIELD_LEN) > totalLen)
		return NOK;

	(char *)(reAssocReqPtr->listenInterval) = bodyPtr;
	bodyPtr += FIX_FIELD_LEN;
	if((currLen += FIX_FIELD_LEN) > totalLen)
		return NOK;

	reAssocReqPtr->currentAP = bodyPtr;
	bodyPtr += MAC_ADDR_LEN;
	if((currLen += MAC_ADDR_LEN) > totalLen)
		return NOK;
 	

	/* Must be here, in order to reset this varlable. */
	/* It may be still TRUE or FALSE from the parsing of the previous frame */
	reAssocReqPtr->rsnIe = NULL;
	reAssocReqPtr->fourXSupport = 0;
	reAssocReqPtr->TIprotectionSupport = FALSE;
	reAssocReqPtr->TIouiSupport = FALSE;
	wdrv_dbGetFeatureSet(&MyFeatureSet);
	reAssocReqPtr->CommonFeatureSet = MyFeatureSet;
	reAssocReqPtr->ConcatMaxSize = (UINT16)wdrv_txGetMaxConcatSize();


	/* Init all IE pointers to NULL */
	reAssocReqPtr->SSID = NULL;
	reAssocReqPtr->supportedRates = NULL;
	reAssocReqPtr->extSupportedRates = NULL;

	/* now information elements */
	(char *)eleHdrPtr = bodyPtr;

	while (currLen < totalLen /*+ sizeof(dot11_eleHdr_T) <= totalLen*/)
	{
		switch (ELE_ID(eleHdrPtr))
		{
			case DOT11_SSID_ELE_ID:
				if (ELE_LEN(eleHdrPtr) > DOT11_SSID_MAX)
				{
					if(REPORT_LEVEL(RL_MGMT)) 
						wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error parsing assoc req, Invalid eleLen = %d, eleId = %d\n", ELE_LEN(eleHdrPtr), ELE_ID(eleHdrPtr));
					return NOK;
				}
				(dot11_eleHdr_T *)(reAssocReqPtr->SSID) = eleHdrPtr;
				currLen += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				(char *)eleHdrPtr += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				break;

			case DOT11_SUPPORTED_RATES_ELE_ID:
				if (ELE_LEN(eleHdrPtr) > DOT11_SUPPORTED_RATES_MAX)
				{
					if(REPORT_LEVEL(RL_MGMT)) 
						wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error parsing assoc req, Invalid eleLen = %d, eleId = %d\n", ELE_LEN(eleHdrPtr), ELE_ID(eleHdrPtr));
					return NOK;
				}
				(dot11_eleHdr_T *)(reAssocReqPtr->supportedRates) = eleHdrPtr;
				currLen += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				(char *)eleHdrPtr += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				break;

			case DOT11_EXT_SUPPORTED_RATES_ELE_ID:
				if (ELE_LEN(eleHdrPtr) > DOT11_EXT_SUPPORTED_RATES_MAX)
				{
					if(REPORT_LEVEL(RL_MGMT)) 
						wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error parsing assoc req, Invalid eleLen = %d, eleId = %d\n", ELE_LEN(eleHdrPtr), ELE_ID(eleHdrPtr));
					return NOK;
				}
				(dot11_eleHdr_T *)(reAssocReqPtr->extSupportedRates) = eleHdrPtr;
				currLen += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				(char *)eleHdrPtr += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				break;

			case DOT11_TI_CAP_ELE_ID:
				if (ELE_LEN(eleHdrPtr) > DOT11_TI_CAP_MAX_LEN)
				{
					if(REPORT_LEVEL(RL_MGMT)) 
						wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error parsing assoc req, Invalid eleLen = %d, eleId = %d\n", ELE_LEN(eleHdrPtr), ELE_ID(eleHdrPtr));
					return NOK;
				}
				/* Now, if the info element is with TI OUI and currently supported version, */
				/* set fourXSupport to TRUE. Otherwise, set FALSE */
				wdrv_dbGetTIInfoElement(apTIcap.cap, &(apTIcap.hdr.eleLen));

				capPtr = (char *)eleHdrPtr;
				capPtr += 2;

				/* compare OUI field only */
				if( CMPOUI( apTIcap.cap, capPtr))
				{
					/* check the common supported feature set */
					if((capPtr[DOT11_OUI_LEN] & 0xF) < MyFeatureSet)
						reAssocReqPtr->CommonFeatureSet = capPtr[DOT11_OUI_LEN] & 0xF;

					if(reAssocReqPtr->CommonFeatureSet == 0)
					{
						wdrv_dbGet4xEnable(&FourXen);
						if(FourXen)
						{
							reAssocReqPtr->TIouiSupport = TRUE;
							reAssocReqPtr->fourXSupport |= FOURX_SUPPORT_CONCAT_MASK;
							reAssocReqPtr->fourXSupport |= FOURX_SUPPORT_CW_MIN_MASK;
							reAssocReqPtr->fourXSupport |= FOURX_SUPPORT_ACK_EMUL_MASK;
						}
					}
					else /* may be 4x and Trick Packet support should be checked here */
						reAssocReqPtr->TIouiSupport = TRUE;


					/* now parse the capabilities */
					if(reAssocReqPtr->CommonFeatureSet > 0)
					{
						curr = DOT11_OUI_LEN+1;

						while(curr < ELE_LEN(eleHdrPtr))
						{
							cap_type = (UINT16) capPtr[curr];
							switch(cap_type)
							{
							case TI_CAP_4X_CONCATENATION:							
								reAssocReqPtr->fourXSupport |= FOURX_SUPPORT_CONCAT_MASK;
                                len = (UINT16) (capPtr[curr+sizeof(UINT16)] | (capPtr[curr+sizeof(UINT16)+1] << 8));
                                apMaxConcatSize = (UINT16)wdrv_txGetMaxConcatSize();
                                PAL_memCopy(&staMaxConcatSize,&capPtr[curr+sizeof(UINT16)+sizeof(UINT16)],sizeof(UINT16));
                                staMaxConcatSize = ENDIAN_HANDLE_WORD(staMaxConcatSize);
                                if(staMaxConcatSize <= apMaxConcatSize)
                                    reAssocReqPtr->ConcatMaxSize = staMaxConcatSize;
								else
									reAssocReqPtr->ConcatMaxSize = apMaxConcatSize;
                                curr += sizeof(UINT16) /* type field */ + sizeof(UINT16) /* len field */ + len /* val field */;

								break;
							case TI_CAP_4X_CONT_WINDOW:
								/* currently unsupported */
                                len = (UINT16) (capPtr[curr+sizeof(UINT16)] | (capPtr[curr+sizeof(UINT16)+1] << 8));
                                curr += sizeof(UINT16) /* type field */ + sizeof(UINT16) /* len field */ + len /* val field */;
								break;
							case TI_CAP_4X_TCP_ACK_EMUL:
								reAssocReqPtr->fourXSupport |= FOURX_SUPPORT_ACK_EMUL_MASK;
                                len = (UINT16) (capPtr[curr+sizeof(UINT16)] | (capPtr[curr+sizeof(UINT16)+1] << 8));
                                curr += sizeof(UINT16) /* type field */ + sizeof(UINT16) /* len field */ + len /* val field */;
								break;
							case TI_CAP_TRICK_PACKET_ERP:
								reAssocReqPtr->TIprotectionSupport = TRUE;
                                len = (UINT16) (capPtr[curr+sizeof(UINT16)] | (capPtr[curr+sizeof(UINT16)+1] << 8));
                                curr += sizeof(UINT16) /* type field */ + sizeof(UINT16) /* len field */ + len /* val field */;
								break;
							default:
								if(REPORT_LEVEL(RL_MGMT))
									printf("unrecognized capability in TI IE: %d\n",cap_type);
								return NOK;
							}
						}
					}
				}

				else if ( CMPOUI( capPtr, msOui))		/* WPA IE */
				{
					if (ELE_LEN(eleHdrPtr) > DOT11_RSNE_MAX)
					{
						if(REPORT_LEVEL(RL_MGMT)) 
							wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error parsing assoc req, Invalid eleLen = %d, eleId = %d\n", ELE_LEN(eleHdrPtr), ELE_ID(eleHdrPtr));
						return NOK;
					}
					(dot11_eleHdr_T *)(reAssocReqPtr->rsnIe) = eleHdrPtr;
				}

				currLen += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				(char *)eleHdrPtr += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				break;

			default:
				/* if IE is not recognized, just print them but do not fail */
				if(REPORT_LEVEL(RL_MGMT)) 
					wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error parsing reAssoc req, Unexpected eleId = %d\n", ELE_ID(eleHdrPtr));
				/* if IE is not recognized, just print them but do not fail */
            /* skip the unrecognized element */
			   currLen += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
			   (char *)eleHdrPtr += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);

				break;
		}

		if (currLen > totalLen)
		{
			return NOK;
		}
	}

	/* Check if received the required information elements */
	/* note that extSupportedRates is not a require. */
	if ((reAssocReqPtr->SSID == NULL) || (reAssocReqPtr->supportedRates == NULL))
	{
		return NOK;
	}

	*protoEvent = EV_11_RE_ASSOC_REQ;
	return OK;
}

/*************************************************************************
 *                        DisAssocParse			                         *
 *************************************************************************
DESCRIPTION: This function parses a Disassociation frame                 
                                                                                                   
INPUT:      bodyPtr - pointer to the frame body
			totalLen - frame data total len

OUTPUT:		 protoEvent - protocol event corresponds to the frame subtype
			 disAssocPtr - a structure contsins the information received in the frame.    

RETURN:		OK - parsing successful
			NOK -parsing failed	

************************************************************************/
static STATUS DisAssocParse(char *bodyPtr, UINT32 totalLen, UINT32 *protoEvent, dot11_disAssoc_T *disAssocPtr)
{
	UINT32				currLen = 0;

	(char *)(disAssocPtr->reason) = bodyPtr;
	*protoEvent = EV_11_DIS_ASSOC;
	if((currLen += FIX_FIELD_LEN) > totalLen)
		return NOK;
	
	return OK;
}

/* return OK if the MSDU is valid, otherwise returns NOK */
static STATUS mmpduCheckValidity(mem_MSDU_T *msduPtr)
{

	/* verify msdu is not NULL */
	if (msduPtr == NULL)
	{
		if(REPORT_LEVEL(RL_MGMT)) 
			wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error filtering mmpdu, NULL MSDU\n");
		return NOK;
	}

	/* verify first BD is not NULL */
	if (msduPtr->firstBDPtr == NULL)
	{
		if(REPORT_LEVEL(RL_MGMT)) 
			wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error filtering mmpdu, first BD is NULL\n");
		return NOK;
	}

	if (msduPtr->dataLen > sizeof (dot11_mgmtFrame_T))
	{
		if(REPORT_LEVEL(RL_MGMT)) 
			wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error filtering mmpdu, body is too long\n");
		return NOK;
	}

	return OK;
}

/*************************************************************************
 *                        BeaconParse                        			 *
 *************************************************************************
DESCRIPTION: This function parses received Beacon.

OUTPUT: nonERPpresentBit
		useProtectionBit
		noOFDMap

RETURN:  STATUS
************************************************************************/
STATUS BeaconParse(mem_MSDU_T *msduPtr,
				   UINT8  *nonERPpresentBit,
				   UINT8  *useProtectionBit,
				   UINT8  *noOFDMap,
				   UINT16 *beaconInterval,
				   BOOL   *pbcc)
{
	dot11_beacon   				*mmpduPtr;
	dot11_eleHdr_T 				*eleHdrPtr;
	UINT32						currLen=0, totalLen=0;
	dot11_supRates_T			*supportedRates=NULL;
	dot11_extSupRates_T			*extSupportedRates=NULL;
	UINT8						*erpContext=NULL;
	int 						i=0;
	UINT8						*rates;
	UINT8						found=0;
	UINT16						capabilities;
	UINT32						len;

	mmpduPtr = (dot11_beacon *)(msduPtr->firstBDPtr->data + msduPtr->firstBDPtr->dataOffset); 

	/* Read capabilities */
	capabilities = ENDIAN_HANDLE_WORD(mmpduPtr->capabilities);
	if ((capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK)
		*pbcc = TRUE;
	else
		*pbcc = FALSE;

	/* Read beacon interval */
	*beaconInterval = ENDIAN_HANDLE_WORD(mmpduPtr->beaconInterval);

	totalLen = msduPtr->dataLen - sizeof(dot11_beacon_fixedFields_T);
	(char *)eleHdrPtr = mmpduPtr->infoElements;

	/* start IEs parsing */
	while (currLen + sizeof(dot11_eleHdr_T) <= totalLen)
	{
		switch (ELE_ID(eleHdrPtr))
		{
			case DOT11_ERP_ELE_ID:
				erpContext = (UINT8 *) eleHdrPtr + sizeof(dot11_eleHdr_T);
				*nonERPpresentBit = (*erpContext) & DOT11_ERP_NONERP_PRESENT;
				*useProtectionBit = ((*erpContext) & DOT11_ERP_USE_PROTECTION) >> 1;
				*noOFDMap = 0;
				return OK;

			case DOT11_SUPPORTED_RATES_ELE_ID:
				if (ELE_LEN(eleHdrPtr) > DOT11_SUPPORTED_RATES_MAX)
				{					
					if(REPORT_LEVEL(RL_MGMT))
						printf("WDRV_RX: Error parsing beacon, Invalid eleLen = %d, eleId = %d\n", ELE_LEN(eleHdrPtr), ELE_ID(eleHdrPtr));
					return NOK;
				}
				supportedRates = (dot11_supRates_T *) eleHdrPtr;
				currLen += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				(char *)eleHdrPtr += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				break;

			case DOT11_EXT_SUPPORTED_RATES_ELE_ID:
				if (ELE_LEN(eleHdrPtr) > DOT11_EXT_SUPPORTED_RATES_MAX)
				{
					if(REPORT_LEVEL(RL_MGMT))
						printf("WDRV_RX: Error parsing beacon, Invalid eleLen = %d, eleId = %d\n", ELE_LEN(eleHdrPtr), ELE_ID(eleHdrPtr));
					return NOK;
				}
				extSupportedRates = (dot11_extSupRates_T *) eleHdrPtr;
				currLen += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				(char *)eleHdrPtr += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				break;

			default:
				/* just skip not interesting elements */
				currLen += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				(char *)eleHdrPtr += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				break;
		}

		if (currLen > totalLen)
		{
			if(REPORT_LEVEL(RL_MGMT))
			{			
				printf("Error parsing beacon, wrong length, currLen=%d, totalLen=%d, beaconInterval=%d, cap=0x%X\n",
					   currLen, totalLen,*beaconInterval,capabilities);
			}
			return NOK;
		}
	}

	if(supportedRates)
	{		
		len = supportedRates->hdr.eleLen;
		rates = (UINT8 *) supportedRates + sizeof(dot11_eleHdr_T);
		found = 0;
		/* determine whether this AP supports any of OFDM rates */
		for (i=0; i<len; i++)
		{
			switch (rates[i]&0x7F)
			{
				case DOT11_RATE_6:
				case DOT11_RATE_9:
				case DOT11_RATE_12:
				case DOT11_RATE_18:
				case DOT11_RATE_24:
				case DOT11_RATE_36:
				case DOT11_RATE_48:
				case DOT11_RATE_54:
					found = 1;
					i = len; /* end the loop */
					break;
		
				default:
					break;
			}
		}
	}
	if(!found && extSupportedRates)
	{		
		len = extSupportedRates->hdr.eleLen;
		rates = (UINT8 *) extSupportedRates + sizeof(dot11_eleHdr_T);
		/* determine whether this AP supports any of OFDM rates */
		for (i=0; i<len; i++)
		{
			switch (rates[i]&0x7F)
			{
				case DOT11_RATE_6:
				case DOT11_RATE_9:
				case DOT11_RATE_12:
				case DOT11_RATE_18:
				case DOT11_RATE_24:
				case DOT11_RATE_36:
				case DOT11_RATE_48:
				case DOT11_RATE_54:
					found = 1;
					i = len; /* end the loop */
					break;
		
				default:
					break;
			}
		}
	}
	
	*nonERPpresentBit = 0;
	*useProtectionBit = 0;
	*noOFDMap = found ^ 1; /* noOFDMap is the opposite of the found */

	return OK;
}



int  wdrv_mgmtPrintCounters(char *buf)
{

	int len = 0; 
	len +=sprintf(buf+len,"Mgmt Counters: \n");
	len +=sprintf(buf+len,"	Total Count	%d: \n", mgmtCounters.totalCount);
	len +=sprintf(buf+len,"	Auth Count	%d: \n", mgmtCounters.authCount);
	len +=sprintf(buf+len,"	De Auth Count	%d: \n", mgmtCounters.deauthCount);
	len +=sprintf(buf+len,"	Assoc Count	%d: \n", mgmtCounters.assocCount);
	len +=sprintf(buf+len,"	ReAssoc Count	%d: \n", mgmtCounters.reassocCount);
	len +=sprintf(buf+len,"	Dis Assoc Count	%d: \n", mgmtCounters.disassocCount);
	
	return len;
}

/* print the frame for debugging purposes */
void wdrv_mgmtPrintFrame(mem_MSDU_T *msduPtr)
{
	int i,j, strLen;
	unsigned char *data;
	unsigned char   outbuf[EV_MESSAGE_LEN + 1];
	unsigned char   outtmp[EV_MESSAGE_LEN + 1];
	dot11_mgmtFrame_T		printBuf;

	if(!REPORT_LEVEL(RL_MGMT)) 
		return;
	
	if (wlan_memMngrMsduIsFragmented(msduPtr))
	{
		wlan_memMngrMsduReassemble(msduPtr, (unsigned char *)&printBuf);
		data = (unsigned char *)&printBuf; 
	}
	else
		data = (msduPtr->firstBDPtr->data + msduPtr->firstBDPtr->dataOffset); 

	wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "Header");
	outbuf[0] = '\0';
	strLen = 0;
	for (i = 0, j = 0; i < sizeof(dot11_mgmtHeader_T); i++)
	{
		sprintf(outtmp,"%02x ", (UINT)(*(data + i)));
		strLen = strlen(outtmp);
		if (strLen + strlen(outbuf) < EV_MESSAGE_LEN)
			strcat(outbuf, outtmp);
		else
		{
			wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "Print buffer overflow");
			break;
		}

		j++;

		if (j == 16)
		{
			j = 0;
			wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "%s", outbuf);
			outbuf[0] = '\0';
			strLen = 0;
		}
	}
	wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "%s", outbuf);

	data += sizeof(dot11_mgmtHeader_T);

	wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "Body");
	outbuf[0] = '\0';
	strLen = 0;
	for (i = 0, j = 0; i < (msduPtr->dataLen - sizeof(dot11_mgmtHeader_T)); i++)
	{
		sprintf(outtmp,"%x ", *(data + i));
		strLen = strlen(outtmp);
		if (strLen + strlen(outbuf) < EV_MESSAGE_LEN)
			strcat(outbuf, outtmp);
		else
		{
			wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "Print buffer overflow");
			break;
		}

		j++;

		if (j == 16)
		{
			j = 0;
			wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "%s", outbuf);
			outbuf[0] = '\0';
			strLen = 0;
		}
	}
	wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "%s\n", outbuf);
}


STATUS probeReqParse(char *bodyPtr, UINT32 totalLen, UINT32 *protoEvent, dot11_probeReq *probeReqPtr)
{
	UINT32 currLen = 0;
	dot11_eleHdr_T *eleHdrPtr;

	/* Init all IE pointers to NULL */
	probeReqPtr->SSID = NULL;
	probeReqPtr->supportedRates = NULL;

	/* now information elements */
	(char *)eleHdrPtr = bodyPtr;

	while (currLen + sizeof(dot11_eleHdr_T) < totalLen)
	{

		switch (ELE_ID(eleHdrPtr))
		{
			case DOT11_SSID_ELE_ID:
				if (ELE_LEN(eleHdrPtr) > DOT11_SSID_MAX)
				{
					if(REPORT_LEVEL(RL_MGMT)) 
						wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: 111 Error parsing probe req, Invalid eleLen = %d, eleId = %d\n", ELE_LEN(eleHdrPtr), ELE_ID(eleHdrPtr));
					return NOK;
				}
				(dot11_eleHdr_T *)(probeReqPtr->SSID) = eleHdrPtr;
				currLen += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				(char *)eleHdrPtr += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				break;

			case DOT11_SUPPORTED_RATES_ELE_ID:
				if (ELE_LEN(eleHdrPtr) > DOT11_SUPPORTED_RATES_MAX)
				{
					if(REPORT_LEVEL(RL_MGMT)) 
						wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: 222 Error parsing probe req, Invalid eleLen = %d, eleId = %d\n", ELE_LEN(eleHdrPtr), ELE_ID(eleHdrPtr));
					return NOK;
				}
				(dot11_eleHdr_T *)(probeReqPtr->supportedRates) = eleHdrPtr;
				currLen += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				(char *)eleHdrPtr += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				break;

			default:
				currLen += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				(char *)eleHdrPtr += ELE_LEN(eleHdrPtr) + sizeof(dot11_eleHdr_T);
				break;
		} /* switch */

		if (currLen > totalLen)
		{
			return NOK;
		}
	}

	/* Check if received the required information elements */
	/* note that extSupportedRates is not a require. */
	if ((probeReqPtr->SSID == NULL) || (probeReqPtr->supportedRates == NULL))
	{
		return NOK;
	}

	*protoEvent = EV_11_ASSOC_REQ;
   
	return OK;
}


