/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************/
/*																			*/
/*		MODULE:	mgmtSm.c													*/
/*		PURPOSE:	Authorization/association state machine implementation	*/
/*																			*/
/****************************************************************************/
#include "pform_types.h"
#include <stdio.h>
#include "mgmtSm.h"
#include "mainsm.h"
#include "memMngr.h"
#include "mgmtParser.h"
#include "mgmtSystem.h"
#include "mgmtBuilder.h"
#include "mgmtIf.h"
#include "mgmtAccess.h"
#include "drvDbHash.h"
#include "drvDbMain.h"
#include "utils.h"
#include "pal.h"
#include "whal.h"
#include "dda_contexts.h"
#include "dda.h"
#include "dda_wpa_interface.h"
#include "4xMngr.h"
#include "whalApi.h"
#include "whalAcxCmd.h"
#include "whalAcxProcess.h"
#include "whalAcxEeprom.h"
#include "whalParams.h"

#define	MAX_PRINT_STRING		128		

/* number of inputs */
#define	NUM_INPUTS		21		

#define SM_NAME "MGMT_SM"
#include "utilsGenericSM.h"

/* task variables */
/* Mgmt messages queue size                     */
#define WDRV_MGMT_QSIZE         20

/* Mgmt constants */ 
#define AGING_TIMEOUT		600000	/* 600 seconds (10 minutes). Used for periodic station aging. */

/*-----------------------------------------------------------*/
/*         Local Vartiables									 */
/*-----------------------------------------------------------*/

/* aging timer */
static PALSRV_TcbObjTYPE   agingTimer;
/* aging mode */
static BOOL agingMode;
/* aging timeout in ticks */
static UINT32 agingTimeoutInTicks;

/* Dynamic Algorithms Timers (used for PBCC, Slot time, ERP algorithms) */
static PALSRV_TcbObjTYPE   DynamicAlgorithmsTimer;
/* ERP beacon aging timer */
static PALSRV_TcbObjTYPE   ERPbeaconAgingTimer;
/* PBCC beacon aging timer */
static PALSRV_TcbObjTYPE   PBCCbeaconAgingTimer;
/* radio calibrate timer */
static PALSRV_TcbObjTYPE   radioCalibrateTimer, radioCalibrateTimerCrt;

/* used to pass management messages information between the mgmt sub-modules */
static void *protoPtrLocal;

/* state machine actions */
typedef enum {
	ACTION_NOP						= 0x00,
	ACTION_START_TIMER				= 0x01,
	ACTION_STOP_TIMER				= 0x02,
	ACTION_NOTIFY_CONNECT			= 0x04,
	ACTION_NOTIFY_DIS_CONNECT		= 0x08,
	ACTION_SEND_AUTH				= 0x10,
	ACTION_SEND_DE_AUTH				= 0x20,
	ACTION_SEND_ASSOC_RSP			= 0x40,
	ACTION_SEND_RE_ASSOC_RSP		= 0x80,
	ACTION_SEND_DIS_ASSOC			= 0x100,
	ACTION_SET_STATION_PARAMS		= 0x200,
	ACTION_UPDATE_STATION_PARAMS	= 0x400,
	ACTION_RESET_STATION_PARAMS		= 0x800,
	ACTION_REMOVE_STATION			= 0x1000,
	ACTION_SEND_WPA_ASS_REQ			= 0x2000,
	ACTION_SEND_WPA_REASS_REQ		= 0x4000
} actions;

/* management table */
typedef struct 
{
	UINT32				timeStamp;  							/* Timestamp of the last frame received from the station */      
	UINT32				authTimeStamp;  						/* Timestamp of the last successful authentication of the station */      
	UINT32				assocTimeStamp;  						/* Timestamp of the last successful association of the station */      
	char				challengeText[DOT11_CHALLENGE_TEXT_MAX];/* Challenge text that were sent to the station in the authentication process. */
	PALSRV_TcbObjTYPE	authTimer;								/* Authentication response timer of the station */
} mgmtEntry_T;

/* The table holds the station information. */
#if SM_DEBUG
UINT32 smIndexError = 0;
#endif
static mgmtEntry_T	mgmtTable[MAX_STATIONS];

static BOOL mgmtLog;

/* The following structures are used in order to print */
static char smInputDesc[NUM_INPUTS][MAX_PRINT_STRING] = {
    "INPUT_AUTH_1_OS_A",                                                                           /* 0 */
    "INPUT_AUTH_1_SK_A",                                                                          /* 1 */
	"INPUT_AUTH_1_D",	
	"INPUT_AUTH_3_A",
	"INPUT_AUTH_3_D",	
	"INPUT_DE_AUTH_STATION",
	"INPUT_ASSOC_REQ_CHECK",
	"INPUT_ASSOC_REQ_A",
	"INPUT_ASSOC_REQ_D",
	"INPUT_RE_ASSOC_REQ_CHECK",
	"INPUT_RE_ASSOC_REQ_A",
	"INPUT_RE_ASSOC_REQ_D",
	"INPUT_DIS_ASSOC_STATION",
	"INPUT_CLASS_3",
	"INPUT_AUTH_TIMEOUT",
	"INPUT_DE_AUTH_MNG_UNICAST",
	"INPUT_DE_AUTH_MNG_BROADCAST",
	"INPUT_DIS_ASSOC_MNG_UNICAST",
	"INPUT_DIS_ASSOC_MNG_BROADCAST",
	"INPUT_MAC_AUTHORIZE",
	"INPUT_MAC_DE_AUTHORIZE",
};

static char smStatusDesc[STATUS_MAX][MAX_PRINT_STRING] = {
	"STATUS_SUCCESSFUL",
	"STATUS_UNSPECIFIED",												
	"STATUS_AUTH_NO_LONGER_VALID",										
	"STATUS_DE_AUTH_STATION_IS_LEAVING",								
	"STATUS_DIS_ASSOC_INACTIVITY",										
	"STATUS_DIS_ASSOC_UNABLE_TO_HANDLE_ALL_CURRENTLY_ASSOC_STATIONS",		
	"STATUS_CLASS_2_FROM_NON_AUTH_STATION",								
	"STATUS_CLASS_3_FROM_NON_ASSOC_STATION",								
	"STATUS_DIS_ASSOC_STATION_IS_LEAVING",									
	"STATUS_STATION_REQUESTING_ASSOC_IS_NOT_AUTH",							
	"STATUS_CAPABILITIES_FAILURE",											
	"STATUS_REASSOC_DENIED_UNABLE_TO_CONFIRM_ASSOC",						
	"STATUS_ASSOC_DENIED_OUT_OF_SCOPE",									
	"STATUS_ALGO_IS_NOT_SUPPORTTED",										
	"STATUS_AUTH_TRANSC_NUMBER_OUT_OF_SEQUENCE",							
	"STATUS_AUTH_REJ_CHALLENGE_FAILURE",									
	"STATUS_AUTH_REJ_TIMEOUT",											
	"STATUS_ASSOC_DENIED_UNABLE_TO_HANDLE_ADDITIONAL_ASSOC",				
	"STATUS_ASSOC_DENIED_RATE_FAILURE",									
	"STATUS_ASSOC_DENIED_PREAMBLE_FAILURE",
	"STATUS_ASSOC_DENIED_PBCC_FAILURE",
	"STATUS_ASSOC_DENIED_AGILITY_FAILURE",
};

char smStateDesc[NUM_STATES][MAX_PRINT_STRING] = {
	"     IDLE    ",
	"   AUTH_KEY  ",
	"AUTHENTICATED",
	"WAIT_FOR_ASS_RES",
	"  ASSOCIATED ",
	"  AUTHORIZED ",
};

/* ERP types */
typedef struct
{
	UINT32 timestamp;   /* last "bad" beacon timestamp */
	BOOL   valid;       /* TRUE - the last decision made is valid */
	BOOL   active;      /* TRUE - there was no timeout, we get "bad" beacons continiously*/
} beaconERPinfo_T;

typedef struct
{
	UINT8 nonERPpresentBit;
	UINT8 useProtectionBit;
	BOOL  decisionDone;
} ERPdecision_T;

static beaconERPinfo_T BeaconERPinfo;

static UINT32 ERPmaxBeaconInterval=0; /* maximal beaconInterval received since the AP started to run */

/* PBCC types */
typedef struct
{
	BOOL   valid;       /* TRUE - the last decision made is valid */
	BOOL   active;      /* TRUE - there was no timeout, we get "bad" beacons continiously*/
} beaconPBCCinfo_T;

static beaconPBCCinfo_T BeaconPBCCinfo;

static UINT32 PBCCmaxBeaconInterval=0;



#define RADIO_CALIBRATE_TIMER_2 1     /* This is the timer between calibration step 1->2  */
#define RADIO_CALIBRATE_TIMER_3 1     /* This is the timer between calibration step 2->3 */
#define RADIO_CALIBRATE_INTERVAL_RADIA_ST_1      300 /* 5 minutes */
#define RADIO_CALIBRATE_INTERVAL_CRT_ST_1        25  /* 25 seconds */
#define RADIO_CALIBRATE_INTERVAL_CRT_ST_4        2   /* 2 seconds */

#define m_ADJUST_CALIBRATE_INTERVAL(interval)                   \
       if((interval) == 0xFFFF) {                               \
           /* Use default values depend on radio module type */ \
           (interval) = (crtAlgorithmType) ?                 \
                    RADIO_CALIBRATE_INTERVAL_CRT_ST_1 :         \
                    RADIO_CALIBRATE_INTERVAL_RADIA_ST_1;        \
       }


static UINT32 crtAlgorithmType;
static UINT32 currentRadioCalibrateState;

BOOL ERPcommandOn = 0;

/* WPA package support */
states	mgmtBcastState;

/*-----------------------------------------------------------*/
/*         Local Functions                                   */
/*-----------------------------------------------------------*/
static void		actionTake(output_T *smOutput, void	*protoPtr, UINT16 index);
static void		buildStateMachineMatrix(void);
static STATUS	resetParams(void);
static void		startAuthTimer(UINT16 index);
static void		stopAuthTimer(UINT16 index);
static void		updateTimeStamp(UINT16 idx);
static void		printSmInput(wdrv_mgmtSysInfo_T systemInfo, UINT16 index);
static void		printSmOutput(output_T smOutput, UINT16 index);
static void		setStationParams(void *protoPtr, UINT16 index, UINT32 action);
static void		updateStationParams(void *protoPtr, UINT16 index, UINT32 action);
static void		resetStationParams(UINT16 index);
static void		notifyConnect(UINT16 index);
static void		notifyDisConnect(UINT16 index);
static void		implementPbccStaticAlgorithmIfNeeded(UINT16 index, algo_action_t action);
static void		implementPreambleStaticAlgorithmIfNeeded(UINT16 index, algo_action_t action);
static void		implementSlotTimeStaticAlgorithmIfNeeded(UINT16 index, algo_action_t action);
static void		implementERPProtectionAlgorithmIfNeeded(UINT16 index, algo_action_t action);
static STATUS   wdrv_mgmtERPtimeoutHandler(void);
static void     wdrv_mgmtERPdynamicManager(UINT8 nonERPpresent,UINT8 useProtection,UINT8 noOFDMap,ERPdecision_T *decision, UINT16 beaconInterval);								  						  
static STATUS   wdrv_mgmtPerformDynamicAlgorithms(mem_MSDU_T *msduPtr);
static STATUS   wdrv_mgmtPBCCtimeoutHandler(void);
static void     wdrv_mgmtPBCCdynamicManager(BOOL pbcc, UINT16 beaconInterval);
static void 	wdrv_mgmtDynamicAlgTimeoutHandler(void);
static void 	wdrv_mgmtHandleSetTemplateReq(UINT32 ReqType, UINT32 val);
static void		wdrv_mgmtRadioCalibrateTimeoutHandler(void);
static void		wdrv_mgmtRadioCalibrateCrtTimeoutHandler(void);
static void		agingTimerCB(UINT32 data);
static void		dynamicAlgTimerCB(UINT32 data);
static void     ERPbeaconAgingTimerCB(UINT32 data);
static void		PBCCbeaconAgingTimerCB(UINT32 data);
static void		RadioCalibrationTimerCB(UINT32 data);
static void		agingTimerCB(UINT32 data);
static void		dynamicAlgTimerCB(UINT32 data);
static void     ERPbeaconAgingTimerCB(UINT32 data);
static void		PBCCbeaconAgingTimerCB(UINT32 data);
static void		RadioCalibrationTimerCrtCB(UINT32 data);
static BOOL		isMacBcast(char *mac);

/*************************************************************************
 *                        wdrv_mgmtInit		                             *
 *************************************************************************
DESCRIPTION: Initializes the mgmt module                 
                                                                                                   
INPUT:      

OUTPUT:      

RETURN:     OK - Initialization OK 
			NOK - Initialization NOK 

************************************************************************/
STATUS wdrv_mgmtInit(void) 
{
	int /*rc,*/ i;
	UINT32	protoMsgMaxSize;
    UINT8   PBCCalgoMode,ERPProtectionMode,slotTimeMode;
/*	UINT32	timeout;*/

	
    /* Init Access Control DB */
/*    mgmtAccessInit();*/
	/* Create the aging timer */
	if(PALSRV_tcbNew(&agingTimer))
	{
		wdrv_utilsReport(RL_MGMT, PRI_EMERGENCY, "WDRV_MGMT: Error Creating Aging Timer\n");
		return NOK;
	}

	/* Allocated authRspTimer (tcb timer)for each station.*/
	for (i = 0; i < MAX_STATIONS; i++)
	{
		if(PALSRV_tcbNew(&mgmtTable[i].authTimer))
		{
			if(REPORT_LEVEL(RL_MGMT)) 
				wdrv_utilsReport(RL_MGMT, PRI_EMERGENCY, "WDRV_MGMT: Error Creating Station Auth Response Timer\n");

			return NOK;
		}
	}

	/* reset */
	wdrv_mgmtReset();
	if (resetParams() == NOK)
		return NOK;

	/* Start the aging timer */
	PALSRV_tcbQueueAct(&agingTimer, AGING_TIMEOUT, agingTimerCB, 0, 0);

	agingTimeoutInTicks = PALSRV_msec2ticks(AGING_TIMEOUT);

	wdrv_mgmtSetAging(TRUE);

	/* compute protocol message maximumu size */
	protoMsgMaxSize =	WDRV_UTILS_MAX(sizeof(dot11_disAssoc_T),
						WDRV_UTILS_MAX(sizeof(dot11_reAssocRsp_T),
						WDRV_UTILS_MAX(sizeof(dot11_reAssocReq_T),
						WDRV_UTILS_MAX(sizeof(dot11_assocRsp_T), 
						WDRV_UTILS_MAX(sizeof(dot11_assocReq_T),
						WDRV_UTILS_MAX(sizeof(dot11_auth_T), sizeof(dot11_deAuth_T)))))));
	
	/* allocated protoInfoPtr */
	if ((protoPtrLocal = PAL_memAlloc(0,protoMsgMaxSize,0,0))==NULL) 
		return NOK;

	/* build the state machine matrix */
	buildStateMachineMatrix();

	mgmtBcastState = /*STATE_AUTHORIZED*/STATE_ASSOCIATED;		/* Always start from this state. */

#if SM_DEBUG	
	wdrv_mgmtSetLog(TRUE);
#else
	wdrv_mgmtSetLog(FALSE);
#endif
    
    /* PBCC, ERP, Slot time enhanced dynamic algorithms init */
	wdrv_dbGetPBCCAlgorithmMode(0,&PBCCalgoMode);
	wdrv_dbGetSlotTimeMode(0,&slotTimeMode);
	wdrv_dbGetERPProtectionMode(0,&ERPProtectionMode);
	if((PBCCalgoMode == PBCC_ALGO_ENHANCED_DYNAMIC) ||
	   (slotTimeMode == SLOTTIME_MODE_ENHANCED_DYNAMIC) ||
	   (slotTimeMode == SLOTTIME_MODE_LOCAL_STATIONS) ||
	   (ERPProtectionMode == PROTECTION_MODE_ENHANCED_DYNAMIC))
	{
		/* Create Dynamic Algorithm timer */
		if(PALSRV_tcbNew(&DynamicAlgorithmsTimer))
		{
			if(REPORT_LEVEL(RL_DYNAMIC_ALG)) 
				wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_EMERGENCY, "WDRV_MGMT: Error Creating DynamicAlgorithmsTimer\n");
			return NOK;
		}
			
		PALSRV_tcbQueueAct(&DynamicAlgorithmsTimer, 50, dynamicAlgTimerCB, 0, 0);
	}

	/* ERP init */
	BeaconERPinfo.valid = FALSE;
	BeaconERPinfo.active = FALSE;
	BeaconERPinfo.timestamp = 0;

    if(PALSRV_tcbNew(&ERPbeaconAgingTimer))
	{
		if(REPORT_LEVEL(RL_DYNAMIC_ALG)) 
			wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_EMERGENCY, "WDRV_MGMT: Error Creating ERPbeaconAging Timer\n");
		return NOK;
	}

	/* PBCC init */
	BeaconPBCCinfo.valid = FALSE;
	BeaconPBCCinfo.active = FALSE;

	if(PALSRV_tcbNew(&PBCCbeaconAgingTimer))
	{
		if(REPORT_LEVEL(RL_DYNAMIC_ALG)) 
			wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_EMERGENCY, "WDRV_MGMT: Error Creating PBCCbeaconAgingTimer Timer\n");
		return NOK;
	}

	/* Init the Radio Calibrate timer */
    wdrv_mgmtRadioCalibrateInit();

	PAL_spinLock_init(&mgmtLock);

	if(REPORT_LEVEL(RL_MGMT)) 
		wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Management module initialized successfully\n");

	wdrv_mainsmSendModuleUp(WDRV_MGMT_MODULE_ID);

	return OK;
}

/*************************************************************************
 *                        wdrv_mgmtUnload	                             *
 *************************************************************************
DESCRIPTION: Deinitializes the mgmt module                 
                                                                                                   
INPUT:      

OUTPUT:      

RETURN:     
************************************************************************/
void wdrv_mgmtUnload(void)
{
	int i;

	if(protoPtrLocal)
		PAL_memFree(0,protoPtrLocal,0);

	/* delete timers */
	PALSRV_tcbDelete(&agingTimer);
	PALSRV_tcbDelete(&DynamicAlgorithmsTimer);
	PALSRV_tcbDelete(&ERPbeaconAgingTimer);
	PALSRV_tcbDelete(&PBCCbeaconAgingTimer);
#ifdef TIWLAN_FW_RADIA_CRT
	PALSRV_tcbDelete(&radioCalibrateTimerCrt);
#endif
	PALSRV_tcbDelete(&radioCalibrateTimer);		

	for (i=0; i<MAX_STATIONS; i++)
		PALSRV_tcbDelete(&mgmtTable[i].authTimer);

	wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: mgmtSm unloaded !!!\n");
}

static void agingTimerCB(UINT32 data)
{
	mgmtMessage_T	msg;

	msg.type = AGING;

	wdrv_DDA_mgmt_setEvent(&msg);

	/* reschedule the timer */
	PALSRV_tcbQueueAct(&agingTimer, AGING_TIMEOUT, agingTimerCB, 0, 0);
}

static void dynamicAlgTimerCB(UINT32 data)
{
	mgmtMessage_T	msg;

	msg.type = DYNAMIC_ALG_TIMEOUT;

	wdrv_DDA_mgmt_setEvent(&msg);

	/* reschedule the timer */
	PALSRV_tcbQueueAct(&DynamicAlgorithmsTimer, 50, dynamicAlgTimerCB, 0, 0);
}

/******************************************************************************
*                      wdrv_mgmt_getEvent											  *
*******************************************************************************
DESCRIPTION:	Called from DDA on mgmt event reception.
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
STATUS wdrv_mgmt_getEvent(mgmtMessage_T	*pMessage) {
	int             rc;
	UINT16			i;

	UINT32			ticks;
	wdrv_mgmtSysInfo_T	systemInfo;	
	output_T		smOutput;
	UINT32			protoEvent;
	UINT32 			reason;
#if SM_DEBUG
	char			staMac[MAC_ADDR_LEN];
#endif

	switch (pMessage->type)
	{
		case MGMT_FRAME:		/* a management frame received from Rx data task */
				
			if(REPORT_LEVEL(RL_MGMT)) 
				wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: MGMT_FRAME message received\n");
		
			/* check index is valid */
			if (pMessage->content.frame.index >= MAX_STATIONS)
			{
				if(REPORT_LEVEL(RL_MGMT)) 
					wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Index is out of range\n");
				wlan_memMngrFreeMSDU(pMessage->content.frame.msduPtr->handle);
				break;
			}
			
			/* parse MSDU */
			rc = wdrv_mgmtMmpduParse(pMessage->content.frame.msduPtr, pMessage->content.frame.index, &protoEvent, protoPtrLocal); 

			if (rc != OK)
			{
				wlan_memMngrFreeMSDU(pMessage->content.frame.msduPtr->handle);
				break;
			}
			
			/* perform a system check to know the status of the request */
			wdrv_mgmtSystemCheck(pMessage->content.frame.index, protoEvent, protoPtrLocal, &systemInfo);
			
			if(REPORT_LEVEL(RL_MGMT)) 
				printSmInput(systemInfo, pMessage->content.frame.index);
			
			/* activate the state machine */
			rc = lbt_smActions(systemInfo.event, pMessage->content.frame.index, &smOutput);
			
			if (rc != OK)
			{
				wlan_memMngrFreeMSDU(pMessage->content.frame.msduPtr->handle);
				break;
			}

			if (smOutput.status == STATUS_UNSPECIFIED)
				smOutput.status = systemInfo.status;

			if(REPORT_LEVEL(RL_MGMT)) 
				printSmOutput(smOutput, pMessage->content.frame.index);

			/* perform the actions as returned from the state machine */
			actionTake(&smOutput, protoPtrLocal, pMessage->content.frame.index); 

			/* finally, free MSDU */
			wlan_memMngrFreeMSDU(pMessage->content.frame.msduPtr->handle);
			break;

		case WPA_ASSOC_REQ_REP:			 			

			 if(!wdrv_dbIsTableIndexExists(pMessage->content.wpaAssCheckRep.index))
				 break;

			 wdrv_dbGetWpaReq(pMessage->content.wpaAssCheckRep.index, &reason);
			 if(pMessage->content.wpaAssCheckRep.flag)
			 {
				if(reason == ACTION_SEND_WPA_ASS_REQ)
				{
					systemInfo.event  = INPUT_ASSOC_REQ_A;
					systemInfo.status = STATUS_SUCCESSFUL;
				}
				else if(reason == ACTION_SEND_WPA_REASS_REQ)
				{
					systemInfo.event  = INPUT_RE_ASSOC_REQ_A;
					systemInfo.status = STATUS_SUCCESSFUL;
				}
				else
					printf("WPA_ASSOC_REQ_REP message: wrong wpa request type = %d\n",reason);
			 }
			 else
			 {
				if(reason == ACTION_SEND_WPA_ASS_REQ)
				{
					systemInfo.event = INPUT_ASSOC_REQ_D;
					systemInfo.status = STATUS_UNSPECIFIED;
				}
				else if(reason == ACTION_SEND_WPA_REASS_REQ)
				{
					systemInfo.event = INPUT_RE_ASSOC_REQ_D;
					systemInfo.status = STATUS_UNSPECIFIED;
				}
				else
					printf("WPA_ASSOC_REQ_REP message: wrong wpa request type = %d\n",reason);
				
				printf("Receive assoc failed from wpa,station index %d.\n",pMessage->content.wpaAssCheckRep.index);
			 }

			 if(REPORT_LEVEL(RL_MGMT)) 
				printSmInput(systemInfo, pMessage->content.wpaAssCheckRep.index);

			 /* activate the state machine */
			 rc = lbt_smActions(systemInfo.event, pMessage->content.wpaAssCheckRep.index, &smOutput);
			 
   			 if(REPORT_LEVEL(RL_MGMT)) 
				printSmOutput(smOutput, pMessage->content.wpaAssCheckRep.index);

			 /* sm output*/	
			 if (smOutput.status == STATUS_UNSPECIFIED)
				 smOutput.status = systemInfo.status;

			 /* perform the actions as returned from the state machine */
			 actionTake(&smOutput, protoPtrLocal, pMessage->content.wpaAssCheckRep.index); 

			 break;

		case BEACON_FRAME:

			wdrv_mgmtPerformDynamicAlgorithms(pMessage->content.frame.msduPtr);
			rc = wlan_memMngrFreeMSDU(pMessage->content.frame.msduPtr->handle);
			if(rc == NOK)
				printf("mgmt (beaconFrame) failed in wlan_memMngrFreeMSDU()\n");
			break;

		case BEACON_ERP_TIMEOUT:
			if(REPORT_LEVEL(RL_DYNAMIC_ALG)) 
				wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_INFORMATION, "WDRV_MGMT: Beacon ERP timeout received\n");
			wdrv_mgmtERPtimeoutHandler();
			break;
	
		case BEACON_PBCC_TIMEOUT:
			if(REPORT_LEVEL(RL_DYNAMIC_ALG)) 
				wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_INFORMATION, "WDRV_MGMT: Beacon PBCC timeout received\n");
			wdrv_mgmtPBCCtimeoutHandler();
			break;

		case SET_TEMPLATE_REQ:
			wdrv_mgmtHandleSetTemplateReq((UINT32)pMessage->content.MessageInfo[0],
										  (UINT32)pMessage->content.MessageInfo[1]);
			break;

        case DYNAMIC_ALG_TIMEOUT:
            wdrv_mgmtDynamicAlgTimeoutHandler();
            break;

		case CLASS3:				/* a class 3 message arrived from Rx data task */
			if(REPORT_LEVEL(RL_MGMT)) 
				wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: CLASS_3 message received\n");

			/* if index is invalid, means no index exist for the station, send de authentication by the station mac */
			if (pMessage->content.class3.index == INVALID_INDEX)
			{
				wdrv_mgmtSendDeAuthUnicastByMac(STATUS_CLASS_3_FROM_NON_ASSOC_STATION, 
                                                pMessage->content.class3.macAddress,
                                                pMessage->content.class3.wone_index);						
				break;
			}					

			/* check index is valid */
			if (pMessage->content.class3.index >= MAX_STATIONS)
			{
				if(REPORT_LEVEL(RL_MGMT)) 
					wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Index is out of range\n");
				break;
			}

#if SM_DEBUG
			/* TODO: maybe removed in a later phase, */
			wdrv_dbGetStaMacAddress(pMessage->content.class3.index, staMac);
			/* verify that src MAC is equal to table MAC */
			if (PAL_memCmp(staMac, pMessage->content.class3.macAddress, MAC_ADDR_LEN))
			{
				if(REPORT_LEVEL(RL_MGMT)) 
					wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error class 3 message, message MAC incompatible with db MAC\n");
					smIndexError++;
					break;
			}
#endif
			
			systemInfo.event = INPUT_CLASS_3;
			systemInfo.status = STATUS_CLASS_3_FROM_NON_ASSOC_STATION;

			if(REPORT_LEVEL(RL_MGMT)) 
				printSmInput(systemInfo, pMessage->content.class3.index);

			/* activate the state machine */
			rc = lbt_smActions(systemInfo.event, pMessage->content.class3.index, &smOutput);
			
			if (rc != OK)
				break;

			if (smOutput.status == STATUS_UNSPECIFIED)
				smOutput.status = systemInfo.status;

			if(REPORT_LEVEL(RL_MGMT)) 
				printSmOutput(smOutput, pMessage->content.class3.index);

			/* perform the actions as returned from the state machine */
			actionTake(&smOutput, protoPtrLocal, pMessage->content.class3.index);
			break;

		case AUTH_RESPONSE_TIMEOUT:			/* authentication response timeout message received from the TCB*/
			if(REPORT_LEVEL(RL_MGMT)) 
				wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: AUTH_RESPONSE_TIMEOUT message received\n");

			systemInfo.event = INPUT_AUTH_TIMEOUT;
			systemInfo.status = STATUS_UNSPECIFIED;

			if(REPORT_LEVEL(RL_MGMT)) 
				printSmInput(systemInfo, pMessage->content.authTimeout.index);

			/* activate the state machine */
			rc = lbt_smActions(systemInfo.event, pMessage->content.authTimeout.index, &smOutput);
			
			if (rc != OK)
				break;

			if(REPORT_LEVEL(RL_MGMT)) 
				printSmOutput(smOutput, pMessage->content.authTimeout.index);

			/* perform the actions as returned from the state machine */
			actionTake(&smOutput, protoPtrLocal, pMessage->content.authTimeout.index);
			break;

		case AGING:							/* aging message received from the TCB*/
			if (wdrv_mgmtAgingEnable())
			{
				if(REPORT_LEVEL(RL_MGMT)) 
					wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: AGING message received\n");
				
				ticks = PALSRV_getTick();
				systemInfo.event = INPUT_DE_AUTH_MNG_UNICAST;
				systemInfo.status = STATUS_AUTH_NO_LONGER_VALID;
				
				/* For every station, check if aging need to be taken, 
						according to the current timestamp and the station timestamp.*/
				for (i = 0; i < MAX_STATIONS; i++)
				{
					if ((smCurrentState[i] != STATE_IDLE) && ((UINT32)(ticks - wdrv_mgmtReadTimeStamp(i)) > agingTimeoutInTicks))
					{

						if(REPORT_LEVEL(RL_MGMT)) 
							printSmInput(systemInfo, i);

						/* activate the state machine */
						rc = lbt_smActions(systemInfo.event, i, &smOutput);
						
						if (rc != OK)
							break;
					
						if (smOutput.status == STATUS_UNSPECIFIED)
							smOutput.status = systemInfo.status;
			
						if(REPORT_LEVEL(RL_MGMT)) 
							printSmOutput(smOutput, i);

						/* perform the actions as returned from the state machine */
						actionTake(&smOutput, protoPtrLocal, i);
					}
				}
			}

			break;

		case DEAUTH_MNG:			/* a DeAuth manager message arrived, force deauthentication for all stations. */
			if(REPORT_LEVEL(RL_MGMT)) 
				wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: DEAUTH_MNG message received\n");

			if (isMacBcast(pMessage->content.deAuthMng.macAddress))	/* broadcast DeAuth*/
			{
				systemInfo.event = INPUT_DE_AUTH_MNG_BROADCAST;
				systemInfo.status = pMessage->content.deAuthMng.reason;

				for (i = 0; i < MAX_STATIONS; i++)
				{
					/* reset the station state*/
					if (smCurrentState[i] != STATE_IDLE)
					{
						if(REPORT_LEVEL(RL_MGMT)) 
							printSmInput(systemInfo, i);

						/* activate the state machine */
						rc = lbt_smActions(systemInfo.event, i, &smOutput);
						
						if (rc != OK)
							break;

						if (smOutput.status == STATUS_UNSPECIFIED)
							smOutput.status = systemInfo.status;

						if(REPORT_LEVEL(RL_MGMT)) 
							printSmOutput(smOutput, i);

						/* perform the actions as returned from the state machine */
						actionTake(&smOutput, protoPtrLocal, i);
					}
				}

				/* send a deauthentication broadcast frame */
				wdrv_mgmtSendDeAuthBcast(smOutput.status);
			}
			else			/* unicast DeAuth*/
			{
				rc = wdrv_dbFindStaIndex(pMessage->content.deAuthMng.macAddress, &i);
				if (rc != OK)
					break;

				if (smCurrentState[i] == STATE_IDLE)
					break;

				systemInfo.event = INPUT_DE_AUTH_MNG_UNICAST;
				systemInfo.status = pMessage->content.deAuthMng.reason;

				if(REPORT_LEVEL(RL_MGMT)) 
					printSmInput(systemInfo, i);

				/* activate the state machine */
				rc = lbt_smActions(systemInfo.event, i, &smOutput);
				
				if (rc != OK)
					break;

				if (smOutput.status == STATUS_UNSPECIFIED)
					smOutput.status = systemInfo.status;

				if(REPORT_LEVEL(RL_MGMT)) 
					printSmOutput(smOutput, i);

				/* perform the actions as returned from the state machine */
				actionTake(&smOutput, protoPtrLocal, i);
			}
			break;

		case DISASSOC_MNG:			/* a DisAssoc manager message arrived, force Disassociation for all stations. */
			if(REPORT_LEVEL(RL_MGMT)) 
				wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: DISASSOC_MNG message received\n");

			if (isMacBcast(pMessage->content.disAssocMng.macAddress))	/* broadcast DisAssoc*/
			{
				systemInfo.event = INPUT_DIS_ASSOC_MNG_BROADCAST;
				systemInfo.status = pMessage->content.disAssocMng.reason;

				for (i = 0; i < MAX_STATIONS; i++)
				{
					if ((smCurrentState[i] == STATE_ASSOCIATED) || (smCurrentState[i] == STATE_AUTHORIZED))
					{
						if(REPORT_LEVEL(RL_MGMT)) 
							printSmInput(systemInfo, i);

						/* activate the state machine */
						rc = lbt_smActions(systemInfo.event, i, &smOutput);
						
						if (rc != OK)
							break;

						if (smOutput.status == STATUS_UNSPECIFIED)
							smOutput.status = systemInfo.status;

						if(REPORT_LEVEL(RL_MGMT)) 
							printSmOutput(smOutput, i);

						/* perform the actions as returned from the state machine */
						actionTake(&smOutput, protoPtrLocal, i);
					}
				}

				/* send a deauthentication broadcast frame */
				wdrv_mgmtSendDisAssocBcast(smOutput.status);
			}
			else			/* unicast DisAssoc*/
			{
				rc = wdrv_dbFindStaIndex(pMessage->content.disAssocMng.macAddress, &i);
				if (rc != OK)
					break;

				if ((smCurrentState[i] != STATE_ASSOCIATED) && (smCurrentState[i] != STATE_AUTHORIZED))
					break;

				systemInfo.event = INPUT_DIS_ASSOC_MNG_UNICAST;
				systemInfo.status = pMessage->content.disAssocMng.reason;

				if(REPORT_LEVEL(RL_MGMT)) 
					printSmInput(systemInfo, i);

				/* activate the state machine */
				rc = lbt_smActions(systemInfo.event, i, &smOutput);
				
				if (rc != OK)
					break;

				if (smOutput.status == STATUS_UNSPECIFIED)
					smOutput.status = systemInfo.status;

				if(REPORT_LEVEL(RL_MGMT)) 
					printSmOutput(smOutput, i);

				/* perform the actions as returned from the state machine */
				actionTake(&smOutput, protoPtrLocal, i);
			}
			break;

		case MAC_AUTHORIZE:			/* a DisAssoc manager message arrived, force Disassociation for all stations. */
			if(REPORT_LEVEL(RL_MGMT)) 
				wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: MAC_AUTHORIZE message received\n");

			rc = wdrv_dbFindStaIndex(pMessage->content.macAuthorize.macAddress, &i);

			if (rc != OK)
				break;

			if (smCurrentState[i] != STATE_ASSOCIATED)
				break;

			systemInfo.event  = INPUT_MAC_AUTHORIZE;
			systemInfo.status = STATUS_UNSPECIFIED;


			if(REPORT_LEVEL(RL_MGMT)) 
				printSmInput(systemInfo, i);

			/* activate the state machine */
			rc = lbt_smActions(systemInfo.event, i, &smOutput);

			
			if (rc != OK)
				break;

			if (smOutput.status == STATUS_UNSPECIFIED)
				smOutput.status = systemInfo.status;

			if(REPORT_LEVEL(RL_MGMT)) 
				printSmOutput(smOutput, i);

			/* perform the actions as returned from the state machine */
			actionTake(&smOutput, protoPtrLocal, i);
			break;

		case MAC_DE_AUTHORIZE:			/* a DisAssoc manager message arrived, force Disassociation for all stations. */
			if(REPORT_LEVEL(RL_MGMT)) 
				wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: MAC_DE_AUTHORIZE message received\n");

			rc = wdrv_dbFindStaIndex(pMessage->content.macDeAuthorize.macAddress, &i);
			if (rc != OK)
				break;

			if (smCurrentState[i] != STATE_AUTHORIZED)
				break;

			systemInfo.event = INPUT_MAC_DE_AUTHORIZE;
			systemInfo.status = STATUS_UNSPECIFIED;

			if(REPORT_LEVEL(RL_MGMT)) 
				printSmInput(systemInfo, i);

			/* activate the state machine */
			rc = lbt_smActions(systemInfo.event, i, &smOutput);
			
			if (rc != OK)
				break;

			if (smOutput.status == STATUS_UNSPECIFIED)
				smOutput.status = systemInfo.status;

			if(REPORT_LEVEL(RL_MGMT)) 
				printSmOutput(smOutput, i);

			/* perform the actions as returned from the state machine */
			actionTake(&smOutput, protoPtrLocal, i);
			break;

		case RADIO_CALIBRATE_TIMEOUT:
			wdrv_mgmtRadioCalibrateTimeoutHandler();
			break;

	case RADIO_CALIBRATE_TIMEOUT_CRT:
			wdrv_mgmtRadioCalibrateCrtTimeoutHandler();
			break;

		default:
			break;
	}

	return OK;
}


/*************************************************************************
*                        resetParams									 *
**************************************************************************
DESCRIPTION:This function resets the main data structures of the module.
			It performs the following:
			1. Empty message queue
			2. Reset management table
			3. Stop timers
			4. Remove WEP mapping keys

INPUT:      None
OUTPUT:     None
RETURN:     OK/NOK
**************************************************************************/
static STATUS resetParams(void)
{
	int i;
	char staMac[MAC_ADDR_LEN];

	/* Stop the aging timer */
	PALSRV_tcbCancel(&agingTimer);

	/* Reset management table and stop authRspTimer (tcb timer) for each station.*/
	for (i = 0; i < MAX_STATIONS; i++)
	{
		/* reset the station state*/
		lbt_smResetState(i);

		PALSRV_tcbCancel(&mgmtTable[i].authTimer);

		/* reset entry except of the auth timer */
		PAL_memSet(&mgmtTable[i].timeStamp, 0, 4);	
		PAL_memSet(&mgmtTable[i].challengeText, 0, DOT11_CHALLENGE_TEXT_MAX);	

		wdrv_dbGetStaMacAddress(i, staMac);
	}

	return OK;
}


/*************************************************************************
*                        actionTake										 *
**************************************************************************
DESCRIPTION:This function performs actions as indicated by the state machine.
			There could be more than one action to be takes.
			The 	

INPUT:      smOutput - the action to be performed and the status to be asociatted with them.
			protoPtr - information as received in the mgmt frame.
			index - station index.	

OUTPUT:     None
RETURN:     NOne
**************************************************************************/
static void actionTake(output_T *smOutput, void	*protoPtr, UINT16 index)
{

	if(smOutput->action & ACTION_NOP)
		return; 

	if(smOutput->action & ACTION_SEND_WPA_ASS_REQ)
	{
		UINT8 rsnInfoEleLen;
		dot11_assocReq_T* assoPtr = (dot11_assocReq_T*)protoPtr;

		if (assoPtr->rsnIe == NULL)
			rsnInfoEleLen = 0;
		else
			rsnInfoEleLen = assoPtr->rsnIe->hdr.eleLen + 2;

		wdrv_dbSetWpaReq(index, ACTION_SEND_WPA_ASS_REQ);
		dda_wpa_canStationAss(index,(UINT8 *)(assoPtr->rsnIe), rsnInfoEleLen);
	}
	if(smOutput->action & ACTION_SEND_WPA_REASS_REQ)
	{
		UINT8 rsnInfoEleLen;
		dot11_reAssocReq_T* reassoPtr = (dot11_reAssocReq_T*)protoPtr;

		if (reassoPtr->rsnIe == NULL)
			rsnInfoEleLen = 0;
		else
			rsnInfoEleLen = reassoPtr->rsnIe->hdr.eleLen + 2;

		wdrv_dbSetWpaReq(index, ACTION_SEND_WPA_REASS_REQ);
		dda_wpa_canStationAss(index,(UINT8 *)(reassoPtr->rsnIe), rsnInfoEleLen);
	}
	if (smOutput->action & ACTION_START_TIMER)
		startAuthTimer(index);
	
	if (smOutput->action & ACTION_STOP_TIMER)
		stopAuthTimer(index);

	if (smOutput->action & ACTION_SEND_AUTH)
		wdrv_mgmtSendAuth(smOutput->status, (dot11_auth_T *)protoPtr, index);
	
	if (smOutput->action & ACTION_SEND_DE_AUTH)
		wdrv_mgmtSendDeAuthUnicastByIndex(smOutput->status, index);

	if (smOutput->action & ACTION_SET_STATION_PARAMS)
	{

		wdrv_dbSetStaSsidByIndex(index, 
								 ((dot11_assocReq_T*)protoPtr)->SSID->serviceSetId,
								 ((dot11_assocReq_T*)protoPtr)->SSID->hdr.eleLen);
		setStationParams(protoPtr, index, ACTION_SEND_ASSOC_RSP);
	}

	else if (smOutput->action & ACTION_UPDATE_STATION_PARAMS)
	{
		wdrv_dbSetStaSsidByIndex(index, 
								 ((dot11_reAssocReq_T*)protoPtr)->SSID->serviceSetId,
								 ((dot11_reAssocReq_T*)protoPtr)->SSID->hdr.eleLen);						
		updateStationParams(protoPtr, index, ACTION_SEND_RE_ASSOC_RSP);
	}

	if (smOutput->action & ACTION_SEND_ASSOC_RSP)
	{
		wdrv_mgmtSendAssocRsp(smOutput->status, index);
	}

/*		else if(REPORT_LEVEL(RL_MGMT)) 
			wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: ACTION_SEND_RE_ASSOC_RSP, and no ACTION_SET_STATION_PARAMS nor ACTION_UPDATE_STATION_PARAMS is required\n");*/
		
	if (smOutput->action & ACTION_SEND_RE_ASSOC_RSP)
	{
		wdrv_mgmtSendReAssocRsp(smOutput->status, index);
	}

	if (smOutput->action & ACTION_SEND_DIS_ASSOC)
		wdrv_mgmtSendDisAssocUnicastByIndex(smOutput->status, index);

	if (smOutput->action & ACTION_NOTIFY_CONNECT)
		notifyConnect(index);
	
	if (smOutput->action & ACTION_NOTIFY_DIS_CONNECT)
		notifyDisConnect(index);
	
	if (smOutput->action & ACTION_RESET_STATION_PARAMS)
		resetStationParams(index);

	/**** very important: must be the last action to be done ****/
	if (smOutput->action & ACTION_REMOVE_STATION)
	{
		/* Reset the station security attack flag */
		dda_wpa_SetStationSecAttack(index, FALSE);
		/****	The following function MUST be the LAST one	****/
		wdrv_dbRemoveStaIndex(index);
	}

	if(REPORT_LEVEL(RL_MGMT)) 
		wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "\n");

}

void staAuthTimerCB(UINT32 data) {
	mgmtMessage_T msg;

	msg.type = data & 0xffff;
	msg.content.authTimeout.index = (data & 0xffff0000) >> 16;
	/*	PAL_memCopy(&msg, (void *)data, sizeof(msg)); */

	wdrv_DDA_mgmt_setEvent(&msg);
}

/* start an authentication response timer for a station */
static void startAuthTimer(UINT16 index)
{
	/*	mgmtMessage_T msginfo; */
	UINT32 authRspTimeout;
	UINT32 msg;
	
	/*
	msginfo.type = AUTH_RESPONSE_TIMEOUT;
	msginfo.content.authTimeout.index = index;
	*/
	msg = AUTH_RESPONSE_TIMEOUT | ( ((UINT32)index) << 16);
	
	wdrv_dbGetStationConfigAuthRspTimeout(&authRspTimeout);

	PALSRV_tcbQueueAct(&mgmtTable[index].authTimer,
					  authRspTimeout, 
                      staAuthTimerCB,
					  (unsigned long *)msg, 0);     

	if(REPORT_LEVEL(RL_MGMT))
		wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Start Auth Timer.");
}

/* stops an authentication response timer for a station */
static void stopAuthTimer(UINT16 index)
{
	PALSRV_tcbCancel(&mgmtTable[index].authTimer);

	if(REPORT_LEVEL(RL_MGMT))
		wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Stop Auth Timer.");
}

/* Stations disconnect notification:
   1. Remove the index associated MAC from the local bridge (in case of fast roaming)
   2. Notify the WPA module that a station connected */
static void	notifyConnect(UINT16 index)
{
	char mac[MAC_ADDR_LEN];

	if(REPORT_LEVEL(RL_MGMT))
		wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Notify Connect.");

	wdrv_dbGetStaMacAddress(index, mac);
	/* should call DDA function which will use PSP API to remove the MAC address from the switch/bridge */
	tiwlan_ddaDeleteMacAddrFunc(mac);
	dda_wpa_notify_station_connect(index);
}


/* Stations disconnect notification:
   1. Notify the WPA module that a station dis connected 
   2. Remove the index associated MAC from the local bridge (in case of fast roaming)
*/
static void	notifyDisConnect(UINT16 index)
{
	char mac[MAC_ADDR_LEN];

	if(REPORT_LEVEL(RL_MGMT))
		wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Notify Dis Connect.");

	dda_wpa_notify_station_disconnect(index);
	wdrv_dbGetStaMacAddress(index, mac);
	tiwlan_ddaDeleteMacAddrFunc(mac);
}


/* receives the station index, returns the station state. */
void wdrv_mgmtGetState(UINT16 index, UINT32 *state)
{
	*state = smCurrentState[index];
}

/* receives the station index, returns the station string, used for denugging/print purposes. */
void wdrv_mgmtGetStateString(UINT16 index, char *stateString)
{
	PAL_memCopy(stateString, smStateDesc[smCurrentState[index]], strlen(smStateDesc[smCurrentState[index]]));
	stateString[strlen(smStateDesc[smCurrentState[index]])] = 0;
}

/* receives a mac address, returns TRUE if broadcat, otherwise returns FALSE. */
BOOL isMacBcast(char *mac)
{
	char	BROADCAST[MAC_ADDR_LEN] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

	return (PAL_memCmp(mac, BROADCAST, MAC_ADDR_LEN) == 0);
}

/* receives the station index, update a station timestamp. */
static void updateTimeStamp(UINT16 idx)
{
	mgmtTable[idx].timeStamp = PALSRV_getTick();
}

/* receives the station index, returns the station timestamp. */
UINT32 wdrv_mgmtReadTimeStamp(UINT16 idx)
{
	int timeStamp;

	timeStamp = mgmtTable[idx].timeStamp;

	return timeStamp;
}

/* receives the station index, update a station authentication timestamp. */
void wdrv_mgmtUpdateAuthTimeStamp(UINT16 idx)
{
	mgmtTable[idx].authTimeStamp = PALSRV_getTick();
}

/* receives the station index, returns the station authentication timestamp. */
UINT32 wdrv_mgmtReadAuthTimeStamp(UINT16 idx)
{
	int timeStamp;

	timeStamp = mgmtTable[idx].authTimeStamp;

	return timeStamp;
}

/* receives the station index, update a station asssociation timestamp. */
void wdrv_mgmtUpdateAssocTimeStamp(UINT16 idx)
{
	mgmtTable[idx].assocTimeStamp = PALSRV_getTick();
}

/* receives the station index, returns the station Association timestamp. */
UINT32 wdrv_mgmtReadAssocTimeStamp(UINT16 idx)
{
	int timeStamp;

	timeStamp = mgmtTable[idx].assocTimeStamp;

	return timeStamp;
}

/* receives the station index, return TRUE if the station is associated, otherwise returns FALSE. */
BOOL wdrv_mgmtIsStaAssoc(UINT16 idx)
{
	PAL_spinLock(&mgmtLock); /* START OF CRITICAL SECTION */
	if ((smCurrentState[idx] == STATE_ASSOCIATED) || (smCurrentState[idx] == STATE_AUTHORIZED))
	{	
	 	PAL_spinUnLock(&mgmtLock);  /* END OF CRITICAL SECTION */
		return TRUE;
	}
   	PAL_spinUnLock(&mgmtLock);  /* END OF CRITICAL SECTION */
	return FALSE;
}

/* receives the station index, return the station AID. */
UINT16 wdrv_mgmtGetStationAid(UINT16 index)
{
	return (0xc000 | (index + 1));
}

/* receives the station index and the challenge as received in the authentication frame sent by the station, and
	 compare it to the station challenge text as saved in the database. Return TRUE if equal, otherwise returns FALSE. */
BOOL wdrv_mgmtIsChallengeEqual(UINT16 index, char *challengeP)
{
	return (!PAL_memCmp(challengeP, mgmtTable[index].challengeText, DOT11_CHALLENGE_TEXT_MAX));
}

/* Fill the database with the challenge that was sent to the station */
void wdrv_mgmtSetChallenge(UINT16 index, char *text)
{
	PAL_memCopy(mgmtTable[index].challengeText, text, DOT11_CHALLENGE_TEXT_MAX);
}

/*************************************************************************
 *                        wdrv_mgmtIsSrcAllowed                          *
 *************************************************************************
DESCRIPTION: This function checks whether a frame is allowed to pass on BASED ON THE FRAME SOURCE. 
			 If needed, it opens an entry for the station in the database.
			 Called by RxData for each frame.

			If the frame is mgmt, the frame is always passed on, unless there is no entry for the station in the DB
			and there is no place for a new entry too.	
                                                                                                   
			If the frame is data, and the station is not associated, the frame is dropped and a	class 3 message 
			is sent to the mgmt task.

			If the frame is data, and the station is associated, the frame is passed on.

INPUT:      mac - the station mac address
			mpduType - The frame type

OUTPUT:      idx - pointer to the station index as retrived from the database
			 assocInd - indication whether this STA is associated or not 

RETURN:      TRUE - frame is allowed to pass on.
			 FALSE - frame is not allowed to pass on.

************************************************************************/
BOOL wdrv_mgmtIsSrcAllowed(char *mac, UINT8 mpduType, UINT8 woneIndex, UINT16 *idx, BOOL *assocInd)
{
   *assocInd = FALSE;
   
   if (mpduType != DOT11_FC_TYPE_MGMT)
   {
      register int rc = wdrv_dbFindStaIndex(mac, idx); 
      
      if( rc== OK)
      {
         if (wdrv_mgmtIsStaAssoc(*idx))
         {
            updateTimeStamp(*idx);
            *assocInd = TRUE;
            return TRUE;   
         } else
         {
            updateTimeStamp(*idx);
            wdrv_mgmtClass3Notify(mac, *idx,0);
            return FALSE; 
            
         }
      } else
      {
         wdrv_mgmtClass3Notify(mac, INVALID_INDEX,woneIndex);
         return FALSE;
      }
   } else
   {
      register int rc = wdrv_dbFindAndInsertStaIndex(mac, woneIndex, idx); 
      
      if( rc != OK) {
         /* no place for the station */
         return FALSE;
      } else 
      {
         updateTimeStamp(*idx);
         return TRUE;   
      }
   }

   return TRUE;
}/* END wdrv_mgmtIsSrcAllowed() */

/*************************************************************************
 *                        wdrv_mgmtIsDstAllowed                          *
 *************************************************************************
DESCRIPTION: This function checks whether a frame is allowed to pass on BASED ON THE FRAME DESTINATION 
			 Called by the bridge module for each frame.

			If the station is not associated, the frame is dropped.

			If the station is associated, the frame is passed on.

INPUT:      mac - the station mac address

OUTPUT:      idx - pointer to the station index as retrived from the database

RETURN:      TRUE - frame is allowed to pass on.
			 FALSE - frame is not allowed to pass on.

************************************************************************/
BOOL wdrv_mgmtIsDstAllowed(char *mac, UINT16 *idx)
{
	int rc;
	
	rc = wdrv_dbFindStaIndex(mac, idx);

	if (rc != OK)
		return FALSE;

	else if (wdrv_mgmtIsStaAssoc(*idx))
		return TRUE;	

	else 
		return FALSE;	
}

BOOL wdrv_mgmtIsStationAuthorized(UINT16 index)
{
	if (smCurrentState[index] == STATE_AUTHORIZED)
	{	
		return TRUE;
	}
	return FALSE;
}

BOOL wdrv_mgmtIsBroadcastAuthorized(void)
{
	if (mgmtBcastState == STATE_AUTHORIZED)
	{	
		return TRUE;
	}
	else
		return FALSE;
}

void wdrv_mgmtSetStationStateDebug(UINT16 index, states state)
{
	smCurrentState[index] = state;
}

/* forward a mgmt frame to the mgmt task. */
STATUS wdrv_mgmtForwardMgmtFrame(mem_MSDU_T *msduP, UINT16 idx)
{
	mgmtMessage_T	msg;

	msg.type = MGMT_FRAME;
	msg.content.frame.msduPtr = msduP;
	msg.content.frame.index = idx;

	wdrv_DDA_mgmt_setEvent(&msg);

	return OK;
}

/* forward a beacon frame to the mgmt task. */
STATUS wdrv_mgmtForwardBeacon(mem_BD_T* pMpdu, UINT32 mpduLen)
{
	mem_MSDU_T*		msduPtr;
	mgmtMessage_T	msg;

	wlan_memMngrAllocMSDUBufferOnly(&msduPtr);
	if(msduPtr == NULL)
	{
        return NOK;
	}
	msduPtr->headerLen = WLAN_HDR_LEN;
	msduPtr->firstBDPtr = pMpdu;
	msduPtr->dataLen = mpduLen;	

	msg.type = BEACON_FRAME;
	msg.content.frame.msduPtr = msduPtr;

	wdrv_DDA_mgmt_setEvent(&msg);

	return OK;
}

/* forward a templates update request to the mgmt task */
STATUS wdrv_mgmtSendTemplateUpdateRequest(UINT32 reqCont, UINT32 val)
{
	mgmtMessage_T msg;

	msg.type = SET_TEMPLATE_REQ;
	msg.content.MessageInfo[0] = reqCont;
	msg.content.MessageInfo[1] = val;

	wdrv_DDA_mgmt_setEvent(&msg);

	return OK;
}


/* send a class 3 message to the mgmt task. */
void wdrv_mgmtClass3Notify(char *mac, UINT16 idx, UINT8 woneIndex )
{
	mgmtMessage_T	msg;

	msg.type = CLASS3;
	PAL_memCopy(msg.content.class3.macAddress, mac, MAC_ADDR_LEN);
	msg.content.class3.index = idx;
    msg.content.class3.wone_index = woneIndex;
    
	wdrv_DDA_mgmt_setEvent(&msg);
}

/* send a de auhtentication MANAGER message to the mgmt task. */
void wdrv_mgmtDeAuthMng(char *mac, UINT16 reason)
{
	mgmtMessage_T	msg;

	msg.type = DEAUTH_MNG;
	PAL_memCopy(msg.content.deAuthMng.macAddress, mac, MAC_ADDR_LEN);
	msg.content.deAuthMng.reason = reason;

    wdrv_DDA_mgmt_setEvent(&msg);
}

/* send a dis association MANAGER message to the mgmt task. */
void wdrv_mgmtDisAssocMng(char *mac, UINT16 reason)
{
	mgmtMessage_T	msg;

	msg.type = DISASSOC_MNG;
	PAL_memCopy(msg.content.disAssocMng.macAddress, mac, MAC_ADDR_LEN);
	msg.content.disAssocMng.reason = reason;

    wdrv_DDA_mgmt_setEvent(&msg);
}

/* send a MAC AUTHORIZE message to the mgmt task. */
void wdrv_mgmtMacAuthorize(char *mac)
{
	mgmtMessage_T	msg;

	msg.type = MAC_AUTHORIZE;
	PAL_memCopy(msg.content.macAuthorize.macAddress, mac, MAC_ADDR_LEN);

    wdrv_DDA_mgmt_setEvent(&msg);
}

/* send a MAC AUTHORIZE message to the mgmt task. */
void wdrv_mgmtMacDeAuthorize(char *mac)
{
	mgmtMessage_T	msg;

	msg.type = MAC_DE_AUTHORIZE;
	PAL_memCopy(msg.content.macDeAuthorize.macAddress, mac, MAC_ADDR_LEN);

    wdrv_DDA_mgmt_setEvent(&msg);
}

/* enable/disable the mgmt log */
void wdrv_mgmtSetLog(BOOL mode)
{
	mgmtLog = mode;
}

/* return the mgmt aging mode */
BOOL wdrv_mgmtAgingEnable(void)
{
	return agingMode;
}

/* enable/disable the aging mechanism */
void wdrv_mgmtSetAging(BOOL mode)
{
	agingMode = mode;
}

/* return the mgmt log status*/
BOOL wdrv_mgmtLogEnable(void)
{
	return mgmtLog;
}

/* reset the mgmt module, called by the console */
void wdrv_mgmtReset(void)
{
	int i;
	char staMac[MAC_ADDR_LEN];

	/* Reset management table and stop authRspTimer (tcb timer) for each station.*/
	for (i = 0; i < MAX_STATIONS; i++)
	{
		/* reset the station state*/
		lbt_smResetState(i);

		PALSRV_tcbCancel(&mgmtTable[i].authTimer);

		/* reset entry except of the auth timer */
		PAL_memSet(&mgmtTable[i].timeStamp, 0, 4);	
		PAL_memSet(&mgmtTable[i].challengeText, 0, DOT11_CHALLENGE_TEXT_MAX);	

		wdrv_dbGetStaMacAddress(i, staMac);
	}
}

int wdrv_mgmtPrintChallenge(UINT16 index ,char *buf)
{
	int i, j = 0,len=0;

	for (i = 0; i < DOT11_CHALLENGE_TEXT_MAX; i++)
	{
		len +=sprintf(buf +len,"%x ", mgmtTable[index].challengeText[i]);
		j++;
		if (j == 16)
		{
			j = 0;
			len +=sprintf(buf +len,"\n");
		}
	}
	len +=sprintf(buf +len,"\n");
	return len;
}

/* ptins the state machine input.output for debugging purposes.*/
static void	printSmInput(wdrv_mgmtSysInfo_T systemInfo, UINT16 index)
{
	wdrv_utilsReport(RL_MGMT, PRI_EMERGENCY, "WDRV_MGMT: Old State - %s	SM Input - %s	Status  - %s", smStateDesc[smCurrentState[index]], smInputDesc[systemInfo.event], smStatusDesc[systemInfo.status]);
}

static void	printSmOutput(output_T smOutput, UINT16 index)
{
	wdrv_utilsReport(RL_MGMT, PRI_EMERGENCY, "WDRV_MGMT: New State - %s	SM Action - 0x%x		Status  - %s\n", smStateDesc[smCurrentState[index]], smOutput.action, smStatusDesc[smOutput.status]);
}

static void	setStationParams(void *protoPtr, UINT16 index, UINT32 action)
{
	dot11_reAssocReq_T *reAssocReqP;
	dot11_assocReq_T *assocReqP;
	BOOL	support;

	switch (action)
	{
	case ACTION_SEND_ASSOC_RSP:
			assocReqP = (dot11_assocReq_T *)protoPtr;

			wdrv_dbSetStationRsnIe(index, assocReqP->rsnIe);
			wdrv_dbSetStationCapabilities(index, assocReqP->capabilities);
			wdrv_dbSetListenInterval(index, assocReqP->listenInterval);
			wdrv_dbSetStationTIcapEnable(index, assocReqP->TIouiSupport);
						
			support = (BOOL)assocReqP->fourXSupport & FOURX_SUPPORT_CONCAT_MASK;
			if(support)
				wdrv_dbSetStationConcatMaxSize(index, assocReqP->ConcatMaxSize);
			wdrv_dbSetStationRates(index, assocReqP->supportedRates, assocReqP->extSupportedRates, support);
			if(assocReqP->TIouiSupport)
			{			
				support = (BOOL)assocReqP->fourXSupport & FOURX_SUPPORT_ACK_EMUL_MASK;
				wdrv_dbSetStationAckEmulSupport(index, support);
			}
			wdrv_dbSetTIprotectionSupport(index, &assocReqP->TIprotectionSupport);
			wdrv_dbSetCommonFeatureSet(index, &assocReqP->CommonFeatureSet);
			break;


	case ACTION_SEND_RE_ASSOC_RSP:
			reAssocReqP = (dot11_reAssocReq_T *)protoPtr;
			wdrv_dbSetStationRsnIe(index, reAssocReqP->rsnIe);
			wdrv_dbSetStationCapabilities(index, reAssocReqP->capabilities);
			wdrv_dbSetListenInterval(index, reAssocReqP->listenInterval);
			wdrv_dbSetStationTIcapEnable(index, reAssocReqP->TIouiSupport);
			support = (BOOL)reAssocReqP->fourXSupport & FOURX_SUPPORT_CONCAT_MASK;
			if(support)
				wdrv_dbSetStationConcatMaxSize(index, reAssocReqP->ConcatMaxSize);
			wdrv_dbSetStationRates(index, reAssocReqP->supportedRates, reAssocReqP->extSupportedRates, support);
			if(reAssocReqP->TIouiSupport)
			{
				support = (BOOL)reAssocReqP->fourXSupport & FOURX_SUPPORT_ACK_EMUL_MASK;
				wdrv_dbSetStationAckEmulSupport(index, support);
			}
			wdrv_dbSetTIprotectionSupport(index, &reAssocReqP->TIprotectionSupport);
			wdrv_dbSetCommonFeatureSet(index, &reAssocReqP->CommonFeatureSet);
			break;

	default:
		return;
	}

	implementPbccStaticAlgorithmIfNeeded(index, ASSOCIATE);

	implementPreambleStaticAlgorithmIfNeeded(index, ASSOCIATE);

	implementSlotTimeStaticAlgorithmIfNeeded(index, ASSOCIATE);

	implementERPProtectionAlgorithmIfNeeded(index, ASSOCIATE);

	if(REPORT_LEVEL(RL_MGMT))
		wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Set station params.");
}

static void	updateStationParams(void *protoPtr, UINT16 index, UINT32 action)
{
	dot11_assocReq_T *assocReqP;
	dot11_reAssocReq_T *reAssocReqP;

	UINT16	oldCaps, newCaps;
	UINT32	oldPbccMode, newPbccMode;
	UINT32	oldPreambleMode, newPreambleMode;
	UINT32	oldSlotTimeMode, newSlotTimeMode;
	BOOL	support;


	/* Get old station capabilities */
	wdrv_dbGetStationCapabilities(index, &oldCaps);

	/* Get new station capabilities */
	/* NOTICE:	we count on the fact that both in association request and re association request
			the capability field is located in the same offset. */
	assocReqP = (dot11_assocReq_T *)protoPtr;
	PAL_memCopy(&newCaps, assocReqP->capabilities, 2);

	oldPbccMode = (oldCaps >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK;
	newPbccMode = (newCaps >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK;

	if (oldPbccMode != newPbccMode)
	{
		implementPbccStaticAlgorithmIfNeeded(index, DISASSOCIATE);
	}

	oldPreambleMode = (oldCaps >> CAP_PREAMBLE_SHIFT) & CAP_PREAMBLE_MASK;
	newPreambleMode = (newCaps >> CAP_PREAMBLE_SHIFT) & CAP_PREAMBLE_MASK;

	if (oldPreambleMode != newPreambleMode)
	{
		implementPreambleStaticAlgorithmIfNeeded(index, DISASSOCIATE);
	}

	oldSlotTimeMode = (oldCaps >> CAP_SLOTTIME_SHIFT) & CAP_SLOTTIME_MASK;
	newSlotTimeMode = (newCaps >> CAP_SLOTTIME_SHIFT) & CAP_SLOTTIME_MASK;

	if (oldSlotTimeMode != newSlotTimeMode)
	{
		implementSlotTimeStaticAlgorithmIfNeeded(index, DISASSOCIATE);
	}

	implementERPProtectionAlgorithmIfNeeded(index, DISASSOCIATE);

	switch (action)
	{
	case ACTION_SEND_ASSOC_RSP:
			assocReqP = (dot11_assocReq_T *)protoPtr;
			wdrv_dbSetStationRsnIe(index, assocReqP->rsnIe);
			wdrv_dbSetStationCapabilities(index, assocReqP->capabilities);
			wdrv_dbSetListenInterval(index, assocReqP->listenInterval);
			wdrv_dbSetStationTIcapEnable(index, assocReqP->TIouiSupport);
			support = (BOOL)assocReqP->fourXSupport & FOURX_SUPPORT_CONCAT_MASK;
			if(support)
				wdrv_dbSetStationConcatMaxSize(index, assocReqP->ConcatMaxSize);
			wdrv_dbSetStationRates(index, assocReqP->supportedRates, assocReqP->extSupportedRates, support);
			if(assocReqP->TIouiSupport)
			{			
				support = (BOOL)assocReqP->fourXSupport & FOURX_SUPPORT_ACK_EMUL_MASK;
				wdrv_dbSetStationAckEmulSupport(index, support);
			}
			wdrv_dbSetTIprotectionSupport(index, &assocReqP->TIprotectionSupport);
			wdrv_dbSetCommonFeatureSet(index, &assocReqP->CommonFeatureSet);
			break;

	case ACTION_SEND_RE_ASSOC_RSP:
			reAssocReqP = (dot11_reAssocReq_T *)protoPtr;
			wdrv_dbSetStationRsnIe(index, reAssocReqP->rsnIe);
			wdrv_dbSetStationCapabilities(index, reAssocReqP->capabilities);
			wdrv_dbSetListenInterval(index, reAssocReqP->listenInterval);
			wdrv_dbSetStationTIcapEnable(index, reAssocReqP->TIouiSupport);
			support = (BOOL)reAssocReqP->fourXSupport & FOURX_SUPPORT_CONCAT_MASK;
			if(support)
				wdrv_dbSetStationConcatMaxSize(index, reAssocReqP->ConcatMaxSize);
			wdrv_dbSetStationRates(index, reAssocReqP->supportedRates, reAssocReqP->extSupportedRates, support);
			if(reAssocReqP->TIouiSupport)
			{
				support = (BOOL)reAssocReqP->fourXSupport & FOURX_SUPPORT_ACK_EMUL_MASK;
				wdrv_dbSetStationAckEmulSupport(index, support);
			}
			wdrv_dbSetTIprotectionSupport(index, &reAssocReqP->TIprotectionSupport);
			wdrv_dbSetCommonFeatureSet(index, &reAssocReqP->CommonFeatureSet);
			break;

	default:
		return;
	}

	if (oldPbccMode != newPbccMode)
	{
		implementPbccStaticAlgorithmIfNeeded(index, ASSOCIATE);
	}

	if (oldPreambleMode != newPreambleMode)
	{
		implementPreambleStaticAlgorithmIfNeeded(index, ASSOCIATE);
	}

	if (oldSlotTimeMode != newSlotTimeMode)
	{
		implementSlotTimeStaticAlgorithmIfNeeded(index, ASSOCIATE);
	}

	implementERPProtectionAlgorithmIfNeeded(index, ASSOCIATE);

	if(REPORT_LEVEL(RL_MGMT))
		wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Update station params.");
}

static void	resetStationParams(UINT16 index)
{
	implementPbccStaticAlgorithmIfNeeded(index, DISASSOCIATE);

	implementPreambleStaticAlgorithmIfNeeded(index, DISASSOCIATE);

	implementSlotTimeStaticAlgorithmIfNeeded(index, DISASSOCIATE);

	implementERPProtectionAlgorithmIfNeeded(index, DISASSOCIATE);

	wdrv_dbResetStationParams(index);

	if(REPORT_LEVEL(RL_MGMT))
		wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Reset station params.");
}

void setBcastState(BOOL authorized)
{
	PAL_spinLock(&mgmtLock); /* START OF CRITICAL SECTION */

	if (authorized == TRUE)
		mgmtBcastState = STATE_AUTHORIZED;
	else
		mgmtBcastState = STATE_ASSOCIATED;

	PAL_spinUnLock(&mgmtLock);  /* END OF CRITICAL SECTION */
}

/* Implement the PBCC static algorithm if needed */
static void implementPbccStaticAlgorithmIfNeeded(UINT16 index, algo_action_t action)
{
	UINT8  PbccAlgorithmMode;

	wdrv_dbGetPBCCAlgorithmMode(0,&PbccAlgorithmMode);

	if ((PbccAlgorithmMode == PBCC_ALGO_LOCAL_STATIONS) || (PbccAlgorithmMode==PBCC_ALGO_DYNAMIC) || (PbccAlgorithmMode==PBCC_ALGO_ENHANCED_DYNAMIC))
	{
		wdrv_dbPerformPBCCAlgorithmStatic(index, action);
		if(REPORT_LEVEL(RL_DYNAMIC_ALG))
			wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_INFORMATION, "WDRV_MGMT: Perform PBCC algorithm.");
	}
}

/* Implement the Preamble static algorithm if needed */
static void implementPreambleStaticAlgorithmIfNeeded(UINT16 index, algo_action_t action)
{
	UINT8  preambleMode;

	wdrv_dbGetPreambleMode(0,&preambleMode);

	if (preambleMode == PREAMBLE_MODE_LOCAL_STATIONS)
	{
		wdrv_dbPerformPreambleAlgorithmStatic(index, action);
		if(REPORT_LEVEL(RL_DYNAMIC_ALG))
			wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_INFORMATION, "WDRV_MGMT: Perform Preamble algorithm.");
	}
}

/* Implement the Slot Time static algorithm if needed */
static void implementSlotTimeStaticAlgorithmIfNeeded(UINT16 index, algo_action_t action)
{
	UINT8  slotTimeMode;

	wdrv_dbGetSlotTimeMode(0,&slotTimeMode);

	if (slotTimeMode == SLOTTIME_MODE_LOCAL_STATIONS || slotTimeMode == SLOTTIME_MODE_ENHANCED_DYNAMIC)
	{
		wdrv_dbPerformSlotTimeAlgorithmStatic(index, action);
		if(REPORT_LEVEL(RL_DYNAMIC_ALG))
			wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_INFORMATION, "WDRV_MGMT: Perform Slot Time algorithm.");
	}
}

/* Implement the ERP Protection algorithm if needed */
static void implementERPProtectionAlgorithmIfNeeded(UINT16 index, algo_action_t action)
{
	UINT8  ERPProtectionMode;
    UINT8  wone_index ,band;
    
    wdrv_dbGetStationWoneIndex(index,&wone_index);
    wdrv_dbGetBand(wone_index, &band);

	wdrv_dbGetERPProtectionMode(0,&ERPProtectionMode);

	if ((ERPProtectionMode == PROTECTION_MODE_LOCAL_STATIONS ||
		ERPProtectionMode == PROTECTION_MODE_DYNAMIC ||
		ERPProtectionMode == PROTECTION_MODE_ENHANCED_DYNAMIC)&&
        (band == RADIO_BAND_2_4GHZ))
	{
		wdrv_dbPerformERPProtectionAlgorithmStatic(index, action);
		if(REPORT_LEVEL(RL_DYNAMIC_ALG))
			wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_INFORMATION, "WDRV_MGMT: Perform ERP Protection algorithm.");
	}
}

/*************************************************************************
 *                        wdrv_mgmtERPtimeoutHandler                     *
 *************************************************************************
DESCRIPTION: This function called on timeout under context of MGMT Task,
             in order to manage ERP beacons aging mechanism.			 

OUTPUT: 

RETURN:  STATUS
************************************************************************/
STATUS wdrv_mgmtERPtimeoutHandler()
{
	UINT16 	      ErpStaNum,NonErpStaNum;

#ifdef ERP_DYNAMIC_DEBUG
	static int count=0;

	count++;

	if(count == 2)
		TrcEn = FALSE;
#endif

	BeaconERPinfo.active = FALSE;

	if(BeaconERPinfo.valid)
	{
		if(REPORT_LEVEL(RL_DYNAMIC_ALG))
			wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_INFORMATION, "WDRV_MGMT: No ERP beacons during timeout");

		/* turn the entry to not valid */
		BeaconERPinfo.valid = FALSE;
		/* reset maxBeaconInterval */			
		ERPmaxBeaconInterval = 0;
		
		wdrv_dbGetStationsNumForERP(&NonErpStaNum, &ErpStaNum);
		if(NonErpStaNum == 0)
		{
			/* update ERP IE */
			if(REPORT_LEVEL(RL_DYNAMIC_ALG))
				printf("going to set ERP4: %d,%d\n",0,0);

			whal_apiSetERPProtection(0,0,0);									 
		}
	}
	return OK; 
}

/*************************************************************************
 *                        ERPbeaconAgingTimerCB                     *
 *************************************************************************
DESCRIPTION: beacon aging timer callback function

OUTPUT: None

RETURN:  None
************************************************************************/
static void ERPbeaconAgingTimerCB(UINT32 data)
{
	mgmtMessage_T	msg;

	msg.type = BEACON_ERP_TIMEOUT;

	wdrv_DDA_mgmt_setEvent(&msg);
}

/*************************************************************************
 *                        wdrv_mgmtERPdynamicManager                     *
 *************************************************************************
DESCRIPTION: This function makes a decision regarding the ERP protection.

OUTPUT: decision

RETURN:  None
************************************************************************/
void wdrv_mgmtERPdynamicManager(UINT8 nonERPpresent,
								UINT8 useProtection,
								UINT8 noOFDMap,
								ERPdecision_T *decision,
								UINT16 beaconInterval)
{
	UINT32 CurrTimestamp = PALSRV_getTick();
	UINT16 ErpStaNum,NonErpStaNum; 

	decision->decisionDone = FALSE;

	/* no trigger for ERP, just return */
	if((nonERPpresent | noOFDMap) == 0)
		return;

	/* update maximal currently known beacon interval */
	if(ERPmaxBeaconInterval < beaconInterval)
	{
		ERPmaxBeaconInterval = beaconInterval;
	}

	/* update the timestamp */
	BeaconERPinfo.timestamp = CurrTimestamp;
	BeaconERPinfo.active = TRUE;
	
	/* make the decision only if the entry is not valid */ 
	if(!BeaconERPinfo.valid)
	{	
		wdrv_dbGetStationsNumForERP(&NonErpStaNum, &ErpStaNum);

		decision->decisionDone = TRUE;
		decision->useProtectionBit = TRUE;
		if(NonErpStaNum > 0)
			decision->nonERPpresentBit = TRUE;
		else
			decision->nonERPpresentBit = FALSE;

		BeaconERPinfo.valid = TRUE;		
	}

	/* stop aging timer */
	PALSRV_tcbCancel(&ERPbeaconAgingTimer);

	/* start aging timer */
	PALSRV_tcbQueueAct(&ERPbeaconAgingTimer,
					   ERPmaxBeaconInterval*6,
					   ERPbeaconAgingTimerCB,
					   0, 0);					   
}

/*************************************************************************
 *                        wdrv_mgmtIsERPexternalActive                   *
 *************************************************************************
DESCRIPTION: This function reports whether there is a need for ERP protection
             as result of dynamic algorithm

OUTPUT: None

RETURN:  None
************************************************************************/
void wdrv_mgmtIsERPexternalActive(BOOL *active)
{
	UINT8  ERPProtectionMode;
	
	wdrv_dbGetERPProtectionMode(0,&ERPProtectionMode);

	if (ERPProtectionMode == PROTECTION_MODE_DYNAMIC || ERPProtectionMode == PROTECTION_MODE_ENHANCED_DYNAMIC)	
		*active = BeaconERPinfo.active;
	else
		*active = FALSE;
}


/*************************************************************************
 *                        wdrv_mgmtPBCCtimeoutHandler                     *
 *************************************************************************
DESCRIPTION: This function called on timeout under context of MGMT Task,
             in order to manage PBCC beacons aging mechanism.			 

OUTPUT: 

RETURN:  STATUS
************************************************************************/
STATUS wdrv_mgmtPBCCtimeoutHandler()
{
	UINT16 notPBCCcapableStaNum;
	BOOL mixMode4x, status4x;
	UINT8	activeRate;

	BeaconPBCCinfo.active = FALSE;

	if(BeaconPBCCinfo.valid)
	{
		if(REPORT_LEVEL(RL_DYNAMIC_ALG))
			wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_INFORMATION, "WDRV_MGMT: No PBCC beacons during timeout");

		/* turn the entry to not valid */
		BeaconPBCCinfo.valid = FALSE;
		/* reset maxBeaconInterval */			
		PBCCmaxBeaconInterval = 0;
		
		wdrv_dbGetNumOfStationNotPBCCCapable(&notPBCCcapableStaNum);
		if(notPBCCcapableStaNum == 0)
		{
			if(REPORT_LEVEL(RL_DYNAMIC_ALG))
				printf("going to set PBCC1: %d\n",1);

			/* enable pbcc */
			wdrv_dbSetPBCCStationsSupport(TRUE);
			whal_apiSetPbcc(0,1);
			wdrv_dbSetCurrentPBCCSupportMode(TRUE);
            
			wdrv_dbGet4xMixMode(&mixMode4x);
            wdrv_dbGet4xEnable(&status4x);
			wdrv_dbGetMaxActiveRate(&activeRate);
			
			if(( mixMode4x == FALSE )&&( status4x == TRUE )&&(activeRate <= DOT11_RATE_22))				
				wdrv_4xMngr_start4x();
		}
	}
	return OK;
}

/*************************************************************************
 *                        PBCCbeaconAgingTimerCB                     *
 *************************************************************************
DESCRIPTION: beacon aging timer callback function

OUTPUT: None

RETURN:  None
************************************************************************/
static void PBCCbeaconAgingTimerCB(UINT32 data)
{
	mgmtMessage_T	msg;

	msg.type = BEACON_PBCC_TIMEOUT;

	wdrv_DDA_mgmt_setEvent(&msg);
}

/*************************************************************************
 *                        wdrv_mgmtPBCCdynamicManager                     *
 *************************************************************************
DESCRIPTION: This function makes a decision regarding the PBCC mode.

OUTPUT: decision

RETURN:  None
************************************************************************/
void wdrv_mgmtPBCCdynamicManager(BOOL pbcc, UINT16 beaconInterval)
{
	BOOL mixMode4x, status4x;
	UINT8	activeRate;
	
	/* AP has nothing to do when pbcc enabled beacon received */
	if(pbcc)
		return;

	/* update maximal currently known beacon interval */
	if(PBCCmaxBeaconInterval < beaconInterval)
	{
		PBCCmaxBeaconInterval = beaconInterval;
	}

	BeaconPBCCinfo.active = TRUE;

	if(!BeaconPBCCinfo.valid)
	{
		if(REPORT_LEVEL(RL_DYNAMIC_ALG))
			printf("going to set PBCC2: %d\n",0);

		/* disable PBCC */
		wdrv_dbSetPBCCStationsSupport(FALSE);
		whal_apiSetPbcc(0,0);
		wdrv_dbSetCurrentPBCCSupportMode(FALSE);

		wdrv_dbGet4xMixMode(&mixMode4x);
		wdrv_dbGet4xEnable(&status4x);
		wdrv_dbGetMaxActiveRate(&activeRate);
		if(( mixMode4x == FALSE )&&( status4x == TRUE )&&(activeRate <= DOT11_RATE_22))         
			wdrv_4xMngr_stop4x();

		BeaconPBCCinfo.valid = TRUE;
	}

	/* stop aging timer */
	PALSRV_tcbCancel(&PBCCbeaconAgingTimer);

	/* start aging timer */
	PALSRV_tcbQueueAct(&PBCCbeaconAgingTimer,
					   PBCCmaxBeaconInterval*6,
					   PBCCbeaconAgingTimerCB,
					   0, 0);					   

}
/*************************************************************************
 *                        wdrv_mgmtIsPBCCexternalActive                  *
 *************************************************************************
DESCRIPTION: This function reports whether there is a need for PBCC disable
             as result of dynamic algorithm

OUTPUT: None

RETURN:  None
************************************************************************/
void wdrv_mgmtIsPBCCexternalActive(BOOL *active)
{
	UINT8  PbccAlgorithmType;

	wdrv_dbGetPBCCAlgorithmMode(0,&PbccAlgorithmType);


	if (PbccAlgorithmType == PBCC_ALGO_DYNAMIC || PbccAlgorithmType == PBCC_ALGO_ENHANCED_DYNAMIC)	
		*active = BeaconPBCCinfo.active;
	else
		*active = FALSE;
}

/*************************************************************************
 *                        wdrv_mgmtPerformDynamicAlgorithms              *
 *************************************************************************
DESCRIPTION: This function calls to Beacon Parser and performs ERP and PBCC
			 dynamic algorithms.

OUTPUT: None

RETURN:  None
************************************************************************/
STATUS wdrv_mgmtPerformDynamicAlgorithms(mem_MSDU_T *msduPtr)
{
	UINT8  nonERPpresent;
	UINT8  useProtection;
	UINT8  nonOFDMap;
	UINT16 beaconInterval;
	ERPdecision_T decision;
	STATUS rc;
	mem_MSDU_T *msduP = msduPtr;
	BOOL   pbccSupportBeacon;
	UINT8  ERPProtectionMode;
	UINT8  PbccAlgorithmMode;

	
	rc = BeaconParse(msduP, &nonERPpresent, &useProtection, &nonOFDMap, &beaconInterval, &pbccSupportBeacon);
	if(rc == NOK)
	{
		if(REPORT_LEVEL(RL_DYNAMIC_ALG))
			printf("Failed in wdrv_rxParseBeacon()\n");
		return NOK;
	}
	
	wdrv_dbGetERPProtectionMode(0,&ERPProtectionMode);
	if (ERPProtectionMode == PROTECTION_MODE_DYNAMIC || ERPProtectionMode == PROTECTION_MODE_ENHANCED_DYNAMIC)
	{
		wdrv_mgmtERPdynamicManager(nonERPpresent, useProtection, nonOFDMap, &decision, beaconInterval);

		if(decision.decisionDone)
		{
			if(REPORT_LEVEL(RL_DYNAMIC_ALG))
				printf("going to set ERP5: %d,%d\n",decision.nonERPpresentBit,decision.useProtectionBit);				   
			
			/* update ERP IE */
			whal_apiSetERPProtection(0, decision.nonERPpresentBit, decision.useProtectionBit);
		}
	}
	
	wdrv_dbGetPBCCAlgorithmMode(0,&PbccAlgorithmMode);

	if (PbccAlgorithmMode == PBCC_ALGO_DYNAMIC || PbccAlgorithmMode == PBCC_ALGO_ENHANCED_DYNAMIC)	
		wdrv_mgmtPBCCdynamicManager(pbccSupportBeacon, beaconInterval);

	return OK;
}

/*************************************************************************
 *                        wdrv_mgmtDynamicAlgTimeoutHandler              *
 *************************************************************************
DESCRIPTION: This function is PBCC, ERP, Slot time dynamic algorithms
             periodic timer callback function.

OUTPUT: None

RETURN:  None
************************************************************************/
void wdrv_mgmtDynamicAlgTimeoutHandler()
{
	UINT32 CurrTimestamp = PALSRV_getTick();
    UINT32 LastTimestamp;
    BOOL   pbccDisableValid, erpEnableValid;
    BOOL   externalActive;
    BOOL   mixMode4x, status4x;
    UINT8  PBCCalgoMode,ERPProtectionMode;
	UINT32 timeout;
	UINT8  activeRate;

	wdrv_dbGetDynamicAlgorithmsTimeout(&timeout);
	timeout = PALSRV_msec2ticks(timeout);	
    
    wdrv_dbGetPBCCAlgorithmMode(0,&PBCCalgoMode);
    if(PBCCalgoMode == PBCC_ALGO_ENHANCED_DYNAMIC)
    {
        wdrv_dbGetPBCCdynamicStatus(&pbccDisableValid);
        wdrv_mgmtIsPBCCexternalActive(&externalActive);
        /* process only if PBCC was disabled due to enhanced dynamic alg. and there is no external interference */
    	if(pbccDisableValid && !externalActive)
        {
            wdrv_dbGetNotPBCCTimestamp(&LastTimestamp);
            if((CurrTimestamp - LastTimestamp) > timeout)
            {
                /* Now we can enble PBCC */
    			wdrv_dbSetPBCCStationsSupport(TRUE);
    			whal_apiSetPbcc(0,1);
    			wdrv_dbSetCurrentPBCCSupportMode(TRUE);
                
                wdrv_dbGet4xMixMode(&mixMode4x);
                wdrv_dbGet4xEnable(&status4x);
				wdrv_dbGetMaxActiveRate(&activeRate);
                /* If 4x can work only in PBCC, enable the 4X support now.*/
    			if(( mixMode4x == FALSE )&&( status4x == TRUE ) &&(activeRate <= DOT11_RATE_22))				
    				wdrv_4xMngr_start4x();
    
                wdrv_dbSetPBCCdynamicStatus(FALSE);
    
            }
        }
    }
	
    wdrv_dbGetERPProtectionMode(0,&ERPProtectionMode);				
    if (ERPProtectionMode == PROTECTION_MODE_ENHANCED_DYNAMIC)
    {
        /* in case of external interference active, we have to activate protection any way*/
        wdrv_mgmtIsERPexternalActive(&externalActive);
        wdrv_dbGetERPdynamicStatus(&erpEnableValid);
        if(erpEnableValid && !externalActive)
        {
            wdrv_dbGetNotErpTimestamp(&LastTimestamp);
            if((CurrTimestamp - LastTimestamp) > timeout)
            {
                /* disable protection */
                whal_apiSetERPProtection(0,0,0);

                wdrv_dbSetERPdynamicStatus(FALSE);
            }
        }
    }

	wdrv_mgmtShortSlotTimeSM(INPUT_SLOTTIME_CHECK_TIMEOUT);
}


/*************************************************************************
 *                        wdrv_mgmtHandleSetTemplateReq		             *
 *************************************************************************
DESCRIPTION: This function updates Beacon and/or ProbeRsp templates as  
			 specified by the request.

OUTPUT: None

RETURN:  None
************************************************************************/
void wdrv_mgmtHandleSetTemplateReq(UINT32 ReqType, UINT32 val)
{
	switch (ReqType)
	{
	case TEMPLATE_SET_PBCC: 
		whal_apiSetPbcc(0,val);
		break;

	case TEMPLATE_SET_ERP:
		whal_apiSetERPProtection(0,val,val);
		break;

	default:
		printf("\nERROR: wrong ReqType (%d) in wdrv_mgmtHandleSetTemplateReq\n\n",ReqType); 
	}
}

static void wdrv_mgmtRadioCalibrateCrtTimeoutHandler(void) {
    whal_acxCmdRadioCalibrate(RADIO_CALIBRATE_STATE_4);

    if(REPORT_LEVEL(RL_MGMT)) 
        wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: CRT Radio Calibrate is executed\n");

	PALSRV_tcbQueueAct(&radioCalibrateTimerCrt,
					   ((UINT32)RADIO_CALIBRATE_INTERVAL_CRT_ST_4)*1000, RadioCalibrationTimerCrtCB, 0, 0);
}

/*************************************************************************
 *                    wdrv_mgmtRadioCalibrateTimeoutHandler              *
 *************************************************************************
DESCRIPTION: This function is the radio calibrate three step state machine 

OUTPUT: None

RETURN:  None
************************************************************************/

static void wdrv_mgmtRadioCalibrateTimeoutHandler(void) {
   UINT16	RadioCalibrateInterval;

	 if (crtAlgorithmType) {
		 /* Handle CRT radio calibration state 1 */


		 
		 whal_acxCrtRadioCalibrateNeedAsync();
#if 0 
		 if (whal_acxCrtRadioCalibrateNeed()) {
			 whal_acxCmdRadioCalibrate(RADIO_CALIBRATE_STATE_1);
           if(REPORT_LEVEL(RL_MGMT)) 
						 wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Radio Calibrate Option %x is executed\n",
															currentRadioCalibrateState);
					 
		 }
#endif
		 
		 wdrv_dbGetRadioCalibrateInterval(&RadioCalibrateInterval);
		 m_ADJUST_CALIBRATE_INTERVAL(RadioCalibrateInterval);
       
		 PALSRV_tcbQueueAct(&radioCalibrateTimer,
												((UINT32)RadioCalibrateInterval)*1000,
												RadioCalibrationTimerCB, 
												0, 0);  
       
   } else {
   whal_acxCmdRadioCalibrate(currentRadioCalibrateState);

    if(REPORT_LEVEL(RL_MGMT)) 
        wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Radio Calibrate Option %x is executed\n",
                         currentRadioCalibrateState);

    switch(currentRadioCalibrateState)
    {
    case RADIO_CALIBRATE_STATE_1:
       currentRadioCalibrateState = RADIO_CALIBRATE_STATE_2;

       PALSRV_tcbQueueAct(&radioCalibrateTimer,
                          (UINT32)(RADIO_CALIBRATE_TIMER_2*1000),
						  RadioCalibrationTimerCB,
						  0, 0);
       break;

    case RADIO_CALIBRATE_STATE_2:
       currentRadioCalibrateState = RADIO_CALIBRATE_STATE_3;

       PALSRV_tcbQueueAct(&radioCalibrateTimer,
                          (UINT32)(RADIO_CALIBRATE_TIMER_3*1000),
						  RadioCalibrationTimerCB,
						  0, 0);
       break;

    case RADIO_CALIBRATE_STATE_3:

       currentRadioCalibrateState = RADIO_CALIBRATE_STATE_1;

       wdrv_dbGetRadioCalibrateInterval(&RadioCalibrateInterval);
       PALSRV_tcbQueueAct(&radioCalibrateTimer,
                          (UINT32)(RadioCalibrateInterval*1000),
						  RadioCalibrationTimerCB,
						  0, 0);
       break;


    default:
       wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Error Radio Calibrate Option %x is Illegal \n",
                        currentRadioCalibrateState);
    }

	 }
}

/*************************************************************************
 *                    wdrv_mgmtDisableShortSlotTime                      *
 *************************************************************************
DESCRIPTION: This function disables short slot time if was enabled.

INPUT:  None
OUTPUT: None

RETURN:  None
************************************************************************/
void wdrv_mgmtDisableShortSlotTime(void)
{
BOOL currentShortSlotTimeSupport;

	wdrv_dbGetCurrentShortSlotTimeSupport(&currentShortSlotTimeSupport);
	if (currentShortSlotTimeSupport == TRUE)
	{
		wdrv_dbSetCurrentShortSlotTimeSupport(FALSE);
	    whal_apiSetSlotTime(0,0);
		
		if (REPORT_LEVEL(RL_DYNAMIC_ALG)) 
			wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_INFORMATION, "WDRV_MGMT: Disable Short Slot Time\n");
	}
}

/*************************************************************************
 *                    wdrv_mgmtEnableShortSlotTime                       *
 *************************************************************************
DESCRIPTION: This function enables short slot time if was disabled.

INPUT:  None
OUTPUT: None

RETURN:  None
************************************************************************/
void wdrv_mgmtEnableShortSlotTime(void)
{
BOOL currentShortSlotTimeSupport;

	wdrv_dbGetCurrentShortSlotTimeSupport(&currentShortSlotTimeSupport);
	if (currentShortSlotTimeSupport == FALSE)
	{
		wdrv_dbSetCurrentShortSlotTimeSupport(TRUE);
	    whal_apiSetSlotTime(0,1);

		if (REPORT_LEVEL(RL_DYNAMIC_ALG)) 
			wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_INFORMATION, "WDRV_MGMT: Enable Short Slot Time\n");
	}
}

/*************************************************************************
 *                    wdrv_mgmtCheckShortSlotTime                        *
 *************************************************************************
DESCRIPTION: This function checks if short slot time mode need to be changed.

INPUT:  UINT16 slotTimeThreshold
OUTPUT: None

RETURN:  None
************************************************************************/
void wdrv_mgmtCheckShortSlotTime(UINT16 slotTimeThreshold)
{
UINT32 LastRxTxBrateTimestamp, RxTxBrateCount;
UINT32 timeout;
static int N = 0;
static int loop = 0;


	if (N == 0) {
		wdrv_dbGetDynamicAlgorithmsTimeout(&timeout);
		N = ((timeout+49)/50); /* round up */
	}

	if (++loop >= N) {

		loop = 0;
		wdrv_dbGetRxTxBrate(&LastRxTxBrateTimestamp, &RxTxBrateCount);

		if (RxTxBrateCount >= slotTimeThreshold) {
			wdrv_mgmtDisableShortSlotTime();
		}
		else {
			wdrv_mgmtEnableShortSlotTime();
		}
		/* Reset timestamp and count */
		wdrv_dbSetRxTxBrate(0, 0);
	}
	
}

/*************************************************************************
 *                    wdrv_mgmtShortSlotTimeSM                           *
 *************************************************************************
DESCRIPTION: This function implements the short slot time state machine.

INPUT:  wdrv_mgmtSlotTimeInputs_T input - input to state machine
OUTPUT: None

RETURN:  None
************************************************************************/
void wdrv_mgmtShortSlotTimeSM(int input)
{
static wdrv_mgmtSlotTimeState_T state=STATE_SLOTTIME_G_STA_ONLY;
UINT16 slotTimeThreshold;
UINT8 slotTimeMode;

    wdrv_dbGetSlotTimeMode(0,&slotTimeMode);
	wdrv_dbGetSlotTimeThreshold(&slotTimeThreshold);

    if (slotTimeMode == SLOTTIME_MODE_LOCAL_STATIONS)
    {
	/************************************************************************
	*                    SLOTTIME_MODE_LOCAL_STATIONS                       *
	************************************************************************/
		switch (state)
		{
			case STATE_SLOTTIME_G_STA_ONLY:
				if (input == INPUT_SLOTTIME_B_STA_IN_BSS)
				{
					state = STATE_SLOTTIME_B_STA_IN_BSS;
					wdrv_mgmtDisableShortSlotTime();
					if (REPORT_LEVEL(RL_DYNAMIC_ALG)) 
						wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_INFORMATION, "WDRV_MGMT: ShortSlotTime state = B_STA_IN_BSS\n");
				}
				else if ((input == INPUT_SLOTTIME_CHECK_TIMEOUT) &&
						(slotTimeThreshold != 0))
				{
					wdrv_mgmtCheckShortSlotTime(slotTimeThreshold);
				}
				break;

			case STATE_SLOTTIME_B_STA_IN_BSS:
				if (input == INPUT_SLOTTIME_G_STA_ONLY)
				{
					state = STATE_SLOTTIME_G_STA_ONLY;
					if (slotTimeThreshold == 0)
					{
						wdrv_mgmtEnableShortSlotTime();
					}
					if (REPORT_LEVEL(RL_DYNAMIC_ALG)) 
						wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_INFORMATION, "WDRV_MGMT: ShortSlotTime state = G_STA_ONLY\n");
				}
				break;
		}
	}

    else if (slotTimeMode == SLOTTIME_MODE_ENHANCED_DYNAMIC)
    {
	/************************************************************************
	*                    SLOTTIME_MODE_ENHANCED_DYNAMIC                     *
	************************************************************************/
		switch (state)
		{
			case STATE_SLOTTIME_G_STA_ONLY:
				if (input == INPUT_SLOTTIME_B_STA_IN_BSS)
				{
					state = STATE_SLOTTIME_B_STA_IN_BSS;
					if (REPORT_LEVEL(RL_DYNAMIC_ALG)) 
						wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_INFORMATION, "WDRV_MGMT: ShortSlotTime state = B_STA_IN_BSS\n");
				}
				else if (input == INPUT_SLOTTIME_CHECK_TIMEOUT)
				{
					wdrv_mgmtCheckShortSlotTime(slotTimeThreshold);
				}
				break;

			case STATE_SLOTTIME_B_STA_IN_BSS:
				if (input == INPUT_SLOTTIME_G_STA_ONLY)
				{
					state = STATE_SLOTTIME_G_STA_ONLY;
					if (REPORT_LEVEL(RL_DYNAMIC_ALG)) 
						wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_INFORMATION, "WDRV_MGMT: ShortSlotTime state = G_STA_ONLY\n");
				}
				else if (input == INPUT_SLOTTIME_CHECK_TIMEOUT)
				{
					wdrv_mgmtCheckShortSlotTime(slotTimeThreshold);
				}
				break;
		}
	}
}

/*************************************************************************
 *                         wdrv_mgmtSetHiddenSSID                        *
 *************************************************************************
DESCRIPTION: This function sets the Hidden SSID to a given buffer.
             The buffer is set with 0x00 according to the given length,
             according to the way it is done in CiscoAP1200.

OUTPUT: the given buffer with 0x00.

RETURN:  None
************************************************************************/
void  wdrv_mgmtSetHiddenSSID(char *pSSID, int len)
{
    PAL_memSet (pSSID,0,len);     
}


/*************************************************************************
 *                     wdrv_mgmtProbeRequestPacketMgmt                   *
 *************************************************************************
DESCRIPTION: This function handles Probe Requests frames when the
             Probe Requests are sent to the AP from the HW. 
             It parses the frame, checks the incoming SSID and send
             a Probe Response accordingly.

INOPUT: The incoming Probe Request PDU and its length

RETURN:  None
************************************************************************/
void wdrv_mgmtProbeRequestPacketMgmt(mem_BD_T *pMpdu, UINT32 mpduLen, UINT8  woneIndex)
{
    STATUS            status;
/*	int			      rc;*/
    dot11_probeReq    probeReq;
	dot11_mgmtFrame_T *mmpduPtr;
    dot11_SSID_T      localSSID;
	UINT32		      totalLen, protoEvent;
    UINT16            hiddenSsidFlag;
    char              ssid[DOT11_SSID_MAX+1];

    mmpduPtr = (dot11_mgmtFrame_T *)(pMpdu->data + pMpdu->dataOffset); 
	totalLen = mpduLen - sizeof(dot11_mgmtHeader_T);
	
	if ((mmpduPtr->hdr.fc  & DOT11_FC_SUB_MASK) != DOT11_FC_PROBE_REQ)
        return;

    status = probeReqParse((char *)&mmpduPtr->body, totalLen, &protoEvent, &probeReq);

    if (status != OK)
        return;

    /* probeReq.SSID contains the incoming packet's SSID */
    /* compare it with BSS SSID - Primary or optionally with the Multiple SSID table */
    if (wdrv_dbSSIDSupport(probeReq.SSID)) 
    {
        /* Send Probe Response */
        /* Probe Response is sent if:
           1. SSID is correct
           2. SSID is incorrect to AP SSID, but MultipleSSID is activated and SSID is equal to one of the multiples SSID.
        */
        whal_apiSendProbeResp(((dot11_mgmtHeader_T *)(pMpdu->data))->SA.addr,probeReq.SSID,woneIndex);
    }
    else
    {
        /* SSID's do not match! */
        /* check if empty SSID */
        if (probeReq.SSID->hdr.eleLen == 0) 
        {
            wdrv_dbGetHiddenSsidStatus(&hiddenSsidFlag);
            if (hiddenSsidFlag == 1)
                return;
            /* not Hidden SSID - send response with AP's main SSID */
            wdrv_dbGetSsid(ssid);
            localSSID.hdr.eleId = probeReq.SSID->hdr.eleId;
            localSSID.hdr.eleLen = strlen(ssid) ;
            strcpy(localSSID.serviceSetId,ssid);
            whal_apiSendProbeResp(((dot11_mgmtHeader_T *)(pMpdu->data))->SA.addr,&localSSID,woneIndex);
        }
    }
}

static void RadioCalibrationTimerCB(UINT32 data) {
	mgmtMessage_T	msg;

	msg.type = RADIO_CALIBRATE_TIMEOUT;

	wdrv_DDA_mgmt_setEvent(&msg);
}

static void RadioCalibrationTimerCrtCB(UINT32 data) {
	mgmtMessage_T	msg;

	msg.type = RADIO_CALIBRATE_TIMEOUT_CRT;

	wdrv_DDA_mgmt_setEvent(&msg);
}

void wdrv_mgmtRadioCalibrateInit()
{
   UINT16	RadioCalibrateInterval;
	 UINT8    radioType;

	 /* Demiurg : sanity check */
	 whal_acxEepromGetRadioType(&radioType);
#if defined(TIWLAN_FW_RADIA)
	 if (radioType != RADIO_TYPE_RADIA)
		 printf("ERROR (in %s) : Wrong radio type !\n", __FUNCTION__);
#elif defined(TIWLAN_FW_RADIA_DB)
	 if (radioType != RADIO_TYPE_RADIA_DB)
		 printf("ERROR (in %s) : Wrong radio type !\n", __FUNCTION__);
#elif defined(TIWLAN_FW_RADIA_CRT)
	 if (radioType != RADIO_TYPE_RADIA_CRT)
		 printf("ERROR (in %s) : Wrong radio type !\n", __FUNCTION__);
#elif
	 #error "Unknwon radio type !!!\n"
#endif
   wdrv_dbGetRadioCalibrateOption(&currentRadioCalibrateState);
   wdrv_dbGetRadioCalibrateInterval(&RadioCalibrateInterval);

#ifdef TIWLAN_FW_RADIA_CRT
	 crtAlgorithmType = TRUE;
#else
	 crtAlgorithmType = FALSE;
#endif

   if( RadioCalibrateInterval > 0)
   {
       m_ADJUST_CALIBRATE_INTERVAL(RadioCalibrateInterval);
       if (PALSRV_tcbNew(&radioCalibrateTimer)) {
           if(REPORT_LEVEL(RL_MGMT)) 
           wdrv_utilsReport(RL_MGMT, PRI_EMERGENCY, "WDRV_MGMT: Error Creating Radio Calibrate Timer\n");
           return;   /* NOK */
       }

       PALSRV_tcbQueueAct(&radioCalibrateTimer,
                          (UINT32)(/*RadioCalibrateInterval**/5000),
													RadioCalibrationTimerCB,
													0, 0);
   }
   /* Use additional timer to handle CRT radio module calibration */
   if(crtAlgorithmType) {
		 if (PALSRV_tcbNew(&radioCalibrateTimerCrt)) {
			 if(REPORT_LEVEL(RL_MGMT)) 
				 wdrv_utilsReport(RL_MGMT, PRI_EMERGENCY, "WDRV_MGMT: Error Creating CRT Radio Calibrate Timer\n");
			 return;   /* NOK */
		 }

		 PALSRV_tcbQueueAct(&radioCalibrateTimerCrt,
												((UINT32)RADIO_CALIBRATE_INTERVAL_CRT_ST_4)*1000,
												RadioCalibrationTimerCrtCB,
												0, 0);
   }
}

void wdrv_mgmtRadioCalibrateStop() {
   PALSRV_tcbDelete(&radioCalibrateTimer);
#ifdef TIWLAN_FW_RADIA_CRT
	 PALSRV_tcbDelete(&radioCalibrateTimerCrt);
#endif
}

/*************************************************************************
*                        buildStateMachineMatrix						 *
**************************************************************************
DESCRIPTION:This function builds the management state machine matrix
			First it resets the state machine matrix. 
			Than for each (state, input) pair it initializes the correspondent matrix cell
			with the next state and the actions to be performed.
			There may be more than one action to be performed.
			The actions are represented as bit flags.

INPUT:      None
OUTPUT:     None
RETURN:     None.
**************************************************************************/
static void buildStateMachineMatrix(void)
{
	lbt_newStateMachines(NUM_STATES, NUM_INPUTS, MAX_STATIONS);


	/************************
	*		STATE_IDLE		*
	************************/
	
	st(STATE_IDLE);
   
	/* INPUT_AUTH_1_OS_A */  
	u(STATE_AUTHENTICATED, ACTION_SEND_AUTH, STATUS_SUCCESSFUL);

	/* INPUT_AUTH_1_SK_A */  
	u(STATE_AUTH_KEY, ACTION_SEND_AUTH | ACTION_START_TIMER, STATUS_SUCCESSFUL);

	/* INPUT_AUTH_1_D */  
	u(STATE_IDLE, ACTION_SEND_AUTH | ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_AUTH_3_A */  
	u(STATE_IDLE, ACTION_SEND_AUTH | ACTION_REMOVE_STATION, STATUS_AUTH_TRANSC_NUMBER_OUT_OF_SEQUENCE);

	/* INPUT_AUTH_3_D */  
	u(STATE_IDLE, ACTION_SEND_AUTH | ACTION_REMOVE_STATION, STATUS_AUTH_TRANSC_NUMBER_OUT_OF_SEQUENCE);

	/* INPUT_DE_AUTH_STATION */  
	u(STATE_IDLE, ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_ASSOC_REQ_CHECK */  
	u(STATE_IDLE, ACTION_NOP, STATUS_SUCCESSFUL);
	
	/* INPUT_ASSOC_REQ_A */  
	u(STATE_IDLE, ACTION_SEND_DE_AUTH | ACTION_REMOVE_STATION, STATUS_STATION_REQUESTING_ASSOC_IS_NOT_AUTH);

	/* INPUT_ASSOC_REQ_D */  
	u(STATE_IDLE, ACTION_SEND_DE_AUTH | ACTION_REMOVE_STATION, STATUS_STATION_REQUESTING_ASSOC_IS_NOT_AUTH);

	/* INPUT_RE_ASSOC_REQ_CHECK */  
	u(STATE_IDLE, ACTION_SEND_DE_AUTH | ACTION_REMOVE_STATION, STATUS_STATION_REQUESTING_ASSOC_IS_NOT_AUTH);

	/* INPUT_RE_ASSOC_REQ_A */  
	u(STATE_IDLE, ACTION_SEND_DE_AUTH | ACTION_REMOVE_STATION, STATUS_STATION_REQUESTING_ASSOC_IS_NOT_AUTH);

	/* INPUT_RE_ASSOC_REQ_D */  
	u(STATE_IDLE, ACTION_SEND_DE_AUTH | ACTION_REMOVE_STATION, STATUS_STATION_REQUESTING_ASSOC_IS_NOT_AUTH);

	/* INPUT_DIS_ASSOC_STATION */  
	u(STATE_IDLE, ACTION_SEND_DE_AUTH | ACTION_REMOVE_STATION, STATUS_CLASS_2_FROM_NON_AUTH_STATION);

	/* INPUT_CLASS_3 */  
	u(STATE_IDLE, ACTION_SEND_DE_AUTH | ACTION_REMOVE_STATION, STATUS_CLASS_3_FROM_NON_ASSOC_STATION);

	/* INPUT_AUTH_TIMEOUT */  
	u(STATE_IDLE, ACTION_NOP /* ACTION_REMOVE_STATION*/, STATUS_UNSPECIFIED);

	/* INPUT_DE_AUTH_MNG_UNICAST */  
	u(STATE_IDLE, ACTION_NOP, STATUS_UNSPECIFIED);

	/* INPUT_DE_AUTH_MNG_BROADCAST */  
	u(STATE_IDLE, ACTION_NOP, STATUS_UNSPECIFIED);

	/* INPUT_DIS_ASSOC_MNG_UNICAST */  
	u(STATE_IDLE, ACTION_NOP, STATUS_UNSPECIFIED);

	/* INPUT_DIS_ASSOC_MNG_BROADCAST */  
	u(STATE_IDLE, ACTION_NOP, STATUS_UNSPECIFIED);

	/* INPUT_MAC_AUTHORIZE */  
	u(STATE_IDLE, ACTION_NOP, STATUS_UNSPECIFIED);

	/* INPUT_MAC_DE_AUTHORIZE */  
	u(STATE_IDLE, ACTION_NOP, STATUS_UNSPECIFIED);


	/************************
	*	STATE_AUTH_KEY		*
	************************/


	st(STATE_AUTH_KEY);
   
	/* INPUT_AUTH_1_OS_A */  
	u(STATE_AUTHENTICATED, ACTION_SEND_AUTH | ACTION_STOP_TIMER, STATUS_SUCCESSFUL);

	/* INPUT_AUTH_1_SK_A */  
	u(STATE_AUTH_KEY, ACTION_SEND_AUTH /*| ACTION_START_TIMER*/, STATUS_SUCCESSFUL);

	/* INPUT_AUTH_1_D */  
	u(STATE_IDLE, ACTION_SEND_AUTH | ACTION_STOP_TIMER | ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_AUTH_3_A */  
	u(STATE_AUTHENTICATED, ACTION_SEND_AUTH | ACTION_STOP_TIMER, STATUS_SUCCESSFUL);

	/* INPUT_AUTH_3_D */  
	u(STATE_IDLE, ACTION_SEND_AUTH | ACTION_STOP_TIMER | ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_DE_AUTH_STATION */  
	u(STATE_IDLE, ACTION_STOP_TIMER | ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_ASSOC_REQ_CHECK */  
	u(STATE_AUTH_KEY, ACTION_NOP, STATUS_STATION_REQUESTING_ASSOC_IS_NOT_AUTH);

	/* INPUT_ASSOC_REQ_A */  
	u(STATE_AUTH_KEY, ACTION_SEND_DE_AUTH, STATUS_STATION_REQUESTING_ASSOC_IS_NOT_AUTH);

	/* INPUT_ASSOC_REQ_D */  
	u(STATE_AUTH_KEY, ACTION_SEND_DE_AUTH, STATUS_STATION_REQUESTING_ASSOC_IS_NOT_AUTH);

	/* INPUT_RE_ASSOC_REQ_CHECK */  
	u(STATE_AUTH_KEY, ACTION_NOP, STATUS_STATION_REQUESTING_ASSOC_IS_NOT_AUTH);

	/* INPUT_RE_ASSOC_REQ_A */  
	u(STATE_AUTH_KEY, ACTION_SEND_DE_AUTH, STATUS_STATION_REQUESTING_ASSOC_IS_NOT_AUTH);

	/* INPUT_RE_ASSOC_REQ_D */  
	u(STATE_AUTH_KEY, ACTION_SEND_DE_AUTH, STATUS_STATION_REQUESTING_ASSOC_IS_NOT_AUTH);

	/* INPUT_DIS_ASSOC_STATION */  
	u(STATE_AUTH_KEY, ACTION_SEND_DE_AUTH, STATUS_CLASS_2_FROM_NON_AUTH_STATION);

	/* INPUT_CLASS_3 */  
	u(STATE_AUTH_KEY, ACTION_SEND_DE_AUTH, STATUS_CLASS_3_FROM_NON_ASSOC_STATION);

	/* INPUT_AUTH_TIMEOUT */  
	u(STATE_IDLE, ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_DE_AUTH_MNG_UNICAST */  
	u(STATE_IDLE, ACTION_SEND_DE_AUTH | ACTION_STOP_TIMER | ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_DE_AUTH_MNG_BROADCAST */  
	u(STATE_IDLE, ACTION_REMOVE_STATION| ACTION_STOP_TIMER, STATUS_UNSPECIFIED);

	/* INPUT_DIS_ASSOC_MNG_UNICAST */  
	u(STATE_AUTH_KEY, ACTION_NOP, STATUS_UNSPECIFIED);

	/* INPUT_DIS_ASSOC_MNG_BROADCAST */  
	u(STATE_AUTH_KEY, ACTION_NOP, STATUS_UNSPECIFIED);

	/* INPUT_MAC_AUTHORIZE */  
	u(STATE_AUTH_KEY, ACTION_NOP, STATUS_UNSPECIFIED);

	/* INPUT_MAC_DE_AUTHORIZE */  
	u(STATE_AUTH_KEY, ACTION_NOP, STATUS_UNSPECIFIED);


	/************************
	*	STATE_AUTHENTICATED	*
	************************/

	st(STATE_AUTHENTICATED);
   
	/* INPUT_AUTH_1_OS_A */  
	u(STATE_AUTHENTICATED, ACTION_SEND_AUTH, STATUS_SUCCESSFUL);

	/* INPUT_AUTH_1_SK_A */  
	u(STATE_AUTH_KEY, ACTION_SEND_AUTH | ACTION_START_TIMER, STATUS_SUCCESSFUL);

	/* INPUT_AUTH_1_D */  
	u(STATE_IDLE, ACTION_SEND_AUTH | ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_AUTH_3_A */  
	u(STATE_IDLE, ACTION_SEND_AUTH | ACTION_REMOVE_STATION, STATUS_AUTH_TRANSC_NUMBER_OUT_OF_SEQUENCE);

	/* INPUT_AUTH_3_D */  
	u(STATE_IDLE, ACTION_SEND_AUTH | ACTION_REMOVE_STATION, STATUS_AUTH_TRANSC_NUMBER_OUT_OF_SEQUENCE);

	/* INPUT_DE_AUTH_STATION */  
	u(STATE_IDLE, ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_ASSOC_REQ_CHECK */  
	u(STATE_WAIT_FOR_ASS_RES, ACTION_SEND_WPA_ASS_REQ | ACTION_SET_STATION_PARAMS, STATUS_SUCCESSFUL);

	/* INPUT_ASSOC_REQ_A */  
	u(STATE_AUTHENTICATED, ACTION_NOP, STATUS_SUCCESSFUL);

	/* INPUT_ASSOC_REQ_D */  
	u(STATE_AUTHENTICATED, ACTION_SEND_ASSOC_RSP, STATUS_UNSPECIFIED);

	/* INPUT_RE_ASSOC_REQ_CHECK */  
	u(STATE_WAIT_FOR_ASS_RES, ACTION_SEND_WPA_REASS_REQ | ACTION_UPDATE_STATION_PARAMS, STATUS_SUCCESSFUL);

	/* INPUT_RE_ASSOC_REQ_A */  
	u(STATE_AUTHENTICATED, ACTION_NOP, STATUS_SUCCESSFUL);

	/* INPUT_RE_ASSOC_REQ_D */  
	u(STATE_AUTHENTICATED, ACTION_NOP, STATUS_UNSPECIFIED);

	/* INPUT_DIS_ASSOC_STATION */  
	u(STATE_AUTHENTICATED, ACTION_NOP, STATUS_UNSPECIFIED);

	/* INPUT_CLASS_3 */  
	u(STATE_AUTHENTICATED, ACTION_SEND_DIS_ASSOC, STATUS_CLASS_3_FROM_NON_ASSOC_STATION);

	/* INPUT_AUTH_TIMEOUT */  
	u(STATE_IDLE, ACTION_NOP /*| ACTION_REMOVE_STATION*/, STATUS_UNSPECIFIED);

	/* INPUT_DE_AUTH_MNG_UNICAST */  
	u(STATE_IDLE, ACTION_SEND_DE_AUTH | ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_DE_AUTH_MNG_BROADCAST */  
	u(STATE_IDLE, ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_DIS_ASSOC_MNG_UNICAST */  
	u(STATE_AUTHENTICATED, ACTION_NOP, STATUS_UNSPECIFIED);

	/* INPUT_DIS_ASSOC_MNG_BROADCAST */  
	u(STATE_AUTHENTICATED, ACTION_NOP, STATUS_UNSPECIFIED);

	/* INPUT_MAC_AUTHORIZE */  
	u(STATE_AUTHENTICATED, ACTION_NOP, STATUS_UNSPECIFIED);

	/* INPUT_MAC_DE_AUTHORIZE */  
	u(STATE_AUTHENTICATED, ACTION_NOP, STATUS_UNSPECIFIED);


	/****************************
	*	STATE_WAIT_FOR_ASS_RES	*
	*****************************/

	st(STATE_WAIT_FOR_ASS_RES);
   
	/* INPUT_AUTH_1_OS_A */  
	u(STATE_AUTHENTICATED, ACTION_SEND_AUTH, STATUS_SUCCESSFUL);

	/* INPUT_AUTH_1_SK_A */  
	u(STATE_AUTH_KEY, ACTION_SEND_AUTH | ACTION_START_TIMER, STATUS_SUCCESSFUL);

	/* INPUT_AUTH_1_D */  
	u(STATE_IDLE, ACTION_SEND_AUTH | ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_AUTH_3_A */  
	u(STATE_IDLE, ACTION_SEND_AUTH | ACTION_REMOVE_STATION, STATUS_AUTH_TRANSC_NUMBER_OUT_OF_SEQUENCE);

	/* INPUT_AUTH_3_D */  
	u(STATE_IDLE, ACTION_SEND_AUTH | ACTION_REMOVE_STATION, STATUS_AUTH_TRANSC_NUMBER_OUT_OF_SEQUENCE);

	/* INPUT_DE_AUTH_STATION */  
	u(STATE_IDLE, ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_ASSOC_REQ_CHECK */  
	u(STATE_WAIT_FOR_ASS_RES, ACTION_NOP, STATUS_SUCCESSFUL);

	/* INPUT_ASSOC_REQ_A */  
	u(STATE_ASSOCIATED, ACTION_SEND_ASSOC_RSP | ACTION_NOTIFY_CONNECT, STATUS_SUCCESSFUL);

	/* INPUT_ASSOC_REQ_D */  
	u(STATE_AUTHENTICATED, ACTION_SEND_ASSOC_RSP | ACTION_RESET_STATION_PARAMS, STATUS_UNSPECIFIED);

	/* INPUT_RE_ASSOC_REQ_CHECK */  
	u(STATE_WAIT_FOR_ASS_RES, ACTION_NOP, STATUS_SUCCESSFUL);

	/* INPUT_RE_ASSOC_REQ_A */  
	u(STATE_ASSOCIATED, ACTION_SEND_RE_ASSOC_RSP | ACTION_NOTIFY_CONNECT, STATUS_SUCCESSFUL);

	/* INPUT_RE_ASSOC_REQ_D */  
	u(STATE_AUTHENTICATED, ACTION_SEND_RE_ASSOC_RSP | ACTION_RESET_STATION_PARAMS, STATUS_UNSPECIFIED);

	/* INPUT_DIS_ASSOC_STATION */  
	u(STATE_AUTHENTICATED, ACTION_NOP, STATUS_UNSPECIFIED);

	/* INPUT_CLASS_3 */  
	u(STATE_AUTHENTICATED, ACTION_SEND_DIS_ASSOC, STATUS_CLASS_3_FROM_NON_ASSOC_STATION);

	/* INPUT_AUTH_TIMEOUT */  
	u(STATE_IDLE, ACTION_NOP /*| ACTION_REMOVE_STATION*/, STATUS_UNSPECIFIED);

	/* INPUT_DE_AUTH_MNG_UNICAST */  
	u(STATE_IDLE, ACTION_SEND_DE_AUTH | ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_DE_AUTH_MNG_BROADCAST */  
	u(STATE_IDLE, ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_DIS_ASSOC_MNG_UNICAST */  
	u(STATE_AUTHENTICATED, ACTION_NOP, STATUS_UNSPECIFIED);

	/* INPUT_DIS_ASSOC_MNG_BROADCAST */  
	u(STATE_AUTHENTICATED, ACTION_NOP, STATUS_UNSPECIFIED);

	/* INPUT_MAC_AUTHORIZE */  
	u(STATE_AUTHENTICATED, ACTION_NOP, STATUS_UNSPECIFIED);

	/* INPUT_MAC_DE_AUTHORIZE */  
	u(STATE_AUTHENTICATED, ACTION_NOP, STATUS_UNSPECIFIED);



	/************************
	*	STATE_ASSOCIATED	*
	************************/

	st(STATE_ASSOCIATED);
   
	/* INPUT_AUTH_1_OS_A */  
	u(STATE_AUTHENTICATED, ACTION_SEND_AUTH | ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS, STATUS_SUCCESSFUL);

	/* INPUT_AUTH_1_SK_A */  
	u(STATE_AUTH_KEY, ACTION_SEND_AUTH | ACTION_START_TIMER | ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS, STATUS_SUCCESSFUL);

	/* INPUT_AUTH_1_D */  
	u(STATE_IDLE, ACTION_SEND_AUTH | ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS | ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_AUTH_3_A */  
	u(STATE_IDLE, ACTION_SEND_AUTH | ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS | ACTION_REMOVE_STATION, STATUS_AUTH_TRANSC_NUMBER_OUT_OF_SEQUENCE);

	/* INPUT_AUTH_3_D */  
	u(STATE_IDLE, ACTION_SEND_AUTH | ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS | ACTION_REMOVE_STATION, STATUS_AUTH_TRANSC_NUMBER_OUT_OF_SEQUENCE);

	/* INPUT_DE_AUTH_STATION */  
	u(STATE_IDLE, ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS | ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_ASSOC_REQ_CHECK */  
	u(STATE_ASSOCIATED, ACTION_NOP, STATUS_SUCCESSFUL);

	/* INPUT_ASSOC_REQ_A */  
	u(STATE_ASSOCIATED, ACTION_NOP, STATUS_SUCCESSFUL);

	/* INPUT_ASSOC_REQ_D */  
	u(STATE_AUTHENTICATED, ACTION_SEND_ASSOC_RSP | ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS, STATUS_UNSPECIFIED);

	/* INPUT_RE_ASSOC_REQ_CHECK */  
	u(STATE_ASSOCIATED, ACTION_NOP, STATUS_SUCCESSFUL);

	/* INPUT_RE_ASSOC_REQ_A */  
	u(STATE_ASSOCIATED, ACTION_NOP, STATUS_SUCCESSFUL);

	/* INPUT_RE_ASSOC_REQ_D */  
	u(STATE_AUTHENTICATED, ACTION_SEND_RE_ASSOC_RSP | ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS, STATUS_UNSPECIFIED);

	/* INPUT_DIS_ASSOC_STATION */  
	u(STATE_AUTHENTICATED, ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS, STATUS_UNSPECIFIED);

	/* INPUT_CLASS_3 */  
	u(STATE_ASSOCIATED, ACTION_NOP, STATUS_UNSPECIFIED);

	/* INPUT_AUTH_TIMEOUT */  
	u(STATE_IDLE, ACTION_NOP /*| ACTION_REMOVE_STATION */, STATUS_UNSPECIFIED);

	/* INPUT_DE_AUTH_MNG_UNICAST */  
	u(STATE_IDLE, ACTION_SEND_DE_AUTH | ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS | ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_DE_AUTH_MNG_BROADCAST */  
	u(STATE_IDLE, ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS | ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_DIS_ASSOC_MNG_UNICAST */  
	u(STATE_AUTHENTICATED, ACTION_SEND_DIS_ASSOC | ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS, STATUS_UNSPECIFIED);

	/* INPUT_DIS_ASSOC_MNG_BROADCAST */  
	u(STATE_AUTHENTICATED, ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS, STATUS_UNSPECIFIED);

	/* INPUT_MAC_AUTHORIZE */  
	u(STATE_AUTHORIZED, ACTION_NOP, STATUS_UNSPECIFIED);

	/* INPUT_MAC_DE_AUTHORIZE */  
	u(STATE_ASSOCIATED, ACTION_NOP, STATUS_UNSPECIFIED);

	/************************
	*	STATE_AUTHORIZED	*
	************************/


	st(STATE_AUTHORIZED);
   
	/* INPUT_AUTH_1_OS_A */  
	u(STATE_AUTHENTICATED, ACTION_SEND_AUTH | ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS, STATUS_SUCCESSFUL);

	/* INPUT_AUTH_1_SK_A */  
	u(STATE_AUTH_KEY, ACTION_SEND_AUTH | ACTION_START_TIMER | ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS, STATUS_SUCCESSFUL);

	/* INPUT_AUTH_1_D */  
	u(STATE_IDLE, ACTION_SEND_AUTH | ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS | ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_AUTH_3_A */  
	u(STATE_IDLE, ACTION_SEND_AUTH | ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS | ACTION_REMOVE_STATION, STATUS_AUTH_TRANSC_NUMBER_OUT_OF_SEQUENCE);

	/* INPUT_AUTH_3_D */  
	u(STATE_IDLE, ACTION_SEND_AUTH | ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS | ACTION_REMOVE_STATION, STATUS_AUTH_TRANSC_NUMBER_OUT_OF_SEQUENCE);

	/* INPUT_DE_AUTH_STATION */  
	u(STATE_IDLE, ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS | ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_ASSOC_REQ_CHECK */  
	u(STATE_ASSOCIATED, ACTION_NOP, STATUS_SUCCESSFUL);

	/* INPUT_ASSOC_REQ_A */  
	u(STATE_ASSOCIATED, ACTION_NOP, STATUS_SUCCESSFUL);

	/* INPUT_ASSOC_REQ_D */  
	u(STATE_AUTHENTICATED, ACTION_SEND_ASSOC_RSP | ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS, STATUS_UNSPECIFIED);

	/* INPUT_RE_ASSOC_REQ_CHECK */  
	u(STATE_ASSOCIATED, ACTION_NOP, STATUS_SUCCESSFUL);

	/* INPUT_RE_ASSOC_REQ_A */  
	u(STATE_ASSOCIATED, ACTION_NOP, STATUS_SUCCESSFUL);

	/* INPUT_RE_ASSOC_REQ_D */  
	u(STATE_AUTHENTICATED, ACTION_SEND_RE_ASSOC_RSP | ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS, STATUS_UNSPECIFIED);

	/* INPUT_DIS_ASSOC_STATION */  
	u(STATE_AUTHENTICATED, ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS, STATUS_UNSPECIFIED);

	/* INPUT_CLASS_3 */  
	u(STATE_AUTHORIZED, ACTION_NOP, STATUS_UNSPECIFIED);

	/* INPUT_AUTH_TIMEOUT */  
	u(STATE_IDLE, ACTION_NOP /*| ACTION_REMOVE_STATION */, STATUS_UNSPECIFIED);

	/* INPUT_DE_AUTH_MNG_UNICAST */  
	u(STATE_IDLE, ACTION_SEND_DE_AUTH | ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS | ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_DE_AUTH_MNG_BROADCAST */  
	u(STATE_IDLE, ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS | ACTION_REMOVE_STATION, STATUS_UNSPECIFIED);

	/* INPUT_DIS_ASSOC_MNG_UNICAST */  
	u(STATE_AUTHENTICATED, ACTION_SEND_DIS_ASSOC | ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS, STATUS_UNSPECIFIED);

	/* INPUT_DIS_ASSOC_MNG_BROADCAST */  
	u(STATE_AUTHENTICATED, ACTION_NOTIFY_DIS_CONNECT | ACTION_RESET_STATION_PARAMS, STATUS_UNSPECIFIED);

	/* INPUT_MAC_AUTHORIZE */  
	u(STATE_AUTHORIZED, ACTION_NOP, STATUS_UNSPECIFIED);

	/* INPUT_MAC_DE_AUTHORIZE */  
	u(STATE_ASSOCIATED, ACTION_NOP, STATUS_UNSPECIFIED);
}


