/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2003 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		                     */
/*		MODULE:	envList.c											             	   */
/*    PURPOSE:	Envelope list implementation        					         */
/*																		                     */
/***************************************************************************/

#include "envList.h"
#include "tx.h"

/*************************************************************************
 *                        wdrv_txNewEnvList                              *
 *************************************************************************
DESCRIPTION: This function creates new envelop list.                 
                                                                                                   
INPUT:       maxNumOfElements : The maximum number of elements allowed inition)    
OUTPUT:      (a list of output parameters (returned by pointer) 
                       and their definitions)                                           
RETURN:      EnvList_T                                                 
                                                                                                   
************************************************************************/
EnvList_T* wdrv_txNewEnvList(UINT16 maxNumOfElements )
{
   
   EnvList_T* envList = (EnvList_T*) PAL_memAlloc(0,sizeof(EnvList_T),0,0);
   
   if( envList == NULL )
        return NULL;
   
   envList->first = NULL;
   envList->last = NULL;
   envList->maxNumOfEnv = maxNumOfElements;
   envList->CurrNumOfEnv = 0;
   envList->maxStoredEnv = 0;

   PAL_spinLock_init(&envList->envListLock);

   return envList;
}

/*************************************************************************
 *                        wdrv_txFreeEnvList                              *
 *************************************************************************
DESCRIPTION: This function frees envelop list.                 
                                                                                                   
INPUT:       
OUTPUT:       
RETURN:      None                                                 
                                                                                                   
************************************************************************/
void wdrv_txFreeEnvList(EnvList_T *this)
{
	PAL_memFree(0,(void *)this,0);
}


/*************************************************************************
 *                        wdrv_txEnvListIns                              *
 *************************************************************************
DESCRIPTION: This function insert envelop to the list pointed by this.                 
                                                                                                   
INPUT:       *this : A pointer to the list to insert. 
             *envelopPtr : Pointer to the envelop to insert, the envelop
                           is inserted to be the last in the list. 

OUTPUT:      *envelopPtr : In case the list is full will hold the dropt 
                           envelop's pointer. 

RETURN:      OK : The envelop has been inserted, there was enough place in 
                  the list.
             NOK: The list was full, the first envelop is dropt, and returned
                  by *envelopPtr.                                                                                                                                                  
************************************************************************/
STATUS  wdrv_txEnvListIns( EnvList_T* this , TxEnvelop_T  **envelopPtr )
{
   unsigned long flags;

   PAL_intrLock(flags); /* START OF CRITICAL SECTION */

   /* in case of High Priority frame we force the insertion of the frame to the waiting queue */
   if (((*envelopPtr)->txFlags == WDRV_TX_HIGH_PRIORITY) &&
	   (this->CurrNumOfEnv >= this->maxNumOfEnv + 5))
   { /* The list is full, return */
      PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
      return NOK;
   }
   if (((*envelopPtr)->txFlags != WDRV_TX_HIGH_PRIORITY) &&
	   (this->CurrNumOfEnv >= this->maxNumOfEnv))
   { /* The list is full, return */
      PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
      return NOK;
   }

   if(  this->CurrNumOfEnv == 0 ){
      this->last = *envelopPtr;
      this->first = *envelopPtr;
      (*envelopPtr)->nextEnvPtr = NULL;
      (*envelopPtr)->prevEnvPtr = NULL;
   }
   else { /* The list is not empty, insert the envelop to be the last. */
      this->last->nextEnvPtr = *envelopPtr;
      (*envelopPtr)->prevEnvPtr = this->last;
      (*envelopPtr)->nextEnvPtr = NULL;
      this->last = *envelopPtr;
   }   

   this->CurrNumOfEnv++;
   if( this->maxStoredEnv < this->CurrNumOfEnv )
      this->maxStoredEnv = this->CurrNumOfEnv;

   PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
   return OK;
}

/*************************************************************************
 *                        wdrv_txEnvListIns                              *
 *************************************************************************
DESCRIPTION: This function insert envelop to the list pointed by this to the first location.                 
                                                                                                   
INPUT:       *this : A pointer to the list to insert. 
             *envelopPtr : Pointer to the envelop to insert, the envelop
                           is inserted to be the last in the list. 

OUTPUT:      *envelopPtr : In case the list is full will hold the dropt 
                           envelop's pointer. 

RETURN:      OK : The envelop has been inserted, there was enough place in 
                  the list.
             NOK: The list was full, the first envelop is dropt, and returned
                  by *envelopPtr.                                                                                                                                                  
************************************************************************/
STATUS wdrv_txEnvListInsFirst (EnvList_T* this , TxEnvelop_T  **envelopPtr )
{
	TxEnvelop_T  *tempEnvelope, *lastEnvelope;
	unsigned long flags;

	PAL_intrLock(flags); /* START OF CRITICAL SECTION */

	if(this->CurrNumOfEnv >= this->maxNumOfEnv)
	{
		/* the list is full, remove the last element and add the current into the first place */
		lastEnvelope = this->last;
		this->last = lastEnvelope->prevEnvPtr;
		this->last->nextEnvPtr = NULL;
		/* now add the current */
		tempEnvelope = this->first;
		this->first = *envelopPtr;
		this->first->nextEnvPtr = tempEnvelope;		
		tempEnvelope->prevEnvPtr = this->first;
		this->first->prevEnvPtr = NULL;
		/* now free the last element, which was removed */
		wlan_memMngrFreeMSDU( lastEnvelope->msduPtr->handle  );
		wdrv_txFreeEnv(&lastEnvelope);
	}
	else if(this->CurrNumOfEnv == 0)
	{
		/* the list is empty, just add the element into the first place */
		this->last = *envelopPtr;
		this->first = *envelopPtr;
		(*envelopPtr)->nextEnvPtr = NULL;
		(*envelopPtr)->prevEnvPtr = NULL;
		this->CurrNumOfEnv++;
	}
	else
	{
		tempEnvelope = this->first;
		this->first = *envelopPtr;
		this->first->nextEnvPtr = tempEnvelope;		
		tempEnvelope->prevEnvPtr = this->first;
		this->first->prevEnvPtr = NULL;
		this->CurrNumOfEnv++;
	}

	if( this->maxStoredEnv < this->CurrNumOfEnv )
		this->maxStoredEnv = this->CurrNumOfEnv;

	PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
	return OK;
}


/*************************************************************************
 *                        wdrv_txEnvListWatchFirst                       *
 *************************************************************************
DESCRIPTION: This function watch at the first envelop in the list. The 
             envelop is not removed yet from the list.                 
                                                                                                   
INPUT:       *this : A pointer to the list to insert. 

OUTPUT:      *envelopPtr : A pointer to the first envelop in the list. 

RETURN:      OK : There was an envelop in the list, and it is assigned 
                  to the *envelopPtr.
             NOK: The list was empty, *envelopPtr is trush.     
************************************************************************/
STATUS wdrv_txEnvListWatchFirst( EnvList_T *this, TxEnvelop_T  **envelopPtr)
{
   unsigned long flags;

   PAL_intrLock(flags); /* START OF CRITICAL SECTION */

   if( this->CurrNumOfEnv == 0 ) {
      PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
      return NOK;
   }
 
   *envelopPtr = this->first;

   PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */

   if (*envelopPtr == NULL) {
      /* FATAL - can't happen unless there is a bug somewhere... */
	  printf("wdrv_txEnvListWatchFirst: Mismatch in EnvList! Curr=%d but first=NULL (envList=0x%x, .last=0x%x)",this->CurrNumOfEnv, (int)this, (int)this->last);
      wdrv_utilsReport(RL_TX, PRI_INFORMATION, "Tx: Mismatch in EnvList! Curr=%d but first=NULL (envList=0x%x, .last=0x%x)",this->CurrNumOfEnv, (int)this, (int)this->last);
      return NOK;
   }


   return OK;
}

/*************************************************************************
 *                        wdrv_txEnvListGetFirst                         *
 *************************************************************************
DESCRIPTION: This function takes the first envelop from the list.                 
                                                                                                   
INPUT:       *this : A pointer to the list to insert. 

OUTPUT:      *envelopPtr : A pointer to the first envelop in the list. 

RETURN:      OK : There was an envelop in the list, and it is assigned 
                  to the *envelopPtr.
             NOK: The list was empty, *envelopPtr is trush.     
************************************************************************/
STATUS wdrv_txEnvListGetFirst( EnvList_T *this, TxEnvelop_T  **envelopPtr)
{
   unsigned long flags;

   PAL_intrLock(flags); /* START OF CRITICAL SECTION */

   if( this->CurrNumOfEnv == 0 ){
      PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
      return NOK;
   }
 
   *envelopPtr = this->first;
   if (*envelopPtr == NULL) {
      /* FATAL - can't happen unless there is a bug somewhere... */
	  PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
      wdrv_utilsReport(RL_TX, PRI_INFORMATION, "Tx: Mismatch in EnvList! Curr=%d but first=NULL (envList=0x%x, .last=0x%x)",this->CurrNumOfEnv, (int)this, (int)this->last);
	  printf("wdrv_txEnvListGetFirst: FATAL ERROR: Mismatch in EnvList! Curr=%d but first=NULL (envList=0x%x, .last=0x%x)",this->CurrNumOfEnv, (int)this, (int)this->last);
      return NOK;
   }

   this->first = this->first->nextEnvPtr;
   this->CurrNumOfEnv--;

   PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */

   return OK;
}


/*************************************************************************
 *                        wdrv_txEnvListGetFirst                         *
 *************************************************************************
DESCRIPTION: This function takes the envelop from the middle of the list.                 
                                                                                                   
INPUT:       *this : A pointer to the list to take. 

OUTPUT:      *envelopPtr : The envelop . 

RETURN:      OK : There was an envelop in the list, and it is assigned 
                  to the *envelopPtr.
             NOK: The list was empty, *envelopPtr is trush.     
************************************************************************/
STATUS wdrv_txEnvListGetMiddle( EnvList_T *this, TxEnvelop_T  *envelopPtr)
{
   unsigned long flags;

   if(!this || !envelopPtr)
   {
	   printf("wdrv_txEnvListGetMiddle: fatal error - envListPtr=0x%X, envelopPtr=0x%X\n",(UINT32)this,(UINT32)envelopPtr);
	   return NOK;
   }

   PAL_intrLock(flags); /* START OF CRITICAL SECTION */

   if( this->CurrNumOfEnv == 0 ){
      PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
      return NOK;
   }

   
   if( this->first == envelopPtr ) {
      /* first */
      if ( this->first == this->last ) {
         /* it was the only one */
         this->CurrNumOfEnv = 1; /* Just to be safe ... */
         this->first = NULL;
         this->last = NULL;
      } else {
         /* there are more env */
         this->first = envelopPtr->nextEnvPtr;
         this->first->prevEnvPtr = NULL;
      }
   }
   else if( this->last == envelopPtr ) {
	   this->last = envelopPtr->prevEnvPtr;
      this->last->nextEnvPtr = NULL;
   }
   else{ /* Middle envelop */
   		/* Attach the previouse envelop to the next */
         /* Missing protection on the prev and next pointers validity */
   		envelopPtr->prevEnvPtr->nextEnvPtr = envelopPtr->nextEnvPtr;
   		envelopPtr->nextEnvPtr->prevEnvPtr = envelopPtr->prevEnvPtr;
   }
   
   this->CurrNumOfEnv--;


   PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */

   return OK;
}


/*************************************************************************
 *                        wdrv_txEnvListGetMaxStored                     *
 *************************************************************************
DESCRIPTION: 
                              
                                                                                                   
INPUT:       

OUTPUT:      
RETURN:      
               
             
************************************************************************/
UINT16 wdrv_txEnvListGetMaxStored( EnvList_T *this )
{
   return( this->maxStoredEnv );
}

/*************************************************************************
 *                   wdrv_txEnvListGetCurrentStored                      *
 *************************************************************************
DESCRIPTION: 
                              
                                                                                                   
INPUT:       

OUTPUT:      
RETURN:      
               
             
************************************************************************/
UINT16 wdrv_txEnvListGetCurrentStored( EnvList_T *this )
{
	if(this)
		return( this->CurrNumOfEnv );
	else
		return 0;
}


/*************************************************************************
 *                        wdrv_txEnvListIns                              *
 *************************************************************************
DESCRIPTION: This function insert envelop to the list pointed by this.                 
                                                                                                   
INPUT:       *this : A pointer to the list to insert. 
             *envelopPtr : Pointer to the envelop to insert, the envelop
                           is inserted to be the last in the list. If the 
                           list is full the first envelop will be taken 
                           out and will be returned in this pointer.

OUTPUT:      *envelopPtr : In case the list is full will hold the dropt 
                           envelop's pointer. 

RETURN:      OK : The envelop has been inserted, there was enough place in 
                  the list.
             NOK: The list was full, the first envelop is dropt, and returned
                  by *envelopPtr.                                                                                                                                                  
************************************************************************/
STATUS  wdrv_txStaEnvListIns( EnvList_T* this , TxEnvelop_T  **envelopPtr )
{
   unsigned long flags;

   PAL_intrLock(flags); /* START OF CRITICAL SECTION */

   if( *envelopPtr == NULL ){
       PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
       printf("Trying to put NULL envelop !!!!\n");
       return NOK;
   }

   if(  this->CurrNumOfEnv == 0 ){
      this->last = *envelopPtr;
      this->first = *envelopPtr;
      (*envelopPtr)->nextSameStaEnv = NULL;
   }
   else { /* The list is not empty, insert the envelop to be the last. */
      this->last->nextSameStaEnv = *envelopPtr;
      this->last = *envelopPtr;
      (*envelopPtr)->nextSameStaEnv = NULL;
   }   

 
   this->CurrNumOfEnv++;
   if( this->maxStoredEnv < this->CurrNumOfEnv )
      this->maxStoredEnv = this->CurrNumOfEnv;


   PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */

   return OK;
}


/*************************************************************************
 *                        wdrv_txStaEnvListGetFirst                      *
 *************************************************************************
DESCRIPTION: This function takes the first envelop from the list.                 
                                                                                                   
INPUT:       *this : A pointer to the list to insert. 

OUTPUT:      *envelopPtr : A pointer to the first envelop in the list. 

RETURN:      OK : There was an envelop in the list, and it is assigned 
                  to the *envelopPtr.
             NOK: The list was empty, *envelopPtr is trush.     
************************************************************************/
STATUS wdrv_txStaEnvListGetFirst( EnvList_T *this, TxEnvelop_T  **envelopPtr)
{
   unsigned long flags;

   PAL_intrLock(flags); /* START OF CRITICAL SECTION */

   if( this->CurrNumOfEnv == 0 ){
       PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
       return NOK;
   }


   *envelopPtr = this->first;

       
   if( this->first == NULL ) {
        /* FATAL - can't happen unless there is a bug somewhere... */
	    PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
        wdrv_utilsReport(RL_TX, PRI_INFORMATION, "Tx: Mismatch in StaEnvList! Curr=%d but first=NUL (envList=0x%x, .last=0x%x)",this->CurrNumOfEnv, (int)this, (int)this->last);
		return NOK;
   }

   this->first = this->first->nextSameStaEnv;

   this->CurrNumOfEnv--; 

   PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */

   return OK;
}



STATUS wdrv_txEnvCheck( EnvList_T *this, int tag)
{

   int  i;
   TxEnvelop_T  *envelopPtr;
   unsigned long flags;

   PAL_intrLock(flags); /* START OF CRITICAL SECTION */

   if( this->CurrNumOfEnv == 0){
       PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
       return OK;
   }

   envelopPtr = this->first;

   for( i=0; i < this->CurrNumOfEnv - 1 ; i++ ){
       if( envelopPtr->nextEnvPtr == NULL ){
           printf("List - BUG !!! tag=%d CurrNumOfEnv=%d  badEnv=%d+1\n ", tag, this->CurrNumOfEnv,i );
    /*       taskSuspend(taskNameToId("tNetTask")); */
           PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */           
           return NOK;
       }
       envelopPtr = envelopPtr->nextEnvPtr;
   }

   PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
   return OK;
}





STATUS wdrv_txStaEnvCheck( EnvList_T *this, int tag)
{

   int  i;
   TxEnvelop_T  *envelopPtr;
   unsigned long flags;

   PAL_intrLock(flags); /* START OF CRITICAL SECTION */

   if( this->CurrNumOfEnv == 0){
       PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
       return OK;
   }

   envelopPtr = this->first;

   for( i=0; i < this->CurrNumOfEnv - 1 ; i++ ){
       if( envelopPtr->nextSameStaEnv == NULL ){
		   PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */           
           printf("**** STA LIst - BUG ****  CurrNumOfEnv=%d env=%d\n", this->CurrNumOfEnv, i+1);           
           return NOK;
        }
       envelopPtr = envelopPtr->nextSameStaEnv;
   }

   PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
   return OK;
}

