/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																			*/
/*	  MODULE:	powerSave.c	                                                */ 
/*   PURPOSE:	Power Save module implementation.	 					    */ 
/*																			*/
/***************************************************************************/
#include "pal.h"
#include "envList.h"
#include "whal.h"
#include "drvDbMain.h"
#include "drvDbHash.h"
#include "tx.h"

#define GetAssocId(x) (x+1)


#define MAX_PS_ENV_PER_STA                      32
#define PS_AGING_TIMEOUT                         6   /* 1 MIN (uints=SEC*10) */

typedef struct {
   UINT8      psStatus;     /* Power Save Status (True,False) */
   UINT32     sentPkts;
   EnvList_T *envListPtr;   /* Evelop list pointer.           */
   UINT8      prevTxFail;   /* The previos Tx fail )SRAM Overflow */
} PsEntry_T;

static PsEntry_T *psTable; /* The main Power Save table. */
UINT8 PsPrevTxFailCounter = 0;

static UINT32 agingTimeStamp;   /* The PS agingTimeStamp counter. */
static UINT32 numOfStaInPsMode;
static UINT32 numOfStaInWonePsMode[MAX_WONE_NET];
/* Definition for Bitmap control. */
#define PART_VIRT_BITMAP_LEN 251
static UINT8 partVirtBitmap[PART_VIRT_BITMAP_LEN]; 
static UINT8 highestAssocId = 0;
static UINT8 lowestAssocId = 0;
static UINT32 numberOfSetBist = 0;

UINT32 psPacketsExpired = 0;
UINT32 psPacketsMissed  = 0;
UINT32 psPacketsCleaned = 0;

static PAL_spinLockHandle psLock;

STATUS updatePartVirtBitmap(UINT16 assocId, UINT16 status );

/*************************************************************************
 *                        wdrv_psInit                                    *
 *************************************************************************
DESCRIPTION: This function initializes the Power save module.                 
                                                                                                   
INPUT:      numOfStations - Number of station in the system 

OUTPUT:      

RETURN:      OK : Initiation succesfull.
             NOK: Initiation unsuccesfull.                                                                                                                                             
************************************************************************/
STATUS wdrv_psInit(UINT16 numOfStations )
{ 
   int index,wone_index;

   /* Init the main table. */
   if((psTable=(PsEntry_T*)PAL_memAlloc( 0,sizeof(PsEntry_T)*numOfStations,0,0)) == NULL)
   {
      wdrv_utilsReport(RL_TX, PRI_EMERGENCY,
       		"PS: Not enough memory to allocate psTable.");
      return NOK;   
   }

   /* For each STA, init the envelop list, and set ps status to false. */     
   for( index=0; index < numOfStations; index++ ){
      if( (psTable[index].envListPtr = wdrv_txNewEnvList(MAX_PS_ENV_PER_STA))==NULL)
      {
         wdrv_utilsReport(RL_TX, PRI_EMERGENCY,
            "TX: Not enough memory to PS EnvList");
         return NOK;
      }
      psTable[index].psStatus = FALSE;
      psTable[index].sentPkts = 0;
      psTable[index].prevTxFail = FALSE;
   }

   PAL_memSet( partVirtBitmap, 0 , PART_VIRT_BITMAP_LEN );

   numOfStaInPsMode = 0;
    for (wone_index = 0 ;wone_index < MAX_WONE_NET; wone_index++)
        numOfStaInWonePsMode[wone_index] = 0;
   agingTimeStamp = 0;

   PAL_spinLock_init(&psLock);

   return OK;
}

/*************************************************************************
 *                        wdrv_psUnload                                    *
 *************************************************************************
DESCRIPTION: This function deinitializes the Power save module.                 
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:     
                                                                                                                                        
************************************************************************/
void wdrv_psUnload(UINT16 numOfStations)
{
	int i;
	TxEnvelop_T *envelop;

	if(psTable)
	{
		for(i=0; i<numOfStations; i++)
		{
			if(psTable[i].envListPtr)
			{
				while(TRUE)
				{
					if(wdrv_txEnvListGetFirst(psTable[i].envListPtr, &envelop) == OK)
						PAL_memFree(0,(void *)envelop,0);
					else
						break;
				}
				
				wdrv_txFreeEnvList(psTable[i].envListPtr);
			}
		}
	
		PAL_memFree(0,(void *)psTable,0);
	}
}

/*************************************************************************
 *                        wdrv_psSetPsStatus                             *
 *************************************************************************
DESCRIPTION: This function initializes the Power save module.                 
                                                                                                   
INPUT:      staIndex - The station index.

OUTPUT:      

RETURN:                                                                                                                                                
************************************************************************/
inline void wdrv_psSetPsStatus( UINT16 staIndex, UINT8 woneIndex, UINT16 status )
{
   PAL_spinLock(&psLock); /* START OF CRITICAL SECTION */
   
   /* 
   * in case there a change in the station Power Save status -> wake up the Tx task 
   * else -> do nothing
   */
   if( psTable[staIndex].psStatus != status ) {
       PAL_spinUnLock(&psLock);  
       wdrv_txSendPsChangeStateMsg( staIndex, status, woneIndex ) ;
   }
   else
       PAL_spinUnLock(&psLock);  
}
    
      
/*************************************************************************
 *                        wdrv_psPowerSaveStatusChangeFunc                *
 *************************************************************************
DESCRIPTION: This function initializes the Power save module.                 
                                                                                                   
INPUT:      staIndex - The station index.

OUTPUT:      

RETURN:                                                                                                                                                
************************************************************************/
void wdrv_psPowerSaveStatusChangeFunc(UINT32 staIndex, UINT16 status, UINT8 woneIndex)
{

   if( psTable[staIndex].psStatus == status )
      return;

   psTable[staIndex].psStatus = status;

   if( status == TRUE){
      numOfStaInPsMode++;
      numOfStaInWonePsMode[woneIndex]++;
      if( numOfStaInPsMode == 1 ){
          if (REPORT_LEVEL(RL_TX))
               wdrv_utilsReport(RL_TX,PRI_INFORMATION,"PS: Set Power save mode to True\n");
          whal_apiSetBssPs( TRUE );
      }

      /* If this station was flaged as TxFail (last out of PS couldn't send all the packets),
         then we need to reset this flag, so the packets will not be transmitted to the open air */
      if(psTable[staIndex].prevTxFail == TRUE) {
          PsPrevTxFailCounter--;
          psTable[staIndex].prevTxFail = FALSE;
      }
   }

   else{ /* status == FALSE */
      /* The station has become active send PS-wake to Tx. */
      numOfStaInPsMode--;
      numOfStaInWonePsMode[woneIndex]--;
      if( numOfStaInPsMode == 0 ){
         if (REPORT_LEVEL(RL_TX))
            wdrv_utilsReport(RL_TX,PRI_INFORMATION,"PS: Set Power save mode to False\n");
         whal_apiSetBssPs( FALSE );
      }
   }
}

/*************************************************************************
 *                        wdrv_psGetStaStatus                            *
 *************************************************************************
DESCRIPTION: This function returns the power save status of the station.                
                                                                                                   
INPUT:      staIndex - The station index.

OUTPUT:      

RETURN: TRUE  - PS in ON .  
        FALSE - PS is OFF.                                                                                                                                               
************************************************************************/
UINT16 wdrv_psGetStaStatus( UINT16 stationIndex )
{
	unsigned long flags;
	UINT8 status;
	
    if((stationIndex == INVALID_INDEX) || (stationIndex >= MAX_STATIONS))
        return FALSE;
    else
	{
		PAL_intrLock(flags);
		status = psTable[ stationIndex ].psStatus;
		PAL_intrUnLock(flags);
	}

	return status;
}


/*************************************************************************
 *                        wdrv_psStoreEnv                                *
 *************************************************************************
DESCRIPTION: This function saves the envelop in the envelop list.                
                                                                                                   
INPUT:      staIndex - The station index.
            envelopPtr - Pointer to the envelop to save.            

OUTPUT:      

RETURN:      OK : The envelop has been inserted, there was enough place in 
                  the list.
             NOK: The list was full, the first envelop is dropt, and returned
                  by *envelopPtr.                                                                                                                                                
************************************************************************/
STATUS wdrv_psStoreEnv( UINT16 staIndex, TxEnvelop_T **envelopPtr )
{
   STATUS retVal;
   unsigned long flags;

   (*envelopPtr)->expirTick = agingTimeStamp + PS_AGING_TIMEOUT;
   if((*envelopPtr)->txFlags & WDRV_TX_EAPOL_FRAME)
	   retVal = wdrv_txEnvListInsFirst( psTable[staIndex].envListPtr, envelopPtr);
   else
	   retVal= wdrv_txEnvListIns( psTable[staIndex].envListPtr, envelopPtr);
   
   PAL_intrLock(flags);
   if(( psTable[staIndex].psStatus == TRUE)&&
      ( psTable[staIndex].envListPtr->CurrNumOfEnv == 1 ))
   {
	   PAL_intrUnLock(flags);
	   updatePartVirtBitmap(staIndex, TRUE );
   }
   else
	   PAL_intrUnLock(flags);
      
   return retVal;
}


/*************************************************************************
 *                        wdrv_psWatchEnv                                *
 *************************************************************************
DESCRIPTION: This function saves the envelop in the envelop list.                
                                                                                                   
INPUT:      staIndex - The station index.
            envelopPtr - Pointer to the envelop to save.            

OUTPUT:      

RETURN:      OK : There was an envelop in the list, and it is assigned 
                  to the *envelopPtr.
             NOK: The list was empty, *envelopPtr is trush.                                                                                                                                                 
************************************************************************/
STATUS wdrv_psWatchEnv( UINT16 staIndex, TxEnvelop_T **envelopPtr)
{
   return( wdrv_txEnvListWatchFirst( psTable[staIndex].envListPtr, 
                                      envelopPtr));
}

/*************************************************************************
 *                        wdrv_psGetEnv                                  *
 *************************************************************************
DESCRIPTION: This function saves the envelop in the envelop list.                
                                                                                                   
INPUT:      staIndex - The station index.
            envelopPtr - Pointer to the envelop to save.            

OUTPUT:      

RETURN:      OK : There was an envelop in the list, and it is assigned 
                  to the *envelopPtr.
             NOK: The list was empty, *envelopPtr is trush.                                                                                                                                                 
************************************************************************/
STATUS wdrv_psGetEnv( UINT16 staIndex, TxEnvelop_T **envelopPtr)
{
    STATUS retVal;
    unsigned long flags;
 
    retVal = wdrv_txEnvListGetFirst( psTable[staIndex].envListPtr, 
                                     envelopPtr);
  
    if(retVal == OK)
	{
		PAL_intrLock(flags);
		if(psTable[staIndex].envListPtr->CurrNumOfEnv == 0)
		{
			PAL_intrUnLock(flags);
			updatePartVirtBitmap(staIndex, FALSE );
		}
		else
			PAL_intrUnLock(flags);
	}

	PAL_intrLock(flags);
    if(retVal == OK)
      psTable[staIndex].sentPkts++;

	if(psTable[staIndex].prevTxFail == TRUE)
    {
       PsPrevTxFailCounter --;
       psTable[staIndex].prevTxFail = FALSE;
    }
	PAL_intrUnLock(flags);

    return retVal;
}



/*************************************************************************
 *                        wdrv_psPsOnAndPksInQueue                       *
 *************************************************************************
DESCRIPTION: This function returns TRUE if the station is in PS mode and
              there are more packets in the PS queue.                 
                                                                                                   
INPUT:      staIndex - The station index.

OUTPUT:      

RETURN: TRUE  - Station in PS more packets to send .  
        FALSE - Otherwise.                                                                                                                                               
************************************************************************/
UINT16 wdrv_psPsOnAndPksInQueue( UINT16 stationIndex )
{
   unsigned long flags;

   PAL_intrLock(flags);
   if(( psTable[ stationIndex ].psStatus == TRUE ) &&
      ( psTable[ stationIndex ].envListPtr->CurrNumOfEnv != 0))
   {
	  PAL_intrUnLock(flags);
      return TRUE;
   }
   else 
   {
	  PAL_intrUnLock(flags);
	  return FALSE;
   }
}

/*************************************************************************
 *                        wdrv_psGetTotalStaInPSMode                     *
 *************************************************************************
DESCRIPTION: Returns the number of packets currently in PS mode.

OUTPUT:      

RETURN: The number of packets in PS mode.                                                                                                                                               
************************************************************************/
UINT32  wdrv_psGetTotalStaInPSMode( void )
{
   return numOfStaInPsMode;
}


/*************************************************************************
 *                        wdrv_psGetTotalWoneStaInPSMode                     *
 *************************************************************************
DESCRIPTION: Returns the number of station currently in PS mode for wone.

OUTPUT:      

RETURN: The number of packets in PS mode for wone.                                                                                                                                               
************************************************************************/
UINT32  wdrv_psGetTotalWoneStaInPSMode( UINT8 wone_Index )
{
    return numOfStaInWonePsMode[wone_Index];
    
}



/*************************************************************************
 *                        updatePartVirtBitmap                           *
 *************************************************************************
DESCRIPTION: Returns the number of packets currently in PS mode.

OUTPUT:      

RETURN: The number of packets in PS mode.                                                                                                                                               
************************************************************************/
STATUS updatePartVirtBitmap(UINT16 staIndex, UINT16 status )
{
   UINT8 N1=0,N2=0;
   UINT8 length=0;
   UINT8 offset=0;
   int i;
   UINT16 assocId;
   UINT8  woneIndex;

   assocId =  GetAssocId(staIndex);

   wdrv_dbGetStationWoneIndex(staIndex, &woneIndex);
   if(woneIndex >= MAX_WONE_NET)
   {
	   printf("updatePartVirtBitmap: staIndex=%d, woneIndex=%d\n",staIndex,woneIndex);
	   return NOK; 
   }

   if (REPORT_LEVEL(RL_TX))
      printf("PS : Virtual BitMap change :: assocId=%d  status=%d\n",assocId,status);

   PAL_spinLock(&psLock);  /* START OF CRITICAL SECTION */
   /*---------------------------------------------------*/
   /*   Set the Bitmap                                  */
   /*---------------------------------------------------*/
   if(  status == TRUE ){
      
       if( (partVirtBitmap[assocId/8] & (1<<(assocId % 8))) == TRUE) {
         /* Bit is already set, don't do anything.  */
         PAL_spinUnLock(&psLock);  /* END OF CRITICAL SECTION */
         return OK;
       }
      else {
         /* Bit=0, need to set it to one.. */ 
         partVirtBitmap[assocId/8] |= 1<<(assocId % 8);
         numberOfSetBist++;
      }
   }


   else{ /* STATUS == FALSE */
      
       if((partVirtBitmap[assocId/8] & (1<<(assocId % 8))) == FALSE) {
         /* Bit is already set to FALSE, don't do anything.  */
         PAL_spinUnLock(&psLock);  /* END OF CRITICAL SECTION */
         return OK;
       }      
      else { 
         /* Bit=1, need to turn it off. */ 
         partVirtBitmap[assocId/8] &= ~(1<<(assocId%8));
         numberOfSetBist--;
      }
   }

   /*---------------------------------------------------*/
   /*   Set the length and the offset fields            */
   /*---------------------------------------------------*/
   if ( numberOfSetBist == 0 ){
      /* List is empty clean it and return. */
      highestAssocId = 0;
      lowestAssocId = 0;
      length = 4;
      offset = 0;
   }

   else if(( numberOfSetBist == 1 )&&( status == TRUE )){
       /* First insertion */
         highestAssocId = assocId;
         lowestAssocId = assocId;
   }
   
   else {
      /* All other cases */
      if(  status == TRUE ){
         if( highestAssocId < assocId )
            highestAssocId = assocId;
   
         else if( lowestAssocId > assocId )
            lowestAssocId = assocId;
      }


      else { /* status == FALSE */
         if( assocId == highestAssocId )
            while(( highestAssocId > lowestAssocId ) &&
               (( partVirtBitmap[highestAssocId/8] & 1<<(highestAssocId%8))==0))
                  highestAssocId--;
      
         else if( assocId == lowestAssocId )
            while(( highestAssocId > lowestAssocId ) &&
               (( partVirtBitmap[lowestAssocId/8] & 1<<(lowestAssocId%8))==0))
                  lowestAssocId++;
      }
   }
   
   if((highestAssocId != 0)&&(lowestAssocId!=0)){

      if( (lowestAssocId/8) < 1 )
         N1 = 0;
      else
         N1 = ((lowestAssocId/8)-1) & 0xfffe;
   
      N2 = (highestAssocId/8)+1;

      length = (N2-N1) + 3;
      offset = N1;
   }
   PAL_spinUnLock(&psLock);  /* END OF CRITICAL SECTION */

   if (REPORT_LEVEL(RL_TX)){
      printf("PS : Virtual Bitmap = N1=%d N2=%d len=%d  offset=%d \n
           (lowestAssocId=%d) (highestAssocId=%d) \n ", 
           N1,N2,length, offset>>1, lowestAssocId, highestAssocId );
      printf("BitMap = ");
      for( i=0; i < length-3; i++ )
         printf("%02X,", partVirtBitmap[offset+i] );
         printf("\n");
   }

   whal_apiSetTim(woneIndex, offset<<1, partVirtBitmap , length-3);

   return OK;
}


/*************************************************************************
 *                        wdrv_txPowerSaveAgingFunc                      *
 *************************************************************************
DESCRIPTION: Power Save aging managment.

OUTPUT:      

RETURN:                                                                                                                                               
************************************************************************/
void wdrv_txPowerSaveAgingFunc( void )
{
   TxEnvelop_T  *envelopPtr;
   EnvList_T    *staEnvListPtr;
   UINT16        staTreated = 0;
   UINT16        stationIndex = 0;
   unsigned long flags;

   agingTimeStamp++;

   while( (staTreated < numOfStaInPsMode) &&  (stationIndex < MAX_STATIONS)){
      /* Looping through all the stations in PS mode */
	  PAL_intrLock(flags);
      if( psTable[ stationIndex ].psStatus == TRUE ){
         staEnvListPtr = psTable[stationIndex].envListPtr;
         PAL_intrUnLock(flags);
         while( wdrv_txEnvListWatchFirst( staEnvListPtr, &envelopPtr) == OK ){ 
            /* For any STA in PS we loop through the packets  */ 
                     /* in its envelop list.                           */    
            
             if( envelopPtr->expirTick < agingTimeStamp ){
				 psPacketsExpired++;
                /* Time to expire the packet !!! */
                if(REPORT_LEVEL(RL_TX))
                     wdrv_utilsReport(RL_DB, PRI_DEBUG, "PS: envelop expired !");
                wdrv_txEnvListGetFirst( staEnvListPtr, &envelopPtr );
                wlan_memMngrFreeMSDU( (envelopPtr)->msduPtr->handle  );
                wdrv_txFreeEnv( &envelopPtr );
             }
             else { /* There is a packet but it is not expired yet -> exit this STA */
                 break;
             }
         } /* while( 1 ) */ 
         staTreated++;
      } /* Staion is PS mode */
	  else
	  {	 
		if(psTable[stationIndex].envListPtr->CurrNumOfEnv > 0)
			psPacketsMissed++;
		PAL_intrUnLock(flags);
	  }

      stationIndex++;
   } /* While */
}


/****************************************************************************
 *                      wdrv_powerSavePrintDb()                             *
 ****************************************************************************
 * DESCRIPTION:
 * 
 * INPUTS:
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int wdrv_powerSavePrintDb( char *buf )
{
   int stationIndex ,len = 0;
   len += sprintf(buf +len,"-------------------------- Power Save -------------------------\n");
   len += sprintf(buf +len,"StaInd     StoredPkst     SentPkts               \n");
   len += sprintf(buf +len,"------     ----------     --------               \n"); 
   for( stationIndex=0; stationIndex < MAX_STATIONS; stationIndex++ ){
      if( psTable[ stationIndex ].psStatus == TRUE ) 
         len += sprintf(buf +len,"%6d      %9d     %8d\n", stationIndex, 
         psTable[ stationIndex ].envListPtr->CurrNumOfEnv, 
         psTable[ stationIndex ].sentPkts);   
   }
   len += sprintf(buf +len,"-------------------------------------------------------------\n");
	
   return len;
}


/****************************************************************************
 *                      wdrv_powerSaveCleanStaList()                       *
 ****************************************************************************
 * DESCRIPTION: call Tx Task to Clean all the packets stored in a station power save list 
 *              and set the PS status to FALSE, used for disconnecting a 
 *              staion from the AP.
 * INPUTS: stationIndex - The station Index
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void wdrv_powerSaveCleanStaList( UINT16 staIndex, UINT8 woneIdx )
{
  wdrv_txSendPsCleanStaListMsg( staIndex, woneIdx ) ;
}

/****************************************************************************
 *                      wdrv_powerSaveCleanStaListFunc()                       *
 ****************************************************************************
 * DESCRIPTION: Clean all the packets stored in a station power save list 
 *              and set the PS status to FALSE, used for disconnecting a 
 *              staion from the AP.
 * INPUTS: stationIndex - The station Index
 *		   woneIndex    - The wone Index 
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void wdrv_powerSaveCleanStaListFunc( UINT16 stationIndex )
{
   TxEnvelop_T *envelopPtr;
 
   while( wdrv_psGetEnv( stationIndex, &envelopPtr) == OK )
   {
      wlan_memMngrFreeMSDU( (envelopPtr)->msduPtr->handle  );
      wdrv_txFreeEnv( &envelopPtr );
	  psPacketsCleaned++;
   }
   return;
}


/****************************************************************************
 *                      wdrv_psGetPsPrevTxFail()                       *
 ****************************************************************************
 * DESCRIPTION: Return the status of previous Ps unicast packet send 
 *
 * INPUTS: stationIndex - The station Index
 *		      
 *		
 * 
 * OUTPUT:	prevTxFail - Indecate if this station fail to send previous 
 *          unicast packet
 * 
 ****************************************************************************/

void  wdrv_psGetPsPrevTxFail( UINT8 stationIndex ,UINT8 *prevTxFail)
{
	unsigned long flags;

	PAL_intrLock(flags);
    *prevTxFail = psTable[stationIndex].prevTxFail;
    PAL_intrUnLock(flags);
}

/****************************************************************************
 *                      wdrv_psSetPsPrevTxFail()                       *
 ****************************************************************************
 * DESCRIPTION: Set the PS Prev send fail flag.
 *
 * INPUTS: stationIndex - The station Index
 *		      
 *		
 * 
 * 
 ****************************************************************************/

void  wdrv_psSetPsPrevTxFail( UINT8 stationIndex)
{
	unsigned long flags;

	PAL_intrLock(flags);
    if(psTable[stationIndex].prevTxFail == FALSE)
		PsPrevTxFailCounter ++;
    psTable[stationIndex].prevTxFail = TRUE;
	PAL_intrUnLock(flags);
}

/****************************************************************************
 *                      wdrv_psGetPsPrevTxFailCounter()                       *
 ****************************************************************************
 * DESCRIPTION: return the number of station with PS Prev send fail flag on.
 * 
 ****************************************************************************/

inline void  wdrv_psGetPsPrevTxFailCounter(UINT8 *PsTxFailurs)
{
   *PsTxFailurs = PsPrevTxFailCounter;
}

