/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2003 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
/**************************************************************************/
/*                                                                        */
/*   MODULE:  boot_msg.c                                                  */
/*   PURPOSE: Processes bootLoader messages, implemetation.               */
/*                                                                        */
/**************************************************************************/
#include <rt_os.h>
#include <stdio.h>
#include <flashmap.h>
#include "boot_msg.h"

/******************
 Helper macros
 ******************/
#define LEAP_YEAR(y) (((y) % 4) == 0)


/******************
 Messages
 ******************/
static char bootm_msg1[] = "\nBoot Loader Message Parser;\n";

/******************
 Helper functions
 ******************/
static int cm_hwSaveCmSetup(NVRamSetupDataTYPE* cm_setup);

/***************************************************************
 * bootm_validate_header()
 *
 * Description:
 * Validate the header of the BootLDR message area.
 *
 * Input:
 * BootParamHeaderTYPE *pHdr = pointer to the header.
 *
 * Return: OK / NOK
 ***************************************************************/
static int bootm_validate_header( BootParamHeaderTYPE *pHdr)
{
   /* Invalid magic or < first version, just ignore */
   if( (pHdr->magic != BOOTM_PARAMETER_PASS_MAGIC) ||
       (pHdr->version < BOOTM_PARAMS_VERSION1_0  ) )
   {
      return NOK;
   }

   /* warn and abort if boot loader has a more advanced format version.
      The application parser should not know how to parse this, so abort.
   */
   if( pHdr->version > BOOTM_PARAMS_VERSION)
   {
      printf(
         "%sReading format version %u.%u, supported up to %u.%u.\n"
         "Please get an update.\n",
         bootm_msg1,
         BOOTM_HIBYTE_16(pHdr->version),
         BOOTM_LOBYTE(pHdr->version),
         BOOTM_HIBYTE_16(BOOTM_PARAMS_VERSION),
         BOOTM_LOBYTE(BOOTM_PARAMS_VERSION)
         );

      return NOK;
   }

   /* Validate more according to the format version */
   if( pHdr->messageCount < 1)
   {
      printf("%sShould be at least one message.\n", bootm_msg1);
      return NOK;
   }

   /* Parser supports versions up to BOOTM_PARAMS_VERSION */
   return OK;
}/* END bootm_validate_header() */

/*************************************************************
 * bootm_validate_format()
 *
 * Description:
 * Validate the format of the BootLDR message area.
 *
 * Input:
 * BootParamHeaderTYPE *pHdr = pointer to the header.
 *
 * Return: OK / NOK
 *************************************************************/
static int bootm_validate_format(BootParamHeaderTYPE *pHdr)
{
   int i, msg_count;
   BootMessageTYPE *pMsg;
   /* Iterate through all the messages and verify BOOTM_MESSAGE_STOP
      and that we do not exceed BOOTM_PARAM_SAFE_MEM_SIZE */
   pMsg = BOOTM_FIRST_MESSAGE( pHdr);
   msg_count = pHdr->messageCount;

   /* Iterate to the last message */
   for( i = 1; i< msg_count; i++)
   {
      pMsg = BOOTM_NEXT_MESSAGE(pMsg);
      /* check address range validity */
      if( ((UINT32)pMsg < (UINT32)pHdr) ||
          ((UINT32)pMsg >= ((UINT32)pHdr + BOOTM_PARAM_SAFE_MEM_SIZE)) )
      {
         printf( "%sRedirection to bad pointer.\n", bootm_msg1);
         return NOK;
      }
   }

   /* Verify last message */
   if( (pMsg->message_type != BOOTM_MESSAGE_STOP) ||
       (pMsg->parameter_length != BOOTM_MESSAGE_STOP_LENGTH))
   {
      printf( "%sCannot find STOP message.\n", bootm_msg1);
      return NOK;
   }

   return OK;
}/* END bootm_validate_format() */

/**********************************************************
 * bootm_invalidate_header()
 *
 * Description:
 * Invalidate the BootLDR message area by erasing the magic.
 *
 * Input:
 * volatile BootParamHeaderTYPE* pHdr = Header start
 *
 * Return: none
 **********************************************************/
static void bootm_invalidate_header(volatile BootParamHeaderTYPE* pHdr)
{
   /* override the magic number */
   pHdr->magic = 0;
}/* END bootm_invalidate_header() */


/**********************************************************
 * cm_hwDumpMessage()
 *
 * Description:
 * Dumps a BootLDR message, using VxWorks d() function.
 *
 * Input:
 * BootMessageTYPE *pMsg
 *
 * Return: none
 **********************************************************/
static int cm_hwDumpMessage( BootMessageTYPE *pMsg)
{
   printf("\nBootLDR Message @ %p, type = 0x%04x argument size = 0x%04x\n",
      (void*)pMsg, (UINT)(pMsg->message_type), (UINT)(pMsg->parameter_length));
   /* call VxWorks Shell dump function Display Memory d(addr, length, size)  */
#ifndef _SAPP_STAND_ALONE_
   d( BOOTM_MSG_ARG(pMsg), pMsg->parameter_length >> 2, 4);
#endif
}/* END cm_hwDumpMessage() */


/***************************************************************
 * cm_hwHandleBootInfo1()
 *
 * Description:
 * Reads or updates boot information. The application may
 * only read this information, using cm_hwGetBootInfo1().
 * The routine validates the date and returns NOK if incvalid.
 *
 * Input:
 * BootInfo1TYPE *pInfo = pointer to result or input data.
 * int update = 0: read, other: write
 *
 * Return: OK / NOK
 ***************************************************************/
static int cm_hwHandleBootInfo1(BootInfo1TYPE *pInfo, int update)
{
   static int valid = NOK;
   static BootInfo1TYPE info1;

   if( update)
   {
      /* get info structure and validate */
      if( (pInfo->year >= 2000)               &&
          BOOTM_INRANGE_MONTH(pInfo->month)  &&
          BOOTM_INRANGE(pInfo->day,
            cm_hwMaxDaysInMonth(pInfo->year, pInfo->month) , 1) )
      {
         BOOTM_COPY_OBJECT( info1, pInfo);
         valid = OK;
      }
   }
   else
   {
      /* store the object if valid, then exit */
      if( valid == OK)
      {
         if( pInfo == NULL) return NOK;
         BOOTM_COPY_OBJECT( *pInfo, &info1);
      }
   }/* endIf */

   return valid;
}/* END cm_hwHandleBootInfo() */


/***************************************************************
 * cm_hwHandleBootNVRAMUpdate()
 *
 * Description:
 * Updates NVRAM according to BOOTLDR request
 *
 * Input:
 * BootMessageTYPE *pMsg
 *
 * Return: OK / NOK
 ***************************************************************/
static int cm_hwHandleBootNVRAMUpdate( BootMessageTYPE *pMsg)
{
   NVRamSetupDataTYPE *pCM_SETUP;
   void *dest ;

   puts("\nBoot Loader caused NVARM rewriting.");
   pCM_SETUP = (NVRamSetupDataTYPE*)BOOTM_MSG_ARG(pMsg);
   dest  = (void*)(NVRAM_USER_ADDR);

   /* Ensure NVRAM is initialized */
   cm_hwInitNVRAM();
   cm_hwSaveCmSetup( pCM_SETUP);
   return OK;
}

/***************************************************************
 * cm_hwProcessMessage()
 *
 * Description:
 * BootLDR Message processor.
 *
 * Input:
 * BootMessageTYPE *pMsg = pointer of message to process.
 *
 * Return: OK / NOK
 ***************************************************************/
static int cm_hwProcessMessage(BootMessageTYPE *pMsg)
{
   int retCode = OK;

   switch( pMsg->message_type)
   {
   case BOOTM_MESSAGE_STOP:
   case BOOTM_MESSAGE_NOP:
      /* ignore */
      break;

   case BOOTM_MESSAGE_BOOTM_INFO1:
      retCode = cm_hwHandleBootInfo1((BootInfo1TYPE *)BOOTM_MSG_ARG(pMsg), 1);
      break;

   case BOOTM_MESSAGE_UPDATE_NVRAM:
      retCode = cm_hwHandleBootNVRAMUpdate(pMsg);
      break;

   default: /* dump unknown messages */
      retCode = cm_hwDumpMessage(pMsg);
   }

   return retCode;
}/* END cm_hwProcessMessage() */


/***************************************************************
 * cm_hwParseBootMessages()
 *
 * Description:
 * Parse the boot messages from a safe memory area.
 *
 * Input:
 * BootMessageTYPE *pMsg = pointer to message.
 *
 * Return: none
 ***************************************************************/
void cm_hwParseBootMessages(void)
{
   BootParamHeaderTYPE *pHdr;
   BootMessageTYPE *pMsg;
   int i;

   /* Get start */
   pHdr = (BootParamHeaderTYPE *)(BOOTM_PARAM_SAFE_MEM);

   /* Get header and test mandatory fields */
   if( bootm_validate_header( pHdr) != OK)
      return;

   /* Validate format */
   if( bootm_validate_format(pHdr) != OK)
      return;

   /* Do the job */
   pMsg = BOOTM_FIRST_MESSAGE( pHdr);
   for( i=1; i< pHdr->messageCount; i++)
   {
      if( cm_hwProcessMessage(pMsg) != OK)
      {
         /* stop on first error */
         printf( "%sError processing message @ %p, type 0x%04x.\n",
            bootm_msg1, (void*)pMsg, (UINT)(pMsg->message_type));
         break;
      }
      pMsg = BOOTM_NEXT_MESSAGE(pMsg);
   }

   /* Block further calls */
   bootm_invalidate_header( pHdr);
}/* END cm_hwParseBootMessages() */


/***************************************************************
 * cm_hwMaxDaysInMonth()
 *
 * Description:
 * Returns maximum days in a month given year and month.
 *
 * Input:
 * int year >=2000
 * int month = [1..12]
 *
 * Return: OK / NOK
 ***************************************************************/
int cm_hwMaxDaysInMonth(int year, int month)
{
   int retVal;
   static char days[]      = {31,28,31,30,31,30,31,31,30,31,30,31};
   static char leap_days[] = {31,29,31,30,31,30,31,31,30,31,30,31};

   if( BOOTM_INRANGE_MONTH(month) && (year >= 2000))
   {
      --month;
      if( LEAP_YEAR(year))
      {
         retVal = leap_days[month];
      }
      else
      {
         retVal = days[month];
      }
   }/* endIf */

   return retVal;
} /* END cm_hwMaxDaysInMonth() */


/******************************************************************
 * cm_hwGetBootInfo1()
 *
 * Description:
 * API to read the BootLoader information. The application should
 * verify the return value.
 *
 * Input:
 * BootInfo1TYPE *pInfo = pointer to result.
 *
 * Return: OK / NOK
 ******************************************************************/
int  cm_hwGetBootInfo1( BootInfo1TYPE *pInfo)
{
   return cm_hwHandleBootInfo1( pInfo, 0);
}/* END cm_hwGetBootInfo1() */

/******************************************************************
 * cm_hwMonthName()
 *
 * Description:
 * API to get the full / short name of a month.
 * 
 *
 * Input:
 * int month      = month [1..12]
 * int longFormat = 0: short format, Other: long format
 *
 * Return: month name or empty string
 ******************************************************************/
const char *cm_hwMonthName(int month, int longFormat)
{
   static const char *nsMonth[] =
   {
      "",
      "Jan",      "Feb",      "Mar",      "Apr",
      "May",      "Jun",      "Jul",      "Aug",
      "Sep",      "Oct",      "Nov",      "Dec"
   };

   static const char *nMonth[] =
   {
      "January",      "February",      "March",
      "April",        "May",           "June",
      "July",         "August",        "September",
      "October",      "November",      "December"
   };

   if( BOOTM_INRANGE_MONTH(month))
   {
      if( longFormat)
      {
         return nMonth[month];
      }
      else
      {
         /* short format */
         return nsMonth[month-1];
      }/* endIf */
   }

   /* retrun empty string */
   return nsMonth[0];
}  /* END cm_hwMonthName() */

/******************************************************************
 * cm_hwFormatBootInfo1()
 *
 * Description:
 * API to get the full name of a month.
 * 
 * Input:
 * char* buf   = buffer to fill
 * int length  = size of the buffer, including the terminating null
 *
 * Return: actual characters written
 ******************************************************************/
int  cm_hwFormatBootInfo1( char* buf, int length)
{
   /*  version: x.y, date: dd-mmm-yyyy */
   BootInfo1TYPE info;
   char mbuf[80];
   int  actualLength = 0;

   mbuf[0] = '\0';
   if( cm_hwGetBootInfo1(&info) == OK)
   {
      sprintf( mbuf,
         "Boot LDR - Ver: %u.%u.%u, Date: %02u-%s-%u",
         (UINT)(info.majorVersion),
         (UINT)(info.minorVersion),
         (UINT)(info.patch),
         (UINT)(info.day),
         cm_hwMonthName(info.month, 0),
         (UINT)(info.year)
         );
   }
   else
   {
      return actualLength;
   }
   
   if( buf != NULL)
   {
      actualLength = min( strlen(mbuf)+1, length);
      strncpy( buf, mbuf, actualLength);
   }

   return actualLength;
} /* END cm_hwFormatBootInfo1() */


static int cm_hwSaveCmSetup(NVRamSetupDataTYPE* cm_setup)
{
   memcpy (
      cmNvramGetShadow(), /* shadow pointer, taken from board/nvram.c */
      cm_setup,
      sizeof(NVRamSetupDataTYPE) /* protect only system area */
   );
   cmNvramWrite();
   return OK;
}

