/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
/**************************************************************************/
/*                                                                        */
/*   MODULE:    NVRAM.C                                                  */
/*   PURPOSE:   Wireless HAL NVRAM services implementatioin               */
/*                                                                        */
/*   Routines list:                                                       */
/**************************************************************************/
#include <rt_os.h>
#include <flashmap.h>
#include <flashacc.h>
#include "boot_msg.h"

static rt_os_sem_t nvramSema;
static int         hwInitNvramFirstTime;
static NVRamSetupDataTYPE hwNvramShadow;


/*****************************************************************/
/* Basic hwNVRAM Functions                                       */
/*****************************************************************/

void* cmNvramGetShadow(void)
{
   return (void*)(&hwNvramShadow);
}

void cmNvramRead(void) 
{
   void *dest, *src;

   dest  = (void *)(&hwNvramShadow);
   src   = (void *)(NVRAM_USER_ADDR);

   rt_osSemTake(nvramSema);
   StorageRead( sizeof(NVRamSetupDataTYPE), src, dest);
   rt_osSemGive(nvramSema);
}

void cmNvramWrite(void) 
{
   void *dest, *src;
   
   src   = (void *)(&hwNvramShadow);
   dest  = (void *)(NVRAM_USER_ADDR);

   rt_osSemTake(nvramSema);
   StorageWrite( sizeof(NVRamSetupDataTYPE), dest, src);
   rt_osSemGive(nvramSema);
}

/**************************
 * Application storage    *
 **************************/
int wlanNVRAMReadCB(unsigned char *pbSrc, unsigned short wLen)
{
   int retVal;
   rt_osSemTake(nvramSema);
   retVal = UsrAppStorageRead( (unsigned int)wLen, (void*)0, (void*)pbSrc);
   rt_osSemGive(nvramSema);
   return retVal;
}

int wlanNVRAMWriteCB(unsigned char *pbSrc, unsigned short wLen)
{
   int retVal;
   rt_osSemTake(nvramSema);
   retVal = UsrAppStorageWrite( (unsigned int)wLen, (void*)0, (void*)pbSrc);
   rt_osSemGive(nvramSema);
   return retVal;
}


/**************************************************************************
 *               cm_hwInitNVRAM()                                         *
 **************************************************************************
 * DESCRIPTION:  Initialize the NVRAM usage.                              *
 *                                                                        *
 **************************************************************************/
void cm_hwInitNVRAM() 
{
   unsigned long rc;
   if (hwInitNvramFirstTime == 1)
	   return;

   hwInitNvramFirstTime = 1;
   rc = rt_osSemCreate("NVRS",1,OS_SEM_FIFO,&nvramSema);
   if (rc != OS_NOERR)
    {
        logMsg("NVRAM Semaphore creation error: %d", rc,0,0,0,0,0);
    }
   cmNvramRead();

}

/***********************************************************************/
/* Special Functions                                                   */
/***********************************************************************/


unsigned long cm_hwNVRAM_takeSem(void)
{
   return rt_osSemTake(nvramSema);
}

unsigned long cm_hwNVRAM_giveSem(void)
{
   return rt_osSemGive(nvramSema);
}

/**************************************************************************
 *               cm_hwRestoreSerialNumber()                               *
 **************************************************************************
 * DESCRIPTION:  Restore Serial Number from NVRAM.                        *
 *                                                                        *
 * INPUT:        no.                                                      *
 * OUTPUT:       SerialNumber - data pointer to copy to NVRAM.            *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwRestoreSerialNumber( char *SerialNumber)
{
   rt_osSemTake(nvramSema);
   if( strlen( hwNvramShadow.SerialNumber) >= HW_SER_NUM_LEN) 
   {
      strcpy( hwNvramShadow.SerialNumber, "0000");
      cmNvramWrite();
   }
   strcpy( SerialNumber, hwNvramShadow.SerialNumber);
   rt_osSemGive(nvramSema);
   return OK;
}

/**************************************************************************
 *               cm_hwSaveSerialNumber()                                  *
 **************************************************************************
 * DESCRIPTION:  Save Serial Number to NVRAM.                             *
 *                                                                        *
 * INPUT:        SerialNumber - data pointer to copy to NVRAM.            *
 * OUTPUT:       no.                                                      *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwSaveSerialNumber( char *SerialNumber)
{
   if( strlen( SerialNumber) >= HW_SER_NUM_LEN)
   {
      return NOK;
   }
   rt_osSemTake(nvramSema);
   strcpy( hwNvramShadow.SerialNumber, SerialNumber);
   rt_osSemGive(nvramSema);
   return OK;
}

/**************************************************************************
 *               cm_hwRestoreHostMacAddress()                             *
 **************************************************************************
 * DESCRIPTION:  Restore MAC address from NVRAM.                          *
 *                                                                        *
 * INPUT:        no.                                                      *
 * OUTPUT:       MacAddr - data pointer to copy from NVRAM.               *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwRestoreHostMacAddress( unsigned char *MacAddr)
{
   rt_osSemTake(nvramSema);
   memcpy( MacAddr, hwNvramShadow.dot11MacAddress, HW_MAC_ADDR_LEN);
   rt_osSemGive(nvramSema);
   return OK;
}

/**************************************************************************
 *               cm_hwSaveHostMacAddress()                                *
 **************************************************************************
 * DESCRIPTION:  Save MAC address to NVRAM.                               *
 *                                                                        *
 * INPUT:        MacAddr - data pointer to copy to NVRAM.                 *
 * OUTPUT:       no.                                                      *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwSaveHostMacAddress( unsigned char *MacAddr)
{
   rt_osSemTake(nvramSema);
   memcpy( hwNvramShadow.dot11MacAddress, MacAddr, HW_MAC_ADDR_LEN);
   rt_osSemGive(nvramSema);
   return OK;
}

/*************************************************************************
*       int cm_hwRestoreProductionPassword(char *pb_password)            *
**************************************************************************
* DESCRIPTION:  Restore Production Password from NVRAM.                  *
*                                                                        *
* INPUT:        no.                                                      *
* OUTPUT:       password data pointer to copy from NVRAM.                *
* RETURN:       0 if successful, otherwise 1.                            *
*                                                                        *
**************************************************************************/
int cm_hwRestoreProductionPassword( char *pb_password)
{
    rt_osSemTake(nvramSema);
    memcpy( pb_password, hwNvramShadow.pb_password, HW_PROD_PASSWORD_LEN);
    pb_password[HW_PROD_PASSWORD_LEN] = 0x00;
    rt_osSemGive(nvramSema);
    return OK;
}

/*************************************************************************
*       int cm_hwSaveProductionPassword(char *pb_password)               *
**************************************************************************
* DESCRIPTION:  Save Production Password to NVRAM.                       *
*                                                                        *
* INPUT:                                                                 *
* OUTPUT:                                                                *
* RETURN:       0 if successful, otherwise 1.                            *
*                                                                        *
**************************************************************************/
int cm_hwSaveProductionPassword( char *pb_password)
{
    if( strlen(pb_password) >= HW_PROD_PASSWORD_LEN)
    {
        return ERROR;
    }
    rt_osSemTake(nvramSema);
    memcpy( hwNvramShadow.pb_password, pb_password, HW_PROD_PASSWORD_LEN);
    rt_osSemGive(nvramSema);
    return OK;
}

/**************************************************************************
 *               cm_hwRestoreSwOperStatus()                               *
 **************************************************************************
 * DESCRIPTION:  Restore upgrade Operat. Status from NVRAM.               *
 *                                                                        *
 * INPUT:        no.                                                      *
 * OUTPUT:       status - pointer to status to copy from NVRAM.           *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwRestoreSwOperStatus( long *status)
{
   *status = (long)hwNvramShadow.SwOperStatus;
   return OK;
}

/**************************************************************************
 *               cm_hwSaveSwOperStatus()                                  *
 **************************************************************************
 * DESCRIPTION:  Save upgrade Operat. Status to NVRAM.                    *
 *                                                                        *
 * INPUT:        status - status value to copy to NVRAM.                  *
 * OUTPUT:       no.                                                      *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwSaveSwOperStatus( long status)
{
   hwNvramShadow.SwOperStatus = (short)status;
   cmNvramWrite();
   return OK;
}


/**************************************************************************
 *               cm_hwRestoreSwAdminStatus()                              *
 **************************************************************************
 * DESCRIPTION:  Restore upgrade Admin Status from NVRAM.                 *
 *                                                                        *
 * INPUT:        no.                                                      *
 * OUTPUT:       status - pointer to status to copy from NVRAM.           *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwRestoreSwAdminStatus( long *status)
{
   *status = hwNvramShadow.SwAdminStatus;
   return OK;
}

/**************************************************************************
 *               cm_hwSaveSwAdminStatus()                                 *
 **************************************************************************
 * DESCRIPTION:  Save upgrade Admin Status to NVRAM.                      *
 *                                                                        *
 * INPUT:        status - status value to copy to NVRAM.                  *
 * OUTPUT:       no.                                                      *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwSaveSwAdminStatus( long status)
{
   hwNvramShadow.SwAdminStatus = status;
   cmNvramWrite();
   return OK;
}

/**************************************************************************
 *               cm_hwRestoreSwServer()                                   *
 **************************************************************************
 * DESCRIPTION:  Restore upgrade SW server address from NVRAM.            *
 *                                                                        *
 * INPUT:        no.                                                      *
 * OUTPUT:       ip_addr - address pointer to copy from NVRAM.            *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwRestoreSwServerAddr( unsigned long *ip_addr)
{
   *ip_addr = hwNvramShadow.SwServerAddr;
   return OK;
}

/**************************************************************************
 *               cm_hwSaveSwServer()                                      *
 **************************************************************************
 * DESCRIPTION:  Save upgrade SW server address to NVRAM.                 *
 *                                                                        *
 * INPUT:        ip_addr - address value to copy to NVRAM.                *
 * OUTPUT:       no.                                                      *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwSaveSwServerAddr( unsigned long ip_addr)
{
   hwNvramShadow.SwServerAddr = ip_addr;
   cmNvramWrite();
   return OK;
}

/**************************************************************************
 *               cm_hwRestoreSwServerFilename()                           *
 **************************************************************************
 * DESCRIPTION:  Restore upgrade SW file name from NVRAM.                 *
 *                                                                        *
 * INPUT:        no.                                                      *
 * OUTPUT:       FileName - data pointer to copy to NVRAM.                *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwRestoreSwServerFilename(char *string)
{
   if( strlen( hwNvramShadow.SwServerFilename) >= HW_FILE_NAME_LEN)
   {
      hwNvramShadow.SwServerFilename[0] = '\0';
      cmNvramWrite();
   }
   rt_osSemTake(nvramSema);
   strcpy( string, hwNvramShadow.SwServerFilename);
   rt_osSemGive(nvramSema);
   if( !strlen(string)) return NOK;
   return OK;
}


/**************************************************************************
 *               cm_hwSaveSwServerFilename()                              *
 **************************************************************************
 * DESCRIPTION:  Save upgrade SW file name to NVRAM.                      *
 *                                                                        *
 * INPUT:        FileName - data pointer to copy to NVRAM.                *
 * OUTPUT:       no.                                                      *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwSaveSwServerFilename( char *FileName)
{
   if( strlen(FileName) >= HW_FILE_NAME_LEN)
   {
      return NOK;
   }
   rt_osSemTake(nvramSema);
   strcpy( hwNvramShadow.SwServerFilename, FileName);
   rt_osSemGive(nvramSema);
   cmNvramWrite();
   return OK;
}

/**************************************************************************
 *               cm_hwRestoreSwFilename()                                 *
 **************************************************************************
 * DESCRIPTION:  Restore image SW file name from NVRAM.                   *
 *                                                                        *
 * INPUT:        no.                                                      *
 * OUTPUT:       FileName - data pointer to copy from NVRAM.              *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwRestoreSwFilename( long ind,char *string)
{
   if( (ind >= HW_MAX_FILE_NUM) || (ind < 0))
      return NOK;

   if( strlen( hwNvramShadow.SwFilename[ind]) >= HW_FILE_NAME_LEN)
   {
      hwNvramShadow.SwFilename[ind][0] = '\0';
      cmNvramWrite();
   }
   rt_osSemTake(nvramSema);
   strcpy( string, hwNvramShadow.SwFilename[ind]);
   rt_osSemGive(nvramSema);
   if( !strlen(string)) return NOK;
   return OK;
}

/**************************************************************************
 *               cm_hwSaveSwFilename()                                    *
 **************************************************************************
 * DESCRIPTION:  Save image SW file name to NVRAM.                        *
 *                                                                        *
 * INPUT:        FileName - data pointer to copy to NVRAM.                *
 * OUTPUT:       no.                                                      *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwSaveSwFilename( long ind,char *FileName)
{
   if ((ind >= HW_MAX_FILE_NUM) || (ind < 0))
      return NOK;
   if( strlen(FileName) >= HW_FILE_NAME_LEN)
      return NOK;
   rt_osSemTake(nvramSema);
   strcpy( hwNvramShadow.SwFilename[ind], FileName);
   rt_osSemGive(nvramSema);
   cmNvramWrite();
   return OK;
}

/**************************************************************************
 *               cm_hwRestoreSwImageOffset()                              *
 **************************************************************************
 * DESCRIPTION:  Restore SW flash image offset from NVRAM.                *
 *                                                                        *
 * INPUT:        ind - image index.                                       *
 * OUTPUT:       offset - offset pointer to copy from NVRAM.              *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwRestoreSwImageOffset(long ind,unsigned long *offset)
{
   if ((ind >= HW_MAX_FILE_NUM) || (ind < 0))
      return NOK;
   *offset = hwNvramShadow.SwImageOffset[ind];
   return OK;
}

/**************************************************************************
 *               cm_hwSaveSwImageOffset()                                 *
 **************************************************************************
 * DESCRIPTION:  Save SW flash image offset to NVRAM.                     *
 *                                                                        *
 * INPUT:        ind - image index.                                       *
 *               offset - offset value to copy to NVRAM.                  *
 * OUTPUT:       no.                                                      *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwSaveSwImageOffset(long ind,unsigned long offset)
{
   if ((ind >= HW_MAX_FILE_NUM) || (ind < 0))
      return NOK;
   hwNvramShadow.SwImageOffset[ind] = offset;
   cmNvramWrite();
   return OK;
}

/**************************************************************************
 *               cm_hwRestoreSwImageLength()                              *
 **************************************************************************
 * DESCRIPTION:  Restore SW flash image length from NVRAM.                *
 *                                                                        *
 * INPUT:        ind - image index.                                       *
 * OUTPUT:       len - length pointer to copy from NVRAM.                 *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwRestoreSwImageLength(long ind,unsigned long *len)
{
   if ((ind >= HW_MAX_FILE_NUM) || (ind < 0))
      return NOK;
   *len = hwNvramShadow.SwImageLength[ind];
   return OK;
}

/**************************************************************************
 *               cm_hwSaveSwImageLength()                                 *
 **************************************************************************
 * DESCRIPTION:  Save SW flash image length to NVRAM.                     *
 *                                                                        *
 * INPUT:        ind - image index.                                       *
 *               len - length value to copy to NVRAM.                     *
 * OUTPUT:       no.                                                      *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwSaveSwImageLength(long ind,unsigned long len)
{
   if ((ind >= HW_MAX_FILE_NUM) || (ind < 0))
      return NOK;
   hwNvramShadow.SwImageLength[ind] = len;
   cmNvramWrite();
   return OK;
}

/**************************************************************************
 *               cm_hwRestoreSwImageCrc()                                 *
 **************************************************************************
 * DESCRIPTION:  Restore SW flash image CRC from NVRAM.                   *
 *                                                                        *
 * INPUT:        ind - image index.                                       *
 * OUTPUT:       crc - CRC pointer to copy from NVRAM.                    *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwRestoreSwImageCrc(long ind,unsigned long *crc)
{
   if ((ind >= HW_MAX_FILE_NUM) || (ind < 0))
      return NOK;
   *crc = hwNvramShadow.SwImageCrc[ind];
   return OK;
}

/**************************************************************************
 *               cm_hwSaveSwImageCrc()                                    *
 **************************************************************************
 * DESCRIPTION:  Save SW flash image CRC to NVRAM.                        *
 *                                                                        *
 * INPUT:        ind - image index.                                       *
 *               crc - CRC value to copy to NVRAM.                        *
 * OUTPUT:       no.                                                      *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwSaveSwImageCrc(long ind,unsigned long crc)
{
   if ((ind >= HW_MAX_FILE_NUM) || (ind < 0))
      return NOK;
   hwNvramShadow.SwImageCrc[ind] = crc;
   cmNvramWrite();
   return OK;
}

/****************************************************************************
*		  	             cm_hwSaveImage                                        *
*****************************************************************************
* DESCRIPTION:	Update image parameters.                                     *
*									                                                 *
* INPUT:	FlashCopy - pointer to flash address.			                      *
* 		DataLen - image length.					                                  *
* OUTPUT:	0 if successful, otherwise 1.				                         *
*									                                                 *
*****************************************************************************/
int cm_hwSaveImage(int ind, char *fname, unsigned char *FlashCopy, 
                   unsigned long DataLen, unsigned long crc) 
{
   cm_hwSaveSwImageLength( ind,DataLen);
   cm_hwSaveSwImageOffset( ind, FlashCopy - (UCHAR *)FLASH_BASE_ADDR);
   if (DataLen > 0) 
   {
        cm_hwSaveSwImageCrc( ind, crc);
        cm_hwSaveSwFilename( ind, fname);
   }
   return OK;
}

/**************************************************************************
 *               cm_hwRestoreStartupParams()                              *
 **************************************************************************
 * DESCRIPTION:  Restore startup parameters from NVRAM.                   *
 *                                                                        *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwRestoreStartupParams( unsigned long *sig, unsigned long *lp1, unsigned long *lp2)
{
   rt_osSemTake(nvramSema);
   if( sig != NULL) *sig = hwNvramShadow.startup_param_signature;
   if( lp1 != NULL) *lp1 = hwNvramShadow.startup_lparam1;
   if( lp2 != NULL) *lp2 = hwNvramShadow.startup_lparam2;
   rt_osSemGive(nvramSema);
   return OK;
}

/**************************************************************************
 *               cm_hwSaveStartupParams()                                 *
 **************************************************************************
 * DESCRIPTION:  Save startup parameters to NVRAM.                        *
 *                                                                        *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
int cm_hwSaveStartupParams( unsigned long sig, unsigned long lp1, unsigned long lp2)
{
   rt_osSemTake(nvramSema);
   hwNvramShadow.startup_param_signature = sig;
   hwNvramShadow.startup_lparam1 = lp1;
   hwNvramShadow.startup_lparam2 = lp2;
   rt_osSemGive(nvramSema);
   return OK;
}



/**************************************************************************
 *               cm_hwSetDefMacAddr()                                     *
 **************************************************************************
 * DESCRIPTION:  Set the deffault mac address to NVRAM.                   *
 *                                                                        *
 * RETURN:       0 if successful, otherwise 1.                            *
 *                                                                        *
 **************************************************************************/
void cm_hwSetDefMacAddr(void)
{
   UINT32 serial=0;
   char sernum[HW_SER_NUM_LEN];
   char devmac[HW_MAC_ADDR_LEN];
   rt_osSemTake(nvramSema);

   cm_hwRestoreSerialNumber( sernum);
   if( 1 == sscanf( sernum, "%x", &serial))
   {
      cm_hwRestoreHostMacAddress( devmac);
      devmac[4] = (serial & 0x0000FF00)>>8;
      devmac[5] = (serial & 0x000000FF);
      cm_hwSaveHostMacAddress( devmac);
   }
   rt_osSemGive(nvramSema);

}
