/****************************************************************************
**+-----------------------------------------------------------------------+**
**|                              ****                                     |**
**|                              ****                                     |**
**|                           ******o***                                  |**
**|                    ********_///_****                                  |**
**|                     ***** /_//_/ ****                                 |**
**|                      ** ** (__/ ****                                  |**
**|                            *********                                  |**
**|                             ****                                      |**
**|                              ***                                      |**
**|                                                                       |**
**|     Copyright (c) 1998 - 2002 Texas Instruments Incorporated          |**
**|                        ALL RIGHTS RESERVED                            |**
**|                                                                       |**
**| Permission is hereby granted to licensees of Texas Instruments        |**
**| Incorporated (TI) products to use this computer program for the sole  |**
**| purpose of implementing a licensee product based on TI products.      |**
**| No other rights to reproduce, use, or disseminate this computer       |**
**| program, whether in part or in whole, are granted.                    |**
**|                                                                       |**
**| TI makes no representation or warranties with respect to the          |**
**| performance of this computer program, and specifically disclaims      |**
**| any responsibility for any damages, special or consequential,         |**
**| connected with the use of this program.                               |**
**|                                                                       |**
**+-----------------------------------------------------------------------+**
****************************************************************************/
/***************************************************************************/
/*                                                                         */
/*   MODULE:  user_leds.c                                                  */
/*   PURPOSE: Modem User Interface Functions.                              */
/*                                                                         */
/***************************************************************************/

#ifdef _VXWORKS_RTOS_
	#include <vxworks.h>
#endif

/* RTOS calls rt_osMsecs2Ticks() */
#include "rt_os.h"

#include "utilsTcb.h"
#include "usr_leds.h"

/* Low level HW access from the BSP */
#include "leds.h"

#ifdef USB_DRIVER
#include "RndisSnmp.h"
#endif

/* Helper Macro */
#define LED2MASK(ledId)   (UINT32)(1L << (ledId))
#define LEDSETMASK(msk, ledId)   (msk) |= LED2MASK(ledId)
#define LEDXORMASK(msk, ledId)   (msk) ^= LED2MASK(ledId)
#define LEDCLRMASK(msk, ledId)   (msk) &= ~(LED2MASK(ledId))

/* Static variables */

static lbt_tcbObjTYPE * p_tcb_LedsTimer;
USR_getCountCB getCountCB = NULL;

/* Static functions */
static void USR_leds_handler(void);

#define LEDS_TIMER_RATE_MSEC 100  /* LEDS timer tick is 100ms, fast blink */
#define LEDS_SLOW_BLINK_COUNT 5   /* count value, ticks for slow blink    */


/*******************
 HW initialization
********************/

void USR_leds_hw_init(void)
{
	leds_hw_init();	/* Turn Avalanche LEDs off */
}

/**************************************************************************/
/*                USR_ledsInit()                                          */
/**************************************************************************/
/* DESCRIPTION: Initializes LEDs hardware.  Hooks to TCB timer            */
/*                                                                        */
/* INPUT:       no                                                        */
/* OUTPUT:      0 OK, else fail.                                          */
/*                                                                        */
/**************************************************************************/
int USR_ledsInit(USR_getCountCB getCountCB_param)
{
	/* Save callback param */
	getCountCB = getCountCB_param;

	/* initialize LEDs HW, turning all SW controllable LEDs off */
	USR_leds_hw_init();

	/*
	 Create and Hook a TCB timer, LEDs does not require a
	 message queue , therefore we don't call rt_osQueueCreate(...)
	 and call lbt_tcbNew(...) with NULL.
	*/
	p_tcb_LedsTimer = lbt_tcbNew( NULL);
	if( p_tcb_LedsTimer == NULL)
	{
		printf("LEDs: Error creating timer\n");
		return -1;
	}

	lbt_tcbQueueAct( p_tcb_LedsTimer,
                     (unsigned long)(rt_osMsecs2Ticks( LEDS_TIMER_RATE_MSEC)),
                     USR_leds_handler,
                     1 /* periodic*/ );

	return 0;
}

/**************************************************************************/
/*                USR_leds_handler()                                      */
/**************************************************************************/
/* DESCRIPTION: leds updating and logic.                                  */
/*                                                                        */
/* INPUT:       no                                                        */
/* OUTPUT:      no.                                                       */
/*                                                                        */
/**************************************************************************/
static void USR_leds_handler(void)
{
   static UINT32 ulLedsMask=0L; /* holds LEDs state , forcing initial update */
   static UINT32 ulLedsMask_old=-1L;
   long l_dbStatCbl;
   
   /* POWER LED, if available */
   LEDSETMASK( ulLedsMask, e_LED_POWER);	/* solid on, indicates AP is OK */
   
   {
      static UINT32 count=0;
      UINT32 n_count;
      
      /* WLAN_LINK LED, if available */
      LEDSETMASK( ulLedsMask, e_LED_WLAN_LINK);	/* solid on, indicates WLAN broadcasts */
      
      /* WLAN_ACT LED, WLAN always active */
      /* blink - on packets */
      if (getCountCB != NULL)
         getCountCB(&n_count);
      else
         count = 0;
      
      if( n_count != count)
      {
         count = n_count;
         LEDXORMASK( ulLedsMask, e_LED_WLAN_ACT);
      }
      else
      {
         LEDCLRMASK( ulLedsMask, e_LED_WLAN_ACT);
      }
   }
   
   
   /* Set the LEDs according to ulLedsMask */
   if( ulLedsMask_old != ulLedsMask)
   {
      int i;
      
      ulLedsMask_old = ulLedsMask;
      for( i=0; i<e_LEDS_COUNT; i++)
      {
         /* Call directly BSP */
         leds_hw_set( i,
            (ulLedsMask & LED2MASK(i))?	LED_COLOR_GREEN:LED_COLOR_DARK);
      }
   }
   
} /* END USR_leds_handler */

