/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/*********************************************************************************
 *  ------------------------------------------------------------------------------
 *   Module      : vlnq.h
 *   Owner       : 
 *  ------------------------------------------------------------------------------
 *   Application : Wireless LAN
 *   Subsystem   : Infrastructure / BSP
 *  ------------------------------------------------------------------------------
 *   Description :
 *   This header file provides the exported functions of the VLYNQ device driver
 *   for the PUMA side only
 *   Based on Yaron Vaknin's VLYNQ-VDSP code
 *   This file is included from "tnetd53xx.h"
 *  ------------------------------------------------------------------------------
 *   Author    : Joseph Gabbay
 *  ------------------------------------------------------------------------------
 */

#ifndef _VLYNQ_H_
#define _VLYNQ_H_

#include "tnetd73xx.h"
#include "tnetd53xxerr.h"
#include "tnetd53xx_vlynq.h"
                 
/************
 * Flags
 ************/
#ifdef _VLYNQ_C_
UINT32 VLYNQ_LOG_ENABLE = 0;  /* Enable this macro for debugging output */
#else
extern UINT32 VLYNQ_LOG_ENABLE;
#endif

/************
 * enums
 ************/
typedef enum
{
   /* NOTE:
    * Values 0x00 and 0x01 are reserved for VLYNQ local and remote
    * VLYNQ modules. Maps IRQ from remote machine to local machine.
    */
   VLYNQ_MODULE_LOCAL  = 0x00,
   VLYNQ_MODULE_REMOTE = 0x01,
   VLYNQ_REMOTE_02 	= 0x02, /* Remote IRQ that comes through VLYNQ vector 2 */
   VLYNQ_REMOTE_03 	= 0x03, /* Remote IRQ that comes through VLYNQ vector 3 */
   VLYNQ_REMOTE_04 	= 0x04, /* Remote IRQ that comes through VLYNQ vector 4 */
   VLYNQ_REMOTE_05 	= 0x05, /* Remote IRQ that comes through VLYNQ vector 5 */
   VLYNQ_REMOTE_06 	= 0x06, /* Remote IRQ that comes through VLYNQ vector 6 */
   VLYNQ_REMOTE_07 	= 0x07, /* Remote IRQ that comes through VLYNQ vector 7 */
   VLYNQ_REMOTE_NUM_OF_INTRRUPTS = 0x08	/* Do not delete, Future VLYNQ will support 32 IRQs */
} eVLYNQ_REMOTE_INTRRUPT_SRC;

typedef enum
{
   VLYNQ_INTR_ACTIVE_HIGH = 0x00,
   VLYNQ_INTR_ACTIVE_LOW  = 0x01
} eVLYNQ_INTR_POLARITY;			/* Interrupt polarity enumerator*/

typedef enum
{
   VLYNQ_INTR_LEVEL  = 0x00,
   VLYNQ_INTR_PULSED = 0x01
} eVLYNQ_INTR_TYPE;				/* Interrupt type enumerator*/

typedef enum
{
   VLYNQ_LOCAL_DVC  = 0x00,  	/* vlynq local device (puma's vlynq module)  */
   VLYNQ_REMOTE_DVC = 0x01   	/* vlynq remote device (remote vlynq module) */
} eVLYNQ_DVC;

typedef enum
{
	VLYNQ_CLK_SOURCE_NONE   = 0x00,	/* do not initialize clock generator    */
	VLYNQ_CLK_SOURCE_LOCAL  = 0x01,	/* clock is generated by local machine  */
	VLYNQ_CLK_SOURCE_REMOTE	= 0x02   /* clock is generated by remote machine */
} eVLYNQ_CLK_SOURCE;

typedef enum
{
   VLYNQ_BUS_WIDTH_3 =  3,
   VLYNQ_BUS_WIDTH_5 =  5,
   VLYNQ_BUS_WIDTH_7 =  7,
   VLYNQ_BUS_WIDTH_9 =  9
} eVLYNQ_BUS_WIDTH;

typedef enum
{
   VLYNQ_DRV_STATE_UNINIT = 0,  /* driver is uninitialized or unloaded */
   VLYNQ_DRV_STATE_ININIT = 1,  /* VLYNQ is being initialized */
   VLYNQ_DRV_STATE_RUN    = 2,  /* VLYNQ is running properly  */
   VLYNQ_DRV_STATE_HOLD   = 3,  /* driver stopped temporarily as a result of user request  */
   VLYNQ_DRV_STATE_ERROR  = 4,  /* driver stopped on unrecoverable error, e.g. link failed */
   VLYNQ_DRV_STATE_UNLOAD = 5   /* driver is being unloaded from the system. */
} eVLYNQ_DRV_STATE;  /* state of the driver */

typedef enum
{
   VLYNQ_RESET_ASSERT,    /* hold device in reset state */
   VLYNQ_RESET_DEASSERT,  /* release device from reset state */
   VLYNQ_RESET_INITFAIL,  /* handle the device in case driver initialization fails */
   VLYNQ_RESET_LINKESTABLISH,  /* handle the device in case driver established link */
   VLYNQ_RESET_INITFAIL2, /* Driver initialization failed but VLYNQ link exist. */
   VLYNQ_RESET_INITOK,    /* Driver initialization finished OK. */
   VLYNQ_RESET_UNLOAD,    /* Notification before VLYNQ driver is shut down. no interrupts
                             can come from the device. */
   VLYNQ_RESET_ONFINALRELEASE     /* Notification after VLYNQ driver is shut down.
                                     VLYNQ Device is not accessible any more!!
                                     User might want to reestablish connection */
} eVLYNQ_RESET_MODE;

/* options for init (bitmasks) */
#define   VLYNQ_OPTION_INIT_HOST_REGIONS     0x00000001ul
#define   VLYNQ_OPTION_INIT_REMOTE_REGIONS   0x00000002ul
#define   VLYNQ_OPTION_INIT_INTERRUPTS       0x00000004ul
#define   VLYNQ_OPTION_INIT_CHECK_LINK       0x00000008ul
#define   VLYNQ_OPTION_INIT_CONFIG_CLK       0x00000010ul
#define   VLYNQ_OPTION_INIT_CLEAR_ERRORS     0x00000020ul

/*************
 * callbacks
 *************/
struct VLYNQ_DEV_t;  /* forward declaration */
typedef  void (*VLYNQ_INTR_CNTRL_ISR)(struct VLYNQ_DEV_t *pDev);
typedef  void (*VLYNQ_RESET_REMOTE)(struct VLYNQ_DEV_t *pDev, eVLYNQ_RESET_MODE mode);
typedef  void (*VLYNQ_REPORT_CB)
(
   struct VLYNQ_DEV_t *pDev,     /* This VLYNQ */
   eVLYNQ_DVC          aSrcDvc,  /* Who casused the event, local or remote? */
   UINT32        dwStatRegVal    /* Value of the relevant status register */
);

/***********
 * types
 ***********/

typedef struct VLYNQ_MEMORY_MAP_t
{
   UINT32	TxMap;
   UINT32	RxOffset[TNETD53XX_VLYNQ_MAX_MEMORY_REGIONS];
   UINT32   RxSize[TNETD53XX_VLYNQ_MAX_MEMORY_REGIONS];
} VLYNQ_MEMORY_MAP;

typedef struct VLYNQ_INTR_CNTRL_ICB_t
{
   VLYNQ_INTR_CNTRL_ISR	   isr;	/* interrupt service routine */
   void			           *arg;	/* arg to be passed into ISR */
} VLYNQ_INTR_CNTRL_ICB;          /* interrupt control block   */ 

typedef struct VLYNQ_DEV_t
{
   unsigned int      dev_idx;	    /* VLYNQ module index: 1,2,3,4... */
   UINT32            module_base; /* VLYNQ module base address, set to 0, used by the driver */
   eVLYNQ_DRV_STATE  state;       /* State of the VLYNQ driver, set to VLYNQ_DRV_STATE_UNINIT, when initializing */
   eVLYNQ_CLK_SOURCE	clk_source;	 /* clock source selection */
   unsigned int      clk_div;     /* 1 to 8. VLYNQ_clk = VBUSCLK/clk_div */
   eVLYNQ_BUS_WIDTH  width;		 /* Valid VLYNQ bus width, filled by driver  */
   VLYNQ_MEMORY_MAP	local_mem;	 /* local memory mapping   */
   VLYNQ_MEMORY_MAP	remote_mem;	 /* remote memory mapping  */
   VLYNQ_INTR_CNTRL_ICB *pIntrCB; /* pointer to interrupt control blocks, allocated
                                     by the driver. must be initialized to NULL. */
   VLYNQ_REPORT_CB	report_cb;	 /* user callback for VLYNQ events. NULL if not used. */
   VLYNQ_RESET_REMOTE reset_cb;   /* user callback for resetting/releasing the remote device. NULL if not used. */
} VLYNQ_DEV;



/************************
 * function prototypes
 ************************/
#ifdef __cplusplus
extern "C" {
#endif


/* Initialize VLYNQ BUS communication, interrupts, and data structures. */
TNETD53XX_ERR tnetd53xx_vlynq_Init( VLYNQ_DEV* pDev, UINT32 opt);
/* Shut down the VLYNQ interface, free all resources taken by the driver. */
TNETD53XX_ERR tnetd53xx_vlynq_Unload( VLYNQ_DEV* pDev);


TNETD53XX_ERR tnetd53xx_vlynq_IRQ_Enable
(
    VLYNQ_DEV                  *pDev,
    eVLYNQ_REMOTE_INTRRUPT_SRC  nSrc,
    int                         flag	/* 0 = disable, else = enable */
);

void tnetd53xx_vlynq_RootISR( VLYNQ_DEV *pDev);

#ifdef __cplusplus
}
#endif

/***********************
 * Manipulation Macros
 ***********************/
#define VLYNQ_INRANGE(x,hi,lo)   (((x) <= (hi)) && ((x) >= (lo)))
#define VLYNQ_OUTRANGE(x,hi,lo)  (((x) > (hi)) || ((x) < (lo)))
#define VLYNQ_LOBYTE(w)          ((unsigned int)((w) & 0xff))
#define VLYNQ_HIBYTE_16(w)       ((unsigned int)(((w) >> 8) & 0x00ff))
#define VLYNQ_ARRSIZE(a)         (sizeof(a)/sizeof(a[0]))


#endif  /* _VLYNQ_H_ */


