/*****************************************************************************
 * fw2asm.c -- Converts FW.bin file into FW.S assembly file
 *
 * Written by: Joseph Gabbay
 *
 * Usage:
 *
 * fw2asm fname oname startLbl endLbl versionLbl "version description string" [-b]
 *****************************************************************************/

#include	<stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/stat.h>
#include <errno.h>

/* globals */

#ifdef __linux__
#define _stat stat
#define _MAX_PATH 256
#define _fullpath(x, y, z) realpath(x, y)
#endif

char iname[_MAX_PATH];
char oname[_MAX_PATH];
char fname[_MAX_PATH];
char ofname[_MAX_PATH];

static const char hdr1[]=
"/****************************************************************************\n"
" *\n"
" *   MODULE:  %s.S\n"
" *   PURPOSE: Contains Adaptor firmware as an array\n"
" *   This file was auto generated by FW2ASM utility version 1.0.0\n"
" *   which converts %s.bin image file into ASM file.\n"
" *\n"
" ****************************************************************************/\n\n"
;

static const char hdr2[]=
"\n\n"
"/* Created from %s.bin\n"
" * On %s"   /* full time & date info */
" */\n\n"
;

static const char hdr3[]=
"#define _ASMLANGUAGE\n"
"\n"
"#ifndef __linux__\n"
"#include \"vxWorks.h\"\n"
"#include \"asm.h\"\n"
"#endif /* __linux__ */\n\n\n"
;

static const char directive_align[] ="\n\t.align 4\n";
static const char directive_globl[] ="\t.globl %s\n";
static const char directive_text[]  ="\n\t.text\n";
static const char directive_data[]  ="\n\t.data\n";
static const char directive_byte[]  ="\t.byte "; /* string of octal numbers (up to 16) */
static const char directive_asciz[] ="\t.asciz \"%s\"\n";
static const char directive_label[] ="\n%s:\n";



/*****************************************/
static char *GetWhyError(int ret)
{
	printf("ERROR : %s\n", strerror(errno));
   switch( ret)
   {
   case 11:
      return "Cannot open input file";
   case 12:
      return "Cannot open output file";
   case 13:
      return "Error writing hdr1";
   case 14:
      return "Error writing hdr2";
   case 15:
      return "Error writing hdr3";
   case 16:
   case 17:
   case 18:
      return "Error writing .globl directives";
   case 19:
      return "Error writing .text directive";
   case 20:
      return "Error writing .data directive";
   case 21:
   case 24:
      return "Error writing .align directive";
   case 22:
   case 25:
   case 26:
      return "Error writing label";
   case 23:
      return "Error writing description";
   case 27:
      return "Error reading input file";
   case 28:
      return "Error writing .byte";
   default:
      return "Unspecified";
   }
}/* END GetWhyError() */


/*****************************************/
static int buildOutput
   (
   FILE *infile,
   FILE *outfile,
   const char* mdlName,
   const char* infName,
   const char* timeStamp,
   const char* lblStart,
   const char* lblEnd,
   const char* lblVer,
   const char* description,
   int supressHdr
   )
{
   int ret = 0;
   int numWritten, numExpected, numRead;
   char buf[1024];

   if( infile == NULL)
   {
      ret = 11;
   }

   if( (ret == 0) && (outfile == NULL))
   {
      ret = 12;
   }

   /* output headers */
   if( ret == 0)
   {
      numExpected = sprintf( buf, hdr1, mdlName, infName);
      numWritten = fwrite( buf, sizeof(char), numExpected, outfile);
      if( numExpected != numWritten) ret = 13;
   }
   
   if( (ret == 0) && (supressHdr == 0))
   {
      numExpected = strlen( hdr3);
      numWritten = fwrite( hdr3, sizeof(char), numExpected, outfile);
      if( numExpected != numWritten) ret = 15;
   }

   if( ret == 0)
   {
      numExpected = sprintf( buf, hdr2, infName, timeStamp);
      numWritten = fwrite( buf, sizeof(char), numExpected, outfile);
      if( numExpected != numWritten) ret = 14;
   }


   /* globl directives  */
   if( ret == 0)
   {
      numExpected = sprintf( buf, directive_globl, lblVer);
      numWritten = fwrite( buf, sizeof(char), numExpected, outfile);
      if( numExpected != numWritten) ret = 16;
   }

   if( ret == 0)
   {
      numExpected = sprintf( buf, directive_globl, lblStart);
      numWritten = fwrite( buf, sizeof(char), numExpected, outfile);
      if( numExpected != numWritten) ret = 17;
   }

   if( ret == 0)
   {
      numExpected = sprintf( buf, directive_globl, lblEnd);
      numWritten = fwrite( buf, sizeof(char), numExpected, outfile);
      if( numExpected != numWritten) ret = 18;
   }

   /* text and data */
   if( ret == 0)
   {
      numExpected = strlen( directive_text);
      numWritten = fwrite( directive_text, sizeof(char), numExpected, outfile);
      if( numExpected != numWritten) ret = 19;
   }

   if( ret == 0)
   {
      numExpected = strlen( directive_data);
      numWritten = fwrite( directive_data, sizeof(char), numExpected, outfile);
      if( numExpected != numWritten) ret = 20;
   }

   /* .align directive */
   if( ret == 0)
   {
      numExpected = strlen( directive_align);
      numWritten = fwrite( directive_align, sizeof(char), numExpected, outfile);
      if( numExpected != numWritten) ret = 21;
   }

   /* version label */
   if( ret == 0)
   {
      numExpected = sprintf( buf, directive_label, lblVer);
      numWritten = fwrite( buf, sizeof(char), numExpected, outfile);
      if( numExpected != numWritten) ret = 22;
   }

   /* .asciz directive with description */
   if( ret == 0)
   {
      numExpected = sprintf( buf, directive_asciz, description);
      numWritten = fwrite( buf, sizeof(char), numExpected, outfile);
      if( numExpected != numWritten) ret = 23;
   }

   /* .align directive */
   if( ret == 0)
   {
      numExpected = strlen( directive_align);
      numWritten = fwrite( directive_align, sizeof(char), numExpected, outfile);
      if( numExpected != numWritten) ret = 24;
   }

   /* start label */
   if( ret == 0)
   {
      numExpected = sprintf( buf, directive_label, lblStart);
      numWritten = fwrite( buf, sizeof(char), numExpected, outfile);
      if( numExpected != numWritten) ret = 25;
   }

   /* And now for the main loop */
   while( !feof( infile ) )
   {
      unsigned char bts[16];
   
      /* Attempt to read infile (bin file) */
      numRead = fread( bts, sizeof( unsigned char ), 16, infile );
      if( ferror( infile ) )      {
         perror( "Read error" );
         ret = 27;
         break;
      }

      /* write .byte directive in the output file (*.S) */
      if( ret == 0)
      {
         numExpected = strlen( directive_byte);
         numWritten = fwrite( directive_byte, sizeof(char), numExpected, outfile);
         if( numExpected != numWritten) ret = 28;
      }

      /* write the line in the output file (*.S) */
      if( ret == 0)
      {
         char bb[8];
         int i;
         
         buf[0] = '\0';
         for( i=0; i< numRead; i++)
         {
            sprintf( bb, "%04o", (unsigned)(bts[i]));
            strcat(buf, bb);
            if( i < numRead-1)
            {
               strcat(buf,",");
            }
         }
         strcat(buf,"\n");

         numExpected = strlen( buf);
         numWritten = fwrite( buf, sizeof(char), numExpected, outfile);
         if( numExpected != numWritten) ret = 28;
      }

   }/* end While */
   
   /* end label */
   if( ret == 0)
   {
      numExpected = sprintf( buf, directive_label, lblEnd);
      numWritten = fwrite( buf, sizeof(char), numExpected, outfile);
      if( numExpected != numWritten) ret = 26;
   }

   /* analyze results */
   if( ret != 0)
   {
      printf( "Error %u: %s\n", ret, GetWhyError(ret));
   }
   else
   {
      puts("Successful");
   }

   return ret;
}

/*****************************************/

int main( int argc, char* argv[])
{
   int ret = 0;
   FILE *infile, *outfile;
   time_t ltime;
   int supressHeader = 0;
   struct _stat stt;
    
   /* check arguments */
#ifndef __linux__
   _splitpath( argv[0], NULL /*drv*/, NULL /*dir*/, fname, NULL/*ext*/);
#else
	 strcpy(fname, argv[0]);
#endif
   printf("%s utility. Converts FW.bin file into FW.S assembly file, version 1.0.0\n"
      "Written by: Joseph Gabbay, Texas Instruments 03-Sep-2003 (c).\n\n",
      fname);

   if( argc < 7)
   {
      printf("Usage:\n   "
         "%s <fname> <oname> <startLbl> <endLbl> <verLbl> <\"ver desc\"> [-b]\n\n",
         fname);
      puts("<fname>\t\tFW input file name, bin extention is added automatically");
      puts("<oname>\t\tASM output File name, S extention is added automatically");
      puts("<startLbl>\tLabel of array start");
      puts("<endLbl>\tLabel of array end");
      puts("<verLbl>\tLabel of array version information string");
      puts("<\"ver desc\">\tQuoted string for version information");
      puts("[-b]\t\tOptional parameter, indicates bare format (no headers).");
      return 1;
   }

   if( argc > 7)
   {
      if( !strcmp(argv[7], "-b"))
         supressHeader = 1;
   }

   /* full path to files */
   sprintf(fname, "%s.bin", argv[1]);
#if 0
   if( _fullpath( iname, fname, _MAX_PATH ) == NULL)
   {
      printf("Bad input file name %s\n", fname);
      return 2;
   }
#endif

	 strcpy(iname, fname);
	 sprintf(ofname, "%s.S", argv[2]);

	 strcpy(oname, ofname);
#if 0 
   if( _fullpath( oname, ofname, _MAX_PATH ) == NULL)
   {
      printf("Bad output file name %s\n", ofname);
      return 3;
   }
#endif
   /* show what you are about to do */
   time( &ltime );
   printf( "Time and date   : %s", ctime( &ltime ) );
   printf( "Input  file path: %s\n", iname);
   printf( "Output file path: %s\n", oname);
   printf( "Start Label: %s\n", argv[3]);
   printf( "End   Label: %s\n", argv[4]);
   printf( "Ver   Label: %s\n", argv[5]);
   printf( "Description: %s\n", argv[6]);
   if( supressHeader ==1 ) puts("Supress header");

   /* open the files */
   ret = _stat( iname, &stt);
   if( ret == 0)
   {
      infile  = fopen( iname, "rb");   /* binary mode */
      outfile = fopen( oname, "w+");   /* text mode */

      ret = buildOutput(
         infile,
         outfile,
         argv[2],
         argv[1],
         ctime( &ltime),
         argv[3],
         argv[4],
         argv[5],
         argv[6],
         supressHeader
         );

      /* cleanup */
      if( infile != NULL)
         fclose(infile);

      if( outfile != NULL)
         fclose(outfile);
   }
   else
   {
			printf("Cannot open input file - %s\n", strerror(errno));
   }

   return ret;
}/* END main() */




