/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2002 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalAcxCmdBit.c
 *   PURPOSE: ACX100 BIT commands handler
 *		The object support the followinf ACX100 commands:
 *			
 *		whal_acxCmdBit_Fcc	
 *		whal_acxCmdBit_Telec	
 *    whal_acxCmdBit_perTxCfg
 *    whal_acxCmdBit_perRxCfg
 *    whal_acxCmdBit_perClrStats
 *    whal_acxCmdBit_perTxStop
 *    whal_acxCmdBit_perRxStats
 *    whal_acxCmdBit_perTxStats
 *		whal_acxCmdBit_Exit	
 * 
 ****************************************************************************/

#include "pal.h"
#include "whalCommon.h"
#include "whalAcxMbox.h"
#include "whalFrameBuilder.h"
#include "whalAcxCmdBit.h"
#include "whalTrace.h"

/*******************
 Helper functions
 *******************/
#if 1
static void errPrint_ChID(void)
{
   printf("Channel ID %d out of range ,max channel %d\n", MIN_CHANNEL_ID, MAX_CHANNEL_ID);
}
#endif

static void errPrint_rate(void)
{
   puts("Error in rate / mode combination.\n"
        "Rate = [1,2,5,11,22], mode = [0-QPSK, 1-CCK, 2-PBCC].\n"
        "Rates 1, 2 support QPSK only.\n"
        "Rates 5, 11 support CCK and PBCC.\n"
        "Rate 22 supports only PBCC.");
}

static void errPrint_preamble(void)
{
   puts("Error. Use valid preamble values: 0=long, 1=short");
}

static void errPrint_MAC(void)
{
   puts("Error. Use valid MAC address, format xx-xx-xx-xx-xx-xx");
}

static void errPrint_BodyLen(void)
{
   puts("Error. Use valid frame body len: [0..2304] integral multiple or 4.");
}

/* Rate definitions */
static int whal_acx_compute_rate_octet(int rate, int mode /*0-QPSK, 1=CCK , 2-PBCC*/)
{
   int val = -1;

   if( OUTRANGE(mode, TEST_MOD_QPSK, TEST_MOD_PBCC))
      return val;

   switch( rate)
   {
   case 1:
      if( mode == TEST_MOD_QPSK)       val = RATE_1;
      break;
   case 2:
      if( mode == TEST_MOD_QPSK)       val = RATE_2;
      break;
   case 5:
      if      ( mode == TEST_MOD_CCK ) val = RATE_5_5;
      else if ( mode == TEST_MOD_PBCC) val = RATE_5_5_PBCC;
      break;
   case 11:
      if      ( mode == TEST_MOD_CCK ) val = RATE_11;
      else if ( mode == TEST_MOD_PBCC) val = RATE_11_PBCC;
      break;
   case 22:
      if( mode == TEST_MOD_PBCC)       val = RATE_22_PBCC;
      break;
   default:
      break;
   }

   return val; /* <0 means error, take the lower 8 bits */
}/* END whal_acx_compute_rate_octet() */


static int whal_acx111_compute_rate_octet(int rate, int mode)
{
   int val = -1;

   if( OUTRANGE(mode, TEST_MOD_QPSK, TEST_MOD_OFDM))
      return val;

   switch( rate)
   {
   case 1:
      if( mode == TEST_MOD_QPSK)       val = RATE_1;
      break;
   case 2:
      if( mode == TEST_MOD_QPSK)       val = RATE_2;
      break;
   case 5:
      if(( mode == TEST_MOD_CCK ) || ( mode == TEST_MOD_PBCC )) val = RATE_5_5;
      break;
   case 11:
      if(( mode == TEST_MOD_CCK ) || ( mode == TEST_MOD_PBCC )) val = RATE_11;
      break;
   case 22:
      if( mode == TEST_MOD_PBCC)       val = RATE_22_PBCC;
      break;
   case 6:
      if( mode == TEST_MOD_OFDM)       val = RATE_6MBPS;
      break;
   case 9:
      if( mode == TEST_MOD_OFDM)       val = RATE_9MBPS;
      break;
   case 12:
      if( mode == TEST_MOD_OFDM)       val = RATE_12MBPS;
      break;
   case 18:
      if( mode == TEST_MOD_OFDM)       val = RATE_18MBPS;
      break;
   case 24:
      if( mode == TEST_MOD_OFDM)       val = RATE_24MBPS;
      break;
   case 36:
      if( mode == TEST_MOD_OFDM)       val = RATE_36MBPS;
      break;
   case 48:
      if( mode == TEST_MOD_OFDM)       val = RATE_48MBPS;
      break;
   case 54:
      if( mode == TEST_MOD_OFDM)       val = RATE_54MBPS;
      break;
   default:
      break;
   }

   return val; /* <0 means error, take the lower 8 bits */
}/* END whal_acx111_compute_rate_octet() */


/* MACROS */
#if 1
#define VALIDATE_CHID(chID) \
   if( !IS_CHID_OK(chID))  \
   {                       \
      errPrint_ChID();     \
      return NOK;          \
   }
#endif   
 

/****************************************************************************
 * 0x01                      whal_acxCmdBit_Fcc()
 ****************************************************************************
 * DESCRIPTION:	Performs FCC test
 * 
 * INPUTS: chID, rate, mode
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/


int whal_acxCmdBit_Fcc(int band, int chID, int rate, int mode)
{
	AcxCmd_Test_T   AcxCmd_TestFCC;
	AcxCmd_Test_T  *pCmd = &AcxCmd_TestFCC;
	int rateV, modPreamble;

	/*VALIDATE_CHID(chID);*/

	/* param2 = data rate */
	rateV = whal_acx111_compute_rate_octet( rate, mode);
	if( rateV < 0)
	{
		errPrint_rate();
		return NOK;
	}

	/* param3 = modPreamble */
	if (mode == TEST_MOD_PBCC)
	{
		modPreamble = PBCC_RATE_BIT;
	}
	else if(mode == TEST_MOD_OFDM)
	{
		modPreamble = OFDM_RATE_BIT;
	}
	else
		modPreamble = CCK_LONG;


	if (band == RADIO_BAND_5GHZ)	/* Per FW change, the Preamble contains BIT_4 set if 5GHz is the chosen band */
		modPreamble |= BIT_4;

	/* Set command fields */
	PAL_memSet(pCmd, 0, sizeof(*pCmd));
	pCmd->testID = TESTID_FCC;
	pCmd->param1 = (UINT8)chID;
	pCmd->param2 = (UINT8)(rateV)/* & 0xff)*/;
	pCmd->param3 = (UINT8)(modPreamble);
	
	/* Send the command */
	return (whal_acxMboxCommand(CMD_TEST, (char *)pCmd, sizeof(*pCmd), NO_WAIT));
}/* END whal_acxCmdBit_Fcc() */

/****************************************************************************
 * 0x02                     whal_acxCmdBit_Telec()
 ****************************************************************************
 * DESCRIPTION:	Performs TELEC test
 * 
 * INPUTS: chID
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxCmdBit_Telec(int band, int chID)
{
	AcxCmd_Test_T   AcxCmd_TestTelec;
	AcxCmd_Test_T  *pCmd = &AcxCmd_TestTelec;

	/* Set command fields; param1= chID, param2 are ignored */
	PAL_memSet(pCmd,  0, sizeof(*pCmd ));
	pCmd->testID = TESTID_TELEC;
	pCmd->param1 = (UINT8)chID;
	pCmd->param2 = (UINT8)band;
	
	/* Send the command */
	return (whal_acxMboxCommand(CMD_TEST, (char *)pCmd, sizeof(*pCmd), NO_WAIT));
}/* END whal_acxCmdBit_Telec() */

/********************
   PER TEST
 ********************/

int whal_acxCmdBit_perTxCfg
(
  int   band,
  int   chID,
  int   rate,
  int   mode,
  int   preamble,     /* 0=long 1=short*/
  int   numFrames,
  int   gap,          /* should be >= 200usec */
  int   bodySize,     /* [0..2304] multiples of 4 */
  char *destMac       /* destination MAC address (6 octets) */
)
{
	AcxCmd_Test_PER_TX_CFG_T   AcxCmd_perTxCfg;
	AcxCmd_Test_PER_TX_CFG_T  *pCmd = &AcxCmd_perTxCfg;
   int rateV, modPreamble;

   rateV = whal_acx111_compute_rate_octet( rate, mode);
   if( rateV < 0)
   {
      errPrint_rate();
      return NOK;
   }

   if( !TEST_MOD_IS_PREAMBLE_OK( preamble))
   {
      errPrint_preamble();
      return NOK;
   }

   if (mode == TEST_MOD_PBCC)
   {
      modPreamble = PBCC_RATE_BIT | preamble;
   }
   else if (mode == TEST_MOD_OFDM)
   {
      modPreamble = OFDM_RATE_BIT | preamble;
   }
   else
   {
      modPreamble = preamble;
   }

   if (band == RADIO_BAND_5GHZ)	/* Per FW change, the Preamble contains BIT_4 set if 5GHz is the chosen band */
	   modPreamble |= BIT_4;

   if( destMac == NULL)
   {
      errPrint_MAC();
      return NOK;
   }

   if( !TEST_MOD_IS_GAP_OK(gap))
   {
      gap = TEST_MOD_MIN_GAP;
      printf("Assuming minimum gap = %uusec\n", gap);
   }

   if( !TEST_MOD_IS_TX_BODYLEN_OK(bodySize))
   {
      errPrint_BodyLen();
      return NOK;
   }

   /* Fill command structure */
   PAL_memSet(pCmd, 0, sizeof(*pCmd));
   pCmd->testID      = (UINT8)TESTID_PER_TX_CFG;
   pCmd->Channel     = (UINT8)chID;
   pCmd->DataRate    = (UINT8)rateV;
   pCmd->ModPreamble = (UINT8)modPreamble;
   pCmd->NumFrames   = ENDIAN_HANDLE_LONG(numFrames);
   pCmd->InterFrameGap  = ENDIAN_HANDLE_LONG(gap);
   pCmd->FrameBodySize  = ENDIAN_HANDLE_WORD(bodySize);
   PAL_memCopy( &(pCmd->Dest[0]), (const void*)destMac, MAC_ADDR_LEN);
   pCmd->TestModeCtrl= 3; /* Auto sequence, FW provides frame body */
   
	/* Send the command */
	return (whal_acxMboxCommand(CMD_TEST, (char *)pCmd, sizeof(*pCmd), INIT_TIMEOUT));
}/* END whal_acxCmdBit_perTxCfg() */

/****************************************************************************
 *  0x04                    whal_acxCmdBit_perRxCfg()
 ****************************************************************************
 * DESCRIPTION:	Packet Error Rate - Rcvr configuration, param1= chID
 * 
 * INPUTS: chID	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxCmdBit_perRxCfg(int band, int chID)
{
	AcxCmd_Test_T   AcxCmd_perRxCfg;
	AcxCmd_Test_T  *pCmd = &AcxCmd_perRxCfg;

	/* Set command fields; param1= chID, param2 are ignored */
	PAL_memSet(pCmd,  0, sizeof(*pCmd ));
	pCmd->testID = TESTID_PER_RX_CFG;
	pCmd->param1 = (UINT8)chID;
	pCmd->param2 = (UINT8)band;
	
	/* Send the command */
	return (whal_acxMboxCommand(CMD_TEST, (char *)pCmd, sizeof(*pCmd), INIT_TIMEOUT));
}/* END whal_acxCmdBit_perRxCfg() */

/****************************************************************************
 *  0x05                    whal_acxCmdBit_perClrStats()
 ****************************************************************************
 * DESCRIPTION:   Packet Error Rate - clear all statistics, no params.
 * 
 * INPUTS: None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxCmdBit_perClrStats(void)
{
	AcxCmd_Test_T   AcxCmd_perRxCfg;
	AcxCmd_Test_T  *pCmd = &AcxCmd_perRxCfg;

	/* Set command fields; param1= chID, param2 are ignored */
	PAL_memSet(pCmd,  0, sizeof(*pCmd ));
	pCmd->testID = TESTID_PER_CLR_STATS;
	
	/* Send the command */
	return (whal_acxMboxCommand(CMD_TEST, (char *)pCmd, sizeof(*pCmd), NO_WAIT));
}/* END whal_acxCmdBit_perClrStats() */

/****************************************************************************
 *  0x06                    whal_acxCmdBit_perTxStop()
 ****************************************************************************
 * DESCRIPTION:   Packet Error Rate - stop test, no params.
 *                Does not clear statistics.
 * INPUTS: None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxCmdBit_perTxStop(void)
{
	AcxCmd_Test_T   AcxCmd_perRxCfg;
	AcxCmd_Test_T  *pCmd = &AcxCmd_perRxCfg;

	/* Set command fields; param1= chID, param2 are ignored */
	PAL_memSet(pCmd,  0, sizeof(*pCmd ));
	pCmd->testID = TESTID_PER_TX_STOP;
	
	/* Send the command */
	return (whal_acxMboxCommand(CMD_TEST, (char *)pCmd, sizeof(*pCmd), NO_WAIT));
}/* END whal_acxCmdBit_perTxStop() */

/****************************************************************************
 *  0x07                    whal_acxCmdBit_perRxStats()
 ****************************************************************************
 * DESCRIPTION:   PER - read RX statistics into structure and print it.
 * 
 * INPUTS: None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxCmdBit_perRxStats(void  *ParamsBuf,
							  whal_userMboxCB userCB, 
							  UINT8 *replyBuf)
{
   AcxCmd_Test_PER_RX_STATS  *pCmd = (AcxCmd_Test_PER_RX_STATS  *)ParamsBuf;
   /* Set command fields; param1= chID, param2 are ignored */
	PAL_memSet(pCmd,  0, sizeof(*pCmd ));
	pCmd->testID = TESTID_PER_RX_STATS;

   return whal_acxMboxInterrogateNoWait	(CMD_TEST,
										 pCmd,
										 sizeof(*pCmd),
										 userCB,
										 replyBuf
										);
}/* END whal_acxCmdBit_perRxStats() */

/****************************************************************************
 *  0x08                    whal_acxCmdBit_perTxStats()
 ****************************************************************************
 * DESCRIPTION:	PER - read TX statistics into structure and print it.
 * 
 * INPUTS: None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxCmdBit_perTxStats(
							  void  *ParamsBuf,
							  whal_userMboxCB userCB, 
							  UINT8 *replyBuf
							  )
{
    AcxCmd_Test_PER_TX_STATS  *pCmd = ( AcxCmd_Test_PER_TX_STATS  *)ParamsBuf;

   /* Set command fields; param1= chID, param2 are ignored */
	PAL_memSet(pCmd,  0, sizeof(*pCmd ));
	PAL_memSet(pCmd,  0, sizeof(*pCmd ));
	pCmd->testID = TESTID_PER_TX_STATS;

    return whal_acxMboxInterrogateNoWait(CMD_TEST,
										 pCmd,
										 sizeof(*pCmd),
										 userCB,
										 replyBuf
										 );
   
}


/****************************************************************************
 *  0xff                    whal_acxCmdBit_Exit()
 ****************************************************************************
 * DESCRIPTION:	Exit test mode
 * 
 * INPUTS: None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxCmdBit_Exit(void)
{
	AcxCmd_Test_T   AcxCmd_TestExit;
	AcxCmd_Test_T  *pCmd = &AcxCmd_TestExit;

	/* Set command fields */
	PAL_memSet(pCmd, 0, sizeof(*pCmd));
	pCmd->testID = TESTID_EXIT ;
	
	/* Send the command */
	return (whal_acxMboxCommand(CMD_TEST, (char *)pCmd, sizeof(*pCmd), INIT_TIMEOUT));
}/* END whal_acxCmdBit_Exit() */

