/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 2002 Texas Instruments Incorporated                |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
/**************************************************************************/
/*                                                                        */
/*   MODULE:   whalAcxEeprom.h                                            */
/*   PURPOSE:  Defining APIs for serial EEPROM attached to the ACX100/101 */ 
/*                                                                        */
/*                                                                        */
/**************************************************************************/
#ifndef _WHALACXEEPROM_H_
#define _WHALACXEEPROM_H_

/**************************
 Offsets of ACX100 data 
 **************************/
#define ACX_EEPROM_AGC_TABLE_ADDR   0x18C
#define ACX_EEPROM_DAC_ADDR         0x20D
#define ACX_EEPROM_BIAS_ADDR        0x20E    
#define ACX_EEPROM_OSC_ADDR         0x215
#define ACX_EEPROM_CARD_TYPE_ADDR	0x004	/* 0x0D - MAXIM2820, 0x11 - RFMD */
#define MAX_AGC_TABLE_ENTRIES       128

/*******************
 EEPROM access APIs
 *******************/
#define MAX_OSC_CAL         15
#define EEPROM_ACCESS_TO    10000   /* timeout counter */
#define INVALID_PARAMETER1      -101

/*******************
 Radio constants
 *******************/
#define RADIO_MAXIM2820_ID    (0x0D)
#define RADIO_RFMD_ID         (0x11)
#define DEFAULT_RADIO_TYPE	   RADIO_MAXIM2820_ID



/************************************************************************
*        UINT32 whal_acxEepromReadByte(UINT16 wAddr,UINT8 *pbVal)       *
*************************************************************************
* DESCRIPTION:  Reads a single byte from EEPROM.                        *
*                                                                       *
* INPUT:        wAddr - 16-bits EEPROM addrress                         *
*               pbVal - pointer the to output parameter - EEPROM value  *     
*                                                                       *
* OUTPUT:       *pbVal contains EEPROM value                            *   
*                                                                       *
* RETURN:       OK - successful                                         *
*               NOK  timeout                                            *
*************************************************************************/
UINT32   whal_acxEepromReadByte(UINT16 wAddr,UINT8 *pbVal);

/************************************************************************
*       UINT32 whal_acxEepromWriteByte(UINT16 wAddr,UINT8 bVal)         *
*************************************************************************
* DESCRIPTION:  Writes a single byte to EEPROM                          *  
*                                                                       *
* INPUT:        wAddr - 16-bits EEPROM addrress                         *
*               bVal  - new value                                       *
*                                                                       *
* OUTPUT:       N/A                                                     *
*                                                                       *
* RETURN:       OK - successful                                         *
*               NOK  timeout                                            *
*************************************************************************/
UINT32	whal_acxEepromWriteByte(UINT16 wAddr,UINT8 bVal);

/************************************************************************
*     UINT32 whal_acxEepromWriteByteNoUnp(UINT16 wAddr,UINT8 bVal)      *
*************************************************************************
* DESCRIPTION:  Writes a single byte to EEPROM. The caller must         *
*               unprotect the EEPROM first. This is done for fast       *
*               programming of several data bytes.                      *
*                                                                       *
* INPUT:        wAddr - 16-bits EEPROM addrress                         *
*               bVal  - new value                                       *
*                                                                       *
* OUTPUT:       N/A                                                     *
*                                                                       *
* RETURN:       OK - successful                                         *
*               NOK  timeout                                            *
*************************************************************************/
UINT32	whal_acxEepromWriteByteNoUnp(UINT16 wAddr,UINT8 bVal);

/************************************************************************
*           void whal_acxEepromProtect(void)                            *
*************************************************************************
* DESCRIPTION:  Set EEPROM write protection.                            *
*               Inhibits writing to the EEPROM.                         *
*                                                                       *
* INPUT:        N/A                                                     *
*                                                                       *
* OUTPUT:       N/A                                                     *
*                                                                       *
* RETURN:       N/A                                                     *
*************************************************************************/
void	   whal_acxEepromProtect(void);

/************************************************************************
*           void whal_acxEepromUnprotect(void)                          *
*************************************************************************
* DESCRIPTION:  Remove EEPROM write protection.                         *
*               Enables writing to the EEPROM.                          *
*                                                                       *
* INPUT:        N/A                                                     *
*                                                                       *
* OUTPUT:       N/A                                                     *
*                                                                       *
* RETURN:       N/A                                                     *
*************************************************************************/
void	   whal_acxEepromUnprotect(void);

/*************************
 EEPROM data manipulation
 *************************/

/************************************************************************
*           UINT32 whal_acxEepromGetCalValue(UINT8 *pbVal)              *
*************************************************************************
* DESCRIPTION:  Reads oscillator cal. value from EEPROM                 *
*                                                                       *
* INPUT:        N/A                                                     *
*                                                                       *
* OUTPUT:       pbVal - pointer to the output parameter                 *
*                                                                       *
* RETURN:       OK   successful                                         *
*               NOK  timeout                                            *
*************************************************************************/
UINT32	whal_acxEepromGetCalValue(UINT8 *pbVal);

/************************************************************************
*           UINT32 whal_acxEepromSetCalValue(UINT8 bVal)                *
*************************************************************************
* DESCRIPTION:  Writes new oscillator cal. value to EEPROM              *
*                                                                       *
* INPUT:        bVal - new oscillator cal. value                        *
*                                                                       *
* OUTPUT:       N/A                                                     *
*                                                                       *
* RETURN:       OK   successful                                         *
*               NOK  timeout or invalid value                           *
*************************************************************************/
UINT32	whal_acxEepromSetCalValue(UINT8 bVal);

/************************************************************************
*           UINT32 whal_acxEepromGetBiasValue(UINT8 *pbVal)             *
*************************************************************************
* DESCRIPTION:  Reads bias value from EEPROM                            *
*                                                                       *
* INPUT:        N/A                                                     *
*                                                                       *
* OUTPUT:       pbVal - pointer to the output parameter                 *
*                                                                       *
* RETURN:       OK   successful                                         *
*               NOK  timeout                                            *
*************************************************************************/
UINT32	whal_acxEepromGetBiasValue(UINT8 *pbVal);

/************************************************************************
*           UINT32 whal_acxEepromSetBiasValue(UINT8 bVal)               *
*************************************************************************
* DESCRIPTION:  Writes new bias value to EEPROM                         *
*                                                                       *
* INPUT:        bVal - new bias value                                   *
*                                                                       *
* OUTPUT:       N/A                                                     *
*                                                                       *
* RETURN:       OK   successful                                         *
*               NOK  timeout                                            *
*************************************************************************/
UINT32	whal_acxEepromSetBiasValue(UINT8 bVal);

/************************************************************************
*           UINT32 whal_acxEepromGetDACValue(UINT8 *pbVal)              *
*************************************************************************
* DESCRIPTION:  Reads DAC value from EEPROM                             *
*                                                                       *
* INPUT:        N/A                                                     *
*                                                                       *
* OUTPUT:       pbVal - pointer to the output parameter                 *
*                                                                       *
* RETURN:       OK   successful                                         *
*               NOK  timeout                                            *
*************************************************************************/
UINT32	whal_acxEepromGetDACValue(UINT8 *pbVal);

/************************************************************************
*           UINT32 whal_acxEepromSetDACValue(UINT8 bVal)                *
*************************************************************************
* DESCRIPTION:  Writes new DAC value to EEPROM                          *
*                                                                       *
* INPUT:        bVal - new DAC value                                    *
*                                                                       *
* OUTPUT:       N/A                                                     *
*                                                                       *
* RETURN:       OK   successful                                         *
*               NOK  timeout                                            *
*************************************************************************/
UINT32	whal_acxEepromSetDACValue(UINT8 bVal);

/************************************************************************
*           int whal_acxEepromLoadBaseBandTable(void)                   *
*************************************************************************
* DESCRIPTION:  Loads BB registers table from EEPROM                    *
*                                                                       *
* INPUT:        N/A                                                     *
*                                                                       *
* OUTPUT:       N/A                                                     *
*                                                                       *
* RETURN:       N/A                                                     *
*************************************************************************/
int		whal_acxEepromLoadBaseBandTable(void);

/************************************************************************
*   UINT32 whal_acxEepromGetAGCCell(UINT8 bTableOffset, UINT8 *pbVal)   *
*************************************************************************
* DESCRIPTION:  Reads one cell from ACG table                           *
*                                                                       *
* INPUT:        bTableOffset - zero-based offset of the cell in AGC     *   
*                              table                                    *
* OUTPUT:       pbVal - pointer to the output parameter                 *       
*                                                                       *
* RETURN:       OK                                                      *
*               NOK - EEPROM write error                                *
*               INVALID_PARAMETER1 - invalid parameter 1                *  
*************************************************************************/
UINT32	whal_acxEepromGetAGCCell(UINT8 bTableOffset, UINT8 *pbVal);

/************************************************************************
*   UINT32 whal_acxEepromSetAGCCell(UINT8 bTableOffset, UINT8 bVal)     *
*************************************************************************
* DESCRIPTION:  Writes one AGC table cell to EEPROM                     *
*                                                                       *
* INPUT:        bTableOffset - zero-based offset of the cell in AGC     *   
*                              table                                    *
*               bVal         - new cell value                           *
* OUTPUT:       N/A                                                     *
*                                                                       *
* RETURN:       OK                                                      *
*               NOK - EEPROM write error                                *
*               INVALID_PARAMETER1 - invalid parameter 1                *  
*************************************************************************/
UINT32	whal_acxEepromSetAGCCell(UINT8 bTableOffset, UINT8 bVal);

/************************************************************************
*           UINT32 whal_acxEepromGetRadioType(UINT8 *pbVal)             *
*************************************************************************
* DESCRIPTION:  Reads radio type value from EEPROM                      *
*                                                                       *
* INPUT:        N/A                                                     *
*                                                                       *
* OUTPUT:       pbVal - pointer to the output parameter                 *
*                                                                       *
* RETURN:       OK   successful                                         *
*               NOK  timeout                                            *
*************************************************************************/
UINT32	whal_acxEepromGetRadioType(UINT8 *pbVal);

/************************************************************************
*           UINT32 whal_acxEepromSetRadioType(UINT8 bVal)               *
*************************************************************************
* DESCRIPTION:  Writes new radio type to EEPROM                         *
*                                                                       *
* INPUT:        bVal - new radio type value                             *
*                                                                       *
* OUTPUT:       N/A                                                     *
*                                                                       *
* RETURN:       OK   successful                                         *
*               NOK  timeout                                            *
*************************************************************************/
UINT32	whal_acxEepromSetRadioType(UINT8 bVal);


#endif	/* _WHALACXEEPROM_H_ */



