/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalApi.c
 *   PURPOSE: whal module API (Init, Receive MPDU, send MSDU, ...)
 *
 ****************************************************************************/

#include "whalCommon.h"
#include "whal.h"
#include "whalAcxRegs.h"
#include "whalAcxProcess.h"
#include "whalAcxAccess.h"
#include "whalAcxMbox.h"
#include "whalAcxCmd.h"
#include "whalAcxCmdBit.h"
#include "whalInterrupt.h"
#include "whalAcxConfig.h"
#include "whalParams.h"
#include "memMngr.h"
#include "whalFrameBuilder.h"
#include "whalRx.h"
#include "whalTxQueue.h"
#include "whalRxQueue.h"
#include "whalTxBufs.h"
#include "whalRxBufs.h"
#include "whalTx.h"
#include "whalTrace.h"
#include "whalAcxEeprom.h"
#include "whalRxMaster.h"
#include "dda.h"
#include "drvDbMain.h"
#include "drvDbHash.h"
#include "drvDbRegDomain.h"
#include "WoneBalance.h"
#include "dda_contexts.h"
#include "AcxInfoElements.h"
#include "mainsm.h"


/* 16 lsb indicate tx complete threshold, 16 msb indicate tx complete timeout */
#define DEFAULT_TX_COMPLETE_TRSH 16
#define DEFAULT_TX_COMPLETE_TO   500<<16

#define WHAL_PRINT_RES_TIMEOUT   5000 /* msec */

int ProbeRespCounter = 0;
int Bss_Started = 0;
UINT32 AcxRegAddr;
UINT32 AcxMemAddr;

PAL_spinLockHandle whalLock;

UINT32 FcsMacRxResetThreshold = 10;

static watchdogFailCB_T watchdogFailCB = NULL;

void whal_apiRestartWatchDog_PacketReceived(void);
extern int whal_txGetBufsNum (TxQueueId_T TxQid, UINT32 *FreeHostDescNum, UINT32 *CurrUsedAcxDescNum);
extern int whal_txQueue_GetDescNum(TxQueueId_T Qid);
extern int whal_txCleanAll(void);


/* Interference WA definitions */

#define RECOVERY	 0
#define NO_RECOVERY	 1

enum
{
	INTERF_NORMAL_STATE,
	INTERF_ACI_STATE,
	INTERF_NB_STATE
}InterferenceFSMstate_E;

typedef  struct 
{
	BOOL 	aciTrigger;
	BOOL 	beaconStuck;
	BOOL 	txStuck;
	BOOL    mediumBusy;
	BOOL    nbStateEnabled;
}InterfStateInput_T;

typedef struct
{	
	UINT8   rxFreeMem;
	UINT8   bcnNotSent;
	UINT8   txStuck;
	UINT16  consFcsErr;
	UINT8   aci;
	UINT8   nb;
} WAoptions_T;

/*Interference WA variables */

static WAoptions_T waOptions;

/* Interference WD parameters*/
static struct 
{
	UINT32 aciCount;
	UINT32 mediumUsage;
	UINT32 usageIntvl;
}interfWDparams;

/* Interference state machine state input*/
static InterfStateInput_T stateInput;
/* used for beacon intervals count, minimum value = 1 */
static UINT wd_beacon_limiter = 1;  
/* interfernce state machine states strings*/
static char interfStateString[3][15] = {"NORMAL", "ACI", "NB"};
/* Number of TX ACX free falures that indicates TX stuck  */
static UINT32 txStuckThreshold = 3;
/* Indicates if ACX counters contain valid values */
static BOOL acxCountersValid = FALSE;
/* temporary storage for FW current parameters to be restored*/
static UINT32 saveCurRXmode = 0;
static UINT8 savedConfigERPmode;
static BOOL  savedCurERPmode;


/* TODO remove these variables -used for Unit Test */
BOOL forceACI; 
BOOL forceNB;
BOOL forceRecovery;


/*************** local func ************/
static int  whal_apiAcxStatCB(BOOL Sts, UINT8 *buf);
static int  whal_RxStatStatCB(BOOL Sts,char *buf);
static int  whal_TxStatStatCB(BOOL Sts,char *buf);
static int 	whal_apiAcxBIT_FillRxStatsBuff(char *buf);
static int 	whal_apiAcxBIT_FillTxStatsBuff(char *buf);
static int 	whal_apiFcsErrCB(BOOL Sts, UINT8 *buf);
static int  whal_apiACIcountCB(BOOL Sts, UINT8 *buf);
static int  whal_apiMediumUsageCB(BOOL Sts, UINT8 *buf);
static int  whal_apiEnergyDetectCB(BOOL Sts, UINT8 *buf);
static int  whal_apiPacketDetectCB(BOOL Sts, UINT8 *buf);
static int  whal_apiRxDot11ModeCB(BOOL Sts, UINT8 *buf);
static int  whal_apiHDK_VersionCB(BOOL Sts, UINT8 *buf);

static int  interfWD_saveAndSetRxDot11ModeCB(BOOL Sts, UINT8 *buf);
static int  interfWD_ReadAciCountAndProceedCB(BOOL Sts, UINT8 *buf);
static int  interfWD_ReadMediumUsageAndProceedCB(BOOL Sts, UINT8 *buf);



/* call back wait mechanism init*/
DDA_DECLARE_WAIT_MECHANISM(AcxStatWait);
DDA_DECLARE_WAIT_MECHANISM(AcxRxStatWait);
DDA_DECLARE_WAIT_MECHANISM(AcxTxStatWait);
DDA_DECLARE_WAIT_MECHANISM(FcsErrWait);
DDA_DECLARE_WAIT_MECHANISM(ACIcountWait);
DDA_DECLARE_WAIT_MECHANISM(mediumUsageWait);
DDA_DECLARE_WAIT_MECHANISM(edWait);
DDA_DECLARE_WAIT_MECHANISM(pdWait);
DDA_DECLARE_WAIT_MECHANISM(rxDot11modeWait);
DDA_DECLARE_WAIT_MECHANISM(hdkVersionWait);

/****************************************************************************
 *                      whal_apiSetTxCompleteThreshold()
 ****************************************************************************
 * DESCRIPTION:	Set Tx complete interrupt threshold
 *
 * INPUTS:	
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetTxCompleteThreshold(int Trsh)
{
	whal_apiMacWrite(SCR_PAD9, Trsh);	
	return OK;
}

/****************************************************************************
 *                      whal_apiSetPDthreshold()
 ****************************************************************************
 * DESCRIPTION:	Set packet detection threshold
 *
 * INPUTS:	
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetPDthreshold(UINT16 pdThresh)
{
	return whal_acxConfigPDThreshold(pdThresh);
}

/****************************************************************************
 *                      whal_apiGetPDthreshold()
 ****************************************************************************
 * DESCRIPTION:	Get packet detection threshold
 *
 * INPUTS:	
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiGetPDthreshold(UINT32 *pdThresh)
{
	WlanElm_PDThreshold_T ie;
	
	whal_acxConfigReadPacketDetection(&ie, (void *)whal_apiPacketDetectCB, (UINT8 *)&ie);

	/* block here and wait for mailbox mechanism wake up */
	if((wdrv_DDA_sleep_timeout((void*)&pdWait, WHAL_PRINT_RES_TIMEOUT)) == 0)
	{
		printf("%s: whal not responding",__func__);
		return NOK;
	}
	
	*pdThresh = ie.dot11PDThreshold;

	return OK;
}

/****************************************************************************
 *                      whal_apiSetEnergyDetect()
 ****************************************************************************
 * DESCRIPTION:	Set energy detection threshold
 *
 * INPUTS:	
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetEnergyDetect(UINT32 ed)
{
	return whal_acxConfigEnergyDetect(ed);
	
}
/****************************************************************************
 *                      whal_apiGetEnergyDetect()
 ****************************************************************************
 * DESCRIPTION:	Get energy detection threshold
 *
 * INPUTS:	
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiGetEnergyDetect(UINT32 *ed)
{
	WlanElm_EDThreshold_T ie;

	whal_acxConfigReadEnergyDetection(&ie, (void *)whal_apiEnergyDetectCB, (UINT8 *)&ie);

	/* block here and wait for mailbox mechanism wake up */
	if((wdrv_DDA_sleep_timeout((void*)&edWait, WHAL_PRINT_RES_TIMEOUT)) == 0)
	{
		printf("%s: whal not responding",__func__);
		return NOK;
	}
	
	*ed = ie.dot11EDThreshold;

	return OK;
	
}
/****************************************************************************
 *                      whal_apiSetRxDot11mode()
 ****************************************************************************
 * DESCRIPTION:	Set RX dot11 mode
 *
 * INPUTS:	
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetRxDot11mode(UINT32 mode)
{
	return whal_acxConfigRXdot11Mode(mode);
	
}
/****************************************************************************
 *                      whal_apiGetRxDot11mode()
 ****************************************************************************
 * DESCRIPTION:	Get RX dot11 mode
 *
 * INPUTS:	
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiGetRxDot11mode(UINT32 *mode)
{

	WlanElm_RxDot11Mode_T ie;

	whal_acxConfigReadRXdot11Mmode(&ie, (void *)whal_apiRxDot11ModeCB, (UINT8 *)&ie);

	/* block here and wait for mailbox mechanism wake up */
	if((wdrv_DDA_sleep_timeout((void*)&rxDot11modeWait, WHAL_PRINT_RES_TIMEOUT)) == 0)
	{
		printf("%s: whal not responding",__func__);
		return NOK;
	}
	
	*mode = ie.dot11RxMode;

	return OK;
}
/****************************************************************************
 *                      whal_apiGetACIcounter()
 ****************************************************************************
 * DESCRIPTION:	Get the ACI counter
 *				(CANNOT BE CALLED FROM (S)IRQ CONTEXT)
 * INPUTS:	
 *		UINT32 *count		The value pointer 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiGetACIcounter(UINT32 *count)
{
	
	AcxElm_AciEvent_T ie;

	whal_acxConfigReadAciCounter(&ie, (void *)whal_apiACIcountCB, (UINT8 *)&ie);

	/* block here and wait for mailbox mechanism wake up */
	if((wdrv_DDA_sleep_timeout((void*)&ACIcountWait, WHAL_PRINT_RES_TIMEOUT)) == 0)
	{
		printf("%s: whal not responding",__func__);
		return NOK;
	}
	
	*count = ie.aciEventsCounter;

	
	return OK;
	
}

/****************************************************************************
 *                      whal_apiGetMediumUsage()
 ****************************************************************************
 * DESCRIPTION:	Get the medium usage statistics 
 *				(CANNOT BE CALLED FROM (S)IRQ CONTEXT !!!!)
 * INPUTS:	
 *		UINT32 *count		The value pointer 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiGetMediumUsage(UINT32 *usage, UINT32 *intvl)
{
	
	AcxElm_MediumUsage_T ie;

	whal_acxConfigReadMediumUsage(&ie, (void *)whal_apiMediumUsageCB, (UINT8 *)&ie);

	/* block here and wait for mailbox mechanism to finish */
	
	if((wdrv_DDA_sleep_timeout((void*)&mediumUsageWait, WHAL_PRINT_RES_TIMEOUT)) == 0)
	{
		printf("%s: whal not responding",__func__);
		return NOK;
	}

	*usage = ie.mediumUsage;
	*intvl = ie.period;
	
	return OK;

}

/****************************************************************************
 *                      whal_apiInit()
 ****************************************************************************
 * DESCRIPTION:	Initiate the Hal objects and the ACX100
 * 
 * INPUTS: None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiInit(whal_UsrInitFuncCB CbFunc, int retArg)
{
	int 	Status;
	UINT8 	numSupWone;
	
	Bss_Started = 0;

   /* Must check status and abort on fail otherwise, system hangs! */
    Status = boardInitAcx111Addr(&AcxRegAddr, &AcxMemAddr);
	if( OK == Status)
	{
	   printf("AcxRegAddr = 0x%X, AcxMemAddr = 0x%X\n", AcxRegAddr,AcxMemAddr);
       /*Status = boardInitAcx111Intr((INT_HANDLER)whal_acxIntrHandler);*/
	   /*Status = dda_init_vlynq_int();*/
	}
    if( OK != Status)
    {
	   printf("whal_apiInit: ERROR, Fail to initiate Vlynq\n");
	   return NOK;
    }      
	
	/* 
	 * initiate ACX100 modules 
	 */
	Status = whal_ParamsInit();
	if (Status != OK)
	{
		printf("whal_apiInit: ERROR, Failed in whal_ParamsInit\n");
		return NOK; 		
	}

	Status = whal_acxProcInitiate();
	if (Status != OK)
	{
		printf("whal_apiInit: ERROR, Failed to initiate the ACX100\n");
		return NOK; 
	}

	whal_acxIntrInit();
	whal_acxMboxInit();
	
	/*
	 * Initiate the hal objects
	 */
	Status = whal_txInit();
	if (Status != OK)
	{
		printf("whal_apiInit: ERROR, Failed in whal_txInit\n");
		return NOK; 
	}

	Status = whal_rxInit();
	if (Status != OK)
	{
		printf("whal_apiInit: ERROR, Failed in whal_rxInit\n");
		return NOK; 
	}

    /*
	 * Config the ACX111.
	 */
    Status = whal_apiConfig();
	if (Status != OK)
	{
		printf("whal_apiInit: ERROR, Failed in whal_apiConfig\n");
		return NOK; 
	}

   /*
	 * Notify the upper layer upon init completion.
	 */

   wdrv_dbGetNumSupWone(&numSupWone);
   if (numSupWone == WONE_DUAL_BAND)
       whal_apiSetTxCompleteThreshold(0);
   else
	   whal_apiSetTxCompleteThreshold(DEFAULT_TX_COMPLETE_TO | DEFAULT_TX_COMPLETE_TRSH);


   if( 0 != wdrv_psGetTotalStaInPSMode())
   {
	  whal_apiSetBssPs( TRUE );
   }

   PAL_spinLock_init(&whalLock);

   if( CbFunc != NULL )
	   CbFunc( retArg );

   return OK;
}

/****************************************************************************
 *                      whal_apiUnload()
 ****************************************************************************
 * DESCRIPTION:	Unload the Hal objects and the ACX100
 * 
 * INPUTS: None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiUnload(int stage)
{
	switch (stage)
	{
		case 0:
			/*whal_acxCmdDisableTx();
			whal_acxCmdDisableRx();
			whal_apiAcxReset();*/
			whal_acxMboxUnload();
#ifdef TIWLAN_DEBUG
			whal_trcUnload();
#endif
			whal_acxIntrDisable(-1);
			break;

		case 1:
			boardUnloadAcx111();
			break;
	}

	return OK;
}

/****************************************************************************
 *                      whal_apiGetAcxRegAddr()
 ****************************************************************************
 * DESCRIPTION:	Get the Acx100 registers address
 ****************************************************************************/
UINT32 whal_apiGetAcxRegAddr(void)
{
	return AcxRegAddr;
}
/****************************************************************************
 *                      whal_apiGetAcxMemAddr()
 ****************************************************************************
 * DESCRIPTION:	Get the Acx100 memory address
 ****************************************************************************/
UINT32 whal_apiGetAcxMemAddr(void)
{
	return AcxMemAddr;
}

/****************************************************************************
 *                      whal_apiConfig()
 ****************************************************************************
 * DESCRIPTION:	Configure the Hal objects and the ACX100
 * 
 * INPUTS: None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiConfig(void)
{
	UINT8	TotalWones, wone_index;


	wdrv_dbGetNumSupWone(&TotalWones);
	/*
	 * Read config options (ACX100 EEPROM), Must be before any configuration 
	 * because the ACX100 put the data in the mbox
	 */
	whal_acxConfigRead_ConfigOptions();
	whal_acxMboxSetCommandStatus(0);

	/*
	 * Read Fw version
	 */
	whal_acxConfigRead_FwVersion();
	whal_ParamsPrintFwVersion();
	/*
	 * Configure the ACX100 memory (WEP, Templates, Queue, Buffers)
	 */
    whal_acxProcMemoryConfigure();

	/*
	 * OVERIDE ACX100 DEFAULTS
	 * -----------------------
	 * Configure StationId, Rx filters and Enable Tx/Rx
	 * (!!!) Change the name to OverideDefualts
	 */
	whal_acxConfigStationId();
	whal_acxConfigFragThreshold();
	whal_acxConfigCWminWin();
	whal_acxConfigShortRetryLimit();
	whal_acxConfigLongRetryLimit();
	whal_acxConfigRateFallback();
	whal_acxConfigRxConfig(); 


    for (wone_index = 0 ;wone_index <TotalWones; wone_index++)
        whal_acxConfigCurrRegDomain(whal_ParamsGetGenParams(wone_index)->CurrRegDomain,
                                    whal_ParamsGetGenParams(wone_index)->Band); 

    
    
    whal_acxConfigPowerLevel();

	/* Set energy detection threshold */
	if(whal_ParamsGetGenParams(0)->enEnergyDetect) 
	   {
		   whal_apiSetEnergyDetect(ENERGY_DETECT_ON);
	   }
	else 
	   {
		   whal_apiSetEnergyDetect(ENERGY_DETECT_OFF);
	   }

    /* Set packet detection threshold */
    whal_apiSetPDthreshold(whal_ParamsGetGenParams(0)->packetDetectThresh);

	/* Set short slot time capability */
	if ((whal_ParamsGetGenParams(0)->BeaconCapabilities) & DOT11_CAPS_SLOTTIME)
	{
        whal_acxConfigSlotTime(SLOT_TIME_SHORT, 0);
	}
	else
	{
		whal_acxConfigSlotTime(SLOT_TIME_LONG,0);
	}

	/* Disable OFDM receiver if dot11 mode is B-only*/
	if (whal_ParamsGetGenParams(0) -> Dot11Mode == DOT11_MODE_B_ONLY ||
		whal_ParamsGetGenParams(0) -> Dot11Mode == DOT11_MODE_B_ONLY_PLUS)
	{
		whal_apiSetRxDot11mode(RX_DOT11_MODE_CCK);
	}
	/* Disable OFDM receiver in channel 14 (overcome FCS errors problem) */
	if (whal_ParamsGetGenParams(0)->StartRadioChannel == 14)
	{
		 whal_apiSetRxDot11mode(RX_DOT11_MODE_CCK);
	}

	/* Enable Rx antenna diversity as it was disabled by default from FW 0.0.0.83 */
    whal_apiPhyWrite(0x4bb1, 0x4a);

   /* Configure radio params */
   whal_apiSetRadioParams();


   if (TotalWones == WONE_DUAL_BAND)
	{
		printf("whal_apiConfig: WONE Config \n");
		whal_acxConfigWoneConfig();
	}
	
  
	/* configure wd_beacon_limiter = 1+ ((beaconInterval * 2)/WATCH_DOG_TIMER_VAL) */
   {
      UINT16 beaconInterval;
      wdrv_dbGetStationConfigBeaconPeriod( &beaconInterval);
      wd_beacon_limiter = 1 + ((beaconInterval * 2)/WATCH_DOG_TIMER_VAL);
      printf( "whal_apiConfig: beaconInterval = %u\n\t"
              "WATCH_DOG_TIMER_VAL = %u\n\t"
              "wd_beacon_limiter = %u\n"
              , (UINT)beaconInterval, WATCH_DOG_TIMER_VAL , wd_beacon_limiter);
   }

   /* FW counters now may contain invalid values */
   acxCountersValid = FALSE;

   return OK;
}/* END whal_apiConfig() */

/****************************************************************************
 *                      whal_apiStartBss()
 ****************************************************************************
 * DESCRIPTION:	Enable Tx/Rx and Start the AP 
 * 
 * INPUTS: None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiStartBss(void)
{
	unsigned char *pBssId = whal_ParamsGetBssId();
	UINT16 Caps;
	UINT8 WoneIndex, TotalWones;
    UINT8 PowerLevel, RegDomain;
	char  temp[DOT11_SSID_MAX+1];

	memcpy(temp, whal_ParamsGetElm_Ssid()->serviceSetId, whal_ParamsGetElm_Ssid()->hdr.eleLen);
	temp[whal_ParamsGetElm_Ssid()->hdr.eleLen] = '\0';

	wdrv_dbGetNumSupWone(&TotalWones);

    wdrv_dbGetPowerLevel(whal_ParamsGetRadioChannel(0),&PowerLevel);

#ifndef CONFIG_MIPS_PUMAS
	/* WiFi mode optimizations */
	if (wdrv_dbGetApBridgeMode())
			whal_apiPhyWrite(0x20a8, 0x76);
#endif

	wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "whal_apiStartBss: Enable Tx, Rx and Start the Bss\n");
	wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "----------------------------------------------------------------\n");
	wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "----------------------------------------------------------------\n");
	wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "  START BSS, SSID=%s, BSSID=%02X-%02X-%02X-%02X-%02X-%02X\n", /*whal_ParamsGetElm_Ssid()->serviceSetId*/temp, pBssId[0], pBssId[1], pBssId[2], pBssId[3], pBssId[4], pBssId[5]);
	wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "----------------------------------------------------------------\n");
	wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "----------------------------------------------------------------\n");
    wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, " AP Power Level = %d\n",PowerLevel);
    wdrv_dbGetdot11CurrentRegDomain(0,&RegDomain);    
	switch(RegDomain)
    {
		case REG_DOMAIN_FCC:
			wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, " Regulatory Domain = FCC\n");
			break;
        case REG_DOMAIN_IC:
            wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, " Regulatory Domain = IC\n");
            break;
        case REG_DOMAIN_ETSI:
            wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, " Regulatory Domain = ETSI\n");
            break;
        case REG_DOMAIN_SPAIN:
            wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, " Regulatory Domain = SPAIN\n");
            break;
        case REG_DOMAIN_FRANCE:
            wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, " Regulatory Domain = FRANCE\n");
            break;
        case REG_DOMAIN_MKK:
            wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, " Regulatory Domain = MKK\n");
            break;
        case REG_DOMAIN_MKK1:
            wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, " Regulatory Domain = MKK1\n");
            break;
        case REG_DOMAIN_US:
            wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, " Regulatory Domain = US\n");
            break;
        case REG_DOMAIN_WORLD:
            wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, " Regulatory Domain = WORLD\n");
            break;
        default:
            wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, " Regulatory Domain = 0x%X\n",RegDomain);
            break;
	}


	for	(WoneIndex = 0; WoneIndex < TotalWones; WoneIndex++)
	{
		wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "          Net[%1d] : Channel=%d\n", WoneIndex, whal_ParamsGetRadioChannel(WoneIndex));
	}
	wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "----------------------------------------------------------------\n");

#ifndef NO_PCF_PATCH
	if (whal_ParamsGetNvParams()->PcfPatch == 1)
	{
		wdrv_dbGetApCapabilities(0,&Caps);
		wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "Pcf patch - change driver capability from 0x%x", Caps);
		Caps |= (UINT16)(whal_ParamsGetNvParams()->PcfPatch_Caps);
		wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "Pcf patch - change driver capability to   0x%x", Caps);
		wdrv_dbSetApCapabilities(0,Caps);
	}
#endif

	whal_acxIntrEnable(INTR_RX_TRF);

 	if (TotalWones == WONE_DUAL_BAND)
	{
	    wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "whal_acxProcConfigure: WONE Slice Config ");
		whal_acxConfigWoneSlice();              
	}
    else
    {
        whal_acxCmdEnableTx();
    }

	PAL_sleepLong(10);
	whal_acxCmdEnableRx();
	

	for	(WoneIndex = 0; WoneIndex < TotalWones; WoneIndex++)
	{
		PAL_sleepLong(10);
		whal_acxCmdStartBss(WoneIndex);
	}

	Bss_Started = 1;

 	if (TotalWones == WONE_DUAL_BAND)
	      wdrv_WoneBalanceInit();
	
	/* read leds mask on AR7VWi only */
	PAL_ConfigVWiLeds();
	
	return OK;
}


/****************************************************************************
 *                      whal_apiStopBss()
 ****************************************************************************
 * DESCRIPTION:	Disables Tx/Rx, closes BSS for incoming messages
 *				and stops the watchdog
 * 
 * INPUTS: None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiStopBss(void)
{
	UINT8	TotalWones;
	
	wdrv_dbGetNumSupWone(&TotalWones);
	
 	if (TotalWones == WONE_DUAL_BAND)
	{
		wdrv_WoneBalanceShutdown();
	}

	whal_acxIntrDisable(INTR_RX_TRF);
	Bss_Started = 0;

/*	whal_acxCmdDisableRx(); */
	whal_acxCmdDisableTx();

	return OK;
}


/****************************************************************************
 *                      whal_apiAcxReset()
 ****************************************************************************
 * DESCRIPTION:	reset the ACX100 
 * 
 * INPUTS: None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiAcxReset(void)
{	
	whal_acxCmdReset();
	return OK;
}

/****************************************************************************
 *                      whal_apiRxIntrEnable()
 ****************************************************************************
 * DESCRIPTION:	Enable Rx interrupt  
 * 
 * INPUTS: 
 *		int Action	Enable(1), DIsable(0)	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiRxIntrEnable(int Action)
{
	if (Action==0)
		whal_acxIntrDisable(INTR_RX_DATA);
	else
		whal_acxIntrEnable(INTR_RX_DATA);
	return OK;
}
/****************************************************************************
 *                      whal_apiRxRecv()
 ****************************************************************************
 * DESCRIPTION:	Read one MPDU from the ACX100  
 * 
 * INPUTS: 
 *		int *Len				The received MPDU size to be returned
 *		RxAttr_T *pRxAttr		The received MPDU attributes
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	MPDU handle
 ****************************************************************************/
MEM_HANDLE whal_apiRxRecv(int *Len, RxAttr_T *pRxAttr)
{
	MEM_HANDLE pHostBufs;
	/*
	 * Copy the next frame from the ACX100
	 */
	pHostBufs = whal_rxRecv(Len, pRxAttr);

	if (pHostBufs != 0)
	{
		whal_apiRestartWatchDog_PacketReceived();
	}

	return (pHostBufs); /* could be NULL */
}

/****************************************************************************
 *                      whal_apiRxClean()
 ****************************************************************************
 * DESCRIPTION:	Reclaim Rx queue descriptors and Rx buffers
 * 
 * INPUTS:  None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void whal_apiRxClean(void)
{
	whal_rxClean();
}

/****************************************************************************
 *                      whal_apiSendMsdu()
 ****************************************************************************
 * DESCRIPTION:	Send one MSDU to the ACX100  
 * 
 * INPUTS: 
 *		wTxQueueId_T TxQid		The required transmit queue
 *		MEM_HANDLE Msdu_Handle	The MSDU descriptor handle
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSendMsdu(TxQueueId_T TxQid, MEM_HANDLE Msdu_Handle, TxAttr_T *pTxAttr)
{
	int Status;

	if (!Bss_Started)
		Status = NOK;
	else
		Status = whal_txSendMsdu(TxQid, Msdu_Handle, pTxAttr);

	if(Status != OK)
	{	
		wlan_memMngrFreeMSDU( ((mem_MSDU_T *)Msdu_Handle)->handle  );
	}

	return (Status);
}

/****************************************************************************
 *                      whal_apiSendProbeResp()
 ****************************************************************************
 * DESCRIPTION:	Send Probe response
 *
 * INPUTS:	None
 *
 * OUTPUT:	None
 * 
 * RETURNS:	
 ****************************************************************************/
void whal_apiSendProbeResp(UINT8 *DstMac, dot11_SSID_T *pWlanElm_Ssid, UINT8 wone_index)
{
	mem_MSDU_T *pMsdu;
	TxAttr_T TxAttr;
	UINT32 FrameSize;
	dot11_probeRsp *pFrame;
	int Status;
    TxQueueId_T txQid;

	/* 
	 * Allocate MSDU 
	 */
	Status = wlan_memMngrAllocMSDU(&pMsdu, 100 /* dummy */);
	if (Status == NOK)
	{
		printf("whal_apiSendProbeResp: ERROR on wlan_memMngrAllocMSDU\n");
		return;
	}

	/* 
	 * Fill MSDU, Header+Data in the same host block
	 */
	pFrame = (dot11_probeRsp *)memMgr_MsduHdrAddr(pMsdu);
	whal_frameBuildProbeResp(wone_index,pFrame, &FrameSize, DstMac, pWlanElm_Ssid);
	pFrame->hdr.fc = DOT11_FC_PROBE_RESP;
	memMgr_MsduFirstLen(pMsdu) = FrameSize;
	memMgr_MsduDataSize(pMsdu) = memMgr_MsduFirstLen(pMsdu);
	memMgr_MsduHdrLen(pMsdu) = 24; /*sizeof(dot11_header_T);*/

	/* 
	 * Send the MSDU 
	 */

	TxAttr.ShortPreamble = 0;
	TxAttr.ProtectionType = TXATTR_PROTECTION_NONE;
    wdrv_dbGetDefaultActiveRateAcx(wone_index,&(TxAttr.Rate), &(TxAttr.RateWithFallback), TXATTR_UNICAST);
	

    wdrv_dbGetAcxGlobalQindex(wone_index, WONE_LOW_PRIOR_QUEUE, &txQid);
    whal_apiSendMsdu(txQid, (MEM_HANDLE)pMsdu, &TxAttr);

}

/****************************************************************************
 *                      whal_apiTxCheckSize()
 ****************************************************************************
 * DESCRIPTION:	Check if there is enough place in the Tx  
 * 
 * INPUTS: 
 *		int Len		The required transmit length
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiTxCheckSize(TxQueueId_T TxQid, int Len)
{
#if 0  /* MICHI - TODO  !!!! need to cleanup here removed fopr debug */
	/*
	 * Free Tx queue before the check
	 */
	whal_txClean(BcastTxQid);
	whal_txClean(UcastTxQid);
#endif 
	/*
	 * Check if there is enough place in the Acx100 memory
	 */
	return(whal_txCheckSize(TxQid, Len));
}

/****************************************************************************
 *                      whal_apiTxGetBufsNum()
 ****************************************************************************
 * DESCRIPTION:	Returns number of free descriptors in the host Q
 *              and number of currently used descriptors in the Acx Q. 
 * 
 * INPUTS: 
 *		TxQid
 * 
 * OUTPUT:	Host FreeDescNum, ACX UsedDescNum
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiTxGetBufsNum(TxQueueId_T TxQid, UINT32 *FreeDescNum, UINT32 *UsedDescNum)
{
	/*
	 * Check if there is enough place in the Acx100 memory
	 */
	return (whal_txGetBufsNum(TxQid, FreeDescNum, UsedDescNum));
}

/****************************************************************************
 *                      whal_apiTxGetAcxQsize()
 ****************************************************************************
 * DESCRIPTION:	Returns Acx Tx Q size
 * 
 * INPUTS: 
 *		TxQid
 * 
 * OUTPUT:	
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void whal_apiTxGetAcxQsize(TxQueueId_T TxQid, UINT32 *queueSize)
{
	*queueSize = whal_txQueue_GetDescNum(TxQid);
}

/****************************************************************************
 *                      whal_apiTxClean()
 ****************************************************************************
 * DESCRIPTION:	Reclaim Tx queue descriptors and Rx buffers
 * 
 * INPUTS:  
 *		wTxQueueId_T TxQid		The required transmit queue
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void whal_apiTxClean(TxQueueId_T TxQid)
{
	whal_txClean(TxQid);
}

/****************************************************************************
 *                      whal_apiTxCleanAll()
 ****************************************************************************
 * DESCRIPTION:	Free Tx unicast and broadcast host queue MSDUs
 * 
 * INPUTS:  
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void whal_apiTxCleanAll(void)
{
	whal_txCleanAll();
}

/****************************************************************************
 *                      whal_apiSetTxStatusCB()
 ****************************************************************************
 * DESCRIPTION:	Set the callback for the return Tx status.
 *
 * INPUTS:	
 *		TxStatus_Callback_T TxStatusCB		The callback address
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetTxStatusCB(TxStatus_Callback_T TxStatusCB)
{
	whal_txSetStatusCB(TxStatusCB);
	return OK;
}

/****************************************************************************
 *                      whal_apiSetRxDataCB()
 ****************************************************************************
 * DESCRIPTION:	Set the callback for the Rx interrupt.
 *
 * INPUTS:	
 *		TxStatus_Callback_T TxStatusCB		The callback address
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetRxDataCB(RxData_Callback_T RxDataCB, int Threshold)
{
	if (whal_ParamsGetNvParams()->UseIntrThreshold == 0)  /* Get interrupt every frame */
	{
		whal_acxIntrSetRxDataCB(RxDataCB, Threshold);
	}
	else
	{
		whal_acxIntrSetRxDataCB(RxDataCB, 1);
		whal_acxConfigRxIntr(Threshold, 10000 /* timeout in usec */);
	}
	return OK;
}


/****************************************************************************
 *                      whal_apiSetBssPs()
 ****************************************************************************
 * DESCRIPTION:	Set the power save mode of the bss.
 *
 * INPUTS:	
 *		int BssPsMode	Bss in Power save (1) or not (0)
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetBssPs(int BssPsMode)
{
	whal_acxConfigBssPowerSave(BssPsMode);
	return OK;
}




/****************************************************************************
 *                      whal_apiSetTim()
 ****************************************************************************
 * DESCRIPTION:	Set the Tim element
 *
 * INPUTS:	
 *		char BmapControl		bitmap control of the tim element 
 *		char *PartialBmapVec	bitmap of AIDs
 *		int  PartialBmapLen		number of bytes in PartialBmapVec
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetTim(UINT8 WoneIndex, char BmapControl, char *PartialBmapVec, int PartialBmapLen)
{
	UINT8  TotalWones;

	wdrv_dbGetNumSupWone(&TotalWones);
	if(WoneIndex >= TotalWones)
	{
		printf("whal_apiSetTim: wrong WoneIndex=%d\n",WoneIndex);
		return NOK;
	}
	
	whal_ParamsSetElm_Tim(BmapControl, PartialBmapVec, PartialBmapLen);
	
	return whal_acxCmdTimTemplate(WoneIndex);
}




/*
 * -----------------------------------------------------------------------
 *						PARAMETERS SETTING API
 * -----------------------------------------------------------------------
 */



/****************************************************************************
 *                      whal_apiSetRadioChannel()
 ****************************************************************************
 * DESCRIPTION:	Set the radio channel
 *
 * INPUTS:	
 *		int Channel		The channel value 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetRadioChannel(UINT8 WoneIndex,int Channel)
{
	whal_ParamsSetRadioChannel(WoneIndex, Channel);

	/* Change the channel in beacon & probe res frame */
	whal_acxCmdBeaconTemplate(WoneIndex); 
	whal_acxCmdProbeRespTemplate(WoneIndex); 

	/*	whal_acxCmdEnableTx();*/
	return OK;
}

/****************************************************************************
 *                      whal_apiSetFragmentThreshold()
 ****************************************************************************
 * DESCRIPTION:	Set the fragment threshold
 *
 * INPUTS:	
 *		int FragSize		The fragment threshold value (minimum 256) 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetFragmentThreshold(int FragSize)
{
	if (FragSize < 256)
		return NOK;
	whal_ParamsSetFragmentThreshold(FragSize);
	whal_acxConfigFragThreshold();
	return OK;
}

/****************************************************************************
 *                      whal_apiSetRtsThreshold()
 ****************************************************************************
 * DESCRIPTION:	Set the rts threshold
 *
 * INPUTS:	
 *		int RtsSize		The rts threshold value 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetRtsThreshold(int RtsSize)
{
	whal_ParamsSetRtsThreshold(RtsSize);
	return OK;
}

/****************************************************************************
 *                      whal_apiSetShortRetryLimit()
 ****************************************************************************
 * DESCRIPTION:	Set the short retry limit
 *
 * INPUTS:	
 *		int Val		The short retry limit 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetShortRetryLimit(UINT8 Val)
{
	whal_ParamsSetShortRetryLimit(Val);
	whal_acxConfigShortRetryLimit();
	return OK;
}

/****************************************************************************
 *                      whal_apiSetLongRetryLimit()
 ****************************************************************************
 * DESCRIPTION:	Set the Long retry limit
 *
 * INPUTS:	
 *		int Val		The Long retry limit 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetLongRetryLimit(UINT8 Val)
{
	whal_ParamsSetLongRetryLimit(Val);
	whal_acxConfigLongRetryLimit();
	return OK;
}

/****************************************************************************
 *                      whal_apiSetMaxTxMsduLifetime()
 ****************************************************************************
 * DESCRIPTION:	Set the Max Tx Msdu Life time
 *
 * INPUTS:	
 *		int Val		The time value 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetMaxTxMsduLifetime(UINT32 Val)
{
	return NOK;
}

/****************************************************************************
 *                      whal_apiSetPbcc()
 ****************************************************************************
 * DESCRIPTION:	change the PBCC bit in the beacon frame
 *
 * INPUTS:	
 *		PbccVal		The PBCC bit value 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetPbcc(UINT8 WoneIndex, int PbccVal)
{
	if (PbccVal == 0)
		whal_ParamsGetGenParams(WoneIndex)->BeaconCapabilities &= ~DOT11_CAPS_PBCC;
	else
		whal_ParamsGetGenParams(WoneIndex)->BeaconCapabilities |=  DOT11_CAPS_PBCC;

	/* change the PBCC bit in the beacon frame */
	whal_acxCmdBeaconTemplate(WoneIndex); 
	return OK;
}

/****************************************************************************
 *                      whal_apiSetSlotTime()
 ****************************************************************************
 * DESCRIPTION:	change the Short SlotTime bit in the beacon & probe res frame
 *
 * INPUTS:	
 *		SlotTimeVal		The Short SlotTime bit value in the Capabilities
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetSlotTime(UINT8 WoneIndex, int SlotTimeVal)
{
	UINT8	TotalWones;
    UINT8   SlotTime;

	if (SlotTimeVal == 0)
	{
        SlotTime = SLOT_TIME_LONG;
		whal_ParamsGetGenParams(WoneIndex)->BeaconCapabilities &= ~DOT11_CAPS_SLOTTIME;
		whal_ParamsGetGenParams(WoneIndex)->ProbResCapabilities &= ~DOT11_CAPS_SLOTTIME;
	}
	else
	{
        SlotTime = SLOT_TIME_SHORT;
		whal_ParamsGetGenParams(WoneIndex)->BeaconCapabilities |=  DOT11_CAPS_SLOTTIME;
		whal_ParamsGetGenParams(WoneIndex)->ProbResCapabilities |=  DOT11_CAPS_SLOTTIME;
	}

	/* Change the Short SlotTime bit in the beacon & probe res frame */
	whal_acxCmdBeaconTemplate(WoneIndex); 
	whal_acxCmdProbeRespTemplate(WoneIndex); 

	wdrv_dbGetNumSupWone(&TotalWones);
	
	if (TotalWones == WONE_SINGLE_BAND)
	{

	    /* configure slot time  */
		whal_acxConfigSlotTime(SlotTime,WoneIndex);
	}

	return OK;
}


/****************************************************************************
 *                      whal_apiSetERPLongPreambleOnly()
 ****************************************************************************
 * DESCRIPTION:	Set/Clear the ERP IE Preamble bit
 *
 * INPUTS:	longPreambleOnly - bit 2 in ERP IE
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetERPLongPreambleOnly(UINT8 WoneIndex, UINT8 longPreambleOnly)
{
	/* Set the ERP IE preamble bit */
	whal_ParamsSetElm_ERPLongPreambleOnly(WoneIndex, longPreambleOnly);

	/* change the ERP Protection IE in beacon & probe response frames */
	whal_acxCmdTimTemplate(WoneIndex); /* the IE is located after the TIM in Beacon frame */
	whal_acxCmdProbeRespTemplate(WoneIndex); 
	return OK;
}

/****************************************************************************
 *                      whal_apiSetERPProtection()
 ****************************************************************************
 * DESCRIPTION:	Set/Clear the ERP IE protection bits
 *
 * INPUTS:	nonERPpresent - bit 0 in ERP IE
            useProtection - bit 1 in ERP IE
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetERPProtection(UINT8 WoneIndex, UINT8 nonERPpresent, UINT8 useProtection)
{
	wdrv_dbSetCurrentERPProtectionMode(useProtection);
    wdrv_dbSetCurrentERPNonPresent(nonERPpresent);

	/* Set the ERP IE protection bits */
	whal_ParamsSetElm_ERPProtection(WoneIndex,nonERPpresent,useProtection);

	/* change the ERP Protection IE in beacon & probe response frames */
	whal_acxCmdTimTemplate(WoneIndex);  /* the IE is located after the TIM in Beacon frame */
	whal_acxCmdProbeRespTemplate(WoneIndex); 
	return OK;
}

/****************************************************************************
 *                      whal_apiGetFcsError()
 ****************************************************************************
 * DESCRIPTION:	Get the Fcs error counter
 *
 * INPUTS:	
 *		UINT32 *Val		The value pointer 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiGetFcsError(UINT32 *Val)
{
	
	WlanElm_FcsError_T Elem;
	
	whal_acxConfigRead_FcsError(&Elem, (void *)whal_apiFcsErrCB, (UINT8 *)&Elem);
	
	/* block here and wait for mailbox mechanism wake up */
	if((wdrv_DDA_sleep_timeout((void*)&FcsErrWait, WHAL_PRINT_RES_TIMEOUT)) == 0)
	{
		printf("%s: whal not responding",__func__);
		return NOK;
	}
	
	*Val = Elem.FcsErrorCount;

	return OK;
}

/****************************************************************************
 *                      whal_apiGetHDKVersion()
 ****************************************************************************
 * DESCRIPTION:	Print the HDK version
 *
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiGetHDKVersion(UINT8 *buf)
{
	
	WlanElm_HDKModuleVersion_t Elem, *reply;
	UINT8 tmp_buf[255];
	int len;
	
	whal_acxConfigRead_HDK_Version(&Elem, (void *)whal_apiHDK_VersionCB, buf);

	/* block here and wait for mailbox mechanism wake up */
	if((wdrv_DDA_sleep_timeout((void*)&hdkVersionWait, WHAL_PRINT_RES_TIMEOUT)) == 0)
	{
		printf("%s: whal not responding",__func__);
		return 0;
	}

	reply = (WlanElm_HDKModuleVersion_t *)buf;
	len = sprintf(tmp_buf,"Radio Module Version = %d.%d.%d.%d\n", reply->majorVersion, reply->minorVersion, 
								   reply->minorMinorVersion, reply->hdkModuleReleaseType);

	memcpy(buf, tmp_buf, sizeof(tmp_buf));

	return len;
}


/****************************************************************************
 *                      whal_apiGetFailedCount()
 ****************************************************************************
 * DESCRIPTION:	Get the Failed counter
 * 
 * INPUTS:	
 *		UINT32 *Val		The value pointer 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiGetFailedCount(UINT8 WoneIndex, UINT32 *Val)
{
	TxQueueId_T 	TxQid;
	UINT8 			WoneTxQueues, tmpIndex;
    UINT8			Type, Priority;

	/* Skip all the non related queues ... */
	for (TxQid = 0, tmpIndex = 0; tmpIndex < WoneIndex;TxQid++, tmpIndex++)
	{
		wdrv_dbGetNumSupWoneQueues	( tmpIndex, &WoneTxQueues );
		TxQid += WoneTxQueues;
	}
	
	wdrv_dbGetNumSupWoneQueues	( WoneIndex, &WoneTxQueues );

	/* Sum counters of all the related unicast queues ... */
	*Val = 0;
	TxQid = 0;
	for (tmpIndex = 0; tmpIndex < WoneTxQueues; tmpIndex++, TxQid++)
	{
		wdrv_dbGetAcxQindex	(	WoneIndex,
								tmpIndex,
								&Type,
								&Priority	);
		if (!Type)
		{
		/*
			This counter shall increment when an MSDU is not transmitted
        	successfully due to the number of transmit attempts exceeding
        	either the  dot11ShortRetryLimit or dot11LongRetryLimit. 	
		*/
			*Val += (UINT32)whal_txQueue_GetRetryFailCount(TxQid);
		}
	}

	return OK;
}

/****************************************************************************
 *                      whal_apiGetRetryCount()
 ****************************************************************************
 * DESCRIPTION:	Get the Retry counter
 * 
 * INPUTS:	
 *		UINT32 *Val		The value pointer 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiGetRetryCount(UINT8 WoneIndex, UINT32 *Val)
{
	TxQueueId_T 	TxQid;
	UINT8 			WoneTxQueues, tmpIndex;
    UINT8			Type, Priority;

	/* Skip all the non related queues ... */
	for (TxQid = 0, tmpIndex = 0; tmpIndex < WoneIndex; tmpIndex++,TxQid++)
	{
		wdrv_dbGetNumSupWoneQueues	( tmpIndex, &WoneTxQueues );
		TxQid += WoneTxQueues;
	}
	
	wdrv_dbGetNumSupWoneQueues	( WoneIndex, &WoneTxQueues );

	/* Sum counters of all the related unicast queues ... */
	*Val = 0;
	for (tmpIndex = 0; tmpIndex < WoneTxQueues; tmpIndex++, TxQid++)
	{
		wdrv_dbGetAcxQindex	(	WoneIndex,
								tmpIndex,
								&Type,
								&Priority	);
		if (!Type)
		{
		/*
			This counter shall increment when an MSDU is successfully
        	transmitted after one or more retransmissions.	
		*/
			*Val += (UINT32)whal_txQueue_GetRetryCount(TxQid);
		}
	}

	return OK;
}

/****************************************************************************
 *                      whal_apiGetMultipleRetryCount()
 ****************************************************************************
 * DESCRIPTION:	Get the Multiple Retry counter
 * 
 * INPUTS:	
 *		UINT32 *Val		The value pointer 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiGetMultipleRetryCount(UINT8 WoneIndex, UINT32 *Val)
{
	TxQueueId_T 	TxQid;
	UINT8 			WoneTxQueues, tmpIndex;
    UINT8			Type, Priority;

	/* Skip all the non related queues ... */
	for (TxQid = 0, tmpIndex = 0; tmpIndex < WoneIndex; tmpIndex++)
	{
		wdrv_dbGetNumSupWoneQueues	( tmpIndex, &WoneTxQueues );
		TxQid += WoneTxQueues;
	}
	
	wdrv_dbGetNumSupWoneQueues	( WoneIndex, &WoneTxQueues );

	/* Sum counters of all the related unicast queues ... */
	*Val = 0;
	for (tmpIndex = 0; tmpIndex < WoneTxQueues; tmpIndex++, TxQid++)
	{
		wdrv_dbGetAcxQindex	(	WoneIndex,
								tmpIndex,
								&Type,
								&Priority	);
		if (!Type)
		{
		/*
			This counter shall increment when an MSDU is successfully
            transmitted after more than one retransmission.	
		*/
			*Val += (UINT32)whal_txQueue_GetMultipleRetryCount(TxQid);
		}
	}

	return OK;
}

/****************************************************************************
 *                      whal_apiGetAckFailCount()
 ****************************************************************************
 * DESCRIPTION:	Get the Ack fail counter
 * 
 * INPUTS:	
 *		UINT32 *Val		The value pointer 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiGetAckFailCount(UINT8 WoneIndex, UINT32 *Val)
{
	TxQueueId_T 	TxQid;
	UINT8 			WoneTxQueues, tmpIndex;
    UINT8			Type, Priority;

	/* Skip all the non related queues ... */
	for (TxQid = 0, tmpIndex = 0; tmpIndex < WoneIndex; tmpIndex++)
	{
		wdrv_dbGetNumSupWoneQueues	( tmpIndex, &WoneTxQueues );
		TxQid += WoneTxQueues;
	}
	
	wdrv_dbGetNumSupWoneQueues	( WoneIndex, &WoneTxQueues );

	/* Sum counters of all the related unicast queues ... */
	*Val = 0;
	for (tmpIndex = 0; tmpIndex < WoneTxQueues; tmpIndex++, TxQid++)
	{
		wdrv_dbGetAcxQindex	(	WoneIndex,
								tmpIndex,
								&Type,
								&Priority	);
		if (!Type)
		{
		/*
			This counter shall increment when an ACK is not received
           	when expected.
		*/
			*Val += (UINT32)whal_txQueue_GetAckFailCount(TxQid);
		}
	}

	return OK;
}

/****************************************************************************
 *                      whal_apiGetRtsFailCount()
 ****************************************************************************
 * DESCRIPTION:	Get the Rts fail counter
 * 
 * INPUTS:	
 *		UINT32 *Val		The value pointer 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiGetRtsFailCount(UINT8 WoneIndex, UINT32 *Val)
{
	TxQueueId_T 	TxQid;
	UINT8 			WoneTxQueues, tmpIndex;
    UINT8			Type, Priority;

	/* Skip all the non related queues ... */
	for (TxQid = 0, tmpIndex = 0; tmpIndex < WoneIndex; tmpIndex++)
	{
		wdrv_dbGetNumSupWoneQueues	( tmpIndex, &WoneTxQueues );
		TxQid += WoneTxQueues;
	}
	
	wdrv_dbGetNumSupWoneQueues	( WoneIndex, &WoneTxQueues );

	/* Sum counters of all the related unicast queues ... */
	*Val = 0;
	for (tmpIndex = 0; tmpIndex < WoneTxQueues; tmpIndex++, TxQid++)
	{
		wdrv_dbGetAcxQindex	(	WoneIndex,
								tmpIndex,
								&Type,
								&Priority	);
		if (!Type)
		{
		/*
			This counter shall increment when a CTS is not received in
			response to an RTS.	
		*/
			*Val += (UINT32)whal_txQueue_GetRtsFailCount(TxQid);
		}
	}

	return OK;
}

/****************************************************************************
 *                      whal_apiGetRtsSuccCount()
 ****************************************************************************
 * DESCRIPTION:	Get the Rts succ counter
 * 
 * INPUTS:	
 *		UINT32 *Val		The value pointer 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiGetRtsSuccCount(UINT8 WoneIndex, UINT32 *Val)
{
	TxQueueId_T 	TxQid;
	UINT8 			WoneTxQueues, tmpIndex,Priority,Type;

 
	/* Skip all the non related queues ... */
	for (TxQid = 0, tmpIndex = 0; tmpIndex < WoneIndex; tmpIndex++)
	{
		wdrv_dbGetNumSupWoneQueues	( tmpIndex, &WoneTxQueues );
		TxQid += WoneTxQueues;
	}
	
	wdrv_dbGetNumSupWoneQueues	( WoneIndex, &WoneTxQueues );

	/* Sum counters of all the related unicast queues ... */
	*Val = 0;
	for (tmpIndex = 0; tmpIndex < WoneTxQueues; tmpIndex++, TxQid++)
	{
		wdrv_dbGetAcxQindex	(	WoneIndex,
								tmpIndex,
								&Type,
								&Priority	);
		if (!Type)
		{
		/*
			This counter shall increment when a CTS is received in
            response to an RTS.	
		*/
			*Val += (UINT32)whal_txQueue_GetRtsSuccCount(TxQid);
		}
	}

	return OK;
}

/****************************************************************************
 *                      whal_apiGetTxFrameCount()
 ****************************************************************************
 * DESCRIPTION:	Get the Tx frame counter
 * 
 * INPUTS:	
 *		UINT32 *Val		The value pointer 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiGetTxFrameCount(UINT8 WoneIndex, UINT32 *Val)
{
	TxQueueId_T 	TxQid;
	UINT8 			WoneTxQueues, tmpIndex;
    UINT8			Type, Priority;

	/* Skip all the non related queues ... */
	for (TxQid = 0, tmpIndex = 0; tmpIndex < WoneIndex; tmpIndex++)
	{
		wdrv_dbGetNumSupWoneQueues	( tmpIndex, &WoneTxQueues );
		TxQid += WoneTxQueues;
	}
	
	wdrv_dbGetNumSupWoneQueues	( WoneIndex, &WoneTxQueues );

	/* Sum counters of all the related unicast queues ... */
	*Val = 0;
	for (tmpIndex = 0; tmpIndex < WoneTxQueues; tmpIndex++, TxQid++)
	{
		wdrv_dbGetAcxQindex	(	WoneIndex,
								tmpIndex,
								&Type,
								&Priority	);
		if (!Type)
		{
		/*
			This counter shall increment for each successfully transmitted MSDU.
		*/
			*Val += (UINT32)whal_txTxFrameCount(TxQid);
		}
	}

	return OK;
}

/****************************************************************************
 *                      whal_apiGetTxFragmentCount()
 ****************************************************************************
 * DESCRIPTION:	Get the Tx fragment counter
 * 
 * INPUTS:	
 *		UINT32 *Val		The value pointer 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiGetTxFragmentCount(UINT8 WoneIndex, UINT32 *Val)
{
	TxQueueId_T 	TxQid;
	UINT8 			WoneTxQueues, tmpIndex;


	/* Skip all the non related queues ... */
	for (TxQid = 0, tmpIndex = 0; tmpIndex < WoneIndex; tmpIndex++)
	{
		wdrv_dbGetNumSupWoneQueues	( tmpIndex, &WoneTxQueues );
		TxQid += WoneTxQueues;
	}
	
	/*
		This counter shall be incremented for an acknowledged MPDU
            with an individual address in the address 1 field or an MPDU
            with a multicast address in the address 1 field of type Data
            or Management.	
	*/
	wdrv_dbGetNumSupWoneQueues	( WoneIndex, &WoneTxQueues );
	
	/* Sum counters of all the related queues ... */
	*Val = 0;
	for (tmpIndex = 0; tmpIndex < WoneTxQueues; tmpIndex++, TxQid++)
	{
		*Val +=  (UINT32) whal_txQueue_GetSuccCount(TxQid);
	}

	return OK;
}

/****************************************************************************
 *                      whal_apiGetTxMulticastCount()
 ****************************************************************************
 * DESCRIPTION:	Get the Tx Multicast counter
 * 
 * INPUTS:	
 *		UINT32 *Val		The value pointer 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiGetTxMulticastCount(UINT8 WoneIndex, UINT32 *Val)
{
	TxQueueId_T 	TxQid;
	UINT8 			WoneTxQueues, tmpIndex;
    UINT8			Type, Priority;

	/* Skip all the non related queues ... */
	for (TxQid = 0, tmpIndex = 0; tmpIndex < WoneIndex; tmpIndex++)
	{
		wdrv_dbGetNumSupWoneQueues	( tmpIndex, &WoneTxQueues );
		TxQid += WoneTxQueues;
	}
	
	wdrv_dbGetNumSupWoneQueues	( WoneIndex, &WoneTxQueues );

	/* Sum counters of the related broadcast queue ... */
	*Val = 0;
	for (tmpIndex = 0; tmpIndex < WoneTxQueues; tmpIndex++, TxQid++)
	{
		wdrv_dbGetAcxQindex	(	WoneIndex,
								tmpIndex,
								&Type,
								&Priority	);
		if (Type)
		{
		/*
			This counter shall increment only when the multicast bit
	            is set in the destination MAC address of a successfully
	            transmitted MSDU.  When operating as a STA in an ESS, where
	            these frames are directed to the AP, this implies having
	            received an acknowledgment to all associated MPDUs. 	
		*/
			*Val += (UINT32)whal_txQueue_GetSuccCount(TxQid);
		}
	}

	return OK;
}


/****************************************************************************
 *                      whal_apiSetMappingKey()
 ****************************************************************************
 * DESCRIPTION:	Set (Add or Delete) Mapping key from the ACX100
 * 
 * INPUTS:	
 *		int   Action		Add(1), ot Delete(0) 
 *		char *MacAddr		The station Mac address
 *		int   KeySize		The size of the key to be Add/Del 
 *		int   Key     		The key to be Add/Del 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
#if 0
int whal_apiSetMappingKey(int Action, char *MacAddr, int KeySize, char *Key)
{
	return whal_acxCmdSetKey(Action, MacAddr, ENTRY_INDEX_IGNORE,
								KeySize, KEY_WEP_ADDR, IGNORE_KEY_ID, Key);
}
#else

int whal_apiSetMappingKey(int Action, char *MacAddr, int KeySize,int KeyType,int Index,int KeyId, char *Key)
{

	return whal_acxCmdSetKey(Action, MacAddr, Index,
								KeySize, KeyType, KeyId, Key);
}

#endif
/****************************************************************************
 *                      whal_apiSetDefaultKey()
 ****************************************************************************
 * DESCRIPTION:	Set (Add or Delete) default key from the ACX100
 * 
 * INPUTS:	
 *		int   Action		Add(1), ot Delete(0) 
 *		int   KeyId			The Key index in the table 
 *		int   KeySize		The size of the key to be Add/Del 
 *		int   Key     		The key to be Add/Del 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetDefaultKey(int Action, int KeyId, int KeySize, char *Key)
{
	char MacAddrDummy[6];

	return whal_acxCmdSetKey(Action, MacAddrDummy, ENTRY_INDEX_IGNORE,
								KeySize, KEY_WEP_DEFAULT, KeyId, Key);
}
 
/****************************************************************************
 *                      whal_apiGetExcludeUnencryptedCount()
 ****************************************************************************
 * DESCRIPTION:	Get the ExcludeUnencrypted counter
 * 
 * INPUTS:	
 *		UINT32 *Val		The value pointer 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiGetExcludeUnencryptedCount(UINT32 *Val)
{
	/*
        "When this attribute is true, the STA shall not indicate at
        the MAC service interface received MSDUs that have the WEP
        subfield of the Frame Control field equal to zero. When this
        attribute is false, the STA may accept MSDUs that have the WEP
        subfield of the Frame Control field equal to zero. The default
        value of this attribute shall be false."
	*/
	*Val = 0;

	return OK;
}

/****************************************************************************
 *                      whal_apiGetUnencryptableCount()
 ****************************************************************************
 * DESCRIPTION:	Get UnencryptableCount counter
 * 
 * INPUTS:	
 *		UINT32 *Val		The value pointer 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiGetUnencryptableCount(UINT32 *Val)
{
	/*
        "This counter shall increment when a frame is received with
        the WEP subfield of the Frame Control field set to one and the
        WEPOn value for the key mapped to the TA's MAC address
        indicates that the frame should not have been encrypted or
        that frame is discarded due to the receiving STA not
        implementing the privacy option."
	*/
	ACXStatisticsStruct StatElem;
	UINT32 UnencryptableCount;

	/* Use the KeyNotFound counter from the statistics */
	whal_acxConfigRead_Statistics ( &StatElem, NULL, NULL);
	UnencryptableCount = StatElem.wep.WepKeyNotFound;

	*Val = UnencryptableCount;

	return OK;
}

/****************************************************************************
 *                      whal_apiGetIcvErrorCount()
 ****************************************************************************
 * DESCRIPTION:	Get the IcvError counter
 * 
 * INPUTS:	
 *		UINT32 *Val		The value pointer 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiGetIcvErrorCount(UINT32 *Val)
{
	/*
        "This counter shall increment when a frame is received with the
        WEP subfield of the Frame Control field set to one and the value
        of the ICV as received in the frame does not match the ICV value
        that is calculated for the contents of the received frame."
	*/
	ACXStatisticsStruct StatElem;

	whal_acxConfigRead_Statistics ( &StatElem, NULL, NULL);
	*Val = StatElem.wep.WepDecryptFail;

	return OK;
}


/****************************************************************************
 *                      whal_apiSetDefaultKeyId()
 ****************************************************************************
 * DESCRIPTION:	Set (Add or Delete) default key from the ACX100
 * 
 * INPUTS:	
 *		int   KeyId			The default Key index in the table 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetDefaultKeyId(int KeyId)
{
	return (whal_acxConfigWepDefaultKeyId(KeyId));
}

/****************************************************************************
 *                      whal_apiSetWepExcludeUnencrypted()
 ****************************************************************************
 * DESCRIPTION:	Set the ExcludeUnencrypted flag
 * 
 * INPUTS:	
 *		int   Val			The value of the ExcludeUnencrypted flag
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetWepExcludeUnencrypted(int Val)
{
	whal_ParamsGetWepParams()->dot11ExcludeUnencrypted = Val;
	return OK;
}

/****************************************************************************
 *                      whal_apiSetWepOptionImplemented()
 ****************************************************************************
 * DESCRIPTION:	Set the OptionImplemented flag
 * 
 * INPUTS:	
 *		int   Val			The value of the OptionImplemented flag
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetWepOptionImplemented(int Val)
{
	whal_ParamsGetWepParams()->dot11PrivacyOptionImplemented = Val;
	return OK;
}

/****************************************************************************
 *                      whal_apiSetWepInvoke()
 ****************************************************************************
 * DESCRIPTION:	Set the Invoke flag
 * 
 * INPUTS:	
 *		int   Val			The value of the Invoke flag
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetWepInvoke(int Val)
{
	whal_ParamsGetWepParams()->dot11PrivacyInvoked = Val;
	return OK;
}

/****************************************************************************
 *                      whal_apiSetRadioParams()
 ****************************************************************************
 * DESCRIPTION:	Set radio parameters from WhalNVRAM, if required.
 * 
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetRadioParams(void)
{
   int retCode = OK;
   RadioParam_t  rdp = {0};

   /* Fill the rdp structure using core db */
   wdrv_dbGetRadioParams( &rdp);

   /* ACX configuration if required */
   if( rdp.overrideBitmask) {
      retCode = whal_acxConfigRadioParams( &rdp);
   }
   return retCode;
}

/****************************************************************************
 *                      whal_apiEnableDeviceErrorIntr()
 ****************************************************************************
 * DESCRIPTION:	Enable DEVICE_ERROR interrupt
 * 
 * INPUTS: DeviceError_Callback_T external_DeviceError_Callback		The callback address
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiEnableDeviceErrorIntr(void *external_DeviceError_Callback)
{
	whal_acxIntrSetDeviceErrorCB(external_DeviceError_Callback);
	whal_acxIntrEnable(INTR_DEVICE_ERROR);

	return OK;
}


/****************************************************************************
 *                      whal_apiDisableDeviceErrorIntr()
 ****************************************************************************
 * DESCRIPTION:	Enable DEVICE_ERROR interrupt
 * 
 * INPUTS: None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiDisableDeviceErrorIntr(void)
{
	whal_acxIntrDisable(INTR_DEVICE_ERROR);

	return OK;
}


/*
 * -----------------------------------------------------------------------
 *						4X Support
 * -----------------------------------------------------------------------
 */

/****************************************************************************
 *                      whal_apiEnableTxDataIntr()
 ****************************************************************************
 * DESCRIPTION:	Enable TX_DATA interrupt
 * 
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiEnableTxDataIntr(void)
{
	whal_acxIntrEnable(INTR_TX_DATA);
	whal_acxIntrSetTxDataCB(whal_txHandleTxData /*clean tx queues callback*/);

	return OK;
}


/****************************************************************************
 *                      whal_apiDisableTxDataIntr()
 ****************************************************************************
 * DESCRIPTION:	Disable TX_DATA interrupt
 * 
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiDisableTxDataIntr(void)
{
	whal_acxIntrDisable(INTR_TX_DATA);
	whal_acxIntrSetTxDataCB(whal_txHandleTxData);

	return OK;
}

/****************************************************************************
 *                      whal_apiSetTxDataUserCB()
 ****************************************************************************
 * DESCRIPTION:	Set the user callback for the TX_DATA interrupt
 *
 * INPUTS:	
 *		TxDataUser_Callback_T TxDataUserCB		The callback address
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetTxDataUserCB(TxDataUser_Callback_T TxDataUserCB)
{
	whal_txSetTxDataUserCB(TxDataUserCB);
	return OK;
}


/****************************************************************************
 *                      whal_apiEnable4xFeature()
 ****************************************************************************
 * DESCRIPTION:	Enable 4x support
 * 
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiEnable4xWackFeature(void)
{
	return NOK;
}

/****************************************************************************
 *                      whal_apiDisable4xFeature()
 ****************************************************************************
 * DESCRIPTION:	Disable 4x support
 * 
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiDisable4xWackFeature(void)
{
	return NOK;
}
/****************************************************************************
 *                      whal_apiSetSend4xWackInfo()
 ****************************************************************************
 * DESCRIPTION:	Set the 4X Wack info to send on the ACK
 * 
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetSend4xWackInfo(UINT8 Send4xWackInfo)
{
	whal_acxRegWrite(REG_COMMAND_SCR_PAD0, (UINT32)Send4xWackInfo); 
	return OK;
}

/****************************************************************************
 *                      whal_apiGetSend4xWackInfo()
 ****************************************************************************
 * DESCRIPTION:	Get the 4X Wack info to send on the ACK
 * 
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiGetSend4xWackInfo(UINT8 *Send4xWackInfo)
{
	UINT32 RegisterValue;
	RegisterValue = whal_acxRegRead(REG_COMMAND_SCR_PAD0); 
	*Send4xWackInfo = (UINT8)RegisterValue;

	return OK;
}

/****************************************************************************
 *                      whal_apiSetRecv4xWackInfoCB()
 ****************************************************************************
 * DESCRIPTION:	Set the callback for the return 4X Wack info.
 *
 * INPUTS:	
 *		Recv4xWackInfo_Callback_T Recv4xWackInfoCB		The callback address
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSetRecv4xWackInfoCB(Recv4xWackInfo_Callback_T Recv4xWackInfoCB)
{
	whal_txSetRecv4xWackInfoCB(Recv4xWackInfoCB);
	return OK;
}

/****************************************************************************
 *                      whal_apiEnable4xTemplate()
 ****************************************************************************
 * DESCRIPTION:	Enable sending 4x IE in the beacon/probe resp
 *
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiEnable4xTemplate(void)
{
	UINT8 WoneIndex, TotalWones;

	wdrv_dbGetNumSupWone(&TotalWones);
	
	whal_ParamsGetGenParams(0)->SupportTIcapTemplate = TRUE;
	/* Set the IE fileds */
	whal_ParamsSetElm_TIcapSupport();
	/* reconfigure probe response template (not the beacon) */
	for (WoneIndex = 0;WoneIndex < TotalWones; WoneIndex++)
	{
		whal_acxCmdProbeRespTemplate(WoneIndex);
	}

	return OK;
}

/****************************************************************************
 *                      whal_apiDisable4xTemplate()
 ****************************************************************************
 * DESCRIPTION:	Disable sending 4x IE in the beacon/probe resp
 *
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiDisable4xTemplate(void)
{
	UINT8 WoneIndex, TotalWones;

	wdrv_dbGetNumSupWone(&TotalWones);
	
	whal_ParamsGetGenParams(0)->SupportTIcapTemplate = FALSE;
	/* Reset the IE fileds - all zero !!! */
	whal_ParamsResetElm_TIcapSupport();
	/* reconfigure probe response template (not the beacon) */
	for (WoneIndex = 0;WoneIndex < TotalWones; WoneIndex++)
	{
		whal_acxCmdProbeRespTemplate(WoneIndex);
	}

	return OK;
}



/****************************************************************************
 *                      whal_apiSet4xCwMin()
 ****************************************************************************
 * DESCRIPTION:	Set the 4X CwMin in the ACX100 register
 * 
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiSet4xCwMin(UINT16 CwMin)
{
	whal_acxRegWrite(ACX_CONT_WIND_CFG_REG, (UINT32)(((UINT32)CwMin & ACX_CONT_WIND_MIN_MASK) | ACX_CONT_WIND_MAX)); 
	return OK;
}

/****************************************************************************
 *                      whal_apiGet4xCwMin()
 ****************************************************************************
 * DESCRIPTION:	Set the 4X CwMin from the ACX100 register
 * 
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_apiGet4xCwMin(UINT16 *CwMin)
{
	UINT32 RegisterValue;
	RegisterValue = whal_acxRegRead(ACX_CONT_WIND_CFG_REG) & ACX_CONT_WIND_MIN_MASK; 
	*CwMin = (UINT16)RegisterValue;

	return OK;
}


void whal_apiPrint4xWackParams(void)
{
	UINT8  Send4xWackInfo;
	UINT8  Recv4xWackInfo;
	UINT32 RecvCallbackAddr;

	whal_apiGetSend4xWackInfo(&Send4xWackInfo);
	printf("Send4xWackInfo = 0x%x\n", Send4xWackInfo);

	whal_txGetRecv4xWackParams(&RecvCallbackAddr, &Recv4xWackInfo);
	printf("Recv4xWackInfo = 0x%x, CB=0x%x\n", Recv4xWackInfo, RecvCallbackAddr);
}

/*
 * -----------------------------------------------------------------------
 *						MENU API
 * -----------------------------------------------------------------------
 */



int whal_apiPrintFwVersion(char *buf)
{
	return sprintf(buf,"ACX Firmware Version = %s\n", whal_ParamsGetAcxVersion()->FwVersion);

}

int whal_apiGetFwVersion(char *FwVersionStr)
{
	strcpy(FwVersionStr, whal_ParamsGetAcxVersion()->FwVersion);
	return 0;
}

/*
 * Print Tx information
 * --------------------
 */
int whal_apiPrintTxInfo(int Qid, BOOL QprintEn,char *buf)
{
	return whal_txInfoPrint(Qid, QprintEn,buf);
}

int whal_apiPrintTxQueue(int Qid,char *buf)
{
	UINT8			WoneIndex, TotalWones, WoneTxQueues, TotalQueues;
	int len = 0;

	wdrv_dbGetNumSupWone(&TotalWones);
	for (WoneIndex = 0, TotalQueues = 0; WoneIndex < TotalWones; WoneIndex++)
	{
		wdrv_dbGetNumSupWoneQueues( WoneIndex, &WoneTxQueues );
		TotalQueues +=	WoneTxQueues;
	}
	
	if ((Qid<0) || (Qid>=TotalQueues))
	{
		len +=sprintf(buf +len,"Error: Use Qid between 0 and %d\n", TotalQueues-1);
		return -1;
	}
	return whal_txQueue_Print(Qid,buf);
	
}

int whal_apiPrintTxIndicator(char *buf)
{
	return whal_txIndicatorPrint(buf);

}

int whal_apiPrintTxBufs(void)
{
	return 0;
}

/*
 * Print Rx information
 * --------------------
 */
int whal_apiPrintRxInfo(char *buf)
{
	return whal_rxInfoPrint(buf);
}

int whal_apiPrintRxQueue(char *buf)
{
	return whal_rxInfoPrint(buf);
}

int whal_apiPrintRxBufs(char *buf)
{
	return sprintf(buf,"Not supported in 1130 (MASTER mode).\n");
}

int whal_apiPrintIntr(char *buf)
{
	return whal_acxIntrPrint(buf);
}

int whal_apiPrintMbox(char *buf)
{
	return whal_acxMboxPrint(buf);

}

int whal_apiPrintAcxParams(char *buf)
{
	return whal_ParamsAcxPrintParams(buf);
}


int whal_apiPrintAcxStatistics(char *buf)
{
	ACXStatisticsStruct StatElem;
	UINT32 FcsErrorVal;
	int len = 0;

	whal_acxConfigRead_Statistics( &StatElem, (void *) whal_apiAcxStatCB, (UINT8 *)buf);	


	/* block here and wait for mailbox mechanism wake up */
	if((wdrv_DDA_sleep_timeout((void*)&AcxStatWait, WHAL_PRINT_RES_TIMEOUT)) == 0)
	{
		return sprintf(buf,"whal_apiPrintAcxStatistics:whal not responding");
	}

	len = whal_acxConfigPrint_Statistics(buf);	
		
	whal_apiGetFcsError(&FcsErrorVal);

	len += sprintf(buf+ len,"\n");
	len += sprintf(buf+ len,"FcsError = %d\n", FcsErrorVal);

	return len;
}

int whal_apiPrintRegs(char *buf)
{
	int len =0;

	len+=sprintf(buf+len,"AcxRegsInfo ----------------------------\n");

	len+=sprintf(buf+len,"    0x%04x: Device&VendorId = 0x%08x\n", REG_DEVICE_VENDOR_ID,		whal_acxRegRead(REG_DEVICE_VENDOR_ID));
	len+=sprintf(buf+len,"    0x%04x: Halt_eCpu       = 0x%08x\n", ACX_REG_ECPU_CONTROL,		whal_acxRegRead(ACX_REG_ECPU_CONTROL));
	len+=sprintf(buf+len,"    0x%04x: SenseOnReset    = 0x%08x\n", ACX_SOR_CFG_REG,			whal_acxRegRead(ACX_SOR_CFG_REG));
	len+=sprintf(buf+len,"    0x%04x: SoftReset	      = 0x%08x\n", ACX_REG_SLV_SOFT_RESET ,	whal_acxRegRead(ACX_REG_SLV_SOFT_RESET));
	
	len+=sprintf(buf+len,"    0x%04x: Beacon Seq Num  = 0x%08x\n", ACX_BCN_SEQ_NUM_REG,		whal_acxRegRead(ACX_BCN_SEQ_NUM_REG));
	
	len+=sprintf(buf+len,"    0x%04x: Cmd  Mbox Addr  = 0x%08x\n", REG_COMMAND_MAILBOX_PTR,	whal_acxRegRead(REG_COMMAND_MAILBOX_PTR));
	len+=sprintf(buf+len,"    0x%04x: Info Mbox Addr  = 0x%08x\n", REG_INFO_MAILBOX_PTR,		whal_acxRegRead(REG_INFO_MAILBOX_PTR));

	len+=sprintf(buf+len,"    0x%04x: Interrupt Value = 0x%08x\n", ACX_REG_INTERRUPT_NO_CLEAR,whal_acxRegRead(ACX_REG_INTERRUPT_NO_CLEAR));
	len+=sprintf(buf+len,"    0x%04x: Interrupt Mask  = 0x%08x\n", ACX_REG_INTERRUPT_MASK,	whal_acxRegRead(ACX_REG_INTERRUPT_MASK));

	return len;	
}

int whal_apiRxPromiscuous(int Val)
{
	if (Val == 0)
	{
		whal_acxRegResetBitVal(ACX_RX_CFG_REG, RX_CFG_PROMISCUOUS);
		whal_acxRegSetBitVal(ACX_RX_CFG_REG, RX_CFG_BSSID); /* back to AP */
	}
	else
	{
		whal_acxRegSetBitVal(ACX_RX_CFG_REG, RX_CFG_PROMISCUOUS);
		whal_acxRegResetBitVal(ACX_RX_CFG_REG, RX_CFG_BSSID); /* Any BSSID */
	}
	return 0;
}

int whal_apiModeSniffer(int Val)
{
	if (Val == 1)
	{
		whal_acxCmdDisableTx();
		whal_apiRxPromiscuous(1);
#ifdef TIWLAN_DEBUG
		whal_trcEnable(1);
#endif
	}
	else
	{
		whal_apiRxPromiscuous(0);
		whal_acxCmdEnableTx();
	}
	return 0;
}

void whal_apiPhyWrite(UINT32 PhyRegAddr, UINT32 DataVal)
{
	whal_acxRegWrite(ACX_PHY_ADDR_REG, PhyRegAddr);
	whal_acxRegWrite(ACX_PHY_DATA_REG, DataVal);
	whal_acxRegWrite(ACX_PHY_CTRL_REG, ACX_PHY_REG_WR_MASK);
}

UINT32 whal_apiPhyRead(UINT32 PhyRegAddr,char *buf)
{
	UINT32 DataVal;

	whal_acxRegWrite(ACX_PHY_ADDR_REG, PhyRegAddr);
	whal_acxRegWrite(ACX_PHY_CTRL_REG, ACX_PHY_REG_RD_MASK);
	PAL_sleepLong(100);
	DataVal = whal_acxRegRead(ACX_PHY_DATA_REG);
	return sprintf(buf,"Phy addr = 0x%x, Phy data = 0x%x\n", PhyRegAddr, DataVal);
}

void whal_apiMacWrite(UINT32 MacRegAddr, UINT32 DataVal)
{
	whal_acxRegWrite(MacRegAddr, DataVal);
}

UINT32 whal_apiMacRead(UINT32 MacRegAddr,char *buf)
{
	UINT32 DataVal;

	DataVal = whal_acxRegRead(MacRegAddr);
	return sprintf(buf,"Mac addr = 0x%x, Mac data = 0x%x\n", MacRegAddr, DataVal);
}

int whal_apiDumpEEPROM(char *buf, int buf_len)
{
	UINT16 wAddr;
	UINT8  bVal;
	int len = 0;
	int line_len;
	

	len +=sprintf(buf,"Dump ACX100 EEPROM contents:");
	for( wAddr = 0; wAddr < 0x2ff; wAddr++)
	{
		if(whal_acxEepromReadByte(wAddr,&bVal) == OK)
		{
			line_len = sprintf(buf+len,"\tEEPROM 0x%04X:\t0x%02X.\n",wAddr,bVal);
			len += line_len;
			if(len > buf_len - line_len)
			{
				printf("whal_apiDumpEEPROM error: insufficient buffer space\n");
				return len;
			}
		}
		else
		{
			len +=sprintf(buf+len,"\tERROR: timeout");
			return len;
		}
	}

	return len;
}

int whal_apiReadEEPROM(int addr,char *buf)
{
   UINT8  bVal;
   int iResult ,len =0;

   addr &= 0x07ff;
   iResult = whal_acxEepromReadByte((UINT16)addr,&bVal);
   if(iResult != OK)
   {
      len = sprintf(buf,"\tERROR: read timeout expired");
   }
   else
   {
      len = sprintf(buf,"\tEEPROM-> 0x%04x: 0x%02x\n", addr,(UINT)bVal);
   }

   return len;
}

int  whal_apiWriteEEPROM(int addr, int val)
{
   UINT8  bVerifyVal;
   UINT32 uiResult;

   addr &= 0x07ff;   /* ACX supports up to 2K EEPROM */
   val &= 0xff;

   uiResult = whal_acxEepromWriteByte((UINT16)addr,(UINT8)val);
   if(uiResult != OK)
   {
      puts("\tERROR: write timeout expired");      
      return -1;
   }
   uiResult = whal_acxEepromReadByte((UINT16)addr,&bVerifyVal);
   if(uiResult != OK)
   {
      puts("\tERROR: verify timeout expired");      
      return -1;
   }
   if(bVerifyVal != (UINT8)val)
   {
      printf("\tERROR: EEPROM write 0x%02x read 0x%02x\n",
         val,bVerifyVal); 
   }
   else
   {
      printf("\tWritten 0x%02x to EEPROM address 0x%04x\n",val,addr);
   }
   return 0;
}

int whal_apiPrintFwVersions(char *buf)
{
	return whal_acxProcPrintFwVersions(buf);
}

void whal_apiSelectFwNumber(int Val)
{
	whal_ParamsSetNv((int)&((NvParams_T *)(0))->FwNumber, (UINT8)Val);
}

int whal_apiPrintRadioVersions(char *buf)
{
	return whal_acxProcPrintRadioVersions(buf);
}

void whal_apiSelectRadioNumber(int Val)
{
	whal_ParamsSetNv((int)&((NvParams_T *)(0))->RadioNumber, (UINT8)Val);
}

void whal_apiUseLoadRadio(int Val)
{
	whal_ParamsSetNv((int)&((NvParams_T *)(0))->LoadRadio, (UINT8)Val);
}

void whal_apiSetNvByte(int Offset, int Val)
{
	whal_ParamsSetNv(Offset, (UINT8)Val);
}

void whal_apiTraceEnable(int Val)
{
	whal_ParamsSetNv((int)&((NvParams_T *)(0))->TraceEnable, (UINT8)Val);
#ifdef TIWLAN_DEBUG
	whal_trcEnable(Val);
#endif
}

int whal_apiTracePrint(int Num, int From,char *buf) {
#ifdef TIWLAN_DEBUG
	whal_trcPrint(Num, From,buf);
#else
	sprintf(buf, "Compile with TIWLAN_DEBUG to get this functionality!\n");
#endif
	return 0;
}


int  whal_apiTracePrintRxStat( char *buf ) {
#ifdef TIWLAN_DEBUG
	return whal_trcPrintRxStat(buf);
#else
	return sprintf(buf, "Compile with TIWLAN_DEBUG to get this functionality!\n");
#endif
}


void whal_apiSetQueueSizeType(int Val)
{
	whal_ParamsSetNv((int)&((NvParams_T *)(0))->QueueSizesType, (UINT8)Val);
}

/*
 * ACX100 BIT
 */
int whal_apiAcxBIT_Fcc(int band, int chID, int rate, int mode)
{
   return whal_acxCmdBit_Fcc(band, chID, rate, mode);
}

int whal_apiAcxBIT_Telec(int band, int chID)
{
   return whal_acxCmdBit_Telec(band, chID);
}

int whal_apiAcxBIT_perTxCfg
(
   int   band,
   int   chID,
   int   rate,
   int   mode,
   int   preamble,     /* 0=long 1=short*/
   int   numFrames,
   int   gap,          /* should be >= 200usec */
   int   bodySize,     /* [0..2304] multiples of 4 */
   char *destMac       /* destination MAC address (6 octets) */
)
{
   return whal_acxCmdBit_perTxCfg(
      band, chID, rate, mode, preamble, numFrames, gap, bodySize, destMac);
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int whal_apiAcxBIT_perRxCfg(int band, int chID)
{
   return whal_acxCmdBit_perRxCfg(band, chID);
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int whal_apiAcxBIT_perClrStats(void)
{
   return whal_acxCmdBit_perClrStats();
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int whal_apiAcxBIT_perTxStop(void)
{
   return whal_acxCmdBit_perTxStop();
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int whal_apiAcxBIT_perRxStats(char *buf)
{
    AcxCmd_Test_PER_RX_STATS  Cmd;

/*	printk("\n!!!!befor");*/


	whal_acxCmdBit_perRxStats( (void*)&Cmd, (void *) whal_RxStatStatCB, (UINT8 *)buf);	

	/* block here and wait for mailbox mechanism wake up */
	/* TODOOOO*/
	if((wdrv_DDA_sleep_timeout((void*)&AcxRxStatWait ,WHAL_PRINT_RES_TIMEOUT)) == 0)
	{
		return sprintf(buf,"whal_apiAcxBIT_perRxStats:whal not responding");
	}

/*	printk("\n!!!!after");*/

	return whal_apiAcxBIT_FillRxStatsBuff(buf);
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int whal_apiAcxBIT_perTxStats(char *buf)
{
	AcxCmd_Test_PER_TX_STATS  Cmd;
	
	whal_acxCmdBit_perTxStats( (void*)&Cmd, (void *) whal_TxStatStatCB, (UINT8 *)buf);
	
	/* block here and wait for mailbox mechanism wake up */
	if((wdrv_DDA_sleep_timeout((void*)&AcxTxStatWait ,WHAL_PRINT_RES_TIMEOUT)) == 0)
	{
		return sprintf(buf,"whal_apiAcxBIT_perTxStats:whal not responding");
	}

	return whal_apiAcxBIT_FillTxStatsBuff(buf);
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int whal_apiAcxBIT_ExitTest(void)
{
   return whal_acxCmdBit_Exit();
}


/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
void whal_apiEnableAllInterrupts(void)
{
	whal_acxIntrEnable(INTR_TX_DATA | INTR_RX_TRF | INTR_INFO);
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
void whal_apiDisableAllInterrupts(void)
{
	whal_acxIntrDisable(INTR_TX_DATA | INTR_RX_TRF | INTR_INFO);
}

/* ----------------- */
/* Restart watch dog */
/* ----------------- */

UINT32 mboxFailures = 0;
UINT32 beaconCount = 0;
UINT32 rxFreeMemFailures = 0;
UINT32 txAcxFreeFailures[MAX_NUM_Q_PER_WONE * MAX_WONE_NET] = {0,0,0,0,0,0};
UINT32 deviceError = 0;
UINT32 rxConsFcsFailures = 0;
UINT32 rxStuckFailures=0;
UINT32 PrevRxDescAddr = 0;
UINT32 PacketReceive = FALSE;
BOOL   rxFcsRestartEn = TRUE;

typedef struct
{
	UINT32 mboxFailures;
	UINT32 beaconFailures;
	UINT32 rxFreeMemFailures;
	UINT32 rxConsFcsFailures;
	UINT32 rxConsFcsReset580;
	UINT32 txAcxFreeFailures;
	UINT32 deviceError;
	UINT32 rxStuckFailures;
    UINT32 aciEvents;
} HalRestartStatistics_T;
HalRestartStatistics_T halRestartStatistics = { 0, 0, 0, 0, 0, 0, 0 };

typedef struct
{
	UINT32 spare_b8_acc;
	UINT32 spare_a8_acc;
	UINT32 spare_b7_acc;
	UINT32 spare_a7_acc;
	UINT32 spare_b6_acc;
	UINT32 spare_a6_acc;
	UINT32 spare_b5_acc;
	UINT32 spare_a5_acc;
	UINT32 scr_pad4;
	UINT32 scr_pad8;
} HalFWWAStatistics_T;
HalFWWAStatistics_T halFWWAStatistics = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int whal_apiPrintRestartStatistics(char *buf)
{
	UINT16 consFcsErrWAoption;
	UINT8 rxFreeMemWAoption, bcnNotSentWAoption, txStuckWAoption, aciWAoption, nbWAoption;
	UINT8 en;
	int len = 0;

	wdrv_dbGetEnRestart(&en);
	wdrv_dbGetWAoptions(&rxFreeMemWAoption, &bcnNotSentWAoption,
		&txStuckWAoption, &consFcsErrWAoption, &aciWAoption, &nbWAoption);

	if (en)
		len+=sprintf (buf+len,"HAL FW Watchdog is enabled\n");
	else
		len+=sprintf (buf+len,"HAL FW Watchdog is disabled\n");
	if (rxFreeMemWAoption == 0)
		len+=sprintf (buf+len,"Rx Free Mem workaround is disabled\n");
	else
		len+=sprintf (buf+len,"Rx Free Mem workaround is enabled\n");
	if (bcnNotSentWAoption == 0)
		len +=sprintf(buf+len,"Beacon not sent workaround is disabled\n");
	else
		len +=sprintf(buf+len,"Beacon not sent trigger time is %d second\n", bcnNotSentWAoption);
	if (txStuckWAoption == 0)
		len +=sprintf(buf+len,"Tx Acx free workaround is disabled\n");
	else
		len +=sprintf(buf+len,"Tx Acx free trigger time is %d second\n", txStuckWAoption);
	if (consFcsErrWAoption == 0)
		len +=sprintf(buf+len,"Rx Cons FCS workaround is disabled\n");
	else
		len +=sprintf(buf+len,"Rx Cons FCS trigger threshold is %d\n", consFcsErrWAoption);
    if (aciWAoption == 0)
		len +=sprintf (buf+len,"ACI workaround is disabled\n");
	else
		len +=sprintf (buf+len,"ACI workaround is enabled\n");

	if (nbWAoption == 0)
		len +=sprintf (buf+len,"NB workaround is disabled\n");
	else
		len +=sprintf (buf+len,"NB workaround is enabled\n");


	len +=sprintf(buf+len,"HAL Restart Statistics (Driver)\n");
	len +=sprintf(buf+len,"Mailbox failures     = %d\n", halRestartStatistics.mboxFailures);
	len +=sprintf(buf+len,"Beacon failures      = %d\n", halRestartStatistics.beaconFailures);
	len +=sprintf(buf+len,"Rx Free Mem failures = %d\n", halRestartStatistics.rxFreeMemFailures);
	len +=sprintf(buf+len,"Rx Stuck    failures = %d\n", halRestartStatistics.rxStuckFailures);
	len +=sprintf(buf+len,"Rx Cons FCS failures = %d\n", halRestartStatistics.rxConsFcsFailures);
    len +=sprintf(buf+len,"Rx Cons FCS Reset 580 = %d\n", halRestartStatistics.rxConsFcsReset580);
	len +=sprintf(buf+len,"Tx Acx Free failures = %d\n", halRestartStatistics.txAcxFreeFailures);
    len +=sprintf(buf+len,"ACI events 		  = %d\n", halRestartStatistics.aciEvents);
	len +=sprintf(buf+len,"Device Error         = %d\n", halRestartStatistics.deviceError);
	len +=sprintf(buf+len,"HAL Restart Statistics (FW)\n");
	len +=sprintf(buf+len,"SPARE_B8 (ACC) = %d\n", halFWWAStatistics.spare_b8_acc + whal_acxRegRead(SPARE_B8));
	len +=sprintf(buf+len,"SPARE_A8 (ACC) = %d\n", halFWWAStatistics.spare_a8_acc + whal_acxRegRead(SPARE_A8));
	len +=sprintf(buf+len,"SPARE_B7 (ACC) = %d\n", halFWWAStatistics.spare_b7_acc + (whal_acxRegRead(SPARE_B7) & 0x0000FFFF));
	len +=sprintf(buf+len,"SPARE_A7 (ACC) = %d\n", halFWWAStatistics.spare_a7_acc + (whal_acxRegRead(SPARE_A7) & 0x0000FFFF));
	len +=sprintf(buf+len,"SPARE_B6 (ACC) = %d\n", halFWWAStatistics.spare_b6_acc + (whal_acxRegRead(SPARE_B6) & 0x000000FF));
	len +=sprintf(buf+len,"SPARE_A6 (ACC) = %d\n", halFWWAStatistics.spare_a6_acc + (whal_acxRegRead(SPARE_A6) & 0x000000FF));
	len +=sprintf(buf+len,"SPARE_B5 (ACC) = %d\n", halFWWAStatistics.spare_b5_acc + (whal_acxRegRead(SPARE_B5) & 0x0000000F));
	len +=sprintf(buf+len,"SPARE_A5 (ACC) = %d\n", halFWWAStatistics.spare_a5_acc + (whal_acxRegRead(SPARE_A5) & 0x0000000F));
	len +=sprintf(buf+len,"SCR_PAD4 (ORED)= %08x\n", halFWWAStatistics.scr_pad4   | whal_acxRegRead(SCR_PAD4));
	len +=sprintf(buf+len,"SCR_PAD8 (ORED)= %08x\n", halFWWAStatistics.scr_pad8   | whal_acxRegRead(SCR_PAD8));
	
	return len;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
void whal_apiUpdateFWWAStatistics(void)
{
	halFWWAStatistics.spare_b8_acc += whal_acxRegRead(SPARE_B8);
	halFWWAStatistics.spare_a8_acc += whal_acxRegRead(SPARE_A8);
	halFWWAStatistics.spare_b7_acc += whal_acxRegRead(SPARE_B7) & 0x0000FFFF;
	halFWWAStatistics.spare_a7_acc += whal_acxRegRead(SPARE_A7) & 0x0000FFFF;
	halFWWAStatistics.spare_b6_acc += whal_acxRegRead(SPARE_B6) & 0x000000FF;
	halFWWAStatistics.spare_a6_acc += whal_acxRegRead(SPARE_A6) & 0x000000FF;
	halFWWAStatistics.spare_b5_acc += whal_acxRegRead(SPARE_B5) & 0x0000000F;
	halFWWAStatistics.spare_a5_acc += whal_acxRegRead(SPARE_A5) & 0x0000000F;
	halFWWAStatistics.scr_pad4     |= whal_acxRegRead(SCR_PAD4);
	halFWWAStatistics.scr_pad8     |= whal_acxRegRead(SCR_PAD8);
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
void whal_apiIncrementDeviceErrorCounter(void)
{
	halRestartStatistics.deviceError++;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
void whal_apiRestartWatchDog_PacketReceived(void)
{
	unsigned long flags;

	PAL_intrLock(flags);
	rxFreeMemFailures = 0;
	rxConsFcsFailures = 0;
    PacketReceive = TRUE;
	rxFcsRestartEn = TRUE;
	PAL_intrUnLock(flags);
}


/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
void whal_apiRestartWatchDog_Restart(void)
{
	int i;

	mboxFailures = 0;
	beaconCount = 0;
	rxFreeMemFailures = 0;
	rxConsFcsFailures = 0;
	rxStuckFailures = 0;
	PrevRxDescAddr = 0;
    
	for (i=0;i<MAX_NUM_Q_PER_WONE * MAX_WONE_NET;i++)
        txAcxFreeFailures[i] = 0;
    PacketReceive = FALSE;
}

static BOOL whal_apiTrackTxStuck( UINT8 txStuckWAoption)
{
   int rc = FALSE;
   unsigned long flags;

   /* Check Tx Stuck, return FALSE if txStuckWAoption or Tx OK, else returns TRUE */
   if( txStuckWAoption != 0)
   {
      TxQueueId_T   TxQid;
	  TxQueueId_T  LocalQid;
      UINT8         WoneIndex, TotalWones, WoneTxQueues;

		wdrv_dbGetNumSupWone(&TotalWones);
		for( WoneIndex = 0, TxQid = 0 ; WoneIndex < TotalWones; WoneIndex++)
		{
         
			wdrv_dbGetNumSupWoneQueues	( WoneIndex, &WoneTxQueues );
			for( LocalQid = 0 ; LocalQid < WoneTxQueues ; LocalQid++, TxQid++)
			{
				if( whal_txQueue_GetNumFree(TxQid) == 0 )
				{
					/* fail, advance static counter*/
					PAL_intrLock(flags);  /* START OF CRITICAL SECTION */
                    ++txAcxFreeFailures[TxQid];
					PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */

					if (txAcxFreeFailures[TxQid] >= txStuckThreshold) 
						rc = TRUE;
					
				}
			}
		}
   }
   return rc;

}/* END whal_apiTrackTxStuck() */

static BOOL whal_apiTrackBeaconStuck( UINT8 bcnNotSentWAoption, UINT beaconModulo)
{
   /* Check Beacon Stuck, return FALSE if bcnNotSentWAoption or
    * Beacons are OK, else returns TRUE
    */
   BOOL    beaconStuck = FALSE;
   if( bcnNotSentWAoption != 0 && beaconModulo == 0)
   {
      /* read beacon sequence counter, compare it to previous one */
	   UINT32  currBeaconCount = whal_acxRegRead( BCN_SEQ_NUM);
      if(  currBeaconCount != beaconCount ) /* AP sends beacons */
          beaconCount = currBeaconCount;
      else
		  beaconStuck = TRUE;
      
   }

   return beaconStuck;
}/* END whal_apiTrackBeaconStuck() */


/* whal_apiDetectAciTrigger()
 * Checks if interference state machine should transit to ACI state  
 * If ACI WA option is disabled allways returns FALSE
 * O/W 1. Detects ACI event: ACI pkt counter value > ACI rate threshold
 *     2. Returns TRUE ACI events occured aciEventsThreshold times continiously
 */
BOOL whal_apiDetectAciTrigger( UINT8 aciWAoption, UINT32 aciPktCount, UINT32 aciEventsThresh)
{
	static UINT32 aciEventsCount = 0;
	BOOL   retVal = FALSE;

	if (aciWAoption != 0) {
		
		UINT32 aciPktThreshold;
		
		wdrv_dbGetAciRateThreshold(&aciPktThreshold);

		if (aciPktCount > aciPktThreshold) 
            aciEventsCount ++;
		else
			aciEventsCount = 0;

		if (aciEventsCount >= aciEventsThresh)
		{
			aciEventsCount = 0;
			retVal = TRUE;
		}
	}

	return retVal;
}

/* whal_apiCheckIfMediumBusy()
 * Checks if medium is busy
 * Returns TRUE if mediumUsage/usageInterval > busyMediumThreshold 
 */
BOOL whal_apiCheckIfMediumBusy(UINT32 mediumUsage, UINT32 usageInterval)
{
	
    UINT32 busyMediumThresh;

	wdrv_dbGetMediumBusyThresh(&busyMediumThresh); /* stored in percents */
	
	/* mediumUsage/usageInterval > busyMediumThresh/100 */
	if ( mediumUsage * 100 > busyMediumThresh * usageInterval ) 
		return TRUE;
	else	
		return FALSE;
}

/* checkIfNbStateEnabled ()
 * Checks if NB state exists in interference state machine
 * Returns FALSE if at least one of the following holds:
 * 1. NB WA option is disabled
 * 2. num of Wones > 1
 * 3. dot11 mode is A or B-only   
 */
static BOOL checkIfNbStateEnabled(UINT8 nbWAopt)
{
	UINT8 numWones, dot11mode;

	wdrv_dbGetNumSupWone(&numWones);
	wdrv_dbGetDot11Mode(0, &dot11mode);

	if ( !nbWAopt || (numWones > 1) || (dot11mode == DOT11_MODE_B_ONLY) ||
		  (dot11mode == DOT11_MODE_A_ONLY) ||(dot11mode == DOT11_MODE_B_ONLY_PLUS)  )
		return FALSE;	
	else
		return TRUE;

}

static void disableOFDMreceiver(void)
{
    /* If intially configured rx mode is not saved yet */
	if (!saveCurRXmode)
	{
		WlanElm_RxDot11Mode_T ie;
		/* interrogate current rx dot11 mode and set the CCK-only mode in a CALLBACK*/
		whal_acxConfigReadRXdot11Mmode(&ie, (void *)interfWD_saveAndSetRxDot11ModeCB, (UINT8 *)&ie);
	}
	else
		/*configure the mode to CCK only */
		whal_apiSetRxDot11mode(RX_DOT11_MODE_CCK);
}

static void enableOFDMreceiver(void)
{
	if (saveCurRXmode) 
	{
		whal_apiSetRxDot11mode(saveCurRXmode);
	}
}

void whal_apiForceDot11Bmode(void)
{
	disableOFDMreceiver();
}

void whal_apiRestoreDot11mode(void)
{
	 enableOFDMreceiver();	
}

static void whal_apiRestoreERPprotect(void)
{
		UINT8 curERPmode;
		BOOL curERPnonPresent;
		
		/* update the DB in runtime to  ERP mode as stored in NVRAM*/
		wdrv_dbSetERPprotectionMode(0 , savedConfigERPmode);

		/* Restore current ERP mode and non ERP present bits according to configured ERP mode*/
		if (savedConfigERPmode == PROTECTION_MODE_ENABLED)
		{
			curERPmode = TRUE;
		}
		else 
		{
			if((savedConfigERPmode == PROTECTION_MODE_LOCAL_STATIONS) ||
			   (savedConfigERPmode == PROTECTION_MODE_DYNAMIC       ) ||
			   (savedConfigERPmode == PROTECTION_MODE_ENHANCED_DYNAMIC))
			
				curERPmode = (savedCurERPmode ? TRUE:FALSE) ;	
			else
				curERPmode = FALSE;
		}
		/* synchronize with current TX DB and update tim and probe response templates*/
		/* reminder: if we're in NB state it means that numWones = 1*/
		wdrv_dbGetCurrentERPNonPresent(&curERPnonPresent);
		whal_apiSetERPProtection(0, (curERPnonPresent ? TRUE : FALSE) , curERPmode);
		
		
}


static void resetTxCounters(void)
 {
	 int i;
	 unsigned long flags;

	 PAL_intrLock(flags);
	 
	 for (i=0;i<MAX_NUM_Q_PER_WONE * MAX_WONE_NET;i++)
        txAcxFreeFailures[i] = 0;
	 
	 PAL_intrUnLock(flags);

 }

 /*Switch to the next state in interference state machine*/
 void switchInterfState(UINT32 *curState, UINT32 nextState, ULONG *curStateTimeout)
 {
	 ULONG nextStateTimeout;
	 
	 if(REPORT_LEVEL(RL_HAL)) 
           wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "INTERFERENCE WA: switched from %s to %s state\n",
							interfStateString[*curState], interfStateString[nextState] );
	 *curState = nextState;

	 switch(nextState)
	 {
	 case INTERF_NORMAL_STATE: 
   			nextStateTimeout = 0;
			break;
	 case INTERF_ACI_STATE:
			wdrv_dbGetAciStateTimeout((UINT32 *)&nextStateTimeout);
			break;
	 case INTERF_NB_STATE:
   			wdrv_dbGetNbStateTimeout((UINT32 *)&nextStateTimeout);
			break;
   
	 }

	 nextStateTimeout = nextStateTimeout * 1000; /* convert to milliseconds */
	 *curStateTimeout = nextStateTimeout;

	 resetTxCounters();
 }



STATUS  handleInterfNormalState(InterfStateInput_T *stateInput, UINT32 *curState, ULONG *curStateTimeout)
{

	if (stateInput->aciTrigger || stateInput->beaconStuck || stateInput->txStuck || forceACI)
	{
		/* In FW: turn of ED, set PD to low threshold */
		UINT8 energyDetect;
		UINT16 pdHighThresh;

		/*turn off ED only if it is ON*/
		wdrv_dbGetEnergyDetect(&energyDetect);
		if (energyDetect){
			whal_apiSetEnergyDetect(ENERGY_DETECT_OFF);
			if(REPORT_LEVEL(RL_HAL)) 
				wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "INTERFERENCE WA: Turned off ED\n");
		}

		wdrv_dbGetPacketDetectHighThresh(&pdHighThresh);
		whal_apiSetPDthreshold(pdHighThresh);
		if(REPORT_LEVEL(RL_HAL)) 
           wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "INTERFERENCE WA: PD set to high threshold\n");

		/* transit to ACI state convert timeout to milliseconds  */
		switchInterfState(curState, INTERF_ACI_STATE, curStateTimeout);
	}
		

	return NO_RECOVERY;
        
}

/* reminder: if we're transiting to NB state it means that numWones = 1*/
static void transitFromACItoNBstate(void)
{ 
	 /* save configured ERP protection mode in static variable*/
	 wdrv_dbGetERPProtectionMode(0, &savedConfigERPmode);
	
	 /* save current non ERP protection flag*/
	 wdrv_dbGetCurrentERPProtectionMode(&savedCurERPmode);
	
	 /* set the ERP protection to Always on mode in DB without rebooting the AP*/
	 /* The ERP configuration in NVRAM remains unchanged*/
	 wdrv_dbSetERPprotectionMode(0 ,PROTECTION_MODE_ENABLED); /* numWones =1 */
	
	 /* synchronize with current TX DB and update tim and probe response templates*/
	 whal_apiSetERPProtection(0, TRUE, TRUE);
			
			
	if(REPORT_LEVEL(RL_HAL)) 
		wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "INTERFERENCE WA: Set ERP protection mode to 'Always On'\n");
	
	/* force B-only mode : may return and continue in a callback !!!!*/
	whal_apiForceDot11Bmode();
	
	if(REPORT_LEVEL(RL_HAL)) 
		wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "INTERFERENCE WA: Forced B-only RX mode\n");


}

static STATUS handleInterfAciState(InterfStateInput_T *stateInput, UINT32 *curState, ULONG *curStateTimeout)
{
	STATUS retVal = NO_RECOVERY;
  
	if (*curStateTimeout < WATCH_DOG_TIMER_VAL) 
		*curStateTimeout = 0;
	else
		*curStateTimeout = *curStateTimeout - WATCH_DOG_TIMER_VAL;

	if (*curStateTimeout == 0) 
	{  /* current state expired - return to NORMAL state*/
	
		UINT8  energyDetect;
		UINT16 pdLowThresh;

		if(REPORT_LEVEL(RL_HAL)) 
				wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "INTERFERENCE WA: ACI state expired\n");
		/* set ED as configured */
		wdrv_dbGetEnergyDetect(&energyDetect);
		if (energyDetect)
		{
			whal_apiSetEnergyDetect(ENERGY_DETECT_ON);
			if(REPORT_LEVEL(RL_HAL)) 
				wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "INTERFERENCE WA: turn ON ED\n");
		}
		/* else ED is already off in FW*/
	   
		/* return PD to low threshold */
		wdrv_dbGetPacketDetectLowThresh(&pdLowThresh);
		whal_apiSetPDthreshold(pdLowThresh);
		if(REPORT_LEVEL(RL_HAL)) 
           wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "INTERFERENCE WA: PD set to low threshold\n");

		/* switch to NORMAL state */
		switchInterfState(curState, INTERF_NORMAL_STATE, curStateTimeout);
	   
		retVal = NO_RECOVERY;
	}

	else if ( stateInput->beaconStuck || stateInput->txStuck || forceNB) 
	{
			/* if NB state exists in state machine transit to NB state : B-only + ERP on*/
			if ( stateInput->nbStateEnabled) 
			{
			    transitFromACItoNBstate();
				switchInterfState(curState, INTERF_NB_STATE, curStateTimeout);
				retVal = NO_RECOVERY;

			}
			/* if NB state doesn't exist - try to transit  to NORMAL state and 
			to perform a recovery if medium is not busy*/
			else if (!stateInput->mediumBusy || forceRecovery)
				{
					switchInterfState(curState, INTERF_NORMAL_STATE, curStateTimeout);
					retVal = RECOVERY;
				}
				else /*If medium is busy there is nothing to do - stay as is*/
				{
					if(REPORT_LEVEL(RL_HAL))
						 wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "INTERFERENCE WA: Medium is busy. Couldn't perform recovery\n");
				}
		
	}

	return retVal;

}

static STATUS handleInterfNbState(InterfStateInput_T *stateInput, UINT32 *curState, ULONG *curStateTimeout)
{
	STATUS retVal = NO_RECOVERY;

   if (*curStateTimeout < WATCH_DOG_TIMER_VAL) 
		*curStateTimeout = 0;
	else
		*curStateTimeout = *curStateTimeout - WATCH_DOG_TIMER_VAL;

	if (*curStateTimeout == 0) 
	{  /* current state expired - return to ACI state: resore dot11 mode + reset ERP */

	    if(REPORT_LEVEL(RL_HAL)) 
				wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "INTERFERENCE WA: NB state expired\n");

		/* restore dot11 mode as configured */
		whal_apiRestoreDot11mode(); 
		if(REPORT_LEVEL(RL_HAL)) 
			wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "INTERFERENCE WA: Restored configured dot11 mode\n");

		/* reset ERP protection as configured */
		whal_apiRestoreERPprotect();


		if(REPORT_LEVEL(RL_HAL)) 
			wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "INTERFERENCE WA: Reset ERP protection mode as configured\n");

		/* switch to ACI state */
		switchInterfState(curState, INTERF_ACI_STATE, curStateTimeout);
		
    }
	else if (stateInput->beaconStuck || stateInput->txStuck || forceRecovery)
	{
			/* try to transit  to NORMAL state and perform a recovery if medium is not busy*/
			if (!stateInput -> mediumBusy || forceRecovery )
			{
				/* reset ERP protection as configured */
				whal_apiRestoreERPprotect();
				switchInterfState(curState, INTERF_NORMAL_STATE, curStateTimeout);
				retVal = RECOVERY;
			}
			else
			{
				if(REPORT_LEVEL(RL_HAL))
						 wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "INTERFERENCE WA: Medium is busy. Couldn't perform recovery\n");
			}
	}
	
		
	return retVal;
	
}

 /* Implements interference state machine */
static void actInterfStateMachine(void)
{
	static UINT32 curState = INTERF_NORMAL_STATE;
	static ULONG  curStateTimeout = 0;

	STATUS  recovFlag = NO_RECOVERY;
	BOOL curERPmode;

	/*if it is a very first time after init or recovery, counters
	 may contain invalid values -  return*/
	if (acxCountersValid == FALSE) {
		acxCountersValid = TRUE;
		return;
	}
    
	if(REPORT_LEVEL(RL_HAL)) 
			wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "INTERFERENCE WA: current state: %s. ACI counter = %u\n",
							 interfStateString[curState], interfWDparams.aciCount);

	switch (curState) {
	
	case INTERF_NORMAL_STATE:
			 {
				 UINT32 aciEventsThresh;
				 wdrv_dbGetAciEventsThreshold(&aciEventsThresh);
				 stateInput.aciTrigger = whal_apiDetectAciTrigger(waOptions.aci, interfWDparams.aciCount, aciEventsThresh);
				 recovFlag = handleInterfNormalState(&stateInput, &curState, &curStateTimeout);
			 }
		
			 break;
	case INTERF_ACI_STATE:
			 
			 stateInput.aciTrigger = FALSE;
			 stateInput.mediumBusy = whal_apiCheckIfMediumBusy(interfWDparams.mediumUsage, interfWDparams.usageIntvl);
			 stateInput.nbStateEnabled = checkIfNbStateEnabled(waOptions.nb);
			 recovFlag = handleInterfAciState(&stateInput,  &curState, &curStateTimeout);
		 
			 break;
	case INTERF_NB_STATE:
			 stateInput.aciTrigger = FALSE;
			 stateInput.mediumBusy = whal_apiCheckIfMediumBusy(interfWDparams.mediumUsage, interfWDparams.usageIntvl);
			 /* Ensure that current ERP protection wasn't overwritten by dynamic algorithm manager */
			 wdrv_dbGetCurrentERPProtectionMode(&curERPmode);
			 if (curERPmode == FALSE) 
				 whal_apiSetERPProtection(0, TRUE, TRUE);

			 recovFlag = handleInterfNbState(&stateInput, &curState, &curStateTimeout);
			 
			 break;
	}

	/* Update WA statistics*/
	if (stateInput.beaconStuck) 
	{
		if(REPORT_LEVEL(RL_HAL)) 
			wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "INTERFERENCE WA:  detected beacon stuck\n");
		halRestartStatistics.beaconFailures++;
	}
	if (stateInput.txStuck){
	
		halRestartStatistics.txAcxFreeFailures++;
		if(REPORT_LEVEL(RL_HAL)) 
			wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "INTERFERENCE WA:  detected tx stuck\n");
	}
	if (stateInput.aciTrigger) 
	{
		halRestartStatistics.aciEvents++;
		if(REPORT_LEVEL(RL_HAL)) 
			wdrv_utilsReport(RL_HAL, PRI_EMERGENCY, "INTERFERENCE WA: detected ACI event\n");
	}
		

	if (recovFlag == RECOVERY)
	{
		unsigned long flags;

		printf ("@@@@@@@@@@@@ RECOVERY: EXTERNAL INTERFERENCE @@@@@@@@@@@@\n");				
		PAL_intrLock(flags);
        whal_apiRestartWatchDog_Restart();
      	PAL_intrUnLock(flags);
		if(watchdogFailCB)
			watchdogFailCB();
	}

}

static int interfWD_ReadMediumUsageAndProceedCB(BOOL Sts, UINT8 *buf)
{
	AcxElm_MediumUsage_T *ie = (AcxElm_MediumUsage_T *)buf;
	static AcxElm_AciEvent_T    aciElem;
	
    int ret = -1;

    if( Sts == TRUE)
    {
		ret = 0;
	
		interfWDparams.mediumUsage   = ENDIAN_HANDLE_LONG (ie->mediumUsage);
		interfWDparams.usageIntvl	 = ENDIAN_HANDLE_LONG (ie->period);
	   
		/* proceed and interogate next interference parameter */
		/* read ACI counter and proceed int the callback*/
		whal_acxConfigReadAciCounter(&aciElem, (void *)interfWD_ReadAciCountAndProceedCB, (UINT8 *)&aciElem);
		
	}

	return ret;
}

static int interfWD_ReadAciCountAndProceedCB(BOOL Sts, UINT8 *buf)
{
	AcxElm_AciEvent_T *ie = (AcxElm_AciEvent_T *)buf;
	
	int ret = -1;

	if( Sts == TRUE)
	{
		ret = 0;

		interfWDparams.aciCount = ENDIAN_HANDLE_LONG(ie ->aciEventsCounter);

		/* Take action according to the interference state machine */
	    actInterfStateMachine();
	}
   
	return ret;
}

void whal_apiHandleInterfernceWD(void)
{
	static UINT   beaconModulo=0; /* a counter which wraps arround if two beacon intervals passed */
	static AcxElm_MediumUsage_T muElem;


	/* update beacon modulo */
    if ( ++beaconModulo >= wd_beacon_limiter)
		beaconModulo = 0;

	stateInput.beaconStuck = whal_apiTrackBeaconStuck(waOptions.bcnNotSent, beaconModulo);
	stateInput.txStuck = whal_apiTrackTxStuck(waOptions.txStuck);

	/* It is important to read the ACI and medium usage counters each time the watchdog handler is called
	   even if they are not considered in current state, in order to clear them */

	/* Read medium Usage counters and proceed in callback*/
	whal_acxConfigReadMediumUsage(&muElem, (void *)interfWD_ReadMediumUsageAndProceedCB, (UINT8 *)&muElem);

	 /* Continue in a callback*/
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
void whal_apiCheckFwVerForRestart(BOOL replyRcved, UINT8 *replyBuf)
{
	if(replyRcved != TRUE)
	{
		mboxFailures++;
		if (mboxFailures >= 2)
		{
			printf ("@@@@@@@@@@@@ BUILT_IN_TEST: READ MBOX FAIL @@@@@@@@@@@@\n");
			halRestartStatistics.mboxFailures++;
			whal_apiRestartWatchDog_Restart();
			if(watchdogFailCB)
				watchdogFailCB();
		}			
	}
	else
	{
		mboxFailures = 0;
	}
}



UINT32 rxStuckWAoption=1;
UINT32 printRxInfo=0;

/*******************************************************************************
DESCRIPTION:	Handles hardware falures Work Arounds   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
STATUS whal_apiRestartCheckIfNeeded(watchdogFailCB_T coreCB)
{
    static  UINT rxResetRegCount = 0;
    static  UINT32 lastFcsCounter = 0;	
	unsigned long flags;

	wdrv_dbGetWAoptions(&waOptions.rxFreeMem, &waOptions.bcnNotSent,&waOptions.txStuck,
						&waOptions.consFcsErr, &waOptions.aci, &waOptions.nb);

	/* FW command response tracking */
	
	watchdogFailCB = coreCB;

	/*whal_acxConfigRead_FwVersionForRestart(whal_apiCheckFwVerForRestart);*/
	whal_acxConfigRead_FwVersionForRestart(whal_apiCheckFwVerForRestart);
	
    /*Handle ACI/NB interference WA */
	whal_apiHandleInterfernceWD();

	/* rx free mem tracking */
	if (waOptions.rxFreeMem != 0)
	{
		if ( whal_acxRegRead(RX_FREE_MEM) == 0 )
		{			
			rxFreeMemFailures++;
			if (rxFreeMemFailures >= 2)
			{
				printf ("@@@@@@@@@@@@ BUILT_IN_TEST: RX FREE MEM @@@@@@@@@@@@\n");
				PAL_intrLock(flags);
				halRestartStatistics.rxFreeMemFailures++;
				whal_apiRestartWatchDog_Restart();
				PAL_intrUnLock(flags);
				return OK;
			}
		}
		else
		{
			rxFreeMemFailures = 0;
		}
	}

	/* rx cons fcs tracking */
	if (waOptions.consFcsErr != 0)
	{
        UINT32  currFcsCounter;
        
        currFcsCounter = whal_acxRegRead(CONS_FCS_ERR_CNT);

        if ((currFcsCounter >= (lastFcsCounter + FcsMacRxResetThreshold))||
            (currFcsCounter >= 0xfffe))
        {
            lastFcsCounter = currFcsCounter;

            if (PacketReceive == FALSE)
			{
	
				rxConsFcsFailures++;
				
				if(rxConsFcsFailures >= waOptions.consFcsErr * 2)
				{
					if (rxResetRegCount <= 2)
					{
						whal_acxRegWrite(RX_RESET,1);
						/* The counter inc adds delay between the reg write */
						rxResetRegCount++;
						whal_acxRegWrite(RX_RESET,0);
						halRestartStatistics.rxConsFcsReset580++;						
						if(REPORT_LEVEL(RL_HAL))
							wdrv_utilsReport(RL_HAL, PRI_EMERGENCY,"@@@@@@@@@@@@ BUILT_IN_TEST: RX CONS FCS RESET REG 580 @@@@@@@@@@@@\n");
					}
					else
					{
						if(rxFcsRestartEn && currFcsCounter>= 0xfffe)
						{
						printf ("@@@@@@@@@@@@ BUILT_IN_TEST: RX CONS FCS FW DL @@@@@@@@@@@@\n");
						halRestartStatistics.rxConsFcsFailures++;
							rxResetRegCount = 0;
							rxFcsRestartEn = FALSE;
						whal_apiRestartWatchDog_Restart();
						return OK;
					}
				}
			}
			}
		}
	    else
		{
			rxResetRegCount =0;
			rxConsFcsFailures = 0;
            lastFcsCounter = currFcsCounter;
		}
        PacketReceive = FALSE;
	}


	/* rx stuck tracking */
	if (rxStuckWAoption != 0)
	{
		UINT32 RxStuck = 0;
		UINT32 RxDescAddr, HostOwn, OwnerPercentage;

		whal_rxMaster_getQueueInfo( &RxDescAddr, &HostOwn, &OwnerPercentage );
		
		if ((RxDescAddr == PrevRxDescAddr) && (HostOwn==0) && (OwnerPercentage > 60))
		{
			RxStuck = 1;
		}
		else
		{
			PrevRxDescAddr = RxDescAddr;
		}


		if(RxStuck)
		{
/*			if(printRxInfo && (rxStuckFailures >= 1))
					whal_rxInfoPrint();*/

			PAL_intrLock(flags);
			rxStuckFailures++;
			if(rxStuckFailures >= 2)
			{
				printf ("@@@@@@@@@@@@ BUILT_IN_TEST: RX STUCK @@@@@@@@@@@@\n");
				halRestartStatistics.rxStuckFailures++;
				whal_apiRestartWatchDog_Restart();
				PAL_intrUnLock(flags);
				return OK;
			}
			PAL_intrUnLock(flags);
		}
		else
		{
			rxStuckFailures = 0;
		}
	}

	return NOK;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int whal_apiAcxBIT_FillRxStatsBuff(char *buf)
{
	char tmp_buf[1024*2];
	int len = 0;
	AcxCmd_Test_PER_RX_STATS *pCmd = (AcxCmd_Test_PER_RX_STATS *) buf;

	/* print results */
	len +=sprintf(tmp_buf +len,"Frame body offset = 0x%04x\n", (UINT)(pCmd->FrameBodyOffset) );
	len +=sprintf(tmp_buf +len,"Frames recieved   = %ul\n"   , (UINT32)pCmd->NumFramesRcvd           );
	len +=sprintf(tmp_buf +len,"Checksum errors   = %ul\n"   , pCmd->NumChecksumErrors       );
	len +=sprintf(tmp_buf +len,"FCS errors        = %ul\n"   , pCmd->NumFCSErrors            );
	len +=sprintf(tmp_buf +len,"PLCP errors       = %ul\n"   , pCmd->NumPLCPErrors           );
	len +=sprintf(tmp_buf +len,"SNR minimum       = %u\n"    , (UINT)(pCmd->SNRMin)          );
	len +=sprintf(tmp_buf +len,"SNR maximum       = %u\n"    , (UINT)(pCmd->SNRMax)          );
	len +=sprintf(tmp_buf +len,"SNR average       = %u\n"    , (UINT)(pCmd->SNRAvg)          );
	len +=sprintf(tmp_buf +len,"Rx level minimum  = %u\n"    , (UINT)(pCmd->RxLevelMin)      );
	len +=sprintf(tmp_buf +len,"Rx level maximum  = %u\n"    , (UINT)(pCmd->RxLevelMax)      );
	len +=sprintf(tmp_buf +len,"Rx level average  = %u\n"    , (UINT)(pCmd->RxLevelAvg)      );

	
	memcpy(buf,tmp_buf,len);

	return len;
}


/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
static int whal_apiAcxBIT_FillTxStatsBuff(char *buf)
{
	char tmp_buf[512];
	int len = 0;
	AcxCmd_Test_PER_TX_STATS *pCmd = (AcxCmd_Test_PER_TX_STATS *) buf;

	/* print results */
	len +=sprintf(tmp_buf +len,"Frame body offset  = 0x%04x\n", (UINT)(pCmd->FrameBodyOffset) );
	len +=sprintf(tmp_buf +len,"Frames transmitted = %ul\n"   , pCmd->NumFramesTx);

	memcpy(buf,tmp_buf,len);
	
	return len;
}

/****************************************************************

  CALL BAC FUNC,handle mailbox answer,
*****************************************************************/
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
static int whal_RxStatStatCB(BOOL Sts,char *buf)
{

	AcxCmd_Test_PER_RX_STATS *pCmd = (AcxCmd_Test_PER_RX_STATS *) buf;

	int ret =-1;
	
	/* print the results */
	if( Sts == TRUE)
	{
		ret =0;
		/* Convert endianess */
		pCmd->FrameBodyOffset  = ENDIAN_HANDLE_WORD(pCmd->FrameBodyOffset);
		pCmd->NumFramesRcvd    = ENDIAN_HANDLE_LONG(pCmd->NumFramesRcvd);
		pCmd->NumChecksumErrors= ENDIAN_HANDLE_LONG(pCmd->NumChecksumErrors);
		pCmd->NumFCSErrors     = ENDIAN_HANDLE_LONG(pCmd->NumFCSErrors);
		pCmd->NumPLCPErrors    = ENDIAN_HANDLE_LONG(pCmd->NumPLCPErrors);
		

	}
	
/*	printk("!!!!! i'm in callback");*/

	wdrv_DDA_wake((void*)&AcxRxStatWait);

    return ret;
}

/****************************************************************************
 *                      whal_apiRestartWatchDog_PacketTransmitted()
 ****************************************************************************
 * DESCRIPTION:	Set the 4X CwMin in the ACX100 register
 * 
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void whal_apiRestartWatchDog_PacketTransmitted(void)
{
    int i;
    for (i=0;i<MAX_NUM_Q_PER_WONE * MAX_WONE_NET;i++)
        txAcxFreeFailures[i] = 0;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
static int whal_TxStatStatCB(BOOL Sts,char *buf)
{
	AcxCmd_Test_PER_TX_STATS *pCmd = (AcxCmd_Test_PER_TX_STATS *)buf;
	int ret =-1;
	if( Sts == TRUE)
	{
		ret =0;
		/* Convert endianess */
		pCmd->NumFramesTx = ENDIAN_HANDLE_LONG(pCmd->NumFramesTx);
	}
	wdrv_DDA_wake((void*)&AcxTxStatWait);
	
	return ret;
}


/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
static int whal_apiAcxStatCB(BOOL Sts, UINT8 *buf)
{	
	ACXStatisticsStruct *pElem = (ACXStatisticsStruct *)buf;

	int ret = -1;
   
	if( Sts == TRUE)
	{
		ret = 0;

		/* Isr */
		pElem->isr.ACXRxProcs         = ENDIAN_HANDLE_LONG(pElem->isr.ACXRxProcs);
		pElem->isr.ACXTxProcs         = ENDIAN_HANDLE_LONG(pElem->isr.ACXTxProcs);
		pElem->isr.Cmd_Cmplt          = ENDIAN_HANDLE_LONG(pElem->isr.Cmd_Cmplt);
		pElem->isr.FIQs               = ENDIAN_HANDLE_LONG(pElem->isr.FIQs);
		pElem->isr.RxHeaders          = ENDIAN_HANDLE_LONG(pElem->isr.RxHeaders);
		pElem->isr.RxCompletes        = ENDIAN_HANDLE_LONG(pElem->isr.RxCompletes);
		pElem->isr.RxMemOverflow      = ENDIAN_HANDLE_LONG(pElem->isr.RxMemOverflow);
		pElem->isr.RxRdys             = ENDIAN_HANDLE_LONG(pElem->isr.RxRdys);
		pElem->isr.IRQs               = ENDIAN_HANDLE_LONG(pElem->isr.IRQs);
		pElem->isr.DecryptDone        = ENDIAN_HANDLE_LONG(pElem->isr.DecryptDone);
		pElem->isr.DMA0Done           = ENDIAN_HANDLE_LONG(pElem->isr.DMA0Done);
		pElem->isr.DMA1Done           = ENDIAN_HANDLE_LONG(pElem->isr.DMA1Done);
		pElem->isr.ACXTxExchComplete  = ENDIAN_HANDLE_LONG(pElem->isr.ACXTxExchComplete);
		pElem->isr.ACXCommands        = ENDIAN_HANDLE_LONG(pElem->isr.ACXCommands);
		pElem->isr.HwPMModeChanges    = ENDIAN_HANDLE_LONG(pElem->isr.HwPMModeChanges);
		pElem->isr.HostAcknowledges   = ENDIAN_HANDLE_LONG(pElem->isr.HostAcknowledges);
		pElem->isr.PCI_PM             = ENDIAN_HANDLE_LONG(pElem->isr.PCI_PM);
		pElem->isr.ACMWakeups         = ENDIAN_HANDLE_LONG(pElem->isr.ACMWakeups);

		/* Rx */
		pElem->rx.RxDroppedFrame	= ENDIAN_HANDLE_LONG(pElem->rx.RxDroppedFrame);
		pElem->rx.RxFcsErr   		= ENDIAN_HANDLE_LONG(pElem->rx.RxFcsErr);
		pElem->rx.RxHdrOverflow		= ENDIAN_HANDLE_LONG(pElem->rx.RxHdrOverflow);
		pElem->rx.RxHWStuck		    = ENDIAN_HANDLE_LONG(pElem->rx.RxHWStuck);
		pElem->rx.RxOutOfMem		= ENDIAN_HANDLE_LONG(pElem->rx.RxOutOfMem);
		pElem->rx.RxXfrHintTrig		= ENDIAN_HANDLE_LONG(pElem->rx.RxXfrHintTrig);
        pElem->rx.RxACIevents		= ENDIAN_HANDLE_LONG(pElem->rx.RxACIevents);
		pElem->rx.RxACIresets		= ENDIAN_HANDLE_LONG(pElem->rx.RxACIresets);


		/* Tx */
		pElem->tx.TxInternalDescOverflow = ENDIAN_HANDLE_LONG(pElem->tx.TxInternalDescOverflow);

		/* Dma */
		pElem->dma.RxDMARequested	= ENDIAN_HANDLE_LONG(pElem->dma.RxDMARequested);
		pElem->dma.RxDMAErrors		= ENDIAN_HANDLE_LONG(pElem->dma.RxDMAErrors);
		pElem->dma.TxDMARequested	= ENDIAN_HANDLE_LONG(pElem->dma.TxDMARequested);
		pElem->dma.TxDMAErrors		= ENDIAN_HANDLE_LONG(pElem->dma.TxDMAErrors);

		/* Wep */
		pElem->wep.dot11DefKeyMIB		= ENDIAN_HANDLE_LONG(pElem->wep.dot11DefKeyMIB);
		pElem->wep.WepAddrKeyCount		= ENDIAN_HANDLE_LONG(pElem->wep.WepAddrKeyCount);
		pElem->wep.WepDecryptFail		= ENDIAN_HANDLE_LONG(pElem->wep.WepDecryptFail);
		pElem->wep.WepDefaultKeyCount	= ENDIAN_HANDLE_LONG(pElem->wep.WepDefaultKeyCount);
		pElem->wep.WepKeyNotFound		= ENDIAN_HANDLE_LONG(pElem->wep.WepKeyNotFound);
        pElem->wep.WepPackets			= ENDIAN_HANDLE_LONG(pElem->wep.WepPackets);
	    pElem->wep.WepInterrupt			= ENDIAN_HANDLE_LONG(pElem->wep.WepInterrupt);


		/* Pwr */
		pElem->pwr.AbortTooShort    = ENDIAN_HANDLE_LONG(pElem->pwr.AbortTooShort);
		pElem->pwr.LPPDBegin        = ENDIAN_HANDLE_LONG(pElem->pwr.LPPDBegin);
		pElem->pwr.Match            = ENDIAN_HANDLE_LONG(pElem->pwr.Match);
		pElem->pwr.Noisy            = ENDIAN_HANDLE_LONG(pElem->pwr.Noisy);
		pElem->pwr.RxStartCtr       = ENDIAN_HANDLE_LONG(pElem->pwr.RxStartCtr);
		pElem->pwr.RxTooShort       = ENDIAN_HANDLE_LONG(pElem->pwr.RxTooShort);
		pElem->pwr.TxStartCtr       = ENDIAN_HANDLE_LONG(pElem->pwr.TxStartCtr);
		pElem->pwr.TxTooShort       = ENDIAN_HANDLE_LONG(pElem->pwr.TxTooShort);
		
		/* Aes */
		pElem->aes.AesDecryptFail      = ENDIAN_HANDLE_LONG(pElem->aes.AesDecryptFail);
		pElem->aes.AesDecryptInterrupt = ENDIAN_HANDLE_LONG(pElem->aes.AesDecryptInterrupt);
		pElem->aes.AesDecryptPackets   = ENDIAN_HANDLE_LONG(pElem->aes.AesDecryptPackets);
		pElem->aes.AesEncryptFail      = ENDIAN_HANDLE_LONG(pElem->aes.AesEncryptFail);
		pElem->aes.AesEncryptInterrupt = ENDIAN_HANDLE_LONG(pElem->aes.AesEncryptInterrupt);
		pElem->aes.AesEncryptPackets   = ENDIAN_HANDLE_LONG(pElem->aes.AesEncryptPackets);
		
		/* Mic */
		pElem->mic.MicCalcFailure = ENDIAN_HANDLE_LONG(pElem->mic.MicCalcFailure);
		pElem->mic.MicRxPkts      = ENDIAN_HANDLE_LONG(pElem->mic.MicRxPkts);
		
		/* Event */
		pElem->event.calibration      = ENDIAN_HANDLE_LONG(pElem->event.calibration);
		pElem->event.heartbeat        = ENDIAN_HANDLE_LONG(pElem->event.heartbeat);
		pElem->event.oomLate          = ENDIAN_HANDLE_LONG(pElem->event.oomLate);
		pElem->event.phyTransmitError = ENDIAN_HANDLE_LONG(pElem->event.phyTransmitError);
		pElem->event.rxMemEmpty       = ENDIAN_HANDLE_LONG(pElem->event.rxMemEmpty);
		pElem->event.rxMismatch       = ENDIAN_HANDLE_LONG(pElem->event.rxMismatch);
		pElem->event.rxPool           = ENDIAN_HANDLE_LONG(pElem->event.rxPool);
		pElem->event.txStuck          = ENDIAN_HANDLE_LONG(pElem->event.txStuck);

	}
    
	wdrv_DDA_wake((void*)&AcxStatWait);

    return ret;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int  whal_apiRadarDetSendStopCmd(UINT16 *event_diff,UINT8 *number_of_events)
{
	/* send stop scan channel command */
	return whal_acxCmdSendRDStopCmd(event_diff,number_of_events);
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int  whal_apiRadarDetSendStartCmd(UINT8 channel ,UINT8 band)
{
	/* send start scan channel command */
	return whal_acxCmdSendRDStartCmd(channel,band);
}


/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
static int whal_apiFcsErrCB(BOOL Sts, UINT8 *buf)
{	
	WlanElm_FcsError_T *Elem = (WlanElm_FcsError_T *)buf;

	int ret = -1;

	if( Sts == TRUE)
	{
		ret = 0;

		Elem -> FcsErrorCount = ENDIAN_HANDLE_LONG(Elem -> FcsErrorCount);

	}

	wdrv_DDA_wake((void*)&FcsErrWait);

	return ret;
}

/*******************************************************************************
DESCRIPTION:	    ACI counter interrogate callback
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
static int  whal_apiACIcountCB(BOOL Sts, UINT8 *buf)
{
	AcxElm_AciEvent_T *ie = (AcxElm_AciEvent_T *)buf;
	
	int ret = -1;

	if( Sts == TRUE)
	{
		ret = 0;

		ie ->aciEventsCounter = ENDIAN_HANDLE_LONG(ie ->aciEventsCounter);
	   
	}

	wdrv_DDA_wake((void*)&ACIcountWait);

	return ret;
}

/*******************************************************************************
DESCRIPTION:	    RX dot11 mode callback
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
static int  whal_apiRxDot11ModeCB(BOOL Sts, UINT8 *buf)
{
	WlanElm_RxDot11Mode_T *ie = (WlanElm_RxDot11Mode_T *)buf;
	
	int ret = -1;

	if( Sts == TRUE)
	{
		ret = 0;

		ie ->dot11RxMode = ENDIAN_HANDLE_LONG(ie ->dot11RxMode);

	}

	wdrv_DDA_wake((void*)&rxDot11modeWait);

	return ret;
}

/*******************************************************************************
DESCRIPTION:	    Medium usage statistics interrogate callback
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
static int  whal_apiMediumUsageCB(BOOL Sts, UINT8 *buf)
{
	AcxElm_MediumUsage_T *ie = (AcxElm_MediumUsage_T *)buf;

	int ret = -1;

	if( Sts == TRUE)
	{
		ret = 0;

     ie->mediumUsage = ENDIAN_HANDLE_LONG (ie->mediumUsage);
	 ie->period	 = ENDIAN_HANDLE_LONG (ie->period);

	}

	wdrv_DDA_wake((void*)&mediumUsageWait);

	return ret;
}
 
/*******************************************************************************
DESCRIPTION:	    Energy detect threshold interrogate callback
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
static int  whal_apiEnergyDetectCB(BOOL Sts, UINT8 *buf)
{
	WlanElm_EDThreshold_T *ie = (WlanElm_EDThreshold_T *)buf;
	
	int ret = -1;

	if( Sts == TRUE)
	{
		ret = 0;

     ie->dot11EDThreshold = ENDIAN_HANDLE_LONG (ie->dot11EDThreshold);

	}

	wdrv_DDA_wake((void*)&edWait);

	return ret;
}

/*******************************************************************************
DESCRIPTION: Packet detect threshold interrogate callback
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
static int  whal_apiPacketDetectCB(BOOL Sts, UINT8 *buf)
{
	WlanElm_PDThreshold_T *ie = (WlanElm_PDThreshold_T *)buf;
	
	int ret = -1;

	if( Sts == TRUE)
	{
		ret = 0;

     ie->dot11PDThreshold = ENDIAN_HANDLE_LONG (ie->dot11PDThreshold);

	}

	wdrv_DDA_wake((void*)&pdWait);

	return ret;
}

 

/*******************************************************************************
DESCRIPTION:	   HDK version interrogate callback
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
static int whal_apiHDK_VersionCB(BOOL Sts, UINT8 *buf)
{	
	WlanElm_HDKModuleVersion_t *pElem = (WlanElm_HDKModuleVersion_t *)buf;

	int ret = -1;
	
	if( Sts == TRUE)
	{
		ret = 0;
		

		pElem ->majorVersion = ENDIAN_HANDLE_LONG(pElem ->majorVersion);
		pElem ->minorVersion = ENDIAN_HANDLE_LONG(pElem ->minorVersion);
		pElem ->minorMinorVersion = ENDIAN_HANDLE_LONG(pElem ->minorMinorVersion);
		pElem ->hdkModuleReleaseType = ENDIAN_HANDLE_LONG(pElem ->hdkModuleReleaseType);

	}

	wdrv_DDA_wake((void*)&hdkVersionWait);

	return ret;
}

/*******************************************************************************
DESCRIPTION: RX dot11 mode interrogate callback: saves current RX dot11 mode
	         in a variable and configures RX dot11 mode to CCK-only 																					
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/

static int  interfWD_saveAndSetRxDot11ModeCB(BOOL Sts, UINT8 *buf)
{
	WlanElm_RxDot11Mode_T *ie = (WlanElm_RxDot11Mode_T *)buf;
	
	int ret = -1;

	if( Sts == TRUE)
	{
		ret = 0;
		ie->dot11RxMode = ENDIAN_HANDLE_LONG (ie->dot11RxMode);
		
		saveCurRXmode = ie->dot11RxMode;
	}



	/*configure the mode to CCK only */
	whal_apiSetRxDot11mode(RX_DOT11_MODE_CCK);

	return ret;
}
