/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalInterrupt.c
 *   PURPOSE: Handle all interrupt issues (ISR, trigger)
 *
 ****************************************************************************/

#include "whalCommon.h"
#include "whal.h"
#include "whalRx.h"
#include "whalTx.h"
#include "whalInterrupt.h"
#include "whalAcxAccess.h"
#include "whalAcxRegs.h"
#include "whalAcxMbox.h"

/****************************************************************************
 *                      OBJECT DATA BASE + INTERNALS
 ****************************************************************************
 * 
 ****************************************************************************/
#define whal_acxIntrAck(Intr)	(whal_acxRegWrite(ACX_REG_INTERRUPT_ACK, Intr))
#define INITIAL_INTERRUPTS INTR_INFO

#define IS_1BIT_MASK_ON( field, bitmask)   ((field) & (bitmask))


/* Place all interrupt related data in the same area */
static struct whalIntMCB {
   /* callbacks */
   RxData_Callback_T RxData_Callback;
   RxData_Callback_T TxData_Callback;
   DeviceError_Callback_T DeviceError_Callback;

   /* data */
   UINT32 TotalIntrCounter;
   UINT32 RxDataIntrCounter;
   UINT32 RxTrfIntrCounter;
   UINT32 TxDataIntrCounter;
   UINT32 MboxIntrCounter;
   UINT32 InfoIntrCounter;

   /* control */
   UINT32 RxThresholdCounter;
   UINT32 RxThresholdValue;

   UINT32 LastIntrValue;
   UINT32 InterruptEnabled;
} whalIntMCB = {0};

/****************************************************************************
 *                      whal_acxIntrHandler()
 ****************************************************************************
 * DESCRIPTION:	Handle interrupt from the ACX100
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 *
 ****************************************************************************/
void whal_acxIntrHandler(void)
{
    register UINT32 lastIntr;
    register UINT32 intrEnabled;

	/* Disable interrupts */
	whal_acxRegWrite (ACX_REG_INTERRUPT_MASK, ACX_INTR_ALL );

	whalIntMCB.TotalIntrCounter++;
	/*
	 * Read and clear the interrupts 
	 */
   lastIntr = whal_acxRegRead( ACX_REG_INTERRUPT_NO_CLEAR);
   whalIntMCB.LastIntrValue = lastIntr;
   intrEnabled = whalIntMCB.InterruptEnabled;

	
   /*
    * Handle RX_Transfer interrupt
    * ------------------------
    */
   if( IS_1BIT_MASK_ON( lastIntr, INTR_RX_TRF)) 
   {
	   if( IS_1BIT_MASK_ON( intrEnabled, INTR_RX_TRF)) 
	   {

		   whal_acxIntrAck(INTR_RX_TRF);
		   whalIntMCB.RxTrfIntrCounter++;
  
			/*
			 * Call RxData callback when reaching the threshold
		   */
		   if (++whalIntMCB.RxThresholdCounter >= whalIntMCB.RxThresholdValue)
		   {
			   if (whalIntMCB.RxData_Callback)
				   whalIntMCB.RxData_Callback(whalIntMCB.RxThresholdCounter);
			   whalIntMCB.RxThresholdCounter = 0;
		   }
	   }
   }
   
   /*
    * Handle MAILBOX COMMAND COMPLETE
    * -------------------------------
    */
   if( IS_1BIT_MASK_ON( lastIntr, INTR_CMD_COMPLETE))
   {
	  if( IS_1BIT_MASK_ON( intrEnabled, INTR_CMD_COMPLETE)) 
      {
         whal_acxIntrAck(INTR_CMD_COMPLETE);
         whalIntMCB.MboxIntrCounter++;
         whal_acxMboxSetComplete();
      }
   }

   /*
    * Handle INFO COMPLETE
    * --------------------
    */
   if( IS_1BIT_MASK_ON( lastIntr, INTR_INFO)) 
   {
      if( IS_1BIT_MASK_ON( intrEnabled, INTR_INFO)) 
      {
         whal_acxIntrAck( INTR_INFO);
         whalIntMCB.InfoIntrCounter++;
         /* (!!!) No callbeck yet */
      }
   }

   /*
    * Handle TX_DATA interrupt
    * ------------------------
    */
   if( IS_1BIT_MASK_ON( lastIntr, INTR_TX_DATA)) 
   {
      if( IS_1BIT_MASK_ON( intrEnabled, INTR_TX_DATA)) 
      {
         whal_acxIntrAck( INTR_TX_DATA);
         whalIntMCB.TxDataIntrCounter++;

         if( whalIntMCB.TxData_Callback) {
            whalIntMCB.TxData_Callback(0);
         }
      }
   }

   /*
    * Handle DEVICE_ERROR interrupt
    * -----------------------------
    */
   if( IS_1BIT_MASK_ON( lastIntr, INTR_DEVICE_ERROR)) 
   {
      if( IS_1BIT_MASK_ON( intrEnabled, INTR_DEVICE_ERROR)) 
      {
         whal_acxIntrAck( INTR_DEVICE_ERROR);

         if( whalIntMCB.DeviceError_Callback) {
            whalIntMCB.DeviceError_Callback();
         }
      }
   }

   /* re-enable device interrupts, prefer a direct write into ACX 
      (whalIntMCB.InterruptEnabled may be changed using other functions) */
   whal_acxRegWrite( ACX_REG_INTERRUPT_MASK, ~whalIntMCB.InterruptEnabled);
}

/****************************************************************************
 *                      whal_acxIntrSetRxDataCB()
 ****************************************************************************
 * DESCRIPTION:	Set the callback for the RX_DATA interrupt
 *
 * INPUTS:	
 *		RxData_Callback_T external_RxData_Callback		The callback address
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
int whal_acxIntrSetRxDataCB(void *external_RxData_Callback, int Threshold)
{  
    whalIntMCB.RxData_Callback = (RxData_Callback_T)external_RxData_Callback;
    whalIntMCB.RxThresholdValue = Threshold;	
	return OK;
}

/****************************************************************************
 *                      whal_acxIntrSetTxDataCB()
 ****************************************************************************
 * DESCRIPTION:	Set the callback for the TX_DATA interrupt
 *
 * INPUTS:	
 *		TxData_Callback_T external_TxData_Callback		The callback address
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
int whal_acxIntrSetTxDataCB(void *external_TxData_Callback)
{
    whalIntMCB.TxData_Callback = (RxData_Callback_T)external_TxData_Callback;
	return OK;
}
/****************************************************************************
 *                      whal_acxIntrSetDeviceErrorCB()
 ****************************************************************************
 * DESCRIPTION:	Set the callback for the DEVICE_ERROR interrupt
 *
 * INPUTS:	
 *		DeviceError_Callback_T external_DeviceError_Callback		The callback address
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
int whal_acxIntrSetDeviceErrorCB(void *external_DeviceError_Callback)
{
   whalIntMCB.DeviceError_Callback = (DeviceError_Callback_T)external_DeviceError_Callback;
   return OK;
}

/****************************************************************************
 *                      whal_acxIntrEnable()
 ****************************************************************************
 * DESCRIPTION:	Enable ACX100 interrupts
 * 
 * INPUTS:
 *		int Intr	The interrupts to be enabled
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void whal_acxIntrEnable(UINT32 Intr)
{
   register unsigned long whalIntrLock;

   /* critical section begin */
   PAL_intrLock(whalIntrLock);   
   
   whalIntMCB.InterruptEnabled |= Intr;
   whal_acxRegWrite (ACX_REG_INTERRUPT_MASK, ~whalIntMCB.InterruptEnabled );

   /* critical section end */
   PAL_intrUnLock(whalIntrLock);
}



/****************************************************************************
 *                      whal_acxIntrDisable()
 ****************************************************************************
 * DESCRIPTION:	Disable ACX100 interrupts
 * 
 * INPUTS:	
 *		int Intr	The interrupts to be disabled
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	
 ****************************************************************************/
void whal_acxIntrDisable(UINT32 Intr)
{
	register unsigned long whalIntrLock;
	
	/* critical section begin */
	PAL_intrLock(whalIntrLock);

	whalIntMCB.InterruptEnabled &= ~Intr; 
	if (ACX100_REG_ADDRESS)
		whal_acxRegWrite(ACX_REG_INTERRUPT_MASK, ~whalIntMCB.InterruptEnabled);

	/* critical section end */
	PAL_intrUnLock(whalIntrLock);
}

/****************************************************************************
*                      whal_acxMboxIntrMask()
****************************************************************************
* DESCRIPTION:
* 
* INPUTS:	
*		
* 
* OUTPUT:	None
* 
* RETURNS:	
****************************************************************************/
void whal_acxMboxIntrMask(int Intr, BOOL en)
{
	if(!en)
	    whalIntMCB.InterruptEnabled &= ~Intr;
	else
	    whalIntMCB.InterruptEnabled |= Intr;
}

/****************************************************************************
*                      whal_acxRxIntrEnable()
****************************************************************************
* DESCRIPTION:
* 
* INPUTS:	
*		
* 
* OUTPUT:	None
* 
* RETURNS:	
****************************************************************************/
void whal_acxRxIntrEnable(void)
{
	register unsigned long whalIntrLock;

	/* critical section begin */
	PAL_intrLock(whalIntrLock);

	whalIntMCB.InterruptEnabled |= INTR_RX_TRF;
	
	whal_acxRegWrite (ACX_REG_INTERRUPT_MASK, ~whalIntMCB.InterruptEnabled );

	/* critical section end */
	PAL_intrUnLock(whalIntrLock);
}

/****************************************************************************
*                      whal_acxRxIntrDisable()
****************************************************************************
* DESCRIPTION:	
* 
* INPUTS:	
*	
* 
* OUTPUT:	None
* 
* RETURNS:	
****************************************************************************/
void whal_acxRxIntrDisable(void)
{
	whalIntMCB.InterruptEnabled &= ~INTR_RX_TRF;
}


void inline whal_acxDisableAllLocalIntr(int *Intr)
{
	*Intr = whalIntMCB.InterruptEnabled;
	whalIntMCB.InterruptEnabled = 0;
	whal_acxRegWrite (ACX_REG_INTERRUPT_MASK, ~whalIntMCB.InterruptEnabled );
}

void inline whal_acxEnableAllLocalIntr(int *Intr)
{
	whalIntMCB.InterruptEnabled = *Intr;
	whal_acxRegWrite (ACX_REG_INTERRUPT_MASK, ~whalIntMCB.InterruptEnabled );
}

/****************************************************************************
 *                      whal_acxIntrInit()
 ****************************************************************************
 * DESCRIPTION:	Enable CPU + ACX100 interrupts
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void whal_acxIntrInit(void)
{
	whalIntMCB.TotalIntrCounter	= 0;
	whalIntMCB.RxDataIntrCounter	= 0;
    whalIntMCB.RxTrfIntrCounter	= 0;
	whalIntMCB.TxDataIntrCounter	= 0;
	whalIntMCB.MboxIntrCounter		= 0;
	whalIntMCB.InfoIntrCounter		= 0;
	whalIntMCB.RxThresholdCounter	= 0;
	whalIntMCB.RxThresholdValue	= 1000000;
	whalIntMCB.LastIntrValue		= 0;
	whalIntMCB.InterruptEnabled	= INITIAL_INTERRUPTS;	
}

/****************************************************************************
 *                      whal_acxIntrTrigger()
 ****************************************************************************
 * DESCRIPTION:	Generate interrupt to the ACX100
 * 
 * INPUTS:	
 *		UINT32 Intr		The interrupt bit
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void whal_acxIntrTrigger(UINT32 Intr)
{
	whal_acxRegSetBitVal(ACX_REG_INTERRUPT_TRIG, Intr);
}

/****************************************************************************
 *                      whal_acxIntrTriggerAaa()
 ****************************************************************************
 * DESCRIPTION:	Generate specific interrupt to the ACX100
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void whal_acxIntrTriggerCmd(void)		{ whal_acxIntrTrigger(INTR_TRIG_CMD);		}
void whal_acxIntrTriggerRxProc(void)	{ whal_acxIntrTrigger(INTR_TRIG_RX_PROC);	}
void whal_acxIntrTriggerTxProc(void)	{ whal_acxIntrTrigger(INTR_TRIG_TX_PROC);	}

/****************************************************************************
 *                      whal_acxIntrCheckAndAck()
 ****************************************************************************
 * DESCRIPTION:	Check interrupt and send acknowledge
 * 
 * INPUTS:	
 *		UINT32 Intr		The interrupt bit
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxIntrCheckAndAck(UINT32 Intr)
{
	register UINT32 Reg_IntrNoClear;
		
	Reg_IntrNoClear = whal_acxRegRead(ACX_REG_INTERRUPT_NO_CLEAR);

	if (IS_MASK_ON( Reg_IntrNoClear, Intr)) 
	{
        whal_acxRegWrite(ACX_REG_INTERRUPT_ACK, Intr);
		return 1;
    }
	
	return 0;
}

/****************************************************************************
 *                      whal_acxIntrCheck()
 ****************************************************************************
 * DESCRIPTION:	Check if the active interrupt
 * 
 * INPUTS:	
 *		UINT32 Intr		The interrupt bit
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxIntrCheck(UINT32 Intr)
{
	register UINT32 Reg_IntrNoClear;
		
	Reg_IntrNoClear = whal_acxRegRead(ACX_REG_INTERRUPT_NO_CLEAR);

	if (IS_MASK_ON( Reg_IntrNoClear, Intr)) 
	{
		return 1;
    }
	
	return 0;
}

/****************************************************************************
 *                      whal_acxIntrPrint()
 ****************************************************************************
 * DESCRIPTION:	Print interrupts information
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	
 ****************************************************************************/
int whal_acxIntrPrint(char *buf)
{
	int len =0;

	len +=sprintf(buf+len,"whal_acxIntrPrint ----------------------------\n"); 
	
	len +=sprintf(buf+len,"    Total=%d, RxTrf=%d, Mbox=%d, Info=%d\n", 
		whalIntMCB.TotalIntrCounter, whalIntMCB.RxTrfIntrCounter, whalIntMCB.MboxIntrCounter, whalIntMCB.InfoIntrCounter);

	len +=sprintf(buf+len,"    RxThresholdCounter=%d, RxThresholdValue=%d\n",
		whalIntMCB.RxThresholdCounter, whalIntMCB.RxThresholdValue);
	len +=sprintf(buf+len,"    TxData Counter=%d\n", 
		whalIntMCB.TxDataIntrCounter);
	len +=sprintf(buf+len,"    Last interrupt value=0x%08x, TickTime=%d\n",
		whalIntMCB.LastIntrValue, (int)PALSRV_getTick());
	len +=sprintf(buf+len,"    InterruptEnabled value=0x%08x\n",
		whalIntMCB.InterruptEnabled);
		

	len += sprintf(buf + len, "     Interrupt pending : 0x%x\n",  whal_acxRegRead(ACX_REG_INTERRUPT_NO_CLEAR));

	return len;
}

