/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalHostQueue.c
 *   PURPOSE: ACX100 Rx Host Queue manager
 *
 ****************************************************************************/

#include "pal.h"
#ifndef __linux__ /* DON'T include files from BSP !!! */
#include "tnetd53xx.h"
#endif
#include "acx_hw.h"
#include "whalAcxAccess.h"
#include "whalTrace.h"
#include "memMngr.h"
#include "whalHostQueue.h"
#include "dda.h"

extern PAL_spinLockHandle whalTxLock;

static void *alloc_memalign(int size)
{
#define MIPS_CACHE_ALIGN_SIZE 32
#define MIPS_CACHE_LINE_MASK 0xffffffe0
#define MIPS_CACHE_LINE_WRAP ((~MIPS_CACHE_LINE_MASK)+1)
#define MAKE_ADDR(p) (*((unsigned long *)((unsigned long)(p) - sizeof(unsigned long))))
	void *p, *orig = NULL;
	

	if (!(orig = PAL_memAlloc(0, size+MIPS_CACHE_ALIGN_SIZE, 0, 0)))
		return NULL;

	p = (void *)(((unsigned long)orig & MIPS_CACHE_LINE_MASK) + MIPS_CACHE_LINE_WRAP);
	MAKE_ADDR(p) = (unsigned long)orig;

	return p;
}

static void free_memalign(void *p)
{
	PAL_memFree(0, (void *)MAKE_ADDR(p), 0);
}
 
                                 
 /****************************************************************************
 *                      whal_HostQueue_init()
 ****************************************************************************
 * DESCRIPTION:	Initiate the Rx host queue object.
 *
 * INPUTS:	None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
HostQueue_Obj* whal_HostQueue_init( UINT32 NumOfDesc, UINT32  BlockSize)
{
   HostQueue_Obj *pHostQueue;
   UINT32      i;
   
   /* 
   * Allocate a new HostQueue object
   */	
   if( (pHostQueue = (HostQueue_Obj*) PAL_memAlloc(0,sizeof(HostQueue_Obj),0,0)) == NULL)
   {
      printf("Rx Host Desc Queue allocation failed ! \n");
      return NULL;
   } 
   PAL_memSet( (void*) pHostQueue, 0, sizeof( HostQueue_Obj ));
   
   /* 
   * Set the Queue params
   */
   pHostQueue->NumOfDesc = NumOfDesc;
   pHostQueue->BlockSize = BlockSize;
   
   /*
   * Create the host descriptors list. occupies continuous memory space.
   */
   pHostQueue->FirstDesc = (HostDesc_T*) alloc_memalign(NumOfDesc*sizeof( HostDesc_T));
   if( pHostQueue->FirstDesc == NULL)
   {
      printf("Rx Host Desc list allocation failed ! \n");
      return NULL;
   }
   PAL_memSet(pHostQueue->FirstDesc, 0, sizeof(HostDesc_T) * NumOfDesc);
   
   /* Initialize pointers in the queue to point to the first descriptor */
   pHostQueue->NextToInsert = pHostQueue->NextToRead = pHostQueue->FirstDesc;
   
   /* setup the descriptors */
   if( BlockSize > 0 )
   {
#if 0
      pHostQueue->BlocksPool = (UINT8*) PAL_memAlloc(0,pHostQueue->NumOfDesc*BlockSize,0,0);
      if( pHostQueue->BlocksPool == NULL)
      {
         printf("Rx Host buffer allocation failed ! \n");
         return NULL;
      }
#endif
   }

   for( i=0; i < pHostQueue->NumOfDesc; i++ )
   {
      if( BlockSize > 0 )
      {
		 UINT8 *buffer;
		 UINT32 bufToken;
		 if(tiwlan_ddaRxAllocBufferIfNeeded(BlockSize, &bufToken, &buffer) == OK)
		 {
		 if(buffer)
			whal_HostDesc_setBuffAddr(  &pHostQueue->FirstDesc[i], buffer, bufToken, BlockSize);
		 else
			 {
				 printf("whal_HostQueue_init: RX buffer allocation failed/n");
				 return NULL;
			 }
		 }
		 else
			 whal_HostDesc_setBuffAddr(  &pHostQueue->FirstDesc[i], pHostQueue->BlocksPool+i*BlockSize, 0, BlockSize);
      }
      else
      {
         /* BlockSize == 0, it's a Host ownership . (Tx queue init.)
            In the Tx the buffer pointer is given by the upper layer
         */
         whal_HostDesc_setStatus( &pHostQueue->FirstDesc[i], ACX_HOST_DESC_STAT_OWN );
      }
      
      if( i == (pHostQueue->NumOfDesc-1) ) 
      {
         /* The last decriptoe points to the first in the list (cyclic). */
         whal_HostDesc_setNextDesc( &pHostQueue->FirstDesc[i], pHostQueue->FirstDesc );
      } 
      else 
      {
         /* All the others descriptores points to the next in the queue. */
         whal_HostDesc_setNextDesc( &pHostQueue->FirstDesc[i],  &pHostQueue->FirstDesc[i+1] );
      }

	  PAL_cacheFlush(0, &pHostQueue->FirstDesc[i], sizeof(HostDesc_T));
   }/*endFor*/
   
   /* CurrNumDescInQ is the current number of descriptors with
      valid buffers associated with them */
   if( BlockSize == 0 ) /* Queue is created empty */
      pHostQueue->CurrNumDescInQ = 0;
   else
      pHostQueue->CurrNumDescInQ = pHostQueue->NumOfDesc;
   
   
   /* initiate statisrtics */
   pHostQueue->MinCurrNumDescInQ = pHostQueue->MaxCurrNumDescInQ = pHostQueue->CurrNumDescInQ;
   
   return pHostQueue;	 
}


 /****************************************************************************
 *                      whal_HostQueue_unload()
 ****************************************************************************
 * DESCRIPTION:	Unload the host queue object.
 *
 * INPUTS:	pHostQueue - pointer to the Host Queue object	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
void whal_HostQueue_unload( HostQueue_Obj *pHostQueue, UINT32  BlockSize) {
	HostDesc_T *pDescToFree /*, *pNext*/;
	int  i;

	if (!pHostQueue) {
		printf("\n\n%s: pHostQueue is NULL\n\n\n", __func__);
		return;
	}

	if (pHostQueue->BlockSize > 0) {
#if( WLAN_BUS_TYPE == WLAN_BUS_TYPE_VLYNQ)
		/*PAL_memFree (0,(void*)VLYNQ_ACX111_TO_HOST_SDRAM_PTR_CACHED(pHostQueue->BlocksPool),0);*/
#elif( WLAN_BUS_TYPE == WLAN_BUS_TYPE_PCI)
		/*PAL_memFree( 0, PCI_ACX111_TO_HOST_SDRAM_PTR_CACHED( pHostQueue->BlocksPool),0);*/
#else
#error    whalHostqueue.c -- Check bus type.
#endif
	}

	if (BlockSize) {
		for (i=0, pDescToFree = pHostQueue->FirstDesc; i < pHostQueue->NumOfDesc;i++) {
			if (pDescToFree && pDescToFree->bufToken != 0) {
				tiwlan_ddaRxFreeBuffer(pDescToFree->bufToken);
				pDescToFree->bufToken = 0;
			}

			pDescToFree = (HostDesc_T *)VLYNQ_ACX111_TO_HOST_SDRAM_PTR_CACHED(pDescToFree->next_desc_ptr);
		}
	}

	if (pHostQueue)
	{
		if (pHostQueue->FirstDesc)
#if( WLAN_BUS_TYPE == WLAN_BUS_TYPE_VLYNQ)
			free_memalign((void*)VLYNQ_ACX111_TO_HOST_SDRAM_PTR_CACHED(pHostQueue->FirstDesc));
#elif( WLAN_BUS_TYPE == WLAN_BUS_TYPE_PCI)
			free_memalign((void*)PCI_ACX111_TO_HOST_SDRAM_PTR_CACHED(pHostQueue->FirstDesc));
#else
#error    whalHostqueue.c -- Check bus type.
#endif

		PAL_memFree (0,pHostQueue,0);
	}
}


 /****************************************************************************
 *                      whal_rxHostQueue_getNext()
 ****************************************************************************
 * DESCRIPTION:	Initiate the Rx host queue object.
 *
 * INPUTS:	None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
STATUS whal_HostQueue_watchNextToRead(HostQueue_Obj* this, HostDescAttr_T * pDscAttr)
{
    if( this->CurrNumDescInQ == 0 )
       return  NOK;
    
    whal_HostDesc_getAttr( this->NextToRead, pDscAttr );
    return OK;
}

 /****************************************************************************
 *                      whal_rxHostQueue_getNext()
 ****************************************************************************
 * DESCRIPTION:	
 *
 * INPUTS:	None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
STATUS whal_HostQueue_getNext(HostQueue_Obj* this, HostDescAttr_T* pDscAttr)
{
	unsigned long flags;

    PAL_intrLock(flags);  /* START OF CRITICAL SECTION */
    
    whal_HostDesc_getAttr( this->NextToRead, pDscAttr );

	whal_HostDesc_clearToken (this->NextToRead);

    this->NextToRead = (HostDesc_T*) pDscAttr->next;
    
    this->CurrNumDescInQ--;
    
    if( this->CurrNumDescInQ < this->MinCurrNumDescInQ )
       this->MinCurrNumDescInQ = this->CurrNumDescInQ;
    

    PAL_intrUnLock(flags);   /* END OF CRITICAL SECTION */
    return OK;
}


 /****************************************************************************
 *                      whal_rxHostQueue_insert()
 ****************************************************************************
 * DESCRIPTION:	
 *
 * INPUTS:	None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
STATUS whal_HostQueue_insert(HostQueue_Obj* this, HostDescAttr_T* pDscAttr)
{
   if( this->CurrNumDescInQ >= this->NumOfDesc )
      return  NOK;
   
   /*PAL_spinLock(&whalTxLock);*/ /* START OF CRITICAL SECTION */

   whal_HostDesc_setAttr( this->NextToInsert, pDscAttr );
       
   this->NextToInsert = whal_HostDesc_getNext(this->NextToInsert);
   
   ++(this->CurrNumDescInQ);
   
   /* Statistics for debugging, shows the load on the queue */
   if( this->CurrNumDescInQ > this->MaxCurrNumDescInQ )
      this->MaxCurrNumDescInQ = this->CurrNumDescInQ;
   
   /*PAL_spinUnLock(&whalTxLock);*/  /* END OF CRITICAL SECTION */
   
   return OK;
}



/**************************************************************************
 *                      lbt_hwDsBdRingViewBD                   	  		  *
 **************************************************************************
 * DESCRIPTION: This function shows a BD in the ring table according to   *
 *              its physical location.                                    *
 *                                                                        *
 * INPUTS:  	this = The ring buffer descriptore.                       *
 *              index = Physical index of the BD to get.                  *
 * RETURNS:                                            					  *
 *                                                                        *
 **************************************************************************/
int  whal_HostQueue_show(HostQueue_Obj* this ,char *buf)
{
   int i, readHead, insertHead,len = 0;
   HostDescAttr_T descAttr;
   
   readHead   = ((ULONG)this->NextToRead - (ULONG)this->FirstDesc) / sizeof(HostDesc_T);
   insertHead = ((ULONG)this->NextToInsert - (ULONG)this->FirstDesc) / sizeof(HostDesc_T);
   
   len += sprintf(buf+len,"          #    addr       dataBuf     dataLen next         Ctl    Status");  
   for( i=0; i < this->NumOfDesc ; i++ )
   {
      if( i == readHead )
		  len += sprintf(buf+len,"RD->");
      else 
		  len += sprintf(buf+len,"    ");
      
      if( i == insertHead )
		  len += sprintf(buf+len,"IN->");
      else
		  len += sprintf(buf+len,"    ");
      
      whal_HostDesc_getAttr(  &this->FirstDesc[i], &descAttr );
      
      len += sprintf(buf+len,"%3d    0x%08x 0x%08x  %4d    0x%08x   0x%04x 0x%08x\n",
					i, (UINT32)&this->FirstDesc[i],  descAttr.blk_ptr, (UINT)(descAttr.blk_len), descAttr.next,
					(UINT16)(descAttr.ctrl) , (UINT)(descAttr.status)); 
   }
   
   len += sprintf(buf+len,"\nQueue Statistics: \n");
   len +=sprintf(buf+len,"\tCurrNumDescInQ\t\t%u\n", this->CurrNumDescInQ );
   len +=sprintf(buf+len,"\tMaxCurrNumDescInQ\t%u\n", this->MaxCurrNumDescInQ );
   len +=sprintf(buf+len,"\tMinCurrNumDescInQ\t%u\n\n", this->MinCurrNumDescInQ );

   len +=sprintf(buf+len,"\ttargetAborts\t\t%u\n", this->statistics.targetAborts);
   len +=sprintf(buf+len,"\tmasterAborts\t\t%u\n", this->statistics.masterAborts);
   len +=sprintf(buf+len,"\treqBeforeComplete\t%u\n", this->statistics.reqBeforeComplete);
   len +=sprintf(buf+len,"\tunexpectHostError\t%u\n", this->statistics.unexpectHostError);
   len +=sprintf(buf+len,"\tunexpectAcxError\t%u\n", this->statistics.unexpectAcxError);
   len +=sprintf(buf+len,"\tpciNotAbled\t\t%u\n\n", this->statistics.pciNotAbled);

   return len;
}

/**************************************************************************
 *                      whal_HostQueue_CleanAll()
 **************************************************************************
 * DESCRIPTION:
 *
 *
 * INPUTS:
 *
 * RETURNS:
 *
 **************************************************************************/
STATUS whal_HostQueue_CleanAll(HostQueue_Obj* this)
{
	HostDescAttr_T descAttr;
	mem_MSDU_T *pMsdu;
	mem_MSDU_T *nextMsduToFree;
	HostDesc_T *pDescToFree, *pNext;
	int  i;
	int  freedMsdus=0;
	unsigned long flags;

	if (!this)
		return NOK;

	PAL_intrLock(flags); /* START OF CRITICAL SECTION */  

	pDescToFree = this->FirstDesc;

	for(i=0;i<this->NumOfDesc;i++)
	{
		whal_HostDesc_getAttr( pDescToFree, &descAttr );
		pNext = (HostDesc_T*) descAttr.next;
		pMsdu = (mem_MSDU_T*)(descAttr.msduPtr);
		if( pMsdu == NULL)
		{
			pDescToFree = pNext;
			continue;
		}

		/* Free all the memory taken up by this MSDU */
		if(pMsdu->nextConcatMSDU == NULL)
		{
			freedMsdus++;
			wlan_memMngrFreeMSDU(pMsdu->handle );
		}
		else
		{
			nextMsduToFree = pMsdu;       
	
			while( nextMsduToFree != NULL ){
				nextMsduToFree->lastBDPtr->nextBDPtr = NULL;
				wlan_memMngrFreeMSDU(nextMsduToFree->handle );
				freedMsdus++;
				nextMsduToFree = nextMsduToFree->nextConcatMSDU;
			}
		}

		descAttr.status  = ACX_HOST_DESC_STAT_OWN;   /* take the ownership (TxBD) */
		descAttr.ctrl    = 0;
		descAttr.msduPtr = 0;
		descAttr.blk_len = 0;
		descAttr.blk_offset = 0;
		descAttr.blk_ptr = 0;      /* TxBD */

		whal_HostDesc_setAttr( pDescToFree, &descAttr);
		
		pDescToFree = pNext;
	}

	PAL_intrUnLock(flags);   /* END OF CRITICAL SECTION */
	printf("Freed MSDUs count = %d\n",freedMsdus);
	return OK;
}


STATUS  whal_HostQueue_freeMsdu(HostQueue_Obj* this)
{
   HostDescAttr_T descAttr;
   mem_MSDU_T *pMsdu;
   mem_MSDU_T *nextMsduToFree;
   HostDesc_T *pDescToFree;
   UINT16 msduLength;
   UINT16 totalLength;
   int  iteration = 0;

   /*
    *   Ownership is OK - Get the Free the MSDU and clean the descriptors. 
    */
   pDescToFree = this->NextToRead;
   whal_HostQueue_getNext( this, &descAttr); 

   pMsdu = (mem_MSDU_T*)(descAttr.msduPtr);
   if( pMsdu == NULL)
   {
       printf("whal_HostQueue_freeMsdu() - ERROR, read hostBD points to NULL\n");
       return NOK;
   }

   /* Get the MSDU length */
   msduLength  = memMgr_MsduDataSize( pMsdu);


   /* Free all the memory taken up by this MSDU */
	 if (pMsdu->nextConcatMSDU == NULL) {
		 wlan_memMngrFreeMSDU(pMsdu->handle );
	} else {
	   nextMsduToFree = pMsdu;       
	
	   while( nextMsduToFree != NULL ){
		   nextMsduToFree->lastBDPtr->nextBDPtr = NULL;
		   wlan_memMngrFreeMSDU(nextMsduToFree->handle );
		   nextMsduToFree = nextMsduToFree->nextConcatMSDU;
	   }
	}

   totalLength = 0;


   /* Free all the host descriptors of this packet. */
   /* The number of Tx BDs is 1 or 2.  */
   do
   {
	  iteration++;

	  /*printf("BDlen = %d\n", descAttr.blk_len);*/
	         
      if(( iteration > 1 )&&( descAttr.msduPtr != 0 ))
	  {
		  printf("!!!!! msduPtr!=Null iteration=%d,totalLength=%d, msduPtr=%p, msduLength=%d, nextDescMsduPtr=%p, nextDescBlkLen=%d \n",
				 iteration, totalLength, (void*)pMsdu, msduLength, (void *)((HostDesc_T *)this->NextToRead)->msduPtr, descAttr.blk_len);
		  break;
	  }

	  if(iteration > 1)
	  {
		  if(whal_HostQueue_getNext( this, &descAttr) != OK)
		  {
			  printf("whal_HostQueue_freeMsdu: Fatal error, get failed after watch succeeded\n");
			  break;
		  }
	  }

#if 0
	  if(dbg_HostQueue_freeMsdu)
      {
         printf("whal_HostQueue_freeMsdu() - "
              "hostBD @ 0x%X: stat = 0x%X, ctl = 0x%X, MsduPtr = 0x%X\n",
              (int)this->NextToRead, (int)(descAttr.status),
              (int)(descAttr.ctrl), (int)pMsdu);
      }
#endif

      /* Clean up this host BD */
   /* 
   * DMA error - save statistics and return after recycling the buffer.
   */
   if( descAttr.status & (ACX_HOST_DESC_STAT_ERR_MASK | ACX_HOST_DESC_STAT_DMA_ERR))
   {
      if( descAttr.status & ACX_HOST_DESC_STAT_TAR_ABORT )
         this->statistics.targetAborts++;
      
      if( descAttr.status & ACX_HOST_DESC_STAT_MASTER_ABORT )
         this->statistics.masterAborts++;
      
      if( descAttr.status & ACX_HOST_DESC_STAT_REQ_BEF_COMP )
         this->statistics.reqBeforeComplete++;
      
      if( descAttr.status & ACX_HOST_DESC_STAT_UNEXP_HOST )
         this->statistics.unexpectHostError++;
      
      if( descAttr.status & ACX_HOST_DESC_STAT_UNEXP_ACX )
         this->statistics.unexpectAcxError++;
      
      if( descAttr.status & ACX_HOST_DESC_STAT_PCI_DISABLED )
         this->statistics.pciNotAbled++;
   }

      /* Accumulate the overall length (read from the descriptor). */
      totalLength += descAttr.blk_len;

      descAttr.status  = ACX_HOST_DESC_STAT_OWN;   /* take the ownership (TxBD) */
      descAttr.ctrl    = 0;
      descAttr.msduPtr = 0;
      descAttr.blk_len = 0;
      descAttr.blk_offset = 0;
      descAttr.blk_ptr = 0;      /* TxBD */
    
	  whal_HostDesc_setAttr( pDescToFree, &descAttr);

      if( totalLength >= msduLength ) 
          break;

      pDescToFree = this->NextToRead;
    	  
   } while( whal_HostQueue_watchNextToRead( this, &descAttr) == OK );

   return OK;
}/* END whal_HostQueue_freeMsdu() */




