/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalRxDesc.c
 *   PURPOSE: manage the ACX100 Rx memory blocks list 
 *   
 ****************************************************************************/

#include "whalCommon.h"
#include "whalParams.h"
#include "AcxDataIf.h"
#include "whalRxBufs.h"
#include "whalAcxAccess.h"

/****************************************************************************
 *                      OBJECT DATA BASE
 ****************************************************************************
 *
 *	ACX100 Memory block
 *		The ACX100 blocks header is 32 bits long and consist of the fields
 *		pNext and Control(for LAST block bit) and are initiate by the ACX100 
 *		as a link list.
 *
 * Local Data Base
 *		Holds the number of blocks and the address of the list
 *
 ****************************************************************************/
typedef struct 
{
	int		NumBlocks;
	UINT32	BlockSize;
	UINT8  *BaseAddr;
} RxBufs_Obj_T;

RxBufs_Obj_T RxBufs_Obj, *pRxBufs;

#define BlockHeader_Get(BlockPtr)	   ( whal_acxMemGetU32((UINT32)BlockPtr+ACX100_MEM_ADDRESS) )

/****************************************************************************
 *                      whal_rxBufs_InitBufHandle()
 ****************************************************************************
 * DESCRIPTION:	Initiate RxBufs handle for copy blocks to Host buffers
 * 
 * INPUTS:	
 *		RxBuf_Handle_T *pHandle		The buffer handle which is points to the  
 *									ACX100 buffer and used for copy and append 
 *									from ACX100 to host buffers
 *		UINT32	Len					The length of the requested buffer
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_rxBufs_InitBufHandle(RxBuf_Handle_T *pHandle, UINT8 *MpduAddr, int MpduOffset, int MpduLen)
{
	pHandle->BufAddr	= MpduAddr;
	pHandle->BlockAddr	= MpduAddr;
	pHandle->BlockOffset= MpduOffset;

	return OK;
}

/****************************************************************************
 *                      whal_rxBufs_AppendBuf()
 ****************************************************************************
 * DESCRIPTION:	Copy one or more ACX100 memory blocks into one contiguous 
 *		host block. The number of ACX100 blocks is depends on the requested
 *		copy length and the size of each ACX100 block.
 *		This function is needed because the host buffer is also seperate into
 *		blocks and this function support keeping the existing status (current 
 *		ACX100 block address and offset) for the copy of the next host block.
 * 
 * INPUTS:	
 *		RxBuf_Handle_T *pHandle		The buffer handle which is points to the  
 *									ACX100 buffer blocks and used for 
 *									append host data to existing blocks
 *		UINT8 *HostBufAddr			The host block address
 *		UINT32 HostBufLen			The length of the host block
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_rxBufs_AppendBuf(RxBuf_Handle_T *pHandle, char *HostBufAddr, int HostBufLen)
{
	register UINT32 BlockHeader;
	register int AcxBlock_FreeSize;
	register int RemainHostLen = HostBufLen;

	/*
	 * There is two possible situation:
	 *	1. The remain host size to copy is greater than the size of one ACX100
	 *		block. In this situation the function will copy from the ACX100 block
	 *		and skip to the next ACX100 block for the remainning data
	 *	2. The remain host data size is less than the size of one ACX100
	 *		block. This is the last copy in the loop
	 */
	AcxBlock_FreeSize = pRxBufs->BlockSize - pHandle->BlockOffset;

	/*
	 * Copy full acx100 blocks
	 */
	while (RemainHostLen > AcxBlock_FreeSize)
	{
		/* fill all the current block and move to the next block */
		whal_acxMemCopy(HostBufAddr, (char *)(pHandle->BlockAddr + pHandle->BlockOffset), AcxBlock_FreeSize);
		HostBufAddr += AcxBlock_FreeSize;
		/* go to the next block in the buffer */
		BlockHeader = BlockHeader_Get(pHandle->BlockAddr-ACX100_MEM_ADDRESS);
		/* update handle fields for the next entry */
		pHandle->BlockAddr = GET_NEXT_BLOCK_ADDR(BlockHeader) + ACX100_MEM_ADDRESS;
		pHandle->BlockOffset = BLOCK_HEADER_SIZE;
		RemainHostLen -= AcxBlock_FreeSize;
		AcxBlock_FreeSize = pRxBufs->BlockSize - pHandle->BlockOffset;
	}

	/*
	 * Copy the remain host data
	 */
	if (RemainHostLen > 0)
	{
		whal_acxMemCopy(HostBufAddr, (char *)(pHandle->BlockAddr + pHandle->BlockOffset), RemainHostLen);
		pHandle->BlockOffset += RemainHostLen;
	}

	return OK;
}

/****************************************************************************
 *                      whal_rxBufs_Init()
 ****************************************************************************
 * DESCRIPTION:	Initiate the object data base 
 *		Set BlockSize and NumRxBlocks parameters
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_rxBufs_Init(void)
{
	pRxBufs = &RxBufs_Obj;

	PAL_memSet((void *)pRxBufs, 0, sizeof(*pRxBufs));

	return OK;
}

/****************************************************************************
 *                      whal_rxBufs_Config()
 ****************************************************************************
 * DESCRIPTION:	configure the RxBufs object 
 * 
 * INPUTS:	
 *		UINT32 RxBufs_StartAddress	The start address of the ACX100 buffers
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	The size of the buffers in the ACX100 memory
 ****************************************************************************/
UINT32 whal_rxBufs_Config(UINT32 RxBufs_StartAddress)
{
   return 0;
}

/****************************************************************************
 *                      whal_rxBufs_GetNumBlocks()
 ****************************************************************************
 * DESCRIPTION:	Return the number of ACX100 Rx memory blocks
 ****************************************************************************/
UINT32 whal_rxBufs_GetNumBlocks(void)
{
	return (pRxBufs->NumBlocks);
}

/****************************************************************************
 *                      whal_rxBufs_GetSize()
 ****************************************************************************
 * DESCRIPTION:	Return the size of ACX100 memory for Rx blocks
 ****************************************************************************/
UINT32 whal_rxBufs_GetSize(void)
{
	return (pRxBufs->NumBlocks * pRxBufs->BlockSize);
}

/****************************************************************************
 *                      whal_rxBufs_GetOffsetAddr()
 ****************************************************************************
 * DESCRIPTION:	Return the offset address of the Rx blocks within the ACX100
 ****************************************************************************/
UINT32 whal_rxBufs_GetOffsetAddr(void)
{
	return ((UINT32)(pRxBufs->BaseAddr));
}

/****************************************************************************
 *                      whal_rxBufs_Print()
 ****************************************************************************
 * DESCRIPTION:	Print the RxBufs list and it's main fields
 ****************************************************************************/
int whal_rxBufs_Print(void)
{
	UINT8 *pBlock;
	UINT8 *NextAddr;
	UINT32 BlockHeader;
	int Idx;

	pBlock = pRxBufs->BaseAddr;

	printf("RxBufs ----------------------------------------------\n");
	printf("Addr=0x%x, NumBlocks=0x%x, BlockSize=0x%x\n", (int)pRxBufs->BaseAddr, pRxBufs->NumBlocks, pRxBufs->BlockSize);

	for (Idx=0; Idx<pRxBufs->NumBlocks; Idx++)
	{
		BlockHeader = BlockHeader_Get(pBlock);
		NextAddr = GET_NEXT_BLOCK_ADDR(BlockHeader);
		printf("%02x: Addr=0x%08x, Hdr=0x%08x, Next=0x%08x, LastControl=0x%04x\n", Idx,	
			(int)pBlock, (int)BlockHeader, (int)NextAddr, 
			ENDIAN_HANDLE_WORD(*(UINT16 *)((int)pBlock + whal_ParamsGetGenParams(0)->RxStatusHdrLen + ACX100_MEM_ADDRESS)));
		pBlock += pRxBufs->BlockSize;
	}

	return OK;
}

/****************************************************************************
 *                      whal_rxBufs_InfoPrint()
 ****************************************************************************
 * DESCRIPTION:	Print the RxBufs info
 ****************************************************************************/
int whal_rxBufs_InfoPrint(char *buf)
{
	int len = 0;
	len += sprintf(buf+len,"RxBufs ----------------------------------------------\n");
	len += sprintf(buf+len,"Addr=0x%x, NumBlocks=0x%x, BlockSize=0x%x\n", (int)pRxBufs->BaseAddr, pRxBufs->NumBlocks, pRxBufs->BlockSize);

	return len;
}
