/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 * 
 *   MODULE:  whalRxDesc.c
 *   PURPOSE: ACX100 Rx descriptor object
 *   
 ****************************************************************************/

#include "whalCommon.h"
#include "whal.h"
#include "AcxCommands.h"
#include "whalInterrupt.h"
#include "whalAcxAccess.h"
#include "whalRx.h"
#include "whalRxDesc.h"
#include "whalParams.h"

/****************************************************************************
 *                      whal_rxDesc_Init()
 ****************************************************************************
 * DESCRIPTION:	Initiate ACX100 Rx desctiptor 
 *		The "Next" field is not set because it initiated once to point to the 
 *		next descriptor in the queue
 *
 * 
 * INPUTS:	
 *		RxDesc_T *pRxDesc	The descriptor address 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	
 ****************************************************************************/
void whal_rxDesc_Init(RxDesc_T *pRxDesc)
{
}


/****************************************************************************
 *                      whal_rxDesc_GetAttr()
 ****************************************************************************
 * DESCRIPTION:	Copy requested attribute values from Rx descriptor into 
 *		special host attributes structure
 * 
 * INPUTS:	
 *		RxDesc_T *pDstDesc	Host buffer to copy to 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK
 ****************************************************************************/
int whal_rxDesc_GetAttr(RxDesc_T *pRxDesc, UINT32 *MpduAddr, int *MpduOffset, int *MpduLen, RxAttr_T *pRxAttr)
{
	RxDesc_T localRxDesc;

	whal_rxDesc_Read( &localRxDesc, pRxDesc);

    /* MPDU is in SDRAM, returns pointer to host descriptor    */
    *MpduAddr   = (UINT32)localRxDesc.HostMemPtr + ACX100_MEM_ADDRESS;
	
    *MpduOffset = whal_ParamsGetGenParams(0)->RxStatusHdrLen;
	*MpduLen    = localRxDesc.MPDULen;
    pRxAttr->Ctl	= localRxDesc.Ctl;
	pRxAttr->Rate	= localRxDesc.Rate;
	pRxAttr->Status = localRxDesc.Status;
	pRxAttr->SNR	= 0;
	pRxAttr->SIR	= 0;

	return OK;
}

/****************************************************************************
 *                      whal_rxDesc_Read()
 ****************************************************************************
 * DESCRIPTION:	Copy Rx descriptor from ACX100 to Host buffer
 *		Swap relevant fields only 
 * 
 * INPUTS:	
 *		RxDesc_T *pDstDesc	Host buffer to copy to 
 *		RxDesc_T *pSrcDesc	ACX100 Rx descriptor address 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK
 ****************************************************************************/
int whal_rxDesc_Read(RxDesc_T *pDstDesc, RxDesc_T *pSrcDesc)
{
	whal_acxMemCopy( (char *)pDstDesc, (char *)pSrcDesc, sizeof(RxDesc_T));

	/* 
	 * Convert only 16 and 32 bit fields 
	 * (!!!) can copy only relevat fields
	 */
	pDstDesc->next		  = (UINT8 *)(ENDIAN_HANDLE_LONG((int)(pDstDesc->next)));
	pDstDesc->HostMemPtr  = (UINT8 *)(ENDIAN_HANDLE_LONG((int)(pDstDesc->HostMemPtr)));
	pDstDesc->ACXMemPtr   = (UINT32 *)(ENDIAN_HANDLE_LONG((int)(pDstDesc->ACXMemPtr)));
#if 0
   pDstDesc->time        = ENDIAN_HANDLE_LONG(pDstDesc->time);
#endif
   pDstDesc->MPDULen     = ENDIAN_HANDLE_WORD(pDstDesc->MPDULen);

	return OK;
}

/****************************************************************************
 *                      whal_rxDesc_GetCtl()
 ****************************************************************************
 * DESCRIPTION:	Return the "Ctl" field from the Rx descriptor
 ****************************************************************************/
UINT8 whal_rxDesc_GetCtl(RxDesc_T *pRxDesc)
{
	return(whal_acxMemGetByte((UINT32)pRxDesc, RX_CTL_WORD_OFFSET));
}

/****************************************************************************
 *                      whal_rxDesc_SetCtl()
 ****************************************************************************
 * DESCRIPTION:	Set the "Ctl" field on the Rx descriptor
 ****************************************************************************/
void whal_rxDesc_SetCtl(RxDesc_T *pRxDesc, UINT8 Val)
{
	whal_acxMemSetByte((UINT32)pRxDesc, RX_CTL_WORD_OFFSET, Val);
}

/****************************************************************************
 *                      whal_rxDesc_GetHostNext()
 ****************************************************************************
 * DESCRIPTION:	Return the Next descriptor pointer
 ****************************************************************************/
RxDesc_T *whal_rxDesc_GetHostNext(RxDesc_T *pRxDesc)
{
	return ( (RxDesc_T *) (whal_acxMemGetU32(&pRxDesc->next) + ACX100_MEM_ADDRESS) );
}

/****************************************************************************
 *                      whal_rxDesc_SetHostNext()
 ****************************************************************************
 * DESCRIPTION:	set the "Next" field on the Rx descriptor
 ****************************************************************************/
void whal_rxDesc_SetHostNext(RxDesc_T *pRxDesc, UINT32 HostAddr)
{
	whal_acxMemSetU32(&pRxDesc->next, HostAddr - ACX100_MEM_ADDRESS);
}

/****************************************************************************
 *                      whal_rxDesc_GetAcxBufAddr()
 ****************************************************************************
 * DESCRIPTION:	Return the pointer to the frame data in the ACX100 memory
 ****************************************************************************/
UINT32 whal_rxDesc_GetAcxBufAddr(RxDesc_T *pRxDesc)
{
	return ( whal_acxMemGetU32(&pRxDesc->ACXMemPtr) + whal_ParamsGetGenParams(0)->RxStatusHdrLen + ACX100_MEM_ADDRESS );
}

/****************************************************************************
 *                      whal_rxDesc_GetHostBufAddr()
 ****************************************************************************
 * DESCRIPTION:	Return the pointer to the frame data in the ACX100 memory
 ****************************************************************************/
UINT32 whal_rxDesc_GetHostBufAddr(RxDesc_T *pRxDesc)
{
	return ( whal_acxMemGetU32(&pRxDesc->HostMemPtr) + whal_ParamsGetGenParams(0)->RxStatusHdrLen + ACX100_MEM_ADDRESS );
}

/****************************************************************************
 *                      whal_rxDesc_Print()
 ****************************************************************************
 * DESCRIPTION:	Print the descriptor main fields
 ****************************************************************************/
void whal_rxDesc_Print(RxDesc_T *pHwRxDesc)
{
	RxDesc_T localRxDesc;

	whal_rxDesc_Read( &localRxDesc, pHwRxDesc);

	printf("AcxMemPtr=0x%04x, Next=0x%04x, MpduLen=0x%04x, Ctl=0x%02x, Stt=0x%02x\n",
			(int)localRxDesc.ACXMemPtr, (int)localRxDesc.next, localRxDesc.MPDULen, localRxDesc.Ctl, localRxDesc.Status);
}

