/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalRxMaster.c
 *   PURPOSE: ACX100 Rx Queue manager
 *
 ****************************************************************************/
#include "acx_hw.h" 
#include "whalCommon.h"
#include "whal.h"
#include "whalRx.h"
#include "whalRxQueue.h"
#include "whalRxBufs.h"
#include "whalRxDesc.h"
#include "memMngr.h"
#include "whalInterrupt.h"
#include "whalAcxAccess.h"
#include "whalParams.h"
#include "whalTrace.h"
#include "whalHostQueue.h"
#include "whalRxMaster.h"
#include "dda.h"

extern STATUS whal_HostQueue_watchNextToRead(HostQueue_Obj* this, HostDescAttr_T * pDscAttr);
static HostQueue_Obj* pRxHostQueue = NULL;
/* halRestartSession counts each time HAL is restarted */
int halRestartSession = 0;

 /****************************************************************************
 *                      whal_rxInit()
 ****************************************************************************
 * DESCRIPTION:	Initiate the Rx Acx Queue,and Rx Host Queue.
 *
 * INPUTS:	None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
STATUS whal_rxMasterInit()
{
	halRestartSession++;

#ifdef TIWLAN_DEBUG
	whal_trcInit();
#endif

#if 0
   /* Init Rx queue in ACX, memory is already allocated in the ACX */
	whal_rxQueue_Init(RxQid);
#endif
   /* Init Rx queue ib ACX */
	pRxHostQueue = whal_HostQueue_init( whal_ParamsGetDmaParams(0)->NumRxBlocks, 
										 4096 /*whal_ParamsGetDmaParams()->RxBlockSize*/);
	if( pRxHostQueue == NULL)
		return NOK;

	return OK;
}

 /****************************************************************************
 *                      whal_rxMasterUnload()
 ****************************************************************************
 * DESCRIPTION:	Unload the Rx Host Queue.
 *
 * INPUTS:	None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
void whal_rxMasterUnload()
{
	if (pRxHostQueue) {
		whal_HostQueue_unload(pRxHostQueue, 1);
		pRxHostQueue = NULL;
	}
}

/****************************************************************************
 *                      whal_rxRecv()
 ****************************************************************************
 * DESCRIPTION:	Test function, Recv buffer from the ACX100.
 * 
 * INPUTS:	
 *		char *MpduBuf		The buffer to copy to from the ACX100
 *		int  MpduBufLen		The length of the buffer
 *		RxAttr_T *pRxAttr	The Receive attributes
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK.
 ****************************************************************************/

MEM_HANDLE whal_rxMasterRecv(INT32 *MpduBufLen, RxAttr_T *pRxAttr)
{
   register UINT32	 mpduLen, infoLen;
   mem_BD_T *pBd;
   HostDescAttr_T pDscAttr;
   WlanRxStatusInfo_T *rxStatusInfo;   
   UINT8 *newBuffer;
   UINT32 newBufToken;
  
   /*--------------------------------------------------------------------------
   Phase 1 : Watch Host Descriptor and read it's status.
   --------------------------------------------------------------------------*/
   if( whal_HostQueue_watchNextToRead(pRxHostQueue, &pDscAttr) == NOK )
       return NULL;
   
   /* 
   * Descriptor does not belong to the HOST, this can happen. 
   */
   if( !(pDscAttr.status & ACX_HOST_DESC_STAT_OWN ) )
   {
      return NULL;
   }

   /* Before getting the next descriptor, try to allocate the buffer
      for the next reception, store it in the blk_offset field */
	if(tiwlan_ddaRxAllocBufferIfNeeded(pRxHostQueue->BlockSize,
									&newBufToken, &newBuffer) == OK)
	{
		if(!newBuffer)
		{
			if(REPORT_LEVEL(RL_HAL))
			   wdrv_utilsReport(RL_HAL, PRI_WARNING,"whal_rxMasterRecv: Failed to allocate the new buffer\n");     
			return NULL;
		}
	}
	else /* for VxWorks, new buffer is the same as the previous */
		newBufToken = pDscAttr.bufToken;
   
   /* get and advance the descriptor ptr */
   if( whal_HostQueue_getNext(pRxHostQueue, &pDscAttr) == NOK )
   {
	   tiwlan_ddaRxFreeBuffer(newBufToken);
       return NULL;
   }
   
   /* 
   * DMA error - save statistics and return after recycling the buffer.
   */
   if( pDscAttr.status & (ACX_HOST_DESC_STAT_ERR_MASK | ACX_HOST_DESC_STAT_DMA_ERR))
   {
      if( pDscAttr.status & ACX_HOST_DESC_STAT_TAR_ABORT )
         pRxHostQueue->statistics.targetAborts++;
      
      if( pDscAttr.status & ACX_HOST_DESC_STAT_MASTER_ABORT )
         pRxHostQueue->statistics.masterAborts++;
      
      if( pDscAttr.status & ACX_HOST_DESC_STAT_REQ_BEF_COMP )
         pRxHostQueue->statistics.reqBeforeComplete++;
      
      if( pDscAttr.status & ACX_HOST_DESC_STAT_UNEXP_HOST )
         pRxHostQueue->statistics.unexpectHostError++;
      
      if( pDscAttr.status & ACX_HOST_DESC_STAT_UNEXP_ACX )
         pRxHostQueue->statistics.unexpectAcxError++;
      
      if( pDscAttr.status & ACX_HOST_DESC_STAT_PCI_DISABLED )
         pRxHostQueue->statistics.pciNotAbled++;
      
      /*
       *  Status is not OK, reuse the buffer by clearing it to
       *  the next place availiable. 
      */
      pDscAttr.ctrl = 0;
      pDscAttr.status = 0;
	  tiwlan_ddaRxFreeBuffer(newBufToken);
      whal_HostQueue_insert( pRxHostQueue, &pDscAttr);
	  if(REPORT_LEVEL(RL_HAL))
		  wdrv_utilsReport(RL_HAL, PRI_WARNING,"Error in the Host descriptor :: Status = 0x%8x \n", pDscAttr.status );     
      return NULL;
   }
   
   if ( wlan_memMngrAllocBDs( 1, &pBd) != OK )
   {
	   tiwlan_ddaRxFreeBuffer(newBufToken);
	   pDscAttr.ctrl = 0;
	   pDscAttr.status = 0;
	   whal_HostQueue_insert( pRxHostQueue, &pDscAttr);
       wdrv_utilsReport(RL_HAL, PRI_WARNING,"whal_rxMasterRecv: wlan_memMngrAllocBDs failed");
       return NULL;
   }

   
   /*--------------------------------------------------------------------------
   Phase 3 :
   All descriptors are OK ! Get the Host descriptore prepare MPDU
   BD for the driver.
   --------------------------------------------------------------------------*/

    /*
      31----------------------------------------0
                  Control | MemBlkPtr             0x00
        ----------------------------------------                      
                     RxStatusInfo                 0x04
                     
        ----------------------------------------
                      PLCPHeader                  0x10 
                      
        ----------------------------------------
                         Body                     0x18                      
                      
                      
        ----------------------------------------  0xnn              
     */                                              
   
    rxStatusInfo = (WlanRxStatusInfo_T*) pDscAttr.blk_ptr; 
  
   PAL_cacheInvalidate(0, rxStatusInfo,/* virtual address */      
				       sizeof(WlanRxStatusInfo_T) + sizeof(RxPLCPHeader_T)  );    /* number of bytes to invalidate */


   /*newPlcp*/
   {
	   RxPLCPHeader_T *rxPlcpHeader;
	   rxPlcpHeader = (RxPLCPHeader_T *)(pDscAttr.blk_ptr + sizeof(WlanRxStatusInfo_T));
	   pRxAttr->Mic =  (rxPlcpHeader->status &  PLCP_MIC_FAILURE_STATUS  ) ? NOK : OK ;
   }

   /* our data length is only from mac header without plcp header */
   pRxAttr->woneIndex = (ENDIAN_HANDLE_WORD( rxStatusInfo->length ) & 0xC000)>>14;

   if(pRxAttr->woneIndex >= 2)
   {   
	   printf("whalRxMaster: woneIndex=%d !!!!!!!!!!!!!!!!!!!!!!!!\n",pRxAttr->woneIndex);
	   tiwlan_ddaRxFreeBuffer(newBufToken);
	   pDscAttr.ctrl = 0;
	   pDscAttr.status = 0;
	   whal_HostQueue_insert( pRxHostQueue, &pDscAttr);
	   if (wlan_memMngrFreeBD(pBd->handle) != TRUE)
		   wdrv_utilsReport(RL_HAL, PRI_EMERGENCY,"whal_rxMasterRecv: wlan_memMngrFreeBD failed");
	   return NULL;
   }

   infoLen = (ENDIAN_HANDLE_WORD( rxStatusInfo->length ) & 0x0fff);
   /* W/A we can't pass any packet of zero length up to the wlanEnd */
   if(infoLen <= sizeof(RxPLCPHeader_T))
   {
       wdrv_utilsReport(RL_HAL, 
                        PRI_WARNING,
                        "whal_rxRecv: Error: %d bytes long packet received\n",
                        ENDIAN_HANDLE_WORD( rxStatusInfo->length ));
	  tiwlan_ddaRxFreeBuffer(newBufToken);
      pDscAttr.ctrl = 0;
      pDscAttr.status = 0;
      whal_HostQueue_insert( pRxHostQueue, &pDscAttr);
      if (wlan_memMngrFreeBD(pBd->handle) != TRUE)
          wdrv_utilsReport(RL_HAL, PRI_EMERGENCY,"whal_rxMasterRecv: wlan_memMngrFreeBD failed");
      return NULL;
   }
   mpduLen = infoLen - sizeof(RxPLCPHeader_T);

   /*
    * Wrap the the BD for the Rx task :
    * Set the offset to the MAC header, after PLCP header and Status info.
    */
   /* newPlcp*/
   pBd->data       = (char*)((UINT32)pDscAttr.blk_ptr + sizeof(WlanRxStatusInfo_T) + sizeof(RxPLCPHeader_T) );
   pBd->dataOffset = 0;
   pBd->dataBuf    = NULL;  /* This indicates the memMngr to not return buffer to mem manager pool */


   pBd->length     = mpduLen;
   *MpduBufLen     = mpduLen;

   /*
    * Read the Rx info and keep it in the tracer.
    */
	pRxAttr->Status = rxStatusInfo->mod_mode;

   PAL_cacheInvalidate(0,
      pBd->data,    /* virtual address */
      mpduLen );    /* number of bytes to invalidate */

#ifdef TIWLAN_DEBUG
   if (whal_trcIsEnable())
       whal_trcAddRx(rxStatusInfo, "SUCC");
#endif


   /* 
    * swaping the frame control - Needed to be modified for little endian systems.
    */
   *((UINT16*)(pBd->data)) = ENDIAN_HANDLE_WORD((*((UINT16*)(pBd->data))));
   
   /* free function and args for host reclaiming the whole buffer */
   pBd->freeFunc =  (void*)whal_rxMasterFreeBuffer;

   pBd->freeArgs[0] =  (UINT32)pRxHostQueue;
   pBd->freeArgs[1] =  pDscAttr.blk_ptr;
   pBd->freeArgs[2] =  pDscAttr.blk_len;
   pBd->freeArgs[3] =  halRestartSession;
   pBd->freeArgs[4] =  newBufToken;
   pBd->bufToken	=  pDscAttr.bufToken;

   return (MEM_HANDLE)pBd; 
   
}


/****************************************************************************
 *                      whal_rxFreeBuffer()
 ****************************************************************************
 * DESCRIPTION:	Free a busy buffer to a Rx Queue. 
 * 
 * INPUTS:	
 *		char *MpduBuf		The buffer to copy to from the ACX100
 *		int  MpduBufLen		The length of the buffer
 *		RxAttr_T *pRxAttr	The Receive attributes
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK.
 ****************************************************************************/

void whal_rxMasterFreeBuffer( mem_BD_T *pBd )
{
    HostQueue_Obj   *pRxHostQ;
    HostDescAttr_T  pDscAttr;
    unsigned long   flags;


	if (halRestartSession == pBd->freeArgs[3])
	{
		/* this special case is for deconcatenated packets coming from rx
		   via Bss bridge to the tx - OS buffer should be freed, but no
		   new buffer should be attached to the RX ring;
		   freeArgs[4] = 0  indicates that the BD comes from rx, 
		   if it comes from tx(after BSS bridge), the DMA should not get another buffer instead */
		if(pBd->freeArgs[4])
		{
			pRxHostQ =  (HostQueue_Obj*) pBd->freeArgs[0];		
		
			pDscAttr.bufToken = pBd->freeArgs[4];
			pDscAttr.blk_ptr = (UINT32) TIWLAN_DDA_BUFFER_FROM_TOKEN(pDscAttr.bufToken);
			pDscAttr.blk_len =  (UINT16) pRxHostQ->BlockSize;
			pDscAttr.blk_offset = 0;
			pDscAttr.ctrl = 0;
			pDscAttr.status = 0;
		
			PAL_intrLock(flags);
			if( whal_HostQueue_insert(pRxHostQ, &pDscAttr) != OK ){
				puts("whal_rxMasterFreeBuffer() -- Host queue is full !");
			}
			PAL_intrUnLock(flags);
		
			if(pBd->bufToken != 0)
			{
				pBd->dataBuf = NULL;
				tiwlan_ddaRxFreeBuffer(pBd->bufToken);
			}
		}
		else if(!pBd->freeArgs[4] && pBd->bufToken)
		{
			 tiwlan_ddaRxFreeBuffer(pBd->bufToken);
			 return;
		}
		else
		{
			return;
		}			
	}
}


/****************************************************************************
 * DESCRIPTION:	Print Rx queue/bufs info.
 *
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
int whal_rxMasterInfoPrint(char *buf)
{
    return whal_HostQueue_show( pRxHostQueue ,buf);
}


/****************************************************************************
 *                      whal_rxMaster_getHostQueueAddr()
 ****************************************************************************
 * DESCRIPTION:	 
 * 
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK.
 ****************************************************************************/
UINT32 whal_rxMaster_getHostQueueAddr( void )
{
	return  whal_HostQueue_getFirstDescAddr( pRxHostQueue);
}


/****************************************************************************
 *                      whal_rxMaster_getQueueInfo()
 ****************************************************************************
 * DESCRIPTION:	 
 * 
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK.
 ****************************************************************************/
UINT32 whal_rxMaster_getQueueInfo( UINT32 *RxDescAddr, UINT32 *HostOwn, UINT32 *OwnerPercentage )
{
	HostDesc_T *pDesc;
	UINT32 NumOwnedDesc;
	UINT32 i;
    UINT32 maxCurr;

   /* Get the descriptor address */
   *RxDescAddr = (UINT32)pRxHostQueue->NextToRead;

   /* Get the Ownership */
   *HostOwn = ((ENDIAN_HANDLE_LONG( pRxHostQueue->NextToRead->status))
      & ACX_HOST_DESC_STAT_OWN);

   /* Get the percentage of entries with host ownership */
   maxCurr = pRxHostQueue->MaxCurrNumDescInQ;
   pDesc   = pRxHostQueue->FirstDesc;
   NumOwnedDesc = 0;
   for( i = 0; i < maxCurr; ++i)
   {
      if( ENDIAN_HANDLE_LONG( pDesc->status) & ACX_HOST_DESC_STAT_OWN) {
         ++NumOwnedDesc;
      }
      pDesc = whal_HostDesc_getNext( pDesc);
   }
   
   *OwnerPercentage = (NumOwnedDesc * 100)/(maxCurr ? maxCurr:1);

   return OK;
}

