/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalTxDesc.c
 *   PURPOSE: ACX100 Tx descriptor object
 *   
 ****************************************************************************/


#include "acx_hw.h"
#include "whalCommon.h"
#include "AcxDataIf.h"
#include "whalAcxAccess.h"
#include "whalTxDesc.h"
#include "AcxCommands.h"
#include "whalHostDesc.h"

/****************************************************************************
 *                      whal_txDesc_Init()
 ****************************************************************************
 * DESCRIPTION:	Initiate ACX100 Tx desctiptor (Set OWN and FIRST FRAG bits)
 *		The "Next" field is not set because it initiated once to point to the 
 *		next descriptor in the queue
 *
 * 
 * INPUTS:	
 *		TxDesc_T *pTxDesc	The descriptor address 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	
 ****************************************************************************/
void whal_txDesc_Init(TxDesc_T *pTxDesc)
{
	TxDesc_T localTxDesc;

   /* Initialize all fields to 0 */
	PAL_memSet(&localTxDesc, 0, sizeof(localTxDesc));

   /* Override only what is necessary. */
	localTxDesc.Ctl			= CTL_OWN;
	localTxDesc.Rate        = BIT_RATE_1MBPS;

   /* Skip Next field and copy */
	whal_acxMemCopy(4+(char *)pTxDesc, 4+(char *)&localTxDesc, sizeof(TxDesc_T)-4);
}

/****************************************************************************
 *                      whal_txDesc_reinit()
 ****************************************************************************
 * DESCRIPTION:	ReInitiate ACX100 Tx desctiptor (Set OWN and FIRST FRAG bits)
 * 
 * INPUTS:	
 *		TxDesc_T *pTxDesc	The descriptor address 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	
 ****************************************************************************/
extern int rei;

void whal_txDesc_reinit(TxDesc_T *pTxDesc)
{
/* Override only what is necessary */
   whal_acxMemSetU16( (UINT32)(&(pTxDesc->Ctl)) , CTL_OWN);
   whal_acxMemSetU16( (UINT32)(&(pTxDesc->Rate)), BIT_RATE_1MBPS);

  /* Place 0's in fields we do not use for setMpduInfo, for safety */
   whal_acxMemClrU32(&pTxDesc->ACXMemPtr);
   whal_acxMemClrU32(&pTxDesc->Reserved1[0]);
   whal_acxMemClrU16(&pTxDesc->Reserved2[0]);
   whal_acxMemClrU32(&pTxDesc->DriverWorkspace[8]);
   whal_acxMemClrU32(&pTxDesc->DriverWorkspace[12]);
   /* clear all status / failure bits */
   whal_acxMemClrU16(&pTxDesc->Status);
   whal_acxMemClrU16(&pTxDesc->RTSFailures);
   /* clear unused reserved */
   whal_acxMemClrU32(&pTxDesc->reserved5[0]);
   whal_acxMemClrU32(&pTxDesc->reserved5[4]);    
}

/****************************************************************************
 *                      whal_txDesc_SetMpduInfo()
 ****************************************************************************
 * DESCRIPTION:	set the Descriptors fileds before transmission
 *		FIRST_FRAG  - set only in the first fragment
 *		MORE_FRAG   - set in all fragments except the last fragment
 *		MpduLen		- The size of the MPDU
 *		AcxMemPtr	- The pointer to where the data placed on the ACX100 mem
 * 
 * INPUTS:	
 *		TxDesc_T *pTxDesc	The descriptor address
 *		int MpduAddr		The pointer to where the data placed on the ACX100 mem
 *		int MpduLen			The size of the MPDU
 *		int Frag			The current fragment number
 *		int NumFrags		The total number of fragments in the MSDU
 *		UINT16 Rate			The rate to use in the transmission
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	
 ****************************************************************************/
int whal_txDesc_SetMpduInfo
(
   TxDesc_T *pTxDesc,
   UINT8    *MpduAddr,
   int       MpduLen,
   int       Frag,
   int       NumFrags,
   UINT16    Rate,
   UINT32    MpduFlags,
   char     *DstMac
)
{
	/* Check Mpdu flags */
	if (MpduFlags & MPDU_FLAGS_RTS_SET) 
		whal_acxMemSetByteBits((UINT32)pTxDesc, TX_CTL2_BYTE_OFFSET, CTL2_RTS);
	
	if (MpduFlags & MPDU_FLAGS_ERP_PROTECTION_SET) 
		whal_acxMemSetByteBits((UINT32)pTxDesc, TX_CTL2_BYTE_OFFSET, CTL2_ERP);
	
	if (MpduFlags & MPDU_FLAGS_TI_PROTECTION_SET) 
		whal_acxMemSetByteBits((UINT32)pTxDesc, TX_CTL2_BYTE_OFFSET, CTL2_TI);

	whal_acxMemSetByteBits((UINT32)pTxDesc, TX_SW_ANTENNA_DIVERSITY_BYTE_OFFSET, 0);	

	if (MpduFlags & MPDU_FLAGS_SHORT_PREAMBLE_SET) 
		Rate |= 0x8000;
	
	whal_acxMemSetU16(&pTxDesc->Rate, Rate);
	/* save the Rate also in the workspace for later checking */
	whal_acxMemSetU16(&pTxDesc->DriverWorkspace[0], Rate);
	whal_acxMemSetU16(&pTxDesc->DriverWorkspace[2], *(UINT16 *)&DstMac[0]);
	whal_acxMemSetU16(&pTxDesc->DriverWorkspace[4], *(UINT16 *)&DstMac[2]);
	whal_acxMemSetU16(&pTxDesc->DriverWorkspace[6], *(UINT16 *)&DstMac[4]);
	whal_acxMemSetU16(&pTxDesc->MPDULen, MpduLen);
	whal_acxMemSetU32(&pTxDesc->ACXMemPtr, (UINT32)MpduAddr - ACX100_MEM_ADDRESS);

    
	return 0;
}

/****************************************************************************
 *                      whal_txDesc_SetMpduInfoMaster()
 ****************************************************************************
 * DESCRIPTION:	set the Descriptors fileds before transmission in master 
 *                mode
 *		FIRST_FRAG  - set only in the first fragment
 *		MORE_FRAG   - set in all fragments except the last fragment
 *		MpduLen		- The size of the MPDU
 *		AcxMemPtr	- The pointer to where the data placed on the ACX100 mem
 * 
 * INPUTS:	
 *		TxDesc_T *pTxDesc	The descriptor address
 *		UINT32 hostDescAddr The pointer to the host descriptor in SDRAM
 *		int MpduLen			The size of the MPDU
 *		int Frag			The current fragment number
 *		int NumFrags		The total number of fragments in the MSDU
 *		UINT16 Rate			The rate to use in the transmission
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	
 ****************************************************************************/
int whal_txDesc_SetMpduInfoMaster
(
   TxDesc_T *pTxDesc,
   UINT32    hostDescAddr,
   int       MpduLen,
   /*int       Frag,
   int       NumFrags,*/
   UINT16    Rate,
   UINT32    MpduFlags,
   char      *DstMac
)
{

#if 1
/*	printf("whal_txDesc_SetMpduInfoMaster: pTxDesc=0x%X, hostDescAddr = 0x%X, NumFrags = %d, MpduFlags = 0x%x, Rate = %d, DstMac[5] = %x\n",
		   (UINT32) pTxDesc, hostDescAddr, NumFrags, MpduFlags, Rate, (UINT8)DstMac[5]);		    		   
*/
   /* Declare local TxDesc */
   register UINT8 Ctl2;

    /* enable burst */
    Ctl2 = CTL2_BURST;
    /* Check Mpdu flags */
	if (MpduFlags & MPDU_FLAGS_RTS_SET) 
      Ctl2 |= CTL2_RTS;
	
	if (MpduFlags & MPDU_FLAGS_ERP_PROTECTION_SET) 
      Ctl2 |= CTL2_ERP;

#if 0 /* No support */	
	if (MpduFlags & MPDU_FLAGS_TI_PROTECTION_SET) 
      Ctl2 |= CTL2_TI;
#endif

   whal_acxMemSetByte((UINT32)pTxDesc, TX_CTL2_BYTE_OFFSET, Ctl2);

   if (MpduFlags & MPDU_FLAGS_SHORT_PREAMBLE_SET) 
		Rate |= 0x8000;

   whal_acxMemSetU16( &pTxDesc->Rate, Rate);
	/* save the Rate also in the workspace for later checking */
	whal_acxMemSetU16(&pTxDesc->DriverWorkspace[0], Rate);
	whal_acxMemSetU16(&pTxDesc->DriverWorkspace[2], *(UINT16 *)&DstMac[0]);
	whal_acxMemSetU16(&pTxDesc->DriverWorkspace[4], *(UINT16 *)&DstMac[2]);
	whal_acxMemSetU16(&pTxDesc->DriverWorkspace[6], *(UINT16 *)&DstMac[4]);

   /* clear ant. diversity field */
   whal_acxMemClrU8((UINT32)pTxDesc + TX_ANT_DIVERSITY_OFFSET);

   whal_acxMemSetU16(&pTxDesc->MPDULen, (UINT16)MpduLen);

	/* Host mem pointer points to the HOST SDRAM. The pointer is converted for the ACX */
#if( WLAN_BUS_TYPE == WLAN_BUS_TYPE_VLYNQ)
	whal_acxMemSetU32( &pTxDesc->HostMemPtr, VLYNQ_HOST2ACX111_SDRAM_PTR(hostDescAddr));
#elif( WLAN_BUS_TYPE == WLAN_BUS_TYPE_PCI)
	whal_acxMemSetU32( &pTxDesc->HostMemPtr, PCI_HOST2ACX111_SDRAM_PTR(hostDescAddr));
#else
#error   whalTxDesc.c -- Check bus type.
#endif

   /*whal_acxMemResetByteBits((UINT32)pTxDesc, TX_CTL_BYTE_OFFSET, CTL_OWN);*/

   return 0;
#else
/*	printf("whal_txDesc_SetMpduInfoMaster: pTxDesc=0x%X, hostDescAddr = 0x%X, NumFrags = %d, MpduFlags = 0x%x, Rate = %d, DstMac[5] = %x\n",
		   (UINT32) pTxDesc, hostDescAddr, NumFrags, MpduFlags, Rate, (UINT8)DstMac[5]);		    		   
*/
#if 0
   /* Reset first frag bit - by default the bit is set */
	whal_acxMemResetByteBits((UINT32)pTxDesc, TX_CTL_BYTE_OFFSET, CTL_FIRSTFRAG);
#endif

	/* Check Mpdu flags */
	if (MpduFlags & MPDU_FLAGS_RTS_SET) 
		whal_acxMemSetByteBits((UINT32)pTxDesc, TX_CTL2_BYTE_OFFSET, CTL2_RTS);
	
	if (MpduFlags & MPDU_FLAGS_ERP_PROTECTION_SET) 
		whal_acxMemSetByteBits((UINT32)pTxDesc, TX_CTL2_BYTE_OFFSET, CTL2_ERP);
	

	if (MpduFlags & MPDU_FLAGS_TI_PROTECTION_SET) 
		whal_acxMemSetByteBits((UINT32)pTxDesc, TX_CTL2_BYTE_OFFSET, CTL2_TI);

	if (MpduFlags & MPDU_FLAGS_SHORT_PREAMBLE_SET) 



		Rate |= 0x8000;
	whal_acxMemSetU16(&pTxDesc->Rate, Rate);

    /* clear ant. diversity field */
    whal_acxMemSetByte((UINT32)pTxDesc, TX_ANT_DIVERSITY_OFFSET, 0);

	/* save the Rate also in the workspace for later checking */
	whal_acxMemSetU16(&pTxDesc->DriverWorkspace[0], Rate);
	whal_acxMemSetU16(&pTxDesc->DriverWorkspace[2], *(UINT16 *)&DstMac[0]);
	whal_acxMemSetU16(&pTxDesc->DriverWorkspace[4], *(UINT16 *)&DstMac[2]);
	whal_acxMemSetU16(&pTxDesc->DriverWorkspace[6], *(UINT16 *)&DstMac[4]);
	whal_acxMemSetU16(&pTxDesc->DriverWorkspace[12], 0xffff); /* reserved3 */
	whal_acxMemSetU16(&pTxDesc->DriverWorkspace[14], 0); 	  /* both swAntDivs and reserved fields */

	whal_acxMemSetU16(&pTxDesc->MPDULen, (UINT16)MpduLen);
   

	/* Host mem pointer points to the HOST SDRAM. The pointer is converted for the ACX */
#if( WLAN_BUS_TYPE == WLAN_BUS_TYPE_VLYNQ)
	whal_acxMemSetU32(&pTxDesc->HostMemPtr, VLYNQ_HOST2ACX111_SDRAM_PTR(hostDescAddr));
#elif( WLAN_BUS_TYPE == WLAN_BUS_TYPE_PCI)
	whal_acxMemSetU32(&pTxDesc->HostMemPtr, PCI_HOST2ACX111_SDRAM_PTR(hostDescAddr));
#else
#error   whalTxDesc.c -- Check bus type.
#endif

   whal_acxMemSetByteBits((UINT32)pTxDesc, TX_CTL2_BYTE_OFFSET, CTL2_BURST);

   return 0;
#endif
}

/****************************************************************************
 *                      whal_txDesc_GetStatus()
 ****************************************************************************
 * DESCRIPTION:	Return the "Status" field from the Tx descriptor
 ****************************************************************************/
UINT8 whal_txDesc_GetStatus(TxDesc_T *pTxDesc, UINT8 *AckFail, UINT8 *RtsFail, UINT8 *RtsSucc)
{
#if 0	
	UINT8  StatusByte;
	UINT16 StatusWord;
	UINT16 RtsCounters;

	whal_acxMemCopy((char *)&StatusWord,  (char *)((UINT32)pTxDesc+TX_STATUS_WORD_OFFSET), 2);
	StatusByte = StatusWord >> 8;
	*AckFail = StatusWord & 0xff;

	whal_acxMemCopy((char *)&RtsCounters, (char *)((UINT32)pTxDesc+TX_STATUS_WORD_OFFSET+2), 2);
	*RtsFail = RtsCounters >> 8;
	*RtsSucc = RtsCounters & 0xff;

	return(StatusByte);
#endif
   
	register UINT16 StatusWord = whal_acxMemGetU16( (UINT32)pTxDesc + TX_STATUS_WORD_OFFSET);
    register UINT16 RtsCounters = whal_acxMemGetU16( (UINT32)pTxDesc + TX_STATUS_WORD_OFFSET+2);

   *AckFail = StatusWord & 0xff;
	*RtsFail = RtsCounters >> 8;
	*RtsSucc = RtsCounters & 0xff;

   /* return status byte */
   return( (UINT8)(StatusWord >> 8));

}

/****************************************************************************
 *                      whal_txDesc_GetRate()
 ****************************************************************************
 * DESCRIPTION:	Return the "Rate" field from the Tx descriptor
 ****************************************************************************/
UINT16 whal_txDesc_GetRate(TxDesc_T *pTxDesc)
{
	return ((UINT16)whal_acxMemGetU16(&pTxDesc->Rate));
}

/****************************************************************************
 *                      whal_txDesc_GetCtl()
 ****************************************************************************
 * DESCRIPTION:	Return the "Ctl" field from the Tx descriptor
 ****************************************************************************/
UINT8 whal_txDesc_GetCtl(TxDesc_T *pTxDesc)
{
	return(whal_acxMemGetByte((UINT32)pTxDesc, TX_CTL_WORD_OFFSET));
}

/****************************************************************************
 *                      whal_txDesc_SetCtl()
 ****************************************************************************
 * DESCRIPTION:	Set the "Ctl" field on the Tx descriptor
 ****************************************************************************/
void whal_txDesc_SetCtl(TxDesc_T *pTxDesc, UINT8 Val)
{
	whal_acxMemSetByte((UINT32)pTxDesc, TX_CTL_WORD_OFFSET, Val);
}

/****************************************************************************
 *                      whal_txDesc_Read()
 ****************************************************************************
 * DESCRIPTION:	Copy Tx descriptor from ACX100 to Host buffer
 *		Swap relevant fields only 
 * 
 * INPUTS:	
 *		TxDesc_T *pDstDesc	Host buffer to copy to 
 *		TxDesc_T *pSrcDesc	ACX100 Tx descriptor address 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK
 ****************************************************************************/
int whal_txDesc_Read(TxDesc_T *pDstDesc, TxDesc_T *pSrcDesc)
{
	whal_acxMemCopy( (char *)pDstDesc, (char *)pSrcDesc, sizeof(TxDesc_T));

	/* 
	 * Convert only 16 and 32 bit fields 
	 * (!!!) can copy only relevat fields
	 */
	pDstDesc->next		  = (UINT8 *)(ENDIAN_HANDLE_LONG((int)(pDstDesc->next)));
	pDstDesc->HostMemPtr  = (UINT8 *)(ENDIAN_HANDLE_LONG((int)(pDstDesc->HostMemPtr)));
	pDstDesc->ACXMemPtr   = (UINT32 *)(ENDIAN_HANDLE_LONG((int)(pDstDesc->ACXMemPtr)));
	pDstDesc->MPDULen     = ENDIAN_HANDLE_WORD(pDstDesc->MPDULen);

	return OK;
}

/****************************************************************************
 *                      whal_txDesc_ResetCtlOwn()
 ****************************************************************************
 * DESCRIPTION:	reset OWN bit in the "Ctl" field on the Tx descriptor
 ****************************************************************************/
void whal_txDesc_ResetCtlOwn(TxDesc_T *pTxDesc)
{
	whal_acxMemResetByteBits((UINT32)pTxDesc, TX_CTL_BYTE_OFFSET, CTL_OWN);
}

/****************************************************************************
 *                      whal_txDesc_GetHostNext()
 ****************************************************************************
 * DESCRIPTION:	Return the Next descriptor pointer
 ****************************************************************************/
TxDesc_T *whal_txDesc_GetHostNext(TxDesc_T *pTxDesc)
{
	return ( (TxDesc_T *) (whal_acxMemGetU32(&pTxDesc->next) + ACX100_MEM_ADDRESS) );
}

/****************************************************************************
 *                      whal_txDesc_SetHostNext()
 ****************************************************************************
 * DESCRIPTION:	set the "Next" field on the Tx descriptor
 ****************************************************************************/

void whal_txDesc_SetHostNext(TxDesc_T *pTxDesc, UINT32 HostAddr)
{
	whal_acxMemSetU32(&pTxDesc->next, HostAddr - ACX100_MEM_ADDRESS);
}

/****************************************************************************
 *                      whal_txDesc_GetAcxBufAddr()
 ****************************************************************************
 * DESCRIPTION:	Return the pointer to the frame data in the SDRAM memory
 *
 * NOTE:        The Tx Descriptor MUST be an image in SDRAM of the TxDesc 
 *              in SRAM
 ****************************************************************************/
UINT32 whal_txDesc_GetAcxBufAddr(TxDesc_T pTxDesc)
{

    /* MASTER MODE */

    UINT32 HostMemPtr,BlkPtr;

#if( WLAN_BUS_TYPE == WLAN_BUS_TYPE_VLYNQ)
    HostMemPtr = VLYNQ_ACX111_TO_HOST_SDRAM_PTR_CACHED(pTxDesc.HostMemPtr);
    BlkPtr = (UINT32)VLYNQ_ACX111_TO_HOST_SDRAM_PTR_CACHED(((HostDesc_T *)HostMemPtr)->blk_ptr);
    return BlkPtr;

#elif( WLAN_BUS_TYPE == WLAN_BUS_TYPE_PCI)
    HostMemPtr = PCI_ACX111_TO_HOST_SDRAM_PTR_UNCACHED(pTxDesc.HostMemPtr);
    BlkPtr = (UINT32)PCI_ACX111_TO_HOST_SDRAM_PTR_CACHED(((HostDesc_T *)HostMemPtr)->blk_ptr);
    return BlkPtr;
#else
#error   whalHostDesc.c -- Check bus type.
#endif

}        

/****************************************************************************
 *                      whal_txDesc_GetHostMemPtr()
 ****************************************************************************
 * DESCRIPTION:	Return the pointer to the host descriptor in the ACX100 memory
 ****************************************************************************/

UINT32 whal_txDesc_GetHostMemPtr(TxDesc_T *pTxDesc)
{
    return (whal_acxMemGetU32(&pTxDesc->HostMemPtr));
}

/****************************************************************************
 *                      whal_txDesc_Print()
 ****************************************************************************
 * DESCRIPTION:	Print the descriptor main fields
 ****************************************************************************/
int whal_txDesc_Print(TxDesc_T *pHwTxDesc ,char *buf)
{
	TxDesc_T localTxDesc;

	whal_acxMemCopy( (char *)&localTxDesc, (char *)pHwTxDesc, sizeof(TxDesc_T));
	localTxDesc.next		= (UINT8 *)(ENDIAN_HANDLE_LONG((int)(localTxDesc.next)));
	localTxDesc.ACXMemPtr   = (UINT32 *)(ENDIAN_HANDLE_LONG((int)(localTxDesc.ACXMemPtr)));
	localTxDesc.MPDULen     = ENDIAN_HANDLE_WORD(localTxDesc.MPDULen);

	return sprintf(buf,"AcxMemPtr=0x%04x, Next=0x%04x, MpduLen=0x%04x, Ctl=0x%02x:0x%02x, Stt=0x%02x\n", 
		    		(UINT32)localTxDesc.ACXMemPtr, (UINT32)localTxDesc.next, localTxDesc.MPDULen, localTxDesc.Ctl, localTxDesc.Ctl2, localTxDesc.Status);
}

