/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalTx.c
 *   PURPOSE: manages the transmission of MSDU to the ACX100
 *   
 ****************************************************************************/


#include "acx_hw.h"
#include "whalCommon.h"
#include "whal.h"
#include "whalTxQueue.h"
#include "whalTxBufs.h"
#include "whalInterrupt.h"
#include "whalAcxAccess.h"
#include "whalParams.h"
#include "memMngr.h"
#include "whalTx.h"
#include "whalTrace.h"
#include "whalHostDesc.h"
#include "whalHostQueue.h"
#include "whalInterrupt.h"
#include "drvDbMain.h"
#include "dda.h"
#include "pal.h"


/****************************************************************************
 *                      OBJECT DATA BASE + INTERNALS
 ****************************************************************************
 *
 ****************************************************************************/
extern UINT32 whal_txQueue_GetNumUsed(TxQueueId_T Qid);
extern void whal_apiRestartWatchDog_PacketTransmitted(void);

TxStatus_Callback_T TxStatusCB=NULL;
TxDataUser_Callback_T TxDataUserCB=NULL;
Recv4xWackInfo_Callback_T Recv4xWackInfoCB=NULL;
UINT8 LastRecv4xWackInfo = 0xaa;
int TxFrameCounter [NUM_TX_QUEUES];
HostQueue_Obj   *txHostQueueArray[NUM_TX_QUEUES];

PAL_spinLockHandle whalTxLock;

/****************************************************************************
 *                      whal_txSendMsdu()
 ****************************************************************************
 * DESCRIPTION:	Fragment MSDU into MPDUs and Send(Copy) to the ACX100.
 * 
 *		For each fragment, the function allocates ACX100 buffers from the 
 *		TxBufs object to copy the MPDU data blocks and allocate buffer 
 *		descriptors from the TxQueue object to point to the buffers and set 
 *		the appropriate bits to inform the ACX100 that the MPSU is ready for
 *		transmission.
 *		The function trigger TX_PROC interrupt to the ACX100 only after all 
 *		the MPDUs are ready in the ACX100 memory.
 *		The function also set the Queue indicator filed to inform the ACX100
 *		in which queue there is a bufferrs to process.
 *		Broadcast frames are not fragmented
 *
 * INPUTS:	
 *		TxQueueId_T TxQid	The queue id
 *		MEM_HANDLE *Msdu_Handle	
 *			The MSDU descriptor which contain Header block and link list of 
 *			Data blocks. The Header block contain the 802.11 header and can 
 *			have more data (LLC+SNAP for data frame or Management frame body
 *			for short frames. Each data block has offset filed which indicates
 *			where the data is actually started.
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK.
 ****************************************************************************/

int whal_txSendMsdu(TxQueueId_T TxQid, MEM_HANDLE Msdu_Handle, TxAttr_T *pTxAttr)
{
   TxDesc_T       *pTxDesc;
   mem_BD_T       *pBD;
   UINT32          totalSize, hostDescAddr;
   HostDescAttr_T  hostDscAttr;
   HostQueue_Obj  *pTxHostQ;
   dot11_header_T *pMacHdr;
   UINT32 	       msduSize, MpduSize;
   int             NumFrags;
   UINT32          MpduFlags=0;
   unsigned long   flags;
   mem_BD_T		   *firstBDptr;
   UINT32          tkipMoreBytes = 0;

   pTxHostQ =  txHostQueueArray[TxQid];

   if( Msdu_Handle == NULL ){
      
       printf("whal_txSendMsdu() -- ERROR! Msdu_Handle = NULL\n");
       return NOK;
   }

   PAL_intrLock(flags); 

   /* Check if there are free descriptors */
   if(pTxHostQ->NumOfDesc - pTxHostQ->CurrNumDescInQ < 2)
   {
	   printf("whal_txSendMsdu() - No free tx host descriptors\n");
       PAL_intrUnLock(flags); 
	   return NOK;
   }

   /*
    * Allocate Acx Tx descriptor
    */
   pTxDesc = whal_txQueue_Alloc(TxQid);
   if (pTxDesc == NULL)
   {
    /* This should never happend becuase we request for resources before we decide to send
       but if for some reasone there is no ACX descriptor then we must free the MSDU and 
       return. */

      printf("whal_txSendMsdu: ERROR, no more AcxDesc descriptors ...\n");  
      PAL_intrUnLock(flags); 
      return NOK;
   }

   /* Swap the frame control and the sequence control */
   pMacHdr = (dot11_header_T *)memMgr_MsduHdrAddr(Msdu_Handle);
   pMacHdr->fc = ENDIAN_HANDLE_WORD(pMacHdr->fc);
   /* (!!!) remove it because the ACX100 fill the sequence number field */
   pMacHdr->seqCtrl = ENDIAN_HANDLE_WORD(pMacHdr->seqCtrl);

   /*
    *  Insert every buffer to a host descriptor.
    */
   msduSize = memMgr_MsduDataSize( Msdu_Handle );

   pBD =  memMgr_MsduDataAddr( Msdu_Handle );
   if( pBD == NULL ){
        printf("whal_txSendMsdu() -- ERROR! pBD = NULL\n");
		PAL_intrUnLock(flags); 
        return NOK;
    }

   totalSize = 0;
   
   hostDescAddr = whal_HostQueue_getNextToInsertDescAddr( pTxHostQ);
   hostDscAttr.msduPtr = (UINT32) Msdu_Handle;

   NumFrags = 1;

 
   /* adding 4 bytes for IV after mac header */
   if(pTxAttr->Tkip == TRUE)
   {

       if( pBD->data == NULL ) /* Just to make sure. */
          pBD->data = pBD->dataBuf->data;

       if(pBD->length + WPA_IV_SIZE> DATA_BUF_LEN)
       {
           printf("whal_txSendMsdu: ERROR, no space for IV field  \n");
           PAL_intrUnLock(flags);
           return NOK;
       }

      /* Copy data with IV_SIZE offset, increasing BD length  */
      {
         UINT32  i;
         char   *pTmpSrc;
         char   *pTmpDst;
         UINT32  limit;

         limit   = pBD->length - sizeof( dot11_AP_header_T);
         pTmpSrc = pBD->data + pBD->dataOffset + pBD->length - 1;
         pTmpDst = pTmpSrc + WPA_IV_SIZE;
      
         /* copy data  with IV_SIZE offset */
         for( i=0; i < limit; i++ /*,pTmpSrc,pTmpDst*/)
            *pTmpDst-- = *pTmpSrc--;
      
         /* pad IV with 0 */
         for( i=0; i<WPA_IV_SIZE; i++)
            *pTmpDst-- = 0;
      }
      /* increase BD length */
      pBD->length += WPA_IV_SIZE;
      /* increase msdu size */
      memMgr_MsduDataSize( Msdu_Handle ) += WPA_IV_SIZE;
      /* adjust RTS threshold offset */
      tkipMoreBytes = 8;
   }/* endIf */

   firstBDptr = pBD;

    do {
      
      if( pBD->data == NULL ) /* Just to make sure. */
         pBD->data = pBD->dataBuf->data;

      hostDscAttr.blk_ptr    = (UINT32)( pBD->data) + pBD->dataOffset;
      hostDscAttr.blk_offset = 0;
      hostDscAttr.ctrl       = 0;
      /* Setting the buffer length to be 4 byte aligned */
      hostDscAttr.blk_len    = (pBD->length + 3) & 0xfffc;
      hostDscAttr.status     = 0;

      whal_HostQueue_insert( pTxHostQ, &hostDscAttr );
      /* Only the first descriptor has to keep MSDU pointer,
         for the "free" callback */
      hostDscAttr.msduPtr = 0;
      totalSize +=  pBD->length;

	  PAL_cacheFlush( 0, (void *)hostDscAttr.blk_ptr, hostDscAttr.blk_len);

      pBD = pBD->nextBDPtr;
      /*
       * TODO: Check up and clean the mess in case of mismatch.
       * Exception hazard, or infinite loop. Report if something gets wrong.
       */
      
   } while (pBD);

   MpduSize = totalSize;

   /*
    * prepare Mpdu flags
    */
   if (pTxAttr->ShortPreamble)
      MpduFlags |= MPDU_FLAGS_SHORT_PREAMBLE_SET;

   /* First check the RtsThreshold, if no need for RTS/CTS check if protection needed */
   if( ((MpduSize+tkipMoreBytes) > (whal_ParamsGetGenParams(0)->RtsThreshold-FCS_FIELD_SIZE)) &&
		(pTxAttr->Broadcast == FALSE))
   {
		  MpduFlags |= MPDU_FLAGS_RTS_SET;
   }
   else
   {
	   switch(pTxAttr->ProtectionType)
	   {
	   case TXATTR_PROTECTION_RTS_CTS:
		   MpduFlags |= MPDU_FLAGS_RTS_SET;
		   break;
	   case TXATTR_PROTECTION_CTS_TOSELF:
		   MpduFlags |= MPDU_FLAGS_ERP_PROTECTION_SET;
		   break;
	   case TXATTR_PROTECTION_TI_TRICK:
		   MpduFlags |= MPDU_FLAGS_TI_PROTECTION_SET;
		   break;
	   default: /* no protection in TxAttr */
		   MpduFlags |= TXATTR_PROTECTION_NONE;
		   break;
	   }
   }


  /*
   * set the descriptor fields (OWN, FIRST_FRAG, MORE_FRAGS, RTS, RATE, ...)
   */
#if 0
   printf("whal_txSendMsdu() --\n pTxDesc = %p\n hostDescAddr = %p\n MpduSize = %u\n Frag = %d\n NumFrags = %u\n Rate = 0x%x\n Flags = 0x%08x\n",
      pTxDesc, hostDescAddr, MpduSize, Frag, NumFrags, pTxAttr->RateWithFallback, MpduFlags);
#endif

   whal_txDesc_SetMpduInfoMaster(pTxDesc, hostDescAddr, MpduSize, /*Frag, NumFrags,*/ pTxAttr->RateWithFallback, MpduFlags, pMacHdr->address1.addr);
   
   /* We should only reset CTL_OWN bit to make ACX take over the descriptor,
   other bits are reserved (0) */
   whal_acxMemClrU8( (UINT32)pTxDesc + TX_CTL_BYTE_OFFSET);

#ifdef TIWLAN_DEBUG
   if (whal_trcIsEnable())
      whal_trcAddTx(pTxDesc, (UINT32) (firstBDptr->data + firstBDptr->dataOffset),"SEND");
#endif
   /*
    * trigger the ACX100 once for all the MPDUs
    */
   whal_acxIntrTriggerTxProc();

   /*
    * Increment number of Tx MSDU
    */
   TxFrameCounter[TxQid]++;
   PAL_intrUnLock(flags); 

   bm_trace(11,0,0);

   return 0;
}

/****************************************************************************
 *                      whal_txCheckSize()
 ****************************************************************************
 * DESCRIPTION:	Check if there is enough ACX100 memory/descriptors for the MSDU len
 * 
 * INPUTS:	
 *		UINT32	Len		The length of the requested buffer
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_txCheckSize(TxQueueId_T TxQid, UINT32 MsduLen)
{
    UINT32 NumOfHostDescInQ;

	/*
	 * check ACX100 descriptors
	 */
	if (whal_txQueue_GetNumFree(TxQid) <= 0)
	{
#if 0 /* This can happend no need to print error msg */
		puts("whal_txCheckSize() -- Out of Acx descriptors. \n");
#endif
		return NOK;
	}

    whal_HostDesc_getFreeBufsNum( txHostQueueArray[TxQid], &NumOfHostDescInQ );
    if( NumOfHostDescInQ < 2 ){
        puts("whal_txCheckSize() -- Out of Host descriptors. \n");
        return NOK;
    }

	return OK;
}

/****************************************************************************
 *                      whal_txGetBufsNum()
 ****************************************************************************
 * DESCRIPTION:	Check if there is enough ACX100 memory/descriptors for the MSDU len
 *              and return the FreeHostDescNum and CurrUsedAcxDescNum
 * 
 * INPUTS:	
 *		TxQid
 * 
 * OUTPUT:	FreeHostDescNum - Number of free descriptors in the Host Queue
 *          CurrUsedAcxDescNum - Number of currently used descriptors in the ACX Queue
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_txGetBufsNum (TxQueueId_T TxQid, UINT32 *FreeHostDescNum, UINT32 *CurrUsedAcxDescNum)
{
    /* Get number of Host Q free descriptors */
    whal_HostDesc_getFreeBufsNum (txHostQueueArray[TxQid], FreeHostDescNum );
    if(*FreeHostDescNum < MIN_NUM_OF_BDs_IN_MSDU)
	{
        return NOK;
	}
    
    if (whal_txQueue_GetNumFree(TxQid) == 0)
		return NOK;
    
    /* Get number of ACX Q currently used descriptors */
    *CurrUsedAcxDescNum = whal_txQueue_GetNumUsed(TxQid);

    return OK;
}

/****************************************************************************
 *                      whal_txGetHostQsize()
 ****************************************************************************
 * DESCRIPTION:	Get the Host Queue Size
 * 
 * INPUTS:	
 *		TxQid
 * 
 * OUTPUT:	
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_txGetHostQsize(TxQueueId_T TxQid, UINT32 *queueSize)
{
	*queueSize = txHostQueueArray[TxQid]->NumOfDesc;
	return OK;
}

/****************************************************************************
 *                      whal_txInit()
 ****************************************************************************
 * DESCRIPTION:	Initiate the Tx queues objects and the Tx bufs object.
 *
 * INPUTS:	
 *		TxQueueId_T TxQid	The queue id
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
int whal_txInit()
{
	UINT8			WoneIndex, TotalWones, LocalQid, WoneTxQueues;
	TxQueueId_T 	TxQid;
	
	TxStatusCB = NULL;
   
	wdrv_dbGetNumSupWone(&TotalWones);
	for (WoneIndex = 0,TxQid = 0; WoneIndex < TotalWones; WoneIndex++)
	{
		wdrv_dbGetNumSupWoneQueues( WoneIndex, &WoneTxQueues );
		
		for (LocalQid=0; LocalQid<WoneTxQueues; LocalQid++,TxQid++)                              
		{
			TxFrameCounter[TxQid] = 0;
   
		   /* Init queue in ACX SRAM */
			whal_txQueue_Init(TxQid);
		   /* Init host descriptors queue */
			txHostQueueArray[TxQid] =  whal_HostQueue_init( whal_ParamsGetDmaParams(0)->NumTxBlocks, 0 );
		}
	}

	PAL_spinLock_init(&whalTxLock);
   
	return OK;
}

 /****************************************************************************
 *                      whal_txMasterUnload()
 ****************************************************************************
 * DESCRIPTION:	Unload the Tx Host Queue.
 *
 * INPUTS:	None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
void whal_txMasterUnload()
{
	TxQueueId_T 	TxQid = 0;
	UINT8 			WoneIndex, LocalQid, TotalWones, WoneTxQueues;
	
   	wdrv_dbGetNumSupWone(&TotalWones);
   	
	for (WoneIndex = 0; WoneIndex < TotalWones; WoneIndex++)
	{
		wdrv_dbGetNumSupWoneQueues( WoneIndex, &WoneTxQueues );
		
		for (LocalQid=0; LocalQid<WoneTxQueues; LocalQid++,TxQid++)                              
		{                                                                  
			whal_HostQueue_unload(txHostQueueArray[TxQid], 0);
			txHostQueueArray[TxQid] = NULL;
		}	
	}
}

/****************************************************************************
 *                      whal_txClean()
 ****************************************************************************
 * DESCRIPTION:	Clean the already transmitted Queue descriptors and buffers.
 *		The function check for TX_PROC interrupt and calls the TxQueue object 
 *		to reclaim all transmitted Queues descriptors and Data buffers
 *
 * INPUTS:	
 *		TxQueueId_T TxQid	The queue id
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
int whal_txClean(TxQueueId_T TxQid)
{
	UINT8  WoneIndex, LocalQid, TotalWones, WoneTxQueues;
	
	if (TxQid == ALL_TX_QUEUES)
	{
	   	wdrv_dbGetNumSupWone(&TotalWones);
   	
		for (WoneIndex = 0, TxQid = 0; WoneIndex < TotalWones; WoneIndex++)
		{
			wdrv_dbGetNumSupWoneQueues( WoneIndex, &WoneTxQueues );
		
			for (LocalQid=0; LocalQid<WoneTxQueues; LocalQid++,TxQid++)                              
	    	{                                                                  
				whal_txQueue_Free(TxQid, txHostQueueArray[TxQid]);
			}
		}	
	}
	else
   /* Clean ACX queue */
		whal_txQueue_Free( TxQid, txHostQueueArray[TxQid] );
	

	return OK;
}

/****************************************************************************
 *                      whal_txCleanAll()
 ****************************************************************************
 * DESCRIPTION:	Free Tx unicast and broadcast host queue MSDUs
 *
 * INPUTS:	
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
int whal_txCleanAll(void)
{
	TxQueueId_T TxQid;
	UINT8 		WoneIndex, LocalQid, TotalWones, WoneTxQueues;
	
	wdrv_dbGetNumSupWone(&TotalWones);
   	
	for (WoneIndex = 0, TxQid = 0; WoneIndex < TotalWones; WoneIndex++)
	{
		wdrv_dbGetNumSupWoneQueues( WoneIndex, &WoneTxQueues );
		
		for (LocalQid=0; LocalQid<WoneTxQueues; LocalQid++,TxQid++)                              
	   	{                                                                  
			whal_HostQueue_CleanAll(txHostQueueArray[TxQid]);
		}
	}	

	return OK;
}

/****************************************************************************
 *                      whal_txHandleTxData()
 ****************************************************************************
 * DESCRIPTION:	Clean the already transmitted Queue descriptors and buffers.
 *		The function calls the TxQueue object 
 *		to reclaim all transmitted Queues descriptors and Data buffers
 *
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
void whal_txHandleTxData()
{
#if 0
	UINT8			WoneIndex, TotalWones, LocalQid, WoneTxQueues;
	TxQueueId_T 	TxQid;
	
	wdrv_dbGetNumSupWone(&TotalWones);
	for (WoneIndex = 0,TxQid = 0; WoneIndex < TotalWones; WoneIndex++)
	{
		wdrv_dbGetNumSupWoneQueues( WoneIndex, &WoneTxQueues );
		for (LocalQid=0; LocalQid<WoneTxQueues; LocalQid++,TxQid++)                              
		{
			whal_txClean(TxQid);
		}
	}
#endif	
	/* call user defined callback */
	if (TxDataUserCB != NULL)
		TxDataUserCB();

	whal_apiRestartWatchDog_PacketTransmitted();
}

/****************************************************************************
 *                      whal_txSetTxDataUserCB()
 ****************************************************************************
 * DESCRIPTION:	Set the user callback for TX_DATA Interrupt.
 *
 * INPUTS:	
 *		TxDataUser_Callback_T externalTxDataUserCB		The callback address
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
int whal_txSetTxDataUserCB(TxDataUser_Callback_T externalTxDataUserCB)
{
	TxDataUserCB = externalTxDataUserCB;

	return OK;
}

/****************************************************************************
 *                      whal_txReset()
 ****************************************************************************
 * DESCRIPTION:	Reset the object and the requested tx queues and buffers
 *
 * INPUTS:	
 *		TxQueueId_T TxQid	The queue id
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
int whal_txReset(TxQueueId_T TxQid)
{
#if 0
	if (TxQid == ALL_TX_QUEUES)
	{
		whal_txQueue_Reset(UcastTxQid);
		whal_txQueue_Reset(BcastTxQid);
		whal_txBufs_Reset();
	}
	else
		whal_txQueue_Reset(TxQid);
#endif
	return OK;
}

/****************************************************************************
 *                      whal_txSetStatusCB()
 ****************************************************************************
 * DESCRIPTION:	Set the callback for the return Tx status.
 *
 * INPUTS:	
 *		TxStatus_Callback_T externalTxStatusCB		The callback address
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
int whal_txSetStatusCB(TxStatus_Callback_T externalTxStatusCB)
{
	TxStatusCB = externalTxStatusCB;

	return OK;
}

/****************************************************************************
 *                      whal_txSetStatus()
 ****************************************************************************
 * DESCRIPTION:	update(callback) Tx status of the transmitted frame.
 * 
 * INPUTS:	
 *		UINT8 *DestMacAddress	The destination MAC address of the tx frame
 *		UINT16 ActualTxRate		The actual transmitted rate
 *		UINT8 TxStatus			The status of the transmission
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK.
 ****************************************************************************/
void whal_txSetStatus(UINT8 *DestMacAddress, UINT16 ActualTxRate, UINT8 TxStatus)
{
	if (TxStatusCB != NULL)
		TxStatusCB(DestMacAddress, ActualTxRate, TxStatus);
}

/****************************************************************************
 *                      whal_txSet4xWackInfoCB()
 ****************************************************************************
 * DESCRIPTION:	Set the callback for the return Wack Info.
 *
 * INPUTS:	
 *		4xWackInfo_Callback_T external4xWackInfoCB		The callback address
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
int whal_txSetRecv4xWackInfoCB(Recv4xWackInfo_Callback_T externalRecv4xWackInfoCB)
{
	Recv4xWackInfoCB = externalRecv4xWackInfoCB;

	return OK;
}

/****************************************************************************
 *                      whal_txSetRecv4xWackInfo()
 ****************************************************************************
 * DESCRIPTION:	update(callback) Tx status of the transmitted frame.
 * 
 * INPUTS:	
 *		UINT8 *DestMacAddress	The destination MAC address of the tx frame
 *		UINT8 WackInfo			The Wack info from the FW
 *		UINT8 TxStatus			The status of the transmission
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK.
 ****************************************************************************/
void whal_txSetRecv4xWackInfo(UINT8 *DestMacAddress, UINT8 Recv4xWackInfo)
{
	if ( (Recv4xWackInfoCB != NULL) && (Recv4xWackInfo != LastRecv4xWackInfo) )
	{
		LastRecv4xWackInfo = Recv4xWackInfo;
		Recv4xWackInfoCB(DestMacAddress, Recv4xWackInfo);
	}
}

void whal_txGetRecv4xWackParams(UINT32 *CallbackAddr, UINT8 *Recv4xWackInfo)
{
	*Recv4xWackInfo = LastRecv4xWackInfo;
	*CallbackAddr = (UINT32)Recv4xWackInfoCB;
}

/****************************************************************************
 *                      whal_txTxFrameCount()
 ****************************************************************************
 * DESCRIPTION:	Return the Tx frame counter
 ****************************************************************************/
int whal_txTxFrameCount(TxQueueId_T TxQid)
{
	return (TxFrameCounter[TxQid]);
}

/****************************************************************************
 *                      whal_txInfoPrint()
 ****************************************************************************
 * DESCRIPTION:	Print Tx queue/bufs info.
 *
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
int whal_txInfoPrint(int Qid, BOOL QprintEn,char *buf)
{
	UINT8			WoneIndex, TotalWones, LocalQid, WoneTxQueues, Type, Priority, TotalQueues;
	TxQueueId_T 	TxQid;
	int len = 0;

	wdrv_dbGetNumSupWone(&TotalWones);
	for (WoneIndex = 0, TotalQueues = 0; WoneIndex < TotalWones; WoneIndex++)
	{
		wdrv_dbGetNumSupWoneQueues( WoneIndex, &WoneTxQueues );
		TotalQueues +=	WoneTxQueues;
	}

   if( TotalQueues <= Qid ) return 0;

   len+=whal_txQueue_InfoPrint(Qid,buf);

	for (WoneIndex = 0,TxQid = 0; WoneIndex < TotalWones; WoneIndex++)
	{
		wdrv_dbGetNumSupWoneQueues( WoneIndex, &WoneTxQueues );

		for (LocalQid=0; LocalQid<WoneTxQueues; LocalQid++,TxQid++)
		{
			if (TxQid == Qid)
			{
				wdrv_dbGetAcxQindex( WoneIndex, LocalQid, &Type, &Priority );
			}
		}
	}

   if( Type )
   {
      len+=sprintf(buf+len," Broadcast Host Queue:");
      len+=sprintf(buf+len,"=======================");
   }
   else
   {
      len+=sprintf(buf+len," Unicast Host Queue:");
      len+=sprintf(buf+len,"=====================");
   }
   
   if(QprintEn)
	   len+=whal_HostQueue_show( txHostQueueArray[Qid],buf );
   
   return len;
}

 
