/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalTxQueue.c
 *   PURPOSE: ACX100 Tx Queues manager
 *		The object manages Unicast and Broadcast Tx queues. each queue consists
 *		of MPDU descriptors located in the ACX100 memory.
 *		The Object holds 2 pointers (pFree, pAlloc) to manage the alloc/free
 *		of ACX100 descriptors which are orgenized as a link list
 *		functions: 
 *			Allocation - is from pFree pointer
 *			Free/Reclaim - is from pAlloc pointer.
 *   
 ****************************************************************************/

#include "whalCommon.h"
#include "whal.h"
#include "AcxCommands.h"
#include "AcxDataIf.h"
#include "whalParams.h"
#include "whalAcxAccess.h"
#include "whalTx.h"
#include "whalTxQueue.h"
#include "whalTxBufs.h"
#include "whalTrace.h"
#include "whalHostQueue.h"
#include "whalTxDesc.h"
#include "drvDbMain.h"


/****************************************************************************
 *                      OBJECT DATA BASE
 ****************************************************************************
 *
 *	ACX100 Mpdu descriptor
 *		The ACX100 descriptor hold MPDU information (Size, Tx rate, ...) and 
 *		a pointer to the ACX100 data blocks to be transmitted
 *		pNext and Control(for LAST block bit) and are initiate by the ACX100 
 *		as a link list.
 *
 * Local Data Base
 *		TxQueueDb[NUM_TX_QUEUES]	
 *				Holds the Queues objects data bases. The Queues are:
 *					1. Unicast queue
 *					2. Broadcast queue
 *		Each queue is handled as link list where pFree points always to free 
 *		descriptor on the queue and pAlloc points to allocated descriptors 
 *		that after transmission should be reclaimed
 *
 ****************************************************************************/

TxQueue_T TxQueueDb[NUM_TX_QUEUES];

QueueIndicator_T *pTxQIndicator;
QueueIndicator_T *pTxHostQIndicator;

extern PAL_spinLockHandle whalTxLock;
/****************************************************************************
 *                      whal_txQueue_Alloc()
 ****************************************************************************
 * DESCRIPTION:	Allocate ACX100 descripto from the pFree list
 * 
 * INPUTS:	
 *		TxQueueId_T Qid		The Id of the Queue
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
TxDesc_T *whal_txQueue_Alloc(TxQueueId_T Qid)
{
	TxQueue_T *pTxQ = &TxQueueDb[Qid];
	TxDesc_T  *pReturnDesc = NULL;

	if (pTxQ->NumFree <= 0)
	{
		printf("whal_txQueue_Alloc: WARNNING no more descriptors\n");
		return NULL;
	}

	if ( (whal_txDesc_GetCtl(pTxQ->pFree) & CTL_OWN) == 0)
	{
		printf("whal_txQueue_Alloc: ERROR Desc in the free list but not CTL_OWN ?!\n");
		return NULL;
	}

	/* descriptor address to return */
	pReturnDesc = pTxQ->pFree;

	/* point to the next free descriptor */
	pTxQ->pFree = whal_txDesc_GetHostNext(pTxQ->pFree);
	pTxQ->NumFree--;

	return pReturnDesc;
}			

/****************************************************************************
 *                      whal_txQueue_Alloc()
 ****************************************************************************
 * DESCRIPTION:	Free ACX100 descriptor from the pAlloc list
 *		The function also free data blocks pointed by AcxMemPtr field
 *		by calling the TxBufs object
 * 
 * INPUTS:	
 *		TxQueueId_T Qid		The Id of the Queue
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
#define DMA_IN_PROGRESS                0x8                   /* DMA is currently in progress */
#define DMA_NO_ERROR                   0x0                   /* no error */
#define DMA_TARGET_ABORT               0x1                   /* target abort */
#define DMA_MASTER_ABORT               0x2
#define DMA_CLEARED_ABORT              0x3                   /* request cleared prior to completion */
#define DMA_HOST_END_ABORT             0x4                   /*unexpected end of host descriptor list */
#define DMA_ACX_END_ABORT              0x5                   /* unexpected end of local memory list */ 
#define DMA_ERR_MASK                   0x7                   /* 3 bits of error information */

int wDbg_PrintWack=0x99;
int whal_txQueue_Free(TxQueueId_T Qid, HostQueue_Obj* phostQueue)
{
	register TxQueue_T *pTxQ = &TxQueueDb[Qid];

	UINT8 TxStatus;
	UINT16 ActualTxRate;
	UINT8 DestMacAddress[6];
	UINT8 AckFail, RtsFail, RtsSucc;

   /*PAL_spinLock(&whalTxLock);*/ /* START OF CRITICAL SECTION */
   
   /*whal_apiPrintAcxStatistics();*/

	while (pTxQ->NumDesc > pTxQ->NumFree)
	{
	    register TxDesc_T *pAlloc = pTxQ->pAlloc;
	    UINT16 SendTxRate;

		if (pAlloc == 0)
		{
         /*
			logMsg("whal_txQueue_Free: FATAL ERROR, pTxQ->pAlloc is NULL\n",0,0,0,0,0,0);
         */
			/*PAL_spinUnLock(&whalTxLock);*/  /* END OF CRITICAL SECTION */
			printf("whal_txQueue_Free error 1\n");
			return NOK;
		}

		/*
		 * stop if the ACX100 didn't send yet the first allocated MPDU descriptor, will occur at 1st time. 
		 */
		if 	((whal_txDesc_GetCtl(pAlloc) & (CTL_OWN | CTL_ACXDONE)) != (CTL_OWN | CTL_ACXDONE))
        {
			/*printf("whal_txQueue_Free() - ERROR ACX Descriptore own=ACX\n");*/
            break;
        }

#ifdef TIWLAN_DEBUG
		if (whal_trcIsEnable())
			whal_trcAddTc(pAlloc, " END");
#endif
		/* 
		 * Increment counters - (!!!) add error threshold for FATAL ERROR and reboot
		 */
		TxStatus = whal_txDesc_GetStatus(pAlloc, &AckFail, &RtsFail, &RtsSucc);

		TxStatus = pAlloc->Status;
		/*printf("TxStatus = %d, AckFail = %d, RtsFail = %d, RtsSucc = %d\n",(UINT32)TxStatus,(UINT32)AckFail, (UINT32)RtsFail, (UINT32)RtsSucc);*/

		pTxQ->AckFailCounter += AckFail;
		pTxQ->RtsFailCounter += RtsFail;
		pTxQ->RtsSuccCounter += RtsSucc;
		if (AckFail > 0)
			pTxQ->RetryCounter++;
		if (AckFail > 1)
			pTxQ->MultipleRetryCounter++;

		if ( TxStatus == STAT_TX_SUCCESS)
		{
			pTxQ->SuccCounter++;
		}
		else
      {
			if( TxStatus & STAT_RETRY)
				pTxQ->RetryFailCounter++;
			else
				pTxQ->OtherFailCounter++;

         if( TxStatus & STAT_DMA_ERR)
         {
            UINT32 dma_st;    /* Track what is wrong with the DMA */
   
            pTxQ->DmaErrorCounter++;

            /* Monitor DMA#1 */
            dma_st = whal_acxRegRead( /*DMA_STS1 */ 0x076c );
            /*
            logMsg("DMA error 0x%08x\n", dma_st, 0,0,0,0,0);
            */
            switch( dma_st)
            {
            case  DMA_IN_PROGRESS   : pTxQ->DmaErrorInProgress++    ;  break;
            case  DMA_NO_ERROR      : pTxQ->DmaErrorNoErr_spur++    ;  break;
            case  DMA_TARGET_ABORT  : pTxQ->DmaErrorTargetAbort++   ;  break;
            case  DMA_MASTER_ABORT  : pTxQ->DmaErrorMasterAbort++   ;  break;
            case  DMA_CLEARED_ABORT : pTxQ->DmaErrorClearedAbort++  ;  break;
            case  DMA_HOST_END_ABORT: pTxQ->DmaErrorHostEndAbort++  ;  break;
            case  DMA_ACX_END_ABORT : pTxQ->DmaErrorAcxEndAbort++   ;  break;
            default:                  pTxQ->DmaErrorUnspecified++   ;  break;
            }
         }

         if( TxStatus & STAT_MSDU_TIMEOUT)
            pTxQ->TimeoutErrorCounter++;

      }
#if 0
		printf("---------------------------------------------------------\n");
		whal_txQueue_Print(Qid);
		whal_txInfoPrint(Qid, 0);
		whal_apiPrintAcxStatistics();
		printf("---------------------------------------------------------\n");
#endif

        /*
		 * Call Rate Adaptaion if needed
		 */
		SendTxRate = whal_acxMemGetU16((UINT32)&pAlloc->DriverWorkspace[0]);
		ActualTxRate = whal_txDesc_GetRate(pAlloc);
		*(UINT16 *)&DestMacAddress[0] = whal_acxMemGetU16((UINT32)&pAlloc->DriverWorkspace[2]);
		*(UINT16 *)&DestMacAddress[2] = whal_acxMemGetU16((UINT32)&pAlloc->DriverWorkspace[4]);
		*(UINT16 *)&DestMacAddress[4] = whal_acxMemGetU16((UINT32)&pAlloc->DriverWorkspace[6]);

		if ((ActualTxRate != SendTxRate) || (TxStatus == STAT_RETRY))
		{
			whal_txSetStatus(DestMacAddress, ActualTxRate, TxStatus);
		}


        if( phostQueue ->CurrNumDescInQ )
            whal_HostQueue_freeMsdu( phostQueue );
           /* Free the Host Descr and MSDU */
  
		/*
		 * re-initiate all descriptor fields (except pNext field)
		 */
		whal_txDesc_reinit(pAlloc);
		pTxQ->NumFree++;

		/* move to the next descriptor (cyclic) */
		pTxQ->pAlloc = whal_txDesc_GetHostNext(pAlloc);
	}

    /*PAL_spinUnLock(&whalTxLock);*/  /* END OF CRITICAL SECTION */
	return OK;
}

/****************************************************************************
 *                      whal_txQueue_Init()
 ****************************************************************************
 * DESCRIPTION:	Initiate the selected Queue object 
 *		Note that in this stage there is no ACX100 address
 * 
 * INPUTS:	
 *		TxQueueId_T Qid		 The Id of the Queue 
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_txQueue_Init(TxQueueId_T Qid)
{
	TxQueue_T *pTxQ = &TxQueueDb[Qid];
	UINT8		WoneIndex, TotalWones, WoneTxQueues, LocalQid;
	TxQueueId_T PlainQid = 0;
	
	PAL_memSet((void *)pTxQ, 0, sizeof(*pTxQ));

	wdrv_dbGetNumSupWone(&TotalWones);
	for (WoneIndex = 0; WoneIndex < TotalWones; WoneIndex++)
	{
		wdrv_dbGetNumSupWoneQueues( WoneIndex, &WoneTxQueues );
		
		for (LocalQid=0; LocalQid<WoneTxQueues; LocalQid++,PlainQid++)                              
		{
			if ( Qid == PlainQid )
		{
			pTxQ->NumDesc = whal_ParamsGetDmaParams(WoneIndex)->TxNumDesc[LocalQid];	
				return OK;
			}
		}
	}
	
	return NOK;
}

/****************************************************************************
 *                      whal_txQueue_Configure()
 ****************************************************************************
 * DESCRIPTION:	Configure the selected Queue object and initiate the Discroptors
 *		Queue Link list in the ACX100 memory
 * 
 * INPUTS:	
 *		TxQueueId_T Qid		       The Id of the Queue
 *		UINT32 TxQueue_OffsetAddr  The ACX100 offset address of the queue
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
UINT32 whal_txQueue_Config(TxQueueId_T Qid, UINT32 TxQueue_OffsetAddr)
{
	TxQueue_T *pTxQ = &TxQueueDb[Qid];


	pTxQ->NumFree = pTxQ->NumDesc;

	/*
	 * Save parameters
	 */
	pTxQ->HwBaseAddr = (TxDesc_T *)(TxQueue_OffsetAddr + ACX100_MEM_ADDRESS );
	pTxQ->pFree = pTxQ->HwBaseAddr;
	pTxQ->pAlloc = pTxQ->HwBaseAddr;

	printf("whal_txQueue_Init: [Qid=%1d] HwBaseAddr=0x%08x, NumDesc=%d\n", Qid, (UINT32)pTxQ->HwBaseAddr, pTxQ->NumDesc);	

	return (whal_txQueue_GetSize(Qid));
}

/****************************************************************************
 *                      whal_txIndicatorSetAddr()
 ****************************************************************************
 * DESCRIPTION:	Set the QueueIndicator address
 * 
 * INPUTS:	
 *		UINT32 QueueIndicatorAddress	The Indicator address
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void whal_txIndicatorSetAddr(UINT32 QueueIndicatorAddress)
{
	pTxQIndicator = (QueueIndicator_T *)(QueueIndicatorAddress+ACX100_MEM_ADDRESS);
	whal_acxMemClrU32(&pTxQIndicator->indicator); 
	whal_acxMemClrU16(&pTxQIndicator->fw_lock); 
	whal_acxMemClrU16(&pTxQIndicator->host_lock);

	pTxHostQIndicator = (QueueIndicator_T *)(QueueIndicatorAddress+ACX100_MEM_ADDRESS+sizeof(QueueIndicator_T));
	whal_acxMemClrU32(&pTxHostQIndicator->indicator); 
	whal_acxMemClrU16(&pTxHostQIndicator->fw_lock); 
	whal_acxMemClrU16(&pTxHostQIndicator->host_lock);
}

/****************************************************************************
 *                      whal_txIndicatorSet()
 ****************************************************************************
 * DESCRIPTION:	Set the specific Queue Indicator 
 *		Wait for FW to unlock 
 *		Lock from host side
 *		Set the indicator
 *		Unlock from host side
 * 
 * INPUTS:	
 *		TxQueueId_T Qid		       The Id of the Queue
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_txIndicatorSet(TxQueueId_T Qid)
{
	int Count;

	/* Wait for FW to unlock */
	Count = 0;
	while( whal_acxMemGetRawU16( &pTxQIndicator->fw_lock))
	{
		if (Count++ > QIND_MAX_COUNT)
		{
			printf("whal_txIndicatorSet: FATAL_ERROR firmware lock the queue indicator[1]\n");
			return NOK;
		}
		PAL_sleepLong(QIND_DELAY_VALUE);
	}

	/* Host Lock*/
	whal_acxMemSetU16(&pTxQIndicator->host_lock, 1);

	/* Check FW Lock again, just in case we were in race ! */
	Count = 0;
	while( whal_acxMemGetRawU16( &pTxQIndicator->fw_lock))
	{
		if (Count++ > QIND_MAX_COUNT)
		{
			printf("whal_txIndicatorSet: FATAL_ERROR firmware lock the queue indicator[2]\n");
			return NOK;
		}
		PAL_sleepLong(QIND_DELAY_VALUE);
	}

	/* Set the specific queue indicator */
	whal_acxMemSetU32_Bits(&pTxQIndicator->indicator, (0x1 << Qid));

	/* Host UnLock */
	whal_acxMemClrU16( &pTxQIndicator->host_lock);

	return OK;
}

/****************************************************************************
 *                      whal_txQueue_GetSize()
 ****************************************************************************
 * DESCRIPTION:	Return the size of the Tx queue in the ACX100 Tx
 ****************************************************************************/
UINT32 whal_txQueue_GetSize(TxQueueId_T Qid)
{
	TxQueue_T *pTxQ = &TxQueueDb[Qid];
	return (pTxQ->NumDesc * sizeof(TxDesc_T));
}

/****************************************************************************
 *                      whal_txQueue_GetDescNum()
 ****************************************************************************
 * DESCRIPTION:	Return the number of descriptors in the ACX100 Tx
 ****************************************************************************/
int whal_txQueue_GetDescNum(TxQueueId_T Qid)
{
	TxQueue_T *pTxQ = &TxQueueDb[Qid];
	return (int)(pTxQ->NumDesc);
}

/****************************************************************************
 *                      whal_txQueue_GetBaseAddr()
 ****************************************************************************
 * DESCRIPTION:	Return the ACX100 Tx queue address
 ****************************************************************************/
UINT32 whal_txQueue_GetBaseAddr(TxQueueId_T Qid)
{
	TxQueue_T *pTxQ = &TxQueueDb[Qid];
	return ((UINT32)pTxQ->HwBaseAddr-ACX100_MEM_ADDRESS);
}

/****************************************************************************
 *                      whal_txQueue_GetNumFree()
 ****************************************************************************
 * DESCRIPTION:	Return number of free descriptors in the ACX100 Tx queue 
 ****************************************************************************/
UINT32 whal_txQueue_GetNumFree(TxQueueId_T Qid)
{
	TxQueue_T *pTxQ = &TxQueueDb[Qid];
	return ((UINT32)pTxQ->NumFree);
}

/****************************************************************************
 *                      whal_txQueue_GetNumUsed()
 ****************************************************************************
 * DESCRIPTION:	Return number of free descriptors in the ACX100 Tx queue 
 ****************************************************************************/
UINT32 whal_txQueue_GetNumUsed(TxQueueId_T Qid)
{
	TxQueue_T *pTxQ = &TxQueueDb[Qid];
	return ((UINT32)(pTxQ->NumDesc - pTxQ->NumFree));
}

/****************************************************************************
 *                      whal_txQueue_GetFailedCount()
 ****************************************************************************
 * DESCRIPTION:	Return the Failed counter
 ****************************************************************************/
int whal_txQueue_GetRetryFailCount(TxQueueId_T Qid)
{
	TxQueue_T *pTxQ = &TxQueueDb[Qid];
	return (pTxQ->RetryFailCounter);
}

/****************************************************************************
 *                      whal_txQueue_GetRetryCount()
 ****************************************************************************
 * DESCRIPTION:	Return the Retry counter
 ****************************************************************************/
int whal_txQueue_GetRetryCount(TxQueueId_T Qid)
{
	TxQueue_T *pTxQ = &TxQueueDb[Qid];
	return (pTxQ->RetryCounter);
}

/****************************************************************************
 *                      whal_txQueue_GetMultipleRetryCount()
 ****************************************************************************
 * DESCRIPTION:	Return the Retry counter
 ****************************************************************************/
int whal_txQueue_GetMultipleRetryCount(TxQueueId_T Qid)
{
	TxQueue_T *pTxQ = &TxQueueDb[Qid];
	return (pTxQ->MultipleRetryCounter);
}

/****************************************************************************
 *                      whal_txQueue_GetAckFailCount()
 ****************************************************************************
 * DESCRIPTION:	Return the Retry counter
 ****************************************************************************/
int whal_txQueue_GetAckFailCount(TxQueueId_T Qid)
{
	TxQueue_T *pTxQ = &TxQueueDb[Qid];
	return (pTxQ->AckFailCounter);
}

/****************************************************************************
 *                      whal_txQueue_GetRtsFailCount()
 ****************************************************************************
 * DESCRIPTION:	Return the Retry counter
 ****************************************************************************/
int whal_txQueue_GetRtsFailCount(TxQueueId_T Qid)
{
	TxQueue_T *pTxQ = &TxQueueDb[Qid];
	return (pTxQ->RtsFailCounter);
}

/****************************************************************************
 *                      whal_txQueue_GetRtsSuccCount()
 ****************************************************************************
 * DESCRIPTION:	Return the Retry counter
 ****************************************************************************/
int whal_txQueue_GetRtsSuccCount(TxQueueId_T Qid)
{
	TxQueue_T *pTxQ = &TxQueueDb[Qid];
	return (pTxQ->RtsSuccCounter);
}

/****************************************************************************
 *                      whal_txQueue_GetSuccCount()
 ****************************************************************************
 * DESCRIPTION:	Return the Retry counter
 ****************************************************************************/
int whal_txQueue_GetSuccCount(TxQueueId_T Qid)
{
	TxQueue_T *pTxQ = &TxQueueDb[Qid];
	return (pTxQ->SuccCounter);
}

/****************************************************************************
 *                      whal_txQueue_Print()
 ****************************************************************************
 * DESCRIPTION:	Print the TxQueue list and it's main fields
 ****************************************************************************/
int whal_txQueue_Print(TxQueueId_T Qid,char *buf)
{
	TxQueue_T *pTxQ = &TxQueueDb[Qid];
	TxDesc_T *pHwTxDesc = pTxQ->HwBaseAddr;
	int Idx,len = 0;


	for (Idx=0; Idx<pTxQ->NumDesc; Idx++)
	{
		len +=sprintf(buf+len,"%02x: ", Idx);
		len +=whal_txDesc_Print(&pHwTxDesc[Idx],buf + len);
	}

	return len;
}

/****************************************************************************
 *                      whal_txQueue_InfoPrint()
 ****************************************************************************
 * DESCRIPTION:	Print the TxQueue Info
 ****************************************************************************/
int whal_txQueue_InfoPrint(TxQueueId_T Qid ,char *buf)
{
	TxQueue_T *pTxQ = &TxQueueDb[Qid];
	int len = 0;

	len +=sprintf(buf+len,"TxQueue Id=%d ----------------------------------------------\n", Qid);
	len +=sprintf(buf+len,"Addr=0x%x, NumDesc=0x%x (DescSize=0x%x)\n", (UINT32)pTxQ->HwBaseAddr, pTxQ->NumDesc, sizeof(TxDesc_T));
	len +=sprintf(buf+len,"pFree=0x%x, pAlloc=0x%x, NumFree=0x%x\n", (UINT32)pTxQ->pFree, (UINT32)pTxQ->pAlloc, pTxQ->NumFree);
	len +=sprintf(buf+len,"Counters:\n Success = 0x%x, Retry = 0x%x, Multiretry = 0x%x\n", 
      pTxQ->SuccCounter, pTxQ->RetryCounter, pTxQ->MultipleRetryCounter);
	len +=sprintf(buf+len,"Fails   :\n Retry   = 0x%x, Other = 0x%x, Timeout = 0x%x, DMA Err = 0x%x\n",
      pTxQ->RetryFailCounter, pTxQ->OtherFailCounter,
      pTxQ->TimeoutErrorCounter, pTxQ->DmaErrorCounter);

   len +=sprintf(buf+len,"DMA Errors:\n"
      " InProgress = 0x%x\n"
      " Spurs      = 0x%x\n"
      " Target     = 0x%x\n"
      " Master     = 0x%x\n"
      " Cleared    = 0x%x\n"
      " Host end   = 0x%x\n"
      " ACX end    = 0x%x\n"
      " Other      = 0x%x\n\n",
      pTxQ->DmaErrorInProgress   ,
      pTxQ->DmaErrorNoErr_spur   ,
      pTxQ->DmaErrorTargetAbort  ,
      pTxQ->DmaErrorMasterAbort  ,
      pTxQ->DmaErrorClearedAbort ,
      pTxQ->DmaErrorHostEndAbort ,
      pTxQ->DmaErrorAcxEndAbort  ,
      pTxQ->DmaErrorUnspecified
      );




	return len;
}

/****************************************************************************
 *                      whal_txIndicatorPrint()
 ****************************************************************************
 * DESCRIPTION:	Print the Tx Indicator field
 ****************************************************************************/
int whal_txIndicatorPrint(char *buf)
{
	int len = 0;
	len +=sprintf(buf+len,"---- QueueIndicator = 0x%08x, FwLock=%d, HostLock=%d\n", 
		whal_acxMemGetU32(&pTxQIndicator->indicator), 
		whal_acxMemGetU16(&pTxQIndicator->fw_lock), 
		whal_acxMemGetU16(&pTxQIndicator->host_lock) );
	len +=sprintf(buf+len,"Host QueueIndicator = 0x%08x, FwLock=%d, HostLock=%d\n", 
		whal_acxMemGetU32(&pTxHostQIndicator->indicator), 
		whal_acxMemGetU16(&pTxHostQIndicator->fw_lock), 
		whal_acxMemGetU16(&pTxHostQIndicator->host_lock) );

	return len;
}

