/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2001 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalAcxDataIf.h                
 *   PURPOSE: Contains ACX100 Data (Rx/Tx) interface defines/structures
 *   
 ****************************************************************************/

#ifndef _WHAL_ACX_DATA_IF_H
#define _WHAL_ACX_DATA_IF_H

#ifndef uint32
#define uint32 UINT32
#endif

#ifndef uint16
#define uint16 UINT16
#endif

#ifndef uint8
#define uint8 UINT8
#endif

#include "public_channels.h"
#include "public_rates.h"
#include "public_security_keys.h"

/* Transmit Control bits */

#define CTL_PREAMBLE	0x01			/* Indicates whether the MPDU is to be */
										/* sent with the short or long preamble */
										/* 0 = long, 1 = short. */
#define CTL_RECLAIM		0x08			/*  MPDU pointed to by the xfrbuf element */
										/*  is available for the ACX100 to reuse */
										/* after the MPDU has been acknowledged */
										/* and the HostDone bit is set  */

										/* the data pointed to by WEPStartAddr. */
#define CTL_HOSTDONE	0x20			/* Indicates that the Host has processed  */
										/* the MPDU on this descriptor. */
#define CTL_ACXDONE		 0x40			
#define CTL_OWN          0x80           /* When set it indicates that the Host owns  */
										/* the Descriptor */ 

#define CTL2_SEQUENCE_CTL 0x01			/* When set tells the ACX100 to not augment the */
										/* Sequence Control Field for the Packet being sent. */
#define CTL2_FCS		0x02            /*  When set tells the ACX100 not to append an FCS */
										/* onto the end of the packet. */
#define CTL2_RETRY		0x08			/* When set tells the ACX100 to not augment the Retry  */
										/* bit of the Frame Control Field for the Packet being sent. */
/*#define CTL2_PWRMGMT	0x10   */ 		/*  When set tells the ACX100 to not augment the PwrMgmt */
										/*  bit of the Frame Control Field for the Packet being sent. */
#define CTL2_BURST		0x10            /* Burst all packets */

#define CTL2_RTS		0x20			/* When set instructs the ACX100 to instigate an RTS/CTS */
										/* sequence for this frame. */
#define CTL2_ERP 		0x1				/* When set instructs the ACX100 to instigate an CTS to self */
									    /* protection for this frame. */

#define CTL2_TI  		0x2				/* When set instructs the ACX100 to instigate an TI trick packet */
										/* protection for this frame. */	
									
#define CTL2_DURATION	0x80			/*  When set tells the ACX100 to not augment the Duration field */
										/*  for the Packet being sent.  Note: If in AutoRateFallback */
										/*  mode, the field will be overwritten upon failure. */

/* Transmit & Receive Status Definitions */
#define STAT_KEY_NOT_FOUND	0x08	/* The ACX100 could not employ the encryption algorithm */
										/* because the key for this MAC Address was not found.  */
#define STAT_DMA_ERR    0x80            /* A DMA Error occurred */


/* Transmit Status Bits */
#define STAT_TX_SUCCESS		0x00
#define STAT_TX_FRAG_ERR    0x01        /* This status is returned when an MPDU was not transmitted */
                                        /* because of an error in another MPDU of the given MSDU */
#define STAT_FLUSH	0x02			/* When this bit is set it indicates this packet was  */
										/* either not transmitted, or transmission was aborted  */
										/* between attempts, due to the FLUSH_TX command. */
#define STAT_MSDU_TIMEOUT 0x10		/* When this bit is set it indicates there was an MSDU Timeout */
#define STAT_RETRY		0x20		/* When this bit is set it indicates the maximum number of  */
										/* retries were attempted (as defined by either a  */
										/* ShortRetryLimit or a LongRetryLimit, as configured  */
										/* in the Information element. */

/* additional Receive Status Bits */
#define STAT_IV_ICV			0x04		
#define STAT_FCS			0x02		/* When this bit is set, it indicates the Cyclic Redundancy */
										/* Check failed for the incoming MPDU (Used in Raw mode).  */
#define STAT_PREAMBLE		0x01		/* Determination as to whether this MPDU was received  */
										/* with the short or long preamble */
#define STAT_RX_DMA_ERR     STAT_DMA_ERR /* DMA Error occurred */

#define TX_CTL_WORD_OFFSET		0x24
#define TX_STATUS_WORD_OFFSET	0x26
#define TX_RATE_WORD_OFFSET		0x2A


#define TX_SW_ANTENNA_DIVERSITY_BYTE_OFFSET		0x22
#define TX_CTL_BYTE_OFFSET		0x24
#define TX_CTL2_BYTE_OFFSET		0x25
#define TX_WSPACE_BYTE_OFFSET	0x14

#define TX_ANT_DIVERSITY_OFFSET 0x22



#pragma pack(1)
typedef struct
{
	UINT8		*next;					/* pointer to next descriptor */
	UINT8		*HostMemPtr;			/* Points to Host Memory */
	UINT32   *ACXMemPtr;				/* points to first memoryblock in */
										   /* the list which holds the MPDU */
	UINT8		Reserved1[4];			/* reserved for ACX100 */

   UINT16	MPDULen;				    /* length in bytes of packet */
	UINT8		Reserved2[2];			/* reserved for ACX100 */
	
   UINT8		DriverWorkspace[14];	/* working storage for driver */

   UINT8        swAntDivs;              /* SW ant. diversity config. from host*/
   UINT8        Reserved3;


#if 1
	UINT8		Ctl;					/* control bits */
	UINT8		Ctl2;					/* more control bitsxfr */
#else
	UINT16	Ctl_And_Ctl2;
#endif
	UINT8		Status;					/* Status bits */
	UINT8		AckFailures;

   UINT8		RTSFailures;
	UINT8		RTSSuccess;
   TxdRateSet_t Rate;            /* supported RateSet and Modulation indication. */

   UINT8		reserved5[8];
} TxDesc_T;
#pragma pack()

#define RX_CTL_WORD_OFFSET		         0x28
#define RX_RATE_BYTE_OFFSET		      0x29
#define RX_STATUS_BYTE_OFFSET	         0x2A
#define RX_MODULATION_BYTE_OFFSET	   0x2B
#define RX_STATUS_ERROR_MASK	         (STAT_FCS | STAT_IV_ICV | STAT_KEY_NOT_FOUND)

#pragma pack(1)
typedef struct
{
	UINT8		*next;					/* pointer to next descriptor */
	UINT8		*HostMemPtr;			/* Points to Host Memory */
	UINT32	*ACXMemPtr;				/* points to first memoryblock in */
				    						/* the list which holds the MPDU */
	UINT8 	reserved1[4];		   /* reserved */
	
   UINT16	MPDULen; 				/* length in bytes of packet */
	UINT8 	reserved2[6];		   /* reserved */

	UINT8		DriverWorkspace[16];		/* working storage for driver */

   UINT8		Ctl;			   		/* control bits */
	UINT8		Rate;				   	/* Rate at which the MPDU was received */
	UINT8		Status;					/* Status bits */
	UINT8		modulation;
	
   UINT8		reserved3[8];
} RxDesc_T;
#pragma pack()

/* 
 * Queue indicator
 */
#pragma pack(1)
typedef struct
{
  UINT32 indicator;
  UINT16 host_lock;
  UINT16 fw_lock;
} QueueIndicator_T;
#pragma pack()


/* 
 * Info.and Err code 
 */										
#define INFO_SCAN_COMPLETE		0x01	/* Results from a Scan command */
#define INFO_WEP_KEY_NOT_FOUND  0x02	/* Encryption process on a */
										/* packet failed because a key */
										/* was not available to the ACX100 */
#define INFO_WATCH_DOG_RESET    0x03    /*Watch Dog has fired!!*/
#define INFO_PS_FAIL            0x04    /* Failed to send out NULL frame from PS mode notification to AP */
#define INFO_IV_ICV_FAILURE		0x05	/* encryption/decryption process on a packet failed */

#pragma pack(1)
typedef struct {
    UINT8 TestCmdId;
    UINT8 TestCmdParam1;
    UINT8 TestCmdParam2;
}TestCmdStruct;
#pragma pack()

#define TEST_CMD_FCC            0x01    /* FCC Tx signal test */
#define TEST_CMD_TELEC          0x02    /* Telec test */
#define TEST_CMD_EXIT           0xFF    /* exit the test enviroment */

/*
 *  Acx blocks info
 */
#define DEFAULT_ACX_BLOCK_SIZE			0x100	/* default in the ACX100 */
#define BLOCK_HEADER_SIZE	0x04
#define ACX_RX_STAT_HDR_LEN	0x0C		
#define ACX_RX_PLCP_HDR_LEN	0x08		
#define LOCAL_MEM_LAST		0x02000000
#define MEMBLKPTR_MASK		0x0007FFFF
#define BLOCK_BOUNDRY_MASK	0x1F		/* used to mask off the lower 32 bytes */
#define BLOCK_BOUNDRY		32			/* a memblock must start on this byte boundry */
#define GET_NEXT_BLOCK_ADDR(x)   ((UINT8 *)((x & MEMBLKPTR_MASK) << 5 )) 
#define GET_SHIFT_BLOCK_ADDR(x)  ((x >> 5 ) & MEMBLKPTR_MASK)   

#define MAC_HEADER_SIZE	24
#define IV_FIELD_SIZE	4
#define ICV_FIELD_SIZE	4
#define FCS_FIELD_SIZE	4

/*
 * RX HEADER
 */
#pragma pack(1)
typedef struct 
{

  UINT16 cbReceived : 12;  /* Actually bits 0:11 are the length.  12 - 15 Reserved */
  UINT8  cMemBlocks;       /* number of memblocks */
  UINT8  MACStatus;         
  UINT8  BasebandStatus;
  UINT8  PLCPSignal;       /* This is the Rate ? */
  UINT8  SIR;
  UINT8  SNR;
  UINT32 Timestamp;
} ACX_RX_HEADER;
#pragma pack()

/*
 * PLCP HEADER
 */
#pragma pack(1)
typedef struct 
{
  UINT8  Signal;
  UINT8  Service;
  UINT16 Length;
} PLCP_HEADER;
#pragma pack()

/* NewPlcp*/
#define PLCP_MIC_FAILURE_STATUS 0x04
#pragma pack(1)
typedef struct
{
   uint16 service;
   uint16 length;
   uint8  channel;
   uint8  band;
   uint8  status;
   uint8  reserved;
} RxPLCPHeader_T;
#pragma pack()



#pragma pack(1)
typedef struct
{
    UINT16 length;
    UINT16 rxm_status;
    UINT8  mod_mode;
    UINT8  plcp_signal;
    UINT8  rx_level;
    UINT8  rx_snr;
    UINT32 time_stamp;
} WlanRxStatusInfo_T;
#pragma pack()


#endif
  

