/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2003 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/**************************************************************************
 *
 * MODULE:   memMngr.c
 * PURPOSE:  manage the SDRAM buffers for MPDU/MSDU Data Buffers allocations
 *
 **************************************************************************/
#include "pform_types.h"
#include "pal.h"
#include "memMngr.h"

/*************
 * Constants *
 *************/
#define MSDU_MAX_NUMBER          400
#define BD_MAX_NUMBER            600

/************************************************
 * Internal variables, should be defined static *
 ************************************************/
mem_MSDU_T*    msduPool;        /* list of MSDU Buffer Desciptors  */
mem_BD_T*      bdPool;          /* list of Data Buffer Descriptors */

mem_MSDU_T*    firstFreeMSDU;   /* pointer to the first free MSDU */
mem_BD_T*      firstFreeBD;     /* pointer to the first free BD   */

int            numFreeMSDU;     /* Number of free MSDUs */
int            numFreeBD;       /* Number of free BDs   */

/* pool table, sorted by accesnding data size */
mem_DataPool_T mem_dtPool[NUM_OF_DATA_BUF_POOLS] = {
   {
   /*  dataSize          */   64,
   /*  poolSize          */   300,
   /*  dataSizeMask      */   0x3F,
   /*  dataSizeShift     */   6,
   /*  dataBufPool       */   0,
   /*  firstFreeDataBuf  */   0,
   /*  numFreeDataBuf    */   0
   }
   ,
   {
   /*  dataSize          */   DATA_BUF_LEN,
   /*  poolSize          */   150,
   /*  dataSizeMask      */   0x7FF,
   /*  dataSizeShift     */   11,
   /*  dataBufPool       */   0,
   /*  firstFreeDataBuf  */   0,
   /*  numFreeDataBuf    */   0
   }
};

/*********************
 * Static functions  *
 *********************/
static STATUS freeDataBuf( mem_DataBuf_T* dataBuf);
static int    wlan_memMngrSelectPool( UINT32 len);

/*
 * Memory required for pools, using uniform size of databuf (2048 bytes):
 * msduPool: 300*52   =  15,600
 * bdPool:   600*48   =  28,800
 * DB pool:  300*2060 = 618,000 (!!)
 *                      -------
 *              Total = 662,400 ~647kB.
 * -> memShow
 *  status   bytes    blocks   avg block  max block
 *  ------ --------- -------- ---------- ----------
 * current
 *    free   9537264        2    4768632   9537216
 * ->
 * 
 *
 * Memory required for pools, using two pools of Data Buf (2048, 64 bytes):
 * msduPool: 300*52   =  15,600
 * bdPool:   600*48   =  28,800
 * DB pool0: 150*2060 = 309,000 (!!)
 * DB pool1: 150*76   =  11,400
 *                      -------
 *              Total = 364,800 ~356kB.
 *
 * Expected saving of 291kB of SDRAM !!
 *  status   bytes    blocks   avg block  max block
 *  ------ --------- -------- ---------- ----------
 *   free   9827424        1    9827424   9827424
 */


/*************************************************************************
 *                       wlan_memMngrInit                                *
 *************************************************************************
 DESCRIPTION: Init of the Memory Manager module

 INPUT:
 OUTPUT:
 RETURN:      OK/NOK
**************************************************************************/
STATUS wlan_memMngrInit( void)
{
   register int     count;
   int failAlloc = 0;
   int nDPoolIdx;
   
   /* Allocate memory pools */
   msduPool    = (mem_MSDU_T*)PAL_memAlloc( 0,MSDU_MAX_NUMBER*sizeof(mem_MSDU_T),0,0);
   bdPool      = (mem_BD_T*)PAL_memAlloc( 0,BD_MAX_NUMBER*sizeof(mem_BD_T),0,0);
   for( nDPoolIdx = 0; nDPoolIdx < NUM_OF_DATA_BUF_POOLS; ++nDPoolIdx) {
      mem_dtPool[nDPoolIdx].dataBufPool =
         (mem_DataBuf_T*)PAL_memAlloc( 0,mem_dtPool[nDPoolIdx].poolSize*sizeof(mem_DataBuf_T),0,0);
   }
   
   /* Reset contents to 0's in all allocated buffers */
   if( msduPool != NULL) PAL_memSet( msduPool, 0, (MSDU_MAX_NUMBER * sizeof(mem_MSDU_T)));
   if( bdPool   != NULL) PAL_memSet( bdPool  , 0, (BD_MAX_NUMBER   * sizeof(mem_BD_T)));
   for( nDPoolIdx = 0; nDPoolIdx < NUM_OF_DATA_BUF_POOLS; ++nDPoolIdx) {
      /* Fill with 0's */
      void* mptr = (void*)mem_dtPool[nDPoolIdx].dataBufPool;
      if( mptr != NULL) {
         memset( mptr, 0,
            (mem_dtPool[nDPoolIdx].poolSize * sizeof(mem_DataBuf_T))
            );
         /* allocate data buffers */
         for( count = 0; count < mem_dtPool[nDPoolIdx].poolSize; ++count) {
            void* ptr = PAL_memAlloc( 0,sizeof(UINT8)*mem_dtPool[nDPoolIdx].dataSize,0,0);
            mem_dtPool[nDPoolIdx].dataBufPool[count].data = (UINT8*)ptr;
            if( ptr == NULL) {
               failAlloc = 1;
               break;
            }
         }
      }
      else{
         failAlloc = 1;
         break;
      }
   }/* endFor */

   /* Check for enough resources, exit gracefully if not available */
   if( (msduPool    == NULL) ||
       (bdPool      == NULL) ||
       (failAlloc) )
   {
      puts( "FATAL ERROR: Could not allocate memory for MEM MNGR - Aborting");
      wlan_memMngrFreeAll();
      return NOK;
   }
   
   /* chain the items in MSDU pool, except the last one */
   for( count = 0; count < MSDU_MAX_NUMBER-1; ++count) {
      msduPool[count].handle   = count;
      msduPool[count].nextMSDU = &(msduPool[count+1]);
   }
   /* last element */
   msduPool[count].handle = count;

   /* chain the items in BD pool, except the last one */
   for( count = 0; count < BD_MAX_NUMBER-1; ++count) {
      bdPool[count].handle    = count;
      bdPool[count].nextBDPtr = &(bdPool[count+1]);
      bdPool[count].length    = mem_dtPool[0].poolSize; /* start with minimum length */
   }
   /* last element */
   bdPool[count].handle = count;
   bdPool[count].length = DATA_BUF_LEN;

   /* chain data buffer pool, except the last element */
   for( nDPoolIdx = 0; nDPoolIdx < NUM_OF_DATA_BUF_POOLS; ++nDPoolIdx) {
      for( count = 0; count < mem_dtPool[nDPoolIdx].poolSize-1; ++count) {
         mem_dtPool[nDPoolIdx].dataBufPool[count].handle      = count;
         mem_dtPool[nDPoolIdx].dataBufPool[count].nextDataBuf = &(mem_dtPool[nDPoolIdx].dataBufPool[count+1]);
         mem_dtPool[nDPoolIdx].dataBufPool[count].szData      = (UINT16)mem_dtPool[nDPoolIdx].poolSize;
         mem_dtPool[nDPoolIdx].dataBufPool[count].nPool       = (UINT16)nDPoolIdx;
      }
      /* last element */
      mem_dtPool[nDPoolIdx].dataBufPool[count].handle = count;
      /* pool state */
      mem_dtPool[nDPoolIdx].firstFreeDataBuf = mem_dtPool[nDPoolIdx].dataBufPool;
      mem_dtPool[nDPoolIdx].numFreeDataBuf   = mem_dtPool[nDPoolIdx].poolSize;
   }
   
   /* Assign a pointer for the start of each list */
   firstFreeMSDU    = msduPool;
   firstFreeBD      = bdPool;
   /* Setup the indication of free elements */
   numFreeMSDU    = MSDU_MAX_NUMBER;
   numFreeBD      = BD_MAX_NUMBER;
   
   /*wdrv_mainsmSendModuleUp(WDRV_MEM_MODULE_ID);*/
   
   return OK;
}/* END wlan_memMngrInit() */

/*************************************************************************
 *                        wlan_memMngrAllocDataBuf                       *
 *************************************************************************
 DESCRIPTION: This function allocates BDs and Data Buffers according to the
         required length. The memory manager will allocate the Data
         Buffers, update the buffer pointer in the BD structure and link
         the BDs when more than one Data Buffer is required.

 INPUT:      len - the length of the required data buffer
 OUTPUT:     bdPtr - a pointer in which this function will return a pointer
                     to the allocated BD
 RETURN:     OK/NOK
**************************************************************************/
STATUS wlan_memMngrAllocDataBuf( mem_BD_T** bdPtr, UINT32 len)
{
   int             count = 0;
   int             nPoolIdx;
   int             dataBufNum;
   mem_BD_T*       allocBdTmp;      /* pointer to the currently allocated BD in the new list */
   mem_DataBuf_T*  allocDataBufTmp; /* pointer to the currently allocated Data Buf */
   unsigned long   flags;

   /* decide which data pool to use, and take all the data from it */
   nPoolIdx = wlan_memMngrSelectPool( len);
   
   /* calculate the number of BDs and Data Buffers we need to
    * allocate, whole and partial. Each BD points to one data Buff only. 
    */
   dataBufNum = len >> mem_dtPool[nPoolIdx].dataSizeShift;
   if( len & mem_dtPool[nPoolIdx].dataSizeMask)
      ++dataBufNum;
   
   PAL_intrLock(flags);   /* START OF CRITICAL SECTION */

   /* return a pointer to the allocated BD */
   *bdPtr = firstFreeBD;

   /* set temporary pointers to currently allocated BD and data buffer */
   allocBdTmp      = firstFreeBD;
   allocDataBufTmp = mem_dtPool[nPoolIdx].firstFreeDataBuf;
   
   /* check if we have enough memory */
   if( (numFreeBD < dataBufNum) ||
       (mem_dtPool[nPoolIdx].numFreeDataBuf < dataBufNum))
   {
      PAL_intrUnLock(flags);   /* END OF CRITICAL SECTION */
#if 0
      printf( "DB: mem_allocDataBuf: not enough memory\nnumFreeBD=%d"
         " dtPool=%d numFreeDataBuf=%d reqBDs=%d\n",
         numFreeBD, nPoolIdx, mem_dtPool[nPoolIdx].numFreeDataBuf, dataBufNum);
#endif
      /* invalidate returned pointer, and return NOK */
      *bdPtr = NULL;
      return NOK;
   }
   
   /* update the pointers to the head of the list */
   for( ; count < dataBufNum; ++count) {
      allocBdTmp->freeFunc      = NULL;
      allocBdTmp->refCount      = 1;
      allocBdTmp->dataBuf       = allocDataBufTmp;       /* Data Buffer object */
      allocBdTmp->data          = allocDataBufTmp->data; /* actual data */
      allocDataBufTmp->refCount = 1;

      if( count == (dataBufNum-1)) {
         /* last required BD / Data buff, update internal state  */
         firstFreeBD  = allocBdTmp->nextBDPtr;
         mem_dtPool[nPoolIdx].firstFreeDataBuf = allocDataBufTmp->nextDataBuf;
         /* break the chains */
         allocBdTmp->nextBDPtr        = NULL;
         allocDataBufTmp->nextDataBuf = NULL;
      }
      else {
         /* not last required BD / Data buff, iterate to the next element */
         allocBdTmp = allocBdTmp->nextBDPtr;
         allocDataBufTmp = allocDataBufTmp->nextDataBuf;
      }
   }
   /* Reduce the number of free BDs and data buffers */
   numFreeBD -= dataBufNum;
   mem_dtPool[nPoolIdx].numFreeDataBuf -= dataBufNum;

   PAL_intrUnLock(flags);   /* END OF CRITICAL SECTION */
   
   return OK;
}/* END wlan_memMngrAllocDataBuf() */

/*************************************************************************
 *                        wlan_memMngrAllocBDs                           *
 *************************************************************************
 DESCRIPTION: This function allocates and returns a pointer to an array of
              BDs. This function does not allocate any memory buffers.

 INPUT:      bdNumber - number of required BDs
 OUTPUT:     bdPtr - a pointer in which this function will return a pointer
                     to an array of BD pointers
 RETURN:     OK/NOK
**************************************************************************/
STATUS wlan_memMngrAllocBDs( UINT32 bdNumber, mem_BD_T** bdPtr)
{
   int           count = 0;
   mem_BD_T*     allocBdTmp; /* pointer to the current allocated BD in the new list */
   unsigned long flags;
  
   if( bdNumber == 0) {
      *bdPtr = NULL;
      return NOK;
   }
  
   PAL_intrLock(flags);  /* START OF CRITICAL SECTION */

   /* return a pointer to the allocated BD */
   *bdPtr     = firstFreeBD;
   /* set temporary pointer to currently allocated BD */
   allocBdTmp = firstFreeBD;

   /* check if we have enough BDs to allocate */
   if( numFreeBD < bdNumber) {
      PAL_intrUnLock(flags);   /* END OF CRITICAL SECTION */
      puts( "DB: wlan_memMngrAllocBDs: not enough memory");
      /* Invalidate return pointer and return NOK */
      *bdPtr = NULL;
      return NOK;
   }
  
   /* update the pointers to the head of the list */
   for( ; count < bdNumber; ++count) {
      allocBdTmp->refCount = 1;
      allocBdTmp->freeFunc = NULL;
	  allocBdTmp->freeArgs[4] = FALSE;
      allocBdTmp->bufToken = 0;
      if( count == (bdNumber-1)) {
         /* last required BD, update internal state  */
         firstFreeBD           = allocBdTmp->nextBDPtr;
         allocBdTmp->nextBDPtr = NULL;
      }
      else {
         /* not last required BD, iterate to the next element */
         allocBdTmp = allocBdTmp->nextBDPtr;
      }
   }
   /* Reduce the number of free BDs buffers */
   numFreeBD -= bdNumber;
   PAL_intrUnLock(flags);   /* END OF CRITICAL SECTION */

   return OK;
}/* END wlan_memMngrAllocBDs() */

/*************************************************************************
 *                        wlan_memMngrAllocMSDU                          *
 *************************************************************************
 DESCRIPTION: This function allocates MPDU structure, BDs and Data Buffers
              as required by 'len'.

 INPUT:   len - the length of the required data buffer
                if len=0, than only MSDU buffer will be allocated
 OUTPUT:  MSDUPtr - a pointer that holds the returned pointer
                    to the MSDU structure
 RETURN:     OK/NOK
**************************************************************************/
STATUS wlan_memMngrAllocMSDU( mem_MSDU_T** MSDUPtr, UINT32 len)
{
   int        rc;
   mem_BD_T*  bdTmp;
   unsigned long flags=0;
   
   if( msduPool == NULL) {
      /* object not initiated yet (!!!) */
      *MSDUPtr = NULL;
      return NOK;
   }
   
   if( len > 0) {
      /* we need to allocate BD and Data Buffers */
      rc = wlan_memMngrAllocDataBuf( &bdTmp, len);
      if( rc == NOK)
      {
		  /*printf("failed to alloc data buffer of len = %d\n",len);*/
         /* cannot allocate, return */
         *MSDUPtr = NULL;
         return NOK;
      }
   }
   
   PAL_intrLock(flags);    /* START OF CRITICAL SECTION */
   if( firstFreeMSDU == NULL) {
      /* no free MSDU buffers */
      PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
      wlan_memMngrFreeBD( bdTmp->handle);
      *MSDUPtr = NULL;
	  printf("failed to alloc Msdu\n");
      return NOK;
   }

   /* set return pointer to a free MSDU  */
   *MSDUPtr      = firstFreeMSDU;
   /* update internal state */
   firstFreeMSDU = firstFreeMSDU->nextMSDU;
   --numFreeMSDU;

   PAL_intrUnLock(flags);   /* END OF CRITICAL SECTION */

	 bdTmp->bufToken = 0;
   /* Fill the MSDU details */
   (*MSDUPtr)->firstBDPtr     = bdTmp;
   (*MSDUPtr)->lastBDPtr      = bdTmp;
   (*MSDUPtr)->dataLen        = len;

   (*MSDUPtr)->nextConcatMSDU = NULL;	 

   return OK;
}/* END wlan_memMngrAllocMSDU() */

/*************************************************************************
 *                     wlan_memMngrAllocMSDUBufferOnly                   *
 *************************************************************************
 DESCRIPTION: This function allocates MPDU structure - without Data Buffers

 INPUT:
 OUTPUT:   MSDUPtr - a pointer in which this function will return a pointer
                     to the MSDU structure
 RETURN:   OK/NOK
**************************************************************************/
STATUS wlan_memMngrAllocMSDUBufferOnly( mem_MSDU_T** MSDUPtr)
{
   unsigned long flags;

   PAL_intrLock(flags);      /* START OF CRITICAL SECTION */

   /* no free MSDU buffers, invalidate return pointer and return */
   if( firstFreeMSDU == NULL) {
      PAL_intrUnLock(flags);   /* END OF CRITICAL SECTION */
      *MSDUPtr = NULL;
      return NOK;
   }

   /* set return pointer to a free MSDU */
   *MSDUPtr  = firstFreeMSDU;
   (*MSDUPtr)->nextConcatMSDU = NULL;

   /* update internal state */
   firstFreeMSDU = firstFreeMSDU->nextMSDU;
   --numFreeMSDU;

   PAL_intrUnLock(flags);   /* END OF CRITICAL SECTION */

   return OK;
}/* END wlan_memMngrAllocMSDUBufferOnly() */

#if 0
/*************************************************************************
 *                        wlan_memMngrDuplicateMSDU                      *
 *************************************************************************
 DESCRIPTION: This function duplicates the MSDU.

 INPUT:      pSrcMsdu - pointer of the source MSDU
             pDstMsdu - pointer to destination MSDU
 OUTPUT:
 RETURN:     OK/NOK
**************************************************************************/
STATUS wlan_memMngrDuplicateMSDU( mem_MSDU_T *pDstMsdu, mem_MSDU_T *pSrcMsdu )
{
   mem_BD_T*     bdTmp;   /* pointer to the current allocated BD in the new list */
   mem_MSDU_T*   msduTmp; /* pointer to the duplicated MSDU buffer */
   /* This Function never tested !!!!!!!!!!!!!!!!!!*/
   
   /* If there are any BDs associated with this MSDU,
    * then increment reference counter of all of them.
    */
   PAL_intrLock(flags);   /* START OF CRITICAL SECTION */
   bdTmp  = pSrcMsdu->firstBDPtr;
   while( bdTmp != NULL) {
      if( bdTmp->refCount == 0) {
         /* TODO: handle this case */
         printf( "wlan_memMngrDuplicateMSDU() -- BD ref count = 0\n"
            "srcMSDU=%p, bdTmp=%p\n", pSrcMsdu, bdTmp);
      }
      bdTmp->refCount++;
      bdTmp = bdTmp->nextBDPtr;
   }
   PAL_intrUnLock(flags);   /* END OF CRITICAL SECTION */
   
   /* duplicate the MSDU */
   pDstMsdu->headerLen    =  pSrcMsdu->headerLen;
   pDstMsdu->firstBDPtr   =  pSrcMsdu->firstBDPtr;
   pDstMsdu->lastBDPtr    =  pSrcMsdu->lastBDPtr;
   pDstMsdu->freeFunc     =  pSrcMsdu->freeFunc;
   memcpy( pDstMsdu->freeArgs ,pSrcMsdu->freeArgs, 5);
   /* TODO: nextConcatMSDU ??? */
   pDstMsdu->dataLen      =  pSrcMsdu->dataLen;
   pDstMsdu->reserved     =  pSrcMsdu->reserved;
   
   return OK;
}/* END wlan_memMngrDuplicateMSDU() */
#endif /* 0 */

/*************************************************************************
 *                        wlan_memMngrFreeMSDU                           *
 *************************************************************************
 DESCRIPTION: Free MSDU structure. This function will free all BDs and Data
              Buffers that are bind to this MSDU.

 INPUT:      handle - handle of this MSDU
 OUTPUT:
 RETURN:     OK/NOK
**************************************************************************/
STATUS wlan_memMngrFreeMSDU( UINT32 handle)
{
   int     freeFlag;
   unsigned long flags;
   
   if( handle == WLAN_DRV_NULL_MEM_HANDLE )
      return OK;
  
   if( msduPool[handle].firstBDPtr == NULL) {
      printf( "wlan_memMngrFreeMSDU handle %x This MSDU is already free\n", handle);
      return NOK;
   }

   PAL_intrLock(flags);   /* START OF CRITICAL SECTION */

   /* free all BDs and Data Buffers */
   freeFlag = wlan_memMngrFreeBD( msduPool[handle].firstBDPtr->handle);
   
   if( (freeFlag == TRUE))/* &&
      (msduPool[handle].freeFunc != NULL))*/
   {
      /* call the free function, if valid */	  
      if( msduPool[handle].freeFunc)
	  {
		 /*PAL_intrUnLock(flags);*/  /* END OF CRITICAL SECTION */
         msduPool[handle].freeFunc((unsigned char*)msduPool[handle].freeArgs[0]);
		 /*PAL_intrLock(flags);*/   /* START OF CRITICAL SECTION */
	  }
   }
      
   /* reset the fields of the MSDU buffer */
   msduPool[handle].firstBDPtr  = NULL;
   msduPool[handle].freeFunc    = NULL;
   msduPool[handle].freeArgs[0] = 0;
   msduPool[handle].dataLen     = 0;
   msduPool[handle].headerLen   = 0;

   /* add the MSDU to the free MSDU list */
   msduPool[handle].nextMSDU = firstFreeMSDU;
   
   /* update internal state */
   firstFreeMSDU = &(msduPool[handle]);
   ++numFreeMSDU;
   
   PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
   
   return OK;
}/* END wlan_memMngrFreeMSDU() */

/*************************************************************************
 *                        wlan_memMngrFreeBD                             *
 *************************************************************************
 DESCRIPTION: Free BD structure. This function will free a list of BD
         structures and the Data Buffer that is being pointed by these BD
         if any.

 INPUT:   handle - handle of this BD
 OUTPUT:
 RETURN:  freeFlag - return TRUE if this BD list was freed
                     return FALSE if this BD list was not freed (refCount>0)
**************************************************************************/
int wlan_memMngrFreeBD( UINT32 handle)
{
   int              rc=FALSE;
   mem_DataBuf_T*   dataBuf;
   mem_BD_T*        bdTmp;     /* pointer to the current BD we need to free */
   mem_BD_T*        nextBdTmp; /* pointer to the next BD we need to free */
   unsigned long    flags;

   /* Convert BD handle into its pointer */
   bdTmp = &(bdPool[handle]);

   PAL_intrLock(flags);  /* START OF CRITICAL SECTION */    
   
   while( bdTmp != NULL) {
      /*        printf("wlan_memMngrFreeMSDU bdTmp=%X\n", bdTmp);*/
      dataBuf   = bdTmp->dataBuf;
      nextBdTmp = bdTmp->nextBDPtr;
      if( bdTmp->refCount == 0) {
         PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
         printf( "DB: wlan_memMngrFreeBD FATAL ERROR: bdTmp->refCount = 0\n");
         return -1;
      }
      
      /* call the free function if applicable */
      if( bdTmp->freeFunc != NULL ){
         bdTmp->freeFunc( (unsigned char*)bdTmp );
      }
      
      /* free Data Buffer if allocated from the pool */
      if( bdTmp->dataBuf != NULL) {
         freeDataBuf( bdTmp->dataBuf);
      }
      
      /* decrement reference counter, reset the BD if zero */
      if( --(bdTmp->refCount) == 0) {
         bdTmp->dataBuf    = NULL;
         bdTmp->data       = NULL;
         bdTmp->dataOffset = 0;
         bdTmp->length     = DATA_BUF_LEN;
		 bdTmp->bufToken   = 0;
         /* adding the free BD to the free BD list */
         bdTmp->nextBDPtr  = firstFreeBD;
         firstFreeBD       = bdTmp;
         ++numFreeBD;
      }

      /* if we have reached the last BD to free, and its
       * reference count == 0, then indicate successful call
       */
      if( nextBdTmp == NULL)
         if( bdTmp->refCount == 0)
            rc = TRUE;

      /* Iterate to next element */
      bdTmp = nextBdTmp;
   }/* endWhile */
   
   PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
   return rc;
}/* END wlan_memMngrFreeBD() */

/*************************************************************************
 *                   wlan_memMngrMsduIsFragmented                        *
 *************************************************************************
 DESCRIPTION: Tests whether MSDU is fragmented or not.

 INPUT:      msduPtr - pointer to the MSDU
 OUTPUT:
 RETURN:     TRUE if the MSDU is fragmented, otherwise returns FALSE
**************************************************************************/
BOOL wlan_memMngrMsduIsFragmented( mem_MSDU_T *msduPtr)
{
   return( (msduPtr->firstBDPtr->nextBDPtr) != NULL);
}/* END wlan_memMngrMsduIsFragmented() */

/*************************************************************************
 *                   wlan_memMngrMsduReassemble                          *
 *************************************************************************
 DESCRIPTION: Reassemble a fragmented MSDU.

 INPUT:      msduPtr = pointer to the MSDU
             deFragBuffer = pointer to character buffer of enough size
             (MAX_DATA_BODY_LENGTH).
 OUTPUT:
 RETURN:     NA
**************************************************************************/
void wlan_memMngrMsduReassemble( mem_MSDU_T *msduPtr, char *deFragBuffer)
{
   char     *data = (char *)deFragBuffer;
   mem_BD_T *BD;
   
   /* Iterate on all the BDs bind to this MSDU */
   BD = msduPtr->firstBDPtr;
   while( BD != NULL) {
      /* concatenate the data of the BDs into the deFragBuffer */
      memcpy( data, BD->data + BD->dataOffset, BD->length);
      data += BD->length;
      BD = BD->nextBDPtr;
   }
   return;
}/* END wlan_memMngrMsduReassemble() */

/*************************************************************************
 *                        wlan_memMngrFillField                          *
 *************************************************************************
DESCRIPTION: Sets a field in a frame.
             Called by the previous 'build' functions.                 
                                                                                                   
INPUT:   Data pointer
         Value
         Value len
         BD pointer

OUTPUT:      
RETURN: NA

************************************************************************/
void wlan_memMngrFillField( char **dataP, char *value, UINT32 len, mem_BD_T **BD)
{
   mem_BD_T *BD_P;
   
   BD_P = *BD;
   
   while( len != 0)
   {
      if( BD_P->length + len > DATA_BUF_LEN) /* need to use next BD too */ 
      {
         memcpy( *dataP, value, DATA_BUF_LEN - BD_P->length);
         len -= DATA_BUF_LEN - BD_P->length;
         value += DATA_BUF_LEN - BD_P->length;        
         
         BD_P->length = DATA_BUF_LEN;
         BD_P = BD_P->nextBDPtr;
         BD_P->length = 0;
         BD_P->dataOffset = 0;   
         
         *dataP = BD_P->data + BD_P->dataOffset;
      }
      else                    /* enough place in current BD*/
      {
         memcpy( *dataP, value, len);
         *dataP += len;
         BD_P->length += len;
         len = 0;
      }
   }

   *BD = BD_P;
}/* END wlan_memMngrFillField() */


/*************************************************************************
 *                       wlan_memMngrPrintStatistics                    *
 *************************************************************************
DESCRIPTION: Collects statistics of the MSDU pool, for displaying.
                                                                                                   
INPUT:
OUTPUT: memMngr_Statistics_T *ptr = pointer of statistic structure to fill
RETURN: NA

*************************************************************************/
STATUS wlan_memMngrPrintStatistics( memMngr_Statistics_T *ptr)
{
   int idx;
   if( ptr == NULL) return NOK;

   for( idx = 0; idx < NUM_OF_DATA_BUF_POOLS ; ++idx) {
      ptr->dataBufPool[idx]      = mem_dtPool[idx].dataBufPool;
      ptr->firstFreeDataBuf[idx] = mem_dtPool[idx].firstFreeDataBuf;
      ptr->numFreeDataBuf[idx]   = mem_dtPool[idx].numFreeDataBuf;
   }

   ptr->msduPool    = msduPool;
   ptr->bdPool      = bdPool;

   ptr->firstFreeMSDU    = firstFreeMSDU;
   ptr->firstFreeBD      = firstFreeBD;

   ptr->numFreeMSDU    = numFreeMSDU;
   ptr->numFreeBD      = numFreeBD;
   
   return OK;
}/* END wlan_memMngrPrintStatistics() */


/****************************************************************************
 *                      wlan_memMngrCopyMsdu
 ****************************************************************************
 * DESCRIPTION: Copy MSDU and associated BDs
 *
 * INPUTS:
 *    mem_MSDU_T *pDstMsdu
 *    mem_MSDU_T *pSrcMsdu
 *
 * OUTPUT:  None
 * RETURNS: OK or NOK
 ****************************************************************************/
STATUS wlan_memMngrCopyMsdu( mem_MSDU_T *pDstMsdu, mem_MSDU_T *pSrcMsdu )
{
   mem_BD_T* srcBdPtr;
   mem_BD_T* dstBdPtr;
   int copyOffset = 0;
   
   /* BDs */
   srcBdPtr = pSrcMsdu->firstBDPtr;
   dstBdPtr = pDstMsdu->firstBDPtr;
   
   /* **JG** What happens if one or both BDs are NULL? */

   memcpy( dstBdPtr->data , 
      srcBdPtr->data + srcBdPtr->dataOffset, 
      srcBdPtr->length );
	 dstBdPtr->bufToken = 0; // TODO

   copyOffset = srcBdPtr->length;
   pDstMsdu->dataLen =  srcBdPtr->length;
   while( pDstMsdu->dataLen <  pSrcMsdu->dataLen )
   {
      srcBdPtr = srcBdPtr->nextBDPtr;
      
      memcpy( dstBdPtr->data+copyOffset ,
         srcBdPtr->data+ srcBdPtr->dataOffset, 
         srcBdPtr->length );
      
      copyOffset += srcBdPtr->length;
      pDstMsdu->dataLen +=  srcBdPtr->length;
   }
   
   /* update data length */
   pDstMsdu->dataLen = pSrcMsdu->dataLen;
   dstBdPtr->length  = pSrcMsdu->dataLen;

   return OK;
}/* END wlan_memMngrCopyMsdu() */

/**********************
 * Internal Functions *
 **********************/

/*************************************************************************
 *                         freeDataBuf                                   *
 *************************************************************************
 DESCRIPTION: Free Data Buffer

 INPUT:      dataBuf - pointer to the Data Buffer
 OUTPUT:
 RETURN:     OK/NOK
**************************************************************************/
static STATUS freeDataBuf( mem_DataBuf_T* dataBuf)
{
   unsigned long flags;

   if( dataBuf->refCount == 0) {
      puts( "DB: freeDataBuf FATAL ERROR: dataBuf->refCount = 0");
      return NOK;
   }
   PAL_intrLock(flags);  /* START OF CRITICAL SECTION */
   if( --(dataBuf->refCount) == 0) {
      /* add this Data Buffer to the free list */
      int poolIdx = dataBuf->nPool;
      dataBuf->nextDataBuf = mem_dtPool[poolIdx].firstFreeDataBuf;
      mem_dtPool[poolIdx].firstFreeDataBuf     = dataBuf;
      ++(mem_dtPool[poolIdx].numFreeDataBuf);
   }
   PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
   
   return OK;
}/* END freeDataBuf() */

#if 0
/*
 * **JG** BUG: Should input (mem_DataBuf_T** dataBuf)
 * The function should return the pointer to the allocated
 * Data Buff.
 */
/*************************************************************************
 *                        allocDataBuf                                   *
 *************************************************************************
 DESCRIPTION: Allocate Data Buffer

 INPUT:      dataBuf - pointer to the new allocated Data Buffer
 OUTPUT:
 RETURN:     OK/NOK
**************************************************************************/
static STATUS allocDataBuf( mem_DataBuf_T* dataBuf)
{   
   unsigned long flags = 0;

   PAL_intrLock(flags);   /* START OF CRITICAL SECTION */

   /* Exit if there is no free dataBuf  */
   if( firstFreeDataBuf == NULL) {
      PAL_contextUnLock(&memMngrLock); /* END OF CRITICAL SECTION */
      return NOK;
   }

   dataBuf = firstFreeDataBuf;
   /* update internal state */
   firstFreeDataBuf = firstFreeDataBuf->nextDataBuf;
   --numFreeDataBuf;
   PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
   
   return OK;
}/* END allocDataBuf() */
#endif /* 0 */

/*************************************************************************
 *                        wlan_memMngrFreeAll                            *
 *************************************************************************
 DESCRIPTION: Free all allocated buffers upon initialization failure.

 INPUT:
 OUTPUT:
 RETURN:
**************************************************************************/
void wlan_memMngrFreeAll( void)
{
   int poolIdx;

   /* free and invalidate pools */
   for( poolIdx = 0; poolIdx < NUM_OF_DATA_BUF_POOLS; ++poolIdx)
   {
      if( mem_dtPool[poolIdx].dataBufPool != NULL) {
         /* destroy all data buffers in the data pool */
         int count;
         for( count = 0; count < mem_dtPool[poolIdx].poolSize; ++count) {
            void* ptr = (void*)(mem_dtPool[poolIdx].dataBufPool[count].data);
            if( ptr != NULL) {
               PAL_memFree( 0,ptr,0);
            }
         }
         /* now, destroy the Bata Buf pool */
         PAL_memFree( 0,mem_dtPool[poolIdx].dataBufPool,0);
         mem_dtPool[poolIdx].dataBufPool = NULL;
      }
   }
   /* destroy BD pool */
   if( bdPool != NULL) {
      PAL_memFree( 0,bdPool,0);
      bdPool = NULL;
   }

   /* Destroy MSDU pool */
   if( msduPool != NULL) {
      PAL_memFree( 0,msduPool,0);
      msduPool = NULL;
   }

   /* reset internal state */
   firstFreeMSDU    = msduPool;
   firstFreeBD      = bdPool;
   for( poolIdx = 0; poolIdx < NUM_OF_DATA_BUF_POOLS; ++poolIdx)
   {
      mem_dtPool[poolIdx].firstFreeDataBuf = mem_dtPool[poolIdx].dataBufPool;
      mem_dtPool[poolIdx].numFreeDataBuf = 0;
   }
   numFreeMSDU =  numFreeBD = 0;
}/* END wlan_memMngrFreeAll() */


/*************************************************************************
 *                        wlan_memMngrSelectPool                         *
 *************************************************************************
 DESCRIPTION: Returns the index for the data pool that can hold the
              specified data length.

 INPUT:     len = specified data length
 OUTPUT:
 RETURN:    pool index >= 0, or -1 if error.
**************************************************************************/
static int wlan_memMngrSelectPool( UINT32 len)
{
   int idx = 0;
   for( ; idx < NUM_OF_DATA_BUF_POOLS; ++idx) {
      if( len <= mem_dtPool[idx].dataSize)
         break;
      if( mem_dtPool[idx].dataSize == 0) {
         idx = -1;
         break;
      }
   }
   return idx;
}/* END wlan_memMngrSelectPool() */

/*********************
 * TESTING FUNCTIONS *
 *********************/
void memMngrTestOne( int len)
{
   mem_MSDU_T*       tmpMSDU;
   
   if( wlan_memMngrAllocMSDU( &tmpMSDU, len) == NOK) {
      printf( "memMngrTestOne alloc MSDU failed\n");
   }
   else {
      printf( "memMngrTestOne succeed. first MSDU handle = %d\n", tmpMSDU->handle);
   }
}/* END memMngrTestOne() */

void memMngrAlloc( int len, int numberOfPackets)
{
   int     i;
   
   printf( "memMngrTest\n");
   for( i=0; i<numberOfPackets; ++i) {
      memMngrTestOne( len);
   }
}/* END memMngrAlloc() */

void memMngrFree( int handle)
{   
   if( wlan_memMngrFreeMSDU(handle) == NOK) {
      printf( "memMngrFree failed\n");
   }
   else {
      printf( "memMngrFree succeed\n");
   }
   
}/* END memMngrFree() */

void memMngrPrintHandle( int handle)
{
   mem_BD_T*      tmpBD;
   printf( "MSDU handle = %d firstBDPtr=%X length=%d\n",
      (int)handle, (int)msduPool[handle].firstBDPtr, msduPool[handle].dataLen);
   
   tmpBD = msduPool[handle].firstBDPtr;
   while( tmpBD != NULL) {
      printf( "MSDU BD=%X handle=%d refCount=%d\n",
         (int)tmpBD, (int)tmpBD->handle, tmpBD->refCount);
      tmpBD = tmpBD->nextBDPtr;
   }
}/* END memMngrPrintHandle() */

void memMngrPrint( void)
{
   mem_MSDU_T*      tmpMSDU;
   mem_BD_T*        tmpBD;
   mem_DataBuf_T*   tmpDataBuf;
   int     i=0;
   int     portIdx;
   
   printf( "memMngrPrint\n");
   printf( "numFreeMSDU %d numFreeBD %d\n",
      numFreeMSDU, numFreeBD );

   for( portIdx = 0; portIdx < NUM_OF_DATA_BUF_POOLS; ++portIdx) {
      printf("numFreeDataBuf[%d] %d\n" ,portIdx, mem_dtPool[portIdx].numFreeDataBuf);
   }
   
   tmpMSDU = firstFreeMSDU;
   while( ++i, tmpMSDU != NULL) {
      printf( "tmpMSDU %d = %X handle=%d\n", i, (int)tmpMSDU, tmpMSDU->handle);
      tmpMSDU = tmpMSDU->nextMSDU;
   }
   
   i = 0;
   tmpBD = firstFreeBD;
   while( ++i, tmpBD != NULL) {
      printf( "tmpBD %d = %X handle=%d tmpBD->nextBDPtr=%X\n",
         i, (int)tmpBD, tmpBD->handle, (int)tmpBD->nextBDPtr);
      tmpBD = tmpBD->nextBDPtr;
   }
   

   for( portIdx = 0; portIdx < NUM_OF_DATA_BUF_POOLS; ++portIdx) {
      i = 0;
      tmpDataBuf = mem_dtPool[portIdx].firstFreeDataBuf;
      while( ++i, tmpDataBuf != NULL) {
         printf( "tmpDataBuf %d = %X handle=%d\n", i, (int)tmpDataBuf, tmpDataBuf->handle);
         tmpDataBuf = tmpDataBuf->nextDataBuf;
      }
   }    
}/* END memMngrPrint() */
