/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/**************************************************************************/
/*                                                                        */
/*   MODULE:  genericSM.h                                                 */
/*   PURPOSE: Generic State Machine implementation                        */
/*                                                                        */
/*   This file can be included by any module which needs a state machine  */
/*   or several parallel state machines.                                  */
/*   The state machines are then built by the module in a table-like      */
/*   form. For example:                                                   */
/*                                                                        */
/*   // states                                                            */
/*   #define sIDLE        0  // Idle - do nothing                         */
/*   #define sOPER        1  // OPERational                               */
/*   #define NUM_STATES   2                                               */
/*                                                                        */
/*   // inputs                                                            */
/*   #define iSYNC        0  // Sync command                              */
/*   #define iTOUT        1  // Time out expired                          */
/*   #define iRSET        2  // RESET command / any Failure               */
/*   #define NUM_INPUTS   3                                               */
/*                                                                        */
/*   // actions                                                           */
/*   #define aNOP         0                                               */
/*   #define aSTLK        1  // Start lock process                        */
/*   #define aDSTO        2  // DS lock timeout handle                    */
/*   #define aCLER        3  // Clear all values and timers               */
/*                                                                        */
/*      // Build the state machine                                        */
/*      lbt_newStateMachine(NUM_STATES, NUM_INPUTS);                      */
/*                                                                        */
/*   // State\inp   iSYNC           iTOUT         iRSET                   */
/*   st(sIDLE); u(sOPER,aSTLK); u(sIDLE,aNOP ); u(sIDLE,aCLER);           */
/*   st(sOPER); u(sOPER,aNOP ); u(sIDLE,aDSTO); u(sIDLE,aCLER);           */
/*                                                                        */
/**************************************************************************/

#ifndef _GENERIC_SM_H
#define _GENERIC_SM_H

#include "pal.h"

#define MAX_STATES		10
#define MAX_INPUTS		25
#define MAX_MACHINES	64


/* Data for state machine */
typedef struct 
{
	UINT32	action;
	UINT16	status;
} output_T;

static unsigned int smNextState[MAX_STATES][MAX_INPUTS];
static output_T smOutput[MAX_STATES][MAX_INPUTS];
static unsigned int smBuildState;
static unsigned int smBuildInput;
static unsigned int smBuildNumStates;
static unsigned int smBuildNumInputs;
static          int smCurrentState[MAX_MACHINES];

PAL_spinLockHandle mgmtLock;

/**************************************************************************
 *                      lbt_newStateMachines()               			  *
 **************************************************************************
 * DESCRIPTION: Build the state machines table.                   		  *
 *     			The table is empty and now it can be filled with 		  *
 * 				st() and u()      										  *
 *                                                                        *
 * INPUTS:  	numStates = Number of states                              *
 *          	numInputs = Number of inputs                              *
 *          	numMachines = Number of state machines                    *
 * OUTPUT:      no                                                        *
 *                                                                        *
 **************************************************************************/
static void lbt_newStateMachines( unsigned int numStates,
                                  unsigned int numInputs,
								  int          numMachines )
{
	int i;

	if (numStates > MAX_STATES) {
	wdrv_utilsReport(RL_SM, PRI_INFORMATION,
		 "%s: lbt_newStateMachine() Error: numStates > MAX_STATES\n", SM_NAME);
		return;
	}
	if (numInputs > MAX_INPUTS) {
	wdrv_utilsReport(RL_SM, PRI_INFORMATION,
		 "%s: lbt_newStateMachine() Error: numInputs > MAX_INPUTS\n", SM_NAME);
		return;
	}
	if (numMachines > MAX_MACHINES) {
	wdrv_utilsReport(RL_SM, PRI_INFORMATION,
		 "%s: lbt_newStateMachine() Error: numMachines > MAX_MACHINES\n", SM_NAME);
		return;
	}

	smBuildState = 0;
	smBuildInput = 0;
	smBuildNumStates = numStates;
	smBuildNumInputs = numInputs;

	for (i=0; i<numMachines; ++i) {
		smCurrentState[i] = 0;
	}
}

/**************************************************************************
 *                      lbt_newStateMachine()               			  *
 **************************************************************************
 * DESCRIPTION: Build the state machine table.                   		  *
 *     			The table is empty and now it can be filled with 		  *
 * 				st() and u().      										  *
 *     			This function would support the 'old' single-state 		  *
 *				machine code.    										  *
 *                                                                        *
 * INPUTS:  	numStates = Number of states                              *
 *          	numInputs = Number of inputs                              *
 *          	numMachines = Number of state machines                    *
 * OUTPUT:      no                                                        *
 *                                                                        *
 **************************************************************************/
static void lbt_newStateMachine( unsigned int numStates,
                                 unsigned int numInputs )
{
	lbt_newStateMachines(numStates, numInputs, 1);
}


/**************************************************************************
 *                      st()               			  					  *
 **************************************************************************
 * DESCRIPTION: Start a new state line    								  *
 *                                                                        *
 * INPUTS:  	state = State Number                                      *
 * OUTPUT:      no                                                        *
 *                                                                        *
 **************************************************************************/
static void st(unsigned int state) {
	if (state >= smBuildNumStates) {
		wdrv_utilsReport(RL_SM, PRI_INFORMATION,"%s: StateMachine() Error: state >= smBuildNumStates\n", SM_NAME);
		return;
	}

	smBuildState = state;
	smBuildInput = 0;
}


/**************************************************************************
 *                      u()               			  					  *
 **************************************************************************
 * DESCRIPTION: On next input's - State and Action    					  *
 *                                                                        *
 * INPUTS:  	nextState = State Number to move to                       *
 *              action = Action number to perform                         *
 * OUTPUT:      no                                                        *
 *                                                                        *
 **************************************************************************/
static void u(unsigned int nextState, unsigned int action, UINT32 status) {
	if (nextState >= smBuildNumStates) {
		wdrv_utilsReport(RL_SM, PRI_INFORMATION,
		"%s: StateMachine u() Error: nextState >= smBuildNumStates\n", SM_NAME);
		return;
	}
	if (smBuildInput >= smBuildNumInputs) {
		wdrv_utilsReport(RL_SM, PRI_INFORMATION,
		"%s: StateMachine u() Error: smBuildInput >= smBuildNumInputs\n", SM_NAME);
		return;
	}
	smNextState[smBuildState][smBuildInput] = nextState;

	smOutput[smBuildState][smBuildInput].action = action;
	smOutput[smBuildState][smBuildInput].status = status;

	smBuildInput++;
}


/************************************************************************************
 *                      lbt_smActions()               			  					*
 ************************************************************************************
 * DESCRIPTION: Run the state machine. Give an input and get an action				*
 *     			number to perform while moving to next state.						*
 *																					*
 * INPUTS:  	input = Input number												*
 *              machineNumber = The number of the state machine						*
 * OUTPUT:      action	  = The action to be performed								*
 *              status    = Result code, to be used when sending a message			*
 *																					*
 ************************************************************************************/
STATUS lbt_smActions(unsigned int input, int machineNumber, output_T *outputPtr) {

	int oldState;

	if (machineNumber > MAX_MACHINES) {
	wdrv_utilsReport(RL_SM, PRI_INFORMATION,
		"%s: lbt_newStateMachine() Error: machineNumber > MAX_MACHINES\n",
		SM_NAME);
		return NOK;
	}

	if (input >= smBuildNumInputs) {
		wdrv_utilsReport(RL_SM, PRI_INFORMATION,
		"%s: lbt_smAction() Error: input >= smBuildNumInputs (input = %d)\n", 
		SM_NAME, input);
		return NOK;
	}

	oldState = smCurrentState[machineNumber];

	outputPtr->action = smOutput[smCurrentState[machineNumber]][input].action;
	outputPtr->status = smOutput[smCurrentState[machineNumber]][input].status;

	PAL_spinLock(&mgmtLock); /* START OF CRITICAL SECTION */
	/* state = next state. */
	smCurrentState[machineNumber] = smNextState[smCurrentState[machineNumber]][input];
   	PAL_spinUnLock(&mgmtLock);  /* END OF CRITICAL SECTION */

#if 0
		wdrv_utilsReport(RL_SM, PRI_DEBUG,
		"%s: Oldstate:%d, Input:%d --> NewState:%d, Action:%d, Status:%d\n", SM_NAME,
		oldState, input, smCurrentState[machineNumber], outputPtr->action, outputPtr->status);
#endif
	return OK;
}

/**************************************************************************
 *                      lbt_smAction()               			  		  *
 **************************************************************************
 * DESCRIPTION: Run the state machine. Give an input and get an action    *
 *     			number to perform while moving to next state.             *
 *     			This function would support the 'old' single-state 		  *
 *				machine code    										  *
 *                                                                        *
 * INPUTS:  	input = Input number                                      *
 * OUTPUT:      Action Number to perform                                  *
 *                                                                        *
 **************************************************************************/
STATUS lbt_smAction(unsigned int input, output_T *outputPtr) 
{
	return lbt_smActions(input, 0, outputPtr);
}

/**************************************************************************
 *                      lbt_smResetState()             			  		  *
 **************************************************************************
 * DESCRIPTION: Reset a state machine									  *
 *                                                                        *
 * INPUTS:  	machineNumber = machine number                                      *
 * OUTPUT:      Action Number to perform                                  *
 *                                                                        *
 **************************************************************************/
static void lbt_smResetState(unsigned int machineNumber) {
	
	if (machineNumber > MAX_MACHINES)
		return;

	smCurrentState[machineNumber] = 0;
}

#endif

